Package: peakPantheR
Title: Peak Picking and Annotation of High Resolution Experiments
Version: 1.27.0
Date: 2025-09-27
Authors@R: c(person("Arnaud", "Wolfer", email = "adwolfer@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0001-5856-3218")),
	person("Goncalo", "Correia", email = "gscorreia89@gmail.com", role = "aut", comment = c(ORCID = "0000-0001-8271-9294")),
	person("Jake", "Pearce", email = "jake.pearce@gmail.com", role = "ctb"),
	person("Caroline", "Sands", email = "caz119@hotmail.co.uk", role = "ctb"))
Description: An automated pipeline for the detection, integration and reporting of predefined features across a large number of mass spectrometry data files.
    It enables the real time annotation of multiple compounds in a single file, or the parallel annotation of multiple compounds in multiple files.
    A graphical user interface as well as command line functions will assist in assessing the quality of annotation and update fitting parameters until a satisfactory result is obtained.
Depends: R (>= 4.5)
Imports: foreach (>= 1.4.4), doParallel (>= 1.0.11), ggplot2 (>=
        3.5.0), gridExtra (>= 2.3), MSnbase (>= 2.4.0), mzR (>=
        2.12.0), stringr (>= 1.2.0), methods (>= 3.4.0), XML (>=
        3.98.1.10), minpack.lm (>= 1.2.1), scales(>= 0.5.0), shiny (>=
        1.0.5), bslib, shinycssloaders (>= 1.0.0), DT (>= 0.15), pracma
        (>= 2.2.3), utils, lubridate, svglite (>= 2.1.1)
biocViews: MassSpectrometry, Metabolomics, PeakDetection
License: GPL-3
BugReports: https://github.com/phenomecentre/peakPantheR/issues/new
URL: https://github.com/phenomecentre/peakPantheR
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: testthat, devtools, faahKO, msdata, knitr, rmarkdown, pander,
        BiocStyle
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/peakPantheR
git_branch: devel
git_last_commit: efe4373
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-07-14
NeedsCompilation: no
Packaged: 2026-07-14 23:30:03 UTC; biocbuild
Author: Arnaud Wolfer [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5856-3218>),
  Goncalo Correia [aut] (ORCID: <https://orcid.org/0000-0001-8271-9294>),
  Jake Pearce [ctb],
  Caroline Sands [ctb]
Maintainer: Arnaud Wolfer <adwolfer@gmail.com>

# Generated by roxygen2: do not edit by hand

export(annotation_diagnostic_multiplot_UI_helper)
export(annotation_fit_summary_UI_helper)
export(annotation_showMethod_UI_helper)
export(annotation_showText_UI_helper)
export(initialise_annotation_from_files_UI_helper)
export(load_annotation_from_file_UI_helper)
export(outputAnnotationFeatureMetadata_UI_helper)
export(outputAnnotationSpectraMetadata_UI_helper)
export(peakPantheRAnnotation)
export(peakPantheR_ROIStatistics)
export(peakPantheR_loadAnnotationParamsCSV)
export(peakPantheR_parallelAnnotation)
export(peakPantheR_plotEICFit)
export(peakPantheR_plotPeakwidth)
export(peakPantheR_quickEIC)
export(peakPantheR_singleFileSearch)
export(peakPantheR_start_GUI)
export(spectraPaths_and_metadata_UI_helper)
export(spectra_metadata_colourScheme_UI_helper)
exportMethods(EICs)
exportMethods(FIR)
exportMethods(ROI)
exportMethods(TIC)
exportMethods(acquisitionTime)
exportMethods(annotationDiagnosticPlots)
exportMethods(annotationParamsDiagnostic)
exportMethods(annotationTable)
exportMethods(cpdID)
exportMethods(cpdMetadata)
exportMethods(cpdName)
exportMethods(dataPoints)
exportMethods(filename)
exportMethods(filepath)
exportMethods(isAnnotated)
exportMethods(nbCompounds)
exportMethods(nbSamples)
exportMethods(outputAnnotationDiagnostic)
exportMethods(outputAnnotationParamsCSV)
exportMethods(outputAnnotationResult)
exportMethods(peakFit)
exportMethods(peakTables)
exportMethods(resetAnnotation)
exportMethods(resetFIR)
exportMethods(retentionTimeCorrection)
exportMethods(spectraMetadata)
exportMethods(uROI)
exportMethods(uROIExist)
exportMethods(useFIR)
exportMethods(useUROI)
import(XML)
import(doParallel)
import(foreach)
import(methods)
import(mzR)
import(scales)
import(shiny)
import(shinycssloaders)
import(svglite)
import(utils)
importFrom(DT,DTOutput)
importFrom(DT,datatable)
importFrom(DT,renderDT)
importFrom(bslib,bs_theme)

# peakPantheR <img src="man/figures/peakPantheR-logo.png" align="right" />

[![R-CMD-check-Bioc](https://github.com/phenomecentre/peakPantheR/workflows/R-CMD-check-bioc/badge.svg)](https://github.com/phenomecentre/peakPantheR/actions)
[![BioC status](http://www.bioconductor.org/shields/build/release/bioc/peakPantheR.svg)](https://bioconductor.org/checkResults/release/bioc-LATEST/peakPantheR)
[![R-CMD-check-Bioc-devel](https://github.com/phenomecentre/peakPantheR/workflows/R-CMD-check-bioc-devel/badge.svg)](https://github.com/phenomecentre/peakPantheR/actions)
[![BioC dev status](http://www.bioconductor.org/shields/build/devel/bioc/peakPantheR.svg)](https://bioconductor.org/checkResults/devel/bioc-LATEST/peakPantheR)
[![codecov](https://codecov.io/gh/phenomecentre/peakPantheR/branch/master/graph/badge.svg)](https://codecov.io/gh/phenomecentre/peakPantheR/branch/master)
[![DOI](https://zenodo.org/badge/116680214.svg)](https://zenodo.org/badge/latestdoi/116680214)


Package for *Peak Picking and ANnoTation of High resolution Experiments in R*, implemented in `R` and `Shiny`

## Overview

[**peakPantheR**](https://phenomecentre.github.io/peakPantheR.github.io/) is an R/Bioconductor package that implements functions to detect, integrate and report pre-defined features in MS files (*e.g. compounds, fragments, adducts, …*). It is designed for:

  - **Real time** feature detection and integration (see [Real Time Annotation](https://phenomecentre.github.io/peakPantheR.github.io/articles/real-time-annotation.html))
      - process `multiple` compounds in `one` file at a time
  - **Post-acquisition** feature detection, integration and reporting (see [Parallel Annotation](https://phenomecentre.github.io/peakPantheR.github.io/articles/parallel-annotation.html))
      - process `multiple` compounds in `multiple` files in `parallel`, store results in a `single` object

`peakPantheR` can process LC/MS data files in *NetCDF*, *mzML*/*mzXML* and *mzData* format as data import is achieved using Bioconductor’s [`mzR`](https://bioconductor.org/packages/release/bioc/html/mzR.html) package.

The reference versions of `peakPantheR` is available on the corresponding Bioconductor page ([release](https://bioconductor.org/packages/release/bioc/html/peakPantheR.html) or [dev](https://bioconductor.org/packages/devel/bioc/html/peakPantheR.html) version).

Active development and issue tracking take place on the [github page](https://github.com/phenomecentre/peakPantheR), while an overview of the package, vignettes and documentation are available on the [supporting website](https://phenomecentre.github.io/peakPantheR.github.io/).

## Installation

To install [peakPantheR](https://bioconductor.org/packages/release/bioc/html/peakPantheR.html):
``` r
install.packages("BiocManager")
BiocManager::install("peakPantheR")
```

To install the development version from GitHub:
``` r
BiocManager::install("phenomecentre/peakPantheR")
```

## Usage

Both real time and parallel compound integration require a common set of information:

  - Path(s) to `netCDF` / `mzML` MS file(s)
  - An expected region of interest (`RT` / `m/z` window) for each compound.

## Vignettes
An overview of the package and detailed information on usage are available in the following vignettes:

  - [Getting Started with peakPantheR](https://phenomecentre.github.io/peakPantheR.github.io/articles/getting-started.html)
  - [Real Time Annotation](https://phenomecentre.github.io/peakPantheR.github.io/articles/real-time-annotation.html)
  - [Parallel Annotation](https://phenomecentre.github.io/peakPantheR.github.io/articles/parallel-annotation.html)
  - [Graphical user interface use](https://phenomecentre.github.io/peakPantheR.github.io/articles/peakPantheR-GUI.html)
  
## Examples
Besides the vignettes, more tutorials are available via github:

  - [Targeted integration of metabolites from 3 LC-MS profiling dataset using peakPantheR](https://github.com/phenomecentre/metabotyping-dementia-urine): Application of peakpPantheR to extract features from 3 LC-MS assays from a human urine metabolic profiling study on cognitive decline and dementia.   
  - [Quality-Control of peakPantheR extracted using the nPYc-Toolbox](https://github.com/phenomecentre/nPYc-toolbox-tutorials): Tutorial on how to use the nPYC-Toolbox to perform QC on peakPanther extracted datasets.
  

## Contributing

Suggestions and contributions to `peakPantheR` are welcome, for more information please first refer to the [contribution guide and code of conduct](./CONTRIBUTIONS.md), or get in touch by opening a [Github issue](https://github.com/phenomecentre/peakPantheR/issues/).


## Copyright

`peakPantheR` is licensed under the [GPLv3](http://choosealicense.com/licenses/gpl-3.0/)

As a summary, the GPLv3 license requires attribution, inclusion of copyright and license information, disclosure of source code and changes. Derivative work must be available under the same terms.

© National Phenome Centre (2025)

#' @title Plot samples raw data and detected feature for a single ROI
#'
#' @description Plot a ROI across multiple samples (x axis is RT, y axis is
#' intensity) with the matching detected peak rt and peakwidth under it. If
#' curveFit is provided, the fitted curve for each compound is added. RT and
#' peakwidth are plotted in the order spectra are passed, with the first spectra
#' on top.
#'
#' @param ROIDataPointSampleList (list) list of \code{data.frame} of raw data
#' points for each sample (retention time 'rt', mass 'mz' and intensity 'int'
#' (as column) of each raw data points (as row)).
#' @param cpdID (str) Compound ID
#' @param cpdName (str) Compound Name
#' @param rt (float) vector of detected peak apex retention time (in sec)
#' @param rtMin (float) vector of detected peak minimum retention time (in sec)
#' @param rtMax (float) vector ofdetected peak maximum retention time (in sec)
#' @param mzMin (float) ROI minimum m/z (matching EIC)
#' @param mzMax (float) ROI maximum m/z (matching EIC)
#' @param ratio (float) value between 0 and 1 defining the vertical percentage
#' taken by the EICs subplot
#' @param sampling (int) Number of points to employ when plotting fittedCurve
#' @param curveFitSampleList (list) NULL or a list of
#' \code{peakPantheR_curveFit} (or NA) for each sample
#' @param sampleColour (str) NULL or vector colour for each sample (same length
#' as \code{EICs}, \code{rt}, \code{rtMin}, \code{rtMax})
#' @param verbose (bool) if TRUE message when NA scans are removed
#' 
#' @return Grob (ggplot object)
plotEICDetectedPeakwidth <- function(ROIDataPointSampleList, cpdID, cpdName, rt,
    rtMin, rtMax, mzMin, mzMax, ratio = 0.85, sampling = 250,
    curveFitSampleList = NULL, sampleColour = NULL, verbose = TRUE) {

    # Check input, init
    ratio <- plotEICDetectedPeakwidth_checkInp(ROIDataPointSampleList, rt,
                                                ratio, verbose)
    title <- paste("CpdID: ", cpdID, " - ", cpdName, " ", round(mzMin, 4), "-",
                    round(mzMax, 4))
    
    # Plot raw spectra and curve fit
    p_spec <- peakPantheR_plotEICFit(
        ROIDataPointSampleList = ROIDataPointSampleList,
        curveFitSampleList = curveFitSampleList, rtMin = rtMin, rtMax = rtMax,
        sampling = sampling, sampleColour = sampleColour, verbose = verbose)
    p_spec <- p_spec + ggplot2::ggtitle(title) +
            ggplot2::theme(axis.title.x = ggplot2::element_blank(),
                axis.text.x = ggplot2::element_blank(),
                plot.title = ggplot2::element_text(size = ggplot2::rel(1)))
    # Plot peakwidth
    p_peakwidth <- peakPantheR_plotPeakwidth(apexValue = rt, widthMin = rtMin,
        widthMax = rtMax, varName = "Retention Time (sec)", acquTime = NULL,
        sampleColour = sampleColour, rotateAxis = TRUE, verbose = FALSE)
    
    # Set common x lim (due to the rotation, x on p_peakwidth is originally y
    # and accessed as such)
    minX <- min(ggplot2::layer_scales(p_spec)$x$range$range[1],
        ggplot2::layer_scales(p_peakwidth)$y$range$range[1])
    maxX <- max(ggplot2::layer_scales(p_spec)$x$range$range[2],
        ggplot2::layer_scales(p_peakwidth)$y$range$range[2])
    p_spec <- p_spec + ggplot2::xlim(minX, maxX)
    suppressMessages(p_peakwidth <- p_peakwidth + ggplot2::ylim(minX, maxX))
    # convert to gtables
    p_spec <- ggplot2::ggplot_gtable(ggplot2::ggplot_build(p_spec))
    p_peakwidth <- ggplot2::ggplot_gtable(ggplot2::ggplot_build(p_peakwidth))
    # find the widths of each of the plots, calculate the maximum and then apply
    # it to each of them individually. This effectively applies a uniform layout
    # to each of the plots.
    maxWidth <- grid::unit.pmax(p_spec$widths[2:5], p_peakwidth$widths[2:5])
    p_spec$widths[2:5] <- maxWidth
    p_peakwidth$widths[2:5] <- maxWidth
    
    # Group plots with y axis ratio
    topSize <- ratio * 10
    bottomSize <- (1 - ratio) * 10
    p <- gridExtra::grid.arrange(p_spec, p_peakwidth,
        heights = c(topSize, bottomSize))
    
    return(p)
}
# prepare and check input
plotEICDetectedPeakwidth_checkInp <- function(ROIDataPointSampleList, rt, ratio,
                                                verbose) {
    # check length of input across subplots, (others are checked inside
    # peakPantheR_plotEICFit and peakPantheR_plotPeakwidth)
    nbSpl <- length(ROIDataPointSampleList)
    if (nbSpl != length(rt)) {
        stop("\"ROIDataPointSampleList\", \"rt\", \"rtMin\" and \"rtMax\" must",
            " be the same length")
    }

    # ratio must be between 0 and 1
    if ((ratio < 0) | (ratio > 1)) {
        if (verbose) {
            message("Err","or: ratio must be between 0 and 1, ",
                    "replaced by default value")
        }
        ratio <- 0.85
    }

    return(ratio)
}

context('plotEICDetectedPeakwidth()')

# remove Rplots.pdf created by ggplot2
on.exit( tryCatch({ file.remove('./Rplots.pdf') }, error=function(e){ invisible() }, warning=function(w){ invisible() }) )


## Input and expected data
# fake sample 1
# ROI data points
rt1  <- seq(990, 1010, by=20/250)
mz1  <- rep(522., length(rt1))
int1 <- (dnorm(rt1, mean=1000, sd=1.5) * 100) + 1
tmp_DataPoints1  <- data.frame(rt=rt1, mz=mz1, int=int1)
# fittedCurve
fit1        <- list(amplitude=37.068916502809756, center=999.3734222573454, sigma=0.58493182568124724, gamma=0.090582029276037035, fitStatus=2, curveModel="skewedGaussian")
class(fit1) <- 'peakPantheR_curveFit'

# fake sample 2
# ROI data points
rt2  <- seq(990, 1010, by=20/250)
mz2  <- rep(522., length(rt2))
int2 <- (dnorm(rt2, mean=1002, sd=1.5) * 100) + 1
tmp_DataPoints2  <- data.frame(rt=rt2, mz=mz2, int=int2)
# fittedCurve
fit2        <- list(amplitude=37.073067416755556, center=1001.3736564832565, sigma=0.58496485738212201, gamma=0.090553713725151905, fitStatus=2, curveModel="skewedGaussian")
class(fit2) <- 'peakPantheR_curveFit'


test_that('plot feature in 1 sample', {
	# result plot
  result_plot <- plotEICDetectedPeakwidth(list(tmp_DataPoints1), cpdID='ID-1', cpdName='testCpd 1', rt=1000., rtMin=995., rtMax=1005., mzMin=521., mzMax=523., ratio=0.85, sampling=250, curveFitSampleList=list(fit1), sampleColour=NULL, verbose=FALSE)

  # cannot compare plot to a stored version due to some string mismatch (even if the same plot is called twice)
  # Check plot properties
  expect_is(result_plot, 'gtable')
  expect_is(result_plot, 'gTree')
  expect_is(result_plot, 'grob')
  expect_is(result_plot, 'gDesc')
  expect_equal(length(result_plot), 2)
  # number of subplots
  expect_equal(length(result_plot)[[1]], 2)
})

test_that('plot feature in 2 samples, change colours', {
  # result plot
  result_plot <- plotEICDetectedPeakwidth(list(tmp_DataPoints1, tmp_DataPoints2), cpdID=c('ID-1', 'ID-2'), cpdName=c('testCpd 1', 'testCpd 2'), rt=c(1000., 1002.), rtMin=c(995., 997.), rtMax=c(1005.,1007.), mzMin=c(521.,521.), mzMax=c(523.,523.), ratio=0.85, sampling=250, curveFitSampleList=list(fit1, fit2), sampleColour=c('blue', 'red'), verbose=FALSE)
  
  # cannot compare plot to a stored version due to some string mismatch (even if the same plot is called twice)
  # Check plot properties
  expect_is(result_plot, 'gtable')
  expect_is(result_plot, 'gTree')
  expect_is(result_plot, 'grob')
  expect_is(result_plot, 'gDesc')
  expect_equal(length(result_plot), 2)
  # number of subplots
  expect_equal(length(result_plot)[[1]], 2)
})

test_that('plot feature in 2 samples, rtMin and/or rtMax are NA', {
  # result plot
  result_plotNA <- plotEICDetectedPeakwidth(list(tmp_DataPoints1, tmp_DataPoints2), cpdID=c('ID-1', 'ID-2'), cpdName=c('testCpd 1', 'testCpd 2'), rt=c(1000., 1002.), rtMin=c(NA, 997.), rtMax=c(1005.,NA), mzMin=c(521.,521.), mzMax=c(523.,523.), ratio=0.85, sampling=250, curveFitSampleList=list(fit1, fit2), sampleColour=c('blue', 'red'), verbose=FALSE)
  
  # cannot compare plot to a stored version due to some string mismatch (even if the same plot is called twice)
  # Check plot properties
  expect_is(result_plotNA, 'gtable')
  expect_is(result_plotNA, 'gTree')
  expect_is(result_plotNA, 'grob')
  expect_is(result_plotNA, 'gDesc')
  expect_equal(length(result_plotNA), 2)
  # number of subplots
  expect_equal(length(result_plotNA)[[1]], 2)
})

test_that('colour warning, ratio replacement', {
  # Expected message
  expected_message <- c("Error: ratio must be between 0 and 1, replaced by default value\n", "Warning: sampleColour length must match the number of samples; default colour used\n")
  
  # result plot
  result_plot <- evaluate_promise(plotEICDetectedPeakwidth(list(tmp_DataPoints1, tmp_DataPoints2), cpdID=c('ID-1', 'ID-2'), cpdName=c('testCpd 1', 'testCpd 2'), rt=c(1000., 1002.), rtMin=c(995., 997.), rtMax=c(1005.,1007.), mzMin=c(521.,521.), mzMax=c(523.,523.), ratio=2, sampling=250, curveFitSampleList=list(fit1, fit2), sampleColour=c('blue', 'red', 'grey'), verbose=TRUE))
  
  # check messages confirming the replacements
  expect_equal(result_plot$messages, expected_message)
  
  # cannot compare plot to a stored version due to some string mismatch (even if the same plot is called twice)
  # Check plot properties
  expect_is(result_plot$result, 'gtable')
  expect_is(result_plot$result, 'gTree')
  expect_is(result_plot$result, 'grob')
  expect_is(result_plot$result, 'gDesc')
  expect_equal(length(result_plot$result), 2)
  # number of subplots
  expect_equal(length(result_plot$result)[[1]], 2)
})

test_that('raise errors', {
  # ROIDataPointSampleList is not a list
  msg1    <- c('Error: "ROIDataPointSampleList" must be a list of data.frame')
  expect_error(plotEICDetectedPeakwidth('not a list', cpdID='ID-1', cpdName='testCpd 1', rt=1000., rtMin=995., rtMax=1005., mzMin=521., mzMax=523., ratio=0.85, sampling=250, curveFitSampleList=list(fit1), sampleColour=NULL, verbose=FALSE), msg1, fixed=TRUE)
  # ROIDataPointSampleList length is wrong
  msg2    <- c('"ROIDataPointSampleList", "rt", "rtMin" and "rtMax" must be the same length')
  expect_error(plotEICDetectedPeakwidth(list(tmp_DataPoints1, tmp_DataPoints2), cpdID='ID-1', cpdName='testCpd 1', rt=1000., rtMin=995., rtMax=1005., mzMin=521., mzMax=523., ratio=0.85, sampling=250, curveFitSampleList=list(fit1), sampleColour=NULL, verbose=FALSE), msg2, fixed=TRUE)
  # rt length is wrong
  msg3    <- c('"ROIDataPointSampleList", "rt", "rtMin" and "rtMax" must be the same length')
  expect_error(plotEICDetectedPeakwidth(list(tmp_DataPoints1), cpdID='ID-1', cpdName='testCpd 1', rt=c(1000., 1002.), rtMin=995., rtMax=1005., mzMin=521., mzMax=523., ratio=0.85, sampling=250, curveFitSampleList=list(fit1), sampleColour=NULL, verbose=FALSE), msg3, fixed=TRUE)
  # rtmin length is wrong
  msg4    <- c('"curveFitSampleList", "rtMin", "rtMax" and "ROIDataPointSampleList" must be the same length')
  expect_error(plotEICDetectedPeakwidth(list(tmp_DataPoints1), cpdID='ID-1', cpdName='testCpd 1', rt=1000., rtMin=c(995., 997.), rtMax=1005., mzMin=521., mzMax=523., ratio=0.85, sampling=250, curveFitSampleList=list(fit1), sampleColour=NULL, verbose=FALSE), msg4, fixed=TRUE)
  # rtmax length is wrong
  msg5    <- c('"curveFitSampleList", "rtMin", "rtMax" and "ROIDataPointSampleList" must be the same length')
  expect_error(plotEICDetectedPeakwidth(list(tmp_DataPoints1), cpdID='ID-1', cpdName='testCpd 1', rt=1000., rtMin=995., rtMax=c(1005.,1007.), mzMin=521., mzMax=523., ratio=0.85, sampling=250, curveFitSampleList=list(fit1), sampleColour=NULL, verbose=FALSE), msg5, fixed=TRUE)
  # curveFitSampleList length is wrong
  msg6    <- c('"curveFitSampleList", "rtMin", "rtMax" and "ROIDataPointSampleList" must be the same length')
  expect_error(plotEICDetectedPeakwidth(list(tmp_DataPoints1), cpdID='ID-1', cpdName='testCpd 1', rt=1000., rtMin=995., rtMax=1005., mzMin=521., mzMax=523., ratio=0.85, sampling=250, curveFitSampleList=list(fit1,fit2), sampleColour=NULL, verbose=FALSE), msg6, fixed=TRUE)
})

#' @title Plot samples raw data and detected feature for a single ROI
#'
#' @description plot a ROI across multiple samples (x axis is RT, y axis is
#' intensity). If curveFit is provided, the fitted curve for each sample is
#' added.
#'
#' @param ROIDataPointSampleList (list) list of \code{data.frame} of raw data
#' points for each sample (retention time 'rt', mass 'mz' and intensity 'int'
#' (as column) of each raw data points (as row)).
#' @param curveFitSampleList (list) NULL or a list of
#' \code{peakPantheR_curveFit} (or NA) for each sample
#' @param rtMin (float) NULL or vector of detected peak minimum retention time
#' (in sec)
#' @param rtMax (float) NULL or vector of detected peak maximum retention time
#' (in sec)
#' @param sampling (int) Number of points to employ when plotting fittedCurve
#' @param sampleColour (str) NULL or vector colour for each sample (same length
#' as \code{ROIDataPointSampleList}, \code{rtMin}, \code{rtMax})
#' @param verbose (bool) if TRUE message when NA scans are removed
#' 
#' @return Grob (ggplot object)
#' 
#' @export
#' 
#' @examples
#' ## Input data
#' # fake sample 1
#' # ROI data points
#' rt1             <- seq(990, 1010, by=20/250)
#' mz1             <- rep(522., length(rt1))
#' int1            <- (dnorm(rt1, mean=1000, sd=1.5) * 100) + 1
#' tmp_DataPoints1 <- data.frame(rt=rt1, mz=mz1, int=int1)
#' # fittedCurve
#' fit1        <- list(amplitude=37.068916502809756, center=999.3734222573454,
#'                     sigma=0.58493182568124724, gamma=0.090582029276037035,
#'                     fitStatus=2, curveModel='skewedGaussian')
#' class(fit1)     <- 'peakPantheR_curveFit'
#' 
#' # fake sample 2
#' # ROI data points
#' rt2             <- seq(990, 1010, by=20/250)
#' mz2             <- rep(522., length(rt2))
#' int2            <- (dnorm(rt2, mean=1002, sd=1.5) * 100) + 1
#' tmp_DataPoints2 <- data.frame(rt=rt2, mz=mz2, int=int2)
#' # fittedCurve
#' fit2        <- list(amplitude=37.073067416755556, center=1001.3736564832565,
#'                     sigma=0.58496485738212201, gamma=0.090553713725151905,
#'                     fitStatus=2, curveModel='skewedGaussian')
#' class(fit2)     <- 'peakPantheR_curveFit'
#' 
#' ## Plot features in 1 sample without colours
#' peakPantheR_plotEICFit(ROIDataPointSampleList=list(tmp_DataPoints1),
#'                         curveFitSampleList=list(fit1),
#'                         rtMin=995., rtMax=1005.,
#'                         sampling=250, sampleColour=NULL, verbose=FALSE)
#'
#' ## Plot features in 2 samples with colours
#' peakPantheR_plotEICFit(
#'                 ROIDataPointSampleList=list(tmp_DataPoints1,tmp_DataPoints2),
#'                 curveFitSampleList=list(fit1, fit2),
#'                 rtMin=c(995., 997.), rtMax=c(1005.,1007.),
#'                 sampling=250, sampleColour=c('blue', 'red'), verbose=FALSE)
#'
peakPantheR_plotEICFit  <- function(ROIDataPointSampleList,
                                    curveFitSampleList = NULL,
                                    rtMin = NULL,
                                    rtMax = NULL,
                                    sampling = 250,
                                    sampleColour = NULL,
                                    verbose = TRUE) {
    # Check input
    resInp <- plotEICFit_checkInput(ROIDataPointSampleList, curveFitSampleList,
                                            rtMin, rtMax, sampleColour, verbose)
    nbSpl <- resInp$nbSpl
    plotFit <- resInp$plotFit
    colourSpl <- resInp$colourSpl
    
    # Prepare data
    resPrep <- plotEICFit_prepData(ROIDataPointSampleList, curveFitSampleList,
                                    rtMin, rtMax, sampling, plotFit)
    input_EIC <- resPrep$input_EIC
    input_fit <- resPrep$input_fit


    # Plot raw spectra and curve fit
    # init plot
    p_spec <- ggplot2::ggplot(NULL, ggplot2::aes(x),
                            environment = environment()) +
        ggplot2::theme_bw() + ggplot2::xlab("Retention Time (sec)") +
        ggplot2::ylab("Intensity") +
        ggplot2::scale_y_continuous(expand = c(0.01, 0.01)) +
        ggplot2::scale_color_manual(values = colourSpl) +
        ggplot2::theme(legend.position = "none")
    
    # plot EIC
    p_spec <- p_spec + ggplot2::geom_line(data = input_EIC,
                        ggplot2::aes(x = rt, y = int, group = factor(specID),
                                    colour = factor(specID)))
    
    # plot curve fit
    if (plotFit) {
        p_spec <- p_spec + ggplot2::geom_line(data = input_fit,
                        ggplot2::aes(x = rt, y = int, group = factor(specID),
                            colour = factor(specID)), linetype = "dashed")
    }
    
    return(p_spec)
}


# -----------------------------------------------------------------------------
# peakPantheR_plotEICFit helper functions

# Check input
plotEICFit_checkInput <- function(ROIDataPointSampleList, curveFitSampleList,
                                    rtMin, rtMax, sampleColour, verbose) {
    # Check input in case ROIDataPointSampleList is not a list
    if (!is(ROIDataPointSampleList, "list")) {
        stop("Err",
        "or: \"ROIDataPointSampleList\" must be a list of data.frame")
    }
    nbSpl <- length(ROIDataPointSampleList)

    # check curveFitSampleList, rtMin and rtMax
    plotFit <- FALSE
    if (!is.null(curveFitSampleList) & !is.null(rtMin) & !is.null(rtMax)) {
        if ((nbSpl == length(curveFitSampleList)) & (nbSpl == length(rtMin)) &
            (nbSpl == length(rtMax))) {
            plotFit <- TRUE
        } else {
            stop('"\"curveFitSampleList\", \"rtMin\", \"rtMax\" and ',
                        '\"ROIDataPointSampleList\" must be the same length')
        }
    } else {
        if (verbose) {
            message('\"curveFitSampleList\", \"rtMin\" or \"rtMax\" no',
                            ' provided, curveFit will not be plotted')
        }
    }

    # set default colour (add a sample color ID that will be match in the plot)
    colourSpl <- rep("black", nbSpl)
    if (!is.null(sampleColour)) {
        if (nbSpl == length(sampleColour)) {
            colourSpl <- sampleColour
        } else {
            if (verbose) {
                message('War','ning: sampleColour length must match the ',
                        'number of samples; default colour used')
            }
        }
    }
    sampleIDColour <- paste("spl", seq(1, nbSpl), sep = "")
    names(colourSpl) <- sampleIDColour

    return(list(nbSpl=nbSpl, plotFit=plotFit, colourSpl=colourSpl))
}
# Prepare data
plotEICFit_prepData <- function(ROIDataPointSampleList, curveFitSampleList,
                                rtMin, rtMax, sampling, plotFit) {
    # raw spectra EICs
    all_EIC <- lapply(seq_along(ROIDataPointSampleList), function(x) {
        # generate ion chromatogram and add sampleIDColour
        tmp_EIC <- generateIonChromatogram(ROIDataPointSampleList[[x]],
                                            aggregationFunction = "sum")
        tmp_EIC <- cbind(tmp_EIC, specID = rep(paste("spl", x, sep = ""),
                                                    nrow(tmp_EIC)),
                        stringsAsFactors = FALSE)
    })
    input_EIC <- do.call(rbind, all_EIC)

    # curve fit
    if (plotFit) {
        all_fit <- lapply(seq_along(ROIDataPointSampleList), function(x) {
            # check rtMin, rtMax and curveFit exist, project curveFit
            if (!(is.na(rtMin[x])) & !(is.na(rtMax[x])) &
                all(!is.na(curveFitSampleList[[x]]))) {
                grid_rt <- seq(from = rtMin[x], to = rtMax[x],
                                by = ((rtMax[x] - rtMin[x])/(sampling - 1)))
                tmp_fit <- data.frame(rt = grid_rt,
                    int = predictCurve(curveFitSampleList[[x]], x = grid_rt))
                tmp_fit <- cbind(tmp_fit, specID = rep(paste("spl", x, sep=""),
                                                        nrow(tmp_fit)),
                            stringsAsFactors = FALSE)
            }
        })
        input_fit <- do.call(rbind, all_fit)
        # catch no curve fit left to plot
        if (is.null(input_fit)) {
            input_fit <- data.frame(matrix(, ncol = 3, nrow = 0,
                            dimnames = list(c(), c("rt", "int", "specID"))))
        }
    } else { input_fit <- NULL }

    return(list(input_EIC=input_EIC, input_fit=input_fit))
}

context('annotationDiagnosticMultiplot()')

## Test the generation of multiplot for each compound

skip_if_not_installed('faahKO',  minimum_version = '1.18.0')
library(faahKO)

# remove Rplots.pdf created by ggplot2
on.exit( tryCatch({ file.remove('./Rplots.pdf') }, error=function(e){ invisible() }, warning=function(w){ invisible() }) )


## Input data
# spectraPaths
input_spectraPaths  <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                         system.file('cdf/KO/ko16.CDF', package = "faahKO"),
                         system.file('cdf/KO/ko18.CDF', package = "faahKO"))

# targetFeatTable
input_targetFeatTable     <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390., 522.194778, 522.2, 522.205222)
input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440., 496.195038, 496.2, 496.204962)
input_targetFeatTable[,c(3:8)] <- sapply(input_targetFeatTable[,c(3:8)], as.numeric)

# acquisitionTime
input_acquisitionTime <- as.POSIXct(c(Sys.time(), Sys.time()+900, Sys.time()+1800))

# peakFit
# 1
cFit1.1         <- list(amplitude=162404.8057918259, center=3341.888, sigma=0.078786133031045896, gamma=0.0018336101984172684, fitStatus=2, curveModel="skewedGaussian")
class(cFit1.1)  <- 'peakPantheR_curveFit'
cFit1.2         <- list(amplitude=199249.10572753669, center=3382.577, sigma=0.074904415304607966, gamma=0.0011471899372353885, fitStatus=2, curveModel="skewedGaussian")
class(cFit1.2)  <- 'peakPantheR_curveFit'
# 2
cFit2.1         <- list(amplitude=124090.83425474487, center=3359.102, sigma=0.071061541060964212, gamma=0.0018336072657203239, fitStatus=2, curveModel="skewedGaussian")
class(cFit2.1)  <- 'peakPantheR_curveFit'
cFit2.2         <- list(amplitude=151407.23415130575, center=3399.791, sigma=0.063753866057052563, gamma=0.001676782834598999, fitStatus=2, curveModel="skewedGaussian")
class(cFit2.2)  <- 'peakPantheR_curveFit'
# 3
cFit3.1         <- list(amplitude=122363.51256736703, center=3362.233, sigma=0.075489598945304492, gamma=0.0025160536725299734, fitStatus=2, curveModel="skewedGaussian")
class(cFit3.1)  <- 'peakPantheR_curveFit'
cFit3.2         <- list(amplitude=204749.86097918145, center=3409.182, sigma=0.075731781812843249, gamma=0.0013318670577834328, fitStatus=2, curveModel="skewedGaussian")
class(cFit3.2)  <- 'peakPantheR_curveFit'
input_peakFit   <- list(list(cFit1.1, cFit1.2), list(cFit2.1, cFit2.2), list(cFit3.1, cFit3.2))

# dataPoint
tmp_raw_data1  	  <- MSnbase::readMSData(input_spectraPaths[1], centroided=TRUE, mode='onDisk')
ROIDataPoints1    <- extractSignalRawData(tmp_raw_data1, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
tmp_raw_data2  	  <- MSnbase::readMSData(input_spectraPaths[2], centroided=TRUE, mode='onDisk')
ROIDataPoints2    <- extractSignalRawData(tmp_raw_data2, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
tmp_raw_data3  	  <- MSnbase::readMSData(input_spectraPaths[3], centroided=TRUE, mode='onDisk')
ROIDataPoints3    <- extractSignalRawData(tmp_raw_data3, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
input_dataPoints  <- list(ROIDataPoints1, ROIDataPoints2, ROIDataPoints3)

# input annotationDiagnosticPlotList
# compound 1
expected_EICFit1                  <- peakPantheR_plotEICFit(ROIDataPointSampleList = list(ROIDataPoints1[[1]], ROIDataPoints2[[1]], ROIDataPoints3[[1]]),
                                                            curveFitSampleList = list(cFit1.1, cFit1.2, cFit1.2),
                                                            rtMin = c(3309.7589296586070, 3326.1063495851854, 3333.8625894557053),
                                                            rtMax = c(3385.4098874628098, 3407.2726475892355, 3407.4362838927614),
                                                            sampling = 250,
                                                            sampleColour = NULL,
                                                            verbose = FALSE)
expected_rtPeakwidthVert1         <- peakPantheR_plotPeakwidth(apexValue = c(3346.8277590361445, 3365.102, 3368.233),
                                                               widthMin = c(3309.7589296586070, 3326.1063495851854, 3333.8625894557053),
                                                               widthMax = c(3385.4098874628098, 3407.2726475892355, 3407.4362838927614),
                                                               acquTime = input_acquisitionTime,
                                                               sampleColour = NULL,
                                                               varName = 'Retention Time (sec)',
                                                               rotateAxis = TRUE,
                                                               verbose = FALSE)
expected_rtPeakwidthHorzRunOrder1 <- peakPantheR_plotPeakwidth(apexValue = c(3346.8277590361445, 3365.102, 3368.233),
                                                               widthMin = c(3309.7589296586070, 3326.1063495851854, 3333.8625894557053),
                                                               widthMax = c(3385.4098874628098, 3407.2726475892355, 3407.4362838927614),
                                                               acquTime = input_acquisitionTime,
                                                               sampleColour = NULL,
                                                               varName = 'Retention Time (sec)',
                                                               rotateAxis = FALSE,
                                                               verbose = FALSE)
expected_mzPeakwidthHorzRunOrder1 <- peakPantheR_plotPeakwidth(apexValue = c(522.20001220703125, 522.20001220703125, 522.20001220703125),
                                                               widthMin = c(522.194778, 522.194778, 522.194778),
                                                               widthMax = c(522.205222, 522.205222, 522.205222),
                                                               acquTime = input_acquisitionTime,
                                                               sampleColour = NULL,
                                                               varName = 'm/z',
                                                               rotateAxis = FALSE,
                                                               verbose = FALSE)
expected_areaRunOrder1            <- peakPantheR_plotPeakwidth(apexValue = c(26133726.681124408, 24545301.622835573, 21447174.404490683),
                                                               widthMin = NULL,
                                                               widthMax = NULL,
                                                               acquTime = input_acquisitionTime,
                                                               sampleColour = NULL,
                                                               varName = 'Peak Area',
                                                               rotateAxis = FALSE,
                                                               verbose = FALSE)
expected_rtHistogram1             <- plotHistogram(var = c(3346.8277590361445, 3365.102, 3368.233), 
                                                   varName='Retention Time (sec)',
                                                   density=TRUE)
expected_mzHistogram1             <- plotHistogram(var = c(522.20001220703125, 522.20001220703125, 522.20001220703125),
                                                   varName='m/z',
                                                   density=TRUE)
expected_areaHistogram1           <- plotHistogram(var = c(26133726.681124408, 24545301.622835573, 21447174.404490683),
                                                   varName='Peak Area', 
                                                   density=TRUE)
# compound 2
expected_EICFit2                  <- peakPantheR_plotEICFit(ROIDataPointSampleList = list(ROIDataPoints1[[2]], ROIDataPoints2[[2]], ROIDataPoints3[[2]]),
                                                            curveFitSampleList = list(cFit1.2, cFit2.2, cFit3.2),
                                                            rtMin = c(3345.3766648628907, 3365.0238566258713, 3373.3998828113113),
                                                            rtMax = c(3428.2788374983961, 3453.4049569205681, 3454.4490330927388),
                                                            sampling = 250,
                                                            sampleColour = NULL,
                                                            verbose = FALSE)
expected_rtPeakwidthVert2         <- peakPantheR_plotPeakwidth(apexValue = c(3386.5288072289159, 3405.791, 3413.4952530120481),
                                                               widthMin = c(3345.3766648628907, 3365.0238566258713, 3373.3998828113113),
                                                               widthMax = c(3428.2788374983961, 3453.4049569205681, 3454.4490330927388),
                                                               acquTime = NULL,
                                                               sampleColour = NULL,
                                                               varName = 'Retention Time (sec)',
                                                               rotateAxis = TRUE,
                                                               verbose = FALSE)
expected_rtPeakwidthHorzRunOrder2 <- peakPantheR_plotPeakwidth(apexValue = c(3386.5288072289159, 3405.791, 3413.4952530120481),
                                                               widthMin = c(3345.3766648628907, 3365.0238566258713, 3373.3998828113113),
                                                               widthMax = c(3428.2788374983961, 3453.4049569205681, 3454.4490330927388),
                                                               acquTime = input_acquisitionTime,
                                                               sampleColour = NULL,
                                                               varName = 'Retention Time (sec)',
                                                               rotateAxis = FALSE,
                                                               verbose = FALSE)
expected_mzPeakwidthHorzRunOrder2 <- peakPantheR_plotPeakwidth(apexValue = c(496.20001220703125, 496.20001220703125, 496.20001220703125),
                                                               widthMin = c(496.20001220703125, 496.195038, 496.195038),
                                                               widthMax = c(496.20001220703125, 496.204962, 496.204962),
                                                               acquTime = input_acquisitionTime,
                                                               sampleColour = NULL,
                                                               varName = 'm/z',
                                                               rotateAxis = FALSE,
                                                               verbose = FALSE)
expected_areaRunOrder2            <- peakPantheR_plotPeakwidth(apexValue = c(35472141.333024293, 37207579.286265120, 35659353.614476241),
                                                               widthMin = NULL,
                                                               widthMax = NULL,
                                                               acquTime = input_acquisitionTime,
                                                               sampleColour = NULL,
                                                               varName = 'Peak Area',
                                                               rotateAxis = FALSE,
                                                               verbose = FALSE)
expected_rtHistogram2             <- plotHistogram(var = c(3386.5288072289159, 3405.791, 3413.4952530120481), 
                                                   varName='Retention Time (sec)',
                                                   density=TRUE)
expected_mzHistogram2             <- plotHistogram(var = c(496.20001220703125, 496.20001220703125, 496.20001220703125),
                                                   varName='m/z',
                                                   density=TRUE)
expected_areaHistogram2           <- plotHistogram(var = c(35472141.333024293, 37207579.286265120, 35659353.614476241),
                                                   varName='Peak Area', 
                                                   density=TRUE)
# annotationDiagnosticPlotList
input_annotationDiagnosticPlotList <- list(list(EICFit = expected_EICFit1, 
                                                rtPeakwidthVert = expected_rtPeakwidthVert1, 
                                                rtPeakwidthHorzRunOrder = expected_rtPeakwidthHorzRunOrder1,
                                                mzPeakwidthHorzRunOrder = expected_mzPeakwidthHorzRunOrder1,
                                                areaRunOrder = expected_areaRunOrder1,
                                                rtHistogram = expected_rtHistogram1,
                                                mzHistogram = expected_mzHistogram1,
                                                areaHistogram = expected_areaHistogram1,
                                                title = 'test compound 1'),
                                           list(EICFit = expected_EICFit2, 
                                                rtPeakwidthVert = expected_rtPeakwidthVert2, 
                                                rtPeakwidthHorzRunOrder = expected_rtPeakwidthHorzRunOrder2,
                                                mzPeakwidthHorzRunOrder = expected_mzPeakwidthHorzRunOrder2,
                                                areaRunOrder = expected_areaRunOrder2,
                                                rtHistogram = expected_rtHistogram2,
                                                mzHistogram = expected_mzHistogram2,
                                                areaHistogram = expected_areaHistogram2,
                                                title = 'test compound 2'))



test_that('default multiplot', {
  # input
  input_plots       <- input_annotationDiagnosticPlotList
  # expected
  gtable_class      <- sort(c("gtable", "gTree", "grob", "gDesc"))
  
  # results (output, warnings and messages)
  result_multiplot  <- evaluate_promise(annotationDiagnosticMultiplot(input_plots))

  # Check plots generated
  expect_equal(length(result_multiplot$result), 2)
  expect_equal(sort(class(result_multiplot$result[[1]])), gtable_class)
  expect_equal(length(result_multiplot$result[[1]]), 9)
  expect_equal(sort(class(result_multiplot$result[[2]])), gtable_class)
  expect_equal(length(result_multiplot$result[[2]]), 9)
})

test_that('raise warnings/error', {
  # expected
  gtable_class      <- sort(c("gtable", "gTree", "grob", "gDesc"))
  expected_message  <- "Required plots missing for compound #1\n"
  
  # missing EICFit for cpd 1
  input_missEICFit      <- input_annotationDiagnosticPlotList
  input_missEICFit[[1]] <- input_missEICFit[[1]][c("rtPeakwidthVert", "rtPeakwidthHorzRunOrder", "mzPeakwidthHorzRunOrder", "areaRunOrder", "rtHistogram", "mzHistogram", "areaHistogram", "title")]
  result_warn1          <- evaluate_promise(annotationDiagnosticMultiplot(input_missEICFit))
  expect_equal(result_warn1$message[[1]], expected_message)
  expect_equal(result_warn1$result[[1]], NULL)
  expect_equal(sort(class(result_warn1$result[[2]])), gtable_class)
  expect_equal(length(result_warn1$result[[2]]), 9)
  
  # missing rtPeakwidthVert for cpd 1
  input_missrtPeakwidthVert       <- input_annotationDiagnosticPlotList
  input_missrtPeakwidthVert[[1]]  <- input_missrtPeakwidthVert[[1]][c("EICFit", "rtPeakwidthHorzRunOrder", "mzPeakwidthHorzRunOrder", "areaRunOrder", "rtHistogram", "mzHistogram", "areaHistogram", "title")]
  result_warn2                    <- evaluate_promise(annotationDiagnosticMultiplot(input_missrtPeakwidthVert))
  expect_equal(result_warn2$message[[1]], expected_message)
  expect_equal(result_warn2$result[[1]], NULL)
  expect_equal(sort(class(result_warn2$result[[2]])), gtable_class)
  expect_equal(length(result_warn2$result[[2]]), 9)
  
  # missing rtPeakwidthHorzRunOrder for cpd 1
  input_missrtPeakwidthHorzRunOrder       <- input_annotationDiagnosticPlotList
  input_missrtPeakwidthHorzRunOrder[[1]]  <- input_missrtPeakwidthHorzRunOrder[[1]][c("EICFit", "rtPeakwidthVert", "mzPeakwidthHorzRunOrder", "areaRunOrder", "rtHistogram", "mzHistogram", "areaHistogram", "title")]
  result_warn3                            <- evaluate_promise(annotationDiagnosticMultiplot(input_missrtPeakwidthHorzRunOrder))
  expect_equal(result_warn3$message[[1]], expected_message)
  expect_equal(result_warn3$result[[1]], NULL)
  expect_equal(sort(class(result_warn3$result[[2]])), gtable_class)
  expect_equal(length(result_warn3$result[[2]]), 9)
  
  # missing mzPeakwidthHorzRunOrder for cpd 1
  input_missmzPeakwidthHorzRunOrder       <- input_annotationDiagnosticPlotList
  input_missmzPeakwidthHorzRunOrder[[1]]  <- input_missmzPeakwidthHorzRunOrder[[1]][c("EICFit", "rtPeakwidthVert", "rtPeakwidthHorzRunOrder", "areaRunOrder", "rtHistogram", "mzHistogram", "areaHistogram")]
  result_warn4                            <- evaluate_promise(annotationDiagnosticMultiplot(input_missmzPeakwidthHorzRunOrder))
  expect_equal(result_warn4$message[[1]], expected_message)
  expect_equal(result_warn4$result[[1]], NULL)
  expect_equal(sort(class(result_warn4$result[[2]])), gtable_class)
  expect_equal(length(result_warn4$result[[2]]), 9)
  
  # missing areaRunOrder for cpd 1
  input_missareaRunOrder      <- input_annotationDiagnosticPlotList
  input_missareaRunOrder[[1]] <- input_missareaRunOrder[[1]][c("EICFit", "rtPeakwidthVert", "rtPeakwidthHorzRunOrder", "mzPeakwidthHorzRunOrder", "rtHistogram", "mzHistogram", "areaHistogram", "title")]
  result_warn5                <- evaluate_promise(annotationDiagnosticMultiplot(input_missareaRunOrder))
  expect_equal(result_warn5$message[[1]], expected_message)
  expect_equal(result_warn5$result[[1]], NULL)
  expect_equal(sort(class(result_warn5$result[[2]])), gtable_class)
  expect_equal(length(result_warn5$result[[2]]), 9)
  
  # missing rtHistogram for cpd 1
  input_missrtHistogram      <- input_annotationDiagnosticPlotList
  input_missrtHistogram[[1]] <- input_missrtHistogram[[1]][c("EICFit", "rtPeakwidthVert", "rtPeakwidthHorzRunOrder", "mzPeakwidthHorzRunOrder", "areaRunOrder", "mzHistogram", "areaHistogram", "title")]
  result_warn6                <- evaluate_promise(annotationDiagnosticMultiplot(input_missrtHistogram))
  expect_equal(result_warn6$message[[1]], expected_message)
  expect_equal(result_warn6$result[[1]], NULL)
  expect_equal(sort(class(result_warn6$result[[2]])), gtable_class)
  expect_equal(length(result_warn6$result[[2]]), 9)
  
  # missing mzHistogram for cpd 1
  input_missmzHistogram      <- input_annotationDiagnosticPlotList
  input_missmzHistogram[[1]] <- input_missmzHistogram[[1]][c("EICFit", "rtPeakwidthVert", "rtPeakwidthHorzRunOrder", "mzPeakwidthHorzRunOrder", "areaRunOrder", "rtHistogram", "areaHistogram", "title")]
  result_warn7                <- evaluate_promise(annotationDiagnosticMultiplot(input_missmzHistogram))
  expect_equal(result_warn7$message[[1]], expected_message)
  expect_equal(result_warn7$result[[1]], NULL)
  expect_equal(sort(class(result_warn7$result[[2]])), gtable_class)
  expect_equal(length(result_warn7$result[[2]]), 9)
  
  # missing areaHistogram for cpd 1
  input_missareaHistogram      <- input_annotationDiagnosticPlotList
  input_missareaHistogram[[1]] <- input_missareaHistogram[[1]][c("EICFit", "rtPeakwidthVert", "rtPeakwidthHorzRunOrder", "mzPeakwidthHorzRunOrder", "areaRunOrder", "rtHistogram", "mzHistogram", "title")]
  result_warn8                <- evaluate_promise(annotationDiagnosticMultiplot(input_missareaHistogram))
  expect_equal(result_warn8$message[[1]], expected_message)
  expect_equal(result_warn8$result[[1]], NULL)
  expect_equal(sort(class(result_warn8$result[[2]])), gtable_class)
  expect_equal(length(result_warn8$result[[2]]), 9)
  
  # missing title for cpd 1
  input_misstitleHistogram      <- input_annotationDiagnosticPlotList
  input_misstitleHistogram[[1]] <- input_misstitleHistogram[[1]][c("EICFit", "rtPeakwidthVert", "rtPeakwidthHorzRunOrder", "mzPeakwidthHorzRunOrder", "areaRunOrder", "rtHistogram", "mzHistogram", "areaHistogram")]
  result_warn9                <- evaluate_promise(annotationDiagnosticMultiplot(input_misstitleHistogram))
  expect_equal(result_warn9$message[[1]], expected_message)
  expect_equal(result_warn9$result[[1]], NULL)
  expect_equal(sort(class(result_warn9$result[[2]])), gtable_class)
  expect_equal(length(result_warn9$result[[2]]), 9)
})

context('peakPantheRAnnotation_annotationDiagnosticPlots()')

## Test the generation of fit diagnostic plots

skip_if_not_installed('faahKO',  minimum_version = '1.18.0')
library(faahKO)

# remove Rplots.pdf created by ggplot2
on.exit( tryCatch({ file.remove('./Rplots.pdf') }, error=function(e){ invisible() }, warning=function(w){ invisible() }) )


## Input data
# spectraPaths
input_spectraPaths  <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                         system.file('cdf/KO/ko16.CDF', package = "faahKO"),
                         system.file('cdf/KO/ko18.CDF', package = "faahKO"))

# targetFeatTable
input_targetFeatTable     <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390., 522.194778, 522.2, 522.205222)
input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440., 496.195038, 496.2, 496.204962)
input_targetFeatTable[,c(3:8)] <- sapply(input_targetFeatTable[,c(3:8)], as.numeric)

# FIR
input_FIR     <- data.frame(matrix(vector(), 2, 4, dimnames=list(c(), c("rtMin", "rtMax", "mzMin", "mzMax"))), stringsAsFactors=FALSE)
input_FIR[1,] <- c(1., 2., 3., 4.)
input_FIR[2,] <- c(5., 6., 7., 8.)

# uROI
input_uROI      <- data.frame(matrix(vector(), 2, 6, dimnames=list(c(), c("rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
input_uROI[1,]  <- c(9., 10., 11., 12., 13., 14.)
input_uROI[2,]  <- c(15., 16., 17., 18., 19., 20.)

# TICs
input_TIC <- c(2410533091, 2524040155, 2332817115)

# cpdMetadata
input_cpdMetadata     <- data.frame(matrix(data=c('a','b',1,2), nrow=2, ncol=2, dimnames=list(c(),c('testcol1','testcol2')), byrow=FALSE), stringsAsFactors=FALSE)

# spectraMetadata
input_spectraMetadata <- data.frame(matrix(data=c('c','d','e',3,4,5), nrow=3, ncol=2, dimnames=list(c(),c('testcol1','testcol2')), byrow=FALSE), stringsAsFactors=FALSE)

# acquisitionTime
input_acquisitionTime <- c(as.character(Sys.time()), as.character(Sys.time()+900), as.character(Sys.time()+1800))

# peakTables
# 1
peakTable1     <- data.frame(matrix(vector(), 2, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
peakTable1[1,] <- c(TRUE, 3309.7589296586070, 3346.8277590361445, 3385.4098874628098, 522.194778, 522.20001220703125, 522.205222, 26133726.6811244078, 26133726, 889280, 901015.80529226747, FALSE, 0.023376160866574614, 1.93975903614455092, 1.0153573486330891, 1.0268238825675249)
peakTable1[2,] <- c(TRUE, 3345.3766648628907, 3386.5288072289159, 3428.2788374983961, 496.20001220703125, 496.20001220703125, 496.20001220703125, 35472141.3330242932, 35472141, 1128960, 1113576.69008227298, FALSE, 0.024601030353423384, 0.95180722891564074, 1.0053782620427065, 1.0093180792278085)
peakTable1[,c(1,12)]       <- sapply(peakTable1[,c(1,12)], as.logical)
peakTable1[,c(2:11,13:16)] <- sapply(peakTable1[,c(2:11,13:16)], as.numeric)
# 2
peakTable2     <- data.frame(matrix(vector(), 2, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
peakTable2[1,] <- c(TRUE, 3326.1063495851854, 3365.102, 3407.2726475892355, 522.194778, 522.20001220703125, 522.205222, 24545301.622835573, 24545301, 761664, 790802.2209998488, FALSE, 0.023376160866574614, 0.2139999999999, 1.0339153786516375, 1.0630802030537212)
peakTable2[2,] <- c(TRUE, 3365.0238566258713, 3405.791, 3453.4049569205681, 496.195038, 496.20001220703125, 496.204962, 37207579.286265120, 37207579, 1099264, 1098720.2929832144, FALSE, 0.024601030353423384, 20.2139999999999, 1.0839602450900523, 1.1717845972583161)
peakTable2[,c(1,12)]       <- sapply(peakTable2[,c(1,12)], as.logical)
peakTable2[,c(2:11,13:16)] <- sapply(peakTable2[,c(2:11,13:16)], as.numeric)
# 3
peakTable3     <- data.frame(matrix(vector(), 2, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
peakTable3[1,] <- c(TRUE, 3333.8625894557053, 3368.233, 3407.4362838927614, 522.194778, 522.20001220703125, 522.205222, 21447174.404490683, 21447174, 758336, 765009.9805796633, FALSE, 0.023376160866574614, 23.345000000000255, 1.0609102044546637, 1.1155310457756928)
peakTable3[2,] <- c(TRUE, 3373.3998828113113, 3413.4952530120481, 3454.4490330927388, 496.195038, 496.20001220703125, 496.204962, 35659353.614476241, 35659353, 1149440, 1145857.7611069249, TRUE, 0.024601030353423384, 27.918253012047899, 1.0081407426394933, 1.0143315197994494)
peakTable3[,c(1,12)]       <- sapply(peakTable3[,c(1,12)], as.logical)
peakTable3[,c(2:11, 13:16)] <- sapply(peakTable3[,c(2:11,13:16)], as.numeric)
input_peakTables <- list(peakTable1, peakTable2, peakTable3)

# peakFit
# 1
cFit1.1         <- list(amplitude=162404.8057918259, center=3341.888, sigma=0.078786133031045896, gamma=0.0018336101984172684, fitStatus=2, curveModel="skewedGaussian")
class(cFit1.1)  <- 'peakPantheR_curveFit'
cFit1.2         <- list(amplitude=199249.10572753669, center=3382.577, sigma=0.074904415304607966, gamma=0.0011471899372353885, fitStatus=2, curveModel="skewedGaussian")
class(cFit1.2)  <- 'peakPantheR_curveFit'
# 2
cFit2.1         <- list(amplitude=124090.83425474487, center=3359.102, sigma=0.071061541060964212, gamma=0.0018336072657203239, fitStatus=2, curveModel="skewedGaussian")
class(cFit2.1)  <- 'peakPantheR_curveFit'
cFit2.2         <- list(amplitude=151407.23415130575, center=3399.791, sigma=0.063753866057052563, gamma=0.001676782834598999, fitStatus=2, curveModel="skewedGaussian")
class(cFit2.2)  <- 'peakPantheR_curveFit'
# 3
cFit3.1         <- list(amplitude=122363.51256736703, center=3362.233, sigma=0.075489598945304492, gamma=0.0025160536725299734, fitStatus=2, curveModel="skewedGaussian")
class(cFit3.1)  <- 'peakPantheR_curveFit'
cFit3.2         <- list(amplitude=204749.86097918145, center=3409.182, sigma=0.075731781812843249, gamma=0.0013318670577834328, fitStatus=2, curveModel="skewedGaussian")
class(cFit3.2)  <- 'peakPantheR_curveFit'
input_peakFit   <- list(list(cFit1.1, cFit1.2), list(cFit2.1, cFit2.2), list(cFit3.1, cFit3.2))

# dataPoint
tmp_raw_data1  	  <- MSnbase::readMSData(input_spectraPaths[1], centroided=TRUE, mode='onDisk')
ROIDataPoints1    <- extractSignalRawData(tmp_raw_data1, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
tmp_raw_data2  	  <- MSnbase::readMSData(input_spectraPaths[2], centroided=TRUE, mode='onDisk')
ROIDataPoints2    <- extractSignalRawData(tmp_raw_data2, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
tmp_raw_data3  	  <- MSnbase::readMSData(input_spectraPaths[3], centroided=TRUE, mode='onDisk')
ROIDataPoints3    <- extractSignalRawData(tmp_raw_data3, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
input_dataPoints  <- list(ROIDataPoints1, ROIDataPoints2, ROIDataPoints3)

# Object, fully filled
filledAnnotation        <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable, FIR=input_FIR, uROI=input_uROI, useFIR=TRUE, uROIExist=TRUE, useUROI=TRUE, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata, acquisitionTime=input_acquisitionTime, TIC=input_TIC, peakTables=input_peakTables, dataPoints=input_dataPoints, peakFit=input_peakFit, isAnnotated=TRUE)


test_that('default diagnostic plots, no sampleColour, default sampling, verbose/no verbose', {
  # input
  input_annotation        <- filledAnnotation
  
  # expected
  # check on second compound (need verbose/no verbose for plot_env)
  # with verbose
  expected_EICFit2v       <- peakPantheR_plotEICFit(ROIDataPointSampleList = list(ROIDataPoints1[[2]], ROIDataPoints2[[2]], ROIDataPoints3[[2]]),
                                                    curveFitSampleList = list(cFit1.2, cFit2.2, cFit3.2),
                                                    rtMin = c(3345.3766648628907, 3365.0238566258713, 3373.3998828113113),
                                                    rtMax = c(3428.2788374983961, 3453.4049569205681, 3454.4490330927388),
                                                    sampling = 250,
                                                    sampleColour = NULL,
                                                    verbose = TRUE)
  expected_rtPeakwidth2v  <- peakPantheR_plotPeakwidth(apexValue = c(3386.5288072289159, 3405.791, 3413.4952530120481),
                                                       widthMin = c(3345.3766648628907, 3365.0238566258713, 3373.3998828113113),
                                                       widthMax = c(3428.2788374983961, 3453.4049569205681, 3454.4490330927388),
                                                       acquTime = NULL,
                                                       sampleColour = NULL,
                                                       varName = 'Retention Time (sec)',
                                                       rotateAxis = TRUE,
                                                       verbose = TRUE)
  expected_rtHistogram2   <- plotHistogram(var = c(3386.5288072289159, 3405.791, 3413.4952530120481), 
                                           varName='Retention Time (sec)',
                                           density=TRUE)
  # with verbose
  expected_EICFit2nv      <- peakPantheR_plotEICFit(ROIDataPointSampleList = list(ROIDataPoints1[[2]], ROIDataPoints2[[2]], ROIDataPoints3[[2]]),
                                                    curveFitSampleList = list(cFit1.2, cFit2.2, cFit3.2),
                                                    rtMin = c(3345.3766648628907, 3365.0238566258713, 3373.3998828113113),
                                                    rtMax = c(3428.2788374983961, 3453.4049569205681, 3454.4490330927388),
                                                    sampling = 250,
                                                    sampleColour = NULL,
                                                    verbose = FALSE)
  expected_rtPeakwidth2nv <- peakPantheR_plotPeakwidth(apexValue = c(3386.5288072289159, 3405.791, 3413.4952530120481),
                                                       widthMin = c(3345.3766648628907, 3365.0238566258713, 3373.3998828113113),
                                                       widthMax = c(3428.2788374983961, 3453.4049569205681, 3454.4490330927388),
                                                       acquTime = NULL,
                                                       sampleColour = NULL,
                                                       varName = 'Retention Time (sec)',
                                                       rotateAxis = TRUE,
                                                       verbose = FALSE)
  # message
  expected_message        <- c("Peakwidth values plotted\n", "Values plotted by input order\n", "x and y axis rotated\n", "Peakwidth values plotted\n", "Values plotted by run order\n", "Peakwidth values plotted\n", "Values plotted by run order\n", "Values plotted by run order\n", "Compound 1/2 done\n", "Peakwidth values plotted\n", "Values plotted by input order\n", "x and y axis rotated\n", "Peakwidth values plotted\n", "Values plotted by run order\n", "Peakwidth values plotted\n", "Values plotted by run order\n", "Values plotted by run order\n", "Compound 2/2 done\n")
  expected_plotNames      <- c('EICFit', 'rtPeakwidthVert', 'rtPeakwidthHorzRunOrder', 'mzPeakwidthHorzRunOrder', 'areaRunOrder', 'rtHistogram', 'mzHistogram', 'areaHistogram', 'title')
  
  # results (output, warnings and messages)
  result_diagnosticPlot1  <- evaluate_promise(annotationDiagnosticPlots(input_annotation, sampleColour=NULL, sampling=250, verbose=TRUE))
  result_diagnosticPlot2  <- evaluate_promise(annotationDiagnosticPlots(input_annotation, sampleColour=NULL, sampling=250, verbose=FALSE))
  
  # Check plots generated
  # verbose
  expect_equal(length(result_diagnosticPlot1$result), 2)
  expect_equal(length(result_diagnosticPlot1$result[[1]]), 9)
  expect_equal(names(result_diagnosticPlot1$result[[1]]), expected_plotNames)
  expect_equal(length(result_diagnosticPlot1$result[[2]]), 9)
  expect_equal(names(result_diagnosticPlot1$result[[2]]), expected_plotNames)
  expect_equal(result_diagnosticPlot1$result[[2]]$EICFit, expected_EICFit2v, check.environment=FALSE)
  expect_equal(result_diagnosticPlot1$result[[2]]$rtPeakwidthVert, expected_rtPeakwidth2v, check.environment=FALSE)
  expect_equal(result_diagnosticPlot1$result[[2]]$rtHistogram, expected_rtHistogram2, check.environment=FALSE)
  expect_equal(result_diagnosticPlot1$result[[2]]$title, 'ID-2 - Cpd 2')
  # no verbose
  expect_equal(length(result_diagnosticPlot2$result), 2)
  expect_equal(length(result_diagnosticPlot2$result[[1]]), 9)
  expect_equal(names(result_diagnosticPlot2$result[[1]]), expected_plotNames)
  expect_equal(length(result_diagnosticPlot2$result[[2]]), 9)
  expect_equal(names(result_diagnosticPlot2$result[[2]]), expected_plotNames)
  expect_equal(result_diagnosticPlot2$result[[2]]$EICFit, expected_EICFit2nv, check.environment=FALSE)
  expect_equal(result_diagnosticPlot2$result[[2]]$rtPeakwidthVert, expected_rtPeakwidth2nv, check.environment=FALSE)
  expect_equal(result_diagnosticPlot2$result[[2]]$rtHistogram, expected_rtHistogram2, check.environment=FALSE)
  expect_equal(result_diagnosticPlot2$result[[2]]$title, 'ID-2 - Cpd 2')
  
  # Check result messages
  # verbose
  expect_equal(length(result_diagnosticPlot1$messages), 18)
  expect_equal(result_diagnosticPlot1$messages, expected_message)
  # no verbose
  expect_equal(length(result_diagnosticPlot2$messages), 0)
})

test_that('default diagnostic plots, set sampleColour, change sampling, verbose/no verbose', {
  # input
  input_annotation  <- filledAnnotation
  input_colour      <- c('blue', 'green', 'red')
  
  # expected
  # check on second compound (need verbose/no verbose for plot_env)
  # with verbose
  expected_EICFit2v       <- peakPantheR_plotEICFit(ROIDataPointSampleList = list(ROIDataPoints1[[2]], ROIDataPoints2[[2]], ROIDataPoints3[[2]]),
                                                    curveFitSampleList = list(cFit1.2, cFit2.2, cFit3.2),
                                                    rtMin = c(3345.3766648628907, 3365.0238566258713, 3373.3998828113113),
                                                    rtMax = c(3428.2788374983961, 3453.4049569205681, 3454.4490330927388),
                                                    sampling = 10,
                                                    sampleColour = input_colour,
                                                    verbose = TRUE)
  expected_rtPeakwidth2v  <- peakPantheR_plotPeakwidth(apexValue = c(3386.5288072289159, 3405.791, 3413.4952530120481),
                                                       widthMin = c(3345.3766648628907, 3365.0238566258713, 3373.3998828113113),
                                                       widthMax = c(3428.2788374983961, 3453.4049569205681, 3454.4490330927388),
                                                       acquTime = NULL,
                                                       sampleColour = input_colour,
                                                       varName = 'Retention Time (sec)',
                                                       rotateAxis = TRUE,
                                                       verbose = TRUE)
  expected_rtHistogram2   <- plotHistogram(var = c(3386.5288072289159, 3405.791, 3413.4952530120481), 
                                           varName='Retention Time (sec)',
                                           density=TRUE)
  # with verbose
  expected_EICFit2nv      <- peakPantheR_plotEICFit(ROIDataPointSampleList = list(ROIDataPoints1[[2]], ROIDataPoints2[[2]], ROIDataPoints3[[2]]),
                                                    curveFitSampleList = list(cFit1.2, cFit2.2, cFit3.2),
                                                    rtMin = c(3345.3766648628907, 3365.0238566258713, 3373.3998828113113),
                                                    rtMax = c(3428.2788374983961, 3453.4049569205681, 3454.4490330927388),
                                                    sampling = 10,
                                                    sampleColour = input_colour,
                                                    verbose = FALSE)
  expected_rtPeakwidth2nv <- peakPantheR_plotPeakwidth(apexValue = c(3386.5288072289159, 3405.791, 3413.4952530120481),
                                                       widthMin = c(3345.3766648628907, 3365.0238566258713, 3373.3998828113113),
                                                       widthMax = c(3428.2788374983961, 3453.4049569205681, 3454.4490330927388),
                                                       acquTime = NULL,
                                                       sampleColour = input_colour,
                                                       varName = 'Retention Time (sec)',
                                                       rotateAxis = TRUE,
                                                       verbose = FALSE)
  # message
  expected_message        <- c("Peakwidth values plotted\n", "Values plotted by input order\n", "x and y axis rotated\n", "Peakwidth values plotted\n", "Values plotted by run order\n", "Peakwidth values plotted\n", "Values plotted by run order\n", "Values plotted by run order\n", "Compound 1/2 done\n", "Peakwidth values plotted\n", "Values plotted by input order\n", "x and y axis rotated\n", "Peakwidth values plotted\n", "Values plotted by run order\n", "Peakwidth values plotted\n", "Values plotted by run order\n", "Values plotted by run order\n", "Compound 2/2 done\n")
  expected_plotNames      <- c('EICFit', 'rtPeakwidthVert', 'rtPeakwidthHorzRunOrder', 'mzPeakwidthHorzRunOrder', 'areaRunOrder', 'rtHistogram', 'mzHistogram', 'areaHistogram', 'title')
  
  # results (output, warnings and messages)
  result_diagnosticPlot1  <- evaluate_promise(annotationDiagnosticPlots(input_annotation, sampleColour=input_colour, sampling=10, verbose=TRUE))
  result_diagnosticPlot2  <- evaluate_promise(annotationDiagnosticPlots(input_annotation, sampleColour=input_colour, sampling=10, verbose=FALSE))
  
  # Check plots generated
  # verbose
  expect_equal(length(result_diagnosticPlot1$result), 2)
  expect_equal(length(result_diagnosticPlot1$result[[1]]), 9)
  expect_equal(names(result_diagnosticPlot1$result[[1]]), expected_plotNames)
  expect_equal(length(result_diagnosticPlot1$result[[2]]), 9)
  expect_equal(names(result_diagnosticPlot1$result[[2]]), expected_plotNames)
  expect_equal(result_diagnosticPlot1$result[[2]]$EICFit, expected_EICFit2v, check.environment=FALSE)
  expect_equal(result_diagnosticPlot1$result[[2]]$rtPeakwidthVert, expected_rtPeakwidth2v, check.environment=FALSE)
  expect_equal(result_diagnosticPlot1$result[[2]]$rtHistogram, expected_rtHistogram2, check.environment=FALSE)
  expect_equal(result_diagnosticPlot1$result[[2]]$title, 'ID-2 - Cpd 2')
  # no verbose
  expect_equal(length(result_diagnosticPlot2$result), 2)
  expect_equal(length(result_diagnosticPlot2$result[[1]]), 9)
  expect_equal(names(result_diagnosticPlot2$result[[1]]), expected_plotNames)
  expect_equal(length(result_diagnosticPlot2$result[[2]]), 9)
  expect_equal(names(result_diagnosticPlot2$result[[2]]), expected_plotNames)
  expect_equal(result_diagnosticPlot2$result[[2]]$EICFit, expected_EICFit2nv, check.environment=FALSE)
  expect_equal(result_diagnosticPlot2$result[[2]]$rtPeakwidthVert, expected_rtPeakwidth2nv, check.environment=FALSE)
  expect_equal(result_diagnosticPlot2$result[[2]]$rtHistogram, expected_rtHistogram2, check.environment=FALSE)
  expect_equal(result_diagnosticPlot2$result[[2]]$title, 'ID-2 - Cpd 2')
  
  # Check result messages
  # verbose
  expect_equal(length(result_diagnosticPlot1$messages), 18)
  expect_equal(result_diagnosticPlot1$messages, expected_message)
  # no verbose
  expect_equal(length(result_diagnosticPlot2$messages), 0)
})

test_that('raise warnings/error', {
  # input object is not annotated
  input_warn1       <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable)
  expected_result1  <- list(NULL, NULL)
  expected_msg1     <- "Warning: the object has not been annotated, return an empty diagnostic plot list\n"
  result_warn1      <- evaluate_promise(annotationDiagnosticPlots(input_warn1))
  expect_equal(result_warn1$result, expected_result1)
  expect_equal(result_warn1$messages, expected_msg1)
})

context('peakPantheR_plotEICFit()')

# remove Rplots.pdf created by ggplot2
on.exit( tryCatch({ file.remove('./Rplots.pdf') }, error=function(e){ invisible() }, warning=function(w){ invisible() }) )


## Input data
# fake sample 1
# ROI data points
rt1             <- seq(990, 1010, by=20/250)
mz1             <- rep(522., length(rt1))
int1            <- (dnorm(rt1, mean=1000, sd=1.5) * 100) + 1
tmp_DataPoints1 <- data.frame(rt=rt1, mz=mz1, int=int1)
# fittedCurve
fit1            <- list(amplitude=37.068916502809756, center=999.3734222573454, sigma=0.58493182568124724, gamma=0.090582029276037035, fitStatus=2, curveModel="skewedGaussian")
class(fit1)     <- 'peakPantheR_curveFit'

# fake sample 2
# ROI data points
rt2             <- seq(990, 1010, by=20/250)
mz2             <- rep(522., length(rt2))
int2            <- (dnorm(rt2, mean=1002, sd=1.5) * 100) + 1
tmp_DataPoints2 <- data.frame(rt=rt2, mz=mz2, int=int2)
# fittedCurve
fit2            <- list(amplitude=37.073067416755556, center=1001.3736564832565, sigma=0.58496485738212201, gamma=0.090553713725151905, fitStatus=2, curveModel="skewedGaussian")
class(fit2)     <- 'peakPantheR_curveFit'



test_that('plot feature in 1 sample', {
	# generate plot
  result_plot1SplNoCol <- peakPantheR_plotEICFit(ROIDataPointSampleList=list(tmp_DataPoints1),
                                                 curveFitSampleList=list(fit1),
                                                 rtMin=995., rtMax=1005.,
                                                 sampling=250, sampleColour=NULL, verbose=FALSE)
  
  # Check plot properties
  expect_true(ggplot2::is_ggplot(result_plot1SplNoCol))
  expect_equal(result_plot1SplNoCol$labels$x, "Retention Time (sec)")
  expect_equal(result_plot1SplNoCol$labels$y, "Intensity")
  expect_equal(length(result_plot1SplNoCol), length(ggplot2::ggplot()))
})

test_that('plot feature in 1 sample, no curveFit with warning', {
  # Expected message
  expected_message  <- c("\"curveFitSampleList\", \"rtMin\" or \"rtMax\" no provided, curveFit will not be plotted\n")
  
  # generate plot
  result_plot1SplNoFitNoCol <- evaluate_promise(peakPantheR_plotEICFit(ROIDataPointSampleList=list(tmp_DataPoints1),
                                                                       curveFitSampleList=NULL,
                                                                       rtMin=995., rtMax=1005.,
                                                                       sampling=250, sampleColour=NULL, 
                                                                       verbose=TRUE))
  
  # Check messages confirming the replacements
  expect_equal(result_plot1SplNoFitNoCol$messages, expected_message)

  # Check plot properties
  expect_true(ggplot2::is_ggplot(result_plot1SplNoFitNoCol$result))
  expect_equal(result_plot1SplNoFitNoCol$result$labels$x, "Retention Time (sec)")
  expect_equal(result_plot1SplNoFitNoCol$result$labels$y, "Intensity")
  expect_equal(length(result_plot1SplNoFitNoCol$result), length(ggplot2::ggplot()))
})

test_that('plot feature in 2 samples, change colours and sampling', {
  # generate plot
  result_plot2SplWithCol <- peakPantheR_plotEICFit(ROIDataPointSampleList=list(tmp_DataPoints1, tmp_DataPoints2),
                                                   curveFitSampleList=list(fit1, fit2),
                                                   rtMin=c(995., 997.), rtMax=c(1005.,1007.),
                                                   sampling=300, sampleColour=c('blue', 'red'), verbose=FALSE)
  
  # Check plot properties
  expect_true(ggplot2::is_ggplot(result_plot2SplWithCol))
  expect_equal(result_plot2SplWithCol$labels$x, "Retention Time (sec)")
  expect_equal(result_plot2SplWithCol$labels$y, "Intensity")
  expect_equal(length(result_plot2SplWithCol), length(ggplot2::ggplot()))
})

test_that('plot feature in 2 samples, rtMin and/or rtMax have a NA (cannot plot fit)', {
  # generate plot
  result_plot2SplRTNA <- peakPantheR_plotEICFit(ROIDataPointSampleList=list(tmp_DataPoints1, tmp_DataPoints2),
                                                curveFitSampleList=list(fit1, fit2),
                                                rtMin=c(NA, 997.), rtMax=c(1005.,NA),
                                                sampling=250, sampleColour=NULL, verbose=FALSE)
  
  # Check plot properties
  expect_true(ggplot2::is_ggplot(result_plot2SplRTNA))
  expect_equal(result_plot2SplRTNA$labels$x, "Retention Time (sec)")
  expect_equal(result_plot2SplRTNA$labels$y, "Intensity")
  expect_equal(length(result_plot2SplRTNA), length(ggplot2::ggplot()))
})

test_that('no rtMin and/or rtMax, cannot plot fit warning', {
  # Expected message
  expected_message  <- c("\"curveFitSampleList\", \"rtMin\" or \"rtMax\" no provided, curveFit will not be plotted\n")
  
  # generate plot
  result_plotFitWarning <- evaluate_promise(peakPantheR_plotEICFit(ROIDataPointSampleList=list(tmp_DataPoints1, tmp_DataPoints2),
                                                                   curveFitSampleList=list(fit1, fit2),
                                                                   rtMin=NULL, rtMax=NULL,
                                                                   sampling=250, sampleColour=NULL, verbose=TRUE))
  
  # Check messages confirming the replacements
  expect_equal(result_plotFitWarning$messages, expected_message)
  
  # Check plot properties
  expect_true(ggplot2::is_ggplot(result_plotFitWarning$result))
  expect_equal(result_plotFitWarning$result$labels$x, "Retention Time (sec)")
  expect_equal(result_plotFitWarning$result$labels$y, "Intensity")
  expect_equal(length(result_plotFitWarning$result), length(ggplot2::ggplot()))
})

test_that('sampleColour length warning', {
  # Expected message
  expected_message <- c("Warning: sampleColour length must match the number of samples; default colour used\n")
  
  # generate plot
  result_plotColourWarning <- evaluate_promise(peakPantheR_plotEICFit(ROIDataPointSampleList=list(tmp_DataPoints1),
                                                                      curveFitSampleList=list(fit1),
                                                                      rtMin=995., rtMax=1005., sampling=250,
                                                                      sampleColour=c('green','purple'),
                                                                      verbose=TRUE))
  
  # check messages confirming the replacements
  expect_equal(result_plotColourWarning$messages, expected_message)
  
  # Check plot properties
  expect_true(ggplot2::is_ggplot(result_plotColourWarning$result))
  expect_equal(result_plotColourWarning$result$labels$x, "Retention Time (sec)")
  expect_equal(result_plotColourWarning$result$labels$y, "Intensity")
  expect_equal(length(result_plotColourWarning$result), length(ggplot2::ggplot()))
})

test_that('raise errors', {
  # ROIDataPointSampleList is not a list
  msg1    <- c('Error: "ROIDataPointSampleList" must be a list of data.frame')
  expect_error(peakPantheR_plotEICFit('not a list', rtMin=995., rtMax=1005., sampling=250, curveFitSampleList=list(fit1), sampleColour=NULL, verbose=FALSE), msg1, fixed=TRUE)
  
  # ROIDataPointSampleList length is wrong
  msg2    <- c('"curveFitSampleList", "rtMin", "rtMax" and "ROIDataPointSampleList" must be the same length')
  expect_error(peakPantheR_plotEICFit(list(tmp_DataPoints1, tmp_DataPoints2), curveFitSampleList=list(fit1), rtMin=995., rtMax=1005., sampling=250, sampleColour=NULL, verbose=FALSE), msg2, fixed=TRUE)
  
  # rtMin length is wrong
  msg3    <- c('"curveFitSampleList", "rtMin", "rtMax" and "ROIDataPointSampleList" must be the same length')
  expect_error(peakPantheR_plotEICFit(list(tmp_DataPoints1), curveFitSampleList=list(fit1), rtMin=c(995.,997.), rtMax=1005., sampling=250, sampleColour=NULL, verbose=FALSE), msg3, fixed=TRUE)
  
  # rtmMax length is wrong
  msg4    <- c('"curveFitSampleList", "rtMin", "rtMax" and "ROIDataPointSampleList" must be the same length')
  expect_error(peakPantheR_plotEICFit(list(tmp_DataPoints1), curveFitSampleList=list(fit1), rtMin=995., rtMax=c(1005., 1006.), sampling=250, sampleColour=NULL, verbose=FALSE), msg4, fixed=TRUE)
  
  # curveFitSampleList length is wrong
  msg5    <- c('"curveFitSampleList", "rtMin", "rtMax" and "ROIDataPointSampleList" must be the same length')
  expect_error(peakPantheR_plotEICFit(list(tmp_DataPoints1), curveFitSampleList=list(fit1,fit2), rtMin=995., rtMax=c(1005.,1007.), sampleColour=NULL, verbose=FALSE), msg5, fixed=TRUE)
})

# ui.R

# peakPantheR-App
# Based on peakPantheR v1.15.0, R >= 4.3, shiny >= 1.0.5, bslib
# National Phenome Centre
# 02/10/2023
# Licensed under GPLv3	
#
# Copyright (C) {2023}  {National Phenome Centre}
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#require(shiny)
#require(bslib)

# NOTE:
#
# A page is 12 columns wide
#
# v2.0 with more tabs?
# plot + update uROI on screen?
# update + refit in UI?


shinyUI(fluidPage(theme = bslib::bs_theme(bootswatch = "yeti"), title='peakPantheR',
  navbarPage(title = textOutput("peakPantheR_ver"),
    inverse = FALSE,
    collapsible = TRUE,
    windowTitle = textOutput("peakPantheR_ver"),

    # -- About Tab --
    source(file.path("ui", "ui_about.R"),  local = TRUE)$value,

    # -- Import Tab --
    source(file.path("ui", "ui_import.R"),  local = TRUE)$value,

    # -- Run Tab --
    source(file.path("ui", "ui_run.R"),  local = TRUE)$value,

    # -- Diagnostic Tab --
    source(file.path("ui", "ui_diagnostic.R"),  local = TRUE)$value,

    # -- Results Tab --
    source(file.path("ui", "ui_results.R"),  local = TRUE)$value,

    # -- Export Tab --
    source(file.path("ui", "ui_export.R"),  local = TRUE)$value

  ) # end navbar (at the top)
) # end fluidPage
) # end shinyUI

# EXPORT - Tab panel  -------------------------------------------------------- #

tabPanel("Export results",

# Conditional UI based on import successfully initialised
  uiOutput("noImportForExportUI"),
  conditionalPanel(
    condition = "output.importDone=='yes'",
    fluidRow(

  ## Sidebar
      column(width=2,
        # Current annotation status
        uiOutput("showAnnotStatusExp")
      ), # end Sidebar column

  ## Main panel
      column(10,
        h3('Export Data', style="color:#3e648d;font-weight:bold"),

        # Row export inputs
        fluidRow(
          column(4,
            wellPanel(
              h4("Save annotation as .rData:"),
              span("Create a .RData file containing the annotation results as a peakPantheRAnnotation named ",em("annotationObject")," for future importation and analysis.", style="color:#666666"),
              uiOutput("dlRData")
            )  # end wellPanel
          ),   # end column
          column(8,
            wellPanel(
              fluidRow(
                column(5,
                  h4("Save input parameters as .CSV:"),
                  span("Save the ",
                           em("fit parameters")," defining the targeted features, the ",
                           em("files to process")," defining file paths and spectra metadata, as well as the",
                           em("targeted features metadata"),
                           ". These CSV files enable the reproduction of the current annotation, but does not contain annotation results", style="color:#666666"),
                ), # end column
                column(7,
                  uiOutput("dlCSVs")
                ) # end column
              ) # end fluidRow
            )   # end wellPanel
          ),    # end column

        ),  # end fluidRow Export inputs
        tags$hr(),
        br(),

        h3("Export Annotation Results and Diagnostic Plots",style="color:#3e648d;font-weight:bold"),
        span("Automated summarisation step (diagnostic and results), only possible if Shiny is running on a local machine.", style="color:#666666"), # See ",em("Code")," to run from the command line.

        # Row export results
        fluidRow(
          column(6,
            h4("Save Diagnostic Plots:"),
            span("Plots and save to disk a diagnotic plot for each targeted feature. Each spectra can be coloured based on one of the metadata column previously defined.", style="color:#666666")
          ), # end column
          column(6,
            h4("Save Annotation Values:"),
            span("Saves the annotation results to disk in multiple ",em(".csv")," files containing the compound metadata, spectra metadata and a table for each annotation (fit) property", em("(samples as rows and compounds as columns)"), ".", style="color:#666666")
          ) # end column
        ),  # end fluidRow Export results Text

        # The result export panel only exist if annotation took place
        column(12,
          uiOutput("exportDiagResUI"),
        tags$hr()
        )   # end column export results

      )     # end Main panel column
    )       # end fluidRow (sidebar + menu)
  )         # end conditional panel
)
# end EXPORT Tab panel  ------------------------------------------------------ #

# DIAGNOSTIC - Tab panel  ---------------------------------------------------- #

tabPanel("Diagnostic: plot & update",

  # Conditional UI based on import successfully initialised
  uiOutput("notAnnotForDiagUI"),
  conditionalPanel(
    condition = "output.AnnotationDone=='yes'",
    fluidRow(

  ## Sidebar
      column(width=2,
        # Current annotation status
        uiOutput("showAnnotStatusDiag")
      ), # end Sidebar column

  ## Main panel
      column(10,

        # Overall title
        h3('Diagnostic', style="color:#3e648d;font-weight:bold"),

        # Tabs
        shiny::tabsetPanel( id="diagnosticTabs", type="pills",

        # Annotation success statistics - TAB
          tabPanel("Annotation statistics",
            wellPanel(
              fluidRow(
                column(11, offset=1,
                  h4('Fitting results across samples', style="color:#3e648d;font-weight:bold"),
                  span('Summary of peaks found and Fallback Integration Regions', shiny::span(em('(FIR, if applicable)')), 'for each targeted feature across all samples:', style="color:black"),
                  HTML("<p style='color:black'>Targeted features with a low <i>ratio of peaks found</i>, a large <i>ppm error</i> or a high <i>RT deviation</i> across samples, might benefit from a correction of the targeted <code>rt</code> and <code>m/z</code> used as <code>uROI</code> for example</p>"),
                ) # end column
              )   # end fluidRow
            ),    # end wellPanel
            fluidRow(
              column(12,
                uiOutput("annotationStatisticsTable")
              )
            )     # end fluiRow (fit stat panel)
          ),      # end tabPanel

        # Automatic update uROI/FIR - TAB
          tabPanel("Update uROI/FIR",
            wellPanel(
              fluidRow(
                column(11, offset=1,
                  h4('Update uROI and FIR', style="color:#3e648d;font-weight:bold"),
                  span('Based on the fit results, updated ROI', shiny::span(em('(uROI)')), 'and fallback integration region', shiny::span(em('(FIR)')), 'can be automatically determined:', style="color:black"),
                  HTML("<ul>"),
                  HTML("<li style='color:black'><code>uROI</code> are established as the min/max (<code>rt</code> and <code>m/z</code>) of the found peaks (+/- 5% in RT)</li>"),
                  HTML("<li style='color:black'><code>FIR</code> are established as the median of found <code>rtMin</code>, <code>rtMax</code>, <code>mzMin</code>, <code>mzMax</code></li>"),
                  HTML("</ul>"),
                  div(actionButton("goDiagnosticUpdateUROIFIR", label="Update uROI/FIR",
                                   class="btn btn-primary btn-lg"), align="center")
                ) # end column
              )   # end fluidRow
            ),    # end wellPanel
            fluidRow(
              column(12,
                uiOutput("successUpdateDiagUI") # success message update uROI/FIR
              )
            )   # end fluiRow (success panel)
          ),    # end tabPanel

        # Show diagnostic plots - TAB
          tabPanel("Diagnostic plot",
            # control the plot
            uiOutput("diagPlotControlUI"),
            # plot output
            uiOutput("diagPlotResultUI")
          )#,    # end tabPanel

        # Show/modify updated parameters - TAB
        #  tabPanel("Final parameters",
        #    wellPanel(
        #      fluidRow(
        #        h3('! Under construction !')
        #        # TODO: UI uROI & FIR show and tweaks
        #      ) # end fluidRow
        #    )   # end wellPanel
        #  )     # end tabPanel
        )       # end tabsetPanels
      )   # end Main panel column
    )     # end fluidRow (sidebar + menu)
  )       # end conditional panel
)
# end DIAGNOSTIC Tab panel --------------------------------------------------- #

## ----biocstyle, echo = FALSE, results = "asis"--------------------------------
BiocStyle::markdown()

## ----echo = FALSE-------------------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)

## ----init, message = FALSE, echo = FALSE, results = "hide"--------------------
## Silently loading all packages
library(BiocStyle)
library(peakPantheR)
library(faahKO)
library(pander)
library(doParallel)
library(foreach)

## ----out.width = "700px", echo = FALSE----------------------------------------
knitr::include_graphics("figures/parallelAnnotation.png")

## ----out.width = "700px", echo = FALSE----------------------------------------
knitr::include_graphics("figures/parallelAnnotation_procedure.png")

## -----------------------------------------------------------------------------
library(faahKO)
## file paths
input_spectraPaths  <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                        system.file('cdf/KO/ko16.CDF', package = "faahKO"),
                        system.file('cdf/KO/ko18.CDF', package = "faahKO"))
input_spectraPaths

## ----eval = FALSE-------------------------------------------------------------
# # targetFeatTable
# input_targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#                         c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin",
#                             "mz", "mzMax"))), stringsAsFactors=FALSE)
# input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390.,
#                                 522.194778, 522.2, 522.205222)
# input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440.,
#                                 496.195038, 496.2, 496.204962)
# input_targetFeatTable[,c(3:8)] <- sapply(input_targetFeatTable[,c(3:8)],
#                                         as.numeric)

## ----results = "asis", echo = FALSE-------------------------------------------
# use pandoc for improved readability
input_targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), 
                        c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", 
                        "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390., 
                                522.194778, 522.2, 522.205222)
input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440., 
                                496.195038, 496.2, 496.204962)
input_targetFeatTable[,c(3:8)] <- sapply(input_targetFeatTable[,c(3:8)], 
                                        as.numeric)
rownames(input_targetFeatTable) <- NULL
pander::pandoc.table(input_targetFeatTable, digits = 9)

## ----eval=FALSE---------------------------------------------------------------
# # spectra Metadata
# input_spectraMetadata  <- data.frame(matrix(c("sample type 1", "sample type 2",
#                             "sample type 1"), 3, 1,
#                             dimnames=list(c(),c("sampleType"))),
#                             stringsAsFactors=FALSE)

## ----results = "asis", echo = FALSE-------------------------------------------
# use pandoc for improved readability
input_spectraMetadata  <- data.frame(matrix(c("sample type 1", "sample type 2", 
                                            "sample type 1"), 3, 1, 
                                            dimnames=list(c(),c("sampleType"))),
                                    stringsAsFactors=FALSE)
pander::pandoc.table(input_spectraMetadata)

## -----------------------------------------------------------------------------
library(peakPantheR)
init_annotation <- peakPantheRAnnotation(spectraPaths = input_spectraPaths,
                        targetFeatTable = input_targetFeatTable,
                        spectraMetadata = input_spectraMetadata)

## -----------------------------------------------------------------------------
init_annotation

## -----------------------------------------------------------------------------
# annotate files serially
annotation_result <- peakPantheR_parallelAnnotation(init_annotation, ncores=0,
                                                    curveModel='skewedGaussian',
                                                    verbose=TRUE)

# successful fit
nbSamples(annotation_result$annotation)
data_annotation   <- annotation_result$annotation
data_annotation

# list failed fit
annotation_result$failures

## -----------------------------------------------------------------------------
updated_annotation  <- annotationParamsDiagnostic(data_annotation, verbose=TRUE)

# uROI now exist
updated_annotation

## ----eval=FALSE---------------------------------------------------------------
# # create a colourScale based on the sampleType
# uniq_sType <- sort(unique(spectraMetadata(updated_annotation)$sampleType),
#                     na.last=TRUE)
# col_sType  <- unname( setNames(c('blue', 'red'),
#                 c(uniq_sType))[spectraMetadata(updated_annotation)$sampleType] )
# 
# # create a temporary location to save the diagnotic (otherwise provide the path
# # to the selected location)
# output_folder <- tempdir()
# 
# # output fit diagnostic to disk
# outputAnnotationDiagnostic(updated_annotation, saveFolder=output_folder,
#                             savePlots=TRUE, sampleColour=col_sType,
#                             verbose=TRUE, ncores=2)

## ----results = "asis", echo = FALSE-------------------------------------------
# use pandoc for improved readability, display the diagnostic results
tmp_csv <- data.frame(matrix(nrow=2,ncol=21,dimnames=list(c(), c('cpdID', 
        'cpdName', 'X', 'ROI_rt', 'ROI_mz','ROI_rtMin', 'ROI_rtMax', 
        'ROI_mzMin', 'ROI_mzMax', 'X', 'uROI_rtMin', 'uROI_rtMax', 'uROI_mzMin',
        'uROI_mzMax', 'uROI_rt', 'uROI_mz', 'X', 'FIR_rtMin', 'FIR_rtMax', 
        'FIR_mzMin', 'FIR_mzMax'))), stringsAsFactors=FALSE)
tmp_csv[1,] <- c('ID-1','Cpd 1', '|', 3344.888, 522.2, 3310., 3390., 522.194778,
                522.205222,'|', 3305.75893, 3411.436284, 522.194778, 522.205222,
                3344.888, 522.2, '|', 3326.10635, 3407.272648, 522.194778, 
                522.205222)
tmp_csv[2,] <- c('ID-2','Cpd 2', '|', 3385.577, 496.2, 3280., 3440., 496.195038,
                496.204962,'|',3337.376665, 3462.449033, 496.195038, 496.204962,
                3385.577, 496.2, '|', 3365.023857, 3453.404957, 496.195038, 
                496.204962)
tmp_csv[,-c(1,2,3,10,17)]  <- sapply(tmp_csv[,-c(1,2,3,10,17)], as.numeric)
colnames(tmp_csv) <- c('cpdID', 'cpdName', 'X', 'ROI_rt', 'ROI_mz','ROI_rtMin', 
                    'ROI_rtMax', 'ROI_mzMin', 'ROI_mzMax', 'X', 'uROI_rtMin', 
                    'uROI_rtMax', 'uROI_mzMin', 'uROI_mzMax', 'uROI_rt', 
                    'uROI_mz', 'X', 'FIR_rtMin', 'FIR_rtMax', 'FIR_mzMin', 
                    'FIR_mzMax')
pander::pandoc.table(tmp_csv, digits=9)

## ----out.width = "700px", echo = FALSE----------------------------------------
knitr::include_graphics(
    "figures/parallel_annotation_diagnostic_cpd1.png")

## ----results="asis", echo=FALSE-----------------------------------------------
# Example with constant correction.
rtCorrectionOutput <- retentionTimeCorrection(updated_annotation,
                            rtCorrectionReferences=c('ID-1'),
                            method='constant',
                            robust=FALSE,
                            rtWindowWidth=15,
                            diagnostic=TRUE)
updated_annotation <- rtCorrectionOutput$annotation
# The ggplot2 plot object
rtCorrectionOutput$plot

# Example with second degree polynomial, without using RANSAC
# # to obtain a robust fit
rtCorrectionOutput <- retentionTimeCorrection(updated_annotation,
                            rtCorrectionReferences=NULL,
                            method='polynomial',
                            params=list(polynomialOrder=2),
                            robust=FALSE, rtWindowWidth=15,
                            diagnostic=TRUE)

## ----eval=FALSE---------------------------------------------------------------
# update_csv_path <- '/path_to_new_csv/'
# 
# # load csv
# new_annotation <- peakPantheR_loadAnnotationParamsCSV(update_csv_path)
# #> uROIExist set to TRUE
# #> New peakPantheRAnnotation object initialised for 2 compounds
# 
# new_annotation
# #> An object of class peakPantheRAnnotation
# #>  2 compounds in 0 samples.
# #>   updated ROI exist (uROI)
# #>   does not use updated ROI (uROI)
# #>   does not use fallback integration regions (FIR)
# #>   is not annotated
# 
# new_annotation <- resetFIR(new_annotation)
# #> FIR will be reset with uROI values

## -----------------------------------------------------------------------------
## new files
new_spectraPaths   <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                        system.file('cdf/WT/wt15.CDF', package = "faahKO"),
                        system.file('cdf/KO/ko16.CDF', package = "faahKO"),
                        system.file('cdf/WT/wt16.CDF', package = "faahKO"),
                        system.file('cdf/KO/ko18.CDF', package = "faahKO"),
                        system.file('cdf/WT/wt18.CDF', package = "faahKO"))

new_spectraPaths

## -----------------------------------------------------------------------------
## new spectra metadata
new_spectraMetadata  <- data.frame(matrix(c("KO", "WT", "KO", "WT", "KO", "WT"),
                                        6, 1, dimnames=list(c(), c("Group"))), 
                                    stringsAsFactors=FALSE)

## ----results = "asis", echo = FALSE-------------------------------------------
# use pandoc for improved readability
new_spectraMetadata  <- data.frame(matrix(c("KO", "WT", "KO", "WT", "KO", "WT"),
                                        6, 1, dimnames=list(c(), c("Group"))), 
                                    stringsAsFactors=FALSE)
pander::pandoc.table(new_spectraMetadata)

## ----echo=FALSE---------------------------------------------------------------
new_annotation <- resetAnnotation(updated_annotation, 
                                spectraPaths=new_spectraPaths, 
                                spectraMetadata=new_spectraMetadata, 
                                useUROI=TRUE, useFIR=TRUE, verbose=FALSE) 

## ----eval=FALSE---------------------------------------------------------------
# ## add new samples to the annotation loaded from csv, useUROI, useFIR
# 
# new_annotation <- resetAnnotation(new_annotation, spectraPaths=new_spectraPaths,
#                                 spectraMetadata=new_spectraMetadata,
#                                 useUROI=TRUE, useFIR=TRUE)
# #> peakPantheRAnnotation object being reset:
# #>   Previous "ROI", "cpdID" and "cpdName" value kept
# #>   Previous "uROI" value kept
# #>   Previous "FIR" value kept
# #>   Previous "cpdMetadata" value kept
# #>   New "spectraPaths" value set
# #>   New "spectraMetadata" value set
# #>   Previous "uROIExist" value kept
# #>   New "useUROI" value set
# #>   New "useFIR" value set

## -----------------------------------------------------------------------------
new_annotation

## -----------------------------------------------------------------------------
# annotate files serially
new_annotation_result <- peakPantheR_parallelAnnotation(new_annotation, 
                                                        ncores=0, verbose=FALSE)

# successful fit
nbSamples(new_annotation_result$annotation)

final_annotation      <- new_annotation_result$annotation
final_annotation

# list failed fit
new_annotation_result$failures

## ----eval=FALSE---------------------------------------------------------------
# # create a colourScale based on the sampleType
# uniq_group <- sort(unique(spectraMetadata(final_annotation)$Group),na.last=TRUE)
# col_group  <- unname( setNames(c('blue', 'red'),
#                     c(uniq_sType))[spectraMetadata(final_annotation)$Group] )
# 
# # create a temporary location to save the diagnotic (otherwise provide the path
# # to the selected location)
# final_output_folder <- tempdir()
# 
# # output fit diagnostic to disk
# outputAnnotationDiagnostic(final_annotation, saveFolder=final_output_folder,
#                         savePlots=TRUE, sampleColour=col_group, verbose=TRUE)

## ----eval=FALSE---------------------------------------------------------------
# # peakTables for the first sample
# peakTables(final_annotation)[[1]]

## ----results = "asis", echo = FALSE-------------------------------------------
# use pandoc for improved readability
pander::pandoc.table(peakTables(final_annotation)[[1]])

## ----eval=FALSE---------------------------------------------------------------
# # Extract the found peak area for all compounds and all samples
# annotationTable(final_annotation, column='peakArea')

## ----results = "asis", echo = FALSE-------------------------------------------
# use pandoc for improved readability
pander::pandoc.table(annotationTable(final_annotation, column='peakArea'))

## ----eval=FALSE---------------------------------------------------------------
# # create a temporary location to save the diagnotic (otherwise provide the path
# # to the selected location)
# final_output_folder <- tempdir()
# 
# # save
# outputAnnotationResult(final_annotation, saveFolder=final_output_folder,
#                         annotationName='ProjectName', verbose=TRUE)
# #> Compound metadata saved at /final_output_folder/ProjectName_cpdMetadata.csv
# #> Spectra metadata saved at
# #>     /final_output_folder/ProjectName_spectraMetadata.csv
# #> Peak measurement "found" saved at /final_output_folder/ProjectName_found.csv
# #> Peak measurement "rtMin" saved at /final_output_folder/ProjectName_rtMin.csv
# #> Peak measurement "rt" saved at /final_output_folder/ProjectName_rt.csv
# #> Peak measurement "rtMax" saved at /final_output_folder/ProjectName_rtMax.csv
# #> Peak measurement "mzMin" saved at /final_output_folder/ProjectName_mzMin.csv
# #> Peak measurement "mz" saved at /final_output_folder/ProjectName_mz.csv
# #> Peak measurement "mzMax" saved at /final_output_folder/ProjectName_mzMax.csv
# #> Peak measurement "peakArea" saved at
# #>     /final_output_folder/ProjectName_peakArea.csv
# #> Peak measurement "maxIntMeasured" saved at
# #>     /final_output_folder/ProjectName_maxIntMeasured.csv
# #> Peak measurement "maxIntPredicted" saved at
# #>     /final_output_folder/ProjectName_maxIntPredicted.csv
# #> Peak measurement "is_filled" saved at
# #>     /final_output_folder/ProjectName_is_filled.csv
# #> Peak measurement "ppm_error" saved at
# #>     /final_output_folder/ProjectName_ppm_error.csv
# #> Peak measurement "rt_dev_sec" saved at
# #>     /final_output_folder/ProjectName_rt_dev_sec.csv
# #> Peak measurement "tailingFactor" saved at
# #>     /final_output_folder/ProjectName_tailingFactor.csv
# #> Peak measurement "asymmetryFactor" saved at
# #>     /final_output_folder/ProjectName_asymmetryFactor.csv
# #> Summary saved at /final_output_folder/ProjectName_summary.csv

## ----echo = FALSE-------------------------------------------------------------
devtools::session_info()

#' @import utils
.onLoad <- function(libname = find.package("peakPantheR"),
                    pkgname = "peakPantheR"){
    # CRAN Note avoidance
    if(getRversion() >= "2.15.1") {
        utils::globalVariables(
        # data.frame column names used in ggplot (cannot use aes_string due to
        # transformations to the column in aes())
        c("x","y","colr","rt","int","specID", "rt_dev_sec",
        "isReference", "predictedRtDrift")
        )
        invisible()
    }
}

.onAttach <- function(libname, pkgname) {
    packageStartupMessage(
    paste("\nThis is peakPantheR version",
            utils::packageVersion("peakPantheR"), "\n"))
}

#####################################################################
#####################################################################
## Constructor for \link{peakPantheRAnnotation-class}, initialising a new
## peakPantheRAnnotation with default values, \code{spectraPaths} and
## \code{targetFeatTable} will set the samples and compounds to process
#' @description \code{peakPantheRAnnotation()}: create an instance of the
#' \code{peakPantherAnnotation} class.
#'
#' @param spectraPaths NULL or a character vector of spectra file paths, to set
#' samples to process
#' @param targetFeatTable NULL or a \code{\link{data.frame}} of compounds to
#' target as rows and parameters as columns: \code{cpdID} (str), \code{cpdName}
#' (str), \code{rtMin} (float in seconds), \code{rt} (float in seconds, or
#' \emph{NA}), \code{rtMax} (float in seconds), \code{mzMin} (float), \code{mz}
#' (float or \emph{NA}), \code{mzMax} (float). Set compounds to target.
#' @param cpdID A character vector of compound IDs, of length number of
#' compounds
#' @param cpdName A character vector of compound names, of length number of
#' compounds
#' @param ROI A data.frame of Regions Of Interest (ROI) with compounds as
#' row and ROI parameters as columns: \code{rtMin} (float in seconds), \code{rt}
#' (float in seconds, or \emph{NA}), \code{rtMax} (float in seconds),
#' \code{mzMin} (float), \code{mz} (float or \emph{NA}), \code{mzMax} (float).
#' @param FIR A data.frame of Fallback Integration Regions (FIR) with
#' compounds as row and FIR parameters as columns: \code{rtMin} (float in
#' seconds),
#' \code{rtMax} (float in seconds), \code{mzMin} (float), \code{mzMax} (float).
#' @param uROI A data.frame of updated Regions Of Interest (uROI) with
#' compounds as row and uROI parameters as columns: \code{rtMin} (float in
#' seconds),
#' \code{rt} (float in seconds, or \emph{NA}), \code{rtMax} (float in seconds),
#' \code{mzMin} (float), \code{mz} (float or \emph{NA}), \code{mzMax} (float).
#' @param filepath A character vector of file paths, of length number of spectra
#' files
#' @param cpdMetadata A data.frame of compound metadata, with compounds as row
#' and metadata as columns
#' @param spectraMetadata A data.frame of sample metadata, with samples as row
#' and metadata as columns
#' @param acquisitionTime A character vector of acquisition date-time (converted
#' from POSIXct) or NA
#' @param uROIExist A logical stating if uROI have been set
#' @param useUROI A logical stating if uROI are to be used
#' @param useFIR A logical stating if FIR are to be used
#' @param TIC A numeric vector of TIC or NA, of length number of spectra files
#' @param peakTables A list of peakTable data.frame, of length number of spectra
#' files. Each peakTable data.frame has compounds as rows and peak annotation
#' results as columns.
#' @param dataPoints A list of length number of spectra files. Each list element
#' is a \emph{ROIsDataPoint} list of \code{data.frame} of raw data points for
#' each ROI/uROI (retention time 'rt', mass 'mz' and intensity 'int' (as column)
#' of each raw data points (as row))
#' @param peakFit A list of length number of spectra files. Each list element is
#' a \emph{curveFit} list of \code{peakPantheR_curveFit} or NA for each ROI
#' @param isAnnotated A logical stating in the annotation took place
peakPantheRAnnotation <- function(spectraPaths = NULL,
    targetFeatTable = NULL, cpdID = character(),
    cpdName = character(),
    ROI = data.frame(rtMin = numeric(), rt = numeric(), rtMax = numeric(),
        mzMin = numeric(), mz = numeric(), mzMax = numeric(),
        stringsAsFactors = FALSE),
    FIR = data.frame(rtMin = numeric(), rtMax = numeric(), mzMin = numeric(),
        mzMax = numeric(), stringsAsFactors = FALSE),
    uROI = data.frame(rtMin = numeric(), rt = numeric(), rtMax = numeric(),
        mzMin = numeric(), mz = numeric(), mzMax = numeric(),
        stringsAsFactors = FALSE),
    filepath = character(), cpdMetadata = data.frame(),
    spectraMetadata = data.frame(), acquisitionTime = character(),
    uROIExist = FALSE, useUROI = FALSE, useFIR = FALSE, TIC = numeric(),
    peakTables = list(), dataPoints = list(), peakFit = list(),
    isAnnotated = FALSE) {
    
    # set spectra if spectraPaths is provided
    if (!is.null(spectraPaths)) {
        initSpec <- initialize_spectraPaths(spectraPaths, acquisitionTime, TIC,
                            peakTables, dataPoints, peakFit, spectraMetadata)
        filepath <- initSpec$filepath
        acquisitionTime <- initSpec$acqTime
        TIC <- initSpec$TIC
        peakTables <- initSpec$pkTab
        dataPoints <- initSpec$dataPt
        peakFit <- initSpec$pkFit
        spectraMetadata <- initSpec$specMeta }
    
    # set compounds if targetFeatTable is provided
    if (!is.null(targetFeatTable)) {
        initTarg <- initialize_targetFeatTable(targetFeatTable, cpdMetadata,
                                                FIR, uROI, uROIExist, useUROI)
        cpdID <- initTarg$cpdID
        cpdName <- initTarg$cpdName
        cpdMetadata <- initTarg$cpdMeta
        ROI <- initTarg$ROI
        FIR <- initTarg$FIR
        uROI <- initTarg$uROI
        uROIExist <- initTarg$uROIExist
        useUROI <- initTarg$useUROI }
    
    # set the final values
    new("peakPantheRAnnotation", cpdID = cpdID, cpdName = cpdName, ROI = ROI,
        FIR = FIR, uROI = uROI, filepath = filepath, cpdMetadata = cpdMetadata,
        spectraMetadata = spectraMetadata, acquisitionTime = acquisitionTime,
        uROIExist = uROIExist, useUROI = useUROI, useFIR = useFIR, TIC = TIC,
        peakTables = peakTables, dataPoints = dataPoints, peakFit = peakFit,
        isAnnotated = isAnnotated)
}


# -----------------------------------------------------------------------------
# initialize_peakPantheRAnnotation helper functions

# Set using spectraPaths
initialize_spectraPaths <- function(spectraPaths, acquisitionTime, TIC,
                            peakTables, dataPoints, peakFit, spectraMetadata) {
    # check input is a vector of character
    if (!is.vector(spectraPaths) | is.list(spectraPaths) |
        !is.character(spectraPaths)) {
        stop("specified spectraPaths is not a vector of character") }

    # no duplicated spectraPaths
    if (length(spectraPaths) != length(unique(spectraPaths))) {
        stop("duplicated spectraPaths passed as input") }

    # load values and allocate size
    nbSpectra   <- length(spectraPaths)
    filepath    <- spectraPaths

    # set acquisitionTime default if no acquisitionTime passed in
    if (length(acquisitionTime) == 0) {
        acquisitionTime <- as.character(rep(NA, nbSpectra)) }

    # set TIC default if no TIC passed in
    if (length(TIC) == 0) {
        TIC <- as.numeric(rep(NA, nbSpectra)) }

    # set peakTables default if no peakTables passed in
    if (length(peakTables) == 0) {
        peakTables <- vector("list", nbSpectra) }

    # set dataPoints default if no dataPoints passed in
    if (length(dataPoints) == 0) {
        dataPoints <- vector("list", nbSpectra) }

    # set peakFit default if no peakFit passed in
    if (length(peakFit) == 0) {
        peakFit <- vector("list", nbSpectra) }

    # set spectraMetadata to the correct size if no spectraMetadata passed
    # in (at the correct size)
    if (dim(spectraMetadata)[1] != nbSpectra) {
        spectraMetadata <- data.frame(matrix(, nrow = nbSpectra, ncol = 0))}

    return(list(filepath=filepath, acqTime=acquisitionTime, TIC=TIC,
                pkTab=peakTables, dataPt=dataPoints, pkFit=peakFit,
                specMeta=spectraMetadata))
}
# Set using targetFeatTable
initialize_targetFeatTable <- function(targetFeatTable, cpdMetadata, FIR, uROI,
                                        uROIExist, useUROI){
    # check targetFeatTable
    initialize_checkTargetFeatTable(targetFeatTable)

    # load values and allocate size
    nbCompound  <- dim(targetFeatTable)[1]
    cpdID       <- targetFeatTable$cpdID
    cpdName     <- targetFeatTable$cpdName
    ROI         <- targetFeatTable[, c("rtMin", "rt", "rtMax", "mzMin", "mz",
                                        "mzMax")]
    # set cpdMetadata to the correct size if no cpdMetadata passed in (at the
    # correct size)
    if (dim(cpdMetadata)[1] != nbCompound) {
        cpdMetadata <- data.frame(matrix(, nrow = nbCompound, ncol = 0))
    }
    # only set FIR and uROI to the correct size if not provided in input (at the
    # correct size)
    if (dim(FIR)[1] != nbCompound) {
        FIR <- data.frame(rtMin = as.numeric(rep(NA, nbCompound)),
                            rtMax = as.numeric(rep(NA, nbCompound)),
                            mzMin = as.numeric(rep(NA, nbCompound)),
                            mzMax = as.numeric(rep(NA, nbCompound)),
                            stringsAsFactors = FALSE)
    }
    if (dim(uROI)[1] != nbCompound) {
        uROI <- data.frame(rtMin = as.numeric(rep(NA, nbCompound)),
                            rt = as.numeric(rep(NA, nbCompound)),
                            rtMax = as.numeric(rep(NA, nbCompound)),
                            mzMin = as.numeric(rep(NA, nbCompound)),
                            mz = as.numeric(rep(NA, nbCompound)),
                            mzMax = as.numeric(rep(NA, nbCompound)),
                            stringsAsFactors = FALSE)
        # if we reset, it doesn't exist
        uROIExist <- FALSE
        # if we reset, it can't be used
        useUROI <- FALSE
    }

    return(list(cpdID=cpdID, cpdName=cpdName, cpdMeta=cpdMetadata, ROI=ROI,
                FIR=FIR, uROI=uROI, uROIExist=uROIExist, useUROI=useUROI))
}
# Check targetFeatTable input
initialize_checkTargetFeatTable <- function(targetFeatTable) {

    # check input is data.frame
    if (!is(targetFeatTable, "data.frame")) {
        stop("specified targetFeatTable is not a data.frame")}

    # required columns are present
    if (!all(c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz",
                "mzMax") %in% colnames(targetFeatTable))) {
        stop("expected columns in targetFeatTable are \"cpdID\", ",
            "\"cpdName\", \"rtMin\", \"rt\", \"rtMax\", \"mzMin\", \"mz\" ",
            "and \"mzMax\"")
    }

    # column type
    if (dim(targetFeatTable)[1] != 0) {
        if (!is.character(targetFeatTable$cpdID[1])) {
            stop("targetFeatTable$cpdID must be character") }
        if (!is.character(targetFeatTable$cpdName[1])) {
            stop("targetFeatTable$cpdName must be character") }
        if (!is.numeric(targetFeatTable$rtMin[1])) {
            stop("targetFeatTable$rtMin must be numeric") }
        if (!(is.numeric(targetFeatTable$rt[1]) |
            is.na(targetFeatTable$rt[1]))) {
            stop("targetFeatTable$rt must be numeric or NA") }
        if (!is.numeric(targetFeatTable$rtMax[1])) {
            stop("targetFeatTable$rtMax must be numeric") }
        if (!is.numeric(targetFeatTable$mzMin[1])) {
            stop("targetFeatTable$mzMin must be numeric") }
        if (!(is.numeric(targetFeatTable$mz[1]) |
            is.na(targetFeatTable$mz[1]))) {
            stop("targetFeatTable$mz must be numeric or NA") }
        if (!is.numeric(targetFeatTable$mzMax[1])) {
            stop("targetFeatTable$mzMax must be numeric") }
    }
}

context('peakPantheRAnnotation_validObject-method()')

## Test the validObject method / valid_peakPantheRAnnotation function

skip_if_not_installed('faahKO',  minimum_version = '1.18.0')
library(faahKO)

## Input data
# spectraPaths
input_spectraPaths  <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                         system.file('cdf/KO/ko16.CDF', package = "faahKO"),
                         system.file('cdf/KO/ko18.CDF', package = "faahKO"))

# targetFeatTable
input_targetFeatTable     <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390., 522.194778, 522.2, 522.205222)
input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440., 496.195038, 496.2, 496.204962)
input_targetFeatTable[,c(3:8)] <- sapply(input_targetFeatTable[,c(3:8)], as.numeric)

# FIR
input_FIR  <- data.frame(rtMin=c(1,5), rtMax=c(2,6), mzMin=c(3,7), mzMax=c(4,8), stringsAsFactors=FALSE)

# uROI
input_uROI <- data.frame(rtMin=c(9,13), rt=c(9.5,13.5), rtMax=c(10,14), mzMin=c(11,15), mz=c(11.5,15.5), mzMax=c(12,16), stringsAsFactors = FALSE)

# TICs
input_TIC <- c(2410533091, 2524040155, 2332817115)

# cpdMetadata
input_cpdMetadata     <- data.frame(matrix(data=c('a','b',1,2), nrow=2, ncol=2, dimnames=list(c(),c('testcol1','testcol2')), byrow=FALSE), stringsAsFactors=FALSE)

# spectraMetadata
input_spectraMetadata <- data.frame(matrix(data=c('c','d','e',3,4,5), nrow=3, ncol=2, dimnames=list(c(),c('testcol1','testcol2')), byrow=FALSE), stringsAsFactors=FALSE)

# acquisitionTime
input_acquisitionTime <- c(as.character(Sys.time()), as.character(Sys.time()+900), as.character(Sys.time()+1800))

# peakTables
# 1
peakTable1     <- data.frame(matrix(vector(), 2, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
peakTable1[1,] <- c(TRUE, 3309.7589296586070, 3346.8277590361445, 3385.4098874628098, 522.194778, 522.20001220703125, 522.205222, 26133726.6811244078, 26133726, 889280, 901015.80529226747, FALSE, 0.023376160866574614, 1.93975903614455092, 1.0153573486330891, 1.0268238825675249)
peakTable1[2,] <- c(TRUE, 3345.3766648628907, 3386.5288072289159, 3428.2788374983961, 496.20001220703125, 496.20001220703125, 496.20001220703125, 35472141.3330242932, 35472141, 1128960, 1113576.69008227298, FALSE, 0.024601030353423384, 0.95180722891564074, 1.0053782620427065, 1.0093180792278085)
peakTable1[,c(1,12)]       <- sapply(peakTable1[,c(1,12)], as.logical)
peakTable1[,c(2:11,13:16)] <- sapply(peakTable1[,c(2:11,13:16)], as.numeric)
# 2
peakTable2     <- data.frame(matrix(vector(), 2, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
peakTable2[1,] <- c(TRUE, 3326.1063495851854, 3365.102, 3407.2726475892355, 522.194778, 522.20001220703125, 522.205222, 24545301.622835573, 24545301, 761664, 790802.2209998488, FALSE, 0.023376160866574614, 0.2139999999999, 1.0339153786516375, 1.0630802030537212)
peakTable2[2,] <- c(TRUE, 3365.0238566258713, 3405.791, 3453.4049569205681, 496.195038, 496.20001220703125, 496.204962, 37207579.286265120, 37207579, 1099264, 1098720.2929832144, FALSE, 0.024601030353423384, 20.2139999999999, 1.0839602450900523, 1.1717845972583161)
peakTable2[,c(1,12)]       <- sapply(peakTable2[,c(1,12)], as.logical)
peakTable2[,c(2:11,13:16)] <- sapply(peakTable2[,c(2:11,13:16)], as.numeric)
# 3
peakTable3     <- data.frame(matrix(vector(), 2, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
peakTable3[1,] <- c(TRUE, 3333.8625894557053, 3368.233, 3407.4362838927614, 522.194778, 522.20001220703125, 522.205222, 21447174.404490683, 21447174, 758336, 765009.9805796633, FALSE, 0.023376160866574614, 23.345000000000255, 1.0609102044546637, 1.1155310457756928)
peakTable3[2,] <- c(TRUE, 3373.3998828113113, 3413.4952530120481, 3454.4490330927388, 496.195038, 496.20001220703125, 496.204962, 35659353.614476241, 35659353, 1149440, 1145857.7611069249, TRUE, 0.024601030353423384, 27.918253012047899, 1.0081407426394933, 1.0143315197994494)
peakTable3[,c(1,12)]       <- sapply(peakTable3[,c(1,12)], as.logical)
peakTable3[,c(2:11, 13:16)] <- sapply(peakTable3[,c(2:11,13:16)], as.numeric)
input_peakTables <- list(peakTable1, peakTable2, peakTable3)

# peakFit
# 1
cFit1.1         <- list(amplitude=162404.8057918259, center=3341.888, sigma=0.078786133031045896, gamma=0.0018336101984172684, fitStatus=2, curveModel="skewedGaussian")
class(cFit1.1)  <- 'peakPantheR_curveFit'
cFit1.2         <- list(amplitude=199249.10572753669, center=3382.577, sigma=0.074904415304607966, gamma=0.0011471899372353885, fitStatus=2, curveModel="skewedGaussian")
class(cFit1.2)  <- 'peakPantheR_curveFit'
# 2
cFit2.1         <- list(amplitude=124090.83425474487, center=3359.102, sigma=0.071061541060964212, gamma=0.0018336072657203239, fitStatus=2, curveModel="skewedGaussian")
class(cFit2.1)  <- 'peakPantheR_curveFit'
cFit2.2         <- list(amplitude=151407.23415130575, center=3399.791, sigma=0.063753866057052563, gamma=0.001676782834598999, fitStatus=2, curveModel="skewedGaussian")
class(cFit2.2)  <- 'peakPantheR_curveFit'
# 3
cFit3.1         <- list(amplitude=122363.51256736703, center=3362.233, sigma=0.075489598945304492, gamma=0.0025160536725299734, fitStatus=2, curveModel="skewedGaussian")
class(cFit3.1)  <- 'peakPantheR_curveFit'
cFit3.2         <- list(amplitude=204749.86097918145, center=3409.182, sigma=0.075731781812843249, gamma=0.0013318670577834328, fitStatus=2, curveModel="skewedGaussian")
class(cFit3.2)  <- 'peakPantheR_curveFit'
input_peakFit   <- list(list(cFit1.1, cFit1.2), list(cFit2.1, cFit2.2), list(cFit3.1, cFit3.2))

# dataPoint
tmp_raw_data1  	  <- MSnbase::readMSData(input_spectraPaths[1], centroided=TRUE, mode='onDisk')
ROIDataPoints1    <- extractSignalRawData(tmp_raw_data1, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
tmp_raw_data2  	  <- MSnbase::readMSData(input_spectraPaths[2], centroided=TRUE, mode='onDisk')
ROIDataPoints2    <- extractSignalRawData(tmp_raw_data2, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
tmp_raw_data3  	  <- MSnbase::readMSData(input_spectraPaths[3], centroided=TRUE, mode='onDisk')
ROIDataPoints3    <- extractSignalRawData(tmp_raw_data3, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
input_dataPoints  <- list(ROIDataPoints1, ROIDataPoints2, ROIDataPoints3)

# Object, no samples, no compounds
defaultInit_empty       <- peakPantheRAnnotation()
# Object, init samples and compounds
defaultInit_cpd_spectra <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable)
# Object, fully filled
filledAnnotation        <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable, cpdMetadata=input_cpdMetadata, FIR=input_FIR, uROI=input_uROI, uROIExist=TRUE, useUROI=TRUE, useFIR=TRUE, spectraMetadata=input_spectraMetadata, acquisitionTime=input_acquisitionTime, TIC=input_TIC, peakTables=input_peakTables, dataPoints=input_dataPoints, peakFit=input_peakFit, isAnnotated=TRUE)


test_that('initialised objects are valid', {
  # no samples, no compounds
  expect_true(validObject(defaultInit_empty))
  # init samples and compounds
  expect_true(validObject(defaultInit_cpd_spectra))
})

test_that('a fully filled object is valid', {
  # fully filled
  expect_true(validObject(filledAnnotation))
})

test_that('validObject() raises errors', {
  # number of cpdName
  wrong1          <- filledAnnotation
  wrong1@cpdName  <- c("Cpd 1", "Cpd 2", "Cpd 3")
  msg1            <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: cpdName has 3 elements (compound). Should be 2', sep='')
  expect_error(validObject(wrong1), msg1, fixed=TRUE)

  # ROI number of rows
  wrong2          <- filledAnnotation
  wrong2@ROI      <- wrong2@ROI[1,]
  msg2            <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: ROI has 1 rows (compound). Should be 2', sep='')
  expect_error(validObject(wrong2), msg2, fixed=TRUE)
  # ROI number of columns
  wrong3          <- filledAnnotation
  wrong3@ROI      <- wrong3@ROI[,1:5]
  msg3            <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: ROI has 5 columns. Should be 6 ("rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax")', sep='')
  expect_error(validObject(wrong3), msg3, fixed=TRUE)
  # ROI column names
  wrong4                <- filledAnnotation
  colnames(wrong4@ROI)  <- c("wrongCol", "rt", "rtMax", "mzMin", "mz", "mzMax")
  msg4            <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: ROI columns should be "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", not wrongCol rt rtMax mzMin mz mzMax', sep='')
  expect_error(validObject(wrong4), msg4 , fixed=TRUE)
  # ROI$rtMin is NA
  wrong5a           <- filledAnnotation
  wrong5a@ROI$rtMin  <- c(3310 , NA)
  msg5a              <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: ROI$rtMin, ROI$rtMax, ROI$mzMin and ROI$mzMax cannot be NA', sep='')
  expect_error(validObject(wrong5a), msg5a, fixed=TRUE)
  # ROI$rtMin numeric
  wrong5            <- filledAnnotation
  wrong5@ROI$rtMin  <- c("not numeric", "not numeric")
  msg5              <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: ROI$rtMin should be numeric, not character', sep='')
  expect_error(validObject(wrong5), msg5, fixed=TRUE)
  # ROI$rt numeric
  wrong6            <- filledAnnotation
  wrong6@ROI$rt     <- c("not numeric", "not numeric")
  msg6              <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: ROI$rt should be numeric, not character', sep='')
  expect_error(validObject(wrong6), msg6, fixed=TRUE)
  # ROI$rtMax is NA
  wrong7a            <- filledAnnotation
  wrong7a@ROI$rtMax  <- c(NA , 3440)
  msg7a              <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: ROI$rtMin, ROI$rtMax, ROI$mzMin and ROI$mzMax cannot be NA', sep='')
  expect_error(validObject(wrong7a), msg7a, fixed=TRUE)
  # ROI$rtMax numeric
  wrong7            <- filledAnnotation
  wrong7@ROI$rtMax  <- c("not numeric", "not numeric")
  msg7              <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: ROI$rtMax should be numeric, not character', sep='')
  expect_error(validObject(wrong7), msg7, fixed=TRUE)
  # ROI$mzMin is NA
  wrong8a            <- filledAnnotation
  wrong8a@ROI$mzMin  <- c(522.1948 , NA)
  msg8a              <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: ROI$rtMin, ROI$rtMax, ROI$mzMin and ROI$mzMax cannot be NA', sep='')
  expect_error(validObject(wrong8a), msg8a, fixed=TRUE)
  # ROI$mzMin numeric
  wrong8            <- filledAnnotation
  wrong8@ROI$mzMin  <- c("not numeric", "not numeric")
  msg8              <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: ROI$mzMin should be numeric, not character', sep='')
  expect_error(validObject(wrong8), msg8, fixed=TRUE)
  # ROI$mz numeric
  wrong9            <- filledAnnotation
  wrong9@ROI$mz     <- c("not numeric", "not numeric")
  msg9              <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: ROI$mz should be numeric, not character', sep='')
  expect_error(validObject(wrong9), msg9, fixed=TRUE)
  # ROI$mzMax is NA
  wrong10a            <- filledAnnotation
  wrong10a@ROI$rtMin  <- c(NA, 496.2050)
  msg10a              <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: ROI$rtMin, ROI$rtMax, ROI$mzMin and ROI$mzMax cannot be NA', sep='')
  expect_error(validObject(wrong10a), msg10a, fixed=TRUE)
  # ROI$mzMax numeric
  wrong10            <- filledAnnotation
  wrong10@ROI$mzMax  <- c("not numeric", "not numeric")
  msg10              <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: ROI$mzMax should be numeric, not character', sep='')
  expect_error(validObject(wrong10), msg10, fixed=TRUE)

  # FIR number of rows
  wrong11          <- filledAnnotation
  wrong11@FIR      <- wrong11@FIR[1,]
  msg11            <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: FIR has 1 rows (compound). Should be 2', sep='')
  expect_error(validObject(wrong11), msg11, fixed=TRUE)
  # FIR number of columns
  wrong12          <- filledAnnotation
  wrong12@FIR      <- wrong12@FIR[,1:3]
  msg12            <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: FIR has 3 columns. Should be 4 ("rtMin", "rtMax", "mzMin", "mzMax")', sep='')
  expect_error(validObject(wrong12), msg12, fixed=TRUE)
  # FIR column names
  wrong13                 <- filledAnnotation
  colnames(wrong13@FIR)   <- c("wrongCol", "rtMax", "mzMin", "mzMax")
  msg13                   <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: FIR columns should be "rtMin", "rtMax", "mzMin", "mzMax", not wrongCol rtMax mzMin mzMax', sep='')
  expect_error(validObject(wrong13), msg13, fixed=TRUE)
  # FIR$rtMin is NA
  wrong14a            <- filledAnnotation
  wrong14a@FIR$rtMin  <- c(1 , NA)
  msg14a              <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: FIR$rtMin, FIR$rtMax, FIR$mzMin and FIR$mzMax cannot be NA', sep='')
  expect_error(validObject(wrong14a), msg14a, fixed=TRUE)
  # FIR$rtMin numeric
  wrong14             <- filledAnnotation
  wrong14@FIR$rtMin   <- c("not numeric", "not numeric")
  msg14               <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: FIR$rtMin should be numeric, not character', sep='')
  expect_error(validObject(wrong14), msg14, fixed=TRUE)
  # FIR$rtMax is NA
  wrong15a            <- filledAnnotation
  wrong15a@FIR$rtMax  <- c(NA , 6)
  msg15a              <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: FIR$rtMin, FIR$rtMax, FIR$mzMin and FIR$mzMax cannot be NA', sep='')
  expect_error(validObject(wrong15a), msg15a, fixed=TRUE)
  # FIR$rtMax numeric
  wrong15             <- filledAnnotation
  wrong15@FIR$rtMax   <- c("not numeric", "not numeric")
  msg15               <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: FIR$rtMax should be numeric, not character', sep='')
  expect_error(validObject(wrong15), msg15, fixed=TRUE)
  # FIR$mzMin is NA
  wrong16a            <- filledAnnotation
  wrong16a@FIR$mzMin  <- c(3 , NA)
  msg16a              <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: FIR$rtMin, FIR$rtMax, FIR$mzMin and FIR$mzMax cannot be NA', sep='')
  expect_error(validObject(wrong16a), msg16a, fixed=TRUE)
  # FIR$mzMin numeric
  wrong16             <- filledAnnotation
  wrong16@FIR$mzMin   <- c("not numeric", "not numeric")
  msg16               <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: FIR$mzMin should be numeric, not character', sep='')
  expect_error(validObject(wrong16), msg16, fixed=TRUE)
  # FIR$mzMax is NA
  wrong17a            <- filledAnnotation
  wrong17a@FIR$mzMax  <- c(NA , 8)
  msg17a              <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: FIR$rtMin, FIR$rtMax, FIR$mzMin and FIR$mzMax cannot be NA', sep='')
  expect_error(validObject(wrong17a), msg17a, fixed=TRUE)
  # FIR$mzMax numeric
  wrong17             <- filledAnnotation
  wrong17@FIR$mzMax   <- c("not numeric", "not numeric")
  msg17               <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: FIR$mzMax should be numeric, not character', sep='')
  expect_error(validObject(wrong17), msg17, fixed=TRUE)

  # uROI number of rows
  wrong18           <- filledAnnotation
  wrong18@uROI      <- wrong18@uROI[1,]
  msg18             <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: uROI has 1 rows (compound). Should be 2', sep='')
  expect_error(validObject(wrong18), msg18, fixed=TRUE)
  # uROI number of columns
  wrong19           <- filledAnnotation
  wrong19@uROI      <- wrong19@uROI[,1:5]
  msg19             <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: uROI has 5 columns. Should be 6 ("rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax")', sep='')
  expect_error(validObject(wrong19), msg19, fixed=TRUE)
  # uROI column names
  wrong20                 <- filledAnnotation
  colnames(wrong20@uROI)  <- c("wrongCol", "rt", "rtMax", "mzMin", "mz", "mzMax")
  msg20                   <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: uROI columns should be "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", not wrongCol rt rtMax mzMin mz mzMax', sep='')
  expect_error(validObject(wrong20), msg20, fixed=TRUE)
  # uROI$rtMin is NA
  wrong21a             <- filledAnnotation
  wrong21a@uROI$rtMin  <- c(9 , NA)
  msg21a               <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: uROI$rtMin, uROI$rtMax, uROI$mzMin and uROI$mzMax cannot be NA', sep='')
  expect_error(validObject(wrong21a), msg21a, fixed=TRUE)
  # uROI$rtMin numeric
  wrong21             <- filledAnnotation
  wrong21@uROI$rtMin  <- c("not numeric", "not numeric")
  msg21               <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: uROI$rtMin should be numeric, not character', sep='')
  expect_error(validObject(wrong21), msg21, fixed=TRUE)
  # uROI$rt numeric
  wrong22             <- filledAnnotation
  wrong22@uROI$rt     <- c("not numeric", "not numeric")
  msg22               <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: uROI$rt should be numeric, not character', sep='')
  expect_error(validObject(wrong22), msg22, fixed=TRUE)
  # uROI$rtMax is NA
  wrong23a             <- filledAnnotation
  wrong23a@uROI$rtMax  <- c(NA , 14)
  msg23a               <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: uROI$rtMin, uROI$rtMax, uROI$mzMin and uROI$mzMax cannot be NA', sep='')
  expect_error(validObject(wrong23a), msg23a, fixed=TRUE)
  # uROI$rtMax numeric
  wrong23             <- filledAnnotation
  wrong23@uROI$rtMax  <- c("not numeric", "not numeric")
  msg23               <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: uROI$rtMax should be numeric, not character', sep='')
  expect_error(validObject(wrong23), msg23, fixed=TRUE)
  # uROI$mzMin is NA
  wrong24a             <- filledAnnotation
  wrong24a@uROI$mzMin  <- c(11 , NA)
  msg24a               <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: uROI$rtMin, uROI$rtMax, uROI$mzMin and uROI$mzMax cannot be NA', sep='')
  expect_error(validObject(wrong24a), msg24a, fixed=TRUE)
  # uROI$mzMin numeric
  wrong24             <- filledAnnotation
  wrong24@uROI$mzMin  <- c("not numeric", "not numeric")
  msg24               <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: uROI$mzMin should be numeric, not character', sep='')
  expect_error(validObject(wrong24), msg24, fixed=TRUE)
  # uROI$mz numeric
  wrong25             <- filledAnnotation
  wrong25@uROI$mz     <- c("not numeric", "not numeric")
  msg25               <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: uROI$mz should be numeric, not character', sep='')
  expect_error(validObject(wrong25), msg25, fixed=TRUE)
  # uROI$mzMax is NA
  wrong26a             <- filledAnnotation
  wrong26a@uROI$mzMax  <- c(12 , NA)
  msg26a               <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: uROI$rtMin, uROI$rtMax, uROI$mzMin and uROI$mzMax cannot be NA', sep='')
  expect_error(validObject(wrong26a), msg26a, fixed=TRUE)
  # uROI$mzMax numeric
  wrong26             <- filledAnnotation
  wrong26@uROI$mzMax  <- c("not numeric", "not numeric")
  msg26               <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: uROI$mzMax should be numeric, not character', sep='')
  expect_error(validObject(wrong26), msg26, fixed=TRUE)

  # number of acquisitionTime
  wrong27                 <- filledAnnotation
  wrong27@acquisitionTime <- filledAnnotation@acquisitionTime[c(1,2)]
  msg27                   <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: acquisitionTime has 2 elements (samples). Should be 3', sep='')
  expect_error(validObject(wrong27), msg27, fixed=TRUE)
  
  # cannot useUROI=TRUE if uROIExist=FALSE
  wrong28           <- filledAnnotation
  wrong28@useUROI   <- TRUE
  wrong28@uROIExist <- FALSE
  msg28             <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: useUROI cannot be TRUE while uROIExist is FALSE', sep='')
  expect_error(validObject(wrong28), msg28, fixed=TRUE)
  
  # number of TIC
  wrong29       <- filledAnnotation
  wrong29@TIC   <- c(1, 2)
  msg29         <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: TIC has 2 elements (samples). Should be 3', sep='')
  expect_error(validObject(wrong29), msg29, fixed=TRUE)

  # number of peakTables
  wrong30             <- filledAnnotation
  wrong30@peakTables  <- wrong30@peakTables[1:2]
  msg30               <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: peakTables has 2 elements (samples). Should be 3', sep='')
  expect_error(validObject(wrong30), msg30, fixed=TRUE)
  # not all peakTables are NULL or initialised
  wrong31             <- filledAnnotation
  wrong31@peakTables  <- list(NULL, wrong31@peakTables[[2]], wrong31@peakTables[[3]])
  msg31               <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: peakTables must all either be data.frame or NULL', sep='')
  expect_error(validObject(wrong31), msg31, fixed=TRUE)
  # peakTables is data.frame
  wrong32             <- filledAnnotation
  wrong32@peakTables  <- list("not data.frame", "not data.frame", "not data.frame")
  msg32               <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: peakTables must be data.frame or NULL not character', sep='')
  expect_error(validObject(wrong32), msg32, fixed=TRUE)
  # peakTables data.frame number of rows
  wrong33                 <- filledAnnotation
  wrong33@peakTables[[1]] <- wrong33@peakTables[[1]][1,]
  msg33                   <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: peakTables[[1]] has 1 rows (compounds). Should be 2', sep='')
  expect_error(validObject(wrong33), msg33, fixed=TRUE)
  # peakTables data.frame number of columns
  wrong34                 <- filledAnnotation
  wrong34@peakTables[[1]] <- wrong34@peakTables[[1]][,1:2]
  msg34                   <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: peakTables[[1]] has 2 columns. Should be 16', sep='')
  expect_error(validObject(wrong34), msg34, fixed=TRUE)
  # peakTables column names
  wrong35                           <- filledAnnotation
  colnames(wrong35@peakTables[[1]]) <- c('wrongCol', 'rt', 'rtMin', 'rtMax', 'mz', 'mzMin', 'mzMax', 'peakArea', 'peakAreaRaw', 'maxIntMeasured', 'maxIntPredicted', 'is_filled', 'ppm_error', 'rt_dev_sec', 'tailingFactor', 'asymmetryFactor')
  msg35                             <- paste("invalid class ", dQuote('peakPantheRAnnotation')," object: peakTables[[1]] columns should be 'found', 'rt', 'rtMin', 'rtMax', 'mz', 'mzMin', 'mzMax', 'peakArea', 'peakAreaRaw', 'maxIntMeasured', 'maxIntPredicted', 'is_filled', 'ppm_error', 'rt_dev_sec', 'tailingFactor', 'asymmetryFactor', not wrongCol rt rtMin rtMax mz mzMin mzMax peakArea peakAreaRaw maxIntMeasured maxIntPredicted is_filled ppm_error rt_dev_sec tailingFactor asymmetryFactor", sep='')
  expect_error(validObject(wrong35), msg35, fixed=TRUE)

  # number of dataPoints
  wrong36             <- filledAnnotation
  wrong36@dataPoints  <- wrong36@dataPoints[1:2]
  msg36               <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: dataPoints has 2 elements (samples). Should be 3', sep='')
  expect_error(validObject(wrong36), msg36, fixed=TRUE)
  # not all dataPoints are NULL or initialised
  wrong37             <- filledAnnotation
  wrong37@dataPoints  <- list(NULL, wrong37@dataPoints[[2]], wrong37@dataPoints[[3]])
  msg37               <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: dataPoints must all either be list of ROI data points or NULL', sep='')
  expect_error(validObject(wrong37), msg37, fixed=TRUE)
  # individual dataPoints is list
  wrong38             <- filledAnnotation
  wrong38@dataPoints  <- list("not list", "not list", "not list")
  msg38               <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: dataPoints[[1]] must be a list of ROI data points, not character', sep='')
  expect_error(validObject(wrong38), msg38, fixed=TRUE)
  # individual dataPoints has entry for each compound
  wrong39                 <- filledAnnotation
  wrong39@dataPoints[[1]] <- list(wrong39@dataPoints[[1]][1])
  msg39                   <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: dataPoints[[1]] contains, 1 dataPoints (compound). Should be 2', sep='')
  expect_error(validObject(wrong39), msg39, fixed=TRUE)
  # individual dataPoints compound entry is data.frame
  wrong40                       <- filledAnnotation
  wrong40@dataPoints[[1]][[1]]  <- "not a data.frame"
  msg40                         <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: dataPoints[[1]][[1]] must be a data.frame, not character', sep='')
  expect_error(validObject(wrong40), msg40, fixed=TRUE)
  # individual dataPoints compound data.frame number of columns
  wrong41                       <- filledAnnotation
  wrong41@dataPoints[[1]][[1]]  <- wrong41@dataPoints[[1]][[1]][,1:2]
  msg41                         <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: dataPoints[[1]][[1]] has 2 columns. Should be 3', sep='')
  expect_error(validObject(wrong41), msg41, fixed=TRUE)
  # individual dataPoints compound data.frame column names
  wrong42                                 <- filledAnnotation
  colnames(wrong42@dataPoints[[1]][[1]])  <- c('wrongCol', 'mz', 'int')
  msg42                                   <- paste("invalid class ", dQuote('peakPantheRAnnotation')," object: dataPoints[[1]][[1]] columns should be 'rt', 'mz', 'int', not wrongCol mz int", sep="")
  expect_error(validObject(wrong42), msg42, fixed=TRUE)
  
  # number of peakFit
  wrong43         <- filledAnnotation
  wrong43@peakFit <- wrong43@peakFit[1:2]
  msg43           <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: peakFit has 2 elements (samples). Should be 3', sep='')
  expect_error(validObject(wrong43), msg43, fixed=TRUE)
  # not all peakFit are NULL or initialised
  wrong44         <- filledAnnotation
  wrong44@peakFit <- list(NULL, wrong44@peakFit[[2]], wrong44@peakFit[[3]])
  msg44           <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: peakFit must all either be list of ROI curveFit or NULL', sep='')
  expect_error(validObject(wrong44), msg44, fixed=TRUE)
  # individual peakFit is list
  wrong45         <- filledAnnotation
  wrong45@peakFit <- list("not list", "not list", "not list")
  msg45           <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: peakFit[[1]] must be a list of ROI curveFit or NA, not character', sep='')
  expect_error(validObject(wrong45), msg45, fixed=TRUE)
  # individual peakFit has entry for each compound
  wrong46               <- filledAnnotation
  wrong46@peakFit[[1]]  <- list(wrong46@peakFit[[1]][1])
  msg46                 <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: peakFit[[1]] contains, 1 peakPantheR_curveFit or NA (compound). Should be 2', sep='')
  expect_error(validObject(wrong46), msg46, fixed=TRUE)
  # individual dataPoints compound entry is peakPantheR_curveFit or NA
  wrong47                   <- filledAnnotation
  wrong47@peakFit[[1]][[1]] <- "not a peakPantheR_curveFit or NA"
  msg47                     <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: peakFit[[1]][[1]] must be NA or a peakPantheR_curveFit, not character', sep='')
  expect_error(validObject(wrong47), msg47, fixed=TRUE)
  
  # number of cpdMetadata compounds (rows)
  wrong28             <- filledAnnotation
  wrong28@cpdMetadata <- filledAnnotation@cpdMetadata[c(1,2,1),]
  msg28               <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: cpdMetadata has 3 rows (compounds). Should be 2', sep='')
  expect_error(validObject(wrong28), msg28, fixed=TRUE)
  
  # number of spectraMetadata spectra (rows)
  wrong29                 <- filledAnnotation
  wrong29@spectraMetadata <- filledAnnotation@spectraMetadata[c(1,2),]
  msg29                   <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: spectraMetadata has 2 rows (spectra). Should be 3', sep='')
  expect_error(validObject(wrong29), msg29, fixed=TRUE)
})

context('peakPantheRAnnotation_show-method()')

# Test the show method

test_that('show method outputs right value', {
  # Input and expected
  input_spectraPaths        <- c('./path/file1', './path/file2', './path/file3')
  input_targetFeatTable     <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
  input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390., 522.194778, 522.2, 522.205222)
  input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440., 496.195038, 496.2, 496.204962)
  input_targetFeatTable[,c(3:8)]  <- sapply(input_targetFeatTable[,c(3:8)], as.numeric)
  input_FIR       <- data.frame(matrix(vector(), 2, 4, dimnames=list(c(), c("rtMin", "rtMax", "mzMin", "mzMax"))), stringsAsFactors=FALSE)
  input_FIR[1,]   <- c(1., 2., 3., 4.)
  input_FIR[2,]   <- c(5., 6., 7., 8.)
  input_uROI      <- data.frame(matrix(vector(), 2, 6, dimnames=list(c(), c("rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
  input_uROI[1,]  <- c(10., 11., 12., 13., 14., 15.)
  input_uROI[2,]  <- c(16., 17., 18., 19., 20., 21.)
  expected_msg_default    <- "An object of class peakPantheRAnnotation\n 0 compounds in 0 samples. \n  updated ROI do not exist (uROI)\n  does not use updated ROI (uROI)\n  does not use fallback integration regions (FIR)\n  is not annotated"
  expected_msg_ROIFIRInit <- "An object of class peakPantheRAnnotation\n 2 compounds in 3 samples. \n  updated ROI exist (uROI)\n  uses updated ROI (uROI)\n  uses fallback integration regions (FIR)\n  is annotated"

  # default values, no uROI, no use of FIR
  defaultInit     <- peakPantheRAnnotation()
  result_default  <- evaluate_promise(show(defaultInit))
  expect_equal(result_default$output, expected_msg_default)

  # multiple compounds and spectra with uROIExist and useFIR
  ROIFIRInit        <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable, uROI=input_uROI, FIR=input_FIR, uROIExist=TRUE, useUROI=TRUE, useFIR=TRUE, isAnnotated=TRUE)
  result_ROIFIRInit <- evaluate_promise(show(ROIFIRInit))
  expect_equal(result_ROIFIRInit$output, expected_msg_ROIFIRInit)
})

context('peakPantheRAnnotation_outputAnnotationResult()')

## Test the CSV output of all annotation results

skip_if_not_installed('faahKO',  minimum_version = '1.18.0')
library(faahKO)

## Input data
# spectraPaths
input_spectraPaths  <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                         system.file('cdf/KO/ko16.CDF', package = "faahKO"),
                         system.file('cdf/KO/ko18.CDF', package = "faahKO"))

# targetFeatTable
input_targetFeatTable     <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390., 522.194778, 522.2, 522.205222)
input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440., 496.195038, 496.2, 496.204962)
input_targetFeatTable[,c(3:8)] <- sapply(input_targetFeatTable[,c(3:8)], as.numeric)

# FIR
input_FIR     <- data.frame(matrix(vector(), 2, 4, dimnames=list(c(), c("rtMin", "rtMax", "mzMin", "mzMax"))), stringsAsFactors=FALSE)
input_FIR[1,] <- c(1., 2., 3., 4.)
input_FIR[2,] <- c(5., 6., 7., 8.)

# uROI
input_uROI      <- data.frame(matrix(vector(), 2, 6, dimnames=list(c(), c("rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
input_uROI[1,]  <- c(9., 10., 11., 12., 13., 14.)
input_uROI[2,]  <- c(15., 16., 17., 18., 19., 20.)

# TICs
input_TIC       <- c(2410533091, 2524040155, 2332817115)

# cpdMetadata
input_cpdMetadata     <- data.frame(matrix(data=c('a','b',1,2), nrow=2, ncol=2, dimnames=list(c(),c('testcol1','testcol2')), byrow=FALSE), stringsAsFactors=FALSE)

# spectraMetadata
input_spectraMetadata <- data.frame(matrix(data=c('c','d','e',3,4,5), nrow=3, ncol=2, dimnames=list(c(),c('testcol1','testcol2')), byrow=FALSE), stringsAsFactors=FALSE)

# acquisitionTime
input_acquisitionTime <- c(as.character(Sys.time()), as.character(Sys.time()+900), as.character(Sys.time()+1800))

# peakTables
# 1
peakTable1     <- data.frame(matrix(vector(), 2, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
peakTable1[1,] <- c(TRUE, 3309.7589296586070, 3346.8277590361445, 3385.4098874628098, 522.194778, 522.20001220703125, 522.205222, 26133726.6811244078, 26133726, 889280, 901015.80529226747, FALSE, 0.023376160866574614, 1.93975903614455092, 1.0153573486330891, 1.0268238825675249)
peakTable1[2,] <- c(TRUE, 3345.3766648628907, 3386.5288072289159, 3428.2788374983961, 496.20001220703125, 496.20001220703125, 496.20001220703125, 35472141.3330242932, 35472141, 1128960, 1113576.69008227298, FALSE, 0.024601030353423384, 0.95180722891564074, 1.0053782620427065, 1.0093180792278085)
peakTable1[,c(1,12)]       <- sapply(peakTable1[,c(1,12)], as.logical)
peakTable1[,c(2:11,13:16)] <- sapply(peakTable1[,c(2:11,13:16)], as.numeric)
# 2
peakTable2     <- data.frame(matrix(vector(), 2, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
peakTable2[1,] <- c(TRUE, 3326.1063495851854, 3365.102, 3407.2726475892355, 522.194778, 522.20001220703125, 522.205222, 24545301.622835573, 24545301, 761664, 790802.2209998488, FALSE, 0.023376160866574614, 0.2139999999999, 1.0339153786516375, 1.0630802030537212)
peakTable2[2,] <- c(TRUE, 3365.0238566258713, 3405.791, 3453.4049569205681, 496.195038, 496.20001220703125, 496.204962, 37207579.286265120, 37207579, 1099264, 1098720.2929832144, FALSE, 0.024601030353423384, 20.2139999999999, 1.0839602450900523, 1.1717845972583161)
peakTable2[,c(1,12)]       <- sapply(peakTable2[,c(1,12)], as.logical)
peakTable2[,c(2:11,13:16)] <- sapply(peakTable2[,c(2:11,13:16)], as.numeric)
# 3
peakTable3     <- data.frame(matrix(vector(), 2, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
peakTable3[1,] <- c(TRUE, 3333.8625894557053, 3368.233, 3407.4362838927614, 522.194778, 522.20001220703125, 522.205222, 21447174.404490683, 21447174, 758336, 765009.9805796633, FALSE, 0.023376160866574614, 23.345000000000255, 1.0609102044546637, 1.1155310457756928)
peakTable3[2,] <- c(TRUE, 3373.3998828113113, 3413.4952530120481, 3454.4490330927388, 496.195038, 496.20001220703125, 496.204962, 35659353.614476241, 35659353, 1149440, 1145857.7611069249, TRUE, 0.024601030353423384, 27.918253012047899, 1.0081407426394933, 1.0143315197994494)
peakTable3[,c(1,12)]       <- sapply(peakTable3[,c(1,12)], as.logical)
peakTable3[,c(2:11, 13:16)] <- sapply(peakTable3[,c(2:11,13:16)], as.numeric)
input_peakTables <- list(peakTable1, peakTable2, peakTable3)

# peakFit
# 1
cFit1.1         <- list(amplitude=162404.8057918259, center=3341.888, sigma=0.078786133031045896, gamma=0.0018336101984172684, fitStatus=2, curveModel="skewedGaussian")
class(cFit1.1)  <- 'peakPantheR_curveFit'
cFit1.2         <- list(amplitude=199249.10572753669, center=3382.577, sigma=0.074904415304607966, gamma=0.0011471899372353885, fitStatus=2, curveModel="skewedGaussian")
class(cFit1.2)  <- 'peakPantheR_curveFit'
# 2
cFit2.1         <- list(amplitude=124090.83425474487, center=3359.102, sigma=0.071061541060964212, gamma=0.0018336072657203239, fitStatus=2, curveModel="skewedGaussian")
class(cFit2.1)  <- 'peakPantheR_curveFit'
cFit2.2         <- list(amplitude=151407.23415130575, center=3399.791, sigma=0.063753866057052563, gamma=0.001676782834598999, fitStatus=2, curveModel="skewedGaussian")
class(cFit2.2)  <- 'peakPantheR_curveFit'
# 3
cFit3.1         <- list(amplitude=122363.51256736703, center=3362.233, sigma=0.075489598945304492, gamma=0.0025160536725299734, fitStatus=2, curveModel="skewedGaussian")
class(cFit3.1)  <- 'peakPantheR_curveFit'
cFit3.2         <- list(amplitude=204749.86097918145, center=3409.182, sigma=0.075731781812843249, gamma=0.0013318670577834328, fitStatus=2, curveModel="skewedGaussian")
class(cFit3.2)  <- 'peakPantheR_curveFit'
input_peakFit   <- list(list(cFit1.1, cFit1.2), list(cFit2.1, cFit2.2), list(cFit3.1, cFit3.2))

# dataPoint
tmp_raw_data1  	  <- MSnbase::readMSData(input_spectraPaths[1], centroided=TRUE, mode='onDisk')
ROIDataPoints1    <- extractSignalRawData(tmp_raw_data1, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
tmp_raw_data2  	  <- MSnbase::readMSData(input_spectraPaths[2], centroided=TRUE, mode='onDisk')
ROIDataPoints2    <- extractSignalRawData(tmp_raw_data2, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
tmp_raw_data3  	  <- MSnbase::readMSData(input_spectraPaths[3], centroided=TRUE, mode='onDisk')
ROIDataPoints3    <- extractSignalRawData(tmp_raw_data3, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
input_dataPoints  <- list(ROIDataPoints1, ROIDataPoints2, ROIDataPoints3)

# Object, fully filled
filledAnnotation        <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable, FIR=input_FIR, uROI=input_uROI, useFIR=TRUE, uROIExist=TRUE, useUROI=TRUE, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata, acquisitionTime=input_acquisitionTime, TIC=input_TIC, peakTables=input_peakTables, dataPoints=input_dataPoints, peakFit=input_peakFit, isAnnotated=TRUE)



test_that('csv output with FIR, verbose, no verbose', {
  # temporary file
  savePath1         <- tempdir()
  
  # input
  input_annotation  <- filledAnnotation
  
  # expected
  expected_path_cpdMeta     <- file.path(savePath1, "testProject_cpdMetadata.csv")
  expected_path_specMeta    <- file.path(savePath1, "testProject_spectraMetadata.csv")
  expected_path_summary     <- file.path(savePath1, "testProject_summary.csv")
  expected_path_asym        <- file.path(savePath1, "testProject_asymmetryFactor.csv")
  expected_path_found       <- file.path(savePath1, "testProject_found.csv")
  expected_path_is_filled   <- file.path(savePath1, "testProject_is_filled.csv")
  expected_path_maxIntMeas  <- file.path(savePath1, "testProject_maxIntMeasured.csv")
  expected_path_maxIntPred  <- file.path(savePath1, "testProject_maxIntPredicted.csv")
  expected_path_mz          <- file.path(savePath1, "testProject_mz.csv")
  expected_path_mzMax       <- file.path(savePath1, "testProject_mzMax.csv")
  expected_path_mzMin       <- file.path(savePath1, "testProject_mzMin.csv")
  expected_path_peakArea    <- file.path(savePath1, "testProject_peakArea.csv")
  expected_path_peakAreaRaw <- file.path(savePath1, "testProject_peakAreaRaw.csv")
  expected_path_ppmErr      <- file.path(savePath1, "testProject_ppm_error.csv")
  expected_path_rt          <- file.path(savePath1, "testProject_rt.csv")
  expected_path_rtDev       <- file.path(savePath1, "testProject_rt_dev_sec.csv")
  expected_path_rtMax       <- file.path(savePath1, "testProject_rtMax.csv")
  expected_path_rtMin       <- file.path(savePath1, "testProject_rtMin.csv")
  expected_path_tailfact    <- file.path(savePath1, "testProject_tailingFactor.csv")
  
  # expected values
  expected_cpdMeta            <- data.frame(matrix(data=c('ID-1','Cpd 1',3360.0542530120501,522.20001220703102,'a',1,'ID-2','Cpd 2',3396.1599036144598,496.20001220703102,'b',2), nrow=2,ncol=6,dimnames=list(c(), c('cpdID', 'cpdName', 'Retention.Time','m.z','testcol1', 'testcol2')), byrow=TRUE), stringsAsFactors=FALSE)
  expected_cpdMeta[,c(3,4,6)] <- apply(expected_cpdMeta[,c(3,4,6)], 2, as.numeric)
  expected_specMeta     <- data.frame(matrix(nrow=3,ncol=4,dimnames=list(c(), c('acquisitionTime', 'TIC', 'testcol1', 'testcol2')), byrow=TRUE), stringsAsFactors=FALSE) # no filepath
  expected_specMeta$acquisitionTime <- input_acquisitionTime
  expected_specMeta$TIC             <- input_TIC
  expected_specMeta$testcol1        <- input_spectraMetadata$testcol1
  expected_specMeta$testcol2        <- as.numeric(input_spectraMetadata$testcol2)
  expected_summary        <- data.frame(matrix(data=c('ID-1 - Cpd 1',1,0,0.02337616,8.499586, 'ID-2 - Cpd 2',0.6666667,0.333333,0.02460103,16.361353), nrow=2,ncol=5,dimnames=list(c(), c('X', 'ratio_peaks_found', 'ratio_peaks_filled', 'ppm_error', 'rt_dev_sec')), byrow=TRUE), stringsAsFactors=FALSE)
  expected_summary[,2:5]  <- sapply(expected_summary[,2:5], as.numeric)
  expected_asym         <- annotationTable(input_annotation, column='asymmetryFactor')
  expected_asym         <- data.frame(X=rownames(expected_asym), ID.1=expected_asym$`ID-1`, ID.2=expected_asym$`ID-2`, stringsAsFactors=FALSE)
  expected_found        <- annotationTable(input_annotation, column='found')
  expected_found        <- data.frame(X=rownames(expected_found), ID.1=expected_found$`ID-1`, ID.2=expected_found$`ID-2`, stringsAsFactors=FALSE)
  expected_is_filled    <- annotationTable(input_annotation, column='is_filled')
  expected_is_filled    <- data.frame(X=rownames(expected_is_filled), ID.1=expected_is_filled$`ID-1`, ID.2=expected_is_filled$`ID-2`, stringsAsFactors=FALSE)
  expected_maxIntMeas   <- annotationTable(input_annotation, column='maxIntMeasured')
  expected_maxIntMeas   <- data.frame(X=rownames(expected_maxIntMeas), ID.1=expected_maxIntMeas$`ID-1`, ID.2=expected_maxIntMeas$`ID-2`, stringsAsFactors=FALSE)
  expected_maxIntPred   <- annotationTable(input_annotation, column='maxIntPredicted')
  expected_maxIntPred   <- data.frame(X=rownames(expected_maxIntPred), ID.1=expected_maxIntPred$`ID-1`, ID.2=expected_maxIntPred$`ID-2`, stringsAsFactors=FALSE)
  expected_mz           <- annotationTable(input_annotation, column='mz')
  expected_mz           <- data.frame(X=rownames(expected_mz), ID.1=expected_mz$`ID-1`, ID.2=expected_mz$`ID-2`, stringsAsFactors=FALSE)
  expected_mzMax        <- annotationTable(input_annotation, column='mzMax')
  expected_mzMax        <- data.frame(X=rownames(expected_mzMax), ID.1=expected_mzMax$`ID-1`, ID.2=expected_mzMax$`ID-2`, stringsAsFactors=FALSE)
  expected_mzMin        <- annotationTable(input_annotation, column='mzMin')
  expected_mzMin        <- data.frame(X=rownames(expected_mzMin), ID.1=expected_mzMin$`ID-1`, ID.2=expected_mzMin$`ID-2`, stringsAsFactors=FALSE)
  expected_peakArea     <- annotationTable(input_annotation, column='peakArea')
  expected_peakArea     <- data.frame(X=rownames(expected_peakArea), ID.1=expected_peakArea$`ID-1`, ID.2=expected_peakArea$`ID-2`, stringsAsFactors=FALSE)
  expected_peakAreaRaw  <- annotationTable(input_annotation, column='peakAreaRaw')
  expected_peakAreaRaw  <- data.frame(X=rownames(expected_peakAreaRaw), ID.1=expected_peakAreaRaw$`ID-1`, ID.2=expected_peakAreaRaw$`ID-2`, stringsAsFactors=FALSE)
  expected_ppmErr       <- annotationTable(input_annotation, column='ppm_error')
  expected_ppmErr       <- data.frame(X=rownames(expected_ppmErr), ID.1=expected_ppmErr$`ID-1`, ID.2=expected_ppmErr$`ID-2`, stringsAsFactors=FALSE)
  expected_rt           <- annotationTable(input_annotation, column='rt')
  expected_rt           <- data.frame(X=rownames(expected_rt), ID.1=expected_rt$`ID-1`, ID.2=expected_rt$`ID-2`, stringsAsFactors=FALSE)
  expected_rtDev        <- annotationTable(input_annotation, column='rt_dev_sec')
  expected_rtDev        <- data.frame(X=rownames(expected_rtDev), ID.1=expected_rtDev$`ID-1`, ID.2=expected_rtDev$`ID-2`, stringsAsFactors=FALSE)
  expected_rtMax        <- annotationTable(input_annotation, column='rtMax')
  expected_rtMax        <- data.frame(X=rownames(expected_rtMax), ID.1=expected_rtMax$`ID-1`, ID.2=expected_rtMax$`ID-2`, stringsAsFactors=FALSE)
  expected_rtMin        <- annotationTable(input_annotation, column='rtMin')
  expected_rtMin        <- data.frame(X=rownames(expected_rtMin), ID.1=expected_rtMin$`ID-1`, ID.2=expected_rtMin$`ID-2`, stringsAsFactors=FALSE)
  expected_tailfact     <- annotationTable(input_annotation, column='tailingFactor')
  expected_tailfact     <- data.frame(X=rownames(expected_tailfact), ID.1=expected_tailfact$`ID-1`, ID.2=expected_tailfact$`ID-2`, stringsAsFactors=FALSE)
  
  # results (output, warnings and messages)
  result_save     <- evaluate_promise(outputAnnotationResult(input_annotation, saveFolder=savePath1, annotationName='testProject', verbose=TRUE))
  
  # Check CSV has been produced
  expect_true(all(file.exists(expected_path_cpdMeta, expected_path_specMeta, expected_path_summary, expected_path_asym, expected_path_found, expected_path_is_filled,  expected_path_maxIntMeas,  expected_path_maxIntPred, expected_path_mz, expected_path_mzMax, expected_path_mzMin, expected_path_peakArea, expected_path_peakAreaRaw, expected_path_ppmErr, expected_path_rt, expected_path_rtDev, expected_path_rtMax, expected_path_rtMin, expected_path_tailfact)))
  
  # Check values saved
  saved_cpdMeta     <- read.csv(expected_path_cpdMeta, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_cpdMeta, expected_cpdMeta)
  saved_specMeta    <- read.csv(expected_path_specMeta, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE) # no filepath
  expect_equal(saved_specMeta[,-1], expected_specMeta)
  saved_summary     <- read.csv(expected_path_summary, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_summary, expected_summary, tolerance=1e-6)
  saved_asym        <- read.csv(expected_path_asym, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_asym, expected_asym)
  saved_found       <- read.csv(expected_path_found, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_found, expected_found)
  saved_is_filled   <- read.csv(expected_path_is_filled, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_is_filled, expected_is_filled)
  saved_maxIntMeas  <- read.csv(expected_path_maxIntMeas, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_maxIntMeas, expected_maxIntMeas)
  saved_maxIntPred  <- read.csv(expected_path_maxIntPred, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_maxIntPred, expected_maxIntPred)
  saved_mz          <- read.csv(expected_path_mz, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_mz, expected_mz)
  saved_mzMax       <- read.csv(expected_path_mzMax, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_mzMax, expected_mzMax)
  saved_mzMin       <- read.csv(expected_path_mzMin, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_mzMin, expected_mzMin)
  saved_peakArea    <- read.csv(expected_path_peakArea, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_peakArea, expected_peakArea)
  saved_peakAreaRaw    <- read.csv(expected_path_peakAreaRaw, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_peakAreaRaw, expected_peakAreaRaw)
  saved_ppmErr      <- read.csv(expected_path_ppmErr, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_ppmErr, expected_ppmErr)
  saved_rt          <- read.csv(expected_path_rt, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_rt, expected_rt)
  saved_rtDev       <- read.csv(expected_path_rtDev, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_rtDev, expected_rtDev)
  saved_rtMax       <- read.csv(expected_path_rtMax, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_rtMax, expected_rtMax)
  saved_rtMin       <- read.csv(expected_path_rtMin, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_rtMin, expected_rtMin)
  saved_tailfact    <- read.csv(expected_path_tailfact, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_tailfact, expected_tailfact)
  
  # Check result messages (save path)
  expect_equal(length(result_save$messages), 19)
  
  
  ## no verbose
  savePath2       <- tempdir()
  result_save2    <- evaluate_promise(outputAnnotationResult(input_annotation, saveFolder=savePath2, annotationName='testProject', verbose=FALSE))
  expect_equal(length(result_save2$messages), 0)
})

test_that('csv output without FIR, verbose, no verbose', {
  # temporary file
  savePath3         <- tempdir()
  # clear temp folder
  suppressWarnings(do.call(file.remove, list(list.files(savePath3, full.names = TRUE))))
  
  # input
  filledAnnotation2   <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable, FIR=input_FIR, uROI=input_uROI, useFIR=FALSE, uROIExist=TRUE, useUROI=TRUE, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata, acquisitionTime=input_acquisitionTime, TIC=input_TIC, peakTables=input_peakTables, dataPoints=input_dataPoints, peakFit=input_peakFit, isAnnotated=TRUE)
  input_annotation    <- filledAnnotation2
  
  # expected
  expected_path_cpdMeta     <- file.path(savePath3, "testProject_cpdMetadata.csv")
  expected_path_specMeta    <- file.path(savePath3, "testProject_spectraMetadata.csv")
  expected_path_summary     <- file.path(savePath3, "testProject_summary.csv")
  expected_path_asym        <- file.path(savePath3, "testProject_asymmetryFactor.csv")
  expected_path_found       <- file.path(savePath3, "testProject_found.csv")
  expected_path_is_filled   <- file.path(savePath3, "testProject_is_filled.csv")
  expected_path_maxIntMeas  <- file.path(savePath3, "testProject_maxIntMeasured.csv")
  expected_path_maxIntPred  <- file.path(savePath3, "testProject_maxIntPredicted.csv")
  expected_path_mz          <- file.path(savePath3, "testProject_mz.csv")
  expected_path_mzMax       <- file.path(savePath3, "testProject_mzMax.csv")
  expected_path_mzMin       <- file.path(savePath3, "testProject_mzMin.csv")
  expected_path_peakArea    <- file.path(savePath3, "testProject_peakArea.csv")
  expected_path_peakAreaRaw <- file.path(savePath3, "testProject_peakAreaRaw.csv")
  expected_path_ppmErr      <- file.path(savePath3, "testProject_ppm_error.csv")
  expected_path_rt          <- file.path(savePath3, "testProject_rt.csv")
  expected_path_rtDev       <- file.path(savePath3, "testProject_rt_dev_sec.csv")
  expected_path_rtMax       <- file.path(savePath3, "testProject_rtMax.csv")
  expected_path_rtMin       <- file.path(savePath3, "testProject_rtMin.csv")
  expected_path_tailfact    <- file.path(savePath3, "testProject_tailingFactor.csv")
  
  # expected values
  expected_cpdMeta            <- data.frame(matrix(data=c('ID-1','Cpd 1',3360.0542530120501,522.20001220703102,'a',1,'ID-2','Cpd 2',3396.1599036144598,496.20001220703102,'b',2), nrow=2,ncol=6,dimnames=list(c(), c('cpdID', 'cpdName', 'Retention.Time','m.z','testcol1', 'testcol2')), byrow=TRUE), stringsAsFactors=FALSE)
  expected_cpdMeta[,c(3,4,6)] <- apply(expected_cpdMeta[,c(3,4,6)], 2, as.numeric)
  expected_specMeta     <- data.frame(matrix(nrow=3,ncol=4,dimnames=list(c(), c('acquisitionTime', 'TIC', 'testcol1', 'testcol2')), byrow=TRUE), stringsAsFactors=FALSE) # no filepath
  expected_specMeta$acquisitionTime <- input_acquisitionTime
  expected_specMeta$TIC             <- input_TIC
  expected_specMeta$testcol1        <- input_spectraMetadata$testcol1
  expected_specMeta$testcol2        <- as.numeric(input_spectraMetadata$testcol2)
  expected_summary        <- data.frame(matrix(data=c('ID-1 - Cpd 1',1,0,0.02337616,8.499586, 'ID-2 - Cpd 2',1,0,0.02460103,16.361353), nrow=2,ncol=5,dimnames=list(c(), c('X', 'ratio_peaks_found', 'ratio_peaks_filled', 'ppm_error', 'rt_dev_sec')), byrow=TRUE), stringsAsFactors=FALSE)
  expected_summary[,2:5]  <- sapply(expected_summary[,2:5], as.numeric)
  expected_asym         <- annotationTable(input_annotation, column='asymmetryFactor')
  expected_asym         <- data.frame(X=rownames(expected_asym), ID.1=expected_asym$`ID-1`, ID.2=expected_asym$`ID-2`, stringsAsFactors=FALSE)
  expected_found        <- annotationTable(input_annotation, column='found')
  expected_found        <- data.frame(X=rownames(expected_found), ID.1=expected_found$`ID-1`, ID.2=expected_found$`ID-2`, stringsAsFactors=FALSE)
  expected_is_filled    <- annotationTable(input_annotation, column='is_filled')
  expected_is_filled    <- data.frame(X=rownames(expected_is_filled), ID.1=expected_is_filled$`ID-1`, ID.2=expected_is_filled$`ID-2`, stringsAsFactors=FALSE)
  expected_maxIntMeas   <- annotationTable(input_annotation, column='maxIntMeasured')
  expected_maxIntMeas   <- data.frame(X=rownames(expected_maxIntMeas), ID.1=expected_maxIntMeas$`ID-1`, ID.2=expected_maxIntMeas$`ID-2`, stringsAsFactors=FALSE)
  expected_maxIntPred   <- annotationTable(input_annotation, column='maxIntPredicted')
  expected_maxIntPred   <- data.frame(X=rownames(expected_maxIntPred), ID.1=expected_maxIntPred$`ID-1`, ID.2=expected_maxIntPred$`ID-2`, stringsAsFactors=FALSE)
  expected_mz           <- annotationTable(input_annotation, column='mz')
  expected_mz           <- data.frame(X=rownames(expected_mz), ID.1=expected_mz$`ID-1`, ID.2=expected_mz$`ID-2`, stringsAsFactors=FALSE)
  expected_mzMax        <- annotationTable(input_annotation, column='mzMax')
  expected_mzMax        <- data.frame(X=rownames(expected_mzMax), ID.1=expected_mzMax$`ID-1`, ID.2=expected_mzMax$`ID-2`, stringsAsFactors=FALSE)
  expected_mzMin        <- annotationTable(input_annotation, column='mzMin')
  expected_mzMin        <- data.frame(X=rownames(expected_mzMin), ID.1=expected_mzMin$`ID-1`, ID.2=expected_mzMin$`ID-2`, stringsAsFactors=FALSE)
  expected_peakArea     <- annotationTable(input_annotation, column='peakArea')
  expected_peakArea     <- data.frame(X=rownames(expected_peakArea), ID.1=expected_peakArea$`ID-1`, ID.2=expected_peakArea$`ID-2`, stringsAsFactors=FALSE)
  expected_peakAreaRaw  <- annotationTable(input_annotation, column='peakAreaRaw')
  expected_peakAreaRaw  <- data.frame(X=rownames(expected_peakAreaRaw), ID.1=expected_peakAreaRaw$`ID-1`, ID.2=expected_peakAreaRaw$`ID-2`, stringsAsFactors=FALSE)
  expected_ppmErr       <- annotationTable(input_annotation, column='ppm_error')
  expected_ppmErr       <- data.frame(X=rownames(expected_ppmErr), ID.1=expected_ppmErr$`ID-1`, ID.2=expected_ppmErr$`ID-2`, stringsAsFactors=FALSE)
  expected_rt           <- annotationTable(input_annotation, column='rt')
  expected_rt           <- data.frame(X=rownames(expected_rt), ID.1=expected_rt$`ID-1`, ID.2=expected_rt$`ID-2`, stringsAsFactors=FALSE)
  expected_rtDev        <- annotationTable(input_annotation, column='rt_dev_sec')
  expected_rtDev        <- data.frame(X=rownames(expected_rtDev), ID.1=expected_rtDev$`ID-1`, ID.2=expected_rtDev$`ID-2`, stringsAsFactors=FALSE)
  expected_rtMax        <- annotationTable(input_annotation, column='rtMax')
  expected_rtMax        <- data.frame(X=rownames(expected_rtMax), ID.1=expected_rtMax$`ID-1`, ID.2=expected_rtMax$`ID-2`, stringsAsFactors=FALSE)
  expected_rtMin        <- annotationTable(input_annotation, column='rtMin')
  expected_rtMin        <- data.frame(X=rownames(expected_rtMin), ID.1=expected_rtMin$`ID-1`, ID.2=expected_rtMin$`ID-2`, stringsAsFactors=FALSE)
  expected_tailfact     <- annotationTable(input_annotation, column='tailingFactor')
  expected_tailfact     <- data.frame(X=rownames(expected_tailfact), ID.1=expected_tailfact$`ID-1`, ID.2=expected_tailfact$`ID-2`, stringsAsFactors=FALSE)
  
  # results (output, warnings and messages)
  result_save     <- evaluate_promise(outputAnnotationResult(input_annotation, saveFolder=savePath3, annotationName='testProject', verbose=TRUE))
  
  # Check CSV has been produced
  expect_true(all(file.exists(expected_path_cpdMeta, expected_path_specMeta, expected_path_summary, expected_path_asym, expected_path_found, expected_path_is_filled,  expected_path_maxIntMeas,  expected_path_maxIntPred, expected_path_mz, expected_path_mzMax, expected_path_mzMin, expected_path_peakArea, expected_path_ppmErr, expected_path_rt, expected_path_rtDev, expected_path_rtMax, expected_path_rtMin, expected_path_tailfact)))
  
  # Check values saved
  saved_cpdMeta     <- read.csv(expected_path_cpdMeta, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_cpdMeta, expected_cpdMeta)
  saved_specMeta    <- read.csv(expected_path_specMeta, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE) # no filepath
  expect_equal(saved_specMeta[,-1], expected_specMeta)
  saved_summary     <- read.csv(expected_path_summary, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_summary, expected_summary, tolerance=1e-6)
  saved_asym        <- read.csv(expected_path_asym, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_asym, expected_asym)
  saved_found       <- read.csv(expected_path_found, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_found, expected_found)
  saved_is_filled   <- read.csv(expected_path_is_filled, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_is_filled, expected_is_filled)
  saved_maxIntMeas  <- read.csv(expected_path_maxIntMeas, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_maxIntMeas, expected_maxIntMeas)
  saved_maxIntPred  <- read.csv(expected_path_maxIntPred, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_maxIntPred, expected_maxIntPred)
  saved_mz          <- read.csv(expected_path_mz, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_mz, expected_mz)
  saved_mzMax       <- read.csv(expected_path_mzMax, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_mzMax, expected_mzMax)
  saved_mzMin       <- read.csv(expected_path_mzMin, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_mzMin, expected_mzMin)
  saved_peakArea    <- read.csv(expected_path_peakArea, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_peakArea, expected_peakArea)
  saved_peakAreaRaw    <- read.csv(expected_path_peakAreaRaw, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_peakAreaRaw, expected_peakAreaRaw)
  saved_ppmErr      <- read.csv(expected_path_ppmErr, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_ppmErr, expected_ppmErr)
  saved_rt          <- read.csv(expected_path_rt, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_rt, expected_rt)
  saved_rtDev       <- read.csv(expected_path_rtDev, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_rtDev, expected_rtDev)
  saved_rtMax       <- read.csv(expected_path_rtMax, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_rtMax, expected_rtMax)
  saved_rtMin       <- read.csv(expected_path_rtMin, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_rtMin, expected_rtMin)
  saved_tailfact    <- read.csv(expected_path_tailfact, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_tailfact, expected_tailfact)
  
  # Check result messages (save path)
  expect_equal(length(result_save$messages), 19)
  
  ## no verbose
  savePath4       <- tempdir()
  result_save4    <- evaluate_promise(outputAnnotationResult(input_annotation, saveFolder=savePath4, annotationName='testProject', verbose=FALSE))
  expect_equal(length(result_save4$messages), 0)
})

test_that('cpdMetadata mean rt mz calculation with and without filled, verbose, no verbose', {
  # Calculate mean rt mz of found peaks but not filled
  # only the peakTable is correctly generated in the test data
  
  # temporary file
  savePath5         <- tempdir()
  # clear temp folder
  suppressWarnings(do.call(file.remove, list(list.files(savePath5, full.names = TRUE))))
  
  # cpd 1 (no FIR used) sample 1-2 found (not filled), sample 3 not found, (not filled) 
  # cpd 2 (FIR used) sample 1 found (filled), sample 2-3 found, (not filled) 
  # peakTables
  # sample 1
  tmp_peakTable1     <- data.frame(matrix(vector(), 2, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
  tmp_peakTable1[1,] <- c(TRUE, 3309.7589296586070, 3346.8277590361445, 3385.4098874628098, 522.194778, 522.21, 522.205222, 26133726.6811244078, 26133726, 889280, 901015.80529226747, FALSE, 0.023376160866574614, 1.93975903614455092, 1.0153573486330891, 1.0268238825675249)
  tmp_peakTable1[2,] <- c(TRUE, 3345.3766648628907, 3386.5288072289159, 3428.2788374983961, 496.20001220703125, 496.20001220703125, 496.20001220703125, 35472141.3330242932, 35472141, 1128960, 1113576.69008227298, TRUE, 0.024601030353423384, 0.95180722891564074, 1.0053782620427065, 1.0093180792278085)
  tmp_peakTable1[,c(1,11)]       <- sapply(tmp_peakTable1[,c(1,11)], as.logical)
  tmp_peakTable1[,c(2:10,12:15)] <- sapply(tmp_peakTable1[,c(2:10,12:15)], as.numeric)
  # sample 2
  tmp_peakTable2     <- data.frame(matrix(vector(), 2, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
  tmp_peakTable2[1,] <- c(TRUE, 3326.1063495851854, 3365.102, 3407.2726475892355, 522.194778, 522.20001220703125, 522.205222, 24545301.622835573, 24545301, 761664, 790802.2209998488, FALSE, 0.023376160866574614, 0.2139999999999, 1.0339153786516375, 1.0630802030537212)
  tmp_peakTable2[2,] <- c(TRUE, 3365.0238566258713, 3405.791, 3453.4049569205681, 496.195038, 496.20001220703125, 496.204962, 37207579.286265120, 37207579, 1099264, 1098720.2929832144, FALSE, 0.024601030353423384, 20.2139999999999, 1.0839602450900523, 1.1717845972583161)
  tmp_peakTable2[,c(1,11)]       <- sapply(tmp_peakTable2[,c(1,11)], as.logical)
  tmp_peakTable2[,c(2:10,12:15)] <- sapply(tmp_peakTable2[,c(2:10,12:15)], as.numeric)
  # sample 3
  tmp_peakTable3     <- data.frame(matrix(vector(), 2, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
  tmp_peakTable3[1,] <- c(FALSE, 3333.8625894557053, 3368.233, 3407.4362838927614, 522.194778, 522.20001220703125, 522.205222, 21447174.404490683, 21447174, 758336, 765009.9805796633, FALSE, 0.023376160866574614, 23.345000000000255, 1.0609102044546637, 1.1155310457756928)
  tmp_peakTable3[2,] <- c(TRUE, 3373.3998828113113, 3413.4952530120481, 3454.4490330927388, 496.195038, 496.21, 496.204962, 35659353.614476241, 35659353, 1149440, 1145857.7611069249, FALSE, 0.024601030353423384, 27.918253012047899, 1.0081407426394933, 1.0143315197994494)
  tmp_peakTable3[,c(1,11)]       <- sapply(tmp_peakTable3[,c(1,11)], as.logical)
  tmp_peakTable3[,c(2:10,12:15)] <- sapply(tmp_peakTable3[,c(2:10,12:15)], as.numeric)
  tmp_peakTables <- list(tmp_peakTable1, tmp_peakTable2, tmp_peakTable3)
  
  # input
  filledAnnotation3   <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable, FIR=input_FIR, uROI=input_uROI, useFIR=FALSE, uROIExist=TRUE, useUROI=TRUE, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata, acquisitionTime=input_acquisitionTime, TIC=input_TIC, peakTables=tmp_peakTables, dataPoints=input_dataPoints, peakFit=input_peakFit, isAnnotated=TRUE)
  input_annotation    <- filledAnnotation3
  
  # expected
  expected_path_cpdMeta     <- file.path(savePath5, "testCpdMeta_cpdMetadata.csv")
  
  # expected values
  expected_cpdMeta            <- data.frame(matrix(data=c('ID-1','Cpd 1',3355.9648795180719,522.205,'a',1,'ID-2','Cpd 2',3409.6431265060241,496.205,'b',2), nrow=2,ncol=6,dimnames=list(c(), c('cpdID', 'cpdName', 'Retention.Time','m.z','testcol1', 'testcol2')), byrow=TRUE), stringsAsFactors=FALSE)
  expected_cpdMeta[,c(3,4,6)] <- apply(expected_cpdMeta[,c(3,4,6)], 2, as.numeric)
  
  # results (output, warnings and messages)
  result_save     <- evaluate_promise(outputAnnotationResult(input_annotation, saveFolder=savePath5, annotationName='testCpdMeta', verbose=TRUE))
  
  # Check CSV has been produced
  expect_true(file.exists(expected_path_cpdMeta))
  
  # Check values saved
  saved_cpdMeta     <- read.csv(expected_path_cpdMeta, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_cpdMeta, expected_cpdMeta)
})

test_that('raise error', {
  # error if object is not annotated
  savePath1         <- tempdir()
  input_annotation  <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable)
  msg1              <- 'Object has not been annotated, no annotation results to save'
  expect_error(outputAnnotationResult(input_annotation, saveFolder=savePath1), msg1, fixed=TRUE)
})

context('load_annotation_from_file_UI_helper()')


## Input data
# spectraPath
input_spectraPaths    <- c('./path/file1', './path/file2', './path/file3')

# targetFeatTable
input_targetFeatTable_adv     <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable_adv[1,] <- c("ID-1", "Cpd 1",  3.,  1.,  4.,  5.,  2.,  6.)
input_targetFeatTable_adv[2,] <- c("ID-2", "Cpd 2", 19., 17., 20., 21., 18., 22.)
input_targetFeatTable_adv[,c(3:8)] <- sapply(input_targetFeatTable_adv[,c(3:8)], as.numeric)

# uROI
input_uROI_adv      <- data.frame(matrix(vector(), 2, 6, dimnames=list(c(), c("rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
input_uROI_adv[1,]  <- c( 7., 11. , 8.,  9., 12., 10.)
input_uROI_adv[2,]  <- c(23., 27., 24., 25., 28., 26.)

# FIR
input_FIR_adv       <- data.frame(matrix(vector(), 2, 4, dimnames=list(c(), c("rtMin", "rtMax", "mzMin", "mzMax"))), stringsAsFactors=FALSE)
input_FIR_adv[1,]   <- c(13., 14., 15., 16.)
input_FIR_adv[2,]   <- c(29., 30., 31., 32.)

# correct annotation
annotationObject <- peakPantheRAnnotation(targetFeatTable = input_targetFeatTable_adv,
                                          spectraPaths = input_spectraPaths,
                                          uROI = input_uROI_adv,
                                          FIR = input_FIR_adv,
                                          uROIExist = TRUE)

# temporary files
annotationPath <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.RData')
save(annotationObject, file=annotationPath, compress=TRUE)


test_that('load correct file', {
    # expected
    expected    <- annotationObject

    # results (output, warnings and messages)
    result_load <- evaluate_promise(load_annotation_from_file_UI_helper(annotationPath = annotationPath))
    # Check result
    expect_equal(result_load$result, expected)

    # Check result messages (in output)
    expect_equal(length(result_load$messages), 0)
    expect_equal(result_load$output, "")
})

test_that('raise errors', {

    noFile          <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.RData')

    wrongNameObject <- annotationObject
    wrongName_path  <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.RData')
    save(wrongNameObject, file=wrongName_path, compress=TRUE)

    annotationObject    <- 'notAPeakPantheRAnnotation'
    wrongObject_path    <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.RData')
    save(annotationObject, file=wrongObject_path, compress=TRUE)

    # file doesn't exist
    msg1    <- paste('Error: annotation file does not exist', sep='')
    expect_error(load_annotation_from_file_UI_helper(annotationPath = noFile), msg1, fixed=TRUE)

    # wrong name in .RData
    msg2    <- paste("Error: annotation file must contain a `peakPantheRAnnotation` named 'annotationObject'", sep='')
    expect_error(load_annotation_from_file_UI_helper(annotationPath = wrongName_path), msg2, fixed=TRUE)

    # not a peakPantheRAnnotation
    msg3    <- paste("Error: the variable loaded is not a `peakPantheRAnnotation`", sep='')
    expect_error(load_annotation_from_file_UI_helper(annotationPath = wrongObject_path), msg3, fixed=TRUE)
})

context('peakPantheRAnnotation_resetFIR()')

## Test the resetting of FIR

skip_if_not_installed('faahKO',  minimum_version = '1.18.0')
library(faahKO)

## Input data
# targetFeatTable
input_targetFeatTable     <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c("ID-1", "Cpd 1",  3.,  1.,  4.,  5.,  2.,  6.)
input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 19., 17., 20., 21., 18., 22.)
input_targetFeatTable[,c(3:8)] <- sapply(input_targetFeatTable[,c(3:8)], as.numeric)

# uROI
input_uROI      <- data.frame(matrix(vector(), 2, 6, dimnames=list(c(), c("rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
input_uROI[1,]  <- c( 7., 11. , 8.,  9., 12., 10.)
input_uROI[2,]  <- c(23., 27., 24., 25., 28., 26.)

# FIR
input_FIR       <- data.frame(matrix(vector(), 2, 4, dimnames=list(c(), c("rtMin", "rtMax", "mzMin", "mzMax"))), stringsAsFactors=FALSE)
input_FIR[1,]   <- c(13., 14., 15., 16.)
input_FIR[2,]   <- c(29., 30., 31., 32.)

# object
input_annotation  <- peakPantheRAnnotation(targetFeatTable = input_targetFeatTable,
                                           uROI = input_uROI, 
                                           FIR = input_FIR,
                                           uROIExist = TRUE)



test_that('reset FIR with uROI values, verbose, no verbose', {
  
  # expected
  expected_annotation   <- input_annotation
  expected_annotation@FIR[,c('rtMin','rtMax','mzMin','mzMax')] <- uROI(input_annotation)[,c('rtMin','rtMax','mzMin','mzMax')]
  expected_message      <- c("FIR will be reset with uROI values\n")
  
  # results (output, warnings and messages)
  result_reset1         <- evaluate_promise(resetFIR(input_annotation, verbose=TRUE))
  
  # Check result
  expect_equal(result_reset1$result, expected_annotation)
  
  # Check result messages
  expect_equal(length(result_reset1$messages), 1)
  expect_equal(result_reset1$messages, expected_message)
  
  ## no verbose
  result_reset2         <- evaluate_promise(resetFIR(input_annotation, verbose=FALSE))
  expect_equal(length(result_reset2$messages), 0)
})

test_that('reset FIR with ROI values, verbose, no verbose', {
  
  # input
  tmp_annotation            <- input_annotation
  tmp_annotation@uROIExist  <- FALSE
  
  # expected
  expected_annotation   <- tmp_annotation
  expected_annotation@FIR[,c('rtMin','rtMax','mzMin','mzMax')] <- ROI(input_annotation)[,c('rtMin','rtMax','mzMin','mzMax')]
  expected_message      <- c("FIR will be reset with ROI values as uROI values are not set\n")
  
  # results (output, warnings and messages)
  result_reset3         <- evaluate_promise(resetFIR(tmp_annotation, verbose=TRUE))
  
  # Check result
  expect_equal(result_reset3$result, expected_annotation)
  
  # Check result messages
  expect_equal(length(result_reset3$messages), 1)
  expect_equal(result_reset3$messages, expected_message)
  
  ## no verbose
  result_reset4         <- evaluate_promise(resetFIR(tmp_annotation, verbose=FALSE))
  expect_equal(length(result_reset4$messages), 0)
})

context('spectra_metadata_colourScheme_UI_helper()')

# UI specific function that returns a vector of spectra colours

skip_if_not_installed('faahKO',  minimum_version = '1.18.0')
library(faahKO)

# remove Rplots.pdf created by ggplot2
on.exit( tryCatch({ file.remove('./Rplots.pdf') }, error=function(e){ invisible() }, warning=function(w){ invisible() }) )


## Input data
# spectraPaths
input_spectraPaths  <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                         system.file('cdf/KO/ko16.CDF', package = "faahKO"),
                         system.file('cdf/KO/ko18.CDF', package = "faahKO"))

# targetFeatTable
input_targetFeatTable     <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390., 522.194778, 522.2, 522.205222)
input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440., 496.195038, 496.2, 496.204962)
input_targetFeatTable[,c(3:8)] <- sapply(input_targetFeatTable[,c(3:8)], as.numeric)

# cpdMetadata
input_cpdMetadata     <- data.frame(matrix(data=c('a','b',1,2), nrow=2, ncol=2, dimnames=list(c(),c('testcol1','testcol2')), byrow=FALSE), stringsAsFactors=FALSE)

# spectraMetadata
input_spectraMetadata <- data.frame(matrix(data=c('c','d','e',3,4,5), nrow=3, ncol=2, dimnames=list(c(),c('testcol1','testcol2')), byrow=FALSE), stringsAsFactors=FALSE)

# Object, fully filled
annotation            <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata)


test_that('splColrColum is NULL', {
  # input
  input_annotation  <- annotation
  # expected
  expected_colr     <- NULL

  # results (output, warnings and messages)
  result_colr  <- evaluate_promise(
      spectra_metadata_colourScheme_UI_helper(annot = input_annotation,
                                              splColrColumn = NULL))

  # Check colour vector generated
  expect_equal(result_colr$result, expected_colr)
})

test_that('splColrColum is None', {
  # input
  input_annotation  <- annotation
  # expected
  expected_colr     <- NULL

  # results (output, warnings and messages)
  result_colr  <- evaluate_promise(
      spectra_metadata_colourScheme_UI_helper(annot = input_annotation,
                                              splColrColumn = 'None'))

  # Check colour vector generated
  expect_equal(result_colr$result, expected_colr)
})

test_that('splColrColum is not a spectraMetadata column name', {
  # input
  input_annotation  <- annotation
  # expected
  expected_colr     <- NULL

  # results (output, warnings and messages)
  result_colr  <- evaluate_promise(
      spectra_metadata_colourScheme_UI_helper(annot = input_annotation,
                                              splColrColumn = 'notAColumn'))

  # Check colour vector generated
  expect_equal(result_colr$result, expected_colr)
})

test_that('splColrColum is a spectraMetadata column name', {
  # input
  input_annotation  <- annotation
  # expected
  expected_colr     <- c("blue", "red", "green")

  # results (output, warnings and messages)
  result_colr  <- evaluate_promise(
      spectra_metadata_colourScheme_UI_helper(annot = input_annotation,
                                              splColrColumn = 'testcol1'))

  # Check colour vector generated
  expect_equal(result_colr$result, expected_colr)
})

test_that('12 unique colour (recycle colour palette)', {
  # input
  large_spectraPaths    <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                            system.file('cdf/KO/ko16.CDF', package = "faahKO"),
                            system.file('cdf/KO/ko18.CDF', package = "faahKO"),
                            system.file('cdf/KO/ko19.CDF', package = "faahKO"),
                            system.file('cdf/KO/ko21.CDF', package = "faahKO"),
                            system.file('cdf/KO/ko22.CDF', package = "faahKO"),
                            system.file('cdf/WT/wt15.CDF', package = "faahKO"),
                            system.file('cdf/WT/wt16.CDF', package = "faahKO"),
                            system.file('cdf/WT/wt18.CDF', package = "faahKO"),
                            system.file('cdf/WT/wt19.CDF', package = "faahKO"),
                            system.file('cdf/WT/wt21.CDF', package = "faahKO"),
                            system.file('cdf/WT/wt22.CDF', package = "faahKO"))
  large_spectraMetadata <- data.frame(matrix(data=c(1,2,3,4,5,6,7,8,9,10,11,12), nrow=12, ncol=1, dimnames=list(c(),c('testcol1')), byrow=FALSE), stringsAsFactors=FALSE)

  largeAnnotation       <- peakPantheRAnnotation(spectraPaths=large_spectraPaths, targetFeatTable=input_targetFeatTable, cpdMetadata=input_cpdMetadata, spectraMetadata=large_spectraMetadata)

  # expected
  expected_colr         <- c("blue", "red", "green", "orange", "purple", "seagreen",
                             "darkturquoise", "violetred", "saddlebrown", "black",
                             "blue", "red")

  # results (output, warnings and messages)
  result_colr  <- evaluate_promise(
      spectra_metadata_colourScheme_UI_helper(annot = largeAnnotation,
                                              splColrColumn = 'testcol1'))

  # Check colour vector generated
  expect_equal(result_colr$result, expected_colr)
})

context('peakPantheRAnnotation_annotationParamsDiagnostic-method()')

## Test the annotation parameters diagnostic

skip_if_not_installed('faahKO',  minimum_version = '1.18.0')
library(faahKO)

## Input data
# spectraPaths
input_spectraPaths  <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                         system.file('cdf/KO/ko16.CDF', package = "faahKO"),
                         system.file('cdf/KO/ko18.CDF', package = "faahKO"))

# targetFeatTable
input_targetFeatTable     <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390., 522.194778, 522.2, 522.205222)
input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440., 496.195038, 496.2, 496.204962)
input_targetFeatTable[,c(3:8)] <- vapply(input_targetFeatTable[,c(3:8)], as.numeric, FUN.VALUE=numeric(2))

# FIR
input_FIR     <- data.frame(matrix(vector(), 2, 4, dimnames=list(c(), c("rtMin", "rtMax", "mzMin", "mzMax"))), stringsAsFactors=FALSE)
input_FIR[1,] <- c(1., 2., 3., 4.)
input_FIR[2,] <- c(5., 6., 7., 8.)

# uROI
input_uROI      <- data.frame(matrix(vector(), 2, 6, dimnames=list(c(), c("rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
input_uROI[1,]  <- c(9., 10., 11., 12., 13., 14.)
input_uROI[2,]  <- c(15., 16., 17., 18., 19., 20.)

# TICs
input_TIC <- c(2410533091, 2524040155, 2332817115)

# cpdMetadata
input_cpdMetadata     <- data.frame(matrix(data=c('a','b',1,2), nrow=2, ncol=2, dimnames=list(c(),c('testcol1','testcol2')), byrow=FALSE), stringsAsFactors=FALSE)

# spectraMetadata
input_spectraMetadata <- data.frame(matrix(data=c('c','d','e',3,4,5), nrow=3, ncol=2, dimnames=list(c(),c('testcol1','testcol2')), byrow=FALSE), stringsAsFactors=FALSE)

# acquisitionTime
input_acquisitionTime <- c(as.character(Sys.time()), as.character(Sys.time()+900), as.character(Sys.time()+1800))

# peakTables
# 1
peakTable1     <- data.frame(matrix(vector(), 2, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
peakTable1[1,] <- c(TRUE, 3309.7589296586070, 3346.8277590361445, 3385.4098874628098, 522.194778, 522.20001220703125, 522.205222, 26133726.6811244078, 26133726, 889280, 901015.80529226747, FALSE, 0.023376160866574614, 1.93975903614455092, 1.0153573486330891, 1.0268238825675249)
peakTable1[2,] <- c(TRUE, 3345.3766648628907, 3386.5288072289159, 3428.2788374983961, 496.20001220703125, 496.20001220703125, 496.20001220703125, 35472141.3330242932, 35472141, 1128960, 1113576.69008227298, FALSE, 0.024601030353423384, 0.95180722891564074, 1.0053782620427065, 1.0093180792278085)
peakTable1[,c(1,12)]       <- sapply(peakTable1[,c(1,12)], as.logical)
peakTable1[,c(2:11,13:16)] <- sapply(peakTable1[,c(2:11,13:16)], as.numeric)
# 2
peakTable2     <- data.frame(matrix(vector(), 2, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
peakTable2[1,] <- c(TRUE, 3326.1063495851854, 3365.102, 3407.2726475892355, 522.194778, 522.20001220703125, 522.205222, 24545301.622835573, 24545301, 761664, 790802.2209998488, FALSE, 0.023376160866574614, 0.2139999999999, 1.0339153786516375, 1.0630802030537212)
peakTable2[2,] <- c(TRUE, 3365.0238566258713, 3405.791, 3453.4049569205681, 496.195038, 496.20001220703125, 496.204962, 37207579.286265120, 37207579, 1099264, 1098720.2929832144, FALSE, 0.024601030353423384, 20.2139999999999, 1.0839602450900523, 1.1717845972583161)
peakTable2[,c(1,12)]       <- sapply(peakTable2[,c(1,12)], as.logical)
peakTable2[,c(2:11,13:16)] <- sapply(peakTable2[,c(2:11,13:16)], as.numeric)
# 3
peakTable3     <- data.frame(matrix(vector(), 2, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
peakTable3[1,] <- c(TRUE, 3333.8625894557053, 3368.233, 3407.4362838927614, 522.194778, 522.20001220703125, 522.205222, 21447174.404490683, 21447174, 758336, 765009.9805796633, FALSE, 0.023376160866574614, 23.345000000000255, 1.0609102044546637, 1.1155310457756928)
peakTable3[2,] <- c(TRUE, 3373.3998828113113, 3413.4952530120481, 3454.4490330927388, 496.195038, 496.20001220703125, 496.204962, 35659353.614476241, 35659353, 1149440, 1145857.7611069249, TRUE, 0.024601030353423384, 27.918253012047899, 1.0081407426394933, 1.0143315197994494)
peakTable3[,c(1,12)]       <- sapply(peakTable3[,c(1,12)], as.logical)
peakTable3[,c(2:11, 13:16)] <- sapply(peakTable3[,c(2:11,13:16)], as.numeric)
input_peakTables <- list(peakTable1, peakTable2, peakTable3)
# peakFit
# 1
cFit1.1         <- list(amplitude=162404.8057918259, center=3341.888, sigma=0.078786133031045896, gamma=0.0018336101984172684, fitStatus=2, curveModel="skewedGaussian")
class(cFit1.1)  <- 'peakPantheR_curveFit'
cFit1.2         <- list(amplitude=199249.10572753669, center=3382.577, sigma=0.074904415304607966, gamma=0.0011471899372353885, fitStatus=2, curveModel="skewedGaussian")
class(cFit1.2)  <- 'peakPantheR_curveFit'
# 2
cFit2.1         <- list(amplitude=124090.83425474487, center=3359.102, sigma=0.071061541060964212, gamma=0.0018336072657203239, fitStatus=2, curveModel="skewedGaussian")
class(cFit2.1)  <- 'peakPantheR_curveFit'
cFit2.2         <- list(amplitude=151407.23415130575, center=3399.791, sigma=0.063753866057052563, gamma=0.001676782834598999, fitStatus=2, curveModel="skewedGaussian")
class(cFit2.2)  <- 'peakPantheR_curveFit'
# 3
cFit3.1         <- list(amplitude=122363.51256736703, center=3362.233, sigma=0.075489598945304492, gamma=0.0025160536725299734, fitStatus=2, curveModel="skewedGaussian")
class(cFit3.1)  <- 'peakPantheR_curveFit'
cFit3.2         <- list(amplitude=204749.86097918145, center=3409.182, sigma=0.075731781812843249, gamma=0.0013318670577834328, fitStatus=2, curveModel="skewedGaussian")
class(cFit3.2)  <- 'peakPantheR_curveFit'
input_peakFit   <- list(list(cFit1.1, cFit1.2), list(cFit2.1, cFit2.2), list(cFit3.1, cFit3.2))

# dataPoint
tmp_raw_data1  	  <- MSnbase::readMSData(input_spectraPaths[1], centroided=TRUE, mode='onDisk')
ROIDataPoints1    <- extractSignalRawData(tmp_raw_data1, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
tmp_raw_data2  	  <- MSnbase::readMSData(input_spectraPaths[2], centroided=TRUE, mode='onDisk')
ROIDataPoints2    <- extractSignalRawData(tmp_raw_data2, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
tmp_raw_data3  	  <- MSnbase::readMSData(input_spectraPaths[3], centroided=TRUE, mode='onDisk')
ROIDataPoints3    <- extractSignalRawData(tmp_raw_data3, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
input_dataPoints  <- list(ROIDataPoints1, ROIDataPoints2, ROIDataPoints3)

# Object, fully filled
filledAnnotation        <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable, FIR=input_FIR, uROI=input_uROI, useFIR=TRUE, uROIExist=TRUE, useUROI=TRUE, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata, acquisitionTime=input_acquisitionTime, TIC=input_TIC, peakTables=input_peakTables, dataPoints=input_dataPoints, peakFit=input_peakFit, isAnnotated=TRUE)



test_that('Object not annotated is returned untouched, verbose', {
  # input
  notAnnotated      <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable)
  
  # expected
  expected_message  <- c("Warning: the object has not been annotated, return the object untouched\n")
  
  # results
  result_diagnostic <- evaluate_promise(annotationParamsDiagnostic(notAnnotated, verbose=TRUE))
  
  # check results
  expect_equal(result_diagnostic$result, notAnnotated)
  
  # check messages
  expect_equal(length(result_diagnostic$messages), 1)
  expect_equal(result_diagnostic$messages, expected_message)
})

test_that('uROIExist (no change), useFIR (no change), verbose', {
  # input
  input_annotation  <- filledAnnotation
  
  # expected
  expected_message  <- c("uROI already exist, will not be changed\n", "FIR in use, will not be changed\n")
  
  # results
  result_diagnostic <- evaluate_promise(annotationParamsDiagnostic(input_annotation, verbose=TRUE))
  
  # check results
  expect_equal(result_diagnostic$result, input_annotation)
  
  # check messages
  expect_equal(length(result_diagnostic$messages), 2)
  expect_equal(result_diagnostic$messages, expected_message)
})

test_that('no uROIExist (min/max found peaks), useFIR (no change), verbose', {
  # uROIExist set to FALSE, so uROI will be set with min/max found peak (rt +/-5% ROI width), uROI rt mz set to ROI rt mz, uROIExist set to TRUE
  
  # input
  input_annotation            <- filledAnnotation
  input_annotation@uROIExist  <- FALSE
  
  # expected
  expected_annotation         <- filledAnnotation
  expected_annotation@uROI[ ,c('rt','mz')]                        <- filledAnnotation@ROI[,c('rt','mz')]
  expected_annotation@uROI[1,c('rtMin','rtMax','mzMin','mzMax')]  <- c(3305.758929658607, 3411.4362838927614, 522.194778, 522.205222)
  expected_annotation@uROI[2,c('rtMin','rtMax','mzMin','mzMax')]  <- c(3337.376664862891, 3462.4490330927388, 496.195038, 496.204962)
  expected_message            <- c("uROI will be set as mimimum/maximum of found peaks (+/-5% of ROI in retention time)\n", "FIR in use, will not be changed\n")
  
  # results
  result_diagnostic <- evaluate_promise(annotationParamsDiagnostic(input_annotation, verbose=TRUE))
  
  # check results
  expect_equal(result_diagnostic$result, expected_annotation)
  
  # check messages
  expect_equal(length(result_diagnostic$messages), 2)
  expect_equal(result_diagnostic$messages, expected_message)
})

test_that('uROIExist (no change), no useFIR (median found peaks), verbose', {
  # useFIR set to FALSE, so FIR will be calculated as median
  
  # input
  input_annotation          <- filledAnnotation
  input_annotation@useFIR   <- FALSE
  
  # expected
  expected_annotation         <- filledAnnotation
  expected_annotation@FIR[1,c('rtMin','rtMax','mzMin','mzMax')] <- c(3326.1063495851854, 3407.2726475892355, 522.194778, 522.205222)
  expected_annotation@FIR[2,c('rtMin','rtMax','mzMin','mzMax')] <- c(3365.0238566258713, 3453.4049569205681, 496.195038, 496.204962)
  expected_annotation@useFIR  <- FALSE
  expected_message            <- c("uROI already exist, will not be changed\n", "FIR will be calculated as the median of found \"rtMin\",\"rtMax\",\"mzMin\",\"mzMax\"\n")
  
  # results
  result_diagnostic <- evaluate_promise(annotationParamsDiagnostic(input_annotation, verbose=TRUE))
  
  # check results
  expect_equal(result_diagnostic$result, expected_annotation)
  
  # check messages
  expect_equal(length(result_diagnostic$messages), 2)
  expect_equal(result_diagnostic$messages, expected_message)
})

test_that('no peaks found for cpd 2, uROIExist (no change), no useFIR (median and replacement from uROI), verbose and no verbose', {
  # useFIR set to FALSE, so FIR will be calculated as median, no peaks for cpd 2 so use uROI that are untouched (uROIExist)
  # add NAs to cpd 1 to check na.rm=TRUE
  
  # input
  input_annotation          <- filledAnnotation
  input_annotation@useFIR   <- FALSE
  # no data found for cpd 2
  input_annotation@peakTables[[1]][2,c('rtMin', 'rtMax', 'mzMin', 'mzMax')] <- c(as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  input_annotation@peakTables[[2]][2,c('rtMin', 'rtMax', 'mzMin', 'mzMax')] <- c(as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  input_annotation@peakTables[[3]][2,c('rtMin', 'rtMax', 'mzMin', 'mzMax')] <- c(as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  # NA for cpd 1, change FIR
  input_annotation@peakTables[[2]][1,c('rtMin', 'rtMax', 'mzMin', 'mzMax')] <- c(as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  
  # expected
  expected_annotation             <- filledAnnotation
  expected_annotation@peakTables  <- input_annotation@peakTables
  expected_annotation@FIR[1,c('rtMin','rtMax','mzMin','mzMax')] <- c(3321.8107595571564, 3396.4230856777858, 522.194778, 522.205222)
  expected_annotation@FIR[2,c('rtMin','rtMax','mzMin','mzMax')] <- c(15., 17., 18., 20.)
  expected_annotation@useFIR      <- FALSE
  expected_message                <- c("uROI already exist, will not be changed\n", "FIR will be calculated as the median of found \"rtMin\",\"rtMax\",\"mzMin\",\"mzMax\"\n", "FIR median rtMin which are NA are replaced with uROI rtMin\n", "FIR median rtMax which are NA are replaced with uROI rtMax\n", "FIR median mzMin which are NA are replaced uROI mzMin\n", "FIR median mzMax which are NA are replaced uROI mzMax\n")
  
  # results
  result_diagnostic <- evaluate_promise(annotationParamsDiagnostic(input_annotation, verbose=TRUE))
  
  # check results
  expect_equal(result_diagnostic$result, expected_annotation)
  
  # check messages
  expect_equal(length(result_diagnostic$messages), 6)
  expect_equal(result_diagnostic$messages, expected_message)
  
  
  ## No verbose
  result_diagnostic2 <- evaluate_promise(annotationParamsDiagnostic(input_annotation, verbose=FALSE))
  expect_equal(length(result_diagnostic2$messages), 0)
})

test_that('no peaks found for cpd 2, no uROIExist (min/max and replacement from ROI), useFIR (no change) verbose and no verbose', {
  # uROIExist set to FALSE, so uROI will be calculated as min/max of peaks found, no peaks for cpd 2 so use ROI
  # add NAs to cpd 1 to check na.rm=TRUE
  
  # input
  input_annotation              <- filledAnnotation
  input_annotation@uROIExist    <- FALSE
  # no data found for cpd 2
  input_annotation@peakTables[[1]][2,c('rtMin', 'rtMax', 'mzMin', 'mzMax')] <- c(as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  input_annotation@peakTables[[2]][2,c('rtMin', 'rtMax', 'mzMin', 'mzMax')] <- c(as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  input_annotation@peakTables[[3]][2,c('rtMin', 'rtMax', 'mzMin', 'mzMax')] <- c(as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  # NA for cpd 1, change FIR
  input_annotation@peakTables[[1]][1,c('rtMin', 'mzMin', 'mzMax')]  <- c(as.numeric(NA), as.numeric(NA), as.numeric(NA))
  input_annotation@peakTables[[3]][1,c('rtMax')]                    <- c(as.numeric(NA))
  
  # expected
  expected_annotation             <- filledAnnotation
  expected_annotation@peakTables  <- input_annotation@peakTables
  expected_annotation@uROI[1,c('rtMin','rt','rtMax','mzMin','mz','mzMax')] <- c(3322.10635, 3344.888, 3411.27265, 522.194778, 522.2, 522.205222)
  expected_annotation@uROI[2,c('rtMin','rt','rtMax','mzMin','mz','mzMax')] <- c(3280., 3385.577, 3440., 496.195038, 496.2, 496.204962)
  expected_annotation@uROIExist   <- TRUE
  expected_message                <- c("uROI will be set as mimimum/maximum of found peaks (+/-5% of ROI in retention time)\n", "uROI min rtMin which are NA are replaced with ROI rtMin\n", "uROI max rtMax which are NA are replaced with ROI rtMax\n", "uROI min mzMin which are NA are replaced ROI mzMin\n", "uROI max mzMax which are NA are replaced ROI mzMax\n", "FIR in use, will not be changed\n")
  
  # results
  result_diagnostic <- evaluate_promise(annotationParamsDiagnostic(input_annotation, verbose=TRUE))
  
  # check results
  expect_equal(result_diagnostic$result, expected_annotation)
  
  # check messages
  expect_equal(length(result_diagnostic$messages), 6)
  expect_equal(result_diagnostic$messages, expected_message)
  
  
  ## No verbose
  result_diagnostic2 <- evaluate_promise(annotationParamsDiagnostic(input_annotation, verbose=FALSE))
  expect_equal(length(result_diagnostic2$messages), 0)
})

context('peakPantheRAnnotation_class_initializeMethod()')

# Test the class definition (enforces default slot type)
# Test initialize method (set default values, initialisation from spectraPaths and targetFeatTable)

test_that('initialize with default values', {
  # Expected values
  expected_slotName <- c("cpdID", "cpdName", "ROI", "FIR", "uROI", "filepath", "cpdMetadata", "spectraMetadata", "acquisitionTime", "uROIExist", "useUROI", "useFIR", "TIC", "peakTables", "dataPoints", "peakFit", "isAnnotated")
  expected_ROI              <- data.frame(rtMin=numeric(), rt=numeric(), rtMax=numeric(), mzMin=numeric(), mz=numeric(), mzMax=numeric(), stringsAsFactors=FALSE)
  expected_FIR              <- data.frame(rtMin=numeric(), rtMax=numeric(), mzMin=numeric(), mzMax=numeric(), stringsAsFactors=FALSE)
  expected_uROI             <- data.frame(rtMin=numeric(), rt=numeric(), rtMax=numeric(), mzMin=numeric(), mz=numeric(), mzMax=numeric(), stringsAsFactors=FALSE)
  expected_cpdMetadata      <- data.frame(matrix(, nrow=0, ncol=0))
  expected_spectraMetadata  <- data.frame(matrix(, nrow=0, ncol=0))

  # Object initialised with default values
  defaultInit <- peakPantheRAnnotation()

  # Check object
  expect_true(is(defaultInit, "peakPantheRAnnotation"))
  expect_equal(slotNames(defaultInit), expected_slotName)
  expect_true(is.character(defaultInit@cpdID))
  expect_equal(length(defaultInit@cpdID), 0)
  expect_true(is.character(defaultInit@cpdName))
  expect_equal(length(defaultInit@cpdName), 0)
  expect_equal(defaultInit@ROI, expected_ROI)
  expect_equal(defaultInit@FIR, expected_FIR)
  expect_equal(defaultInit@uROI, expected_uROI)
  expect_true(is.character(defaultInit@filepath))
  expect_equal(defaultInit@cpdMetadata, expected_cpdMetadata)
  expect_equal(defaultInit@spectraMetadata, expected_spectraMetadata)
  expect_equal(length(defaultInit@filepath), 0)
  expect_true(is.character(defaultInit@acquisitionTime))
  expect_equal(length(defaultInit@acquisitionTime), 0)
  expect_false(defaultInit@uROIExist)
  expect_false(defaultInit@useUROI)
  expect_false(defaultInit@useFIR)
  expect_true(is.numeric(defaultInit@TIC))
  expect_equal(length(defaultInit@TIC), 0)
  expect_true(is.list(defaultInit@peakTables))
  expect_equal(length(defaultInit@peakTables), 0)
  expect_true(is.list(defaultInit@dataPoints))
  expect_equal(length(defaultInit@dataPoints), 0)
  expect_true(is.list(defaultInit@peakFit))
  expect_equal(length(defaultInit@peakFit), 0)
  expect_false(defaultInit@isAnnotated)
})

test_that('slot types are set in class definition', {
  # slot cpdID is not character
  msg1 <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: invalid object for slot "cpdID" in class "peakPantheRAnnotation": got class "numeric", should be or extend class "character"', sep='')
  expect_error(peakPantheRAnnotation(cpdID=5), msg1, fixed=TRUE)
  # slot cpdName is not character
  msg2 <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: invalid object for slot "cpdName" in class "peakPantheRAnnotation": got class "numeric", should be or extend class "character"', sep='')
  expect_error(peakPantheRAnnotation(cpdName=5), msg2, fixed=TRUE)
  # slot ROI is not a data.frame
  msg3 <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: invalid object for slot "ROI" in class "peakPantheRAnnotation": got class "character", should be or extend class "data.frame"', sep='')
  expect_error(peakPantheRAnnotation(ROI='notADataFrame'), msg3, fixed=TRUE)
  # slot FIR is not a data.frame
  msg4 <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: invalid object for slot "FIR" in class "peakPantheRAnnotation": got class "character", should be or extend class "data.frame"', sep='')
  expect_error(peakPantheRAnnotation(FIR='notADataFrame'), msg4, fixed=TRUE)
  # slot uROI is not a data.frame
  msg5 <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: invalid object for slot "uROI" in class "peakPantheRAnnotation": got class "character", should be or extend class "data.frame"', sep='')
  expect_error(peakPantheRAnnotation(uROI='notADataFrame'), msg5, fixed=TRUE)
  # slot filepath is not character
  msg6 <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: invalid object for slot "filepath" in class "peakPantheRAnnotation": got class "numeric", should be or extend class "character"', sep='')
  expect_error(peakPantheRAnnotation(filepath=5), msg6, fixed=TRUE)
  # slot acquisitionTime is not character
  msg7 <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: invalid object for slot "acquisitionTime" in class "peakPantheRAnnotation": got class "numeric", should be or extend class "character"', sep='')
  expect_error(peakPantheRAnnotation(acquisitionTime=5), msg7, fixed=TRUE)
  # slot uROIExist is not a logical
  msg8 <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: invalid object for slot "uROIExist" in class "peakPantheRAnnotation": got class "character", should be or extend class "logical"', sep='')
  expect_error(peakPantheRAnnotation(uROIExist='notALogical'), msg8, fixed=TRUE)
  # slot useUROI is not a logical
  msg9 <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: invalid object for slot "useUROI" in class "peakPantheRAnnotation": got class "character", should be or extend class "logical"', sep='')
  expect_error(peakPantheRAnnotation(useUROI='notALogical'), msg9, fixed=TRUE)
  # slot useFIR is not a logical
  msg10 <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: invalid object for slot "useFIR" in class "peakPantheRAnnotation": got class "character", should be or extend class "logical"', sep='')
  expect_error(peakPantheRAnnotation(useFIR='notALogical'), msg10, fixed=TRUE)
  # slot TIC is not numeric
  msg11 <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: invalid object for slot "TIC" in class "peakPantheRAnnotation": got class "character", should be or extend class "numeric"', sep='')
  expect_error(peakPantheRAnnotation(TIC='notNumeric'), msg11, fixed=TRUE)
  # slot peakTables is not a list
  msg12 <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: invalid object for slot "peakTables" in class "peakPantheRAnnotation": got class "character", should be or extend class "list"', sep='')
  expect_error(peakPantheRAnnotation(peakTables='notAList'), msg12, fixed=TRUE)
  # slot dataPoints is not a list
  msg13 <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: invalid object for slot "dataPoints" in class "peakPantheRAnnotation": got class "character", should be or extend class "list"', sep='')
  expect_error(peakPantheRAnnotation(dataPoints='notAList'), msg13, fixed=TRUE)
  # slot peakFit is not a list
  msg14 <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: invalid object for slot "peakFit" in class "peakPantheRAnnotation": got class "character", should be or extend class "list"', sep='')
  expect_error(peakPantheRAnnotation(peakFit='notAList'), msg14, fixed=TRUE)
  # slot isAnnotated is not a logical
  msg15 <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: invalid object for slot "isAnnotated" in class "peakPantheRAnnotation": got class "character", should be or extend class "logical"', sep='')
  expect_error(peakPantheRAnnotation(isAnnotated='notALogical'), msg15, fixed=TRUE)
  # slot cpdMetadata is not a data.frame
  msg16 <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: invalid object for slot "cpdMetadata" in class "peakPantheRAnnotation": got class "character", should be or extend class "data.frame"', sep='')
  expect_error(peakPantheRAnnotation(cpdMetadata='notADataframe'), msg16, fixed=TRUE)
  # slot spectraMetadata is not a data.frame
  msg17 <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: invalid object for slot "spectraMetadata" in class "peakPantheRAnnotation": got class "character", should be or extend class "data.frame"', sep='')
  expect_error(peakPantheRAnnotation(spectraMetadata='notADataframe'), msg17, fixed=TRUE)
})

test_that('initialize with spectraPaths', {
  # Input and expected values
  tmp_peakTables            <-  data.frame(matrix(vector(), 0, 16, dimnames=list(c(), c('found', 'rt', 'rtMin', 'rtMax', 'mz', 'mzMin', 'mzMax', 'peakArea', 'peakAreaRaw', 'maxIntMeasured', 'maxIntPredicted', 'is_filled', 'ppm_error', 'rt_dev_sec', 'tailingFactor', 'asymmetryFactor'))), stringsAsFactors=FALSE)
  input_spectraPaths        <- c('./path/file1', './path/file2', './path/file3')
  input_spectraMetadata     <- data.frame(matrix(data=c('a','b','c'), nrow=3, ncol=1, dimnames=list(c(),c('testcol')), byrow=TRUE), stringsAsFactors=FALSE)
  input_acquisitionTime     <- c(as.character(Sys.time()), as.character(Sys.time()+900), as.character(Sys.time()+1800))
  input_TIC                 <- c(1, 2, 3)
  input_peakTables          <- list(tmp_peakTables, tmp_peakTables, tmp_peakTables)
  # dataPoints and peakFit for 3 samples and 0 cpds
  input_dataPoints          <- list(list(), list(), list())
  input_peakFit             <- list(list(), list(), list())
  expected_slotName         <- c("cpdID", "cpdName", "ROI", "FIR", "uROI", "filepath", "cpdMetadata", "spectraMetadata", "acquisitionTime", "uROIExist", "useUROI", "useFIR", "TIC", "peakTables", "dataPoints", "peakFit", "isAnnotated")
  expected_filepath         <- c('./path/file1', './path/file2', './path/file3')
  expected_acquisitionTime  <- as.character(c(NA, NA, NA))
  expected_ROI              <- data.frame(rtMin=numeric(), rt=numeric(), rtMax=numeric(), mzMin=numeric(), mz=numeric(), mzMax=numeric(), stringsAsFactors=FALSE)
  expected_FIR              <- data.frame(rtMin=numeric(), rtMax=numeric(), mzMin=numeric(), mzMax=numeric(), stringsAsFactors=FALSE)
  expected_uROI             <- data.frame(rtMin=numeric(), rt=numeric(), rtMax=numeric(), mzMin=numeric(), mz=numeric(), mzMax=numeric(), stringsAsFactors=FALSE)
  expected_TIC              <- as.numeric(c(NA, NA, NA))
  expected_peakTables       <- vector('list', 3)
  expected_dataPoints       <- vector('list', 3)
  expected_peakFit          <- vector('list', 3)
  expected_cpdMetadata      <- data.frame(matrix(, nrow=0, ncol=0))
  expected_spectraMetadata  <- data.frame(matrix(, nrow=3, ncol=0))
  
  # Init object with spectraPaths
  spectraAnnotation <- peakPantheRAnnotation(spectraPaths=input_spectraPaths)
  # Check object values
  expect_true(is(spectraAnnotation, "peakPantheRAnnotation"))
  expect_equal(slotNames(spectraAnnotation), expected_slotName)
  expect_true(is.character(spectraAnnotation@cpdID))
  expect_equal(length(spectraAnnotation@cpdID), 0)
  expect_true(is.character(spectraAnnotation@cpdName))
  expect_equal(length(spectraAnnotation@cpdName), 0)
  expect_equal(spectraAnnotation@ROI, expected_ROI)
  expect_equal(spectraAnnotation@FIR, expected_FIR)
  expect_equal(spectraAnnotation@uROI, expected_uROI)
  expect_equal(spectraAnnotation@filepath, expected_filepath)
  expect_equal(spectraAnnotation@cpdMetadata, expected_cpdMetadata)
  expect_equal(spectraAnnotation@spectraMetadata, expected_spectraMetadata)
  expect_equal(spectraAnnotation@acquisitionTime, expected_acquisitionTime)
  expect_false(spectraAnnotation@uROIExist)
  expect_false(spectraAnnotation@useUROI)
  expect_false(spectraAnnotation@useFIR)
  expect_equal(spectraAnnotation@TIC, expected_TIC)
  expect_equal(spectraAnnotation@peakTables, expected_peakTables)
  expect_equal(spectraAnnotation@dataPoints, expected_dataPoints)
  expect_equal(spectraAnnotation@peakFit, expected_peakFit)
  expect_false(spectraAnnotation@isAnnotated)

  # Provide a spectraMetadata that shouldn't get replaced
  specMetaAnnotation <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, spectraMetadata = input_spectraMetadata)
  # Check acquisitionTime get set from input and not default with spectraPaths
  expect_true(is(specMetaAnnotation, "peakPantheRAnnotation"))
  expect_equal(slotNames(specMetaAnnotation), expected_slotName)
  expect_true(is.character(specMetaAnnotation@cpdID))
  expect_equal(length(specMetaAnnotation@cpdID), 0)
  expect_true(is.character(specMetaAnnotation@cpdName))
  expect_equal(length(specMetaAnnotation@cpdName), 0)
  expect_equal(specMetaAnnotation@ROI, expected_ROI)
  expect_equal(specMetaAnnotation@FIR, expected_FIR)
  expect_equal(specMetaAnnotation@uROI, expected_uROI)
  expect_equal(specMetaAnnotation@filepath, expected_filepath)
  expect_equal(specMetaAnnotation@cpdMetadata, expected_cpdMetadata)
  expect_equal(specMetaAnnotation@spectraMetadata, input_spectraMetadata) # change is here
  expect_equal(specMetaAnnotation@acquisitionTime, expected_acquisitionTime)
  expect_false(specMetaAnnotation@uROIExist)
  expect_false(specMetaAnnotation@useUROI)
  expect_false(specMetaAnnotation@useFIR)
  expect_equal(specMetaAnnotation@TIC, expected_TIC)
  expect_equal(specMetaAnnotation@peakTables, expected_peakTables)
  expect_equal(specMetaAnnotation@dataPoints, expected_dataPoints)
  expect_equal(specMetaAnnotation@peakFit, expected_peakFit)
  expect_false(specMetaAnnotation@isAnnotated)
  
  # Provide a acquisitionTime that shouldn't get replaced
  acquAnnotation <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, acquisitionTime=input_acquisitionTime)
  # Check acquisitionTime get set from input and not default with spectraPaths
  expect_true(is(acquAnnotation, "peakPantheRAnnotation"))
  expect_equal(slotNames(acquAnnotation), expected_slotName)
  expect_true(is.character(acquAnnotation@cpdID))
  expect_equal(length(acquAnnotation@cpdID), 0)
  expect_true(is.character(acquAnnotation@cpdName))
  expect_equal(length(acquAnnotation@cpdName), 0)
  expect_equal(acquAnnotation@ROI, expected_ROI)
  expect_equal(acquAnnotation@FIR, expected_FIR)
  expect_equal(acquAnnotation@uROI, expected_uROI)
  expect_equal(acquAnnotation@filepath, expected_filepath)
  expect_equal(acquAnnotation@cpdMetadata, expected_cpdMetadata)
  expect_equal(acquAnnotation@spectraMetadata, expected_spectraMetadata)
  expect_equal(acquAnnotation@acquisitionTime, input_acquisitionTime)  # change is here
  expect_false(acquAnnotation@uROIExist)
  expect_false(acquAnnotation@useUROI)
  expect_false(acquAnnotation@useFIR)
  expect_equal(acquAnnotation@TIC, expected_TIC)
  expect_equal(acquAnnotation@peakTables, expected_peakTables)
  expect_equal(acquAnnotation@dataPoints, expected_dataPoints)
  expect_equal(acquAnnotation@peakFit, expected_peakFit)
  expect_false(acquAnnotation@isAnnotated)

  # Provide a TIC that shouldn't get replaced
  TICAnnotation <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, TIC=input_TIC)
  # Check TIC get set from input and not default with spectraPaths
  expect_true(is(TICAnnotation, "peakPantheRAnnotation"))
  expect_equal(slotNames(TICAnnotation), expected_slotName)
  expect_true(is.character(TICAnnotation@cpdID))
  expect_equal(length(TICAnnotation@cpdID), 0)
  expect_true(is.character(TICAnnotation@cpdName))
  expect_equal(length(TICAnnotation@cpdName), 0)
  expect_equal(TICAnnotation@ROI, expected_ROI)
  expect_equal(TICAnnotation@FIR, expected_FIR)
  expect_equal(TICAnnotation@uROI, expected_uROI)
  expect_equal(TICAnnotation@filepath, expected_filepath)
  expect_equal(TICAnnotation@cpdMetadata, expected_cpdMetadata)
  expect_equal(TICAnnotation@spectraMetadata, expected_spectraMetadata)
  expect_equal(TICAnnotation@acquisitionTime, expected_acquisitionTime)
  expect_false(TICAnnotation@uROIExist)
  expect_false(TICAnnotation@useUROI)
  expect_false(TICAnnotation@useFIR)
  expect_equal(TICAnnotation@TIC, input_TIC)                  # change is here
  expect_equal(TICAnnotation@peakTables, expected_peakTables)
  expect_equal(TICAnnotation@dataPoints, expected_dataPoints)
  expect_equal(TICAnnotation@peakFit, expected_peakFit)
  expect_false(TICAnnotation@isAnnotated)

  # Provide a peakTables that shouldn't get replaced
  peakTablesAnnotation <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, peakTables=input_peakTables)
  # Check peakTables get set from input and not default with spectraPaths
  expect_true(is(peakTablesAnnotation, "peakPantheRAnnotation"))
  expect_equal(slotNames(peakTablesAnnotation), expected_slotName)
  expect_true(is.character(peakTablesAnnotation@cpdID))
  expect_equal(length(peakTablesAnnotation@cpdID), 0)
  expect_true(is.character(peakTablesAnnotation@cpdName))
  expect_equal(length(peakTablesAnnotation@cpdName), 0)
  expect_equal(peakTablesAnnotation@ROI, expected_ROI)
  expect_equal(peakTablesAnnotation@FIR, expected_FIR)
  expect_equal(peakTablesAnnotation@uROI, expected_uROI)
  expect_equal(peakTablesAnnotation@filepath, expected_filepath)
  expect_equal(peakTablesAnnotation@cpdMetadata, expected_cpdMetadata)
  expect_equal(peakTablesAnnotation@spectraMetadata, expected_spectraMetadata)
  expect_equal(peakTablesAnnotation@acquisitionTime, expected_acquisitionTime)
  expect_false(peakTablesAnnotation@uROIExist)
  expect_false(peakTablesAnnotation@useUROI)
  expect_false(peakTablesAnnotation@useFIR)
  expect_equal(peakTablesAnnotation@TIC, expected_TIC)
  expect_equal(peakTablesAnnotation@peakTables, input_peakTables) # change is here
  expect_equal(peakTablesAnnotation@dataPoints, expected_dataPoints)
  expect_equal(peakTablesAnnotation@peakFit, expected_peakFit)
  expect_false(peakTablesAnnotation@isAnnotated)

  # Provide a dataPoints that shouldn't get replaced
  dataPointsAnnotation <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, dataPoints=input_dataPoints)
  # Check dataPoints get set from input and not default with spectraPaths
  expect_true(is(dataPointsAnnotation, "peakPantheRAnnotation"))
  expect_equal(slotNames(dataPointsAnnotation), expected_slotName)
  expect_true(is.character(dataPointsAnnotation@cpdID))
  expect_equal(length(dataPointsAnnotation@cpdID), 0)
  expect_true(is.character(dataPointsAnnotation@cpdName))
  expect_equal(length(dataPointsAnnotation@cpdName), 0)
  expect_equal(dataPointsAnnotation@ROI, expected_ROI)
  expect_equal(dataPointsAnnotation@FIR, expected_FIR)
  expect_equal(dataPointsAnnotation@uROI, expected_uROI)
  expect_equal(dataPointsAnnotation@filepath, expected_filepath)
  expect_equal(dataPointsAnnotation@cpdMetadata, expected_cpdMetadata)
  expect_equal(dataPointsAnnotation@spectraMetadata, expected_spectraMetadata)
  expect_equal(dataPointsAnnotation@acquisitionTime, expected_acquisitionTime)
  expect_false(dataPointsAnnotation@uROIExist)
  expect_false(dataPointsAnnotation@useUROI)
  expect_false(dataPointsAnnotation@useFIR)
  expect_equal(dataPointsAnnotation@TIC, expected_TIC)
  expect_equal(dataPointsAnnotation@peakTables, expected_peakTables)
  expect_equal(dataPointsAnnotation@dataPoints, input_dataPoints)   # change is here
  expect_equal(dataPointsAnnotation@peakFit, expected_peakFit)
  expect_false(dataPointsAnnotation@isAnnotated)

  # Provide a peakFit that shouldn't get replaced
  peakFitAnnotation <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, peakFit=input_peakFit)
  # Check peakFit get set from input and not default with spectraPaths
  expect_true(is(peakFitAnnotation, "peakPantheRAnnotation"))
  expect_equal(slotNames(peakFitAnnotation), expected_slotName)
  expect_true(is.character(peakFitAnnotation@cpdID))
  expect_equal(length(peakFitAnnotation@cpdID), 0)
  expect_true(is.character(peakFitAnnotation@cpdName))
  expect_equal(length(peakFitAnnotation@cpdName), 0)
  expect_equal(peakFitAnnotation@ROI, expected_ROI)
  expect_equal(peakFitAnnotation@FIR, expected_FIR)
  expect_equal(peakFitAnnotation@uROI, expected_uROI)
  expect_equal(peakFitAnnotation@filepath, expected_filepath)
  expect_equal(peakFitAnnotation@cpdMetadata, expected_cpdMetadata)
  expect_equal(peakFitAnnotation@spectraMetadata, expected_spectraMetadata)
  expect_equal(peakFitAnnotation@acquisitionTime, expected_acquisitionTime)
  expect_false(peakFitAnnotation@uROIExist)
  expect_false(peakFitAnnotation@useUROI)
  expect_false(peakFitAnnotation@useFIR)
  expect_equal(peakFitAnnotation@TIC, expected_TIC)
  expect_equal(peakFitAnnotation@peakTables, expected_peakTables)
  expect_equal(peakFitAnnotation@dataPoints, expected_dataPoints)
  expect_equal(peakFitAnnotation@peakFit, input_peakFit)     # change is here
  expect_false(peakFitAnnotation@isAnnotated)
  
  # Trigger checks on spectraPaths input
  # spectraPaths not a character vector
  expect_error(peakPantheRAnnotation(spectraPaths=c(1,2,3)), 'specified spectraPaths is not a vector of character')
  expect_error(peakPantheRAnnotation(spectraPaths=list()), 'specified spectraPaths is not a vector of character')
  # duplicated spectraPaths
  expect_error(peakPantheRAnnotation(spectraPaths=c('./path/file1', './path/file1', './path/file2')), 'duplicated spectraPaths passed as input')
})

test_that('initialize with targetFeatTable', {
  # Input and expected values
  input_targetFeatTable             <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
  input_targetFeatTable[1,]         <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390., 522.194778, 522.2, 522.205222)
  input_targetFeatTable[2,]         <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440., 496.195038, 496.2, 496.204962)
  input_targetFeatTable[,c(3:8)]  <- sapply(input_targetFeatTable[,c(3:8)], as.numeric)
  input_FIR         <- data.frame(rtMin=numeric(), rtMax=numeric(), mzMin=numeric(), mzMax=numeric(), stringsAsFactors=FALSE)
  input_FIR[1,]     <- c(1., 2., 3., 4.)
  input_FIR[2,]     <- c(5., 6., 7., 8.)
  input_uROI        <- data.frame(rtMin=numeric(), rt=numeric(), rtMax=numeric(), mzMin=numeric(), mz=numeric(), mzMax=numeric(), stringsAsFactors=FALSE)
  input_uROI[1,]    <- c(10., 11., 12., 13., 14., 15.)
  input_uROI[2,]    <- c(16., 17., 18., 19., 20., 21.)
  input_cpdMetadata <- data.frame(matrix(data=c('a','b'), nrow=2, ncol=1, dimnames=list(c(),c('testcol')), byrow=TRUE), stringsAsFactors=FALSE)
  expected_slotName         <- c("cpdID", "cpdName", "ROI", "FIR", "uROI", "filepath", "cpdMetadata", "spectraMetadata", "acquisitionTime", "uROIExist", "useUROI", "useFIR", "TIC", "peakTables", "dataPoints", "peakFit", "isAnnotated")
  expected_cpdID            <- c("ID-1", "ID-2")
  expected_cpdName          <- c('Cpd 1', 'Cpd 2')
  expected_ROI              <- data.frame(matrix(vector(), 2, 6, dimnames=list(c(), c("rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
  expected_ROI[1,]          <- c(3310., 3344.888, 3390., 522.194778, 522.2, 522.205222)
  expected_ROI[2,]          <- c(3280., 3385.577, 3440., 496.195038, 496.2, 496.204962)
  expected_FIR              <- data.frame(rtMin=as.numeric(rep(NA,2)), rtMax=as.numeric(rep(NA,2)), mzMin=as.numeric(rep(NA,2)), mzMax=as.numeric(rep(NA,2)), stringsAsFactors=FALSE)
  expected_uROI             <- data.frame(rtMin=as.numeric(rep(NA,2)), rt=as.numeric(rep(NA,2)), rtMax=as.numeric(rep(NA,2)), mzMin=as.numeric(rep(NA,2)), mz=as.numeric(rep(NA,2)), mzMax=as.numeric(rep(NA,2)), stringsAsFactors=FALSE)
  expected_cpdMetadata      <- data.frame(matrix(, nrow=2, ncol=0))
  expected_spectraMetadata  <- data.frame(matrix(, nrow=0, ncol=0))


  # Init object with targetFeatTable
  targetFeatTableAnnotation <- peakPantheRAnnotation(targetFeatTable=input_targetFeatTable)
  # Check object values
  expect_true(is(targetFeatTableAnnotation, "peakPantheRAnnotation"))
  expect_equal(slotNames(targetFeatTableAnnotation), expected_slotName)
  expect_equal(targetFeatTableAnnotation@cpdID, expected_cpdID)
  expect_equal(targetFeatTableAnnotation@cpdName, expected_cpdName)
  expect_equal(targetFeatTableAnnotation@ROI, expected_ROI)
  expect_equal(targetFeatTableAnnotation@FIR, expected_FIR)
  expect_equal(targetFeatTableAnnotation@uROI, expected_uROI)
  expect_true(is.character(targetFeatTableAnnotation@filepath))
  expect_equal(length(targetFeatTableAnnotation@filepath), 0)
  expect_true(is.character(targetFeatTableAnnotation@acquisitionTime))
  expect_equal(targetFeatTableAnnotation@cpdMetadata, expected_cpdMetadata)
  expect_equal(targetFeatTableAnnotation@spectraMetadata, expected_spectraMetadata)
  expect_equal(length(targetFeatTableAnnotation@acquisitionTime), 0)
  expect_false(targetFeatTableAnnotation@uROIExist)
  expect_false(targetFeatTableAnnotation@useUROI)
  expect_false(targetFeatTableAnnotation@useFIR)
  expect_true(is.numeric(targetFeatTableAnnotation@TIC))
  expect_equal(length(targetFeatTableAnnotation@TIC), 0)
  expect_true(is.list(targetFeatTableAnnotation@peakTables))
  expect_equal(length(targetFeatTableAnnotation@peakTables), 0)
  expect_true(is.list(targetFeatTableAnnotation@dataPoints))
  expect_equal(length(targetFeatTableAnnotation@dataPoints), 0)
  expect_true(is.list(targetFeatTableAnnotation@peakFit))
  expect_equal(length(targetFeatTableAnnotation@peakFit), 0)
  expect_false(targetFeatTableAnnotation@isAnnotated)

  # Provide a cpdMetadata that shouldn't get replaced
  cpdMetadataAnnotation <- peakPantheRAnnotation(targetFeatTable=input_targetFeatTable, cpdMetadata=input_cpdMetadata)
  # check object
  expect_true(is(cpdMetadataAnnotation, "peakPantheRAnnotation"))
  expect_equal(slotNames(cpdMetadataAnnotation), expected_slotName)
  expect_equal(cpdMetadataAnnotation@cpdID, expected_cpdID)
  expect_equal(cpdMetadataAnnotation@cpdName, expected_cpdName)
  expect_equal(cpdMetadataAnnotation@ROI, expected_ROI)
  expect_equal(cpdMetadataAnnotation@FIR, expected_FIR)
  expect_equal(cpdMetadataAnnotation@uROI, expected_uROI)
  expect_true(is.character(cpdMetadataAnnotation@filepath))
  expect_equal(length(cpdMetadataAnnotation@filepath), 0)
  expect_equal(cpdMetadataAnnotation@cpdMetadata, input_cpdMetadata) # change is here
  expect_equal(cpdMetadataAnnotation@spectraMetadata, expected_spectraMetadata)
  expect_true(is.character(cpdMetadataAnnotation@acquisitionTime))
  expect_equal(length(cpdMetadataAnnotation@acquisitionTime), 0)
  expect_false(cpdMetadataAnnotation@uROIExist)
  expect_false(cpdMetadataAnnotation@useUROI)
  expect_false(cpdMetadataAnnotation@useFIR)
  expect_true(is.numeric(cpdMetadataAnnotation@TIC))
  expect_equal(length(cpdMetadataAnnotation@TIC), 0)
  expect_true(is.list(cpdMetadataAnnotation@peakTables))
  expect_equal(length(cpdMetadataAnnotation@peakTables), 0)
  expect_true(is.list(cpdMetadataAnnotation@dataPoints))
  expect_equal(length(cpdMetadataAnnotation@dataPoints), 0)
  expect_true(is.list(cpdMetadataAnnotation@peakFit))
  expect_equal(length(cpdMetadataAnnotation@peakFit), 0)
  expect_false(cpdMetadataAnnotation@isAnnotated)
  
  # Provide a FIR that shouldn't get replaced, and set useFIR
  FIRAnnotation <- peakPantheRAnnotation(targetFeatTable=input_targetFeatTable, FIR=input_FIR, useFIR=TRUE)
  # check object
  expect_true(is(FIRAnnotation, "peakPantheRAnnotation"))
  expect_equal(slotNames(FIRAnnotation), expected_slotName)
  expect_equal(FIRAnnotation@cpdID, expected_cpdID)
  expect_equal(FIRAnnotation@cpdName, expected_cpdName)
  expect_equal(FIRAnnotation@ROI, expected_ROI)
  expect_equal(FIRAnnotation@FIR, input_FIR) # change is here
  expect_equal(FIRAnnotation@uROI, expected_uROI)
  expect_true(is.character(FIRAnnotation@filepath))
  expect_equal(length(FIRAnnotation@filepath), 0)
  expect_equal(FIRAnnotation@cpdMetadata, expected_cpdMetadata)
  expect_equal(FIRAnnotation@spectraMetadata, expected_spectraMetadata)
  expect_true(is.character(FIRAnnotation@acquisitionTime))
  expect_equal(length(FIRAnnotation@acquisitionTime), 0)
  expect_false(FIRAnnotation@uROIExist)
  expect_false(FIRAnnotation@useUROI)
  expect_true(FIRAnnotation@useFIR) # change is here
  expect_true(is.numeric(FIRAnnotation@TIC))
  expect_equal(length(FIRAnnotation@TIC), 0)
  expect_true(is.list(FIRAnnotation@peakTables))
  expect_equal(length(FIRAnnotation@peakTables), 0)
  expect_true(is.list(FIRAnnotation@dataPoints))
  expect_equal(length(FIRAnnotation@dataPoints), 0)
  expect_true(is.list(FIRAnnotation@peakFit))
  expect_equal(length(FIRAnnotation@peakFit), 0)
  expect_false(FIRAnnotation@isAnnotated)

  # Provide a uROI that shouldn't get replaced, and set uROIExist
  uROIAnnotation <- peakPantheRAnnotation(targetFeatTable=input_targetFeatTable, uROI=input_uROI, uROIExist=TRUE)
  # check object
  expect_true(is(uROIAnnotation, "peakPantheRAnnotation"))
  expect_equal(slotNames(uROIAnnotation), expected_slotName)
  expect_equal(uROIAnnotation@cpdID, expected_cpdID)
  expect_equal(uROIAnnotation@cpdName, expected_cpdName)
  expect_equal(uROIAnnotation@ROI, expected_ROI)
  expect_equal(uROIAnnotation@FIR, expected_FIR)
  expect_equal(uROIAnnotation@uROI, input_uROI) # change is here
  expect_true(is.character(uROIAnnotation@filepath))
  expect_equal(length(uROIAnnotation@filepath), 0)
  expect_equal(uROIAnnotation@cpdMetadata, expected_cpdMetadata)
  expect_equal(uROIAnnotation@spectraMetadata, expected_spectraMetadata)
  expect_true(is.character(uROIAnnotation@acquisitionTime))
  expect_equal(length(uROIAnnotation@acquisitionTime), 0)
  expect_true(uROIAnnotation@uROIExist) # change is here
  expect_false(uROIAnnotation@useUROI)
  expect_false(uROIAnnotation@useFIR)
  expect_true(is.numeric(uROIAnnotation@TIC))
  expect_equal(length(uROIAnnotation@TIC), 0)
  expect_true(is.list(uROIAnnotation@peakTables))
  expect_equal(length(uROIAnnotation@peakTables), 0)
  expect_true(is.list(uROIAnnotation@dataPoints))
  expect_equal(length(uROIAnnotation@dataPoints), 0)
  expect_true(is.list(uROIAnnotation@peakFit))
  expect_equal(length(uROIAnnotation@peakFit), 0)
  expect_false(uROIAnnotation@isAnnotated)

  # Force uROIExist and useUROI to FALSE (despite setting to TRUE) as uROI is reset
  uROIResetAnnotation <- peakPantheRAnnotation(targetFeatTable=input_targetFeatTable, uROIExist=TRUE, useUROI=TRUE)
  # check object
  expect_true(is(uROIResetAnnotation, "peakPantheRAnnotation"))
  expect_equal(slotNames(uROIResetAnnotation), expected_slotName)
  expect_equal(uROIResetAnnotation@cpdID, expected_cpdID)
  expect_equal(uROIResetAnnotation@cpdName, expected_cpdName)
  expect_equal(uROIResetAnnotation@ROI, expected_ROI)
  expect_equal(uROIResetAnnotation@FIR, expected_FIR)
  expect_equal(uROIResetAnnotation@uROI, expected_uROI)
  expect_true(is.character(uROIResetAnnotation@filepath))
  expect_equal(length(uROIResetAnnotation@filepath), 0)
  expect_equal(uROIResetAnnotation@cpdMetadata, expected_cpdMetadata)
  expect_equal(uROIResetAnnotation@spectraMetadata, expected_spectraMetadata)
  expect_true(is.character(uROIResetAnnotation@acquisitionTime))
  expect_equal(length(uROIResetAnnotation@acquisitionTime), 0)
  expect_false(uROIResetAnnotation@uROIExist) # false as ROI is reset
  expect_false(uROIResetAnnotation@useUROI) # false as ROI is reset
  expect_false(uROIResetAnnotation@useFIR)
  expect_true(is.numeric(uROIResetAnnotation@TIC))
  expect_equal(length(uROIResetAnnotation@TIC), 0)
  expect_true(is.list(uROIResetAnnotation@peakTables))
  expect_equal(length(uROIResetAnnotation@peakTables), 0)
  expect_true(is.list(uROIResetAnnotation@dataPoints))
  expect_equal(length(uROIResetAnnotation@dataPoints), 0)
  expect_true(is.list(uROIResetAnnotation@peakFit))
  expect_equal(length(uROIResetAnnotation@peakFit), 0)
  expect_false(uROIResetAnnotation@isAnnotated)


  # Trigger checks on targetFeatTable input
  # targetFeatTable is not a data.frame
  expect_error(peakPantheRAnnotation(targetFeatTable='notADataFrame'), 'specified targetFeatTable is not a data.frame')
  # targetFeatTable has wrong columns
  wrongDF1           <- input_targetFeatTable
  colnames(wrongDF1) <- c("not cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax")
  expect_error(peakPantheRAnnotation(targetFeatTable=wrongDF1), 'expected columns in targetFeatTable are "cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz" and "mzMax"')
  # targetFeatTable$cpdID not character
  wrongDF2        <- input_targetFeatTable
  wrongDF2$cpdID  <- c(5, 5)
  expect_error(peakPantheRAnnotation(targetFeatTable=wrongDF2), 'cpdID must be character')
  # targetFeatTable$cpdName not character
  wrongDF3          <- input_targetFeatTable
  wrongDF3$cpdName  <- c(5, 5)
  expect_error(peakPantheRAnnotation(targetFeatTable=wrongDF3), 'cpdName must be character')
  # targetFeatTable$rtMin not numeric
  wrongDF4        <- input_targetFeatTable
  wrongDF4$rtMin  <- c('notNumeric', 'notNumeric')
  expect_error(peakPantheRAnnotation(targetFeatTable=wrongDF4), 'rtMin must be numeric')
  # targetFeatTable$rt not numeric or NA
  wrongDF5        <- input_targetFeatTable
  wrongDF5$rt     <- c('notNumeric', 'notNumeric')
  expect_error(peakPantheRAnnotation(targetFeatTable=wrongDF5), 'rt must be numeric or NA')
  # targetFeatTable$rtMax not numeric
  wrongDF6        <- input_targetFeatTable
  wrongDF6$rtMax  <- c('notNumeric', 'notNumeric')
  expect_error(peakPantheRAnnotation(targetFeatTable=wrongDF6), 'rtMax must be numeric')
  # targetFeatTable$mzMin not numeric
  wrongDF7        <- input_targetFeatTable
  wrongDF7$mzMin  <- c('notNumeric', 'notNumeric')
  expect_error(peakPantheRAnnotation(targetFeatTable=wrongDF7), 'mzMin must be numeric')
  # targetFeatTable$mz not numeric or NA
  wrongDF8        <- input_targetFeatTable
  wrongDF8$mz     <- c('notNumeric', 'notNumeric')
  expect_error(peakPantheRAnnotation(targetFeatTable=wrongDF8), 'mz must be numeric or NA')
  # targetFeatTable$mzMax not numeric
  wrongDF9        <- input_targetFeatTable
  wrongDF9$mzMax  <- c('notNumeric', 'notNumeric')
  expect_error(peakPantheRAnnotation(targetFeatTable=wrongDF9), 'mzMax must be numeric')
})

context('initialise_annotation_from_files_UI_helper()')

## Test the initialisation of a full annotation object from parameter files and metadata paths
# No need to check different CSV format as this is managed and tested in peakPantheR_loadAnnotationParamsCSV()
# In order to be more understandable by the user, we raise a warning for cpd and spectra metadata size (instead of catching/crashing it in peakPantherAnnotation_validObject())
# If cpd or spectraMetadata files do not exist, raise an error

## Input data
input_param      <- data.frame(matrix(nrow=2,ncol=21,dimnames=list(c(), c('cpdID', 'cpdName', 'X', 'ROI_rt', 'ROI_mz','ROI_rtMin', 'ROI_rtMax', 'ROI_mzMin', 'ROI_mzMax', 'X', 'uROI_rtMin', 'uROI_rtMax', 'uROI_mzMin', 'uROI_mzMax', 'uROI_rt', 'uROI_mz', 'X', 'FIR_rtMin', 'FIR_rtMax', 'FIR_mzMin', 'FIR_mzMax'))))
input_param[1,]  <- c('ID-1', 'Cpd 1', '|', 1.,  2.,  3.,  4.,  5.,  6.,  '|', 7.,  8.,  9.,  10., 11., 12., '|', 13., 14., 15., 16.)
input_param[2,]  <- c('ID-2', 'Cpd 2', '|', 17., 18., 19., 20., 21., 22., '|', 23., 24., 25., 26., 27., 28., '|', 29., 30., 31., 32.)
input_param[,-c(1,2,3,10,17)]  <- sapply(input_param[,-c(1,2,3,10,17)], as.numeric)

# spectraPath
input_spectraPaths    <- c('./path/file1', './path/file2', './path/file3')

# spectraMetadata
input_spectraMetadata <- data.frame(matrix(data=c('a','b','c'), nrow=3, ncol=1, dimnames=list(c(),c('testcol')), byrow=TRUE), stringsAsFactors=FALSE)

# cpdMetadata
input_cpdMetadata     <- data.frame(matrix(data=c('a','b'), nrow=2, ncol=1, dimnames=list(c(),c('testcol')), byrow=TRUE), stringsAsFactors=FALSE)

# temporary files
paramPath             <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.csv')
utils::write.csv(input_param, file=paramPath, row.names=FALSE)
cpdMetaPath           <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.csv')
utils::write.csv(input_cpdMetadata, file=cpdMetaPath, row.names=FALSE)


## Expected result
# targetFeatTable
input_targetFeatTable_adv     <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable_adv[1,] <- c("ID-1", "Cpd 1",  3.,  1.,  4.,  5.,  2.,  6.)
input_targetFeatTable_adv[2,] <- c("ID-2", "Cpd 2", 19., 17., 20., 21., 18., 22.)
input_targetFeatTable_adv[,c(3:8)] <- sapply(input_targetFeatTable_adv[,c(3:8)], as.numeric)

# uROI
input_uROI_adv      <- data.frame(matrix(vector(), 2, 6, dimnames=list(c(), c("rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
input_uROI_adv[1,]  <- c( 7., 11. , 8.,  9., 12., 10.)
input_uROI_adv[2,]  <- c(23., 27., 24., 25., 28., 26.)

# FIR
input_FIR_adv       <- data.frame(matrix(vector(), 2, 4, dimnames=list(c(), c("rtMin", "rtMax", "mzMin", "mzMax"))), stringsAsFactors=FALSE)
input_FIR_adv[1,]   <- c(13., 14., 15., 16.)
input_FIR_adv[2,]   <- c(29., 30., 31., 32.)

# object
input_annotation_nometa <- peakPantheRAnnotation(targetFeatTable = input_targetFeatTable_adv,
                                                 spectraPaths = input_spectraPaths,
                                                 uROI = input_uROI_adv,
                                                 FIR = input_FIR_adv,
                                                 uROIExist = TRUE)
input_annotation_meta   <- peakPantheRAnnotation(targetFeatTable = input_targetFeatTable_adv,
                                                 spectraPaths = input_spectraPaths,
                                                 uROI = input_uROI_adv,
                                                 FIR = input_FIR_adv,
                                                 uROIExist = TRUE,
                                                 cpdMetadata = input_cpdMetadata,
                                                 spectraMetadata = input_spectraMetadata)


test_that('parameters, spectraPaths, no metadata, verbose, no verbose', {
    # expected
    expected_annotation <- input_annotation_nometa
    expected_message    <- "An object of class peakPantheRAnnotation\n 2 compounds in 3 samples. \n  updated ROI exist (uROI)\n  does not use updated ROI (uROI)\n  does not use fallback integration regions (FIR)\n  is not annotated"

    # results (output, warnings and messages)
    result_load1        <- evaluate_promise(initialise_annotation_from_files_UI_helper(paramPath, input_spectraPaths,
                                                                                     cpdMetadataPath = NULL,
                                                                                     spectraMetadata = NULL,
                                                                                     verbose = TRUE))

    # Check result
    expect_equal(result_load1$result, expected_annotation)

    # Check result messages (in output)
    expect_equal(length(result_load1$output), 1)
    expect_equal(result_load1$output, expected_message)

    ## no verbose
    result_load2        <- evaluate_promise(initialise_annotation_from_files_UI_helper(paramPath, input_spectraPaths,
                                                                                     cpdMetadataPath = NULL,
                                                                                     spectraMetadata = NULL,
                                                                                     verbose = FALSE))
    expect_equal(length(result_load2$messages), 0)
    expect_equal(result_load2$output, "")
})

test_that('parameters, spectraPaths, with correct metadata, verbose, no verbose', {
    # expected
    expected_annotation <- input_annotation_meta
    expected_message    <- "An object of class peakPantheRAnnotation\n 2 compounds in 3 samples. \n  updated ROI exist (uROI)\n  does not use updated ROI (uROI)\n  does not use fallback integration regions (FIR)\n  is not annotated"

    # results (output, warnings and messages)
    result_load1        <- evaluate_promise(initialise_annotation_from_files_UI_helper(paramPath, input_spectraPaths,
                                                                                       cpdMetadataPath = cpdMetaPath,
                                                                                       spectraMetadata = input_spectraMetadata,
                                                                                       verbose = TRUE))

    # Check result
    expect_equal(result_load1$result, expected_annotation)

    # Check result messages
    expect_equal(length(result_load1$output), 1)
    expect_equal(result_load1$output, expected_message)

    ## no verbose
    result_load2        <- evaluate_promise(initialise_annotation_from_files_UI_helper(paramPath, input_spectraPaths,
                                                                                       cpdMetadataPath = cpdMetaPath,
                                                                                       spectraMetadata = input_spectraMetadata,
                                                                                       verbose = FALSE))
    expect_equal(length(result_load2$messages), 0)
    expect_equal(result_load2$output, "")
})

test_that('parameters, spectraPaths, with wrong sized metadata, verbose, no verbose', {
    # input
    wrong_spectraMetadata <- data.frame(matrix(data=c('a','b'), nrow=2, ncol=1, dimnames=list(c(),c('testcol')), byrow=TRUE), stringsAsFactors=FALSE)
    wrong_cpdMetadata     <- data.frame(matrix(data=c('a','b','c'), nrow=3, ncol=1, dimnames=list(c(),c('testcol')), byrow=TRUE), stringsAsFactors=FALSE)
    cpdMetaPath2          <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.csv')
    utils::write.csv(wrong_cpdMetadata, file=cpdMetaPath2, row.names=FALSE)

    # expected (no metadata version as meta files are rejected based on number of rows)
    expected_annotation <- input_annotation_nometa
    expected_output     <- "An object of class peakPantheRAnnotation\n 2 compounds in 3 samples. \n  updated ROI exist (uROI)\n  does not use updated ROI (uROI)\n  does not use fallback integration regions (FIR)\n  is not annotated"
    expected_message    <- c("Warning: cpdMetadata number of rows (3) does not match the number of compounds targeted (2)\n",
                           "Warning: spectraMetadata number of rows (2) does not match the number of compounds targeted (3)\n")

    # results (output, warnings and messages)
    result_load1        <- evaluate_promise(initialise_annotation_from_files_UI_helper(paramPath, input_spectraPaths,
                                                                                     cpdMetadataPath = cpdMetaPath2,
                                                                                     spectraMetadata = wrong_spectraMetadata,
                                                                                     verbose = TRUE))

    # Check result
    expect_equal(result_load1$result, expected_annotation)

    # Check output (show object)
    expect_equal(length(result_load1$output), 1)
    expect_equal(result_load1$output, expected_output)
    # Check message (from metadata number of rows) /!\ 3rd message is only '\n'
    expect_equal(length(result_load1$message), 3)
    expect_equal(result_load1$message[1:2], expected_message)

    ## no verbose
    result_load2        <- evaluate_promise(initialise_annotation_from_files_UI_helper(paramPath, input_spectraPaths,
                                                                                     cpdMetadataPath = cpdMetaPath2,
                                                                                     spectraMetadata = wrong_spectraMetadata,
                                                                                     verbose = FALSE))
    # Check output (show object, return only an empty value)
    expect_equal(length(result_load2$output), 1)
    expect_equal(result_load2$output, "")
    # Check message (from metadata number of rows)
    expect_equal(length(result_load2$message), 2)
    expect_equal(result_load2$message, expected_message)
})

test_that('raise errors', {
    # loadAnnotationParamCSV() error
    # no spectraPaths is safe (spectraPaths = NULL, init with 0 samples)


    noFile  <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.csv')

    # CSVParam file doesn't exist (loadAnnotationParamCSV error)
    msg1    <- paste('specified "CSVParamPath" does not exist', sep='')
    expect_error(initialise_annotation_from_files_UI_helper(noFile, NULL, verbose=TRUE), msg1, fixed=TRUE)

    # cpdMetadata file doesn't exist
    msg2    <- paste('Error: cpdMetadata file does not exist', sep='')
    expect_error(initialise_annotation_from_files_UI_helper(paramPath, NULL, cpdMetadataPath=noFile, verbose=TRUE), msg2, fixed=TRUE)

    # spectraMetadata is not a DataFrame
    msg3    <- paste('Error: spectraMetadata is not a DataFrame', sep='')
    expect_error(initialise_annotation_from_files_UI_helper(paramPath, NULL, spectraMetadata='notADataFrame', verbose=TRUE), msg3, fixed=TRUE)

})

context('peakPantheRAnnotation_retentionTimeCorrection')

## Test the output of retention time correction

skip_if_not_installed('faahKO',  minimum_version = '1.18.0')
library(faahKO)

on.exit( tryCatch({ file.remove('./Rplots.pdf') }, error=function(e){ invisible() }, warning=function(w){ invisible() }) )
on.exit( tryCatch({ file.remove('./test_ggsave.png') }, error=function(e){ invisible() }, warning=function(w){ invisible() }) )

## Input data
# spectraPaths
input_spectraPaths  <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                         system.file('cdf/KO/ko16.CDF', package = "faahKO"),
                         system.file('cdf/KO/ko18.CDF', package = "faahKO"))

# targetFeatTable
input_targetFeatTable     <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390., 522.194778, 522.2, 522.205222)
input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440., 496.195038, 496.2, 496.204962)
input_targetFeatTable[,c(3:8)] <- sapply(input_targetFeatTable[,c(3:8)], as.numeric)

# FIR
input_FIR     <- data.frame(matrix(vector(), 2, 4, dimnames=list(c(), c("rtMin", "rtMax", "mzMin", "mzMax"))), stringsAsFactors=FALSE)
input_FIR[1,] <- c(1., 2., 3., 4.)
input_FIR[2,] <- c(5., 6., 7., 8.)

# uROI
input_uROI      <- data.frame(matrix(vector(), 2, 6, dimnames=list(c(), c("rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
input_uROI[1,]  <- c(9., 10., 11., 12., 13., 14.)
input_uROI[2,]  <- c(15., 16., 17., 18., 19., 20.)

# TICs
input_TIC <- c(2410533091, 2524040155, 2332817115)

# cpdMetadata
input_cpdMetadata     <- data.frame(matrix(data=c('a','b',1,2), nrow=2, ncol=2, dimnames=list(c(),c('testcol1','testcol2')), byrow=FALSE), stringsAsFactors=FALSE)

# spectraMetadata
input_spectraMetadata <- data.frame(matrix(data=c('c','d','e',3,4,5), nrow=3, ncol=2, dimnames=list(c(),c('testcol1','testcol2')), byrow=FALSE), stringsAsFactors=FALSE)

# acquisitionTime
input_acquisitionTime <- c(as.character(Sys.time()), as.character(Sys.time()+900), as.character(Sys.time()+1800))

# peakTables
# 1
peakTable1     <- data.frame(matrix(vector(), 2, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
peakTable1[1,] <- c(TRUE, 3309.7589296586070, 3346.8277590361445, 3385.4098874628098, 522.194778, 522.20001220703125, 522.205222, 26133726.6811244078, 26133726, 889280, 901015.80529226747, FALSE, 0.023376160866574614, 1.93975903614455092, 1.0153573486330891, 1.0268238825675249)
peakTable1[2,] <- c(TRUE, 3345.3766648628907, 3386.5288072289159, 3428.2788374983961, 496.20001220703125, 496.20001220703125, 496.20001220703125, 35472141.3330242932, 35472141, 1128960, 1113576.69008227298, FALSE, 0.024601030353423384, 0.95180722891564074, 1.0053782620427065, 1.0093180792278085)
peakTable1[,c(1,12)]       <- sapply(peakTable1[,c(1,12)], as.logical)
peakTable1[,c(2:11,13:16)] <- sapply(peakTable1[,c(2:11,13:16)], as.numeric)
# 2
peakTable2     <- data.frame(matrix(vector(), 2, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
peakTable2[1,] <- c(TRUE, 3326.1063495851854, 3365.102, 3407.2726475892355, 522.194778, 522.20001220703125, 522.205222, 24545301.622835573, 24545301, 761664, 790802.2209998488, FALSE, 0.023376160866574614, 0.2139999999999, 1.0339153786516375, 1.0630802030537212)
peakTable2[2,] <- c(TRUE, 3365.0238566258713, 3405.791, 3453.4049569205681, 496.195038, 496.20001220703125, 496.204962, 37207579.286265120, 37207579, 1099264, 1098720.2929832144, FALSE, 0.024601030353423384, 20.2139999999999, 1.0839602450900523, 1.1717845972583161)
peakTable2[,c(1,12)]       <- sapply(peakTable2[,c(1,12)], as.logical)
peakTable2[,c(2:11,13:16)] <- sapply(peakTable2[,c(2:11,13:16)], as.numeric)
# 3
peakTable3     <- data.frame(matrix(vector(), 2, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
peakTable3[1,] <- c(TRUE, 3333.8625894557053, 3368.233, 3407.4362838927614, 522.194778, 522.20001220703125, 522.205222, 21447174.404490683, 21447174, 758336, 765009.9805796633, FALSE, 0.023376160866574614, 23.345000000000255, 1.0609102044546637, 1.1155310457756928)
peakTable3[2,] <- c(TRUE, 3373.3998828113113, 3413.4952530120481, 3454.4490330927388, 496.195038, 496.20001220703125, 496.204962, 35659353.614476241, 35659353, 1149440, 1145857.7611069249, TRUE, 0.024601030353423384, 27.918253012047899, 1.0081407426394933, 1.0143315197994494)
peakTable3[,c(1,12)]       <- sapply(peakTable3[,c(1,12)], as.logical)
peakTable3[,c(2:11, 13:16)] <- sapply(peakTable3[,c(2:11,13:16)], as.numeric)
input_peakTables <- list(peakTable1, peakTable2, peakTable3)

# peakFit
# 1
cFit1.1         <- list(amplitude=162404.8057918259, center=3341.888, sigma=0.078786133031045896, gamma=0.0018336101984172684, fitStatus=2, curveModel="skewedGaussian")
class(cFit1.1)  <- 'peakPantheR_curveFit'
cFit1.2         <- list(amplitude=199249.10572753669, center=3382.577, sigma=0.074904415304607966, gamma=0.0011471899372353885, fitStatus=2, curveModel="skewedGaussian")
class(cFit1.2)  <- 'peakPantheR_curveFit'
# 2
cFit2.1         <- list(amplitude=124090.83425474487, center=3359.102, sigma=0.071061541060964212, gamma=0.0018336072657203239, fitStatus=2, curveModel="skewedGaussian")
class(cFit2.1)  <- 'peakPantheR_curveFit'
cFit2.2         <- list(amplitude=151407.23415130575, center=3399.791, sigma=0.063753866057052563, gamma=0.001676782834598999, fitStatus=2, curveModel="skewedGaussian")
class(cFit2.2)  <- 'peakPantheR_curveFit'
# 3
cFit3.1         <- list(amplitude=122363.51256736703, center=3362.233, sigma=0.075489598945304492, gamma=0.0025160536725299734, fitStatus=2, curveModel="skewedGaussian")
class(cFit3.1)  <- 'peakPantheR_curveFit'
cFit3.2         <- list(amplitude=204749.86097918145, center=3409.182, sigma=0.075731781812843249, gamma=0.0013318670577834328, fitStatus=2, curveModel="skewedGaussian")
class(cFit3.2)  <- 'peakPantheR_curveFit'
input_peakFit   <- list(list(cFit1.1, cFit1.2), list(cFit2.1, cFit2.2), list(cFit3.1, cFit3.2))

# dataPoint
tmp_raw_data1  	  <- MSnbase::readMSData(input_spectraPaths[1], centroided=TRUE, mode='onDisk')
ROIDataPoints1    <- extractSignalRawData(tmp_raw_data1, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
tmp_raw_data2  	  <- MSnbase::readMSData(input_spectraPaths[2], centroided=TRUE, mode='onDisk')
ROIDataPoints2    <- extractSignalRawData(tmp_raw_data2, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
tmp_raw_data3  	  <- MSnbase::readMSData(input_spectraPaths[3], centroided=TRUE, mode='onDisk')
ROIDataPoints3    <- extractSignalRawData(tmp_raw_data3, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
input_dataPoints  <- list(ROIDataPoints1, ROIDataPoints2, ROIDataPoints3)

# Object, fully filled
filledAnnotation  <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable, FIR=input_FIR, uROI=input_uROI, useFIR=TRUE, uROIExist=TRUE, useUROI=TRUE, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata, acquisitionTime=input_acquisitionTime, TIC=input_TIC, peakTables=input_peakTables, dataPoints=input_dataPoints, peakFit=input_peakFit, isAnnotated=TRUE)


test_that('rt correction on uROI, no plot', {
  # Test if object is modified correctly using uROI (original RT based on useUROI)

  # input
  annotation <- filledAnnotation

  # expected ROI/uROI
  expected_uROI     <- data.frame(rtMin=double(), rt=double(), rtMax=double(), mzMin=double(), mz=double(), mzMax=double(), stringsAsFactors=FALSE)
  expected_uROI[1,] <- list(rtMin=-5.9995863454, rt=1.500414, rtMax=9.000414, mzMin=12, mz=13, mzMax=14)
  expected_uROI[2,] <- list(rtMi=0.0004136546, rt=7.500414, rtMax=15.000414, mzMin=18, mz=19, mzMax=20)
  expected_FIR     <- data.frame(rtMin=double(), rtMax=double(), mzMin=double(), mzMax=double(), stringsAsFactors=FALSE)
  expected_FIR[1,] <- list(rtMin=-5.9995863454, rtMax=9.000414, mzMin=3, mzMax=4)
  expected_FIR[2,] <- list(rtMin=0.0004136546, rtMax=15.000414, mzMin=7, mzMax=8)

  # run RT correction
  resultCorrected <- evaluate_promise(retentionTimeCorrection(annotation, rtCorrectionReference=c('ID-1'), method='constant', params=list(polynomialOrder=1), robust=TRUE, diagnostic=FALSE))

  # check results (output, warnings and messages)
  expect_equal(resultCorrected$result$annotation@uROI, expected_uROI, tolerance=1e-5)
  expect_equal(resultCorrected$result$annotation@FIR,  expected_FIR, tolerance=1e-5)

  # check no plot (only 'annotation' in return)
  expect_equal(length(resultCorrected$result), 1)
})

test_that('rt correction on ROI, no plot', {
  # Test if object is modified correctly using ROI (original RT based on useUROI)

  # input
  annotation         <- filledAnnotation
  annotation@useUROI <- FALSE

  # expected ROI/uROI
  expected_uROI_useROI     <- data.frame(rtMin=double(), rt=double(), rtMax=double(), mzMin=double(), mz=double(), mzMax=double(), stringsAsFactors=FALSE)
  expected_uROI_useROI[1,] <- list(rtMin=3328.888, rt=3336.388, rtMax=3343.888, mzMin=12, mz=13, mzMax=14)
  expected_uROI_useROI[2,] <- list(rtMin=3369.577, rt=3377.077, rtMax=3384.577, mzMin=18, mz=19, mzMax=20)
  expected_FIR_useROI     <- data.frame(rtMin=double(), rtMax=double(), mzMin=double(), mzMax=double(), stringsAsFactors=FALSE)
  expected_FIR_useROI[1,] <- list(rtMin=3328.888, rtMax=3343.888, mzMin=3, mzMax=4)
  expected_FIR_useROI[2,] <- list(rtMin=3369.577, rtMax=3384.577, mzMin=7, mzMax=8)

  # run RT correction
  resultCorrected_useROI <- evaluate_promise(retentionTimeCorrection(annotation, rtCorrectionReference=c('ID-1'), method='constant', params=list(polynomialOrder=1), robust=TRUE, diagnostic=FALSE))

  # check results (output, warnings and messages)
  expect_equal(resultCorrected_useROI$result$annotation@uROI, expected_uROI_useROI, tolerance=1e-5)
  expect_equal(resultCorrected_useROI$result$annotation@FIR,  expected_FIR_useROI, tolerance=1e-5)

  # check no plot (only 'annotation' in return)
  expect_equal(length(resultCorrected_useROI$result), 1)
})

test_that('rt correction plot (uROI)', {
  # test plot is correctly generated

  # input
  annotation <- filledAnnotation

  # expected ROI/uROI
  expected_uROI     <- data.frame(rtMin=double(), rt=double(), rtMax=double(), mzMin=double(), mz=double(), mzMax=double(), stringsAsFactors=FALSE)
  expected_uROI[1,] <- list(rtMin=-5.9995863454, rt=1.500414, rtMax=9.000414, mzMin=12, mz=13, mzMax=14)
  expected_uROI[2,] <- list(rtMi=0.0004136546, rt=7.500414, rtMax=15.000414, mzMin=18, mz=19, mzMax=20)
  expected_FIR     <- data.frame(rtMin=double(), rtMax=double(), mzMin=double(), mzMax=double(), stringsAsFactors=FALSE)
  expected_FIR[1,] <- list(rtMin=-5.9995863454, rtMax=9.000414, mzMin=3, mzMax=4)
  expected_FIR[2,] <- list(rtMin=0.0004136546, rtMax=15.000414, mzMin=7, mzMax=8)
  # expected plot properties
  expected_plotFrame <- data.frame(cpdID=character(), cpdName=character(), rt=double(), rt_dev_sec=double(), isReference=character(), correctedRt=double(), predictedRtDrift=double(), stringsAsFactors=FALSE)
  expected_plotFrame[1, ] <- list(cpdID='ID-1', cpdName='Cpd 1', rt=10, rt_dev_sec=8.499586, isReference='Reference set', correctedRt=1.500414, predictedRtDrift=8.499586)
  expected_plotFrame[2, ] <- list(cpdID='ID-2', cpdName='Cpd 2', rt=16, rt_dev_sec=16.361353, isReference='External set', correctedRt=7.500414, predictedRtDrift=8.499586)

  # run RT correction
  resultCorrected <- evaluate_promise(retentionTimeCorrection(annotation, rtCorrectionReference=c('ID-1'), method='constant', params=list(polynomialOrder=1), robust=TRUE, diagnostic=TRUE))

  # check both a plot and an annotation are returned
  expect_equal(length(resultCorrected$result), 2)

  # check annotation in results 
  expect_equal(resultCorrected$result$annotation@uROI, expected_uROI, tolerance=1e-5)
  expect_equal(resultCorrected$result$annotation@FIR,  expected_FIR, tolerance=1e-5)

  # check plot
  expect_true(ggplot2::is_ggplot(resultCorrected$result$plot))
  # check plot data
  test_plot_data <- ggplot2::layer_data(resultCorrected$result$plot, 1) 
  expect_equal(test_plot_data$x, expected_plotFrame$rt, tolerance = 1e-6)
  expect_equal(test_plot_data$y, expected_plotFrame$rt_dev_sec, tolerance = 1e-6)
  # check plot structure
  expect_equal(length(resultCorrected$result$plot$layers), 2)
  expect_equal(resultCorrected$result$plot$labels$x, 'Retention time')
  expect_equal(resultCorrected$result$plot$labels$y, 'Retention time deviation')
  expect_is(resultCorrected$result$plot$layers[[1]]$geom, 'GeomPoint')
  expect_is(resultCorrected$result$plot$layers[[2]]$geom, 'GeomLine')
  expect_error(ggplot2::ggsave('./test_ggsave.png', resultCorrected$result$plot), NA)
})

test_that('rt correction no rtCorrectionReferences', {
  # Test if all cpds are as rtCorrectionReferences if set to NULL, not interested in the values themselves

  # input
  annotation <- filledAnnotation[,1]

  # expected ROI/uROI
  expected_uROI     <- data.frame(rtMin=double(), rt=double(), rtMax=double(), mzMin=double(), mz=double(), mzMax=double(), stringsAsFactors=FALSE)
  expected_uROI[1,] <- list(rtMin=-5.999586, rt=1.5004137, rtMax=9.000414, mzMin=12, mz=13, mzMax=14)
  expected_FIR     <- data.frame(rtMin=double(), rtMax=double(), mzMin=double(), mzMax=double(), stringsAsFactors=FALSE)
  expected_FIR[1,] <- list(rtMin=-5.999586, rtMax=9.000414, mzMin=3, mzMax=4)

  # run RT correction
  resultCorrected <- evaluate_promise(retentionTimeCorrection(annotation, rtCorrectionReference=NULL, method='constant', params=list(polynomialOrder=1), robust=TRUE, diagnostic=FALSE))

  # check results (output, warnings and messages)
  expect_equal(resultCorrected$result$annotation@uROI, expected_uROI, tolerance=1e-5)
  expect_equal(resultCorrected$result$annotation@FIR,  expected_FIR, tolerance=1e-5)

  # check no plot (only 'annotation' in return)
  expect_equal(length(resultCorrected$result), 1)
})

test_that('rt correction referenceTable has NA and targetFeatTable has NA', {
  # Test if compound is excluded due to compound not found in referenceTable

  # input
  annotation <- filledAnnotation
  annotation@peakTables[[1]]$rt_dev_sec[1] <- NA
  annotation@peakTables[[2]]$rt_dev_sec[1] <- NA
  annotation@peakTables[[3]]$rt_dev_sec[1] <- NA

  # expected ROI/uROI
  expected_uROI     <- data.frame(rtMin=double(), rt=double(), rtMax=double(), mzMin=double(), mz=double(), mzMax=double(), stringsAsFactors=FALSE)
  expected_uROI[1,] <- list(rtMin=9., rt=10., rtMax=11., mzMin=12, mz=13, mzMax=14)
  expected_uROI[2,] <- list(rtMi=-7.861353, rt=-0.3613534, rtMax=7.138647, mzMin=18, mz=19, mzMax=20)
  expected_FIR     <- data.frame(rtMin=double(), rtMax=double(), mzMin=double(), mzMax=double(), stringsAsFactors=FALSE)
  expected_FIR[1,] <- list(rtMin=1., rtMax=2., mzMin=3, mzMax=4)
  expected_FIR[2,] <- list(rtMin=-7.861353, rtMax=7.138647, mzMin=7, mzMax=8)

  # expected warnings
  expected_warn <- c('The following references could not be integrated previously and will be excluded: ID-1',
                     'The following compounds could not be integrated previously and will be not be corrected: ID-1')

  # run RT correction
  resultCorrected <- evaluate_promise(retentionTimeCorrection(annotation, rtCorrectionReference=c('ID-1','ID-2'), method='constant', params=list(polynomialOrder=1), robust=TRUE, diagnostic=FALSE))

  # check results (output, warnings and messages)
  expect_equal(resultCorrected$result$annotation@uROI, expected_uROI, tolerance=1e-5)
  expect_equal(resultCorrected$result$annotation@FIR,  expected_FIR, tolerance=1e-5)
  # check warning
  expect_equal(resultCorrected$warnings,  expected_warn)

  # check no plot (only 'annotation' in return)
  expect_equal(length(resultCorrected$result), 1)
})

test_that('rt correction raises errors', {
  # not annotated
  wrong1             <- filledAnnotation
  wrong1@isAnnotated <- FALSE
  msg1               <- 'The retention time correction functionality requires an annotated peakPantheRAnnotation object (annotationObject@isAnnotated = TRUE)'
  expect_error(retentionTimeCorrection(wrong1, rtCorrectionReference=c('ID-1')), msg1, fixed=TRUE)

  # not a valid cpdID
  wrong2             <- filledAnnotation
  msg2               <- 'All compound IDs in rtCorrectionReferences must be present on the annotationObject'
  expect_error(retentionTimeCorrection(wrong2, rtCorrectionReference=c('not a', 'correct cpdID')), msg2, fixed=TRUE)

  # not numeric rtWindowWidth
  wrong3             <- filledAnnotation
  msg3               <- 'rtWindowWidth must be a positive number'
  expect_error(retentionTimeCorrection(wrong3, rtCorrectionReference=c('ID-1'), rtWindowWidth='not a number'), msg3, fixed=TRUE)
  # negative rtWindowWidth
  wrong4             <- filledAnnotation
  msg4               <- 'rtWindowWidth must be a positive number'
  expect_error(retentionTimeCorrection(wrong3, rtCorrectionReference=c('ID-1'), rtWindowWidth=-5), msg4, fixed=TRUE)
})

context('peakPantheR_parallelAnnotation()')

skip_if_not_installed('faahKO',  minimum_version = '1.18.0')
library(faahKO)


## Input and expected data
# 3 files
input_spectraPaths          <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                                 system.file('cdf/KO/ko16.CDF', package = "faahKO"),
                                 system.file('cdf/KO/ko18.CDF', package = "faahKO"))
## 1 missing file
input_missingSpectraPaths   <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                                 "aaa/bbb.cdf",
                                 system.file('cdf/KO/ko18.CDF', package = "faahKO"))

# 4 features
input_targetFeatTable     	<- data.frame(matrix(vector(), 4, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))),stringsAsFactors=FALSE)
input_targetFeatTable[1,] 	<- c("ID-1", "Cpd 1", 3310., 3344.888, 3390., 522.194778, 522.2, 522.205222)
input_targetFeatTable[2,] 	<- c("ID-2", "Cpd 2", 3280., 3385.577, 3440., 496.195038, 496.2, 496.204962)
input_targetFeatTable[3,] 	<- c("ID-3", "Cpd 3", 3420., 3454.435, 3495., 464.195358, 464.2, 464.204642)
input_targetFeatTable[4,] 	<- c("ID-4", "Cpd 4", 3670., 3701.697, 3745., 536.194638, 536.2, 536.205362)
input_targetFeatTable[,c(3:8)] <- sapply(input_targetFeatTable[,c(3:8)], as.numeric)

# FIR
input_FIR     	            <- data.frame(matrix(vector(), 4, 4, dimnames=list(c(), c("rtMin", "rtMax", "mzMin", "mzMax"))),stringsAsFactors=FALSE)
input_FIR[1,] 	            <- c(3336.542, 3390.272, 522.1995, 522.2005)
input_FIR[2,] 	            <- c(3378.274, 3426.266, 496.1995, 496.2005)
input_FIR[3,] 	            <- c(3444.524, 3478.431, 464.1995, 464.2005)
input_FIR[4,] 	            <- c(3689.7,   3738.213, 536.1995, 536.2005)
input_FIR[,c(1:4)]          <- sapply(input_FIR[,c(1:4)], as.numeric)

# uROI (ROI are wrong, uROI are right)
input_uROI                  <- input_targetFeatTable[,c("rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax")]
input_badtargetFeatTable    <- input_targetFeatTable
input_badtargetFeatTable[1, c("rtMin", "rtMax", "mzMin", "mzMax")] <- c(0,10000, 0,1000)
input_badtargetFeatTable[2, c("rtMin", "rtMax", "mzMin", "mzMax")] <- c(0,10000, 0,1000)
input_badtargetFeatTable[3, c("rtMin", "rtMax", "mzMin", "mzMax")] <- c(0,10000, 0,1000)
input_badtargetFeatTable[4, c("rtMin", "rtMax", "mzMin", "mzMax")] <- c(0,10000, 0,1000)

# cpdMetadata
input_cpdMetadata     <- data.frame(matrix(data=c('a','b','c','d',1,2,3,4), nrow=4, ncol=2, dimnames=list(c(),c('testcol1','testcol2')), byrow=FALSE), stringsAsFactors=FALSE)

# spectraMetadata
input_spectraMetadata <- data.frame(matrix(data=c('e','f','g',5,6,7), nrow=3, ncol=2, dimnames=list(c(),c('testcol1','testcol2')), byrow=FALSE), stringsAsFactors=FALSE)


# Expected peakTables
peakTable1     <- data.frame(matrix(vector(), 4, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
peakTable1[1,] <- c(TRUE, 3309.7589296586070, 3346.8277590361445, 3385.4098874628098, 522.194778, 522.20001220703125, 522.205222, 26133726.6811244078,  26071378, 889280, 901015.80529226747, FALSE, 0.023376160866574614, 1.93975903614455092, 1.0153573486330891, 1.0268238825675249)
peakTable1[2,] <- c(TRUE, 3345.3766648628907, 3386.5288072289159, 3428.2788374983961, 496.20001220703125, 496.20001220703125, 496.20001220703125, 35472141.3330242932, 36498367, 1128960, 1113576.69008227298, FALSE, 0.024601030353423384, 0.95180722891564074, 1.0053782620427065, 1.0093180792278085)
peakTable1[3,] <- c(TRUE, 3451.2075903614455, 3451.5072891566265, 3501.6697504924518, 464.195358, 464.20001220703125, 464.204642, 7498427.1583901159, 7608218, 380736, 389632.13549519412, FALSE, 0.026296922148575364, -2.92771084337346110, 207.6939219686769036, 380.5019028782010082)
peakTable1[4,] <- c(TRUE, 3670.9201232710743, 3704.1427831325304, 3740.0172511251831, 536.20001220703125, 536.20001220703125, 536.20001220703125, 8626279.9788195733, 8692184, 330176, 326763.87246511364, FALSE, 0.022765817240815486, 2.44578313253032320, 1.0305289730128095, 1.0536948855480386)
peakTable1[,c(1,12)]       <- sapply(peakTable1[,c(1,12)], as.logical)
peakTable1[,c(2:11,13:16)] <- sapply(peakTable1[,c(2:11,13:16)], as.numeric)
# 2
peakTable2     <- data.frame(matrix(vector(), 4, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
peakTable2[1,] <- c(TRUE, 3326.1063495851854, 3365.102, 3407.2726475892355, 522.194778, 522.20001220703125, 522.205222, 24545301.622835573, 23367885, 761664, 790802.2209998488, FALSE, 0.023376160866574614, 20.2139999999999, 1.0339153786516375, 1.0630802030537212)
peakTable2[2,] <- c(TRUE, 3365.0238566258713, 3405.791, 3453.4049569205681, 496.195038, 496.20001220703125, 496.204962, 37207579.286265120, 36858762, 1099264, 1098720.2929832144, FALSE, 0.024601030353423384, 20.2139999999999, 1.0839602450900523, 1.1717845972583161)
peakTable2[3,] <- c(TRUE, 3425.9772908380342, 3466.1733975903617, 3508.0320324994614, 464.195358, 464.20001220703125, 464.204642, 11512269.4488430563, 11245667, 366720, 365928.64209905855, FALSE, 0.026296922148575364, 11.738397590361728, 1.0157024674594695, 1.0275996777323413)
peakTable2[4,] <- c(TRUE, 3678.1204484629088, 3720.347, 3768.8011145239534, 536.194638, 536.20001220703125, 536.205362, 7983406.0412310315, 7383963, 220096, 229651.947643013, FALSE, 0.022765817240815486, 18.65, 1.0738577621711971, 1.1514233243540024)
peakTable2[,c(1,12)]       <- sapply(peakTable2[,c(1,12)], as.logical)
peakTable2[,c(2:11,13:16)] <- sapply(peakTable2[,c(2:11,13:16)], as.numeric)
# 3
peakTable3     <- data.frame(matrix(vector(), 4, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
peakTable3[1,] <- c(TRUE, 3333.8625894557053, 3368.233, 3407.4362838927614, 522.194778, 522.20001220703125, 522.205222, 21447174.404490683, 20364558, 758336, 765009.9805796633, FALSE, 0.023376160866574614, 23.345000000000255, 1.0609102044546637, 1.1155310457756928)
peakTable3[2,] <- c(TRUE, 3373.3998828113113, 3413.4952530120481, 3454.4490330927388, 496.195038, 496.20001220703125, 496.204962, 35659353.614476241, 34792490, 1149440, 1145857.7611069249, FALSE, 0.024601030353423384, 27.918253012047899, 1.0081407426394933, 1.0143315197994494)
peakTable3[3,] <- c(FALSE, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, FALSE, NA, NA, NA, NA)
peakTable3[4,] <- c(TRUE, 3672.3110625980275, 3714.088, 3761.43921706666, 536.194638, 536.20001220703125, 536.205362, 6467062.4309558524, 6360742, 196160, 189416.24807174454, FALSE, 0.022765817240815486, 12.391, 1.0656000616594850, 1.1333185588985102)
peakTable3[,c(1,12)]       <- sapply(peakTable3[,c(1,12)], as.logical)
peakTable3[,c(2:11,13:16)] <- sapply(peakTable3[,c(2:11,13:16)], as.numeric)
expected_peakTables <- list(peakTable1, peakTable2, peakTable3)

# Expected peakFit
# 1
cFit1.1         <- list(amplitude=162404.8057918259, center=3341.888, sigma=0.078786133031045896, gamma=0.0018336101984172684, fitStatus=2, curveModel="skewedGaussian")
class(cFit1.1)  <- 'peakPantheR_curveFit'
cFit1.2         <- list(amplitude=199249.10572753669, center=3382.577, sigma=0.074904415304607966, gamma=0.0011471899372353885, fitStatus=2, curveModel="skewedGaussian")
class(cFit1.2)  <- 'peakPantheR_curveFit'
cFit1.3         <- list(amplitude=31645.961277502651, center=3451.435, sigma=0.064803553287811053, gamma=2.8557893789555022, fitStatus=2, curveModel="skewedGaussian")
class(cFit1.3)  <- 'peakPantheR_curveFit'
cFit1.4         <- list(amplitude=59193.591103772116, center=3698.697, sigma=0.082789238806238355, gamma=0.0026044299691057823, fitStatus=2, curveModel="skewedGaussian")
class(cFit1.4)  <- 'peakPantheR_curveFit'
# 2
cFit2.1         <- list(amplitude=124090.83425474487, center=3359.102, sigma=0.071061541060964212, gamma=0.0018336072657203239, fitStatus=2, curveModel="skewedGaussian")
class(cFit2.1)  <- 'peakPantheR_curveFit'
cFit2.2         <- list(amplitude=151407.23415130575, center=3399.791, sigma=0.063753866057052563, gamma=0.001676782834598999, fitStatus=2, curveModel="skewedGaussian")
class(cFit2.2)  <- 'peakPantheR_curveFit'
cFit2.3         <- list(amplitude=60803.292514896151, center=3460.824, sigma=0.072633009809671245, gamma=0.0015588615168115277, fitStatus=2, curveModel="skewedGaussian")
class(cFit2.3)  <- 'peakPantheR_curveFit'
cFit2.4         <- list(amplitude=31542.623023583215, center=3714.347, sigma=0.062820857630350654, gamma=0.0015366513073237947, fitStatus=2, curveModel="skewedGaussian")
class(cFit2.4)  <- 'peakPantheR_curveFit'
# 3
cFit3.1         <- list(amplitude=122363.51256736703, center=3362.233, sigma=0.075489598945304492, gamma=0.0025160536725299734, fitStatus=2, curveModel="skewedGaussian")
class(cFit3.1)  <- 'peakPantheR_curveFit'
cFit3.2         <- list(amplitude=204749.86097918145, center=3409.182, sigma=0.075731781812843249, gamma=0.0013318670577834328, fitStatus=2, curveModel="skewedGaussian")
class(cFit3.2)  <- 'peakPantheR_curveFit'
cFit3.3         <- NA
cFit3.4         <- list(amplitude=26628.505498512375, center=3708.088, sigma=0.064131129861254479, gamma=0.0015719426982490699, fitStatus=2, curveModel="skewedGaussian")
class(cFit3.4)  <- 'peakPantheR_curveFit'
expected_peakFit <- list(list(cFit1.1, cFit1.2, cFit1.3, cFit1.4), list(cFit2.1, cFit2.2, cFit2.3, cFit2.4), list(cFit3.1, cFit3.2, cFit3.3, cFit3.4))

# Expected dataPoints
tmp_raw_data1  	  <- MSnbase::readMSData(input_spectraPaths[1], centroided=TRUE, mode='onDisk')
ROIDataPoints1    <- extractSignalRawData(tmp_raw_data1, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
tmp_raw_data2  	  <- MSnbase::readMSData(input_spectraPaths[2], centroided=TRUE, mode='onDisk')
ROIDataPoints2    <- extractSignalRawData(tmp_raw_data2, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
tmp_raw_data3  	  <- MSnbase::readMSData(input_spectraPaths[3], centroided=TRUE, mode='onDisk')
ROIDataPoints3    <- extractSignalRawData(tmp_raw_data3, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
expected_dataPoints <- list(ROIDataPoints1, ROIDataPoints2, ROIDataPoints3)


# if ((.Platform$OS.type != "windows") || (.Machine$sizeof.pointer == 8)) {
# test_that('3 files, 4 compounds, no uROI, no FIR, no getAcquTime, no verbose', {
#   # Object fully initialised
#   initAnnotation      <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata)

#   # Expected annotation
#   expected_annotation             <- initAnnotation
#   expected_annotation@TIC         <- c(2410533091, 2524040155, 2332817115)
#   expected_annotation@peakTables  <- expected_peakTables
#   expected_annotation@peakFit     <- expected_peakFit
#   expected_annotation@dataPoints  <- expected_dataPoints
#   expected_annotation@isAnnotated <- TRUE
#   # Expected failures
#   tmp_status          <- NA
#   names(tmp_status)   <- 'test'
#   tmp_failures        <- !is.na(tmp_status)
#   names(tmp_failures) <- NULL
#   expected_failures   <- data.frame(matrix(c(names(tmp_status)[tmp_failures], tmp_status[tmp_failures]), ncol=2, byrow=FALSE, dimnames=list(c(), c('file', 'error'))), stringsAsFactors=FALSE)
#   # Expected message
#   expected_message    <- c("Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n")

#   # results (output, warnings and messages)
#   result_parallelAnnotation <- evaluate_promise(peakPantheR_parallelAnnotation(initAnnotation, ncores=0, getAcquTime=FALSE, verbose=FALSE))

#   # Check results
#   expect_equal(result_parallelAnnotation$result$annotation, expected_annotation, tolerance=1e-5)
#   expect_equal(result_parallelAnnotation$result$failures, expected_failures)

#   # Check messages (centwave output)
#   expect_equal(length(result_parallelAnnotation$messages), 3)
#   expect_equal(result_parallelAnnotation$messages, expected_message)
#   }) }

# if ((.Platform$OS.type != "windows") || (.Machine$sizeof.pointer == 8)) {
# test_that('3 files (1 missing), 4 compounds, no uROI, no FIR, no getAcquTime, no verbose', {
#   # Object fully initialised
#   initAnnotation      <- peakPantheRAnnotation(spectraPaths=input_missingSpectraPaths, targetFeatTable=input_targetFeatTable, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata)

#   # Expected annotation
#   expected_annotation             <- initAnnotation[c(1,3),]
#   expected_annotation@TIC         <- c(2410533091, 2332817115)
#   expected_annotation@peakTables  <- expected_peakTables[c(1,3)]
#   expected_annotation@peakFit     <- expected_peakFit[c(1,3)]
#   expected_annotation@dataPoints  <- expected_dataPoints[c(1,3)]
#   expected_annotation@isAnnotated <- TRUE
#   # Expected failures
#   tmp_status          <- 'Error file does not exist: aaa/bbb.cdf'
#   names(tmp_status)   <- 'aaa/bbb.cdf'
#   tmp_failures        <- !is.na(tmp_status)
#   names(tmp_failures) <- NULL
#   expected_failures   <- data.frame(matrix(c(names(tmp_status)[tmp_failures], tmp_status[tmp_failures]), ncol=2, byrow=FALSE, dimnames=list(c(), c('file', 'error'))), stringsAsFactors=FALSE)
#   # Expected message
#   expected_message    <- c("Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n")

#   # results (output, warnings and messages)
#   result_parallelAnnotation <- evaluate_promise(peakPantheR_parallelAnnotation(initAnnotation, ncores=0, getAcquTime=FALSE, verbose=FALSE))

#   # Check results
#   expect_equal(result_parallelAnnotation$result$annotation, expected_annotation, tolerance=1e-6)
#   expect_equal(result_parallelAnnotation$result$failures, expected_failures)

#   # Check messages (centwave output)
#   expect_equal(length(result_parallelAnnotation$messages), 2)
#   expect_equal(result_parallelAnnotation$messages, expected_message)
#   }) }

test_that('3 files, 4 compounds, no uROI, no FIR, no getAcquTime, no verbose, modify parameter with ... (cpd #3)', {
  # Cpd 3 is now found in 3rd file
  # Object fully initialised
  initAnnotation      <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata)

  # Modify fit of window #3
  tmp_params              <- list(init_params  = list(amplitude=1E5, center=3455., sigma=0.1, gamma=0),
                                  lower_bounds = list(amplitude=0,   center=3450., sigma=0,   gamma=-0.1),
                                  upper_bounds = list(amplitude=1E9, center=3460., sigma=5,   gamma=0.1))
  new_params              <- list('guess', 'guess', tmp_params, 'guess')
  
  # Expected annotation
  expected_annotation             <- initAnnotation
  expected_annotation@TIC         <- c(2410533091, 2524040155, 2332817115)
  # modify peakTable
  tmp_peakTable1                  <- expected_peakTables[[1]]
  tmp_peakTable1[3,c(2:11,13:16)] <- c(3418.0076795585401, 3455.6277710843374, 3495.4734240188186, 464.195358, 464.2000122, 464.204642, 11307215.264967661, 11538988, 380736, 381327.26552768378,  0.026296922148575364, 1.1927710843374371, 1.0223856861398346, 1.0391667928715738)
  tmp_peakTable2                  <- expected_peakTables[[2]]
  tmp_peakTable2[3,c(2:11,13:16)] <- c(3433.0520428402692, 3464.1974939759039, 3518.3673814020449, 464.195358, 464.2000122, 464.204642, 11632481.39997852, 11167979, 366720, 370275.43095360085,  0.026296922148575364, 9.762493975903908, 1.3108566292516153, 1.566712445298531)
  tmp_peakTable3                  <- expected_peakTables[[3]]
  tmp_peakTable3[3,c(2:11,13:16)] <- c(3428.5236305028589, 3459.5514216867468, 3510.3107983995801, 464.195358, 464.2000122, 464.204642, 9697604.1795769241, 9501971, 319488, 321005.94361925457,  0.026296922148575364, 5.1164216867468895, 1.2652077714541707, 1.4814479150398188)
  tmp_peakTable3$found[3]         <- TRUE
  expected_annotation@peakTables  <- list(tmp_peakTable1, tmp_peakTable2, tmp_peakTable3)
  # modify curveFit
  tmp_curveFit1.3         <- list(amplitude=64246.052173667762, center=3450, sigma=0.07533469863886906, gamma=0.0019238229766131536, fitStatus=2, curveModel="skewedGaussian")
  class(tmp_curveFit1.3)  <- 'peakPantheR_curveFit'
  tmp_curveFit2.3         <- list(amplitude=31499.356902732343, center=3454.217764223421, sigma=0.052005180846722628, gamma=0.0030181085286239786, fitStatus=1, curveModel="skewedGaussian")
  class(tmp_curveFit2.3)  <- 'peakPantheR_curveFit'
  tmp_curveFit3.3         <- list(amplitude=29842.598057228814, center=3450, sigma=0.055439446898995937, gamma=0.0031338189157350213, fitStatus=2, curveModel="skewedGaussian")
  class(tmp_curveFit3.3)  <- 'peakPantheR_curveFit'
  expected_annotation@peakFit           <- expected_peakFit
  expected_annotation@peakFit[[1]][[3]] <- tmp_curveFit1.3
  expected_annotation@peakFit[[2]][[3]] <- tmp_curveFit2.3
  expected_annotation@peakFit[[3]][[3]] <- tmp_curveFit3.3
  expected_annotation@dataPoints  <- expected_dataPoints
  expected_annotation@isAnnotated <- TRUE
  # Expected failures
  tmp_status          <- NA
  names(tmp_status)   <- 'test'
  tmp_failures        <- !is.na(tmp_status)
  names(tmp_failures) <- NULL
  expected_failures   <- data.frame(matrix(c(names(tmp_status)[tmp_failures], tmp_status[tmp_failures]), ncol=2, byrow=FALSE, dimnames=list(c(), c('file', 'error'))), stringsAsFactors=FALSE)
  # Expected message
  expected_message    <- c("Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n")

  # results (output, warnings and messages)
  result_parallelAnnotation <- evaluate_promise(peakPantheR_parallelAnnotation(initAnnotation, ncores=0, getAcquTime=FALSE, verbose=FALSE, params=new_params))

  # Check results
  expect_equal(result_parallelAnnotation$result$annotation, expected_annotation, tolerance=1e-5)
  expect_equal(result_parallelAnnotation$result$failures, expected_failures)

  # Check messages (centwave output)
  expect_equal(length(result_parallelAnnotation$messages), 3)
  expect_equal(result_parallelAnnotation$messages, expected_message)
})

test_that('3 files, 4 compounds, no uROI, no FIR, no getAcquTime, no verbose, peaks not found and not replaced (cpd #3)', {
  # Cpd #3 will not give results
  noMatch_ROI3        <- input_targetFeatTable
  noMatch_ROI3[3,6:8] <- c(52.194778, 52.2, 52.205222)
  
  # Object fully initialised
  initAnnotation      <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=noMatch_ROI3, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata)
  
  # Expected annotation
  expected_annotation             <- initAnnotation
  expected_annotation@TIC         <- c(2410533091, 2524040155, 2332817115)
  tmp_peakTables                        <- expected_peakTables
  tmp_peakTables[[1]][3,c(2:11,13:16)]  <- as.numeric(NA)
  tmp_peakTables[[1]]$found[3]          <- FALSE
  tmp_peakTables[[2]][3,c(2:11,13:16)]  <- as.numeric(NA)
  tmp_peakTables[[2]]$found[3]          <- FALSE
  tmp_peakTables[[3]][3,c(2:11,13:16)]  <- as.numeric(NA)
  tmp_peakTables[[3]]$found[3]          <- FALSE
  expected_annotation@peakTables        <- tmp_peakTables
  tmp_peakFit                     <- expected_peakFit
  tmp_peakFit[[1]][[3]]           <- NA
  tmp_peakFit[[2]][[3]]           <- NA
  tmp_peakFit[[3]][[3]]           <- NA
  expected_annotation@peakFit     <- tmp_peakFit
  tmp_dataPoints                  <- expected_dataPoints
  tmp_dataPoints[[1]][[3]]        <- data.frame(rt=numeric(), mz=numeric(), int=numeric())
  tmp_dataPoints[[2]][[3]]        <- data.frame(rt=numeric(), mz=numeric(), int=numeric())
  tmp_dataPoints[[3]][[3]]        <- data.frame(rt=numeric(), mz=numeric(), int=numeric())
  expected_annotation@dataPoints  <- tmp_dataPoints
  expected_annotation@isAnnotated <- TRUE
  # Expected failures
  tmp_status          <- NA
  names(tmp_status)   <- 'test'
  tmp_failures        <- !is.na(tmp_status)
  names(tmp_failures) <- NULL
  expected_failures   <- data.frame(matrix(c(names(tmp_status)[tmp_failures], tmp_status[tmp_failures]), ncol=2, byrow=FALSE, dimnames=list(c(), c('file', 'error'))), stringsAsFactors=FALSE)
  # Expected message
  expected_message    <- c("Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n")
  
  # results (output, warnings and messages)
  result_parallelAnnotation <- evaluate_promise(peakPantheR_parallelAnnotation(initAnnotation, ncores=0, getAcquTime=FALSE, verbose=FALSE))
  
  # Check results
  expect_equal(result_parallelAnnotation$result$annotation, expected_annotation, tolerance=1e-5)
  expect_equal(result_parallelAnnotation$result$failures, expected_failures)
  
  # Check messages (centwave output)
  expect_equal(length(result_parallelAnnotation$messages), 3)
  expect_equal(result_parallelAnnotation$messages, expected_message)
})

test_that('3 files, 4 compounds, no uROI, FIR replace peaks not found (cpd #3), no getAcquTime, no verbose', {
  # Cpd #3 will not give results, FIR will replace
  noMatch_ROI3        <- input_targetFeatTable
  noMatch_ROI3[3,6:8] <- c(52.194778, 52.2, 52.205222)
  
  # Object fully initialised
  initAnnotation      <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=noMatch_ROI3, useFIR=TRUE, FIR=input_FIR, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata)
  
  # Expected annotation
  expected_annotation             <- initAnnotation
  expected_annotation@TIC         <- c(2410533091, 2524040155, 2332817115)
  tmp_peakTables                        <- expected_peakTables
  tmp_peakTables[[1]][3,c(2:11,13:16)]  <- c(3444.524, 3454.435, 3478.431, 464.1995, 464.20001220703125, 464.2005, 8939889, 8939889, 380736, as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  tmp_peakTables[[1]]$is_filled[3]      <- TRUE
  tmp_peakTables[[2]][3,c(2:11,13:16)]  <- c(3444.524, 3463.824, 3478.431, 464.1995, 464.20001220703125, 464.2005, 8863585, 8863585, 366720, as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  tmp_peakTables[[2]]$is_filled[3]      <- TRUE
  tmp_peakTables[[3]][3,c(2:11,13:16)]  <- c(3444.524, 3460.696, 3478.431, 464.1995, 464.20001220703125, 464.2005, 7414831, 7414831, 319488,  as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  tmp_peakTables[[3]]$found[3]      <- TRUE
  tmp_peakTables[[3]]$is_filled[3]      <- TRUE
  expected_annotation@peakTables        <- tmp_peakTables
  tmp_peakFit                     <- expected_peakFit
  tmp_peakFit[[1]][[3]]           <- NA
  tmp_peakFit[[2]][[3]]           <- NA
  tmp_peakFit[[3]][[3]]           <- NA
  expected_annotation@peakFit     <- tmp_peakFit
  tmp_dataPoints                  <- expected_dataPoints
  tmp_dataPoints[[1]][[3]]        <- data.frame(rt=numeric(), mz=numeric(), int=numeric())
  tmp_dataPoints[[2]][[3]]        <- data.frame(rt=numeric(), mz=numeric(), int=numeric())
  tmp_dataPoints[[3]][[3]]        <- data.frame(rt=numeric(), mz=numeric(), int=numeric())
  expected_annotation@dataPoints  <- tmp_dataPoints
  expected_annotation@isAnnotated <- TRUE
  # Expected failures
  tmp_status          <- NA
  names(tmp_status)   <- 'test'
  tmp_failures        <- !is.na(tmp_status)
  names(tmp_failures) <- NULL
  expected_failures   <- data.frame(matrix(c(names(tmp_status)[tmp_failures], tmp_status[tmp_failures]), ncol=2, byrow=FALSE, dimnames=list(c(), c('file', 'error'))), stringsAsFactors=FALSE)
  # Expected message
  expected_message    <- c("Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n")
  
  # results (output, warnings and messages)
  result_parallelAnnotation <- evaluate_promise(peakPantheR_parallelAnnotation(initAnnotation, ncores=0, getAcquTime=FALSE, verbose=FALSE))
  
  # Check results
  expect_equal(result_parallelAnnotation$result$annotation, expected_annotation, tolerance=1e-5)
  expect_equal(result_parallelAnnotation$result$failures, expected_failures)
  
  # Check messages (centwave output)
  expect_equal(length(result_parallelAnnotation$messages), 3)
  expect_equal(result_parallelAnnotation$messages, expected_message)
})

# if ((.Platform$OS.type != "windows") || (.Machine$sizeof.pointer == 8)) {
# test_that('3 files, 4 compounds, uROI, no FIR, no fitGauss, no getAcquTime, no verbose', {
#   # Object fully initialised
#   initAnnotation      <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_badtargetFeatTable, uROIExist=TRUE, useUROI=TRUE, uROI=input_uROI, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata)

#   # Expected annotation
#   expected_annotation             <- initAnnotation
#   expected_annotation@TIC         <- c(2410533091, 2524040155, 2332817115)
#   expected_annotation@peakTables  <- expected_peakTables
#   expected_annotation@peakFit     <- expected_peakFit
#   expected_annotation@dataPoints  <- expected_dataPoints
#   expected_annotation@isAnnotated <- TRUE
#   # Expected failures
#   tmp_status          <- NA
#   names(tmp_status)   <- 'test'
#   tmp_failures        <- !is.na(tmp_status)
#   names(tmp_failures) <- NULL
#   expected_failures   <- data.frame(matrix(c(names(tmp_status)[tmp_failures], tmp_status[tmp_failures]), ncol=2, byrow=FALSE, dimnames=list(c(), c('file', 'error'))), stringsAsFactors=FALSE)
#   # Expected message
#   expected_message    <- c("Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n")

#   # results (output, warnings and messages)
#   result_parallelAnnotation <- evaluate_promise(peakPantheR_parallelAnnotation(initAnnotation, ncores=0, getAcquTime=FALSE, verbose=FALSE))

#   # Check results
#   expect_equal(result_parallelAnnotation$result$annotation, expected_annotation, tolerance=1e-5)
#   expect_equal(result_parallelAnnotation$result$failures, expected_failures)

#   # Check messages (centwave output)
#   expect_equal(length(result_parallelAnnotation$messages), 3)
#   expect_equal(result_parallelAnnotation$messages, expected_message)
#   }) }

test_that('serial: 3 files, (1 missing), 4 compounds, uROI, FIR replace peaks not found (cpd #3), getAcquTime, verbose', {
  # sample 2 is missing
  # Cpd #3 will not give results
  noMatch_uROI3        <- input_uROI
  noMatch_uROI3[3,4:6] <- c(52.194778, 52.2, 52.205222)
  
  # Object fully initialised
  initAnnotation      <- peakPantheRAnnotation(spectraPaths=input_missingSpectraPaths, targetFeatTable=input_badtargetFeatTable, uROIExist=TRUE, useUROI=TRUE, uROI=noMatch_uROI3, useFIR=TRUE, FIR=input_FIR, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata)

  # Expected annotation
  expected_annotation             <- initAnnotation[c(1,3),]
  expected_annotation@TIC         <- c(2410533091, 2332817115)
  tmp_peakTables                        <- expected_peakTables[c(1,3)]
  tmp_peakTables[[1]][3,c(2:11,13:16)]  <- c(3444.524, 3454.435, 3478.431, 464.1995, 464.20001220703125, 464.2005, 8939889, 8939889, 380736, as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  tmp_peakTables[[1]]$is_filled[3]      <- TRUE
  tmp_peakTables[[2]][3,c(2:11,13:16)]  <- c(3444.524, 3460.696, 3478.431, 464.1995, 464.20001220703125, 464.2005, 7414831, 7414831, 319488, as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  tmp_peakTables[[2]]$found[3]          <- TRUE
  tmp_peakTables[[2]]$is_filled[3]      <- TRUE
  expected_annotation@peakTables        <- tmp_peakTables
  tmp_peakFit                     <- expected_peakFit[c(1,3)]
  tmp_peakFit[[1]][[3]]           <- NA
  tmp_peakFit[[2]][[3]]           <- NA
  expected_annotation@peakFit     <- tmp_peakFit
  tmp_dataPoints                  <- expected_dataPoints[c(1,3)]
  tmp_dataPoints[[1]][[3]]        <- data.frame(rt=numeric(), mz=numeric(), int=numeric())
  tmp_dataPoints[[2]][[3]]        <- data.frame(rt=numeric(), mz=numeric(), int=numeric())
  expected_annotation@dataPoints  <- tmp_dataPoints
  expected_annotation@isAnnotated <- TRUE
  # Expected failures
  tmp_status          <- 'Error file does not exist: aaa/bbb.cdf'
  names(tmp_status)   <- 'aaa/bbb.cdf'
  tmp_failures        <- !is.na(tmp_status)
  names(tmp_failures) <- NULL
  expected_failures   <- data.frame(matrix(c(names(tmp_status)[tmp_failures], tmp_status[tmp_failures]), ncol=2, byrow=FALSE, dimnames=list(c(), c('file', 'error'))), stringsAsFactors=FALSE)
  # Expected message
  expected_message    <- c("Processing 4 compounds in 3 samples:\n", "  uROI:\tTRUE\n", "  FIR:\tTRUE\n", "----- ko15 -----\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Check input, mzMLPath must be a .mzML\n", "Reading data from 4 windows\n", "Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #1\n", "Fit of ROI #3 is unsuccessful (try err)\n", "1 features to integrate with FIR\n", "Reading data from 1 windows\n", "Error file does not exist: aaa/bbb.cdf\n", "----- ko18 -----\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Check input, mzMLPath must be a .mzML\n", "Reading data from 4 windows\n", "Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #1\n", "Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #2\n", "Fit of ROI #3 is unsuccessful (try err)\n", "Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #4\n", "1 features to integrate with FIR\n", "Reading data from 1 windows\n", "----------------\n", "1 file(s) failed to process:\n         file                                  error\n1 aaa/bbb.cdf Error file does not exist: aaa/bbb.cdf\n", "Annotation object cannot be reordered by sample acquisition date\n", "----------------\n", "  1 failure(s)\n")

  # results (output, warnings and messages)
  result_parallelAnnotation <- evaluate_promise(peakPantheR_parallelAnnotation(initAnnotation, ncores=0, getAcquTime=TRUE, verbose=TRUE))

  # Check results
  expect_equal(result_parallelAnnotation$result$annotation, expected_annotation, tolerance=1e-6)
  expect_equal(result_parallelAnnotation$result$failures, expected_failures)

  # Check messages (no timing)
  expect_equal(length(result_parallelAnnotation$messages), 40)
  expect_equal(result_parallelAnnotation$messages[c(1:7,9,10,13,14,18:22,24:27,30,31,35,36,37,38,40)], expected_message)
})

test_that('parallel (with cluster reset): 3 files, (1 missing), 4 compounds, uROI, FIR replace peaks not found (cpd #3), getAcquTime, verbose', {
  # sample 2 is missing
  # Cpd #3 will not give results
  noMatch_uROI3        <- input_uROI
  noMatch_uROI3[3,4:6] <- c(52.194778, 52.2, 52.205222)
  
  # Object fully initialised
  initAnnotation      <- peakPantheRAnnotation(spectraPaths=input_missingSpectraPaths, targetFeatTable=input_badtargetFeatTable, uROIExist=TRUE, useUROI=TRUE, uROI=noMatch_uROI3, useFIR=TRUE, FIR=input_FIR, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata)
  
  # Expected annotation
  expected_annotation             <- initAnnotation[c(1,3),]
  expected_annotation@TIC         <- c(2410533091, 2332817115)
  tmp_peakTables                        <- expected_peakTables[c(1,3)]
  tmp_peakTables[[1]][3,c(2:11,13:16)]  <- c(3444.524, 3454.435, 3478.431, 464.1995, 464.20001220703125, 464.2005, 8939889, 8939889, 380736, as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  tmp_peakTables[[1]]$is_filled[3]      <- TRUE
  tmp_peakTables[[2]][3,c(2:11,13:16)]  <- c(3444.524, 3460.696, 3478.431, 464.1995, 464.20001220703125, 464.2005, 7414831, 7414831, 319488, as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  tmp_peakTables[[2]]$found[3]          <- TRUE
  tmp_peakTables[[2]]$is_filled[3]      <- TRUE
  expected_annotation@peakTables        <- tmp_peakTables
  tmp_peakFit                     <- expected_peakFit[c(1,3)]
  tmp_peakFit[[1]][[3]]           <- NA
  tmp_peakFit[[2]][[3]]           <- NA
  expected_annotation@peakFit     <- tmp_peakFit
  tmp_dataPoints                  <- expected_dataPoints[c(1,3)]
  tmp_dataPoints[[1]][[3]]        <- data.frame(rt=numeric(), mz=numeric(), int=numeric())
  tmp_dataPoints[[2]][[3]]        <- data.frame(rt=numeric(), mz=numeric(), int=numeric())
  expected_annotation@dataPoints  <- tmp_dataPoints
  expected_annotation@isAnnotated <- TRUE
  # Expected failures
  tmp_status          <- 'Error file does not exist: aaa/bbb.cdf'
  names(tmp_status)   <- 'aaa/bbb.cdf'
  tmp_failures        <- !is.na(tmp_status)
  names(tmp_failures) <- NULL
  expected_failures   <- data.frame(matrix(c(names(tmp_status)[tmp_failures], tmp_status[tmp_failures]), ncol=2, byrow=FALSE, dimnames=list(c(), c('file', 'error'))), stringsAsFactors=FALSE)
  # Expected message
  expected_message    <- c("Processing 4 compounds in 3 samples:\n", "  uROI:\tTRUE\n", "  FIR:\tTRUE\n", "Running 3 clusters of 1 files over 1 cores:\n", "  starting cluster 1/3\n", "  starting cluster 2/3\n", "  starting cluster 3/3\n", "----------------\n", "1 file(s) failed to process:\n         file                                  error\n1 aaa/bbb.cdf Error file does not exist: aaa/bbb.cdf\n", "Annotation object cannot be reordered by sample acquisition date\n", "----------------\n", "  1 failure(s)\n")
  
  # results (output, warnings and messages)
  result_parallelAnnotation <- evaluate_promise(peakPantheR_parallelAnnotation(initAnnotation, ncores=1, getAcquTime=TRUE, resetWorkers=1, verbose=TRUE))
  
  # Check results
  expect_equal(result_parallelAnnotation$result$annotation, expected_annotation, tolerance=1e-6)
  expect_equal(result_parallelAnnotation$result$failures, expected_failures)
  
  # Check messages (no timing)
  expect_equal(length(result_parallelAnnotation$messages), 13)
  expect_equal(result_parallelAnnotation$messages[c(1:11,13)], expected_message)
})

test_that('parallel (without cluster reset): 3 files, (1 missing), 4 compounds, uROI, FIR replace peaks not found (cpd #3), getAcquTime, verbose', {
  # sample 2 is missing
  # Cpd #3 will not give results
  noMatch_uROI3        <- input_uROI
  noMatch_uROI3[3,4:6] <- c(52.194778, 52.2, 52.205222)
  
  # Object fully initialised
  initAnnotation      <- peakPantheRAnnotation(spectraPaths=input_missingSpectraPaths, targetFeatTable=input_badtargetFeatTable, uROIExist=TRUE, useUROI=TRUE, uROI=noMatch_uROI3, useFIR=TRUE, FIR=input_FIR, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata)
  
  # Expected annotation
  expected_annotation             <- initAnnotation[c(1,3),]
  expected_annotation@TIC         <- c(2410533091, 2332817115)
  tmp_peakTables                        <- expected_peakTables[c(1,3)]
  tmp_peakTables[[1]][3,c(2:11,13:16)]  <- c(3444.524, 3454.435, 3478.431, 464.1995, 464.20001220703125, 464.2005, 8939889, 8939889, 380736, as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  tmp_peakTables[[1]]$is_filled[3]      <- TRUE
  tmp_peakTables[[2]][3,c(2:11,13:16)]  <- c(3444.524, 3460.696, 3478.431, 464.1995, 464.20001220703125, 464.2005, 7414831, 7414831, 319488, as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  tmp_peakTables[[2]]$found[3]          <- TRUE
  tmp_peakTables[[2]]$is_filled[3]      <- TRUE
  expected_annotation@peakTables        <- tmp_peakTables
  tmp_peakFit                     <- expected_peakFit[c(1,3)]
  tmp_peakFit[[1]][[3]]           <- NA
  tmp_peakFit[[2]][[3]]           <- NA
  expected_annotation@peakFit     <- tmp_peakFit
  tmp_dataPoints                  <- expected_dataPoints[c(1,3)]
  tmp_dataPoints[[1]][[3]]        <- data.frame(rt=numeric(), mz=numeric(), int=numeric())
  tmp_dataPoints[[2]][[3]]        <- data.frame(rt=numeric(), mz=numeric(), int=numeric())
  expected_annotation@dataPoints  <- tmp_dataPoints
  expected_annotation@isAnnotated <- TRUE
  # Expected failures
  tmp_status          <- 'Error file does not exist: aaa/bbb.cdf'
  names(tmp_status)   <- 'aaa/bbb.cdf'
  tmp_failures        <- !is.na(tmp_status)
  names(tmp_failures) <- NULL
  expected_failures   <- data.frame(matrix(c(names(tmp_status)[tmp_failures], tmp_status[tmp_failures]), ncol=2, byrow=FALSE, dimnames=list(c(), c('file', 'error'))), stringsAsFactors=FALSE)
  # Expected message
  expected_message    <- c("Processing 4 compounds in 3 samples:\n", "  uROI:\tTRUE\n", "  FIR:\tTRUE\n",  "----------------\n", "1 file(s) failed to process:\n         file                                  error\n1 aaa/bbb.cdf Error file does not exist: aaa/bbb.cdf\n", "Annotation object cannot be reordered by sample acquisition date\n", "----------------\n", "  1 failure(s)\n")
  
  # results (output, warnings and messages)
  result_parallelAnnotation <- evaluate_promise(peakPantheR_parallelAnnotation(initAnnotation, ncores=1, getAcquTime=TRUE, resetWorkers=0, verbose=TRUE))
  
  # Check results
  expect_equal(result_parallelAnnotation$result$annotation, expected_annotation, tolerance=1e-6)
  expect_equal(result_parallelAnnotation$result$failures, expected_failures)
  
  # Check messages (no timing)
  expect_equal(length(result_parallelAnnotation$messages), 9)
  expect_equal(result_parallelAnnotation$messages[c(1:7,9)], expected_message)
})

test_that('serial and parallel (with cluster reset) give the same result: 3 files, (1 missing), 4 compounds, uROI, FIR replace peaks not found (cpd #3), getAcquTime, verbose', {
  # sample 2 is missing
  # Cpd #3 will not give results
  noMatch_uROI3        <- input_uROI
  noMatch_uROI3[3,4:6] <- c(52.194778, 52.2, 52.205222)
  
  # Object fully initialised
  initAnnotation  <- peakPantheRAnnotation(spectraPaths=input_missingSpectraPaths, targetFeatTable=input_badtargetFeatTable, uROIExist=TRUE, useUROI=TRUE, uROI=noMatch_uROI3, useFIR=TRUE, FIR=input_FIR, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata)

  # results
  result_serial   <- evaluate_promise(peakPantheR_parallelAnnotation(initAnnotation, ncores=0, getAcquTime=TRUE, verbose=TRUE))
  result_parallel <- evaluate_promise(peakPantheR_parallelAnnotation(initAnnotation, ncores=1, getAcquTime=TRUE, resetWorkers=1, verbose=TRUE))

  # Check results
  expect_equal(result_serial$result, result_parallel$result, tolerance=1e-6)
})

test_that('serial and parallel (without cluster reset) give the same result: 3 files, (1 missing), 4 compounds, uROI, FIR replace peaks not found (cpd #3), getAcquTime, verbose', {
  # sample 2 is missing
  # Cpd #3 will not give results
  noMatch_uROI3        <- input_uROI
  noMatch_uROI3[3,4:6] <- c(52.194778, 52.2, 52.205222)
  
  # Object fully initialised
  initAnnotation  <- peakPantheRAnnotation(spectraPaths=input_missingSpectraPaths, targetFeatTable=input_badtargetFeatTable, uROIExist=TRUE, useUROI=TRUE, uROI=noMatch_uROI3, useFIR=TRUE, FIR=input_FIR, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata)
  
  # results
  result_serial   <- evaluate_promise(peakPantheR_parallelAnnotation(initAnnotation, ncores=0, getAcquTime=TRUE, verbose=TRUE))
  result_parallel <- evaluate_promise(peakPantheR_parallelAnnotation(initAnnotation, ncores=1, getAcquTime=TRUE, resetWorkers=0, verbose=TRUE))
  
  # Check results
  expect_equal(result_serial$result, result_parallel$result, tolerance=1e-6)
})

test_that('change to resetWorkers alters the number of parallel cluster reset', {
  # sample 2 is missing
  # Cpd #3 will not give results
  noMatch_uROI3        <- input_uROI
  noMatch_uROI3[3,4:6] <- c(52.194778, 52.2, 52.205222)
  
  # Object fully initialised
  initAnnotation      <- peakPantheRAnnotation(spectraPaths=input_missingSpectraPaths, targetFeatTable=input_badtargetFeatTable, uROIExist=TRUE, useUROI=TRUE, uROI=noMatch_uROI3, useFIR=TRUE, FIR=input_FIR, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata)
  
  # Expected annotation
  expected_annotation             <- initAnnotation[c(1,3),]
  expected_annotation@TIC         <- c(2410533091, 2332817115)
  tmp_peakTables                        <- expected_peakTables[c(1,3)]
  tmp_peakTables[[1]][3,c(2:11,13:16)]  <- c(3444.524, 3454.435, 3478.431, 464.1995, 464.20001220703125, 464.2005, 8939889, 8939889, 380736, as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  tmp_peakTables[[1]]$is_filled[3]      <- TRUE
  tmp_peakTables[[2]][3,c(2:11,13:16)]  <- c(3444.524, 3460.696, 3478.431, 464.1995, 464.20001220703125, 464.2005, 7414831, 7414831, 319488, as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  tmp_peakTables[[2]]$found[3]          <- TRUE
  tmp_peakTables[[2]]$is_filled[3]      <- TRUE
  expected_annotation@peakTables        <- tmp_peakTables
  tmp_peakFit                     <- expected_peakFit[c(1,3)]
  tmp_peakFit[[1]][[3]]           <- NA
  tmp_peakFit[[2]][[3]]           <- NA
  expected_annotation@peakFit     <- tmp_peakFit
  tmp_dataPoints                  <- expected_dataPoints[c(1,3)]
  tmp_dataPoints[[1]][[3]]        <- data.frame(rt=numeric(), mz=numeric(), int=numeric())
  tmp_dataPoints[[2]][[3]]        <- data.frame(rt=numeric(), mz=numeric(), int=numeric())
  expected_annotation@dataPoints  <- tmp_dataPoints
  expected_annotation@isAnnotated <- TRUE
  # Expected failures
  tmp_status          <- 'Error file does not exist: aaa/bbb.cdf'
  names(tmp_status)   <- 'aaa/bbb.cdf'
  tmp_failures        <- !is.na(tmp_status)
  names(tmp_failures) <- NULL
  expected_failures   <- data.frame(matrix(c(names(tmp_status)[tmp_failures], tmp_status[tmp_failures]), ncol=2, byrow=FALSE, dimnames=list(c(), c('file', 'error'))), stringsAsFactors=FALSE)
  # Expected message
  expected_message    <- c("Processing 4 compounds in 3 samples:\n", "  uROI:\tTRUE\n", "  FIR:\tTRUE\n", "Running 2 clusters of 2 files over 1 cores:\n", "  starting cluster 1/2\n", "  starting cluster 2/2\n", "----------------\n", "1 file(s) failed to process:\n         file                                  error\n1 aaa/bbb.cdf Error file does not exist: aaa/bbb.cdf\n", "Annotation object cannot be reordered by sample acquisition date\n", "----------------\n", "  1 failure(s)\n")
  
  # results (output, warnings and messages)
  result_parallelAnnotation <- evaluate_promise(peakPantheR_parallelAnnotation(initAnnotation, ncores=1, getAcquTime=TRUE, resetWorkers=2, verbose=TRUE))
  
  # Check results
  expect_equal(result_parallelAnnotation$result$annotation, expected_annotation, tolerance=1e-6)
  expect_equal(result_parallelAnnotation$result$failures, expected_failures)
  
  # Check messages (no timing)
  expect_equal(length(result_parallelAnnotation$messages), 12)
  expect_equal(result_parallelAnnotation$messages[c(1:10,12)], expected_message)
})

test_that('already annotated message in verbose', {
  # use "serial: 3 files, (1 missing), 4 compounds, uROI, FIR replace peaks not found (cpd #3), getAcquTime, verbose"
  
  # sample 2 is missing
  # Cpd #3 will not give results
  noMatch_uROI3        <- input_uROI
  noMatch_uROI3[3,4:6] <- c(52.194778, 52.2, 52.205222)
  
  # Object fully initialised
  initAnnotation      <- peakPantheRAnnotation(spectraPaths=input_missingSpectraPaths, targetFeatTable=input_badtargetFeatTable, uROIExist=TRUE, useUROI=TRUE, uROI=noMatch_uROI3, useFIR=TRUE, FIR=input_FIR, isAnnotated=TRUE, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata)
  
  # Expected annotation
  expected_annotation             <- initAnnotation[c(1,3),]
  expected_annotation@TIC         <- c(2410533091, 2332817115)
  tmp_peakTables                        <- expected_peakTables[c(1,3)]
  tmp_peakTables[[1]][3,c(2:11,13:16)]  <- c(3444.524, 3454.435, 3478.431, 464.1995, 464.20001220703125, 464.2005, 8939889, 8939889, 380736, as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  tmp_peakTables[[1]]$is_filled[3]      <- TRUE
  tmp_peakTables[[2]][3,c(2:11,13:16)]  <- c(3444.524, 3460.696, 3478.431, 464.1995, 464.20001220703125, 464.2005, 7414831, 7414831, 319488, as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  tmp_peakTables[[2]]$found[3]          <- TRUE
  tmp_peakTables[[2]]$is_filled[3]      <- TRUE
  expected_annotation@peakTables        <- tmp_peakTables
  tmp_peakFit                     <- expected_peakFit[c(1,3)]
  tmp_peakFit[[1]][[3]]           <- NA
  tmp_peakFit[[2]][[3]]           <- NA
  expected_annotation@peakFit     <- tmp_peakFit
  tmp_dataPoints                  <- expected_dataPoints[c(1,3)]
  tmp_dataPoints[[1]][[3]]        <- data.frame(rt=numeric(), mz=numeric(), int=numeric())
  tmp_dataPoints[[2]][[3]]        <- data.frame(rt=numeric(), mz=numeric(), int=numeric())
  expected_annotation@dataPoints  <- tmp_dataPoints
  expected_annotation@isAnnotated <- TRUE
  # Expected failures
  tmp_status          <- 'Error file does not exist: aaa/bbb.cdf'
  names(tmp_status)   <- 'aaa/bbb.cdf'
  tmp_failures        <- !is.na(tmp_status)
  names(tmp_failures) <- NULL
  expected_failures   <- data.frame(matrix(c(names(tmp_status)[tmp_failures], tmp_status[tmp_failures]), ncol=2, byrow=FALSE, dimnames=list(c(), c('file', 'error'))), stringsAsFactors=FALSE)
  # Expected message
  expected_message    <- c("!! Data was already annotated, results will be overwritten !!\n", "Processing 4 compounds in 3 samples:\n", "  uROI:\tTRUE\n", "  FIR:\tTRUE\n", "----- ko15 -----\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Check input, mzMLPath must be a .mzML\n", "Reading data from 4 windows\n", "Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #1\n", "Fit of ROI #3 is unsuccessful (try err)\n", "1 features to integrate with FIR\n", "Reading data from 1 windows\n", "Error file does not exist: aaa/bbb.cdf\n", "----- ko18 -----\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Check input, mzMLPath must be a .mzML\n", "Reading data from 4 windows\n", "Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #1\n", "Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #2\n", "Fit of ROI #3 is unsuccessful (try err)\n", "Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #4\n", "1 features to integrate with FIR\n", "Reading data from 1 windows\n", "----------------\n", "1 file(s) failed to process:\n         file                                  error\n1 aaa/bbb.cdf Error file does not exist: aaa/bbb.cdf\n", "Annotation object cannot be reordered by sample acquisition date\n", "----------------\n", "  1 failure(s)\n")
  
  # results (output, warnings and messages)
  result_parallelAnnotation <- evaluate_promise(peakPantheR_parallelAnnotation(initAnnotation, ncores=0, getAcquTime=TRUE, verbose=TRUE))
  
  # Check results
  expect_equal(result_parallelAnnotation$result$annotation, expected_annotation, tolerance=1e-6)
  expect_equal(result_parallelAnnotation$result$failures, expected_failures)
  
  # Check messages (no timing)
  expect_equal(length(result_parallelAnnotation$messages), 41)
  expect_equal(result_parallelAnnotation$messages[c(1:8,10,11,14,15,19:23,25:28,31,32,36,37,38,39,41)], expected_message)
})

test_that('catch file that doesnt exist, catch error processing, no file left', {
  # Object fully initialised
  wrongPaths      <- c("aaa/bbb.cdf", system.file("extdata/test_fakemzML.mzML", package = "peakPantheR"))
  initAnnotation  <- peakPantheRAnnotation(spectraPaths=wrongPaths, targetFeatTable=input_targetFeatTable, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata)

  # Expected annotation
  expected_annotation             <- initAnnotation[c(FALSE, FALSE),]
  expected_annotation@isAnnotated <- FALSE
  # Expected message
  expected_message    <- c("Processing 4 compounds in 2 samples:\n", "  uROI:\tFALSE\n", "  FIR:\tFALSE\n", "Error file does not exist: aaa/bbb.cdf\n", "----- test_fakemzML -----\n", "-----\n", "Error processing file: test_fakemzML\n", "\n-----\n", "----------------\n", "No file left in the object!\n", "Annotation object reordered by sample acquisition date\n", "----------------\n", "  2 failure(s)\n")

  # results (output, warnings and messages)
  result_parallelAnnotation <- evaluate_promise(peakPantheR_parallelAnnotation(initAnnotation, ncores=0, getAcquTime=FALSE, verbose=TRUE))

  # Check results
  expect_equal(result_parallelAnnotation$result$annotation, expected_annotation, tolerance=1e-6)
  # cannot check the failure paths
  expect_equal(dim(result_parallelAnnotation$result$failures)[1], 2)
  expect_equal(dim(result_parallelAnnotation$result$failures)[2], 2)

  # Check messages (remove timing and paths in error messages)
  expect_equal(length(result_parallelAnnotation$messages), 16)
  expect_equal(result_parallelAnnotation$messages[c(1:7, 9, 10, 12, 13, 14, 16)], expected_message)
})

test_that('curveModel emgGaussian: 3 files, 4 compounds, no uROI, no FIR, no getAcquTime, no verbose', {
  # Object fully initialised
  initAnnotation      <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata)

  # Expected peakFit
  # 1
  cFit1.1emg         <- list(amplitude=22322950, center=3341.888, sigma=5, gamma=0.09934113, fitStatus=2, curveModel="emgGaussian")
  class(cFit1.1emg)  <- 'peakPantheR_curveFit'
  cFit1.2emg         <- list(amplitude=29181384, center=3382.577, sigma=5, gamma=0.08974434, fitStatus=2, curveModel="emgGaussian")
  class(cFit1.2emg)  <- 'peakPantheR_curveFit'
  cFit1.3emg         <- list(amplitude=9173607, center=3451.435, sigma=5, gamma=0.1071507, fitStatus=2, curveModel="emgGaussian")
  class(cFit1.3emg)  <- 'peakPantheR_curveFit'
  cFit1.4emg         <- list(amplitude=7719709, center=3698.697, sigma=5, gamma=0.1060304, fitStatus=2, curveModel="emgGaussian")
  class(cFit1.4emg)  <- 'peakPantheR_curveFit'
  # 2
  cFit2.1emg         <- list(amplitude=22094946, center=3359.102, sigma=5, gamma=0.07724317, fitStatus=2, curveModel="emgGaussian")
  class(cFit2.1emg)  <- 'peakPantheR_curveFit'
  cFit2.2emg         <- list(amplitude=34397901, center=3399.791, sigma=5, gamma=0.0651969, fitStatus=2, curveModel="emgGaussian")
  class(cFit2.2emg)  <- 'peakPantheR_curveFit'
  cFit2.3emg         <- list(amplitude=9671145, center=3460.824, sigma=5, gamma=0.08864798, fitStatus=2, curveModel="emgGaussian")
  class(cFit2.3emg)  <- 'peakPantheR_curveFit'
  cFit2.4emg         <- list(amplitude=7296689, center=3714.347, sigma=5, gamma=0.06171862, fitStatus=2, curveModel="emgGaussian")
  class(cFit2.4emg)  <- 'peakPantheR_curveFit'
  # 3
  cFit3.1emg         <- list(amplitude=20390552, center=3362.233, sigma=5, gamma=0.08186031, fitStatus=2, curveModel="emgGaussian")
  class(cFit3.1emg)  <- 'peakPantheR_curveFit'
  cFit3.2emg         <- list(amplitude=29719102, center=3409.182, sigma=5, gamma=0.09218107, fitStatus=2, curveModel="emgGaussian")
  class(cFit3.2emg)  <- 'peakPantheR_curveFit'
  cFit3.3emg         <- NA
  cFit3.4emg         <- list(amplitude=5751872, center=3708.088, sigma=5, gamma=0.06964929, fitStatus=2, curveModel="emgGaussian")
  class(cFit3.4emg)  <- 'peakPantheR_curveFit'
  expected_peakFit_emg <- list(list(cFit1.1emg, cFit1.2emg, cFit1.3emg, cFit1.4emg), list(cFit2.1emg, cFit2.2emg, cFit2.3emg, cFit2.4emg), list(cFit3.1emg, cFit3.2emg, cFit3.3emg, cFit3.4emg))

  # Expected peakTables
  peakTable1emg     <- data.frame(matrix(vector(), 4, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
  peakTable1emg[1,] <- c(TRUE, 3327.999, 3346.996, 3403.919, 522.1948, 522.2, 522.2052, 22261723, 25713232, 889280,  1057506.5, FALSE, 0.02337616, 2.108434, 1.653451, 2.064384)
  peakTable1emg[2,] <- c(TRUE, 3368.739, 3387.926, 3450.433, 496.1950, 496.2, 496.2050, 29098356, 35721108, 1128960, 1313099.4, FALSE, 0.02460103, 2.349398, 1.745483, 2.219176)
  peakTable1emg[3,] <- c(TRUE, 3437.496, 3456.351, 3509.499, 464.1954, 464.2, 464.2046, 9149178, 10905635, 380736,  450748.4,  FALSE, 0.02629692, 1.915663, 1.592171, 1.962157)
  peakTable1emg[4,] <- c(TRUE, 3684.765, 3703.637, 3757.294, 536.1946, 536.2, 536.2054, 7699066, 8612410, 330176,  377425.9,  FALSE, 0.02276582, 1.939759, 1.600693, 1.976502)
  peakTable1emg[,c(1,12)]       <- sapply(peakTable1emg[,c(1,12)], as.logical)
  peakTable1emg[,c(2:11,13:16)] <- sapply(peakTable1emg[,c(2:11,13:16)], as.numeric)
  # 2
  peakTable2emg     <- data.frame(matrix(vector(), 4, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
  peakTable2emg[1,] <- c(TRUE, 3345.357, 3364.813, 3436.703, 522.1948, 522.2, 522.2052, 22028618, 22658480, 761664,  916975.6, FALSE, 0.02337616, 19.92484, 1.898412, 2.477039)
  peakTable2emg[2,] <- c(TRUE, 3386.117, 3405.791, 3490.313, 496.1950, 496.2, 496.2050, 34288936, 35743000, 1099264, 1293794.6, FALSE, 0.02460103, 20.21400, 2.116160, 2.850792)
  peakTable2emg[3,] <- c(TRUE, 3447.002, 3466.197, 3529.422, 464.1954, 464.2, 464.2046, 9643477, 10659308, 366720,  432400.6, FALSE, 0.02629692, 11.76249, 1.757855, 2.240265)
  peakTable2emg[4,] <- c(TRUE, 3700.724, 3720.347, 3809.535, 536.1946, 536.2, 536.2054, 7273154,  7093526, 220096,  265357.4, FALSE, 0.02276582, 18.65000, 2.207992, 3.012906)
  peakTable2emg[,c(1,12)]       <- sapply(peakTable2emg[,c(1,12)], as.logical)
  peakTable2emg[,c(2:11,13:16)] <- sapply(peakTable2emg[,c(2:11,13:16)], as.numeric)
  # 3
  peakTable3emg     <- data.frame(matrix(vector(), 4, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
  peakTable3emg[1,] <- c(TRUE, 3348.458, 3367.799, 3435.888, 522.1948, 522.2, 522.2052, 20330539, 19833736, 758336,  873749.6, FALSE, 0.02337616, 22.91127, 1.837158, 2.373902)
  peakTable3emg[2,] <- c(TRUE, 3395.322, 3414.459, 3475.443, 496.1950, 496.2, 496.2050, 29635397, 32843420, 1149440, 1355916.7, FALSE, 0.02460103, 28.88211, 1.721309, 2.178978)
  peakTable3emg[3,] <- c(FALSE, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, FALSE, NA, NA, NA, NA)
  peakTable3emg[4,] <- c(TRUE, 3694.401, 3714.064, 3793.313, 536.1946, 536.2, 536.2054, 5734001, 6062535, 196160,  225006.2, FALSE, 0.02276582, 12.36690, 2.015364, 2.673714)
  peakTable3emg[,c(1,12)]       <- sapply(peakTable3emg[,c(1,12)], as.logical)
  peakTable3emg[,c(2:11,13:16)] <- sapply(peakTable3emg[,c(2:11,13:16)], as.numeric)
  expected_peakTables_emg <- list(peakTable1emg, peakTable2emg, peakTable3emg)

  # Expected annotation
  expected_annotation             <- initAnnotation
  expected_annotation@TIC         <- c(2410533091, 2524040155, 2332817115)
  expected_annotation@peakTables  <- expected_peakTables_emg
  expected_annotation@peakFit     <- expected_peakFit_emg
  expected_annotation@dataPoints  <- expected_dataPoints
  expected_annotation@isAnnotated <- TRUE
  # Expected failures
  tmp_status          <- NA
  names(tmp_status)   <- 'test'
  tmp_failures        <- !is.na(tmp_status)
  names(tmp_failures) <- NULL
  expected_failures   <- data.frame(matrix(c(names(tmp_status)[tmp_failures], tmp_status[tmp_failures]), ncol=2, byrow=FALSE, dimnames=list(c(), c('file', 'error'))), stringsAsFactors=FALSE)
  # Expected message
  expected_message    <- c("Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Fit of ROI #3 is unsuccessful (cannot determine rtMin/rtMax)\n")

  # results (output, warnings and messages)
  result_parallelAnnotation <- evaluate_promise(peakPantheR_parallelAnnotation(initAnnotation, ncores=0, getAcquTime=FALSE, verbose=FALSE, curveModel='emgGaussian'))

  # Check results
  expect_equal(result_parallelAnnotation$result$annotation, expected_annotation, tolerance=1e-5)
  expect_equal(result_parallelAnnotation$result$failures, expected_failures)

  # Check messages (centwave output)
  expect_equal(length(result_parallelAnnotation$messages), 4)
  expect_equal(result_parallelAnnotation$messages, expected_message)
  })

test_that('curveModel unknown: 3 failures', {
  # Object fully initialised
  initAnnotation      <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata)

  # Expected annotation
  expected_annotation                 <- peakPantheRAnnotation(targetFeatTable=input_targetFeatTable, cpdMetadata=input_cpdMetadata)
  expected_annotation@spectraMetadata <- data.frame(matrix(character(), ncol=2, byrow=FALSE, dimnames=list(c(), c('testcol1', 'testcol2'))), stringsAsFactors=FALSE)

  #expected_annotation@TIC         <- c(2410533091, 2524040155, 2332817115)
  #expected_annotation@peakTables  <- expected_peakTables
  #expected_annotation@peakFit     <- expected_peakFit
  #expected_annotation@dataPoints  <- expected_dataPoints
  #expected_annotation@isAnnotated <- TRUE
  # Expected failures
  tmp_status          <- c('Error: "curveModel" must be one of: skewedGaussian, emgGaussian','Error: "curveModel" must be one of: skewedGaussian, emgGaussian','Error: "curveModel" must be one of: skewedGaussian, emgGaussian')
  names(tmp_status)   <- input_spectraPaths
  tmp_failures        <- !is.na(tmp_status)
  names(tmp_failures) <- NULL
  expected_failures   <- data.frame(matrix(c(names(tmp_status)[tmp_failures], tmp_status[tmp_failures]), ncol=2, byrow=FALSE, dimnames=list(c(), c('file', 'error'))), stringsAsFactors=FALSE)
  # Expected message
  expected_message    <- character(0)

  # results (output, warnings and messages)
  result_parallelAnnotation <- evaluate_promise(peakPantheR_parallelAnnotation(initAnnotation, ncores=0, getAcquTime=FALSE, verbose=FALSE, curveModel='unknown_curveModel'))

  # Check results (all failures)
  expect_equal(result_parallelAnnotation$result$annotation, expected_annotation, tolerance=1e-5)
  expect_equal(result_parallelAnnotation$result$failures, expected_failures)

  # Check messages (no fit, so no message)
  expect_equal(length(result_parallelAnnotation$messages), 0)
  expect_equal(result_parallelAnnotation$messages, expected_message)
  })

test_that('raise errors', {
  # Object fails validation on input check
  wrongInit       <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable)
  wrongInit@TIC   <- c(1, 2)
  msg1            <- paste('invalid class ', dQuote('peakPantheRAnnotation'),' object: TIC has 2 elements (samples). Should be 3', sep='')
  expect_error(peakPantheR_parallelAnnotation(wrongInit, ncores=0, getAcquTime=FALSE, verbose=FALSE), msg1, fixed=TRUE)

  # resetWorkers is not an integer
  initAnnotation2 <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable)
  msg2            <- "Check input, resetWorkers must be an integer"
  expect_error(peakPantheR_parallelAnnotation(initAnnotation2, ncores=0, getAcquTime=FALSE, resetWorkers='not an Integer', verbose=FALSE), msg2, fixed=TRUE)
  
  # resetWorkers is < 0
  initAnnotation3 <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable)
  msg3            <- "Check input, resetWorkers must be a positive integer"
  expect_error(peakPantheR_parallelAnnotation(initAnnotation3, ncores=0, getAcquTime=FALSE, resetWorkers=-10, verbose=FALSE), msg3, fixed=TRUE)
})

context('peakPantheRAnnotation_accessor-method()')

## Test the accessors with the right values

skip_if_not_installed('faahKO',  minimum_version = '1.18.0')
library(faahKO)

## Input data
# spectraPaths
input_spectraPaths  <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                         system.file('cdf/KO/ko16.CDF', package = "faahKO"),
                         system.file('cdf/KO/ko18.CDF', package = "faahKO"))

# targetFeatTable
input_targetFeatTable     <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390., 522.194778, 522.2, 522.205222)
input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440., 496.195038, 496.2, 496.204962)
input_targetFeatTable[,c(3:8)] <- sapply(input_targetFeatTable[,c(3:8)], as.numeric)

# FIR
input_FIR     <- data.frame(matrix(vector(), 2, 4, dimnames=list(c(), c("rtMin", "rtMax", "mzMin", "mzMax"))), stringsAsFactors=FALSE)
input_FIR[1,] <- c(1., 2., 3., 4.)
input_FIR[2,] <- c(5., 6., 7., 8.)

# uROI
input_uROI      <- data.frame(matrix(vector(), 2, 6, dimnames=list(c(), c("rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
input_uROI[1,]  <- c(9., 10., 11., 12., 13., 14.)
input_uROI[2,]  <- c(15., 16., 17., 18., 19., 20.)

# TICs
input_TIC <- c(2410533091, 2524040155, 2332817115)

# cpdMetadata
input_cpdMetadata     <- data.frame(matrix(data=c('a','b',1,2), nrow=2, ncol=2, dimnames=list(c(),c('testcol1','testcol2')), byrow=FALSE), stringsAsFactors=FALSE)

# spectraMetadata
input_spectraMetadata <- data.frame(matrix(data=c('c','d','e',3,4,5), nrow=3, ncol=2, dimnames=list(c(),c('testcol1','testcol2')), byrow=FALSE), stringsAsFactors=FALSE)

# acquisitionTime
input_acquisitionTime <- c(as.character(Sys.time()), as.character(Sys.time()+900), as.character(Sys.time()+1800))

# peakTables
# 1
peakTable1     <- data.frame(matrix(vector(), 2, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
peakTable1[1,] <- c(TRUE, 3309.7589296586070, 3346.8277590361445, 3385.4098874628098, 522.194778, 522.20001220703125, 522.205222, 26133726.6811244078, 26133726, 889280, 901015.80529226747, FALSE, 0.023376160866574614, 1.93975903614455092, 1.0153573486330891, 1.0268238825675249)
peakTable1[2,] <- c(TRUE, 3345.3766648628907, 3386.5288072289159, 3428.2788374983961, 496.20001220703125, 496.20001220703125, 496.20001220703125, 35472141.3330242932, 35472141, 1128960, 1113576.69008227298, FALSE, 0.024601030353423384, 0.95180722891564074, 1.0053782620427065, 1.0093180792278085)
peakTable1[,c(1,12)]       <- sapply(peakTable1[,c(1,12)], as.logical)
peakTable1[,c(2:11,13:16)] <- sapply(peakTable1[,c(2:11,13:16)], as.numeric)
# 2
peakTable2     <- data.frame(matrix(vector(), 2, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
peakTable2[1,] <- c(TRUE, 3326.1063495851854, 3365.102, 3407.2726475892355, 522.194778, 522.20001220703125, 522.205222, 24545301.622835573, 24545301, 761664, 790802.2209998488, FALSE, 0.023376160866574614, 0.2139999999999, 1.0339153786516375, 1.0630802030537212)
peakTable2[2,] <- c(TRUE, 3365.0238566258713, 3405.791, 3453.4049569205681, 496.195038, 496.20001220703125, 496.204962, 37207579.286265120, 37207579, 1099264, 1098720.2929832144, FALSE, 0.024601030353423384, 20.2139999999999, 1.0839602450900523, 1.1717845972583161)
peakTable2[,c(1,12)]       <- sapply(peakTable2[,c(1,12)], as.logical)
peakTable2[,c(2:11,13:16)] <- sapply(peakTable2[,c(2:11,13:16)], as.numeric)
# 3
peakTable3     <- data.frame(matrix(vector(), 2, 16, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
peakTable3[1,] <- c(TRUE, 3333.8625894557053, 3368.233, 3407.4362838927614, 522.194778, 522.20001220703125, 522.205222, 21447174.404490683, 21447174, 758336, 765009.9805796633, FALSE, 0.023376160866574614, 23.345000000000255, 1.0609102044546637, 1.1155310457756928)
peakTable3[2,] <- c(TRUE, 3373.3998828113113, 3413.4952530120481, 3454.4490330927388, 496.195038, 496.20001220703125, 496.204962, 35659353.614476241, 35659353, 1149440, 1145857.7611069249, TRUE, 0.024601030353423384, 27.918253012047899, 1.0081407426394933, 1.0143315197994494)
peakTable3[,c(1,12)]       <- sapply(peakTable3[,c(1,12)], as.logical)
peakTable3[,c(2:11, 13:16)] <- sapply(peakTable3[,c(2:11,13:16)], as.numeric)
input_peakTables <- list(peakTable1, peakTable2, peakTable3)

# peakFit
# 1
cFit1.1         <- list(amplitude=162404.8057918259, center=3341.888, sigma=0.078786133031045896, gamma=0.0018336101984172684, fitStatus=2, curveModel="skewedGaussian")
class(cFit1.1)  <- 'peakPantheR_curveFit'
cFit1.2         <- list(amplitude=199249.10572753669, center=3382.577, sigma=0.074904415304607966, gamma=0.0011471899372353885, fitStatus=2, curveModel="skewedGaussian")
class(cFit1.2)  <- 'peakPantheR_curveFit'
# 2
cFit2.1         <- list(amplitude=124090.83425474487, center=3359.102, sigma=0.071061541060964212, gamma=0.0018336072657203239, fitStatus=2, curveModel="skewedGaussian")
class(cFit2.1)  <- 'peakPantheR_curveFit'
cFit2.2         <- list(amplitude=151407.23415130575, center=3399.791, sigma=0.063753866057052563, gamma=0.001676782834598999, fitStatus=2, curveModel="skewedGaussian")
class(cFit2.2)  <- 'peakPantheR_curveFit'
# 3
cFit3.1         <- list(amplitude=122363.51256736703, center=3362.233, sigma=0.075489598945304492, gamma=0.0025160536725299734, fitStatus=2, curveModel="skewedGaussian")
class(cFit3.1)  <- 'peakPantheR_curveFit'
cFit3.2         <- list(amplitude=204749.86097918145, center=3409.182, sigma=0.075731781812843249, gamma=0.0013318670577834328, fitStatus=2, curveModel="skewedGaussian")
class(cFit3.2)  <- 'peakPantheR_curveFit'
input_peakFit   <- list(list(cFit1.1, cFit1.2), list(cFit2.1, cFit2.2), list(cFit3.1, cFit3.2))

# dataPoint
tmp_raw_data1  	  <- MSnbase::readMSData(input_spectraPaths[1], centroided=TRUE, mode='onDisk')
ROIDataPoints1    <- extractSignalRawData(tmp_raw_data1, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
tmp_raw_data2  	  <- MSnbase::readMSData(input_spectraPaths[2], centroided=TRUE, mode='onDisk')
ROIDataPoints2    <- extractSignalRawData(tmp_raw_data2, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
tmp_raw_data3  	  <- MSnbase::readMSData(input_spectraPaths[3], centroided=TRUE, mode='onDisk')
ROIDataPoints3    <- extractSignalRawData(tmp_raw_data3, rt=input_targetFeatTable[,c('rtMin','rtMax')], mz=input_targetFeatTable[,c('mzMin','mzMax')], verbose=FALSE)
input_dataPoints  <- list(ROIDataPoints1, ROIDataPoints2, ROIDataPoints3)

# Object, fully filled
filledAnnotation        <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable, FIR=input_FIR, uROI=input_uROI, useFIR=TRUE, uROIExist=TRUE, useUROI=TRUE, cpdMetadata=input_cpdMetadata, spectraMetadata=input_spectraMetadata, acquisitionTime=input_acquisitionTime, TIC=input_TIC, peakTables=input_peakTables, dataPoints=input_dataPoints, peakFit=input_peakFit, isAnnotated=TRUE)

# Empty annotation, special case for peakTables
emptyAnnotation         <- peakPantheRAnnotation(spectraPaths=input_spectraPaths, targetFeatTable=input_targetFeatTable)


test_that('accessors return the correct values', {
  expected_ROI              <- input_targetFeatTable[, c("rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "cpdID", "cpdName")]
  expected_FIR              <- cbind.data.frame(input_FIR, cpdID=c("ID-1","ID-2"), cpdName=c("Cpd 1", "Cpd 2"), stringsAsFactors=FALSE)
  expected_uROI             <- cbind.data.frame(input_uROI, cpdID=c("ID-1","ID-2"), cpdName=c("Cpd 1", "Cpd 2"), stringsAsFactors=FALSE)
  expected_acquisitionTime  <- as.POSIXct(input_acquisitionTime)
  expected_peakTables       <- list(cbind.data.frame(peakTable1, cpdID=c("ID-1","ID-2"), cpdName=c("Cpd 1","Cpd 2"), stringsAsFactors=FALSE), cbind.data.frame(peakTable2, cpdID=c("ID-1","ID-2"), cpdName=c("Cpd 1","Cpd 2"), stringsAsFactors=FALSE), cbind.data.frame(peakTable3, cpdID=c("ID-1","ID-2"), cpdName=c("Cpd 1","Cpd 2"), stringsAsFactors=FALSE))
  expected_dataPoints       <- input_dataPoints
  expected_peakFit          <- input_peakFit
  expected_filename         <- c("ko15", "ko16", "ko18")

  # Check accessors
  # Basic slots
  # cpdID
  expect_equal(cpdID(filledAnnotation), c("ID-1","ID-2"))
  # cpdName
  expect_equal(cpdName(filledAnnotation), c("Cpd 1", "Cpd 2"))
  # ROI
  expect_equal(ROI(filledAnnotation), expected_ROI)
  # FIR
  expect_equal(FIR(filledAnnotation), expected_FIR)
  # uROI
  expect_equal(uROI(filledAnnotation), expected_uROI)
  # filepath
  expect_equal(filepath(filledAnnotation), input_spectraPaths)
  # cpdMetadata
  expect_equal(cpdMetadata(filledAnnotation), input_cpdMetadata)
  # spectraMetadata
  expect_equal(spectraMetadata(filledAnnotation), input_spectraMetadata)
  # acquisitionTIme
  expect_equal(acquisitionTime(filledAnnotation), expected_acquisitionTime)
  # uROIExist
  expect_true(uROIExist(filledAnnotation))
  # useUROI
  expect_true(useUROI(filledAnnotation))
  # useFIR
  expect_true(useFIR(filledAnnotation))
  # TIC
  expect_equal(TIC(filledAnnotation), input_TIC)
  # peakTables
  expect_equal(peakTables(filledAnnotation), expected_peakTables)
  # dataPoints
  expect_equal(dataPoints(filledAnnotation), expected_dataPoints)
  # peakFit
  expect_equal(peakFit(filledAnnotation), expected_peakFit)
  # isAnnotated
  expect_true(isAnnotated(filledAnnotation))

  # nbSamples
  expect_equal(nbSamples(filledAnnotation), 3)
  # nbCompounds
  expect_equal(nbCompounds(filledAnnotation), 2)
  # filename
  expect_equal(filename(filledAnnotation), expected_filename)

  # annotationTable
  # simple value
  expected_annotationTable            <- data.frame(matrix(c(TRUE, TRUE, TRUE, TRUE, TRUE, TRUE), 3, 2), stringsAsFactors=FALSE)
  rownames(expected_annotationTable)  <- input_spectraPaths
  colnames(expected_annotationTable)  <- c("ID-1", "ID-2")
  expect_equal(annotationTable(filledAnnotation, 'found'), expected_annotationTable)
  # no sample
  tmp_noSample                <- peakPantheRAnnotation()
  expected_noSample           <- data.frame(matrix(vector(), 0, 0), stringsAsFactors=FALSE)
  rownames(expected_noSample) <- tmp_noSample@filepath
  colnames(expected_noSample) <- tmp_noSample@cpdID
  expect_equal(annotationTable(tmp_noSample, 'found'), expected_noSample)
  # no peakTables (not annotated or no compounds)
  tmp_noPeakTables            <- filledAnnotation
  tmp_noPeakTables@peakTables <- vector("list", 3)
  expected_noPeakTables           <- data.frame(matrix(vector(), 3, 2), stringsAsFactors=FALSE)
  rownames(expected_noPeakTables) <- tmp_noPeakTables@filepath
  colnames(expected_noPeakTables) <- tmp_noPeakTables@cpdID
  expect_equal(annotationTable(tmp_noPeakTables, 'found'), expected_noPeakTables)
  # only 1 compounds (sapply simplify to vector and not matrix)
  tmp_singleCpd         <- filledAnnotation[,1]
  expected_mz           <- data.frame(matrix(c(522.20001220703125, 522.20001220703125, 522.20001220703125), 3, 1), stringsAsFactors=FALSE)
  rownames(expected_mz) <- filledAnnotation@filepath
  colnames(expected_mz) <- filledAnnotation@cpdID[1]
  expect_equal(annotationTable(tmp_singleCpd, 'mz'), expected_mz)
  # raise error if column doesn't exist
  expect_error(annotationTable(filledAnnotation, 'notAnExistingColumn'), 'input column is not a column of peakTables', fixed=TRUE)

  ## try some more of the peakTable columns
  # mz
  expected_mz           <- data.frame(matrix(c(522.20001220703125, 522.20001220703125, 522.20001220703125, 496.20001220703125, 496.20001220703125, 496.20001220703125), 3, 2), stringsAsFactors=FALSE)
  rownames(expected_mz) <- filledAnnotation@filepath
  colnames(expected_mz) <- filledAnnotation@cpdID
  expect_equal(annotationTable(filledAnnotation, 'mz'), expected_mz)
  # mzMin
  expected_mzMin           <- data.frame(matrix(c(522.194778, 522.194778, 522.194778, 496.20001220703125, 496.195038, 496.195038), 3, 2), stringsAsFactors=FALSE)
  rownames(expected_mzMin) <- filledAnnotation@filepath
  colnames(expected_mzMin) <- filledAnnotation@cpdID
  expect_equal(annotationTable(filledAnnotation, 'mzMin'), expected_mzMin)
  # mzMax
  expected_mzMax           <- data.frame(matrix(c(522.205222, 522.205222, 522.205222, 496.20001220703125, 496.204962, 496.204962), 3, 2), stringsAsFactors=FALSE)
  rownames(expected_mzMax) <- filledAnnotation@filepath
  colnames(expected_mzMax) <- filledAnnotation@cpdID
  expect_equal(annotationTable(filledAnnotation, 'mzMax'), expected_mzMax)
  # rt
  expected_rt           <- data.frame(matrix(c(3346.8277590361445, 3365.102, 3368.233, 3386.5288072289159, 3405.791, 3413.4952530120481), 3, 2), stringsAsFactors=FALSE)
  rownames(expected_rt) <- filledAnnotation@filepath
  colnames(expected_rt) <- filledAnnotation@cpdID
  expect_equal(annotationTable(filledAnnotation, 'rt'), expected_rt)
  # rtMin
  expected_rtMin           <- data.frame(matrix(c(3309.7589296586070, 3326.1063495851854, 3333.8625894557053, 3345.3766648628907, 3365.0238566258713, 3373.3998828113113), 3, 2), stringsAsFactors=FALSE)
  rownames(expected_rtMin) <- filledAnnotation@filepath
  colnames(expected_rtMin) <- filledAnnotation@cpdID
  expect_equal(annotationTable(filledAnnotation, 'rtMin'), expected_rtMin)
  # rtMax
  expected_rtMax           <- data.frame(matrix(c(3385.4098874628098, 3407.2726475892355, 3407.4362838927614, 3428.2788374983961, 3453.4049569205681, 3454.4490330927388), 3, 2), stringsAsFactors=FALSE)
  rownames(expected_rtMax) <- filledAnnotation@filepath
  colnames(expected_rtMax) <- filledAnnotation@cpdID
  expect_equal(annotationTable(filledAnnotation, 'rtMax'), expected_rtMax)
  # peakArea
  expected_peakArea           <- data.frame(matrix(c(26133726.6811244078, 24545301.622835573, 21447174.404490683, 35472141.3330242932, 37207579.286265120, 35659353.614476241), 3, 2), stringsAsFactors=FALSE)
  rownames(expected_peakArea) <- filledAnnotation@filepath
  colnames(expected_peakArea) <- filledAnnotation@cpdID
  expect_equal(annotationTable(filledAnnotation, 'peakArea'), expected_peakArea)
  
  # EICs
  # default (sum)
  tmp_EIC_sum_1.1   <- generateIonChromatogram(input_dataPoints[[1]][[1]], aggregationFunction='sum')
  tmp_EIC_sum_1.2   <- generateIonChromatogram(input_dataPoints[[1]][[2]], aggregationFunction='sum')
  tmp_EIC_sum_2.1   <- generateIonChromatogram(input_dataPoints[[2]][[1]], aggregationFunction='sum')
  tmp_EIC_sum_2.2   <- generateIonChromatogram(input_dataPoints[[2]][[2]], aggregationFunction='sum')
  tmp_EIC_sum_3.1   <- generateIonChromatogram(input_dataPoints[[3]][[1]], aggregationFunction='sum')
  tmp_EIC_sum_3.2   <- generateIonChromatogram(input_dataPoints[[3]][[2]], aggregationFunction='sum')
  expected_EIC_sum  <- list(list(tmp_EIC_sum_1.1, tmp_EIC_sum_1.2),
                            list(tmp_EIC_sum_2.1, tmp_EIC_sum_2.2),
                            list(tmp_EIC_sum_3.1, tmp_EIC_sum_3.2))
  expect_equal(EICs(filledAnnotation), expected_EIC_sum)
  # change aggregationFunction (max)
  tmp_EIC_max_1.1   <- generateIonChromatogram(input_dataPoints[[1]][[1]], aggregationFunction='max')
  tmp_EIC_max_1.2   <- generateIonChromatogram(input_dataPoints[[1]][[2]], aggregationFunction='max')
  tmp_EIC_max_2.1   <- generateIonChromatogram(input_dataPoints[[2]][[1]], aggregationFunction='max')
  tmp_EIC_max_2.2   <- generateIonChromatogram(input_dataPoints[[2]][[2]], aggregationFunction='max')
  tmp_EIC_max_3.1   <- generateIonChromatogram(input_dataPoints[[3]][[1]], aggregationFunction='max')
  tmp_EIC_max_3.2   <- generateIonChromatogram(input_dataPoints[[3]][[2]], aggregationFunction='max')
  expected_EIC_max  <- list(list(tmp_EIC_max_1.1, tmp_EIC_max_1.2),
                            list(tmp_EIC_max_2.1, tmp_EIC_max_2.2),
                            list(tmp_EIC_max_3.1, tmp_EIC_max_3.2))
  expect_equal(EICs(filledAnnotation, 'max'), expected_EIC_max)
  
  # Special peakTables behaviour when peakTable is NULL
  expect_equal(peakTables(emptyAnnotation), list(NULL, NULL, NULL))
})

#' @title Check if object is of class peakPantheR_curveFit
#' 
#' @description Check if object is of class peakPantheR_curveFit
#' @param x object to test
#' @return (bool) TRUE or FALSE
is.peakPantheR_curveFit <- function(x) {
    inherits(x, "peakPantheR_curveFit")
}


#' @title Curve fitting using minpack.lm
#' 
#' @description Fit different curve models using \code{minpack}. Fitting
#' parameters can be passed or guessed.
#'
#' @param x (numeric) x values (e.g. retention time)
#' @param y (numeric) y observed values (e.g. spectra intensity)
#' @param curveModel (str) name of the curve model to fit (currently
#' \code{skewedGaussian} and \code{emgGaussian})
#' @param params (list or str) either 'guess' for automated parametrisation or
#' list of initial parameters (\code{$init_params}), lower parameter bounds
#' (\code{$lower_bounds}) and upper parameter bounds (\code{$upper_bounds})
#'
#' @return A 'peakPantheR_curveFit': a list of fitted curve parameters,
#' \code{fitStatus} from \code{nls.lm$info} and curve shape name
#' \code{curveModel}. \code{fitStatus=0} unsuccessful completion: improper input
#' parameters, \code{fitStatus=1} successful completion: first convergence test
#' is successful, \code{fitStatus=2} successful completion: second convergence
#' test is successful, \code{fitStatus=3} successful completion: both
#' convergence test are successful, \code{fitStatus=4} questionable completion:
#' third convergence test is successful but should be carefully examined
#' (maximizers and saddle points might satisfy), \code{fitStatus=5} unsuccessful
#' completion: excessive number of function evaluations/iterations
#' 
#' @details
#' ## Examples cannot be computed as the function is not exported:
#' ## x is retention time, y corresponding intensity
#' input_x  <- c(3362.102, 3363.667, 3365.232, 3366.797, 3368.362, 3369.927,
#'             3371.492, 3373.057, 3374.622, 3376.187, 3377.752, 3379.317,
#'             3380.882, 3382.447, 3384.012, 3385.577, 3387.142, 3388.707,
#'             3390.272, 3391.837, 3393.402, 3394.966, 3396.531, 3398.096,
#'             3399.661, 3401.226, 3402.791, 3404.356, 3405.921, 3407.486,
#'             3409.051)
#' input_y  <- c(51048, 81568, 138288, 233920, 376448, 557288, 753216, 938048,
#'             1091840, 1196992, 1261056, 1308992, 1362752, 1406592, 1431360,
#'             1432896, 1407808, 1345344, 1268480, 1198592, 1126848, 1036544,
#'             937600, 849792, 771456, 692416, 614528, 546088, 492752,
#'             446464, 400632)
#' 
#' ## Fit
#' fitted_curve <- fitCurve(input_x, input_y, curveModel='skewedGaussian',
#'                         params='guess')
#' 
#' ## Returns the optimal fitting parameters
#' fitted_curve
#' #
#' # $amplitude
#' # [1] 275371.1
#' # 
#' # $center
#' # [1] 3382.577
#' # 
#' # $sigma
#' # [1] 0.07904697
#' # 
#' # $gamma
#' # [1] 0.001147647
#' # 
#' # $fitStatus
#' # [1] 2
#' # 
#' # $curveModel
#' # [1] 'skewedGaussian'
#' #
#' # attr(,'class')
#' # [1] 'peakPantheR_curveFit'
fitCurve <- function(x, y, curveModel = "skewedGaussian", params = "guess") {
    # Check inputs x and y length
    if (length(x) != length(y)) {
        stop("Err","or: length of \"x\" and \"y\" must match!") }
    # known curveModel
    known_curveModel <- c("skewedGaussian", "emgGaussian")
    if (!(curveModel %in% known_curveModel)) {
        stop(paste0("Err","or: \"curveModel\" must be one of: ",
            paste(known_curveModel, collapse=', '))) }
    # params
    if (!(typeof(params) %in% c("list", "character"))) {
        stop("Err","or: \"params\" must be a list or \"guess\"") }

    useGuess <- TRUE
    if (any(params != "guess")) {
        useGuess <- FALSE
        # check init_params, lower and upper bounds are defined
        if (!all(c("init_params", "lower_bounds", "upper_bounds") %in%
            names(params))) {
            stop("Err","or: \"params must be a list of \"init_params\", ",
                "\"lower_bounds\" and \"upper_bounds\"") }
        # init_params is list
        if (typeof(params$init_params) != "list") {
            stop("Err", "or: \"params$init_params\" must be ",
            "a list of parameters") }
        # lower_bounds is list
        if (typeof(params$lower_bounds) != "list") {
            stop("Err", "or: \"params$lower_bounds\" must be ",
            "a list of parameters") }
        # upper_bounds is list
        if (typeof(params$upper_bounds) != "list") {
            stop("Err", "or: \"params$upper_bounds\" must be ",
            "a list of parameters") }
    }
    
    # Init
    fittedCurve <- list()
    # Run fitting skewed gaussian
    if (curveModel == "skewedGaussian") {
        fittedCurve <- fitCurve_skewedGaussian(x, y, useGuess, params,
                                                curveModel)
    }
    else if (curveModel == 'emgGaussian') {
        fittedCurve <- fitCurve_emgGaussian(x, y, useGuess, params,
                                                curveModel)
    }
    # for future curve shapes } else if () { }
    return(fittedCurve)
}

# fit skewedGaussian
fitCurve_skewedGaussian <- function(x, y, useGuess, params, curveModel) {
    fittedCurve <- list()

    # Guess parameters and bounds
    if (useGuess) {
        new_params <- skewedGaussian_guess(x, y)
    } else {
        new_params <- params
    }

    # ensure order of init params and bounds (init is a list, lower and
    # upper are ordered numeric vectors)
    init    <- list(amplitude = new_params$init_params$amplitude,
                    center = new_params$init_params$center,
                    sigma = new_params$init_params$sigma,
                    gamma = new_params$init_params$gamma)
    lower   <- unlist(c(new_params$lower_bounds["amplitude"],
                        new_params$lower_bounds["center"],
                        new_params$lower_bounds["sigma"],
                        new_params$lower_bounds["gamma"]))
    upper   <- unlist(c(new_params$upper_bounds["amplitude"],
                        new_params$upper_bounds["center"],
                        new_params$upper_bounds["sigma"],
                        new_params$upper_bounds["gamma"]))

    # perform fit
    resultFit <- minpack.lm::nls.lm(par = init,
                                lower = lower,
                                upper = upper,
                                fn = skewedGaussian_minpack.lm_objectiveFun,
                                observed = y, xx = x)

    # prepare output
    fittedCurve <- resultFit$par
    fittedCurve$fitStatus <- resultFit$info
    fittedCurve$curveModel <- curveModel
    class(fittedCurve) <- "peakPantheR_curveFit"

    return(fittedCurve)
}

# fit emgGaussian
fitCurve_emgGaussian <- function(x, y, useGuess, params, curveModel) {
    fittedCurve <- list()

    # Guess parameters and bounds
    if (useGuess) {
        new_params <- emgGaussian_guess(x, y)
    } else {
        new_params <- params
    }

    # ensure order of init params and bounds (init is a list, lower and
    # upper are ordered numeric vectors)
    init    <- list(amplitude = new_params$init_params$amplitude,
                    center = new_params$init_params$center,
                    sigma = new_params$init_params$sigma,
                    gamma = new_params$init_params$gamma)
    lower   <- unlist(c(new_params$lower_bounds["amplitude"],
                        new_params$lower_bounds["center"],
                        new_params$lower_bounds["sigma"],
                        new_params$lower_bounds["gamma"]))
    upper   <- unlist(c(new_params$upper_bounds["amplitude"],
                        new_params$upper_bounds["center"],
                        new_params$upper_bounds["sigma"],
                        new_params$upper_bounds["gamma"]))

    # perform fit
    resultFit <- minpack.lm::nls.lm(par = init,
                                lower = lower,
                                upper = upper,
                                fn = emgGaussian_minpack.lm_objectiveFun,
                                observed = y, xx = x)

    # prepare output
    fittedCurve <- resultFit$par
    fittedCurve$fitStatus <- resultFit$info
    fittedCurve$curveModel <- curveModel
    class(fittedCurve) <- "peakPantheR_curveFit"

    return(fittedCurve)
}



#' @title Predict curve values
#'
#' @description Evaluate fitted curve values at \code{x} data points
#'
#' @param fittedCurve (peakPantheR_curveFit) A 'peakPantheR_curveFit': a list of
#' curve fitting parameters, curve shape model \code{curveModel} and nls.lm fit
#' status \code{fitStatus}.
#' @param x (numeric) values at which to evaluate the fitted curve
#' 
#' @return  fitted curve values at x
#' 
#' @details
#' ## Examples cannot be computed as the function is not exported:
#' ## Input a fitted curve
#' fittedCurve <- list(amplitude=275371.1, center=3382.577, sigma=0.07904697,
#'                     gamma=0.001147647, fitStatus=2,
#'                     curveModel='skewedGaussian')
#' class(fittedCurve)  <- 'peakPantheR_curveFit'
#' input_x <- c(3290, 3300, 3310, 3320, 3330, 3340, 3350, 3360, 3370, 3380,
#'             3390, 3400, 3410)
#'
#' ## Predict y at each input_x
#' pred_y  <- predictCurve(fittedCurve, input_x)
#' pred_y
#' # [1] 2.347729e-08 1.282668e-05 3.475590e-03 4.676579e-01 3.129420e+01
#' # [6] 1.043341e+03 1.736915e+04 1.447754e+05 6.061808e+05 1.280037e+06
#' # [11] 1.369651e+06 7.467333e+05 2.087477e+05
predictCurve <- function(fittedCurve, x) {
    
    # Check input
    if (!is.peakPantheR_curveFit(fittedCurve)) {
        stop("Err","or: \"fittedCurve\" must be a peakPantheR_curveFit!")
    }
    # known curveModel
    known_curveModel <- c("skewedGaussian", "emgGaussian")
    if (!(fittedCurve$curveModel %in% known_curveModel)) {
        stop(paste0("Err","or: \"curveModel\" must be one of: ",
            paste(known_curveModel, collapse=', ')))
    }
    
    # Select correct model
    if (fittedCurve$curveModel == "skewedGaussian")
        {
            yy <- skewedGaussian_minpack.lm(params = fittedCurve, xx = x)
        }
    else if (fittedCurve$curveModel == 'emgGaussian') {
        yy <- emgGaussian_minpack.lm(params = fittedCurve, xx = x)
        }
    # for future curve shapes
    # else if () {}

    return(yy)
}



## -----------------------------------------------------------------------------
## Skewed Gaussian
## -----------------------------------------------------------------------------

#' @title Gaussian Error function
#'
#' @description Implementation of the gaussian error function
#' 
#' @param x (numeric) value at which to evaluate the gaussian error function
#' 
#' @return Value of the gaussian error function evaluated at x
gaussian_erf <- function(x) {
    return(2 * stats::pnorm(x * sqrt(2)) - 1)
}


#' @title Gaussian Error function
#'
#' @description Implementation of the gaussian error function
#'
#' @param x (numeric) value at which to evaluate the gaussian error function
#'
#' @return Value of the gaussian error function evaluated at x
gaussian_cerf <- function(x) {
    return(1 - (2 * stats::pnorm(x * sqrt(2)) - 1))
}


#' @title Implementation of the Skewed Gaussian peak shape for use with
#' minpack.lm
#'
#' @description Implementation of the Skewed Gaussian peak shape for use with
#' minpack.lm
#'
#' @param params (list) skewed gaussian parameters (\code{params$gamma},
#' \code{params$center}, \code{params$sigma}, \code{params$amplitude})
#' @param xx (numeric) values at which to evalute the skewed gaussian
#' 
#' @return value of the skewed gaussian evaluated at xx
skewedGaussian_minpack.lm <- function(params, xx) {
    erf_term <- 1 + gaussian_erf((params$gamma * (xx-params$center))/
            params$sigma * sqrt(2))
    yy <- (params$amplitude/(params$sigma * sqrt(2 * pi))) *
        exp(-(xx - params$center)^2/2 * params$sigma^2) * erf_term
    
    return(yy)
}

#' @title Implementation of the Exponentially Modified Gaussian (EMG) peak
#' shape for use with minpack.lm
#'
#' @description Implementation of the  Exponentially Modified Gaussian
#' (EMG) peak shape for use with minpack.lm
#'
#' @param params (list) exponentiall modified gaussian parameters
#' (\code{params$gamma}, \code{params$center}, \code{params$sigma},
#' \code{params$amplitude})
#' @param xx (numeric) values at which to evalute the exponentially
#' modified gaussian
#'
#' @return value of the exponentially modified gaussian evaluated at xx
emgGaussian_minpack.lm <- function(params, xx) {
    cerf_term <- gaussian_cerf((params$center + params$gamma *
        (params$sigma^2) - xx)/(params$sigma * sqrt(2)))
    yy <- (params$amplitude*params$gamma/2) *
        exp(params$gamma*(params$center - xx +
        (params$gamma * (params$sigma^2)/2))) * cerf_term

    return(yy)
}

#' @title Skewed Gaussian minpack.lm objective function
#'
#' @description Skewed Gaussian minpack.lm objective function, calculates
#' residuals using the skewed gaussian Peak Shape
#'
#' @param params (list) skewed gaussian parameters (\code{params$gamma},
#' \code{params$center}, \code{params$sigma}, \code{params$amplitude})
#' @param observed (numeric) observed y value at xx
#' @param xx (numeric) value at which to evalute the skewed gaussian
#' 
#' @return difference between observed and expected skewed gaussian value
#' evaluated at xx
skewedGaussian_minpack.lm_objectiveFun <- function(params, observed, xx) {
    return(observed - skewedGaussian_minpack.lm(params, xx))
}

#' @title Exponentially Modified Gaussian minpack.lm objective function
#'
#' @description Exponentially Modified Gaussian (EMG) minpack.lm objective
#' function, calculates residuals using the EMG Peak Shape
#'
#' @param params (list) exponentially modified gaussian parameters
#' (\code{params$gamma}, \code{params$center}, \code{params$sigma},
#' \code{params$amplitude})
#' @param observed (numeric) observed y value at xx
#' @param xx (numeric) value at which to evalute the exponentially
#' modified gaussian
#'
#' @return difference between observed and expected exponentially modified
#' gaussian value evaluated at xx
emgGaussian_minpack.lm_objectiveFun <- function(params, observed, xx) {
    return(observed - emgGaussian_minpack.lm(params, xx))
}

#' @title Guess function for initial skewed gaussian parameters and bounds
#'
#' @description Guess function for initial skewed gaussian parameters and
#' bounds, at the moment only checks the x position
#'
#' @param x (numeric) x values (e.g. retention time)
#' @param y (numeric) y observed values (e.g. spectra intensity)
#' 
#' @return A list of guessed starting parameters \code{list()$init_params},
#' lower \code{list()$lower_bounds} and upper bounds \code{list()$upper_bounds}
#' (\code{$gamma}, \code{$center}, \code{$sigma}, \code{$amplitude})
skewedGaussian_guess <- function(x, y) {
    # set center as x position of max y value (e.g. highest spectra intensity)
    center_guess <- x[which.max(y)]
    # init_param
    init_params <- list(amplitude=1e+07, center=center_guess, sigma=1, gamma=1)
    # lower_bounds
    lower_bounds <- list(amplitude=0, center=center_guess-3, sigma=0,gamma=-0.1)
    # upper_bounds
    upper_bounds <- list(amplitude=1e+09, center=center_guess+3, sigma=5,
        gamma = 5)
    
    return(list(init_params = init_params, lower_bounds = lower_bounds,
                upper_bounds = upper_bounds))
}

#' @title Guess function for initial exponentially modified gaussian
#' parameters and bounds
#'
#' @description Guess function for initial exponentially modified gaussian
#' parameters and bounds, at the moment only checks the x position
#'
#' @param x (numeric) x values (e.g. retention time)
#' @param y (numeric) y observed values (e.g. spectra intensity)
#'
#' @return A list of guessed starting parameters \code{list()$init_params},
#' lower \code{list()$lower_bounds} and upper bounds \code{list()$upper_bounds}
#' (\code{$gamma}, \code{$center}, \code{$sigma}, \code{$amplitude})
emgGaussian_guess <- function(x, y) {
    # set center as x position of max y value (e.g. highest spectra intensity)
    center_guess <- x[which.max(y)]
    # init_param
    init_params <- list(amplitude=1e+07, center=center_guess, sigma=1, gamma=1)
    # lower_bounds
    lower_bounds <- list(amplitude=0, center=center_guess-3, sigma=0,gamma=-0.1)
    # upper_bounds
    upper_bounds <- list(amplitude=1e+09, center=center_guess+3, sigma=5,
        gamma = 5)

    return(list(init_params = init_params, lower_bounds = lower_bounds,
                upper_bounds = upper_bounds))
}

context('fitCurve()')

## Input and expected data
# Use EIC aggregated (sum) from raw data extracted from file ko15.CDf from the pkg faahKO with rt=c(3362.102, 3409.051), mz=c(496., 498.), msLevel=1
EIC_rt  <- c(3362.102, 3363.667, 3365.232, 3366.797, 3368.362, 3369.927, 3371.492, 3373.057, 3374.622, 3376.187, 3377.752, 3379.317, 3380.882, 3382.447, 3384.012, 3385.577, 3387.142, 3388.707, 3390.272, 3391.837, 3393.402, 3394.966, 3396.531, 3398.096, 3399.661, 3401.226, 3402.791, 3404.356, 3405.921, 3407.486, 3409.051)
EIC_int <- c(51048, 81568, 138288, 233920, 376448, 557288, 753216, 938048, 1091840, 1196992, 1261056, 1308992, 1362752, 1406592, 1431360, 1432896, 1407808, 1345344, 1268480, 1198592, 1126848, 1036544, 937600, 849792, 771456, 692416, 614528, 546088, 492752, 446464, 400632)
EIC     <- data.frame(rt=EIC_rt, int=EIC_int)

input_params  <- list( init_params  = list(amplitude=1E7, center=3400, sigma=1, gamma=1),
                       lower_bounds = list(amplitude=0,   center=3397, sigma=0, gamma=-0.1),
                       upper_bounds = list(amplitude=1E9, center=3403, sigma=5, gamma=5))

test_that('fit skewedGaussian, guess params, no lower, no upper', {
  # Input and expected results
  expected_fit        <- list(amplitude=275371.05624872464, center=3382.577, sigma=0.079046969170787212, gamma=0.0011476465885767637, fitStatus=2, curveModel="skewedGaussian")
  class(expected_fit) <- 'peakPantheR_curveFit'
  
  result_fit    <- fitCurve(x=EIC$rt, y=EIC$int, curveModel='skewedGaussian', params='guess')
  
  # Check results
  expect_equal(result_fit, expected_fit)
})

test_that('fit emgGaussian, guess params, no lower, no upper', {
  # Input and expected results
  expected_fit        <- list(amplitude=33686977, center=3382.577, sigma=5, gamma=0.1126386, fitStatus=2, curveModel="emgGaussian")
  class(expected_fit) <- 'peakPantheR_curveFit'

  result_fit    <- fitCurve(x=EIC$rt, y=EIC$int, curveModel='emgGaussian', params='guess')

  # Check results
  expect_equal(result_fit, expected_fit, tolerance=1e-5)
})


test_that('fit skewedGaussian, input params', {
  # Input and expected results
  expected_fit        <- list(amplitude=13203571.527336178, center=3403, sigma=5, gamma=-0.1, fitStatus=1, curveModel="skewedGaussian")
  class(expected_fit) <- 'peakPantheR_curveFit'
  
  result_fit    <- fitCurve(x=EIC$rt, y=EIC$int, curveModel='skewedGaussian', params=input_params)
  
  # Check results
  expect_equal(result_fit, expected_fit)
})

# test_that('fit emgGaussian, input params', {
#   # Input and expected results
#   expected_fit        <- list(amplitude=12968279, center=3398.492, sigma=5, gamma=1.339195, fitStatus=2, curveModel="emgGaussian")
#   class(expected_fit) <- 'peakPantheR_curveFit'

#   result_fit    <- fitCurve(x=EIC$rt, y=EIC$int, curveModel='emgGaussian', params=input_params)

#   # Check results
#   expect_equal(result_fit, expected_fit, tolerance=1e-6)
# })


test_that('raises error()', {
  # length x y doesn't match
  expect_error(fitCurve(x=EIC$rt, y=EIC$int[1:20], curveModel='skewedGaussian', params='guess'), 'Error: length of "x" and "y" must match!', fixed=TRUE)

  # unknown curveModel
  expect_error(fitCurve(x=EIC$rt, y=EIC$int, curveModel='unknownModel', params='guess'), 'Error: "curveModel" must be one of: skewedGaussian', fixed=TRUE)
  
  # params is not a list or character
  expect_error(fitCurve(x=EIC$rt, y=EIC$int, curveModel='skewedGaussian', params=5), 'Error: "params" must be a list or "guess"', fixed=TRUE)
  
  # no init_params
  tmp_params1 <- list(lower_bounds = input_params$lower_bounds,
                      upper_bounds = input_params$upper_bounds)
  expect_error(fitCurve(x=EIC$rt, y=EIC$int, curveModel='skewedGaussian', params=tmp_params1), 'Error: "params must be a list of "init_params", "lower_bounds" and "upper_bounds"', fixed=TRUE)
  
  # no lower_bounds
  tmp_params2 <- list(init_params  = input_params$init_params,
                      upper_bounds = input_params$upper_bounds)
  expect_error(fitCurve(x=EIC$rt, y=EIC$int, curveModel='skewedGaussian', params=tmp_params2), 'Error: "params must be a list of "init_params", "lower_bounds" and "upper_bounds"', fixed=TRUE)
  
  # no upper_bounds
  tmp_params3 <- list(init_params  = input_params$init_params,
                      lower_bounds = input_params$lower_bounds)
  expect_error(fitCurve(x=EIC$rt, y=EIC$int, curveModel='skewedGaussian', params=tmp_params3), 'Error: "params must be a list of "init_params", "lower_bounds" and "upper_bounds"', fixed=TRUE)
  
  # init_params not a list
  tmp_params4 <- list(init_params  = 'not a list',
                      lower_bounds = input_params$lower_bounds,
                      upper_bounds = input_params$upper_bounds)
  expect_error(fitCurve(x=EIC$rt, y=EIC$int, curveModel='skewedGaussian', params=tmp_params4), 'Error: "params$init_params" must be a list of parameters', fixed=TRUE)
  
  # lower_params not a list
  tmp_params5 <- list(init_params  = input_params$init_params,
                      lower_bounds = 'not a list',
                      upper_bounds = input_params$upper_bounds)
  expect_error(fitCurve(x=EIC$rt, y=EIC$int, curveModel='skewedGaussian', params=tmp_params5), 'Error: "params$lower_bounds" must be a list of parameters', fixed=TRUE)
  
  # upper_params not a list
  tmp_params6 <- list(init_params  = input_params$init_params,
                      lower_bounds = input_params$lower_bounds,
                      upper_bounds = 'not a list')
  expect_error(fitCurve(x=EIC$rt, y=EIC$int, curveModel='skewedGaussian', params=tmp_params6), 'Error: "params$upper_bounds" must be a list of parameters', fixed=TRUE)
})

## Diagnostic Tab --------------------------------------------------------------

# - message if no import (conditional panel)
# - status of the annotation (and failures) on the sidebar

# - [TAB] fitting summary (%tage found, %tage filled)
# - [TAB] automatic update uROI/FIR
# - [TAB] show plots
# - [TAB] show updated parameters + modify ?

## not annotated
output$notAnnotForDiagUI <- renderUI ({
  if(runSuccess()=='yes') return()
  tagList(
    HTML("<div class=\"alert alert-dismissible alert-danger\"> <button type=\"button\" class=\"close\" data-dismiss=\"alert\">×</button> <h4 style=\"font-weight:bold\">No annotation computed</h4>Annotate a <i>peakPantheRAnnotation</i></div>")#,
    #includeHTML("data/aboutDiagnotic.html")
  )
})


## show the status of the peakPantheRAnnotation (need to be separated from the previous panel so they can be hidden)
output$showAnnotStatusDiag <- renderUI({
  # Capture the annotation shown and split by line into a list
  tmp_text  <- peakPantheR::annotation_showText_UI_helper(peakPantheR::annotation_showMethod_UI_helper(values$annotation))
  fail_text <- paste(dim(values$failures)[1], 'annotation failure(s)')
  # render the panel
  wellPanel(
    h4('Status:', style="color:#3e648d;font-weight:bold"),
    helpText(tmp_text[[1]],style="color:black"),
    helpText(tmp_text[[2]],style="color:black"),
    helpText(tmp_text[[3]],style="color:black"),
    helpText(tmp_text[[4]],style="color:black"),
    helpText(tmp_text[[5]],style="color:black"),
    helpText(tmp_text[[6]],style="color:black"),
    tags$hr(),
    if (!is.null(values$failures)) {
      helpText(fail_text,style="color:black")
    }
  )
})


## Annotation fitting statistics------------------------------------------------
# render annotation fit statistic table
output$table_fit_stat <- DT::renderDT ({
  DT::datatable(data = annotation_fit_summary_UI_helper(values$annotation),
                options  = list(orderClasses = TRUE),
                rownames = TRUE)
})

# UI block annotation fit statistic table
output$annotationStatisticsTable <- renderUI ({
  fluidRow(
    column(width = 12, offset = 0,
      DT::DTOutput("table_fit_stat")
    )
  )
})


## Automatic update uROI/FIR ---------------------------------------------------
## Run the update to uROI/FIR
observeEvent(input$goDiagnosticUpdateUROIFIR, {
  updated_annotation  <- peakPantheR::annotationParamsDiagnostic(values$annotation, verbose=TRUE)
  values$annotation <- updated_annotation
})

## Check update uROI/FIR is a success
diagSuccess <- reactive({
  # Diagnostic not triggered yet
  if(input$goDiagnosticUpdateUROIFIR == 0) {
    return('no')
  } else {
    # Diagnostic triggered, no absolute certainty of result, but uROIExist will have been set by it
    if(peakPantheR::isAnnotated(values$annotation) & peakPantheR::uROIExist(values$annotation)) {
      return('yes')
    # something is wrong
    } else {
      return('no')
    }
  }
})

# Success update UROI/FIR using diagnostic message
output$successUpdateDiagUI <- renderUI({
  if(diagSuccess()=='yes') {
    tagList(
      HTML("<div class=\"alert alert-dismissible alert-success\"><button type=\"button\" class=\"close\" data-dismiss=\"alert\">×</button><h4 style=\"font-weight:bold\">Success</h4>uROI and FIR updated sucessfully</div>")
    )
  }
})


## Plot Diagnostic -------------------------------------------------------------
# UI to control the diagnostic plot
output$diagPlotControlUI <- renderUI({
  wellPanel(
    fluidRow(
      column(12, offset=1,
        h4('Fit diagnostic plots', style="color:#3e648d;font-weight:bold"),
      )  # end column
    ),   # end fluidRow
    fluidRow(
      column(3, offset=0,
        selectInput("plotFeatDiag", label="Feature", choices=unname(values$featNmeList))
      ), # end column
      column(3, offset=0,
        selectInput("plotMetaSplColr", label="Colour by Sample Metadata", choices=values$spectraMetadataCol)
      ), # end column
      column(3, offset=0,
        numericInput("plotSplNumber", label = paste0("Number of Samples to show (out of ",
                                                     nbSamples(values$annotation), ")"),
                     value = nbSamples(values$annotation),
                     min = 1,
                     max = nbSamples(values$annotation),
                     step = 1)
      ),# end column
      column(3, offset=0,
        numericInput("plotHeightDiag", label = "Plot Height", value = 400, min = 0, step = 1)
      ) # end column
    )   # end fluidRow
  )     # end wellPanel
})

# plot feature diagnostic
output$diagPlot <- renderPlot({
  # find the cpdNb corresponding to the cpdID + cpdName shown
  annotation_diagnostic_multiplot_UI_helper(
    cpdNb = as.numeric(names(values$featNmeList)[values$featNmeList == input$plotFeatDiag]),
    annotation = values$annotation,
    splNum = input$plotSplNumber,
    splColrColumn = input$plotMetaSplColr)
})

# UI block with plot feature diagnostic
output$diagPlotResultUI <- renderUI ({
  # must be annotated and a feature selected (make sure it doesn't throw an error for nothing)
  if (is.null(input$plotFeatDiag)) {
    h5('Nothing to plot!')
  } else if (peakPantheR::isAnnotated(values$annotation) & (as.numeric(names(values$featNmeList)[values$featNmeList == input$plotFeatDiag]) != 0) ){
    shinycssloaders::withSpinner(
      plotOutput("diagPlot", height=input$plotHeightDiag)
    )
  } else {
    h5('Nothing to plot!')
  }
})


## Show final parameters and modify --------------------------------------------
# TODO: tab for uROI, FIR show and manually tweak

#' @title Generate ion chromatogram from raw data points
#'
#' @description On the input raw data, aggregate intensites across the mz range
#' at each retention time to generate an ion chromatogram: \code{sum} for
#' EIC/TIC, \code{max}, \code{min} or \code{mean}. The number of data points
#' returned correspond to the number of unique scans/retention time measurements
#' in the input data
#' @param ROIDataPoint (data.frame) retention time 'rt', mass 'mz' and intensity
#' 'int' (as column) of each raw data points (as row) to use for the ion
#' chromatogram
#' @param aggregationFunction (str) Function to use in order to aggregate
#' intensities across mz in each scan. One of \code{sum}, \code{max},
#' \code{min}, \code{mean}
#'
#' @return A data.frame of retention time 'rt' and aggregated intensities 'int'
#' 
#' @details
#' ## Examples cannot be computed as the function is not exported:
#' ## Input data points
#' in_rt <- c(3362.102, 3362.102, 3363.667, 3363.667, 3365.232, 3365.232,
#'             3366.797, 3366.797, 3368.362, 3368.362)
#' in_mz <- c(496.2, 497.2, 496.2, 497.2, 496.2, 497.2, 496.2, 497.2, 496.2,
#'             497.2)
#' in_int <- c(39616, 11432, 63344, 18224, 107352, 30936, 182144, 51776, 295232,
#'             81216)
#' input_ROIDataPoints <- data.frame(rt=in_rt, mz=in_mz, int=in_int)
#'
#' ## Aggregate mz to generate EIC
#' EIC <- generateIonChromatogram(input_ROIDataPoints,aggregationFunction='sum')
#' EIC
#' #         rt    int
#' # 1 3362.102  51048
#' # 2 3363.667  81568
#' # 3 3365.232 138288
#' # 4 3366.797 233920
#' # 5 3368.362 376448
generateIonChromatogram <- function(ROIDataPoint, aggregationFunction = "sum") {
    
    # Check input
    if (!is(ROIDataPoint, "data.frame")) {
        stop("Check input \"ROIDataPoint\" must be a data.frame")
    } else if (!all(c("rt", "mz", "int") %in% colnames(ROIDataPoint))) {
        stop("Check input \"ROIDataPoint must have the following columns: ",
            "\"rt\", \"mz\", \"int\"")
    }
    if (!(aggregationFunction) %in% c("sum", "mean", "min", "max")) {
        stop("Check input \"aggregationFunction\" must be one of: \"sum\", ",
            "\"mean\", \"min\", \"max\"")
    }
    
    # No data to aggregate
    if (dim(ROIDataPoint)[1] == 0) {
        return(data.frame(rt = numeric(), int = numeric()))
    }
    
    # Init
    unique_scans <- sort(unique(ROIDataPoint$rt))
    res <- data.frame(matrix(vector(), nrow = length(unique_scans), ncol = 2,
                                dimnames = list(c(), c("rt", "int"))))
    
    # for each scan, aggregate intensity
    for (i in seq_len(length(unique_scans))) {
        cur_scan    <- unique_scans[i]
        tmp_int     <- ROIDataPoint[ROIDataPoint$rt == cur_scan, c("int")]
        res[i, ]    <- c(cur_scan, do.call(aggregationFunction, list(tmp_int)))
    }
    
    return(res)
}

# server.R

# peakPantheR-App
# Based on peakPantheR v1.15.0, R >= 4.3, shiny >= 1.0.5, bslib
# National Phenome Centre
# 02/10/2023
# Licensed under GPLv3	
#
# Copyright (C) {2023}  {National Phenome Centre}
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#require(shiny)


# increase upload size to 30Gb
options(shiny.maxRequestSize=30000*1024^2)
# define max number of parallel cores
maxCores <- parallel::detectCores()
# known curve fitting methods
knownCurveModel <- c("skewedGaussian", "emgGaussian")


shinyServer( function(input, output, session){

  # -- General Initialisation --

  # close app if tab is shut
  session$onSessionEnded(stopApp)

  # get peakPantheR version
  output$peakPantheR_ver <- renderText({ paste("peakPantheR v",packageVersion('peakPantheR'),sep="") })

  # get max number of cpu cores
  output$cpu <- renderText({ maxCores })
  
  
  # -- Import Tab --
  source(file.path("server", "server_import.R"),  local = TRUE)$value

  # -- Run Tab --
  source(file.path("server", "server_run.R"),  local = TRUE)$value

  # -- Diagnostic Tab --
  source(file.path("server", "server_diagnostic.R"),  local = TRUE)$value

  # -- Results Tab --
  source(file.path("server", "server_results.R"),  local = TRUE)$value

  # -- Export Tab --
  source(file.path("server", "server_export.R"),  local = TRUE)$value


  # force update of conditional statement triggers and "generate Input" buttons even if not yet open
  outputOptions(output, "importDone", suspendWhenHidden=FALSE)
  outputOptions(output, "AnnotationDone", suspendWhenHidden=FALSE)
})

#' @title Save to disk a plot of all ROI EIC and detected feature range
#'
#' @description Plot and save a \code{.png} of all ROI (x is RT, y is
#' intensity), with the matching detected peak rt and peakwidth under it.
#'
#' @param ROIsDataPoint (list) a list of \code{data.frame} of raw data points
#' for each ROI (retention time 'rt', mass 'mz' and intensity 'int' (as column)
#' of each raw data points (as row)).
#' @param curveFit (list) a list of \code{peakPantheR_curveFit} or NA for each
#' ROI
#' @param foundPeakTable (data.frame) \code{data.frame} as generated by
#' \link{findTargetFeatures}, with features as rows and peak properties as
#' columns. The following columns are mandatory: \code{cpdID}, \code{cpdName},
#' \code{rt}, \code{rtmin}, \code{rtmax}, \code{mzmin}, \code{mzmax}.
#' @param savePath (str) Full path to save a \emph{.png} of all ROI EICs,
#' expect \code{'filepath/filename.png'}.
#' @param width (float) Width in cm for a single ROI plot (if more than one plot
#' in total, 2 columns will be used). dpi set to a 100.
#' @param height (float) height in a cm for a single ROI plot. dpi set to 100
#' @param verbose (bool) if TRUE message progress
#'
#' @return None
saveSingleFileMultiEIC <- function(ROIsDataPoint, curveFit, foundPeakTable,
    savePath, width = 15, height = 15, verbose = TRUE) {

    # check input
    nbROI <- length(ROIsDataPoint)
    if (nbROI != dim(foundPeakTable)[1]) {
        stop("Number of ROI datapoints in \"ROIsDataPoint\" (", nbROI,
            ") and features in \"foundPeakTable\" (",
            dim(foundPeakTable)[1], ") do not match!")
    }
    if (nbROI != length(curveFit)) {
        stop("Number of ROI datapoints in \"ROIsDataPoint\" (", nbROI,
            ") and fitted curves in \"curveFit\" (",
            length(curveFit), ") do not match!")
    }
    # Generate each ROI plot
    p_all <- vector("list", nbROI)
    for (i in seq_len(nbROI)) {
        p_all[[i]] <- plotEICDetectedPeakwidth(
            ROIDataPointSampleList = list(ROIsDataPoint[[i]]),
            cpdID = foundPeakTable$cpdID[i],
            cpdName = foundPeakTable$cpdName[i],
            rt = foundPeakTable$rt[i],
            rtMin = foundPeakTable$rtMin[i], rtMax = foundPeakTable$rtMax[i],
            mzMin = foundPeakTable$mzMin[i], mzMax = foundPeakTable$mzMax[i],
            ratio = 0.85,
            curveFitSampleList = list(curveFit[[i]]),
            sampling = 250,
            sampleColour = "black",
            verbose = verbose)
    }
    # Set save parameters
    targetFolder <- dirname(savePath)
    targetFile <- basename(savePath)
    nSubplot <- length(p_all)
    if (nSubplot == 1) {
        ncol <- 1
    } else {
        ncol <- 2
        nrow <- ceiling(nSubplot/2)
        width <- width * ncol
        height <- height * nrow
    }
    # Save
    ggplot2::ggsave(file = targetFile,
        plot = gridExtra::arrangeGrob(grobs = p_all, ncol = ncol),
        device = "png", path = targetFolder, dpi = 100, width = width,
        height = height, units = "cm", limitsize = FALSE)
    if (verbose) { message("Summary EIC plot saved at: ", savePath) }
}

context('saveMultiEIC()')

# remove Rplots.pdf created by ggplot2
on.exit( tryCatch({ file.remove('./Rplots.pdf') }, error=function(e){ invisible() }, warning=function(w){ invisible() }) )

## Input and expected data
# fake ROI 1
# ROI data points
rt1  <- seq(990, 1010, by=20/250)
mz1  <- rep(522.2, length(rt1))
int1 <- (dnorm(rt1, mean=1000, sd=1.5) * 100) + 1
tmp_DataPoints1  <- data.frame(rt=rt1, mz=mz1, int=int1)
# fittedCurve
fit1        <- list(amplitude=37.068916502809756, center=999.3734222573454, sigma=0.58493182568124724, gamma=0.090582029276037035, fitStatus=2, curveModel="skewedGaussian")
class(fit1) <- 'peakPantheR_curveFit'
# fake ROI 2
# ROI data points
rt2  <- seq(990, 1010, by=20/250)
mz2  <- rep(464.2, length(rt2))
int2 <- (dnorm(rt2, mean=1002, sd=1.5) * 100) + 1
tmp_DataPoints2  <- data.frame(rt=rt2, mz=mz2, int=int2)
# fittedCurve
fit2        <- list(amplitude=37.073067416755556, center=1001.3736564832565, sigma=0.58496485738212201, gamma=0.090553713725151905, fitStatus=2, curveModel="skewedGaussian")
class(fit2) <- 'peakPantheR_curveFit'
# ROIsDataPoints
input_ROIsDataPoints  <- list(tmp_DataPoints1, tmp_DataPoints2)
# curveFit
input_curveFit        <- list(fit1, fit2)

# peakTable
input_peakTable     <- data.frame(matrix(vector(), 2, 10, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "maxIntMeasured", "maxIntPredicted"))),stringsAsFactors=FALSE)
input_peakTable[1,] <- c(TRUE, 995., 1000., 1005., 522.19, 522.2, 522.21, 20000, 21, 21)
input_peakTable[2,] <- c(TRUE, 997., 1002., 1007., 464.19, 464.2, 464.21, 20000, 21, 21)
input_peakTable[,1] <- sapply(input_peakTable[,c(1)], as.logical)


test_that('default parameters, verbose', {
  # temporary file
  savePath1  <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.png')

	# results (output, warnings and messages)
  result_plot <- evaluate_promise(saveSingleFileMultiEIC(ROIsDataPoint=input_ROIsDataPoints, curveFit=input_curveFit, foundPeakTable=input_peakTable, savePath=savePath1, width=15, height=15, verbose=TRUE))

  # Check plot has been produced
  expect_true(file.exists(savePath1))

  # Check result messages (save path)
  expect_equal(length(result_plot$messages), 1)
})

test_that('default parameters, no verbose', {
  # temporary file
  savePath2  <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.png')

  # results (output, warnings and messages)
  result_plot <- evaluate_promise(saveSingleFileMultiEIC(ROIsDataPoint=input_ROIsDataPoints, curveFit=input_curveFit, foundPeakTable=input_peakTable, savePath=savePath2, width=15, height=15, verbose=FALSE))

  # Check plot has been produced
  expect_true(file.exists(savePath2))

  # Check no result messages
  expect_equal(length(result_plot$messages), 0)
})

test_that('only one plot, no verbose', {
  # temporary file
  savePath3  <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.png')

  # results (output, warnings and messages)
  result_plot <- evaluate_promise(saveSingleFileMultiEIC(ROIsDataPoint=list(input_ROIsDataPoints[[1]]), curveFit=list(input_curveFit[[1]]), foundPeakTable=input_peakTable[1,], savePath=savePath3, width=15, height=15, verbose=FALSE))

  # Check plot has been produced
  expect_true(file.exists(savePath3))

  # Check no result messages
  expect_equal(length(result_plot$messages), 0)
})

test_that('raise errors', {
  savePath4  <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.png')
  
  # ROIsDataPoint and foundPeakTable dimension mismatch
  msg1 <- 'Number of ROI datapoints in "ROIsDataPoint" (1) and features in "foundPeakTable" (2) do not match!'
  expect_error(saveSingleFileMultiEIC(ROIsDataPoint=list(input_ROIsDataPoints[[1]]), curveFit=list(input_curveFit[[1]]), foundPeakTable=input_peakTable, savePath=savePath4, width=15, height=15, verbose=TRUE), msg1, fixed=TRUE)
  
  # ROIsDataPoint and curveFit dimension mismatch
  msg2 <- 'Number of ROI datapoints in "ROIsDataPoint" (1) and fitted curves in "curveFit" (2) do not match!'
  expect_error(saveSingleFileMultiEIC(ROIsDataPoint=list(input_ROIsDataPoints[[1]]), curveFit=input_curveFit, foundPeakTable=input_peakTable[1,], savePath=savePath4, width=15, height=15, verbose=TRUE), msg2, fixed=TRUE)
})

#' @title Find and integrate target features in each ROI
#' 
#' @description For each ROI, fit a curve and integrate the largest feature in
#' the box. Each entry in \code{ROIsDataPoints} must match the corresponding row
#' in \code{ROI}. The curve shape to employ for fitting can be changed with
#' \code{curveModel} while fitting parameters can be changed with \code{params}
#' (list with one param per ROI window). \code{rtMin} and \code{rtMax} are
#' established at 0.5% of apex intensity using a moving window from the apex
#' outward (the window is the ROI width); if after 8 iterations \code{rtMin} or
#' \code{rtMax} is not found, NA is returned and the peak fit rejected.
#' \code{peakArea} is calculated from \code{rtMin} to \code{rtMax}.
#' \code{peakAreaRaw} is calculated from \code{rtMin} to \code{rtMax} but
#' using the raw data points instead of the modelled line-shape.
#' \code{mz} is the weighted (by intensity)
#' average mz of datapoints falling into the \code{rtMin} to \code{rtMax}
#' range, \code{mzMin} and \code{mzMax} are the
#' minimum and maxmimum mass in these range. If \code{rtMin} or \code{rtMax}
#' falls outside of ROI (extracted scans), \code{mzMin} or \code{mzMax} are
#' returned as the input ROI limits and \code{mz} is an approximation on the
#' datapoints available (if no scan of the ROI fall between rtMin/rtMax, mz
#' would be NA, the peak is rejected). If any of the two following ratio are
#' superior to \code{maxApexResidualRatio}, the fit is rejected: 1) ratio of fit
#' residuals at the apex (predicted apex fit intensity vs measured apex
#' intensity: fit overshoots the apex), 2) ratio of predicted apex fit intensity
#' vs maximum measured peak intensity (fit misses the real apex in the peak).
#'
#' @param ROIsDataPoints (list) A list (one entry per ROI window) of data.frame
#' with signal as row and retention time ('rt'), mass ('mz') and intensity
#' ('int) as columns. Must match each row of ROI.
#' @param ROI (data.frame) A data.frame of compounds to target as rows. Columns:
#' \code{rtMin} (float in seconds), \code{rtMax} (float in seconds),
#' \code{mzMin} (float), \code{mzMax} (float)
#' @param curveModel (str) Name of the curve model to fit (currently
#' \code{skewedGaussian} and \code{emgGaussian})
#' @param params (list or str) Either 'guess' for automated parametrisation or
#' list (one per ROI windows) of 'guess' or list of curve fit parameters
#' @param sampling (int) Number of points to employ when subsampling the
#' fittedCurve (rt, rtMin, rtMax, integral calculation)
#' @param maxApexResidualRatio (float) Ratio of maximum allowed fit residual at
#' the peak apex, compared to the fit max intensity. (e.g. 0.2 for a maximum
#' residual of 20\% of apex intensity)
#' @param verbose (bool) If TRUE message the time taken and number of features
#' found
#' @param ... Passes arguments to \code{fitCurve} to alter peak fitting
#' (\code{params})
#'
#' @return A list: \code{list()$peakTable} (\emph{data.frame}) with targeted
#' features as rows and peak measures as columns (see Details),
#' \code{list()$curveFit} (\emph{list}) a list of \code{peakPantheR_curveFit} or
#' NA for each ROI.
#'
#' \subsection{Details:}{
#' The returned \code{data.frame} is structured as follow:
#' \tabular{ll}{
#' found \tab was the peak found\cr
#' rt \tab retention time of peak apex (sec)\cr
#' rtMin \tab leading edge of peak retention time (sec) determined at 0.5\% of
#' apex intensity\cr
#' rtMax \tab trailing edge of peak retention time (sec) determined at 0.5\% of
#' apex intensity\cr
#' mz \tab weighted (by intensity) mean of peak m/z across scans\cr
#' mzMin \tab m/z peak minimum (between rtMin, rtMax)\cr
#' mzMax \tab m/z peak maximum (between rtMin, rtMax)\cr
#' peakArea \tab integrated peak area\cr
#' peakAreaRaw \tab integrated peak area from raw data points\cr
#' maxIntMeasured \tab maximum peak intensity in raw data\cr
#' maxIntPredicted \tab maximum peak intensity based on curve fit (at apex)\cr
#' }
#' }
#'
#' @details
#' ## Examples cannot be computed as the function is not exported:
#' ## Load data
#' library(faahKO)
#' library(MSnbase)
#' netcdfFilePath <- system.file('cdf/KO/ko15.CDF', package = 'faahKO')
#' raw_data <- MSnbase::readMSData(netcdfFilePath,centroided=TRUE,mode='onDisk')
#'
#' ## targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                         c('cpdID','cpdName','rtMin','rt','rtMax','mzMin',
#'                         'mz','mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,3:8] <- vapply(targetFeatTable[,3:8], as.numeric,
#'                         FUN.VALUE=numeric(2))
#'
#' ROIsPt <- extractSignalRawData(raw_data,
#'                                 rt=targetFeatTable[,c('rtMin','rtMax')],
#'                                 mz=targetFeatTable[,c('mzMin','mzMax')],
#'                                 verbose=TRUE)
#' # Reading data from 2 windows
#'
#' foundPeaks <- findTargetFeatures(ROIsPt, targetFeatTable, verbose=TRUE)
#' # Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
#' # mzMin/mzMax calculation,
#' # approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #1
#' # Found 2/2 features in 0.07 secs
#' 
#' foundPeaks
#' # $peakTable
#' #   found    rtMin       rt    rtMax    mzMin    mz    mzMax peakArea
#' # 1  TRUE 3309.759 3346.828 3385.410 522.1948 522.2 522.2052 26133727
#' # 2  TRUE 3345.377 3386.529 3428.279 496.2000 496.2 496.2000 35472141
#' #   peakAreaRaw maxIntMeasured maxIntPredicted
#' # 1    26071378         889280        901015.8
#' # 2    36498367        1128960       1113576.7
#' #
#' # $curveFit
#' # $curveFit[[1]]
#' # $amplitude
#' # [1] 162404.8
#' # 
#' # $center
#' # [1] 3341.888
#' # 
#' # $sigma
#' # [1] 0.07878613
#' # 
#' # $gamma
#' # [1] 0.00183361
#' # 
#' # $fitStatus
#' # [1] 2
#' # 
#' # $curveModel
#' # [1] 'skewedGaussian'
#' # 
#' # attr(,'class')
#' # [1] 'peakPantheR_curveFit'
#' # 
#' # $curveFit[[2]]
#' # $amplitude
#' # [1] 199249.1
#' # 
#' # $center
#' # [1] 3382.577
#' # 
#' # $sigma
#' # [1] 0.07490442
#' # 
#' # $gamma
#' # [1] 0.00114719
#' # 
#' # $fitStatus
#' # [1] 2
#' # 
#' # $curveModel
#' # [1] 'skewedGaussian'
#' # 
#' # attr(,'class')
#' # [1] 'peakPantheR_curveFit'
findTargetFeatures  <- function(ROIsDataPoints, ROI,
                                curveModel = "skewedGaussian", params = "guess",
                                sampling = 250, maxApexResidualRatio = 0.2,
                                verbose = FALSE,...) {
    stime <- Sys.time()
    nROI <- nrow(ROI)

    # Check inputs (ROIsDataPoints match ROI, inputs, length)
    findTargetFeatures_checkInput(ROIsDataPoints, ROI, params)
    
    # Init output
    opt <- findTargetFeatures_initOutput(ROI, params, verbose)
    outTable <- opt$outTable; outCurveFit <- opt$outCurveFit
    useParams <- opt$useParams
    
    # Fit each ROI
    for (i in seq_len(nROI)) {
        fitRes <- findTargetFeatures_fitFeature(i,ROI,ROIsDataPoints,curveModel,
                                                useParams, params, sampling,
                                                maxApexResidualRatio, verbose,
                                                ...)
        # NULL if no fit, pass to next ROI
        if (is.null(fitRes)) {
            next
        } else {
            # Set all values: curveFit
            outCurveFit[[i]]            <- fitRes$fittedCurve
            # peaktable
            outTable$found[i]           <- TRUE
            outTable$rt[i]              <- fitRes$rt
            outTable$rtMin[i]           <- fitRes$rtMin
            outTable$rtMax[i]           <- fitRes$rtMax
            outTable$mz[i]              <- fitRes$mz
            outTable$mzMin[i]           <- fitRes$mzMin
            outTable$mzMax[i]           <- fitRes$mzMax
            outTable$peakArea[i]        <- fitRes$peakArea
            outTable$peakAreaRaw[i]     <- fitRes$peakAreaRaw
            outTable$maxIntMeasured[i]  <- fitRes$maxIntMeasured
            outTable$maxIntPredicted[i] <- fitRes$maxIntPredicted
        }
    }

    # Output
    etime <- Sys.time()
    if (verbose) {
        message("Found ", sum(outTable$found), "/", nROI, " features in ",
                round(as.double(difftime(etime, stime)), 2), " ",
                units(difftime(etime, stime))) }
    return(list(peakTable = outTable, curveFit = outCurveFit))
}


# -----------------------------------------------------------------------------
# findTargetFeatures helper functions

## Check inputs (ROIsDataPoints match ROI, inputs, length)
findTargetFeatures_checkInput <- function(ROIsDataPoints, ROI, params) {
    nROI <- nrow(ROI)
    # ROIsDataPoints match ROI
    if (length(ROIsDataPoints) != nROI) {
        stop('Check input, number of ROIsDataPoints entries must match ',
            'the number of rows of ROI')
    }
    # Check all data points fall into the corresponding ROI
    for (r in seq_len(nROI)) {
        if (!all((ROIsDataPoints[[r]]$rt >= ROI[r, c("rtMin")]) &
            (ROIsDataPoints[[r]]$rt <= ROI[r, c("rtMax")]))) {
            stop("Check input not all datapoints for window #", r,
                " are into the corresponding ROI (rt)")
        }
        if (!all((ROIsDataPoints[[r]]$mz >= ROI[r, c("mzMin")]) &
            (ROIsDataPoints[[r]]$mz <= ROI[r, c("mzMax")]))) {
            stop("Check input not all datapoints for window #", r,
                " are into the corresponding ROI (mz)")
        }
    }
    # Check params input
    if (!(is.character(params) | is.list(params))) {
        stop("Check input, \"params\" must be \"guess\" or list")}
    # params is 'guess' if character
    if (is.character(params)) {
        if (params != "guess") {
            stop("Check input, \"params\" must be \"guess\" if not list")}
    }
    # length if params is list
    if (is.list(params)) {
        if (length(params) != nROI) {
            stop('Check input, number of parameters must match number ',
                'of rows of ROI')}
    }
}


## Init output
findTargetFeatures_initOutput <- function(ROI, params, verbose) {
    nROI <- nrow(ROI)

    outTable <- data.frame(matrix(vector(), nROI, 11, dimnames = list(c(),
        c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea",
        "peakAreaRaw", "maxIntMeasured", "maxIntPredicted"))),
    stringsAsFactors = FALSE)
    outTable$found  <- rep(FALSE, nROI)  # set found to FALSE
    outCurveFit     <- rep(list(NA), nROI)

    # use input params or guess
    if (any(params != "guess")) {
        useParams <- TRUE
        if (verbose) {
            message("Curve fitting parameters passed as input employed")}
    } else {
        useParams <- FALSE
    }
    return(list(outTable=outTable, outCurveFit=outCurveFit,
                useParams=useParams))
}


## Fit single feature
# fit feature i, returns NULL to pass to the next feature (empty df row
# already initialised)
findTargetFeatures_fitFeature <- function(i, ROI, ROIsDataPoints, curveModel,
                                            useParams, params, sampling,
                                            maxApexResidualRatio, verbose, ...){
    # set params for fitting
    new_params <- "guess"
    if (useParams) { new_params <- params[[i]] }
    # extract EIC to fit
    tmp_EIC <- generateIonChromatogram(ROIDataPoint = ROIsDataPoints[[i]],
                                        aggregationFunction = "sum")

    # fit curve to EIC, in case of failure move to next window
    fittedCurve <- findTargetFeatures_fitcurve(i, tmp_EIC, curveModel,
                                                new_params, verbose, ...)
    if (is.null(fittedCurve)){ return(NULL) }

    # find rt, rtMin, rtMax and maxIntPredicted for a fitted curve
    rtRes <- findTargetFeatures_findRTproperties(i, ROI, tmp_EIC, fittedCurve,
                                                sampling, verbose)
    if (is.null(rtRes)){ return(NULL) }
    rt <- rtRes$rt; rtMin <- rtRes$rtMin; rtMax <- rtRes$rtMax
    maxIntPredicted <- rtRes$maxIntPredicted

    # maxIntMeasured (max raw data intensity; 0 in case of no scans)
    maxIntMeasured <- max(0, tmp_EIC$int[(tmp_EIC$rt < rtMax) &
                            (tmp_EIC$rt > rtMin)])

    # find mz, mzMin and mzMax
    mzRes <- findTargetFeatures_findMZ(i,ROI,rtMin,rtMax,ROIsDataPoints,verbose)
    if (is.null(mzRes)){ # move to next window
        return(NULL) }
    mz <- mzRes$mz; mzMin <- mzRes$mzMin; mzMax <- mzRes$mzMax

    # integrate curve
    peakArea <- findTargetFeatures_integCurve(fittedCurve,rtMin,rtMax,sampling)
    # integrate curve - raw datapoints
    rtBound <- (tmp_EIC$rt < rtMax) & (tmp_EIC$rt > rtMin)
    peakAreaRaw <- pracma::trapz(x=tmp_EIC$rt[rtBound], y=tmp_EIC$int[rtBound])

    # Check quality of fit (residuals at apex,and max predicted to max measured)
    qcRes <- findTargetFeatures_fitQuality(i, tmp_EIC, rt, maxIntPredicted,
                                maxIntMeasured, maxApexResidualRatio, verbose)
    if (!qcRes) { return(NULL) }

    return(list(fittedCurve=fittedCurve, rt=rt, rtMin=as.numeric(rtMin),
                rtMax=as.numeric(rtMax), mz=mz, mzMin=mzMin, mzMax=mzMax,
                peakArea=peakArea, peakAreaRaw=peakAreaRaw,
                maxIntMeasured=maxIntMeasured, maxIntPredicted=maxIntPredicted))
}
## Other EIC extraction ----
# resample to a standardised grid of size the maximum between
# 2*sampling and raw number of data points (double sampling used for
# integration)
# rawData_EIC <- generateIonChromatogram(ROIDataPoint=
# ROIsDataPoints[[i]], aggregationFunction='sum')
# tmp_gridMin <- min(rawData_EIC$rt)
# tmp_gridMax <- max(rawData_EIC$rt)
# tmp_gridSampling <- max(2*sampling, dim(rawData_EIC)[1])
# grid_rt_EIC <- seq(from=tmp_gridMin, to=tmp_gridMax,
# by=((tmp_gridMax-tmp_gridMin)/(tmp_gridSampling-1)))
# raw_approx_fun <- stats::approxfun(x=rawData_EIC$rt,
#                                      y=rawData_EIC$int)
# tmp_EIC <- data.frame(rt=grid_rt_EIC,
#                          int=raw_approx_fun(grid_rt_EIC))


## fit curve to EIC
findTargetFeatures_fitcurve <- function(i, tmp_EIC, curveModel,
                                            new_params, verbose, ...){
    # fit curve to EIC
    # return fittedCurve. If failure return NULL to move to next window
    fittedCurve <- tryCatch({
        ## try
        fittedCurve <- fitCurve(x = tmp_EIC$rt, y = tmp_EIC$int,
                                curveModel = curveModel, params = new_params,
                                ...)
    }, error = function(cond) {
        ## catch
        return(NA)
    })
    # catch fit failure
    if (all(is.na(fittedCurve))) {
        if (verbose) {
            message("Fit of ROI #", i, " is unsuccessful (try err)")}
        # indicate failure
        return(NULL)
    }
    # discard fit if nls.lm fit status indicates unsuccessful completion
    if ((fittedCurve$fitStatus == 0) | (fittedCurve$fitStatus == 5) |
        (fittedCurve$fitStatus == -1)) {
        if (verbose) {
            message("Fit of ROI #", i, " is unsuccessful (fit status)")}
        # indicate failure
        return(NULL)
    }
    return(fittedCurve)
}


## find rt, rtMin, rtMax and maxIntPredicted for a fitted curve
findTargetFeatures_findRTproperties <- function(i, ROI, tmp_EIC, fittedCurve,
                                                sampling, verbose){
    # If failure return NULL to move to next window

    # rt (search on same bounds as peak fit +/-3s)
    rt_EICmax   <- tmp_EIC$rt[which.max(tmp_EIC$int)]
    grid_rt     <- seq(from = rt_EICmax - 3, to = rt_EICmax + 3,
                        by = (6/(sampling - 1)))
    close_apex_int  <- predictCurve(fittedCurve, x = grid_rt)
    rt              <- grid_rt[which.max(close_apex_int)]

    # maxIntPredicted (fit apex intensity)
    maxIntPredicted <- predictCurve(fittedCurve = fittedCurve, x = rt)

    # rtMin, rtMax (look for 0.5% from max int, by rolling away from apex
    # until match or too many iterations)
    rtMin <- findTargetFeatures_findRTMinMax(min=TRUE, i, ROI, fittedCurve, rt,
                                            maxIntPredicted, sampling, verbose)
    rtMax <- findTargetFeatures_findRTMinMax(min=FALSE, i, ROI, fittedCurve, rt,
                                            maxIntPredicted, sampling, verbose)

    # if rtMin or rtMax cannot be determined the fit is not successful
    if (is.na(rtMin) | is.na(rtMax)) {
        message("Fit of ROI #", i,
                " is unsuccessful (cannot determine rtMin/rtMax)")
        # indicate failure
        return(NULL)
    }
    return(list(rt = rt, rtMin = rtMin, rtMax = rtMax,
                maxIntPredicted = maxIntPredicted))
}

## find rtMin or rtMax for a fitted curve
findTargetFeatures_findRTMinMax <- function(min, i, ROI, fittedCurve, rt,
                                        maxIntPredicted, sampling, verbose) {
    # rtMin/Max (look for 0.5% from max int, by rolling away from apex
    # until match or too many iterations)
    peakLim_int <- 0.005 * maxIntPredicted
    deltaRt     <- ROI$rtMax[i] - ROI$rtMin[i]

    rtMinMax    <- as.numeric(NA)
    cntr        <- 0
    if (min) { boxMin <- rt } else { boxMax <- rt } # Up / Down slope init

    # search rtMin/Max
    while (is.na(rtMinMax) & cntr <= 8) {
        cntr <- cntr + 1

        if (min) {
            # box moves earlier in rt each time
            boxMax  <- boxMin
            boxMin  <- boxMax - deltaRt
            grid_rt <- seq(from = boxMax, to = boxMin,
                            by = ((boxMin - boxMax)/(sampling - 1)))
            # reverse order for up slope
        } else {
            # box moves later in rt each time
            boxMin  <- boxMax
            boxMax  <- boxMin + deltaRt
            grid_rt <- seq(from = boxMin, to = boxMax,
                            by = ((boxMax - boxMin)/(sampling - 1)))
        }

        slope_int <- predictCurve(fittedCurve, x = grid_rt)
        cutoff_pt <- match(-1, sign(slope_int - peakLim_int))
            # pos of 1st point past cutoff
        if (is.na(cutoff_pt)) {
            rtMinMax <- as.numeric(NA)
            next
        }
        key_pt  <- c(cutoff_pt - 1, cutoff_pt)
            # points left and right from rtMin/Max
        rtMinMax <- stats::approx(x = slope_int[key_pt], y = grid_rt[key_pt],
                                xout = peakLim_int)$y
            # linear interpolation of exact rt
    }
    if (is.na(rtMinMax) & verbose) {
        if (min) { txt <- 'rtMin' } else { txt <- 'rtMax' }
        message("War","ning: ", txt, " cannot be determined for ROI #", i)}

    return(rtMinMax)
}

## find mz, mzMin and mzMax at a given rtMin rtMax
# If failure return NULL to move to next window
findTargetFeatures_findMZ <- function(i, ROI, rtMin, rtMax, ROIsDataPoints,
                                        verbose){
    # mz, mzMin, mzMax: if rtMin, rtMax are outside of ROI, we cannot calculate
    # mzMin, mzMax and mz precisely: default to ROI$mzMIn, ROI$mzMax as a safe
    # choice, and approximate mz
    mzMin <- ROI$mzMin[i]; mzMax <- ROI$mzMax[i]; ROIData <- ROIsDataPoints[[i]]
    isValid <- TRUE

    # deal with rtMin rtMax outside of ROI (warning)
    if ((rtMin < ROI$rtMin[i]) | (rtMax > ROI$rtMax[i])) {
        isValid <- FALSE
        if (verbose) {
            message('War','ning: rtMin/rtMax outside of ROI; datapoints ',
                    'cannot be used for mzMin/mzMax calculation, ',
                    'approximate mz and returning ROI$mzMin and ROI$mzMax ',
                    'for ROI #', i)
    }}

    # subset datapoints mz to rtMin/rtMax range
    tmpPt <- ROIData[(ROIData$rt > rtMin) & (ROIData$rt < rtMax), ]

    # rtMin rtMax range can be used for mzMin mzMax calculation
    # (else init to ROI mzMin/Max)
    if (isValid) {
        mzMin <- min(tmpPt$mz); mzMax <- max(tmpPt$mz)
    }

    # calculate mz (might be an approx)
    # (weighted average of total intensity across all rt for each unique mz)
    mzRange             <- unique(tmpPt$mz)
    mzTotalIntensity    <- vapply(mzRange, function(x) {
        sum(tmpPt$int[tmpPt$mz == x])}, FUN.VALUE = numeric(1))
    if (length(mzTotalIntensity) > 0) {
        # make sure weighted.mean doesn't crash if no scan match
        mz <- stats::weighted.mean(mzRange, mzTotalIntensity)
    } else {
        mz <- NA }

    # if mz, mzMin or mzMax cannot be determined the fit is not successful
    # (mzMin/mzMax default to ROI)
    if (is.na(mz) | is.na(mzMin) | is.na(mzMax)) {
        if (verbose) {
            message("Fit of ROI #", i,
                    " is unsuccessful (cannot determine mz/mzMin/mzMax)")
        }
        # indicate failure
        return(NULL) }
    return(list(mz=mz, mzMin=mzMin, mzMax=mzMax))
}

## integrate a fittedCurve on the rtMin rtMax range
findTargetFeatures_integCurve <- function(fittedCurve, rtMin, rtMax, sampling){
    ## integrate curve
    #       __a__
    #      /|     \
    #    /  h      \
    #  /____|__b____\
    # \     |      /
    #  \    h     /
    #   \___|__c_/
    # Area  = (a+b)/2 * h + (b+c)/2 * h
    #       = (a+2b+c)/2 * h
    h           <- (rtMax - rtMin)/(sampling - 1)
    grid_rt     <- seq(from = rtMin, to = rtMax, by = h)
    val_int     <- predictCurve(fittedCurve, x = grid_rt)
    peakArea    <- pracma::trapz(x=grid_rt, y=val_int)
}


## ! Unused filtering based on residuals ! ----
# @param maxResidualRatio (float) Ratio of maximum allowed residual
# area compared to the fit area. (e.g. 0.20 for a maximum residual area
# of 20% of fit area)
## Check quality of fit (residual area compared to peakArea) use the
## common range of ROI(raw data) and curve fit
# tmpRtMin <- max(rtMin, min(tmpROIData$rt))
# tmpRtMax <- min(rtMax, max(tmpROIData$rt))
# h_res <- (tmpRtMax-tmpRtMin)/(sampling-1)
# grid_rt_res <- seq(from=tmpRtMin, to=tmpRtMax, by=h_res)
## project raw data approx and curve fit on common grid
# raw_approx_fun <- stats::approxfun(x=tmp_EIC$rt, y=tmp_EIC$int)
# raw_proj <- raw_approx_fun(grid_rt_res)
# fit_proj <- predictCurve(fittedCurve, x=grid_rt_res)
## calculate residuals area on section considered
# fit_residuals <- abs(raw_proj - fit_proj)
# dist_residuals <- sum( c(fit_residuals,
#                       fit_residuals[2:(sampling-1)]) )/2
# area_residuals <- dist_residuals * h_res
## calculate peak area on section considered
# dist_subfit <- sum( c(fit_proj, fit_proj[2:(sampling-1)]) )/2
# area_subfit <- dist_subfit * h_res
## compare residual area to fit area
# if (area_residuals > (area_subfit * maxResidualRatio)) {
#   if (verbose) {message('Fit of ROI #', i,
#                           ' is unsuccessful (fit residuals is ',
#                           round(area_residuals / area_subfit, 2),
#                           ' of peak area)')}
# # move to next window (empty df row was already initialised) next
# }

## Quality of fit QC check, TRUE if pass QC check
findTargetFeatures_fitQuality <- function(i, tmp_EIC, rt, maxIntPredicted,
                                maxIntMeasured, maxApexResidualRatio, verbose) {
    # Check quality of fit (residuals at apex,and max predicted to max measured)
    # maxIntMeasured: peak max value in raw data maxIntPredicted: fit apex value
    # IntRawApex: raw data apex value
    raw_approx_fun  <- stats::approxfun(x = tmp_EIC$rt, y = tmp_EIC$int)
    IntRawApex      <- raw_approx_fun(rt)
    if (is.na(IntRawApex)) {IntRawApex <- 0}
    # residual at apex
    apex_residuals  <- abs(maxIntPredicted - maxIntMeasured)
    # residual between maximums (raw vs fit)
    max_residuals   <- abs(maxIntPredicted - IntRawApex)
    # compare residuals to max fit intensity
    if (((apex_residuals/maxIntPredicted) > maxApexResidualRatio) |
        ((max_residuals/maxIntPredicted) > maxApexResidualRatio)) {
        if (verbose) {
            message("Fit of ROI #",i," is unsuccessful (apex residuals is ",
                    round(apex_residuals/maxIntPredicted, 2),
                    " of max fit intensity, max intensity residuals is ",
                    round(max_residuals/maxIntPredicted, 2),
                    " of max fit intensity)")
        }
        return(FALSE)
    }
    return(TRUE)
}

## integrate a fittedCurve on the rtMin rtMax range
# Previous version without pracma

#findTargetFeatures_integCurve <- function(fittedCurve, rtMin, rtMax, sampling){
#    ## integrate curve
#    #       __a__
    #      /|     \
    #    /  h      \
    #  /____|__b____\
    # \     |      /
    #  \    h     /
    #   \___|__c_/
    # Area  = (a+b)/2 * h + (b+c)/2 * h
    #       = (a+2b+c)/2 * h
#    h           <- (rtMax - rtMin)/(sampling - 1)
#    grid_rt     <- seq(from = rtMin, to = rtMax, by = h)
#    val_int     <- predictCurve(fittedCurve, x = grid_rt)
#    dist        <- sum(c(val_int, val_int[2:(sampling - 1)]))/2
#    peakArea    <- dist * h
#}

context('findTargetFeatures()')

skip_if_not_installed('faahKO',  minimum_version = '1.18.0')
library(faahKO)


## Input and expected data
# use ko15.CDf file from the pkg faahKO
singleSpectraDataPath <- system.file('cdf/KO/ko15.CDF', package = "faahKO")
raw_data  						<- MSnbase::readMSData(singleSpectraDataPath, centroided=TRUE, mode='onDisk')

# targeted features in faahKO
input_ROI     	<- data.frame(matrix(vector(), 4, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))),stringsAsFactors=FALSE)
input_ROI[1,] 	<- c("ID-1", "testCpd 1", 3310., 3344.888, 3390., 522.194778, 522.2, 522.205222)
input_ROI[2,] 	<- c("ID-2", "testCpd 2", 3280., 3385.577, 3440., 496.195038, 496.2, 496.204962)
input_ROI[3,] 	<- c("ID-3", "testCpd 3", 3420., 3454.435, 3495., 464.195358, 464.2, 464.204642)
input_ROI[4,] 	<- c("ID-4", "testCpd 4", 3670., 3701.697, 3745., 536.194638, 536.2, 536.205362)
input_ROI[,3:8] <- vapply(input_ROI[,3:8], as.numeric, FUN.VALUE=numeric(4))

# ROIDataPoints for each window
input_ROIsDataPoints <- extractSignalRawData(raw_data, rt=input_ROI[,c('rtMin','rtMax')], mz=input_ROI[,c('mzMin','mzMax')], verbose=FALSE)

# found peaks
found_peakTable     <- data.frame(matrix(vector(), 4, 11, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted"))),stringsAsFactors=FALSE)
found_peakTable[1,] <- c(TRUE, 3309.7589296586070, 3346.8277590361445, 3385.4098874628098, 522.194778, 522.20001220703125, 522.205222, 26133726.6811244078,  26071378, 889280, 901015.80529226747)
found_peakTable[2,] <- c(TRUE, 3345.3766648628907, 3386.5288072289159, 3428.2788374983961, 496.20001220703125, 496.20001220703125, 496.20001220703125, 35472141.3330242932, 36498367, 1128960, 1113576.69008227298)
found_peakTable[3,] <- c(TRUE, 3451.2075903614455, 3451.5072891566265, 3501.6697504924518, 464.195358, 464.20001220703125, 464.204642, 7498427.1583901159, 7608218, 380736, 389632.13549519412)
found_peakTable[4,] <- c(TRUE, 3670.9201232710743, 3704.1427831325304, 3740.0172511251831, 536.20001220703125, 536.20001220703125, 536.20001220703125, 8626279.9788195733, 8692184, 330176, 326763.87246511364)
found_peakTable[,1] <- vapply(found_peakTable[,c(1)], as.logical, FUN.VALUE=logical(1))

cFit1           <- list(amplitude=162404.8057918259, center=3341.888, sigma=0.078786133031045896, gamma=0.0018336101984172684, fitStatus=2, curveModel="skewedGaussian")
class(cFit1)    <- 'peakPantheR_curveFit'
cFit2           <- list(amplitude=199249.10572753669, center=3382.577, sigma=0.074904415304607966, gamma=0.0011471899372353885, fitStatus=2, curveModel="skewedGaussian")
class(cFit2)    <- 'peakPantheR_curveFit'
cFit3           <- list(amplitude=31645.961277502651, center=3451.435, sigma=0.064803553287811053, gamma=2.8557893789555022, fitStatus=2, curveModel="skewedGaussian")
class(cFit3)    <- 'peakPantheR_curveFit'
cFit4           <- list(amplitude=59193.591103772116, center=3698.697, sigma=0.082789238806238355, gamma=0.0026044299691057823, fitStatus=2, curveModel="skewedGaussian")
class(cFit4)    <- 'peakPantheR_curveFit'
found_curveFit  <- list(cFit1, cFit2, cFit3, cFit4)

foundPeaks <- list(peakTable=found_peakTable, curveFit=found_curveFit)


test_that('default parameters, skewedGaussian, guess params, sampling 250, no verbose', {
  # expected foundPeaks
  expected_foundPeaks <- foundPeaks
  
	# results (output, warnings and messages)
  result_foundPeaks   <- evaluate_promise(findTargetFeatures(input_ROIsDataPoints, input_ROI, curveModel='skewedGaussian', params='guess', sampling=250, verbose=FALSE))

  # Check result table
  expect_equal(result_foundPeaks$result, expected_foundPeaks, tolerance=1e-6)

  # Check result messages
  expect_equal(length(result_foundPeaks$messages), 0)
})

test_that('trigger fitCurve TryCatch, with verbose', {
  # no scans in window #3
  tmpDPoints          <- input_ROIsDataPoints
  tmpDPoints[[3]]     <- extractSignalRawData(raw_data, rt=c(3454.435, 3454.435), mz=c(464.2, 464.2), verbose=FALSE)[[1]]
  
  # expected foundPeaks
  expected_foundPeaks               <- foundPeaks
  expected_foundPeaks$peakTable[3,] <- c(FALSE, as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA),  as.numeric(NA), as.numeric(NA), as.numeric(NA))
  expected_foundPeaks$peakTable[,1] <- as.logical(expected_foundPeaks$peakTable[,1])
  expected_foundPeaks$curveFit[[3]] <- NA
  
  # expected messages
  expected_messages <- c("Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #1\n", "Fit of ROI #3 is unsuccessful (try err)\n")
    
  # results (output, warnings and messages)
  result_foundPeaks   <- evaluate_promise(findTargetFeatures(tmpDPoints, input_ROI, curveModel='skewedGaussian', params='guess', sampling=250, verbose=TRUE))
  
  # Check result table
  expect_equal(result_foundPeaks$result, expected_foundPeaks, tolerance=1e-6)
  
  # Check result messages (no timing)
  expect_equal(length(result_foundPeaks$messages), 3)
  expect_equal(result_foundPeaks$messages[1:2], expected_messages)
})

test_that('failed fit (fitCurve status 0/5/-1), with verbose', {
  # no scans in window #3
  tmpDPoints          <- input_ROIsDataPoints
  tmpDPoints[[3]]     <- extractSignalRawData(raw_data, rt=c(3454., 3455.), mz=c(464.195358, 464.204642), verbose=FALSE)[[1]]
  
  # expected foundPeaks
  expected_foundPeaks               <- foundPeaks
  expected_foundPeaks$peakTable[3,] <- c(FALSE, as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  expected_foundPeaks$peakTable[,1] <- as.logical(expected_foundPeaks$peakTable[,1])
  expected_foundPeaks$curveFit[[3]] <- NA
  
  # expected messages
  expected_messages <- c("Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #1\n", "Fit of ROI #3 is unsuccessful (fit status)\n")
  
  # results (output, warnings and messages)
  result_foundPeaks   <- evaluate_promise(findTargetFeatures(tmpDPoints, input_ROI, curveModel='skewedGaussian', params='guess', sampling=250, verbose=TRUE))
  
  # Check result table
  expect_equal(result_foundPeaks$result, expected_foundPeaks, tolerance=1e-6)
  
  # Check result messages (no timing)
  expect_equal(length(result_foundPeaks$messages), 3)
  expect_equal(result_foundPeaks$messages[1:2], expected_messages)
})

test_that('mzMin mzMax cannot be calculated due to rtMin (#3) rtMax (#4) outside of ROI, verbose', {
  # expected foundPeaks
  expected_foundPeaks <- foundPeaks
  # expected messages
  expected_messages   <- c("Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #1\n", "Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #3\n")
  
  # results (output, warnings and messages)
  result_foundPeaks   <- evaluate_promise(findTargetFeatures(input_ROIsDataPoints, input_ROI, curveModel='skewedGaussian', params='guess', sampling=250, verbose=TRUE))
  
  # Check result table
  expect_equal(result_foundPeaks$result, expected_foundPeaks, tolerance=1e-6)
  
  # Check result messages
  expect_equal(length(result_foundPeaks$messages), 3)
  expect_equal(result_foundPeaks$messages[1:2], expected_messages)
})

test_that('rtMin rtMax cannot be found, fit is rejected, verbose', {
  # fake data which never reaches 0.5% of maxInt in 5x ROI rt width
  tmp_ROI     	  <- data.frame(matrix(vector(), 1, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))),stringsAsFactors=FALSE)
  tmp_ROI[1,] 	  <- c("ID-1", "testCpd 1", 990., 1000., 1010., 521., 522., 523.)
  tmp_ROI[,3:8]   <- vapply(tmp_ROI[,3:8], as.numeric, FUN.VALUE=numeric(1))
  
  # fake ROI data points
  #int <- (dnorm(rt, mean=1000, sd=0.5) * 100) + 20000
  #rt  <- seq(990, 1010, by=20/250)
  rt  <- c(990.00, 990.08, 990.16, 990.24, 990.32, 990.40, 990.48, 990.56, 990.64, 990.72, 990.80, 990.88, 990.96, 991.04, 991.12, 991.20, 991.28, 991.36, 991.44, 991.52, 991.60, 991.68, 991.76, 991.84, 991.92, 992.00, 992.08, 992.16, 992.24, 992.32, 992.40, 992.48, 992.56, 992.64, 992.72, 992.80, 992.88, 992.96, 993.04, 993.12, 993.20, 993.28, 993.36, 993.44, 993.52, 993.60, 993.68, 993.76, 993.84, 993.92, 994.00, 994.08, 994.16, 994.24, 994.32, 994.40, 994.48, 994.56, 994.64, 994.72, 994.80, 994.88, 994.96, 995.04, 995.12, 995.20, 995.28, 995.36, 995.44, 995.52, 995.60, 995.68, 995.76, 995.84, 995.92, 996.00, 996.08, 996.16, 996.24, 996.32, 996.40, 996.48, 996.56, 996.64, 996.72, 996.80, 996.88, 996.96, 997.04, 997.12, 997.20, 997.28, 997.36, 997.44, 997.52, 997.60, 997.68, 997.76, 997.84, 997.92, 998.00, 998.08, 998.16, 998.24, 998.32, 998.40, 998.48, 998.56, 998.64, 998.72, 998.80, 998.88, 998.96, 999.04, 999.12, 999.20, 999.28, 999.36, 999.44, 999.52, 999.60, 999.68, 999.76, 999.84, 999.92, 1000.00, 1000.08, 1000.16, 1000.24, 1000.32, 1000.40, 1000.48, 1000.56, 1000.64, 1000.72, 1000.80, 1000.88, 1000.96, 1001.04, 1001.12, 1001.20, 1001.28, 1001.36, 1001.44, 1001.52, 1001.60, 1001.68, 1001.76, 1001.84, 1001.92, 1002.00, 1002.08, 1002.16, 1002.24, 1002.32, 1002.40, 1002.48, 1002.56, 1002.64, 1002.72, 1002.80, 1002.88, 1002.96, 1003.04, 1003.12, 1003.20, 1003.28, 1003.36, 1003.44, 1003.52, 1003.60, 1003.68, 1003.76, 1003.84, 1003.92, 1004.00, 1004.08, 1004.16, 1004.24, 1004.32, 1004.40, 1004.48, 1004.56, 1004.64, 1004.72, 1004.80, 1004.88, 1004.96, 1005.04, 1005.12, 1005.20, 1005.28, 1005.36, 1005.44, 1005.52, 1005.60, 1005.68, 1005.76, 1005.84, 1005.92, 1006.00, 1006.08, 1006.16, 1006.24, 1006.32, 1006.40, 1006.48, 1006.56, 1006.64, 1006.72, 1006.80, 1006.88, 1006.96, 1007.04, 1007.12, 1007.20, 1007.28, 1007.36, 1007.44, 1007.52, 1007.60, 1007.68, 1007.76, 1007.84, 1007.92, 1008.00, 1008.08, 1008.16, 1008.24, 1008.32, 1008.40, 1008.48, 1008.56, 1008.64, 1008.72, 1008.80, 1008.88, 1008.96, 1009.04, 1009.12, 1009.20, 1009.28, 1009.36, 1009.44, 1009.52, 1009.60, 1009.68, 1009.76, 1009.84, 1009.92, 1010.00)
  mz  <- rep(522., length(rt))
  int <- c(20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000.000000000004, 20000.000000000011, 20000.000000000044, 20000.000000000138, 20000.000000000440, 20000.000000001379, 20000.000000004202, 20000.000000012471, 20000.000000036085, 20000.000000101762, 20000.000000279735, 20000.000000749529, 20000.000001957542, 20000.000004983285, 20000.000012365243, 20000.000029906903, 20000.000070505404, 20000.000162015076, 20000.000362886240, 20000.000792259820, 20000.001685958276, 20000.003497100206, 20000.007070520602, 20000.013934030874, 20000.026766045154, 20000.050115688977, 20000.091462962813, 20000.162704246217, 20000.282120451389, 20000.476817640294, 20000.785510725786, 20001.261345279254, 20001.974230758951, 20003.011923265476, 20004.478906058968, 20006.492053128739, 20009.172215254213, 20012.631312287038, 20016.955272261606, 20022.184166935895, 20028.291993044964, 20035.169486059534, 20042.613829355148, 20050.328868219622, 20057.938310552297, 20065.012452816813, 20071.106505701198, 20075.806105230542, 20078.773672313706, 20079.788456080285, 20078.773672313706, 20075.806105230542, 20071.106505701198, 20065.012452816813, 20057.938310552297, 20050.328868219622, 20042.613829355148, 20035.169486059534, 20028.291993044964, 20022.184166935895, 20016.955272261606, 20012.631312287038, 20009.172215254213, 20006.492053128739, 20004.478906058968, 20003.011923265476, 20001.974230758951, 20001.261345279254, 20000.785510725786, 20000.476817640294, 20000.282120451389, 20000.162704246217, 20000.091462962813, 20000.050115688977, 20000.026766045154, 20000.013934030874, 20000.007070520602, 20000.003497100206, 20000.001685958276, 20000.000792259820, 20000.000362886240, 20000.000162015076, 20000.000070505404, 20000.000029906903, 20000.000012365243, 20000.000004983285, 20000.000001957542, 20000.000000749529, 20000.000000279735, 20000.000000101762, 20000.000000036085, 20000.000000012471, 20000.000000004202, 20000.000000001379, 20000.000000000440, 20000.000000000138, 20000.000000000044, 20000.000000000011, 20000.000000000004, 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000., 20000.)
  tmp_DataPoints  <- list(data.frame(rt=rt, mz=mz, int=int))
  
  # expected foundPeaks
  expected_peakTable      <- data.frame(matrix(vector(), 1, 11, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted"))),stringsAsFactors=FALSE)
  expected_peakTable[1,]  <- c(FALSE, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA)
  expected_peakTable[,1]  <- sapply(expected_peakTable[,c(1)], as.logical)
  
  cFit                <- NA
  expected_foundPeaks <- list(peakTable=expected_peakTable, curveFit=list(cFit))
  
  # expected messages
  expected_messages   <- c("Warning: rtMin cannot be determined for ROI #1\n", "Warning: rtMax cannot be determined for ROI #1\n", "Fit of ROI #1 is unsuccessful (cannot determine rtMin/rtMax)\n")
  
  # results (output, warnings and messages)
  result_foundPeaks   <- evaluate_promise(findTargetFeatures(tmp_DataPoints, tmp_ROI, curveModel='skewedGaussian', params='guess', sampling=250, verbose=TRUE))
  
  # Check result table only
  expect_equal(result_foundPeaks$result, expected_foundPeaks, tolerance=1e-6)
  
  # Check result messages
  expect_equal(length(result_foundPeaks$messages), 4)
  expect_equal(result_foundPeaks$messages[1:3], expected_messages)
})

test_that('mz cannot be calculated, fit is rejected, verbose', {
  # no scan fall between rtMin/rtMax after fitting, therefore nothing can be used for mz calculation (mz is NA). (fit isn't satisfactory)
  
  tmp_ROI     	  <- data.frame(matrix(vector(), 1, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))),stringsAsFactors=FALSE)
  tmp_ROI[1,] 	  <- c("ID-1", "testCpd 1", 149.112, NA, 158.67, 126.9795, 127.0195, 127.0595)
  tmp_ROI[,3:8]   <- sapply(tmp_ROI[,3:8], as.numeric)
  
  # ROI data points
  rt  <- c(149.394, 150.068, 150.741, 150.910, 150.910, 150.910, 151.078, 151.247, 152.089, 152.257, 152.257, 152.594, 152.594, 153.773, 154.615, 154.952, 155.458, 155.458, 156.300, 156.468, 156.637, 156.637, 156.805, 156.974, 157.142, 157.142, 157.310, 157.310, 157.479, 157.479, 157.647, 157.647, 157.647, 157.816, 157.816, 157.984, 157.984, 157.984, 158.153, 158.153, 158.153, 158.321, 158.321, 158.321, 158.490, 158.490, 158.490, 158.490, 158.659, 158.659, 158.659)
  mz  <- c(127.0464, 127.0561, 127.0535, 127.0154, 127.0293, 127.0540, 127.0532, 127.0261, 127.0519, 127.0234, 127.0486, 127.0257, 127.0544, 127.0471, 127.0453, 127.0565, 127.0426, 127.0544, 127.0477, 127.0220, 127.0139, 127.0250, 127.0189, 127.0189, 127.0187, 127.0509, 127.0197, 127.0552, 127.0193, 127.0536, 127.0192, 127.0395, 127.0526, 127.0192, 127.0545, 126.9916, 127.0192, 127.0518, 126.9877, 127.0196, 127.0518, 126.9825, 127.0196, 127.0502, 126.9820, 126.9944, 127.0196, 127.0490, 126.9972, 127.0195, 127.0493)
  int <- c(176.9860, 100.9088, 114.0000, 121.5390, 114.4610, 105.2671, 164.5325, 143.0000, 231.2491, 100.2164, 157.3531, 107.5632, 106.0000, 199.6444, 138.0000, 154.0000, 101.8883, 116.5039, 146.0000, 105.0000, 179.9103, 256.8206, 769.8887, 1134.0303, 2396.0000, 110.4933, 6405.2383, 144.0000, 10276.6250, 157.0000, 18852.2969, 110.2715, 127.9841, 31468.8906, 153.8988, 133.8497, 48662.4062, 248.3162, 128.1760, 72571.4375, 637.0933, 136.0000, 101874.3750, 715.7915, 100.3529, 252.6471, 131491.1250, 1474.4824, 107.2497, 160474.5000, 2006.7666)
  tmp_DataPoints  <- list(data.frame(rt=rt, mz=mz, int=int))
  
  # expected foundPeaks
  expected_peakTable      <- data.frame(matrix(vector(), 1, 11, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted"))),stringsAsFactors=FALSE)
  expected_peakTable[1,]  <- c(FALSE, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA)
  expected_peakTable[,1]  <- sapply(expected_peakTable[,c(1)], as.logical)
  
  cFit                <- NA
  expected_foundPeaks <- list(peakTable=expected_peakTable, curveFit=list(cFit))
  
  # expected messages
  expected_messages   <- c("Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #1\n", "Fit of ROI #1 is unsuccessful (cannot determine mz/mzMin/mzMax)\n")
  
  # results (output, warnings and messages)
  result_foundPeaks   <- evaluate_promise(findTargetFeatures(tmp_DataPoints, tmp_ROI, curveModel='skewedGaussian', params='guess', sampling=250, verbose=TRUE))
  
  # Check result table only
  expect_equal(result_foundPeaks$result, expected_foundPeaks, tolerance=1e-6)
  
  # Check result messages
  expect_equal(length(result_foundPeaks$messages), 3)
  expect_equal(result_foundPeaks$messages[1:2], expected_messages)
})

# No other curveModel currently available

# if ((.Platform$OS.type != "windows") || (.Machine$sizeof.pointer == 8)) {
# test_that('ratio of fit residuals at apex or across maximum is superior to "maxApexResidualRatio", fit is rejected, verbose', {
#   # Cpd3 fit fails, detected using the apex residuals ratio
#   # Need another sample, same ROI

#   ## Prepare data
#   tmp_singleSpectraDataPath <- system.file('cdf/KO/ko18.CDF', package = "faahKO")
#   tmp_raw_data  						<- MSnbase::readMSData(tmp_singleSpectraDataPath, centroided=TRUE, mode='onDisk')
#   # ROIDataPoints for each window
#   tmp_input_ROIsDataPoints  <- extractSignalRawData(tmp_raw_data, rt=input_ROI[,c('rtMin','rtMax')], mz=input_ROI[,c('mzMin','mzMax')], verbose=FALSE)
#   # found peaks
#   tmp_found_peakTable                 <- data.frame(matrix(vector(), 4, 11, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted"))),stringsAsFactors=FALSE)
#   tmp_found_peakTable[1,]             <- c(TRUE, 3333.8625894557053, 3368.233, 3407.4362838927614, 522.194778, 522.20001220703125, 522.205222, 21447174.404490683, 20364558, 758336, 765009.9805796633)
#   tmp_found_peakTable[2,]             <- c(TRUE, 3373.3998828113113, 3413.4952530120481, 3454.4490330927388, 496.195038, 496.20001220703125, 496.204962, 35659353.614476241, 34792490, 1149440, 1145857.7611069249)
#   tmp_found_peakTable[3,]             <- c(FALSE, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA)
#   tmp_found_peakTable[4,]             <- c(TRUE, 3672.3110625980275, 3714.088, 3761.43921706666, 536.194638, 536.20001220703125, 536.205362, 6467062.4309558524, 6360742, 196160, 189416.24807174454)
#   tmp_found_peakTable[,c(1)]       <- sapply(tmp_found_peakTable[,c(1)], as.logical)
#   tmp_found_peakTable[,c(2:10)] <- sapply(tmp_found_peakTable[,c(2:10)], as.numeric)
#   # fit
#   tmp_cFit1           <- list(amplitude=122363.51256736703, center=3362.233, sigma=0.075489598945304492, gamma=0.0025160536725299734, fitStatus=2, curveModel="skewedGaussian")
#   class(tmp_cFit1)    <- 'peakPantheR_curveFit'
#   tmp_cFit2           <- list(amplitude=204749.86097918145, center=3409.182, sigma=0.075731781812843249, gamma=0.0013318670577834328, fitStatus=2, curveModel="skewedGaussian")
#   class(tmp_cFit2)    <- 'peakPantheR_curveFit'
#   tmp_cFit3           <- NA
#   tmp_cFit4           <- list(amplitude=26628.505498512375, center=3708.088, sigma=0.064131129861254479, gamma=0.0015719426982490699, fitStatus=2, curveModel="skewedGaussian")
#   class(tmp_cFit4)    <- 'peakPantheR_curveFit'
#   tmp_found_curveFit  <- list(tmp_cFit1, tmp_cFit2, tmp_cFit3, tmp_cFit4)

#   # expected results
#   expected_foundPeaks <- list(peakTable=tmp_found_peakTable, curveFit=tmp_found_curveFit)

#   # expected messages
#   expected_messages   <- c("Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #1\n", "Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #2\n", "Fit of ROI #3 is unsuccessful (apex residuals is 0.45 of max fit intensity, max intensity residuals is 0.46 of max fit intensity)\n", "Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #4\n")

#   # results (output, warnings and messages)
#   result_foundPeaks   <- evaluate_promise(findTargetFeatures(tmp_input_ROIsDataPoints, input_ROI, curveModel='skewedGaussian', params='guess', sampling=250, verbose=TRUE))

#   # Check result table only
#   expect_equal(result_foundPeaks$result, expected_foundPeaks, tolerance=1e-6)

#   # Check result messages
#   expect_equal(length(result_foundPeaks$messages), 5)
#   expect_equal(result_foundPeaks$messages[1:4], expected_messages)
#   }) }

test_that('change params for window #3', {
  # input params
  tmp_params    <- list( init_params  = list(amplitude=1E5, center=3455., sigma=0.1, gamma=0),
                         lower_bounds = list(amplitude=0,   center=3450., sigma=0,   gamma=-0.1),
                         upper_bounds = list(amplitude=1E9, center=3460., sigma=5,   gamma=1))
  input_params  <- list('guess', 'guess', tmp_params, 'guess')
  # expected foundPeaks
  expected_foundPeaks                         <- foundPeaks
  expected_foundPeaks$peakTable[3,2:11]       <- c(3418.0076795585401, 3455.6277710843374, 3495.4734240188186, 464.195358, 464.2000122, 464.204642, 11307215.264967661, 11538988, 380736, 381327.26552768378)
  expected_foundPeaks$curveFit[[3]]$amplitude <- 64246.052173667762
  expected_foundPeaks$curveFit[[3]]$center    <- 3450
  expected_foundPeaks$curveFit[[3]]$sigma     <- 0.07533469863886906
  expected_foundPeaks$curveFit[[3]]$gamma     <- 0.0019238229766131536
  expected_foundPeaks$curveFit[[3]]$fitStatus <- 2
  # expected messages
  expected_messages   <- c("Curve fitting parameters passed as input employed\n","Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #1\n", "Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #3\n")
  
  # results (output, warnings and messages)
  result_foundPeaks   <- evaluate_promise(findTargetFeatures(input_ROIsDataPoints, input_ROI, curveModel='skewedGaussian', params=input_params, sampling=250, verbose=TRUE))
  
  # Check result table
  expect_equal(result_foundPeaks$result, expected_foundPeaks, tolerance=1e-5)
  
  # Check result messages
  expect_equal(length(result_foundPeaks$messages), 4)
  expect_equal(result_foundPeaks$messages[1:3], expected_messages)
})

test_that('change sampling', {
  # expected foundPeaks
  expected_peakTable      <- data.frame(matrix(vector(), 4, 11, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted"))),stringsAsFactors=FALSE)
  expected_peakTable[1,]  <- c(TRUE, 3309.6884911519801, 3346.7859591836732, 3385.4800395368861, 522.194778, 522.20001220703125, 522.205222, 26133882.5658131838, 26071378, 889280, 901012.10599065467)
  expected_peakTable[2,]  <- c(TRUE, 3345.0894935650540, 3386.4953673469390, 3428.4561855932079, 496.20001220703125, 496.20001220703125, 496.20001220703125, 35474025.5332010239, 36498367, 1128960, 1113574.43999634334)
  expected_peakTable[3,]  <- c(TRUE, 3450.0344897959185, 3451.5574489795918, 3501.7095764174951, 464.195358, 464.20001220703125, 464.204642, 7456470.7446634285, 7608218, 380736, 389624.14409317775)
  expected_peakTable[4,]  <- c(TRUE, 3670.8488688282255, 3704.0847551020411, 3740.0915161411931, 536.20001220703125, 536.20001220703125, 536.20001220703125, 8626359.7845999431, 8692184, 330176, 326758.81671814714)
  expected_peakTable[,1]  <- sapply(expected_peakTable[,c(1)], as.logical)
  expected_foundPeaks     <- list(peakTable=expected_peakTable, curveFit=found_curveFit)
  
  # results (output, warnings and messages)
  result_foundPeaks   <- evaluate_promise(findTargetFeatures(input_ROIsDataPoints, input_ROI, curveModel='skewedGaussian', params='guess', sampling=50, verbose=FALSE))
  
  # Check result table
  expect_equal(result_foundPeaks$result, expected_foundPeaks, tolerance=1e-6)
  
  # Check result messages
  expect_equal(length(result_foundPeaks$messages), 0)
})

test_that('In fit ratio calculation, special case when "IntRawApex" cannot be determined (no raw data scans exist a fit apex), verbose', {
  # no scan fall at both sides of the fit apex (cannot be interpolated): the raw data intensity at fit apex ("IntRawApex") is set to 0
  
  tmp_ROI     	  <- data.frame(matrix(vector(), 1, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))),stringsAsFactors=FALSE)
  tmp_ROI[1,] 	  <- c("ID-1", "testCpd 1", 216.4, 224.4, 232.4, 428.367, 428.3734, 428.3799)
  tmp_ROI[,3:8]   <- sapply(tmp_ROI[,3:8], as.numeric)
  
  # ROI data points
  rt  <- c(218.327, 219.601, 222.090, 224.978, 229.526)
  mz  <- c(428.3786, 428.3707, 428.3705, 428.3772, 428.3787)
  int <- c(249.0000, 183.1346, 112.0000, 134.0000, 104.0000)
  tmp_DataPoints  <- list(data.frame(rt=rt, mz=mz, int=int))
  
  # expected foundPeaks
  expected_peakTable      <- data.frame(matrix(vector(), 1, 11, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted"))),stringsAsFactors=FALSE)
  expected_peakTable[1,]  <- c(FALSE, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA)
  expected_peakTable[,1]  <- sapply(expected_peakTable[,c(1)], as.logical)
  
  cFit                <- NA
  expected_foundPeaks <- list(peakTable=expected_peakTable, curveFit=list(cFit))
  
  # expected messages
  expected_messages   <- c("Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #1\n", "Fit of ROI #1 is unsuccessful (apex residuals is 0.18 of max fit intensity, max intensity residuals is 1 of max fit intensity)\n")
  
  # results (output, warnings and messages)
  result_foundPeaks   <- evaluate_promise(findTargetFeatures(tmp_DataPoints, tmp_ROI, curveModel='skewedGaussian', params='guess', sampling=250, verbose=TRUE))
  
  # Check result table only
  expect_equal(result_foundPeaks$result, expected_foundPeaks, tolerance=1e-6)
  
  # Check result messages
  expect_equal(length(result_foundPeaks$messages), 3)
  expect_equal(result_foundPeaks$messages[1:2], expected_messages)
})

test_that('raise errors', {
  # ROIsDataPoints doesn't match number of ROI
  expect_error(findTargetFeatures(input_ROIsDataPoints, input_ROI[1:3,], curveModel='skewedGaussian', params='guess'), 'Check input, number of ROIsDataPoints entries must match the number of rows of ROI', fixed=TRUE)
  
  # data points rt outside of ROI
  # rtMin side
  tmpDPoints      <- input_ROIsDataPoints
  tmpDPoints[[2]] <- extractSignalRawData(raw_data, rt=c(3000, 3440), mz=c(496.195038, 496.204962), verbose=FALSE)[[1]]
  expect_error(findTargetFeatures(tmpDPoints, input_ROI, curveModel='skewedGaussian', params='guess'), 'Check input not all datapoints for window #2 are into the corresponding ROI (rt)', fixed=TRUE)
  # rtMax
  tmpDPoints      <- input_ROIsDataPoints
  tmpDPoints[[3]] <- extractSignalRawData(raw_data, rt=c(3420., 3600.), mz=c(464.195358, 464.204642), verbose=FALSE)[[1]]
  expect_error(findTargetFeatures(tmpDPoints, input_ROI, curveModel='skewedGaussian', params='guess'), 'Check input not all datapoints for window #3 are into the corresponding ROI (rt)', fixed=TRUE)
  
  # data points mz outside of ROI
  # mzMin side
  tmpDPoints      <- input_ROIsDataPoints
  tmpDPoints[[2]] <- extractSignalRawData(raw_data, rt=c(3280, 3440), mz=c(494.195038, 496.204962), verbose=FALSE)[[1]]
  expect_error(findTargetFeatures(tmpDPoints, input_ROI, curveModel='skewedGaussian', params='guess'), 'Check input not all datapoints for window #2 are into the corresponding ROI (mz)', fixed=TRUE)
  # mzMax
  tmpDPoints      <- input_ROIsDataPoints
  tmpDPoints[[3]] <- extractSignalRawData(raw_data, rt=c(3420., 3495.), mz=c(464.195358, 466.204642), verbose=FALSE)[[1]]
  expect_error(findTargetFeatures(tmpDPoints, input_ROI, curveModel='skewedGaussian', params='guess'), 'Check input not all datapoints for window #3 are into the corresponding ROI (mz)', fixed=TRUE)
  
  # params is not character or list
  expect_error(findTargetFeatures(input_ROIsDataPoints, input_ROI, curveModel='skewedGaussian', params=5), 'Check input, "params" must be "guess" or list', fixed=TRUE)
  
  # params is character but not 'guess'
  expect_error(findTargetFeatures(input_ROIsDataPoints, input_ROI, curveModel='skewedGaussian', params='not guess'), 'Check input, "params" must be "guess" if not list', fixed=TRUE)
  
  # params is a list of wrong length
  expect_error(findTargetFeatures(input_ROIsDataPoints, input_ROI, curveModel='skewedGaussian', params=vector('list',2)), 'Check input, number of parameters must match number of rows of ROI', fixed=TRUE)
})

################################################################################
#                                                                              #
#                                                                              #
#       -- peakPantheR: Peak Picking and ANnoTation of High resolution         #
#                              Experiments in R --                             #
#                                                                              #
# Arnaud M. Wolfer                                                             #
# National Phenome Centre                                                      #
# Licensed under GPLv3                                                         #
################################################################################
#
# Copyright (C) {2020}  {Arnaud M. Wolfer}
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.



#' peakPantheR: A package for Peak Picking and ANnoTation of High resolution
#' Experiments
#'
#' \pkg{peakPantheR} detects, integrates and reports pre-defined features in
#' mass spectrometry data files. It enables the real time annotation of multiple
#' compounds in a single file, or the parallel annotation of multiple compounds
#' in multiple files.
#'
#' The main functions of \pkg{peakPantheR} are
#' \code{\link{peakPantheR_singleFileSearch}} for realtime annotation, and
#' \code{\link{peakPantheR_parallelAnnotation}} for parallel annotation. The
#' \code{peakPantheRAnnotation} object stores parallel annotation results, while
#' reporting functions help assess the quality of annotation and update
#' fitting parameters. Refer to the vignettes for graphical user interface and
#' command line tutorials.
#'
#' @aliases peakPantheR-package
#'
#' @docType package
#' @name peakPantheR
#'
#' @import foreach
#' @import doParallel
#'
"_PACKAGE"
#> [1] "_PACKAGE"

#' @title Extract signal in a multiple defined mz rt window from a raw data file
#'
#' @description Extract all signal from multiple defined mz rt window from raw
#' data and returns a data.frame. If no rt-mz window is provided, all signal in
#' the raw data file are returned
#'
#' @param rawSpec an \code{\link[MSnbase]{OnDiskMSnExp-class}}
#' @param rt (numeric(2) or two-column matrix) the lower and upper retention
#' time range from which the data should be extracted. If a matrix is passed,
#' each row corresponds to a different window. If not provided, the full
#' retention time range will be extracted.
#' @param mz (numeric(2) or two-column matrix) the lower and upper mass range
#' from which the data should be extracted. If a matrix is passed, each row
#' corresponds to a different window. If not provided, the full mass range will
#' be extracted.
#' @param msLevel (int) the MS level at which the data should be extracted
#' (default to MS level 1)
#' @param verbose (bool) If TRUE message progress and warnings
#'
#' @return a list (one entry per window) of data.frame with signal as row and
#' retention time ('rt'), mass ('mz') and intensity ('int) as columns.
#' 
#' @details
#' ## Examples cannot be computed as the function is not exported:
#' ## Use a file form the faahKO package and extract data from a region of
#' ## interest
#' library(faahKO)
#' rawSpec <- MSnbase::readMSData(
#'                             system.file('cdf/KO/ko15.CDF',package='faahKO'),
#'                             centroided=TRUE,
#'                             mode='onDisk')
#' dataPoints  <- extractSignalRawData(rawSpec,
#'                                     rt = c(3290., 3410.), 
#'                                     mz = c(522.194778, 522.205222), 
#'                                     verbose=TRUE)
#' # Reading data from 1 windows
#' 
#' dataPoints
#' # [[1]]
#' #          rt    mz    int
#' # 1  3290.115 522.2   1824
#' # 2  3291.680 522.2   1734
#' # 3  3293.245 522.2   1572
#' # 4  3294.809 522.2   1440
#' # 5  3299.504 522.2   1008
#' # 6  3301.069 522.2    871
#' # 7  3302.634 522.2    786
#' # 8  3304.199 522.2    802
#' # 9  3305.764 522.2    834
#' # 10 3307.329 522.2    839
#' # 11 3315.154 522.2   2187
#' # 12 3316.719 522.2   3534
#' # 13 3318.284 522.2   6338
#' # 14 3319.849 522.2  11718
#' # 15 3321.414 522.2  21744
#' # 16 3322.979 522.2  37872
#' # 17 3324.544 522.2  62424
#' # 18 3326.109 522.2  98408
#' # 19 3327.673 522.2 152896
#' # 20 3329.238 522.2 225984
#' # 21 3330.803 522.2 308672
#' # 22 3332.368 522.2 399360
#' # 23 3333.933 522.2 504000
#' # 24 3335.498 522.2 614656
#' # 25 3337.063 522.2 711872
#' # 26 3338.628 522.2 784704
#' # 27 3340.193 522.2 836608
#' # 28 3341.758 522.2 866304
#' # 29 3343.323 522.2 882304
#' # 30 3344.888 522.2 889280
#' # 31 3346.453 522.2 888256
#' # 32 3348.018 522.2 866816
#' # 33 3349.583 522.2 827392
#' # 34 3351.148 522.2 777728
#' # 35 3352.713 522.2 727040
#' # 36 3354.278 522.2 678464
#' # 37 3355.843 522.2 629120
#' # 38 3357.408 522.2 578048
#' # 39 3358.973 522.2 524288
#' # 40 3360.538 522.2 471040
#' # 41 3362.102 522.2 416320
#' # 42 3363.667 522.2 360064
#' # 43 3365.232 522.2 302400
#' # 44 3366.797 522.2 249152
#' # 45 3368.362 522.2 202560
#' # 46 3369.927 522.2 161024
#' # 47 3371.492 522.2 123520
#' # 48 3373.057 522.2  93160
#' # 49 3374.622 522.2  71856
#' # 50 3376.187 522.2  58392
#' # 51 3377.752 522.2  51072
#' # 52 3379.317 522.2  48376
#' # 53 3380.882 522.2  49168
#' # 54 3382.447 522.2  53120
#' # 55 3384.012 522.2  62488
#' # 56 3385.577 522.2  78680
#' # 57 3387.142 522.2 102840
#' # 58 3388.707 522.2 134656
#' # 59 3390.272 522.2 173440
#' # 60 3391.837 522.2 217088
#' # 61 3393.402 522.2 268864
#' # 62 3394.966 522.2 330496
#' # 63 3396.531 522.2 395776
#' # 64 3398.096 522.2 453376
#' # 65 3399.661 522.2 499072
#' # 66 3401.226 522.2 537024
#' # 67 3402.791 522.2 570304
#' # 68 3404.356 522.2 592512
#' # 69 3405.921 522.2 598912
#' # 70 3407.486 522.2 595008
#' # 71 3409.051 522.2 588416
extractSignalRawData <- function(rawSpec, rt, mz, msLevel = 1L, verbose = TRUE){
    stime <- Sys.time()

    # Check input and init output
    outParam <- extractSignalRawData_init(rt, mz, msLevel, verbose)
    rt <- outParam$rt
    mz <- outParam$mz
    empty_res <- outParam$empty_res

    # Read scan in file
    scanRead <- extractSignalRawData_extractScans(rawSpec, rt, msLevel, verbose)
    if (is.null(scanRead)) { return(empty_res) } # no scans were found
    spectraData <- scanRead$spectraData
    spec_rt <- scanRead$spec_rt


    # Get data points from each window (subset rt and check mz)
    res <- extractSignalRawData_extractFromWindows(empty_res, rt, mz, spec_rt,
                                                    spectraData, verbose)
    
    # clear variables
    rm(spectraData, spec_rt)
    gc(verbose = FALSE)
    
    # Out
    etime <- Sys.time()
    if (verbose) {
        message("Data read in: ",
                round(as.double(difftime(etime, stime)), 2),
                " ", units(difftime(etime, stime)))
    }
    return(res)
}

# Looping tryCatch with count and sleep ---------------------------------------
retry <- function(expr, isError=function(x) "try-error" %in% class(x), 
                maxErrors=5, sleep=0) {
    attempts = 0
    retval = try(eval(expr))
    while (isError(retval)) {
        attempts = attempts + 1
        if (attempts >= maxErrors) {
            msg = sprintf("retry: too many retries [[%s]]", 
                            utils::capture.output(str(retval)))
            stop(msg)
        } else {
            msg = sprintf("retry: error in attempt %i/%i", attempts, maxErrors)
            message(msg)
        }
        if (sleep > 0) Sys.sleep(sleep)
        retval = try(eval(expr))
    }
    return(retval)
}

# -----------------------------------------------------------------------------
# extractSignalRawData helper functions

# Check input type and dimensions of msLevel
extractSignalRawData_checkInput <- function(rt, mz, msLevel, verbose) {
    # Check input check type and dimensions msLevel
    if (!is.integer(msLevel)) {
        stop("Check input \"msLevel\" must be integer")
    }
    # rt
    if (!missing(rt)) {
        if (!is(rt, "numeric") & !is(rt, "matrix") & !is(rt, "data.frame")) {
            stop("Check input \"rt\" must be numeric, matrix or data.frame")}
        if (is(rt, "numeric") & (length(rt) != 2)) {
            stop("Check input \"rt\" must be numeric of length 2")}
        if (is(rt, "matrix") | is(rt, "data.frame")) {
            if (ncol(rt) != 2) {
                stop('Check input \"rt\" must be a ',
                'matrix or data.frame with 2 columns')
            }
        }
    }
    # mz
    if (!missing(mz)) {
        if (!is(mz, "numeric") & !is(mz, "matrix") & !is(mz, "data.frame")) {
            stop("Check input \"mz\" must be numeric, matrix or data.frame")}
        if (is(mz, "numeric") & (length(mz) != 2)) {
            stop("Check input \"mz\" must be numeric of length 2")}
        if (is(mz, "matrix") | is(mz, "data.frame")) {
            if (ncol(mz) != 2) {
                stop('Check input \"mz\" must be a matrix',
                ' or data.frame with 2 columns')
            }
        }
    }
    # both rt and mz have same number of rows (unless one of them has only 1row)
    if (!missing(rt) & !missing(mz)) {
        if (any(is(rt, "matrix"), is(rt, "data.frame")) &
            any(is(mz, "matrix"), is(mz, "data.frame"))) {
            if (nrow(rt) != nrow(mz)) {
                if ((nrow(rt) != 1) & (nrow(mz) != 1)) {
                    stop('Check input \"rt\" and \"mz\" matrix or ',
                    'data.frame must have the same number of rows')
                } else {
                    if (verbose) {
                        message('\"rt\" or \"mz\" is a matrix/data.frame',
                        ' of 1 row, rows will be duplicated to match',
                        ' the other input')
                    }
                }
            }
        }
    }
}
# Init outputs
extractSignalRawData_init <- function(rt, mz, msLevel, verbose) {
    # Check input check type and dimensions msLevel
    extractSignalRawData_checkInput(rt, mz, msLevel, verbose)

    # Express rt and mz as matrix/data.frame of identical number of rows
    # replace missing by whole range
    if (missing(rt)) {
        rt <- matrix(c(-Inf, Inf), ncol = 2, byrow = TRUE)}
    if (missing(mz)) {
        mz <- matrix(c(-Inf, Inf), ncol = 2, byrow = TRUE)}

    # convert all numeric to matrix
    if (is(rt, "numeric")) {
        rt <- matrix(rt, nrow = 1, ncol = 2, byrow = TRUE)}
    if (is(mz, "numeric")) {
        mz <- matrix(mz, nrow = 1, ncol = 2, byrow = TRUE)}

    # Replicate rows (if only 1) to match other
    if (nrow(rt) == 1) {
        rt <- matrix(rep(as.numeric(rt), nrow(mz)), ncol = 2, byrow = TRUE)}
    if (nrow(mz) == 1) {
        mz <- matrix(rep(as.numeric(mz), nrow(rt)), ncol = 2, byrow = TRUE)}
    # now both rt and mz are either a matrix/data.frame of matching size

    if (verbose) {message("Reading data from ", nrow(rt), " windows")}
    # empty output
    empty_res <- lapply(vector("list", nrow(rt)), function(x) {
        data.frame(rt = numeric(), mz = numeric(), int = integer())
    })

    return(list(rt=rt, mz=mz, empty_res=empty_res))
}
# Extract scans for all windows in a single file access
extractSignalRawData_extractScans <- function(rawSpec, rt, msLevel, verbose) {
    # Filter msLevel
    msFilteredSpec <- MSnbase::filterMsLevel(rawSpec, msLevel = msLevel)
    # if msLevel doesn't exist, exit
    if (length(msFilteredSpec) == 0) {
        if (verbose) {message("No data exist for MS level ", msLevel)}
        return(NULL)
    }

    # Filter only scans falling into the rt of interest (across all windows)
    file_rt         <- MSnbase::rtime(msFilteredSpec)
    keep_scan_idx   <- sort(unique(as.integer(unlist(apply(rt, MARGIN = 1,
        function(x) {
            which((file_rt >= x[1]) & (file_rt <= x[2]))
        }), use.names = FALSE))))
    # if no scans
    if (length(keep_scan_idx) == 0) {
        if (verbose) {message("No data exist for the rt provided")}
        return(NULL)
    }
    # file with only the needed scans
    rtFilteredSpec <- msFilteredSpec[keep_scan_idx]

    # Extract only scans we need (only file access)
    spectraData <- retry(MSnbase::spectra(rtFilteredSpec), 
                        maxErrors=1000, sleep=1)
    spec_rt     <- MSnbase::rtime(rtFilteredSpec)
    
    # clear variables
    rm(msFilteredSpec, file_rt, keep_scan_idx, rtFilteredSpec)
    gc(verbose = FALSE)

    return(list(spectraData=spectraData, spec_rt=spec_rt))
}
# Get data points from each window (subset rt and check mz)
extractSignalRawData_extractFromWindows <- function(empty_res, rt, mz, spec_rt,
                                                    spectraData, verbose) {
    res <- empty_res

    # iterage over windows
    for (i in seq_len(nrow(rt))) {

        # subset the scans we need from the ones we have extracted
        scans_to_keep <- (spec_rt >= rt[i, 1]) & (spec_rt <= rt[i, 2])
        if (!any(scans_to_keep)) {
            if (verbose) {message("No data exist for window ", i)}
            # move to next window (empty df was already initialised)
            next
        }

        # only keep scans matching the window
        scanSubset <- spectraData[scans_to_keep]

        # subset each scan based on mz and extract datapoints
        scanDatapoint <- lapply(scanSubset, function(scan, mzScan) {
            # filter mz
            suppressWarnings(filtScan <- MSnbase::filterMz(scan, mzScan))
            # extract datapoint
            if (!filtScan@peaksCount) {
                # no datapoints
                data.frame(rt = numeric(), mz = numeric(), int = integer())
            } else {
                data.frame(rt = rep_len(filtScan@rt, length(filtScan@mz)),
                            mz = filtScan@mz, int = filtScan@intensity)
            }
        }, mzScan = mz[i, ])

        # concatenate all scans data.frame
        scanTable           <- do.call(rbind, scanDatapoint)
        rownames(scanTable) <- NULL

        # store results
        res[[i]] <- scanTable
    }

    # clear variables
    rm(scanDatapoint, scanTable)
    gc(verbose = FALSE)

    return(res)
}

context('peakPantheR_singleFileSearch()')

skip_if_not_installed('faahKO',  minimum_version = '1.18.0')
library(faahKO)

# remove Rplots.pdf created by ggplot2
on.exit( tryCatch({ file.remove('./Rplots.pdf') }, error=function(e){ invisible() }, warning=function(w){ invisible() }) )


## Input and expected data
# use ko15.CDf file from the pkg faahKO
singleSpectraDataPath <- system.file('cdf/KO/ko15.CDF', package = "faahKO")
tmp_raw_data  				<- MSnbase::readMSData(singleSpectraDataPath, centroided=TRUE, mode='onDisk')

# targeted features in faahKO
input_ROI     	<- data.frame(matrix(vector(), 4, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))),stringsAsFactors=FALSE)
input_ROI[1,] 	<- c("ID-1", "testCpd 1", 3310., 3344.888, 3390., 522.194778, 522.2, 522.205222)
input_ROI[2,] 	<- c("ID-2", "testCpd 2", 3280., 3385.577, 3440., 496.195038, 496.2, 496.204962)
input_ROI[3,] 	<- c("ID-3", "testCpd 3", 3420., 3454.435, 3495., 464.195358, 464.2, 464.204642)
input_ROI[4,] 	<- c("ID-4", "testCpd 4", 3670., 3701.697, 3745., 536.194638, 536.2, 536.205362)
input_ROI[,3:8] <- vapply(input_ROI[,3:8], as.numeric, FUN.VALUE=numeric(4))

# Expected peakTables
found_peakTable     <- data.frame(matrix(vector(), 4, 18, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "cpdID", "cpdName", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
found_peakTable[1,] <- c(TRUE, 3309.7589296586070, 3346.8277590361445, 3385.4098874628098, 522.194778, 522.20001220703125, 522.205222, 26133726.6811244078, 26071378, 889280, 901015.80529226747, 'ID-1', 'testCpd 1', FALSE, 0.023376160866574614, 1.93975903614455092, 1.0153573486330891, 1.0268238825675249)
found_peakTable[2,] <- c(TRUE, 3345.3766648628907, 3386.5288072289159, 3428.2788374983961, 496.20001220703125, 496.20001220703125, 496.20001220703125, 35472141.3330242932, 36498367, 1128960, 1113576.69008227298, 'ID-2', 'testCpd 2', FALSE, 0.024601030353423384, 0.95180722891564074, 1.0053782620427065, 1.0093180792278085)
found_peakTable[3,] <- c(TRUE, 3451.2075903614455, 3451.5072891566265, 3501.6697504924518, 464.195358, 464.20001220703125, 464.204642, 7498427.1583901159, 7608218, 380736, 389632.13549519412, 'ID-3', 'testCpd 3', FALSE, 0.026296922148575364, -2.92771084337346110, 207.6939219686769036, 380.5019028782010082)
found_peakTable[4,] <- c(TRUE, 3670.9201232710743, 3704.1427831325304, 3740.0172511251831, 536.20001220703125, 536.20001220703125, 536.20001220703125, 8626279.9788195733, 8692184, 330176, 326763.87246511364, 'ID-4', 'testCpd 4', FALSE, 0.022765817240815486, 2.44578313253032320, 1.0305289730128095, 1.0536948855480386)
found_peakTable[,c(1,14)]       <- vapply(found_peakTable[,c(1,14)], as.logical, FUN.VALUE=logical(4))
found_peakTable[,c(2:11,15:18)] <- vapply(found_peakTable[,c(2:11,15:18)], as.numeric, FUN.VALUE=numeric(4))

# found curveFit
cFit1           <- list(amplitude=162404.8057918259, center=3341.888, sigma=0.078786133031045896, gamma=0.0018336101984172684, fitStatus=2, curveModel="skewedGaussian")
class(cFit1)    <- 'peakPantheR_curveFit'
cFit2           <- list(amplitude=199249.10572753669, center=3382.577, sigma=0.074904415304607966, gamma=0.0011471899372353885, fitStatus=2, curveModel="skewedGaussian")
class(cFit2)    <- 'peakPantheR_curveFit'
cFit3           <- list(amplitude=31645.961277502651, center=3451.435, sigma=0.064803553287811053, gamma=2.8557893789555022, fitStatus=2, curveModel="skewedGaussian")
class(cFit3)    <- 'peakPantheR_curveFit'
cFit4           <- list(amplitude=59193.591103772116, center=3698.697, sigma=0.082789238806238355, gamma=0.0026044299691057823, fitStatus=2, curveModel="skewedGaussian")
class(cFit4)    <- 'peakPantheR_curveFit'
found_curveFit  <- list(cFit1, cFit2, cFit3, cFit4)

# found ROIsDataPoint
found_ROIsDataPoints <- extractSignalRawData(tmp_raw_data, rt=input_ROI[,c('rtMin','rtMax')], mz=input_ROI[,c('mzMin','mzMax')], verbose=FALSE)


test_that('no peakStatistic, no plotEICsPath, no getAcquTime, no FIR, no verbose', {
  # Expected TIC
  expected_TIC            <- 2410533091
  # Expected peakTable
  expected_peakTable      <- found_peakTable[,1:14]
  # Expected curveFit
  expected_curveFit       <- found_curveFit
  # Expected ROIsDataPoint
  expected_ROIsDataPoint  <- found_ROIsDataPoints
  # Expected acquTime
  expected_acquTime       <- NA
  # Expected messages
  expected_messages       <- c("Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n")

	# results (output, warnings and messages)
  result_singleFileSearch <- evaluate_promise(peakPantheR_singleFileSearch(singleSpectraDataPath, input_ROI, peakStatistic=FALSE, plotEICsPath=NA, getAcquTime=FALSE, FIR=NULL, verbose=FALSE))

  # Check results
  expect_equal(result_singleFileSearch$result$TIC, expected_TIC)
  expect_equal(result_singleFileSearch$result$peakTable, expected_peakTable, tolerance=1e-6)
  expect_equal(result_singleFileSearch$result$curveFit, expected_curveFit, tolerance=1e-6)
  expect_equal(result_singleFileSearch$result$ROIsDataPoint, expected_ROIsDataPoint)
  expect_equal(result_singleFileSearch$result$acquTime, expected_acquTime)

  # Check messages (loading file)
  expect_equal(length(result_singleFileSearch$messages), 1)
  expect_equal(result_singleFileSearch$messages, expected_messages)
})

test_that('peakStatistic, no plotEICsPath, no getAcquTime, no FIR, no verbose', {
  # Expected TIC
  expected_TIC            <- 2410533091
  # Expected peakTable
  expected_peakTable      <- found_peakTable
  # Expected curveFit
  expected_curveFit       <- found_curveFit
  # Expected ROIsDataPoint
  expected_ROIsDataPoint  <- found_ROIsDataPoints
  # Expected acquTime
  expected_acquTime       <- NA
  # Expected messages
  expected_messages       <- c("Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n")
  
  # results (output, warnings and messages)
  result_singleFileSearch <- evaluate_promise(peakPantheR_singleFileSearch(singleSpectraDataPath, input_ROI, peakStatistic=TRUE, plotEICsPath=NA, getAcquTime=FALSE, FIR=NULL, verbose=FALSE))
  
  # Check results
  expect_equal(result_singleFileSearch$result$TIC, expected_TIC)
  expect_equal(result_singleFileSearch$result$peakTable, expected_peakTable, tolerance=1e-6)
  expect_equal(result_singleFileSearch$result$curveFit, expected_curveFit, tolerance=1e-6)
  expect_equal(result_singleFileSearch$result$ROIsDataPoint, expected_ROIsDataPoint)
  expect_equal(result_singleFileSearch$result$acquTime, expected_acquTime)
  
  # Check messages (loading file)
  expect_equal(length(result_singleFileSearch$messages), 1)
  expect_equal(result_singleFileSearch$messages, expected_messages)
})

test_that('no peakStatistic, no plotEICsPath, with getAcquTime, no FIR, verbose', {
  # Expected TIC
  expected_TIC            <- 2410533091
  # Expected peakTable
  expected_peakTable      <- found_peakTable[,1:14]
  # Expected curveFit
  expected_curveFit       <- found_curveFit
  # Expected ROIsDataPoint
  expected_ROIsDataPoint  <- found_ROIsDataPoints
  # Expected acquTime
  expected_acquTime       <- NA
  # Expected messages
  expected_messages       <- c("Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Check input, mzMLPath must be a .mzML\n", "Reading data from 4 windows\n", "Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #1\n", "Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #3\n")
  
  # results (output, warnings and messages)
  result_singleFileSearch <- evaluate_promise(peakPantheR_singleFileSearch(singleSpectraDataPath, input_ROI, peakStatistic=FALSE, plotEICsPath=NA, getAcquTime=TRUE, FIR=NULL, verbose=TRUE))
  
  # Check results
  expect_equal(result_singleFileSearch$result$TIC, expected_TIC)
  expect_equal(result_singleFileSearch$result$peakTable, expected_peakTable, tolerance=1e-6)
  expect_equal(result_singleFileSearch$result$curveFit, expected_curveFit, tolerance=1e-6)
  expect_equal(result_singleFileSearch$result$ROIsDataPoint, expected_ROIsDataPoint)
  expect_equal(result_singleFileSearch$result$acquTime, expected_acquTime)
  
  # Check messages (can't check timing)
  expect_equal(length(result_singleFileSearch$messages), 8)
  expect_equal(result_singleFileSearch$messages[c(1:3,5,6)], expected_messages)
})

test_that('no peakStatistic, with plotEICsPath, no getAcquTime, no FIR, verbose', {
  # Expected TIC
  expected_TIC            <- 2410533091
  # Expected peakTable
  expected_peakTable      <- found_peakTable[,1:14]
  # Expected curveFit
  expected_curveFit       <- found_curveFit
  # Expected ROIsDataPoint
  expected_ROIsDataPoint  <- found_ROIsDataPoints
  # Expected acquTime
  expected_acquTime       <- NA
  # Expected messages
  expected_messages       <- c("Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Reading data from 4 windows\n", "Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #1\n", "Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #3\n")
  
  # temporary file
  savePath                <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.png')

  # results (output, warnings and messages)
  result_singleFileSearch <- evaluate_promise(peakPantheR_singleFileSearch(singleSpectraDataPath, input_ROI, peakStatistic=FALSE, plotEICsPath=savePath, getAcquTime=FALSE, FIR=NULL, verbose=TRUE))
  
  # Check results
  expect_equal(result_singleFileSearch$result$TIC, expected_TIC)
  expect_equal(result_singleFileSearch$result$peakTable, expected_peakTable, tolerance=1e-6)
  expect_equal(result_singleFileSearch$result$curveFit, expected_curveFit, tolerance=1e-6)
  expect_equal(result_singleFileSearch$result$ROIsDataPoint, expected_ROIsDataPoint)
  expect_equal(result_singleFileSearch$result$acquTime, expected_acquTime)
  
  # Check plot has been produced
  expect_true(file.exists(savePath))

  # Check messages (can't check timing and save path)
  expect_equal(length(result_singleFileSearch$messages), 8)
  expect_equal(result_singleFileSearch$messages[c(1,2,4,5)], expected_messages)
})

test_that('peakStatistic, plotEICsPath, getAcquTime, FIR, verbose', {
  # Expected TIC
  expected_TIC            <- 2410533091
  # Expected peakTable
  expected_peakTable      <- found_peakTable
  # Expected curveFit
  expected_curveFit       <- found_curveFit
  # Expected ROIsDataPoint
  expected_ROIsDataPoint  <- found_ROIsDataPoints
  # Expected acquTime
  expected_acquTime       <- NA
  # Expected messages
  expected_messages       <- c("Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Check input, mzMLPath must be a .mzML\n", "Reading data from 4 windows\n", "Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #1\n", "Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #3\n")
  
  # temporary file
  savePath2               <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.png')
  
  # results (output, warnings and messages)
  result_singleFileSearch <- evaluate_promise(peakPantheR_singleFileSearch(singleSpectraDataPath, input_ROI, peakStatistic=TRUE, plotEICsPath=savePath2, getAcquTime=TRUE, FIR=NULL, verbose=TRUE))
  
  # Check results
  expect_equal(result_singleFileSearch$result$TIC, expected_TIC)
  expect_equal(result_singleFileSearch$result$peakTable, expected_peakTable, tolerance=1e-6)
  expect_equal(result_singleFileSearch$result$curveFit, expected_curveFit, tolerance=1e-6)
  expect_equal(result_singleFileSearch$result$ROIsDataPoint, expected_ROIsDataPoint)
  expect_equal(result_singleFileSearch$result$acquTime, expected_acquTime)
  
  # Check plot has been produced
  expect_true(file.exists(savePath2))
  
  # Check messages (can't check timing and save path)
  expect_equal(length(result_singleFileSearch$messages), 10)
  expect_equal(result_singleFileSearch$messages[c(1:3,5,6)], expected_messages)
})

test_that('change peak fitting params with ..., no peakStatistic, no plotEICsPath, no getAcquTime, no verbose', {
  # Expected TIC
  expected_TIC                      <- 2410533091
  # Expected peakTable
  expected_peakTable                <- found_peakTable[,1:14]
  expected_peakTable[3,2:11]        <- c(3418.0076795585401, 3455.6277710843374, 3495.4734240188186, 464.195358, 464.2000122, 464.204642, 11307215.264967661, 11538988, 380736, 381327.26552768378)
  # Expected curveFit
  expected_curveFit                 <- found_curveFit
  expected_curveFit[[3]]$amplitude  <- 64246.052173667762
  expected_curveFit[[3]]$center     <- 3450
  expected_curveFit[[3]]$sigma      <- 0.07533469863886906
  expected_curveFit[[3]]$gamma      <- 0.0019238229766131536
  expected_curveFit[[3]]$fitStatus  <- 2
  # Expected ROIsDataPoint
  expected_ROIsDataPoint  <- found_ROIsDataPoints
  # Expected acquTime
  expected_acquTime       <- NA
  # Expected messages
  expected_messages       <- c("Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n")
  
  # Modify fit of window #3
  tmp_params              <- list(init_params  = list(amplitude=1E5, center=3455., sigma=0.1, gamma=0),
                                  lower_bounds = list(amplitude=0,   center=3450., sigma=0,   gamma=-0.1),
                                  upper_bounds = list(amplitude=1E9, center=3460., sigma=5,   gamma=0.1))
  new_params              <- list('guess', 'guess', tmp_params, 'guess')
  
  # results (output, warnings and messages)
  result_singleFileSearch <- evaluate_promise(peakPantheR_singleFileSearch(singleSpectraDataPath, input_ROI, peakStatistic=FALSE, plotEICsPath=NA, getAcquTime=FALSE, FIR=NULL, verbose=FALSE, params=new_params))
  
  # Check results
  expect_equal(result_singleFileSearch$result$TIC, expected_TIC)
  expect_equal(result_singleFileSearch$result$peakTable, expected_peakTable, tolerance=1e-6)
  expect_equal(result_singleFileSearch$result$curveFit, expected_curveFit, tolerance=1e-5)
  expect_equal(result_singleFileSearch$result$ROIsDataPoint, expected_ROIsDataPoint)
  expect_equal(result_singleFileSearch$result$acquTime, expected_acquTime)
  
  # Check messages (loading file)
  expect_equal(length(result_singleFileSearch$messages), 1)
  expect_equal(result_singleFileSearch$messages, expected_messages)
})

test_that('no targetFeatures on import, no peakStatistic', {
  # Empty targetFeatures
  empty_ROI   <- data.frame(matrix(vector(), 0, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))),stringsAsFactors=FALSE)
  
  # Expected TIC
  expected_TIC            <- 2410533091
  # Expected peakTable
  expected_peakTable      <- data.frame(matrix(vector(), 0, 14, dimnames=list(c(), c('cpdID', 'cpdName', 'found', 'rt', 'rtMin', 'rtMax', 'mz', 'mzMin', 'mzMax', 'peakArea', 'peakAreaRaw', 'maxIntMeasured', 'maxIntPredicted', 'is_filled'))), stringsAsFactors=FALSE)
  # Expected curveFit
  expected_curveFit       <- list()
  # Expected ROIsDataPoint
  expected_ROIsDataPoint  <- list()
  # Expected acquTime
  expected_acquTime       <- NA
  # Expected messages
  expected_messages       <- c("Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Reading data from 0 windows\n", "No data exist for the rt provided\n", "- No target features passed in 'targetFeatTable', no integration, only TIC will be reported -\n")
  
  # results (output, warnings and messages)
  result_singleFileSearch <- evaluate_promise(peakPantheR_singleFileSearch(singleSpectraDataPath, empty_ROI, peakStatistic=FALSE, plotEICsPath=NA, getAcquTime=FALSE, FIR=NULL, verbose=TRUE))
  
  # Check results
  expect_equal(result_singleFileSearch$result$TIC, expected_TIC)
  expect_equal(result_singleFileSearch$result$peakTable, expected_peakTable, tolerance=1e-6)
  expect_equal(result_singleFileSearch$result$curveFit, expected_curveFit, tolerance=1e-6)
  expect_equal(result_singleFileSearch$result$ROIsDataPoint, expected_ROIsDataPoint)
  expect_equal(result_singleFileSearch$result$acquTime, expected_acquTime)
  
  # Check messages (loading file)
  expect_equal(length(result_singleFileSearch$messages), 5)
  expect_equal(result_singleFileSearch$messages[1:4], expected_messages)
})

test_that('no targetFeatures on import, peakStatistic', {
  # Empty targetFeatures
  empty_ROI   <- data.frame(matrix(vector(), 0, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))),stringsAsFactors=FALSE)
  
  # Expected TIC
  expected_TIC            <- 2410533091
  # Expected peakTable
  expected_peakTable      <- data.frame(matrix(vector(), 0, 18, dimnames=list(c(), c('cpdID', 'cpdName', 'found', 'rt', 'rtMin', 'rtMax', 'mz', 'mzMin', 'mzMax', 'peakArea', 'peakAreaRaw', 'maxIntMeasured', 'maxIntPredicted', 'is_filled', 'ppm_error', 'rt_dev_sec', 'tailingFactor', 'asymmetryFactor'))), stringsAsFactors=FALSE)
  # Expected curveFit
  expected_curveFit       <- list()
  # Expected ROIsDataPoint
  expected_ROIsDataPoint  <- list()
  # Expected acquTime
  expected_acquTime       <- NA
  # Expected messages
  expected_messages       <- c("Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Reading data from 0 windows\n", "No data exist for the rt provided\n", "- No target features passed in 'targetFeatTable', no integration, only TIC will be reported -\n")
  
  # results (output, warnings and messages)
  result_singleFileSearch <- evaluate_promise(peakPantheR_singleFileSearch(singleSpectraDataPath, empty_ROI, peakStatistic=TRUE, plotEICsPath=NA, getAcquTime=FALSE, FIR=NULL, verbose=TRUE))
  
  # Check results
  expect_equal(result_singleFileSearch$result$TIC, expected_TIC)
  expect_equal(result_singleFileSearch$result$peakTable, expected_peakTable, tolerance=1e-6)
  expect_equal(result_singleFileSearch$result$curveFit, expected_curveFit, tolerance=1e-6)
  expect_equal(result_singleFileSearch$result$ROIsDataPoint, expected_ROIsDataPoint)
  expect_equal(result_singleFileSearch$result$acquTime, expected_acquTime)
  
  # Check messages (loading file)
  expect_equal(length(result_singleFileSearch$messages), 5)
  expect_equal(result_singleFileSearch$messages[1:4], expected_messages)
})

test_that('no features found, no FIR', {
  # targetFeature that doesn't give result
  noMatch_ROI                   <- input_ROI[1,]
  noMatch_ROI[1, 6:8]           <- c(52.194778, 52.2, 52.205222)

  # Expected TIC
  expected_TIC                  <- 2410533091
  # Expected peakTable
  expected_peakTable            <- data.frame(matrix(vector(), 1, 14, dimnames=list(c(), c('found', 'rtMin', 'rt', 'rtMax', 'mzMin', 'mz', 'mzMax', 'peakArea', 'peakAreaRaw', 'maxIntMeasured', 'maxIntPredicted', 'cpdID', 'cpdName', 'is_filled'))), stringsAsFactors=FALSE)
  expected_peakTable$cpdID      <- "ID-1"
  expected_peakTable$cpdName    <- "testCpd 1"
  expected_peakTable$found      <- FALSE
  expected_peakTable$is_filled  <- FALSE
  # Expected curveFit
  expected_curveFit             <- list(NA)
  # Expected ROIsDataPoint
  expected_ROIsDataPoint        <- list(data.frame(rt=numeric(), mz=numeric(), int=numeric(), stringsAsFactors=FALSE))
  # Expected acquTime
  expected_acquTime             <- NA
  # Expected messages
  expected_messages             <- c( "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Reading data from 1 windows\n", "Fit of ROI #1 is unsuccessful (try err)\n")

  # results (output, warnings and messages)
  result_singleFileSearch       <- evaluate_promise(peakPantheR_singleFileSearch(singleSpectraDataPath, noMatch_ROI, peakStatistic=FALSE, plotEICsPath=NA, getAcquTime=FALSE, FIR=NULL, verbose=TRUE))
  
  # Check results
  expect_equal(result_singleFileSearch$result$TIC, expected_TIC)
  expect_equal(result_singleFileSearch$result$peakTable, expected_peakTable, tolerance=1e-6)
  expect_equal(result_singleFileSearch$result$curveFit, expected_curveFit, tolerance=1e-6)
  expect_equal(result_singleFileSearch$result$ROIsDataPoint, expected_ROIsDataPoint)
  expect_equal(result_singleFileSearch$result$acquTime, expected_acquTime)
  
  # Check messages (loading file)
  expect_equal(length(result_singleFileSearch$messages), 6)
  expect_equal(result_singleFileSearch$messages[c(1,2,4)], expected_messages)
})

test_that('one feature not found (#3), use FIR', {
  # input ROI, no window #3
  tmp_ROI                             <- input_ROI
  tmp_ROI[3,c('mzMin','mz','mzMax')]  <- c(46.19, 46.2, 46.21)
  # input FIR (use previously found peaks dimension. Need to widen mz as this test data doesn't give any mz width)
  input_FIR               <- found_peakTable[,c("mzMin", "mzMax", "rtMin", "rtMax")]
  input_FIR$mzMin         <- input_FIR$mzMin - 0.0005
  input_FIR$mzMax         <- input_FIR$mzMax + 0.0005
  
  # Expected TIC
  expected_TIC            <- 2410533091
  # Expected peakTable
  expected_peakTable      <- found_peakTable[,1:14]
  expected_peakTable[3,c('rtMin','rt', 'rtMax', 'mzMin', 'mz', 'mzMax', 'peakArea', 'peakAreaRaw', 'maxIntMeasured', 'maxIntPredicted')] <- c(3451.20759036145, 3454.435, 3501.66975049245, 464.194858, 464.2, 464.205142, 7744647, 7744647, 380736, as.numeric(NA))
  expected_peakTable$found[3]     <- TRUE
  expected_peakTable$is_filled[3] <- TRUE
  # Expected curveFit
  expected_curveFit       <- found_curveFit
  expected_curveFit[[3]]  <- NA
  # Expected ROIsDataPoint
  expected_ROIsDataPoint      <- found_ROIsDataPoints
  expected_ROIsDataPoint[[3]] <- data.frame(rt=numeric(), mz=numeric(), int=numeric(), stringsAsFactors=FALSE)
  # Expected acquTime
  expected_acquTime       <- NA
  # Expected messages
  expected_messages       <- c("Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Reading data from 4 windows\n", "Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #1\n", "Fit of ROI #3 is unsuccessful (try err)\n", "1 features to integrate with FIR\n", "Reading data from 1 windows\n")
  
  # results (output, warnings and messages)
  result_singleFileSearch <- evaluate_promise(peakPantheR_singleFileSearch(singleSpectraDataPath, tmp_ROI, peakStatistic=FALSE, plotEICsPath=NA, getAcquTime=FALSE, FIR=input_FIR, verbose=TRUE))
  
  # Check results
  expect_equal(result_singleFileSearch$result$TIC, expected_TIC)
  expect_equal(result_singleFileSearch$result$peakTable, expected_peakTable, tolerance=1e-6)
  expect_equal(result_singleFileSearch$result$curveFit, expected_curveFit, tolerance=1e-6)
  expect_equal(result_singleFileSearch$result$ROIsDataPoint, expected_ROIsDataPoint)
  expect_equal(result_singleFileSearch$result$acquTime, expected_acquTime)
  
  # Check messages (can't check timing)
  expect_equal(length(result_singleFileSearch$messages), 11)
  expect_equal(result_singleFileSearch$messages[c(1,2,4,5,7,8)], expected_messages)
})

test_that('raise errors', {
  
  # paths to trigger errors
  fakeInputPath     <- './notAPath/test.CDF'
  fakeOutputPath    <- './notAPath/save.png'
  wrongOutputFormat <- './notAPNG.txt'
  # FIR to trigger errors
  notADataFrame     <- 'not at dataframe'
  wrongNbrows       <- data.frame(matrix(vector(), 2, 4, dimnames=list(c(), c("mzMin", "mzMax", "rtMin", "rtMax"))), stringsAsFactors=FALSE)
  wrongCols         <- data.frame(matrix(vector(), 4, 4, dimnames=list(c(), c("wrongCol name", "mzMax", "rtMin", "rtMax"))), stringsAsFactors=FALSE)

  # singleSpectraDataPath doesnt exist
  expect_error(peakPantheR_singleFileSearch(singleSpectraDataPath=fakeInputPath, input_ROI), "Check input, file*")
  # save folder doesnt exist
  expect_error(peakPantheR_singleFileSearch(singleSpectraDataPath, input_ROI, plotEICsPath=fakeOutputPath), "Check input, plotEICsPath folder*")
  # save file is not a .png
  expect_error(peakPantheR_singleFileSearch(singleSpectraDataPath, input_ROI, plotEICsPath=wrongOutputFormat), "Check input, plotEICsPath file name*")

  # not implemented curveModel
  expect_error(peakPantheR_singleFileSearch(singleSpectraDataPath, input_ROI, curveModel='FakeModel'), "Error: \"curveModel\" must be one of: skewedGaussian, emgGaussian")
  # FIR is not a data.frame
  expect_error(peakPantheR_singleFileSearch(singleSpectraDataPath, input_ROI, FIR=notADataFrame), "Check input, FIR must be a data.frame not character")
  # FIR has wrong number of rows
  expect_error(peakPantheR_singleFileSearch(singleSpectraDataPath, input_ROI, FIR=wrongNbrows), "Check input, FIR must have the same number of rows as targetFeatTable")
  # FIR columns are wrong
  expect_error(peakPantheR_singleFileSearch(singleSpectraDataPath, input_ROI, FIR=wrongCols), 'Check input, FIR must have "mzMin", "mzMax", "rtMin" and "rtMax" as columns')
  
  # rtMin > rtMax
  wrong_ROI_rt     <- input_ROI
  wrong_ROI_rt[2,] <- c("ID-2", "testCpd 2", 3440., 3385.577, 3280., 496.195038, 496.2, 496.204962)
  expect_error(peakPantheR_singleFileSearch(singleSpectraDataPath, wrong_ROI_rt), 'Check input, "rtMin" must be <= to "rtMax"')
  # mzMin > mzMax
  wrong_ROI_mz     <- input_ROI
  wrong_ROI_mz[3,] <- c("ID-3", "testCpd 3", 3420., 3454.435, 3495., 464.204642, 464.2, 464.195358)
  expect_error(peakPantheR_singleFileSearch(singleSpectraDataPath, wrong_ROI_mz), 'Check input, "mzMin" must be <= to "mzMax"')
})

#' peakPantheR Graphical User Interface
#'
#' peakPantheR Graphical User Interface (GUI) implements all the functions for
#' the parallel detection, integration and reporting of pre-defined features in
#' multiple mass spectrometry data files. To exit press \code{ESC} in the
#' command line.
#' 
#' @param browser If TRUE open the graphical user interface in a web browser
#' instead of a R window. Default is TRUE
#'
#' @return None, start GUI. To exit press \code{ESC} in the command line.
#'
#' @examples
#' print("Start graphical interface, press 'ESC' in the command line to stop")
#' # peakPantheR_start_GUI()
#'
#' @import foreach
#' @import doParallel
#' @import shiny
#' @importFrom bslib bs_theme
#' @import shinycssloaders
#' @importFrom DT DTOutput renderDT datatable
#'
#' @export
peakPantheR_start_GUI <- function(browser=TRUE) {
    appDir <- system.file("shiny-GUI", "peakPantheR-App", package="peakPantheR")
    if (appDir == "") {
        stop("Could not find example directory. Try re-installing ",
            "`peakPantheR`.", call. = FALSE)
    }
    # Either browser or interactive as defined in the IDE/RStudio
    if( browser ){
        shiny::runApp(appDir, launch.browser=TRUE)
    } else {
        shiny::runApp(appDir, launch.browser=getOption("shiny.launch.browser",
                        interactive()))
    }
}

#' UI data import helper - initialise new annotation from files
#'
#' Fully initialise a \code{peakPantheRAnnotation} using the target files path,
#' CSV parameter path and metadata.
#'
#' @param CSVParamPath (str) Path to a CSV file of fit parameters
#' @param spectraPaths (str) character vector of spectra file paths, to set
#' samples to process
#' @param cpdMetadataPath NULL or path to a csv of compound metadata, with
#' compounds as row and metadata as columns
#' @param spectraMetadata NULL or DataFrame of sample metadata, with
#' samples as row and metadata as columns
#' @param verbose (bool) If TRUE message progress
#'
#' @return (peakPantheRAnnotation) Object initialised with ROI, uROI and FIR
#' read from the CSV file
#'
#' @export
#'
#' @examples
#' ## Input data
#' input_CSV <- data.frame(matrix(nrow=2,ncol=21,dimnames=list(c(),
#'     c('cpdID', 'cpdName',
#'     'X','ROI_rt', 'ROI_mz', 'ROI_rtMin', 'ROI_rtMax','ROI_mzMin','ROI_mzMax',
#'     'X','uROI_rtMin', 'uROI_rtMax', 'uROI_mzMin', 'uROI_mzMax', 'uROI_rt',
#'     'uROI_mz', 'X', 'FIR_rtMin', 'FIR_rtMax', 'FIR_mzMin', 'FIR_mzMax'))))
#' input_CSV[1,]  <- c('ID-1', 'Cpd 1', '|', 1.,  2.,  3.,  4.,  5.,  6.,  '|',
#'                     7.,  8.,  9.,  10., 11., 12., '|', 13., 14., 15., 16.)
#' input_CSV[2,]  <- c('ID-2', 'Cpd 2', '|', 17., 18., 19., 20., 21., 22., '|',
#'                     23., 24., 25., 26., 27., 28., '|', 29., 30., 31., 32.)
#' input_CSV[,-c(1,2,3,10,17)]  <- vapply(input_CSV[,-c(1,2,3,10,17)],
#'                                             as.numeric, FUN.VALUE=numeric(2))
#'
#' input_spectraPaths    <- c('./path/file1', './path/file2', './path/file3')
#'
#' # temporary file location
#' savePath1      <- tempfile(pattern='file', tmpdir=tempdir(), fileext='.csv')
#' # save csv
#' utils::write.csv(input_CSV, file=savePath1, row.names=FALSE)
#'
#' # Load parameters from CSV
#' loadedAnnotation <- initialise_annotation_from_files_UI_helper(savePath1,
#'                                                         input_spectraPaths,
#'                                                         verbose=TRUE)
#' # An object of class peakPantheRAnnotation
#' #   2 compounds in 3 samples.
#' #   updated ROI exist (uROI)
#' #   does not use updated ROI (uROI)
#' #   does not use fallback integration regions (FIR)
#' #   is not annotated
initialise_annotation_from_files_UI_helper <- function(CSVParamPath,
                                                        spectraPaths,
                                                        cpdMetadataPath = NULL,
                                                        spectraMetadata = NULL,
                                                        verbose = TRUE) {
    # Initialise with parameters
    init_annotation <- peakPantheR_loadAnnotationParamsCSV(CSVParamPath,
                                                            verbose = FALSE)
    # Add spectraPaths
    init_annotation <- resetAnnotation(init_annotation,
                                        spectraPaths = spectraPaths,
                                        verbose = FALSE)
    # Load metadata
    cpdMetadata     <- NULL
    if (!is.null(cpdMetadataPath)) {
        if (!file.exists(cpdMetadataPath)) {
            stop('Err','or: cpdMetadata file does not exist') }
        tmp_cpdMeta <- read.csv(cpdMetadataPath, header=TRUE, sep=",",
                                quote="\"", stringsAsFactors=FALSE)
        # check nb rows match the nb of cpd
        if (dim(tmp_cpdMeta)[1] != nbCompounds(init_annotation)) {
            message('War','ning: cpdMetadata number of rows (',
                    dim(tmp_cpdMeta)[1],
                    ') does not match the number of compounds targeted (',
                    nbCompounds(init_annotation), ')')
        } else { cpdMetadata <- tmp_cpdMeta }
    }
    if (!is.null(spectraMetadata)) {
        if (!is.data.frame(spectraMetadata)) {
            stop('Err','or: spectraMetadata is not a DataFrame') }
        # check nb of rows match the nb of spectra
        if (dim(spectraMetadata)[1] != nbSamples(init_annotation)) {
            message('War','ning: spectraMetadata number of rows (',
                    dim(spectraMetadata)[1],
                    ') does not match the number of compounds targeted (',
                    nbSamples(init_annotation), ')')
            spectraMetadata <- NULL
        } # else keep the spectraMetadata value
    }

    # Update annotation with metadata
    init_annotation <- resetAnnotation(init_annotation,
                                        cpdMetadata = cpdMetadata,
                                        spectraMetadata = spectraMetadata,
                                        verbose = FALSE)
    # return success message
    if(verbose){ message(show(init_annotation)) }
    return(init_annotation)
}


#' UI data import helper - prepare file paths and metadata
#'
#' Return spectraPaths and spectraMetadata from a .csv file (if available).
#' If reading from the spectraMetadata file, the spectraPaths are taken from
#' the `filepath` column
#'
#' @param spectraPaths NULL or character vector of spectra file paths, to set
#' samples to process
#' @param spectraMetadataPath NULL or path to a csv of spectra metadata, with
#' spectra as row and metadata as columns. (spectraPaths in column `filepath`)
#'
#' @return spectraPaths (str) and spectraMetadata (DataFrame or NULL) read from
#' the CSV file
#'
#' @export
#'
#' @examples
#' ## Input data
#' # spectraPath
#' input_spectraPaths    <- c('./path/file1', './path/file2', './path/file3')
#'
#' # spectraMetadata
#' input_spectraMetadata <- data.frame(matrix(data=c(input_spectraPaths,
#'                          c('a','b','c')), nrow=3, ncol=2,
#'                          dimnames=list(c(),c('filepath', 'testcol')),
#'                          byrow=FALSE), stringsAsFactors=FALSE)
#'
#' # temporary file location
#' spectraMetaPath <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.csv')
#' # save csv
#' utils::write.csv(input_spectraMetadata,
#'                  file=spectraMetaPath,
#'                  row.names=FALSE)
#'
#' # load data from CSV
#' spectraPaths_and_metadata_UI_helper(spectraPaths = NULL,
#'                                      spectraMetadataPath = spectraMetaPath)
spectraPaths_and_metadata_UI_helper <- function(spectraPaths = NULL,
                                                spectraMetadataPath = NULL) {
    # None are set
    if (is.null(spectraPaths) & is.null(spectraMetadataPath)) {
        stop('Err','or: spectraPaths and spectraMetadataPath are not set')
    }

    # Only spectraPaths are set
    if (!is.null(spectraPaths) & is.null(spectraMetadataPath)) {
        out_path <- spectraPaths
        out_meta <- NULL
    }

    # spectraMetadataPath is set
    if (!is.null(spectraMetadataPath)) {

        # load file
        if (!file.exists(spectraMetadataPath)) {
                stop('Err','or: spectraMetadata file does not exist') }
        tmp_spectraMeta <- read.csv(spectraMetadataPath, header=TRUE, sep=",",
                                    quote="\"", stringsAsFactors=FALSE)

        # check 'filepath' is in columns
        if (!('filepath' %in% colnames(tmp_spectraMeta))) {
            stop("Err","or: the column 'filepath' must be present in the ",
                "spectraMetadata")
        }

        # extract filepaths
        out_path <- tmp_spectraMeta[,'filepath']
        # remove filepaths from metadata
        out_meta <- tmp_spectraMeta[ , !(names(tmp_spectraMeta) %in%
                                        c('filepath')), drop=FALSE]
    }

    return(list(spectra=out_path, meta=out_meta))
}


#' UI data import helper - check loaded annotation
#'
#' Load a .RData file (check it exists) and that a peakPantheRAnnotation named
#' "annotationObject" is present. Returns the annotation if everything is valid
#'
#' @param annotationPath (str) Path to a RData file containing a
#' peakPantheRAnnotation names `annotationObject`
#'
#' @return (peakPantheRAnnotation) Object loaded from file
#'
#' @export
#'
#' @examples
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 compounds
#'
#' ## Inputs
#' # spectraPaths
#' spectraPaths <- c('./path/file1', './path/file2', './path/file3')
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotationObject <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                         targetFeatTable=targetFeatTable)
#'
#' # save annotation to disk
#' annotPath <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.RData')
#' save(annotationObject, file=annotPath, compress=TRUE)
#'
#' # Load annotation
#' load_annotation_from_file_UI_helper(annotationPath = annotPath)
#' # An object of class peakPantheRAnnotation
#' #   2 compounds in 3 samples.
#' #   updated ROI do not exist (uROI)
#' #   does not use updated ROI (uROI)
#' #   does not use fallback integration regions (FIR)
#' #   is not annotated
load_annotation_from_file_UI_helper <- function(annotationPath) {
    # Check file exist
    if (!file.exists(annotationPath)) {
        stop('Err','or: annotation file does not exist') }

    # load file content
    load(annotationPath)

    # check it exist and is named correctly
    if (length(ls()[ls() == 'annotationObject']) !=1) {
        stop("Err","or: annotation file must contain a ",
            "`peakPantheRAnnotation` named 'annotationObject'") }
    # dummy initialisation to pass BiocCheck. In no case would the code reach
    # this section if `annotationObject` wasn't present in the environment
    if (0) {annotationObject <- NULL}

    # check it's a peakPantheRAnnotation
    if (!is(annotationObject, "peakPantheRAnnotation")) {
        stop("Err","or: the variable loaded is not a `peakPantheRAnnotation`") }

    return(annotationObject)
}


#' UI show annotation helper - list of properties
#'
#' show method specific to the UI, that returns each field in a named list to
#' ease display
#'
#' @param annotation (peakPantherAnnotation) Object to describe
#'
#' @return (list) Named list of annotation properties
#'
#' @export
#'
#' @examples
#' # Initialise an empty annotation, no uROI, no use of FIR
#' annotInit     <- peakPantheRAnnotation()
#'
#' # return properties
#' annotation_showMethod_UI_helper(annotInit)
#' # $nbCompounds
#' # [1] 0
#' #
#' # #$nbSamples
#' # [1] 0
#' #
#' # #$uROIExist
#' # [1] FALSE
#' #
#' # $useUROI
#' # [1] FALSE
#' #
#' # $useFIR
#' # [1] FALSE
#' #
#' # $isAnnotated
#' # [1] FALSE
annotation_showMethod_UI_helper <- function(annotation){
    properties <- list(nbCompounds = nbCompounds(annotation),
                        nbSamples = nbSamples(annotation),
                        uROIExist = uROIExist(annotation),
                        useUROI = useUROI(annotation),
                        useFIR = useFIR(annotation),
                        isAnnotated = isAnnotated(annotation)
    )

    return(properties)
}


#' UI show annotation helper - UI sidebar string
#'
#' Return a text description of an annotation for UI presentation
#'
#' @param annotProp (list) Named list of annotation properties as
#' created by \code{annotation_showMethod_UI_helper()}
#'
#' @return (str) Textual description of the annotation to show on UI
#'
#' @export
#'
#' @examples
#' # Input
#' properties_default <- list(nbCompounds = 0,
#'                            nbSamples = 0,
#'                            uROIExist = FALSE,
#'                            useUROI = FALSE,
#'                            useFIR = FALSE,
#'                            isAnnotated = FALSE)
#'
#' # Generate description
#' annotation_showText_UI_helper(properties_default)
#' # [[1]]
#' # [1] "Not annotated"
#' #
#' # [[2]]
#' # [1] "0 compounds"
#' #
#' # [[3]]
#' # [1] "0 samples"
#' #
#' # [[4]]
#' # [1] "updated ROI do not exist (uROI)"
#' #
#' # [[5]]
#' # [1] "does not use updated ROI (uROI)"
#' #
#' # [[6]]
#' # [1] "does not use fallback integration regions (FIR)"
annotation_showText_UI_helper <- function(annotProp){
    UI_string <- list(
        if (annotProp$isAnnotated) {'Is annotated'} else {'Not annotated'},
        paste0(annotProp$nbCompounds, ' compounds'),
        paste0(annotProp$nbSamples, ' samples'),
        if (annotProp$uROIExist) {'updated ROI exist (uROI)'
        } else {'updated ROI do not exist (uROI)'},
        if (annotProp$useUROI) {'uses updated ROI (uROI)'
        } else {'does not use updated ROI (uROI)'},
        if (annotProp$useFIR) {'uses fallback integration regions (FIR)'
        } else {'does not use fallback integration regions (FIR)'}
    )

    return(UI_string)
}

#' UI diagnostic plot helper - single feature multiplot
#'
#' Return a ggplot object of a feature diagnostic multiplot
#'
#' @param cpdNb (int) position of the feature to extract (1 to nbCpd)
#' @param annotation (peakPantheRAnnotation) Annotation object
#' @param splNum (int) NULL or number of spectra to plot, chosen randomly
#' from all spectra. If NULL or equal to the total number of spectra, plot all
#' spectra
#' @param splColrColumn (str) NULL, None or a spectraMetadata column for
#' colouring each sample
#' @param ... Additional parameters for plotting
#'
#' @return (ggplotObject) Diagnostic multiplot for a feature
#'
#' @export
#'
#' @examples
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' spectraPaths <- c('./path/file1', './path/file2', './path/file3')
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' emptyAnnotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                         targetFeatTable=targetFeatTable)
#'
#' # Plot of an empty annotation
#' annotation_diagnostic_multiplot_UI_helper(cpdNb = 2,
#'                                          annotation = emptyAnnotation,
#'                                          splNum = NULL,
#'                                          splColrColumn = NULL)
#' # Warning: the object has not been annotated, return an empty diagnostic
#' # plot list
annotation_diagnostic_multiplot_UI_helper <- function(cpdNb, annotation,
                                        splNum=NULL, splColrColumn=NULL, ...) {
    # subset the compound and sample according to inputs
    tmp_annotat <- subset_annot_diag_plot_UI_helper(cpdNb, annotation, splNum)

    # convert sampleColourColumn to a colour scale to use
    sampleColour <- spectra_metadata_colourScheme_UI_helper(tmp_annotat,
                                                            splColrColumn)

    # diagnostic plots
    tmp_diagPlotList <- annotationDiagnosticPlots(tmp_annotat,
        sampleColour = sampleColour, verbose = FALSE, ...)

    # multiplot
    suppressMessages(suppressWarnings(
        tmp_multiPlot <- annotationDiagnosticMultiplot(
            tmp_diagPlotList)))

    # something to plot
    if (length(tmp_multiPlot) != 0) {
        return(tmp_multiPlot)
    # nothing to plot
    } else { return(ggplot2::ggplot() + ggplot2::theme_void()) }
}

subset_annot_diag_plot_UI_helper <- function(cpdNb, annotation, splNum=NULL) {
    # subset the compound and sample according to inputs

    nSpl <- nbSamples(annotation)
    if (is.null(splNum)) { splNum <- nSpl }

    # fix extrem values allowed by UI
    if (splNum < 1) {
        splNum <- 1
        warning("Negative number of samples to show, 1 spectra will be shown!")
    }
    if (splNum > nSpl) {
        splNum <- nSpl
        warning("More samples to show than available,",
                " all spectra will be shown!")
    }

    # appoximately equally spaced
    currentSplChoice  <- round(seq(1, nSpl, length.out=splNum))

    # subset annotation to only 1 cpd and subset of samples
    subsetAnnotation <- annotation[currentSplChoice, cpdNb]

    return(subsetAnnotation)
}

#' UI diagnostic table - fit summary
#'
#' Return a table of fit statistic (ratio of peaks found, ratio of peaks filled,
#' ppm error, RT deviation)
#'
#' @param annot (peakPantheRAnnotation) Annotation object
#'
#' @return (data.frame) Fit statistics
#'
#' @export
#'
#' @examples
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' spectraPaths <- c('./path/file1', './path/file2', './path/file3')
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' emptyAnnotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                         targetFeatTable=targetFeatTable)
#'
#' # statistics of an empty annotation
#' annotation_fit_summary_UI_helper(emptyAnnotation)
#' #              Ratio peaks found (%) Ratio peaks filled (%) ppm error
#' # ID-1 - Cpd 1                    NA                      0       NaN
#' # ID-2 - Cpd 2                    NA                      0       NaN
#' #               RT deviation (s)
#' # ID-1 - Cpd 1               NaN
#' # ID-2 - Cpd 2               NaN
annotation_fit_summary_UI_helper <- function(annot) {
    summary <- data.frame(matrix(ncol = 0, nrow = nbCompounds(annot)),
                                    stringsAsFactors = FALSE)
    rownames(summary) <- paste(cpdID(annot), "-", cpdName(annot))

    # used FIR (found = not filled, filled from is_filled)
    if (annot@useFIR) {
        summary$ratio_peaks_found  <- 1 - (colSums(
            annotationTable(annot, column = "is_filled"))/nbSamples(annot))
        summary$ratio_peaks_filled <- (colSums(
            annotationTable(annot, column = "is_filled"))/nbSamples(annot))

    # didn't use FIR (found from found, None are filled)
    } else {
        summary$ratio_peaks_found  <- colSums(
            annotationTable(annot, column = "found"))/nbSamples(annot)
        summary$ratio_peaks_filled <- 0
    }

    summary$ppm_error <-
        colMeans(annotationTable(annot, column = "ppm_error"), na.rm = TRUE)
    summary$rt_dev_sec <-
        colMeans(annotationTable(annot, column = "rt_dev_sec"), na.rm = TRUE)

    # nicer format for output
    summary$ratio_peaks_found  <- round(summary$ratio_peaks_found*100,1)
    summary$ratio_peaks_filled <- round(summary$ratio_peaks_filled*100,1)
    colnames(summary) <- c('Ratio peaks found (%)','Ratio peaks filled (%)',
                            'ppm error', 'RT deviation (s)')

    return(summary)
}


#' UI export helper - spectra path and metadata
#'
#' Return a table with spectra as rows and filepath and all spectra metadata
#' columns
#'
#' @param annot (peakPantheRAnnotation) Annotation object
#'
#' @return (data.frame) Spectra paths and metadata
#'
#' @export
#'
#' @examples
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' spectraPaths <- c('./path/file1', './path/file2', './path/file3')
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' emptyAnnotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                         targetFeatTable=targetFeatTable)
#'
#' # spectraMetada of an empty annotation
#' outputAnnotationSpectraMetadata_UI_helper(emptyAnnotation)
#' #       filepath
#' # 1 ./path/file1
#' # 2 ./path/file2
#' # 3 ./path/file3
outputAnnotationSpectraMetadata_UI_helper <- function(annot) {
    # collect the data
    tmp_filepath        <- filepath(annot)
    tmp_spectraMetadata <- spectraMetadata(annot)

    # generate table
    outSpecMeta <- data.frame(filepath = tmp_filepath)
    outSpecMeta <- cbind(outSpecMeta, tmp_spectraMetadata)

    return(outSpecMeta)
}


#' UI export helper - feature metadata
#'
#' Return a table with features as rows and all feature metadata as columns
#'
#' @param annot (peakPantheRAnnotation) Annotation object
#'
#' @return (data.frame) Features metadata
#'
#' @export
#'
#' @examples
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' spectraPaths <- c('./path/file1', './path/file2', './path/file3')
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' emptyAnnotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                         targetFeatTable=targetFeatTable)
#'
#' # featureMetadata of an empty annotation
#' outputAnnotationFeatureMetadata_UI_helper(emptyAnnotation)
#' # data frame with 0 columns and 2 rows
outputAnnotationFeatureMetadata_UI_helper <- function(annot) {
    # collect the data
    outCpdMetadata <- cpdMetadata(annot)

    return(outCpdMetadata)
}


#' UI export plot helper - sample colour
#'
#' Return a vector of spectra colours based on a metadata column
#'
#' @param annot (peakPantheRAnnotation) Annotation object
#' @param splColrColumn (str) NULL, None or a spectraMetadata column for
#' colouring each sample
#'
#' @return (character) Vector of colours
#'
#' @export
#'
#' @examples
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' spectraPaths <- c('./path/file1', './path/file2', './path/file3')
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' emptyAnnotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                         targetFeatTable=targetFeatTable)
#'
#' # colour scheme with no spectraMetadata
#' outputAnnotationFeatureMetadata_UI_helper(emptyAnnotation)
#' # NULL
spectra_metadata_colourScheme_UI_helper <- function(annot,splColrColumn=NULL) {
    # convert sampleColourColumn to a colour scale to use
    if (is.null(splColrColumn)) {
        sampleColour <- NULL
    } else if (splColrColumn=='None') { # separate for case not set yet
        sampleColour <- NULL
    } else if (!(splColrColumn %in% colnames(
        peakPantheR::spectraMetadata(annot)))) {
        sampleColour <- NULL
    } else {
        # extract metadata column of interest and unique colours needed
        tmp_meta <- peakPantheR::spectraMetadata(annot)[[splColrColumn]]
        uniq_val <- unique(tmp_meta)
        n_colr   <- length(uniq_val)

        # known colors (repeat if more groups than colours available)
        colorVect <- c("blue", "red", "green", "orange", "purple", "seagreen",
                        "darkturquoise", "violetred", "saddlebrown", "black")
        if (length(colorVect) < n_colr) {
            colorVect <- rep(colorVect, ceiling(n_colr/length(colorVect)))
        }
        # iteratively replace values by colours
        for (i in seq_len(n_colr)) {
            tmp_meta <- replace(tmp_meta, tmp_meta==uniq_val[i], colorVect[i])
        }
        sampleColour <- tmp_meta
    }

    return(sampleColour)
} # TODO: unittest

context('spectraPaths_and_metadata_UI_helper()')

# test only spectraPaths
# test only spectraMetadataPaths (or both)
# test error

## Input data
# spectraPath
input_spectraPaths    <- c('./path/file1', './path/file2', './path/file3')

# spectraMetadata
input_spectraMetadata <- data.frame(matrix(data=c(input_spectraPaths, c('a','b','c')), nrow=3, ncol=2, dimnames=list(c(),c('filepath', 'testcol')), byrow=FALSE), stringsAsFactors=FALSE)

# temporary files
spectraMetaPath       <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.csv')
utils::write.csv(input_spectraMetadata, file=spectraMetaPath, row.names=FALSE)


test_that('spectraPaths, no spectraMetadata', {
    # expected
    expected    <- list(spectra=input_spectraPaths, meta=NULL)

    # results (output, warnings and messages)
    result_load <- evaluate_promise(spectraPaths_and_metadata_UI_helper(spectraPaths = input_spectraPaths,
                                                                        spectraMetadataPath = NULL))
    # Check result
    expect_equal(result_load$result, expected)

    # Check result messages (in output)
    expect_equal(length(result_load$messages), 0)
    expect_equal(result_load$output, "")
})

test_that('no spectraPaths, spectraMetadata', {
    # expected
    expected    <- list(spectra=input_spectraPaths, meta=input_spectraMetadata[,'testcol', drop=FALSE])

    # results (output, warnings and messages)
    result_load <- evaluate_promise(spectraPaths_and_metadata_UI_helper(spectraPaths = NULL,
                                                                        spectraMetadataPath = spectraMetaPath))
    # Check result
    expect_equal(result_load$result, expected)

    # Check result messages (in output)
    expect_equal(length(result_load$messages), 0)
    expect_equal(result_load$output, "")
})

test_that('spectraPaths and spectraMetadata', {
    # If both are provided, only use the spectraMetadata (different size between both to ensure we have the right one)

    # expected
    expected    <- list(spectra=input_spectraPaths, meta=input_spectraMetadata[,'testcol', drop=FALSE])

    # results (output, warnings and messages)
    result_load <- evaluate_promise(spectraPaths_and_metadata_UI_helper(spectraPaths = input_spectraPaths[1:2],
                                                                        spectraMetadataPath = spectraMetaPath))
    # Check result
    expect_equal(result_load$result, expected)

    # Check result messages (in output)
    expect_equal(length(result_load$messages), 0)
    expect_equal(result_load$output, "")
})

test_that('raise errors', {

    noFile  <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.csv')

    wrong_spectraMetadata <- input_spectraMetadata[,'testcol', drop=FALSE]
    wrong_spectraMetaPath <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.csv')
    utils::write.csv(wrong_spectraMetadata, file=wrong_spectraMetaPath, row.names=FALSE)

    # Both inputs are NULL
    msg1    <- paste('Error: spectraPaths and spectraMetadataPath are not set', sep='')
    expect_error(spectraPaths_and_metadata_UI_helper(NULL, NULL), msg1, fixed=TRUE)

    # spectraMetadata CSV file doesn't exist
    msg2    <- paste('Error: spectraMetadata file does not exist', sep='')
    expect_error(spectraPaths_and_metadata_UI_helper(NULL, spectraMetadataPath=noFile), msg2, fixed=TRUE)

    # spectraMetadata does not have a 'filepath' column
    msg3    <- paste("Error: the column 'filepath' must be present in the spectraMetadata", sep='')
    expect_error(spectraPaths_and_metadata_UI_helper(NULL, spectraMetadataPath=wrong_spectraMetaPath), msg3, fixed=TRUE)
})

context('annotation_showText_UI_helper()')

# UI specific function that returns a textual description of an annotation to show in UI

test_that('UI show text outputs right values', {
  # Input and expected
  properties_default <- list(nbCompounds = 0,
                             nbSamples = 0,
                             uROIExist = FALSE,
                             useUROI = FALSE,
                             useFIR = FALSE,
                             isAnnotated = FALSE)
  properties_ROIFIRInit <- list(nbCompounds = 2,
                                nbSamples = 3,
                                uROIExist = TRUE,
                                useUROI = TRUE,
                                useFIR = TRUE,
                                isAnnotated = TRUE)
  expected_text_default    <- list('Not annotated', '0 compounds', '0 samples', 'updated ROI do not exist (uROI)', 'does not use updated ROI (uROI)', 'does not use fallback integration regions (FIR)')
  expected_text_ROIFIRInit <- list('Is annotated',  '2 compounds', '3 samples', 'updated ROI exist (uROI)',        'uses updated ROI (uROI)',         'uses fallback integration regions (FIR)')

  # default values, no uROI, no use of FIR
  result_default  <- evaluate_promise(annotation_showText_UI_helper(properties_default))
  expect_equal(result_default$result, expected_text_default)

  # multiple compounds and spectra with uROIExist and useFIR
  result_ROIFIRInit <- evaluate_promise(annotation_showText_UI_helper(properties_ROIFIRInit))
  expect_equal(result_ROIFIRInit$result, expected_text_ROIFIRInit)
})

## Run Tab ---------------------------------------------------------------------

# - message if no import (conditional panel)
# - status of the annotation (and failures) on the sidebar
# - message if already annotated (also triggered after computation)
# - useUROI, useFIR and parallelisation controls [pre-checked based on annotation properties, useUROI strikedthrough if uROI do not exist]
# - run annotation after updating useUROI useFIR (erase previous data)
# - progress indicator during computation
# - success/failure message with number of failed samples


## no import
output$noImportForFitUI <- renderUI ({
  if(importSuccess()=='yes') return()
  tagList(
    HTML("<div class=\"alert alert-dismissible alert-danger\"> <button type=\"button\" class=\"close\" data-dismiss=\"alert\">×</button> <h4 style=\"font-weight:bold\">No annotation imported</h4>Create or load a <i>peakPantheRAnnotation</i></div>")#,
    #includeHTML("data/aboutRunAnnotation.html")
  )
})


## show the status of the peakPantheRAnnotation
output$showAnnotStatus <- renderUI({
  # Capture the annotation shown and split by line into a list
  tmp_text <- annotation_showText_UI_helper(annotation_showMethod_UI_helper(values$annotation))
  fail_text <- paste(dim(values$failures)[1], 'annotation failure(s)')
  # render the panel
  wellPanel(
    h4('Status:', style="color:#3e648d;font-weight:bold"),
    helpText(tmp_text[[1]],style="color:black"),
    helpText(tmp_text[[2]],style="color:black"),
    helpText(tmp_text[[3]],style="color:black"),
    helpText(tmp_text[[4]],style="color:black"),
    helpText(tmp_text[[5]],style="color:black"),
    helpText(tmp_text[[6]],style="color:black"),
    tags$hr(),
    if (!is.null(values$failures)) {
      helpText(fail_text,style="color:black")
    }
  )
})


## Message if already annotated
output$alreadyAnnotatedUI <- renderUI({
  # annotated, warning message
  if(peakPantheR::isAnnotated(values$annotation)) {
    return(
      tagList(
        HTML("<div class=\"alert alert-dismissible alert-warning\"><button type=\"button\" class=\"close\" data-dismiss=\"alert\">×</button><h4 style=\"font-weight:bold\">Warning</h4>This dataset is already annotated, if selecting `Annotate` the data will be overwritten</div>")
      )
    )
  # not annotated, no msg
  } else { return() }
})


## Run UI buttons
# Checkbox use uROI (set default to current annotation value)
output$useUROICheckbox <- renderUI({
  # if uROI does not exist, strikethrough the label
  if (peakPantheR::uROIExist(values$annotation)) {
    lbl <- span("use ", shiny::span(em("updated Regions of Interest")))
  } else {
    lbl <- span(shiny::tags$s("use "), shiny::span(em(shiny::tags$s("updated Regions of Interest"))))
  }
  # set the default value based on the annotation status
  tagList(
    h5(HTML("uROI"), style="color:#3e648d;font-weight:bold"),
    checkboxInput("useUROI",
                   label = lbl,
                    value = peakPantheR::useUROI(values$annotation))
  )
})

# Checkbox use FIR (set default to current annotation value)
output$useFIRCheckbox <- renderUI({
  tagList(
    h5(HTML("FIR"), style="color:#3e648d;font-weight:bold"),
    checkboxInput("useFIR",
                  label = span("use ", shiny::span(em("Fallback Integration Regions"))),
                  value = peakPantheR::useFIR(values$annotation))
  )
}) # TODO: CHECK FIR checkbox is activated correctly [cannot do the strikethrough]

# selectInput choose curveModel (set default to current annotation value)
output$curveModelSelectInput <- renderUI({
  tagList(
    selectInput("curveModel",
                  label = 'Fitting model to use (curveModel)',
                  choices = knownCurveModel,
                  selected = NULL, # will be peakPantheR::curveModel(values$annotation))
                  multiple = FALSE)
  )
})

# Cpu slider appears if parallelisation is selected
output$cpuSlider <- renderUI({
  if(input$parallelisation == 0) return(NULL)
  tagList(
    sliderInput("ncores",
      label = paste("Available cores: ",maxCores,sep=""),
      min = 0, max = maxCores, value = maxCores, step=1
    )
  )
})
# correction when slider doesn't appear
ncoresInput <- reactive ({
  if( input$parallelisation != 0 ) { input$ncores }
  else { return(0) }
})


## Run the annotation (simplest setup possible)
observeEvent(input$goAnnotation, {
  ## Need to update an annotation first with useUROI and useFIR
  tmp_annotation <- values$annotation
  # proxy for useUROI (if uROI does not exist: it is always FALSE)
  if (peakPantheR::uROIExist(tmp_annotation)) { tmp_useUROI <- input$useUROI } else { tmp_useUROI <- FALSE }
  # finalise the setup of the annotation (useUROI, useFIR)
  tmp_annotation <- peakPantheR::resetAnnotation(tmp_annotation,
                                                 useUROI=tmp_useUROI,
                                                 useFIR=input$useFIR,
                                                 verbose=FALSE)
  ## Annotate!
  result <- peakPantheR_parallelAnnotation(tmp_annotation, ncores=ncoresInput(), curveModel=input$curveModel, verbose=TRUE)

  # Store the annotation and failures into the reactiveValue
  values$annotation <- result$annotation
  values$failures   <- result$failures
  # Set a list of feature name for later use
  values$featNmeList        <- paste(peakPantheR::cpdID(values$annotation), peakPantheR::cpdName(values$annotation), sep=' - ')
  names(values$featNmeList) <- seq_len(length(values$featNmeList))
  # Set a list of spectraMetadata columns (+ None)
  tmp_splCol                <- c(list('None'), colnames(peakPantheR::spectraMetadata(values$annotation)))
  values$spectraMetadataCol <- tmp_splCol
  # Set a list of filename (sample list)
  values$filename           <- peakPantheR::filename(values$annotation)
})


## Progress bar
output$progressBarUI <- renderUI({
  conditionalPanel(condition="$('html').hasClass('shiny-busy')",
                   column(10, align="center",
                          h4('Processing ...', style="color:#3e648d;font-weight:bold"),
                          shiny::img(src="annot_progress.gif", width="350px")
                   )
  )
})


## Check annotation run is a success
# value for success
runSuccess <- reactive({
  # no annotation computed yet (cover the case of no import without having to do an isAnnotated on NULL which would throw an error)
  if(is.null(values$annotation)) {
    return('no')
  }
  # annotation successful or already annotated
  if(peakPantheR::isAnnotated(values$annotation)) {
    return('yes')
  # annotation not yet run or all spectra fail (trigger is checked in successAnnotationUI
  } else {
    return('no')
  }
})

## Success message
output$successAnnotationUI <- renderUI({
  failure_text <- paste(dim(values$failures)[1], 'annotation failure(s)')
  if(runSuccess()=='no') {
    # not imported yet
    if(input$goAnnotation == 0) {
      return()
    # Run failed
    } else {
      return(
        tagList(
          HTML(paste0("<div class=\"alert alert-dismissible alert-danger\"><button type=\"button\" class=\"close\" data-dismiss=\"alert\">×</button><h4 style=\"font-weight:bold\">Error</h4>Annotation run failed<br>", failure_text, "</div>"))
        )
      )
    }
  # Run is successful
  } else if(runSuccess()=='yes') {
    tagList(
      HTML(paste0("<div class=\"alert alert-dismissible alert-success\"><button type=\"button\" class=\"close\" data-dismiss=\"alert\">×</button><h4 style=\"font-weight:bold\">Success</h4>Annotation successfully finished<br>", failure_text, "</div>"))
    )
  }
})


# Conditional panels control (next tab)
output$AnnotationDone <- renderText({
  if( runSuccess()=='yes' ) { 'yes' } else { 'no' }
})

#' @title Save to disk each ROI EIC and mean IS RT
#' @description Using reference samples (\code{referenceSpectraFiles}), save
#' (to \code{saveFolder}) each ROI EIC (\code{ROI}) and reports the mean apex RT
#' for all IS (\code{IS_ROI}) across samples
#' @param referenceSpectraFiles (str) A character vector of paths to the
#' reference spectra files
#' @param saveFolder (str) Path to the folder where EICs and IS mean RT
#' (\code{IS_mean_RT.csv}) will be saved
#' @param ROI (data.frame) NULL or a data.frame of Regions Of Interest (ROI)
#' with compounds as row and ROI parameters as columns: \code{rtMin} (float in
#' seconds), \code{rt} (float in seconds, or \emph{NA}), \code{rtMax} (float in
#' seconds), \code{mzMin} (float), \code{mz} (float or \emph{NA}), \code{mzMax}
#' (float) (if NULL, ROI EICs are not saved)
#' @param IS_ROI (data.frame) NULL or a data.frame of IS ROI with IS as row and
#' ROI parameters as columns: \code{rtMin} (float in seconds), \code{rt} (float
#' in seconds, or \emph{NA}), \code{rtMax} (float in seconds), \code{mzMin}
#' (float), \code{mz} (float or \emph{NA}), \code{mzMax} (float)  (if NULL IS
#' mean RT is not calculated and saved in \code{IS_mean_RT.csv})
#' @param sampleColour (str) NULL or vector colour for each sample
#' @param ncores (int) Number of cores to use to integrate IS in parallel
#' @param saveISPlots (bool) If TRUE save a diagnostic plot for each IS to
#' \code{saveFolder/IS_search} compound
#' @param verbose (bool) If TRUE message progress
#' @return None
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 2 samples and 1 targeted
#' ## compound
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 1, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(1))
#'
#' # input
#' refSpecFiles  <- spectraPaths
#' input_ROI     <- targetFeatTable
#' input_IS_ROI  <- targetFeatTable
#' sampleColour  <- c('blue', 'red')
#'
#' # temporary saveFolder
#' saveFolder1   <- tempdir()
#'
#' # Calculate ROI statiscs
#' peakPantheR_ROIStatistics(refSpecFiles, saveFolder1, ROI=input_ROI,
#'                             IS_ROI=input_IS_ROI, sampleColour=sampleColour,
#'                             ncores=0, saveISPlots=TRUE, verbose=TRUE)
#' }
peakPantheR_ROIStatistics   <- function(referenceSpectraFiles, saveFolder,
                                        ROI = NULL, IS_ROI = NULL,
                                        sampleColour = NULL, ncores = 0,
                                        saveISPlots = TRUE, verbose = TRUE) {
    # Check and process input parameters
    resInit <- ROIStatistics_init_checks(referenceSpectraFiles, saveFolder, ROI,
                                        IS_ROI, sampleColour, verbose)
    saveFolder <- resInit$saveFolder
    sampleColour <- resInit$sampleColour
    saveEICsROI <- resInit$saveEICsROI
    calculateMeanISRT <- resInit$calculateMeanISRT
    
    # save EICs ROI
    if (saveEICsROI) {
        ROIStatistics_saveEICsROI(referenceSpectraFiles, saveFolder, ROI,
                                    sampleColour, verbose)
    }

    # calculate mean IS for each RT
    if (calculateMeanISRT) {
        ROIStatistics_calculateMeanISRT(referenceSpectraFiles, saveFolder,
                            IS_ROI, saveISPlots, ncores, sampleColour, verbose)
    }
}


# -----------------------------------------------------------------------------
# peakPantheR_ROIStatistics helper functions

# Check and process input parameters
ROIStatistics_init_checks <- function(referenceSpectraFiles, saveFolder, ROI,
                            IS_ROI, sampleColour, verbose){
    # check input reference spectra
    if (typeof(referenceSpectraFiles) != "character") {
        stop('Check input, \"referenceSpectraFiles\" must be a vector ',
                    'of spectra paths')
    }
    nbSpectra <- length(referenceSpectraFiles)

    # save folder
    if ((typeof(saveFolder) != "character") | (length(saveFolder) != 1)) {
        stop("Check input, \"saveFolder\" must be a path") }
    saveFolder <- normalizePath(saveFolder, mustWork = FALSE)
    dir.create(saveFolder, recursive = TRUE, showWarnings = FALSE)
    # sampleColour
    if (!is.null(sampleColour)) {
        if ((typeof(sampleColour) != "character") |
            (length(sampleColour) != nbSpectra)) {
            if (verbose) {
                message('Check input, \"sampleColour\" must be a vector',
                    ' of colour of same length as \"referenceSpectraFile\": ',
                    'default colour used instead') }
            sampleColour <- rep("black", nbSpectra) }
    } else { sampleColour <- rep("black", nbSpectra) }

    # Check if can save EICs for each ROI and calculate mean IS RT
    res_check <- ROIStatistics_check_ROI_meanIS(ROI, IS_ROI, verbose)
    saveEICsROI <- res_check$save; calculateMeanISRT <- res_check$calculate

    # summary
    if (verbose) {
        # save ROI EICs
        if (saveEICsROI) {
            message("- EICs for each ROI windows will be saved to: ",
                saveFolder)
            message("    ", dim(ROI)[1], " ROI in ",
                length(referenceSpectraFiles), " reference samples")
        } else { message("- EICs of ROI windows will not be saved") }
        # calculate IS mean RT
        if (calculateMeanISRT) {
            message("- Mean RT of IS across reference samples will be ",
                    "saved to: ", saveFolder)
            message("    ", dim(IS_ROI)[1], " IS in ",
            length(referenceSpectraFiles), " reference samples")
        } else { message("- Mean RT of IS will not be calculated") }
    }
    return(list(saveFolder=saveFolder, sampleColour=sampleColour,
                saveEICsROI=saveEICsROI, calculateMeanISRT=calculateMeanISRT))
}
# Check saveEICs for each ROI and calculate mean IS RT
ROIStatistics_check_ROI_meanIS <- function(ROI, IS_ROI, verbose) {
    # save EICs for each ROI if ROI is properly defined
    saveEICsROI <- FALSE
    if (!is.null(ROI)) {
        # ROI is a data.frame
        if (is.data.frame(ROI)) {
            # ROI data.frame columns
            if (all(c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz",
                "mzMax") %in% colnames(ROI))) {
                saveEICsROI <- TRUE
            } else {
                if (verbose) {
                    message('ROI columns must contain \"cpdID\", \"cpdName\", ',
                            '\"rtMin\", \"rt\", \"rtMax\", \"mzMin\", \"mz\" ',
                            'and \"mzMax\", EICs of ROI windows will not be ',
                            'saved') }}
        } else {
            if (verbose) {
                message('ROI is not a data.frame, EICs of ROI windows ',
                                'will not be saved') }}
    } else {
        if (verbose) {
            message("No ROI provided, EICs of ROI windows will not be saved") }}

    # calculate mean IS RT if IS_ROI is properly defined
    calculateMeanISRT <- FALSE
    if (!is.null(IS_ROI)) {
        # ROI is a data.frame
        if (is.data.frame(IS_ROI)) {
            # ROI data.frame columns
            if (all(c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz",
                "mzMax") %in% colnames(IS_ROI))) {
                calculateMeanISRT <- TRUE
            } else {
                if (verbose) {
                    message('IS_ROI columns must contain \"cpdID\", ',
                            '\"cpdName\", \"rtMin\", \"rt\", \"rtMax\", ',
                            '\"mzMin\", \"mz\" and \"mzMax\", mean RT of IS ',
                            'will not be calculated') }}
        } else {
            if (verbose) {
                message('IS_ROI is not a data.frame, mean RT of IS will',
                                ' not be calculated') }}
    } else {
        if (verbose) {
            message("No IS_ROI provided, mean RT of IS will not be calculated")
        }}

    return(list(save=saveEICsROI, calculate=calculateMeanISRT))
}
# Save EIC for each ROI
ROIStatistics_saveEICsROI <- function(referenceSpectraFiles, saveFolder, ROI,
                                        sampleColour, verbose) {
    if (verbose) { message("\n-- Saving EICs for each ROI --") }
    nbROI <- dim(ROI)[1]
    # extract all ROIs in all files
    all_ROIs <- lapply(referenceSpectraFiles, function(x) {
        # if file doesn't exist, pass
        singleSpectraDataPath <- normalizePath(x, mustWork = FALSE)
        if (!file.exists(singleSpectraDataPath)) {
            message("File \"", singleSpectraDataPath, "\" does not exist")
            return(NULL) }
        raw_data <- MSnbase::readMSData(singleSpectraDataPath,
                                        centroided = TRUE,
                                        mode = "onDisk")
        ROIsDataPoint <- extractSignalRawData(raw_data,
                                            rt = ROI[, c("rtMin", "rtMax")],
                                            mz = ROI[, c("mzMin", "mzMax")],
                                            verbose = verbose)
        return(ROIsDataPoint) })
    # remove failures
    specToKeep <- vapply(all_ROIs, function(x) {!is.null(x)},
                        FUN.VALUE = logical(1))
    all_ROIs <- all_ROIs[specToKeep]
    sampleColour <- sampleColour[specToKeep]
    # generate and save plots
    for (i in seq(nbROI)) {
        tmp_ROI_datapoints <- unlist(lapply(all_ROIs, function(x, y) {x[y]},
                                        y = i), recursive = FALSE)
        tmp_EIC_plot <- peakPantheR_plotEICFit(
                            ROIDataPointSampleList = tmp_ROI_datapoints,
                            curveFitSampleList = NULL, rtMin = NULL,
                            rtMax = NULL, sampling = 250,
                            sampleColour = sampleColour, verbose = FALSE)
        # add box limits and apex
        tmp_EIC_plot <- tmp_EIC_plot + ggplot2::geom_vline(
            ggplot2::aes(xintercept = c(ROI[i, "rtMin"], ROI[i, "rtMax"])),
            colour = "darkgrey", linetype = "dashed")
        tmp_EIC_plot <- tmp_EIC_plot + ggplot2::geom_vline(
            ggplot2::aes(xintercept = ROI[i, "rt"]), colour = "red")
        tmp_EIC_plot <- tmp_EIC_plot +
            ggplot2::ggtitle(paste(ROI[i, "cpdID"], "-", ROI[i, "cpdName"],
                "|", ROI[i, "rtMin"], "-", ROI[i, "rtMax"], "s |",
                ROI[i, "mzMin"], "-", ROI[i, "mzMax"], "m/z"))
        # save
        saveFileName <- paste(ROI[i, "cpdID"], ".png", sep = "")
        ggplot2::ggsave(file=saveFileName, plot=tmp_EIC_plot, device="png",
            path = saveFolder, dpi=100, width=25, height=25, units="cm",
            limitsize = FALSE) }
    if (verbose) { message(nbROI, " ROIs saved to ", saveFolder) }
}
# Calculate mean RT for each IS
ROIStatistics_calculateMeanISRT <- function(referenceSpectraFiles, saveFolder,
                            IS_ROI, saveISPlots, ncores, sampleColour, verbose){
    if (verbose) {
        message("\n-- Calculating mean RT for each IS --")
    }
    IS_annotation <- peakPantheRAnnotation(
        spectraPaths = referenceSpectraFiles,
        targetFeatTable = IS_ROI)
    IS_annotation_results <- peakPantheR_parallelAnnotation(IS_annotation,
        ncores = ncores, verbose = verbose)
    IS_annotation <- IS_annotation_results$annotation

    # save IS fit diagnostic plots
    if (saveISPlots) {
        outputAnnotationDiagnostic(IS_annotation,
            saveFolder = file.path(saveFolder, "IS_search"),
            savePlots = TRUE, sampleColour=sampleColour, verbose = verbose,
            ncores = ncores)
    }

    # calculate statistics
    mean_IS_rt <- data.frame(
        colMeans(annotationTable(IS_annotation, column = "rt"), na.rm=TRUE))
    colnames(mean_IS_rt) <- "mean_rt"
    # save to disk
    path_meanRT <- file.path(saveFolder, "IS_mean_RT.csv")
    utils::write.csv(mean_IS_rt, file = path_meanRT, row.names = TRUE,
                    fileEncoding = "UTF-8")
    if (verbose) {
        message("IS mean RT saved at ", path_meanRT)
    }
}

context('peakPantheR_ROIStatistics()')

skip_if_not_installed('faahKO',  minimum_version = '1.18.0')
library(faahKO)

# remove Rplots.pdf created by ggplot2
on.exit( tryCatch({ file.remove('./Rplots.pdf') }, error=function(e){ invisible() }, warning=function(w){ invisible() }) )

## Input and expected data
# 3 files
input_spectraPaths          <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                                 system.file('cdf/KO/ko16.CDF', package = "faahKO"),
                                 system.file('cdf/KO/ko18.CDF', package = "faahKO"))

# 4 features
input_targetFeatTable     	<- data.frame(matrix(vector(), 4, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))),stringsAsFactors=FALSE)
input_targetFeatTable[1,] 	<- c("ID-1", "Cpd 1", 3310., 3344.888, 3390., 522.194778, 522.2, 522.205222)
input_targetFeatTable[2,] 	<- c("ID-2", "Cpd 2", 3280., 3385.577, 3440., 496.195038, 496.2, 496.204962)
input_targetFeatTable[3,] 	<- c("ID-3", "Cpd 3", 3420., 3454.435, 3495., 464.195358, 464.2, 464.204642)
input_targetFeatTable[4,] 	<- c("ID-4", "Cpd 4", 3670., 3701.697, 3745., 536.194638, 536.2, 536.205362)
input_targetFeatTable[,c(3:8)] <- vapply(input_targetFeatTable[,c(3:8)], as.numeric, FUN.VALUE=numeric(4))



test_that('3 files, save EICS, mean IS RT, save IS fit, with sampleColour, verbose, no verbose', {
  # temp saveFolder
  saveFolder1   <- tempdir()
  # clear temp folder
  suppressWarnings(do.call(file.remove, list(list.files(saveFolder1, full.names=TRUE))))
  
  # input
  refSpecFiles  <- input_spectraPaths
  input_ROI     <- input_targetFeatTable
  input_IS_ROI  <- input_targetFeatTable
  sampleColour  <- c('blue', 'red', 'green')
  
  # Expected save paths
  expected_path_plot1 <- file.path(saveFolder1, "ID-1.png")
  expected_path_plot2 <- file.path(saveFolder1, "ID-2.png")
  expected_path_plot3 <- file.path(saveFolder1, "ID-3.png")
  expected_path_plot4 <- file.path(saveFolder1, "ID-4.png")
  expected_path_CSV   <- file.path(saveFolder1, "IS_mean_RT.csv")
  expected_path_IS_plot1 <- file.path(saveFolder1,"IS_search", "cpd_1.png")
  expected_path_IS_plot2 <- file.path(saveFolder1,"IS_search", "cpd_2.png")
  expected_path_IS_plot3 <- file.path(saveFolder1,"IS_search", "cpd_3.png")
  expected_path_IS_plot4 <- file.path(saveFolder1,"IS_search", "cpd_4.png")
  # Expected IS mean RT
  expected_meanIS     <- data.frame(matrix(nrow=4,ncol=2,dimnames=list(c(), c('X','mean_rt'))))
  expected_meanIS[1,] <- c('ID-1', 3360.054253)
  expected_meanIS[2,] <- c('ID-2', 3401.938353)
  expected_meanIS[3,] <- c('ID-3', 3458.840343)
  expected_meanIS[4,] <- c('ID-4', 3712.859261)
  expected_meanIS[,2] <- vapply(expected_meanIS[,2], as.numeric, FUN.VALUE=numeric(1))
  # Expected message
  expected_message    <- c("    4 ROI in 3 reference samples\n", "    4 IS in 3 reference samples\n", "\n-- Saving EICs for each ROI --\n","Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Reading data from 4 windows\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Reading data from 4 windows\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Reading data from 4 windows\n", "\n-- Calculating mean RT for each IS --\n", "Processing 4 compounds in 3 samples:\n", "  uROI:\tFALSE\n", "  FIR:\tFALSE\n", "----- ko15 -----\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Check input, mzMLPath must be a .mzML\n", "Reading data from 4 windows\n", "Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #1\n", "Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #3\n", "Annotation object cannot be reordered by sample acquisition date\n", "----------------\n", "  0 failure(s)\n", "Saving diagnostic plots:\n")

  
	# results (output, warnings and messages)
  result_ROIstatsV <- evaluate_promise(peakPantheR_ROIStatistics(refSpecFiles, saveFolder1, ROI=input_ROI, IS_ROI=input_IS_ROI, sampleColour=sampleColour, ncores=0, saveISPlots=TRUE, verbose=TRUE))

  # Check saved EIC plots
  expect_true(file.exists(expected_path_plot1))
  expect_true(file.exists(expected_path_plot2))
  expect_true(file.exists(expected_path_plot3))
  expect_true(file.exists(expected_path_plot4))
  # Check saved IS fit
  expect_true(file.exists(expected_path_IS_plot1))
  expect_true(file.exists(expected_path_IS_plot2))
  expect_true(file.exists(expected_path_IS_plot3))
  expect_true(file.exists(expected_path_IS_plot4))
  # Check mean IS RT
  expect_true(file.exists(expected_path_CSV))
  saved_CSV       <- read.csv(expected_path_CSV, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_CSV, expected_meanIS, tolerance=1e-4)
  
  # # Check messages (no filepaths)
  # expect_equal(length(result_ROIstatsV$messages), 64)
  # expect_equal(result_ROIstatsV$messages[c(2, 4:7, 9:10, 12:13, 16:23, 25:26, 54:55, 57, 59)], expected_message)
  
  # no verbose
  result_ROIstatsNoV <- evaluate_promise(peakPantheR_ROIStatistics(refSpecFiles, saveFolder1, ROI=input_ROI, IS_ROI=input_IS_ROI, sampleColour=sampleColour, ncores=0, saveISPlots=TRUE, verbose=FALSE))
  expect_equal(length(result_ROIstatsNoV$messages), 6)
})

test_that('3 files (1 missing), save EICs, no IS RT, no IS fit, sampleColour is not of length nb spectra, verbose', {
  # message for wrong sampleColour length
  
  # temp saveFolder
  saveFolder2   <- tempdir()
  # clear temp folder
  suppressWarnings(do.call(file.remove, list(list.files(saveFolder2, full.names=TRUE))))
  
  # input
  refSpecFiles  <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                     'not/a/valid/path',
                     system.file('cdf/KO/ko18.CDF', package = "faahKO"))
  input_ROI     <- input_targetFeatTable
  sampleColour  <- c('blue', 'red', 'green', 'yellow')
  
  # Expected save paths
  expected_path_plot1 <- file.path(saveFolder2, "ID-1.png")
  expected_path_plot2 <- file.path(saveFolder2, "ID-2.png")
  expected_path_plot3 <- file.path(saveFolder2, "ID-3.png")
  expected_path_plot4 <- file.path(saveFolder2, "ID-4.png")
  expected_path_CSV   <- file.path(saveFolder2, "IS_mean_RT.csv")
  # Expected message
  expected_message    <- c("Check input, \"sampleColour\" must be a vector of colour of same length as \"referenceSpectraFile\": default colour used instead\n", "No IS_ROI provided, mean RT of IS will not be calculated\n", "    4 ROI in 3 reference samples\n", "- Mean RT of IS will not be calculated\n", "\n-- Saving EICs for each ROI --\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Reading data from 4 windows\n")
  
  # results (output, warnings and messages)
  result_ROIstatsV <- evaluate_promise(peakPantheR_ROIStatistics(refSpecFiles, saveFolder2, ROI=input_ROI, IS_ROI=NULL, sampleColour=sampleColour, ncores=0, saveISPlots=FALSE, verbose=TRUE))
  
  # Check saved EIC plots
  expect_true(file.exists(expected_path_plot1))
  expect_true(file.exists(expected_path_plot2))
  expect_true(file.exists(expected_path_plot3))
  expect_true(file.exists(expected_path_plot4))
  
  # Check messages (no filepaths)
  expect_equal(length(result_ROIstatsV$messages), 14)
  expect_equal(result_ROIstatsV$messages[c(1:2, 4:8)], expected_message)
})

test_that('3 files, no save EICs, mean IS RT, no IS fit, without sampleColour, verbose, no verbose', {
  # temp saveFolder
  saveFolder3   <- tempdir()
  # clear temp folder
  suppressWarnings(do.call(file.remove, list(list.files(saveFolder3, full.names=TRUE))))
  
  # input
  refSpecFiles  <- input_spectraPaths
  input_ROI     <- input_targetFeatTable
  input_IS_ROI  <- input_targetFeatTable
  
  # Expected save paths
  expected_path_CSV   <- file.path(saveFolder3, "IS_mean_RT.csv")
  # Expected IS mean RT
  expected_meanIS     <- data.frame(matrix(nrow=4,ncol=2,dimnames=list(c(), c('X','mean_rt'))))
  expected_meanIS[1,] <- c('ID-1', 3360.054253)
  expected_meanIS[2,] <- c('ID-2', 3401.938353)
  expected_meanIS[3,] <- c('ID-3', 3458.840343)
  expected_meanIS[4,] <- c('ID-4', 3712.859261)
  expected_meanIS[,2] <- vapply(expected_meanIS[,2], as.numeric, FUN.VALUE=numeric(1))
  # Expected message
  expected_message    <- c("No ROI provided, EICs of ROI windows will not be saved\n", "- EICs of ROI windows will not be saved\n" , "    4 IS in 3 reference samples\n", "\n-- Calculating mean RT for each IS --\n", "Processing 4 compounds in 3 samples:\n", "  uROI:\tFALSE\n", "  FIR:\tFALSE\n", "----- ko15 -----\n", "Polarity can not be extracted from netCDF files, please set manually the polarity with the 'polarity' method.\n", "Check input, mzMLPath must be a .mzML\n", "Reading data from 4 windows\n", "----------------\n", "  0 failure(s)\n")

  
  # results (output, warnings and messages)
  result_ROIstatsV <- evaluate_promise(peakPantheR_ROIStatistics(refSpecFiles, saveFolder3, ROI=NULL, IS_ROI=input_IS_ROI, sampleColour=NULL, ncores=0, saveISPlots=FALSE, verbose=TRUE))
  
  # Check mean IS RT
  expect_true(file.exists(expected_path_CSV))
  saved_CSV       <- read.csv(expected_path_CSV, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_CSV, expected_meanIS, tolerance=1e-4)
  
  # # Check messages (no filepaths)
  # expect_equal(length(result_ROIstatsV$messages), 47)
  # expect_equal(result_ROIstatsV$messages[c(1:2, 4:12, 44,46)], expected_message)
})

test_that('3 files, no save EICs (not a data.frame), no mean IS RT (not a data.frame), sampleColour is not character, verbose', {
  # message for ROI not a data.frame, IS_ROI not a data.frame, sampleColour not a character
  
  # temp saveFolder
  saveFolder4   <- tempdir()
  # clear temp folder
  suppressWarnings(do.call(file.remove, list(list.files(saveFolder4, full.names=TRUE))))
  
  # input
  refSpecFiles  <- input_spectraPaths
  input_ROI     <- 'notADataframe'
  input_IS_ROI  <- 'notADataframe'
  
  # Expected message
  expected_message    <- c("Check input, \"sampleColour\" must be a vector of colour of same length as \"referenceSpectraFile\": default colour used instead\n", "ROI is not a data.frame, EICs of ROI windows will not be saved\n",  "IS_ROI is not a data.frame, mean RT of IS will not be calculated\n", "- EICs of ROI windows will not be saved\n", "- Mean RT of IS will not be calculated\n")
  
  
  # results (output, warnings and messages)
  result_ROIstatsV <- evaluate_promise(peakPantheR_ROIStatistics(refSpecFiles, saveFolder4, ROI=input_ROI, IS_ROI=input_IS_ROI, sampleColour=42, ncores=0, saveISPlots=FALSE, verbose=TRUE))
  
  # Check messages
  expect_equal(length(result_ROIstatsV$messages), 5)
  expect_equal(result_ROIstatsV$messages, expected_message)
})

test_that('3 files, no save EICs (wrong columns), no mean IS RT (wrong columns), verbose', {
  # message for ROI and IS_ROI missing a column (cpdID, cpdName, rt, mz, rtMin, rtMax, mzMin, mzMax)
  
  # temp saveFolder
  saveFolder5   <- tempdir()
  # clear temp folder
  suppressWarnings(do.call(file.remove, list(list.files(saveFolder5, full.names=TRUE))))
  
  # input
  refSpecFiles  <- input_spectraPaths
  input_ROI     <- input_targetFeatTable
  input_IS_ROI  <- input_targetFeatTable
  # Expected message
  expected_message    <- c("ROI columns must contain \"cpdID\", \"cpdName\", \"rtMin\", \"rt\", \"rtMax\", \"mzMin\", \"mz\" and \"mzMax\", EICs of ROI windows will not be saved\n", "IS_ROI columns must contain \"cpdID\", \"cpdName\", \"rtMin\", \"rt\", \"rtMax\", \"mzMin\", \"mz\" and \"mzMax\", mean RT of IS will not be calculated\n", "- EICs of ROI windows will not be saved\n", "- Mean RT of IS will not be calculated\n")
  
  
  ## no cpdID
  # results (output, warnings and messages)
  result_ROIstatsNoCpdID <- evaluate_promise(peakPantheR_ROIStatistics(refSpecFiles, saveFolder5, ROI=input_ROI[,-1], IS_ROI=input_IS_ROI[,-1], sampleColour=NULL, ncores=0, saveISPlots=FALSE, verbose=TRUE))
  # Check messages
  expect_equal(length(result_ROIstatsNoCpdID$messages), 4)
  expect_equal(result_ROIstatsNoCpdID$messages, expected_message)
  
  ## no cpdName
  # results (output, warnings and messages)
  result_ROIstatsNoCpdName <- evaluate_promise(peakPantheR_ROIStatistics(refSpecFiles, saveFolder5, ROI=input_ROI[,-2], IS_ROI=input_IS_ROI[,-2], sampleColour=NULL, ncores=0, saveISPlots=FALSE, verbose=TRUE))
  # Check messages
  expect_equal(length(result_ROIstatsNoCpdName$messages), 4)
  expect_equal(result_ROIstatsNoCpdName$messages, expected_message)
  
  ## no rtMin
  # results (output, warnings and messages)
  result_ROIstatsNoRtMin <- evaluate_promise(peakPantheR_ROIStatistics(refSpecFiles, saveFolder5, ROI=input_ROI[,-3], IS_ROI=input_IS_ROI[,-3], sampleColour=NULL, ncores=0, saveISPlots=FALSE, verbose=TRUE))
  # Check messages
  expect_equal(length(result_ROIstatsNoRtMin$messages), 4)
  expect_equal(result_ROIstatsNoRtMin$messages, expected_message)
  
  ## no rt
  # results (output, warnings and messages)
  result_ROIstatsNoRt <- evaluate_promise(peakPantheR_ROIStatistics(refSpecFiles, saveFolder5, ROI=input_ROI[,-4], IS_ROI=input_IS_ROI[,-4], sampleColour=NULL, ncores=0, saveISPlots=FALSE, verbose=TRUE))
  # Check messages
  expect_equal(length(result_ROIstatsNoRt$messages), 4)
  expect_equal(result_ROIstatsNoRt$messages, expected_message)
  
  ## no rtMax
  # results (output, warnings and messages)
  result_ROIstatsNoRtMax <- evaluate_promise(peakPantheR_ROIStatistics(refSpecFiles, saveFolder5, ROI=input_ROI[,-5], IS_ROI=input_IS_ROI[,-5], sampleColour=NULL, ncores=0, saveISPlots=FALSE, verbose=TRUE))
  # Check messages
  expect_equal(length(result_ROIstatsNoRtMax$messages), 4)
  expect_equal(result_ROIstatsNoRtMax$messages, expected_message)
  
  ## no mzMin
  # results (output, warnings and messages)
  result_ROIstatsNoMzMin <- evaluate_promise(peakPantheR_ROIStatistics(refSpecFiles, saveFolder5, ROI=input_ROI[,-6], IS_ROI=input_IS_ROI[,-6], sampleColour=NULL, ncores=0, saveISPlots=FALSE, verbose=TRUE))
  # Check messages
  expect_equal(length(result_ROIstatsNoMzMin$messages), 4)
  expect_equal(result_ROIstatsNoMzMin$messages, expected_message)
  
  ## no mz
  # results (output, warnings and messages)
  result_ROIstatsNoMz <- evaluate_promise(peakPantheR_ROIStatistics(refSpecFiles, saveFolder5, ROI=input_ROI[,-7], IS_ROI=input_IS_ROI[,-7], sampleColour=NULL, ncores=0, saveISPlots=FALSE, verbose=TRUE))
  # Check messages
  expect_equal(length(result_ROIstatsNoMz$messages), 4)
  expect_equal(result_ROIstatsNoMz$messages, expected_message)
  
  ## no mzMax
  # results (output, warnings and messages)
  result_ROIstatsNoMzMax <- evaluate_promise(peakPantheR_ROIStatistics(refSpecFiles, saveFolder5, ROI=input_ROI[,-8], IS_ROI=input_IS_ROI[,-8], sampleColour=NULL, ncores=0, saveISPlots=FALSE, verbose=TRUE))
  # Check messages
  expect_equal(length(result_ROIstatsNoMzMax$messages), 4)
  expect_equal(result_ROIstatsNoMzMax$messages, expected_message)
})

test_that('parallel give the same result: 3 files, no save EICs, mean IS RT, save IS fit, verbose', {
  
  # temp saveFolder
  saveFolder6   <- tempdir()
  # clear temp folder
  suppressWarnings(do.call(file.remove, list(list.files(saveFolder6, full.names=TRUE))))
  
  # input
  refSpecFiles  <- input_spectraPaths
  input_ROI     <- input_targetFeatTable
  input_IS_ROI  <- input_targetFeatTable
  sampleColour  <- c('blue', 'red', 'green')
  
  # Expected IS mean RT
  expected_meanIS     <- data.frame(matrix(nrow=4,ncol=2,dimnames=list(c(), c('X','mean_rt'))))
  expected_meanIS[1,] <- c('ID-1', 3360.054253)
  expected_meanIS[2,] <- c('ID-2', 3401.938353)
  expected_meanIS[3,] <- c('ID-3', 3458.840343)
  expected_meanIS[4,] <- c('ID-4', 3712.859261)
  expected_meanIS[,2] <- sapply(expected_meanIS[,2], as.numeric)
  # Expected save paths
  expected_path_CSV       <- file.path(saveFolder6, "IS_mean_RT.csv")
  expected_path_IS_plot1  <- file.path(saveFolder6,"IS_search", "cpd_1.png")
  expected_path_IS_plot2  <- file.path(saveFolder6,"IS_search", "cpd_2.png")
  expected_path_IS_plot3  <- file.path(saveFolder6,"IS_search", "cpd_3.png")
  expected_path_IS_plot4  <- file.path(saveFolder6,"IS_search", "cpd_4.png")
  # Expected message
  expected_message_parallel <- c("No ROI provided, EICs of ROI windows will not be saved\n", "- EICs of ROI windows will not be saved\n", "    4 IS in 3 reference samples\n", "\n-- Calculating mean RT for each IS --\n", "Processing 4 compounds in 3 samples:\n", "  uROI:\tFALSE\n", "  FIR:\tFALSE\n", "Running 3 clusters of 1 files over 1 cores:\n", "  starting cluster 1/3\n", "  starting cluster 2/3\n", "  starting cluster 3/3\n", "Annotation object cannot be reordered by sample acquisition date\n", "----------------\n", "  0 failure(s)\n", "All plots saved\n")
  
  
  # results (output, warnings and messages)
  result_ROIstats_parallel <- evaluate_promise(peakPantheR_ROIStatistics(refSpecFiles, saveFolder6, ROI=NULL, IS_ROI=input_IS_ROI, sampleColour=sampleColour, ncores=1, saveISPlots=TRUE, verbose=TRUE))
  
  # Check saved IS fit
  expect_true(file.exists(expected_path_IS_plot1))
  expect_true(file.exists(expected_path_IS_plot2))
  expect_true(file.exists(expected_path_IS_plot3))
  expect_true(file.exists(expected_path_IS_plot4))
  # Check mean IS RT
  expect_true(file.exists(expected_path_CSV))
  saved_CSV       <- read.csv(expected_path_CSV, header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE)
  expect_equal(saved_CSV, expected_meanIS, tolerance=1e-4)
  
  # Check messages (no filepaths)
  expect_equal(length(result_ROIstats_parallel$messages), 20)
  expect_equal(result_ROIstats_parallel$messages[c(1,2,4:14,16,19)], expected_message_parallel)
})

test_that('raise errors', {
  # temp saveFolder
  saveFolder8   <- tempdir()
  # clear temp folder
  suppressWarnings(do.call(file.remove, list(list.files(saveFolder8, full.names=TRUE))))
  refSpecFiles  <- input_spectraPaths
  input_ROI     <- input_targetFeatTable
  input_IS_ROI  <- input_targetFeatTable
  sampleColour  <- c('blue', 'red', 'green')
  
  # referenceSpectraFiles is not a character vector
  error_msg1 <- 'Check input, "referenceSpectraFiles" must be a vector of spectra paths'
  expect_error(peakPantheR_ROIStatistics(referenceSpectraFiles=42, saveFolder8, ROI=input_ROI, IS_ROI=input_IS_ROI, sampleColour=sampleColour, ncores=0, saveISPlots=TRUE, verbose=TRUE), error_msg1, fixed=TRUE)
  
  # saveFolder is not a character
  error_msg2 <- 'Check input, "saveFolder" must be a path'
  expect_error(peakPantheR_ROIStatistics(referenceSpectraFiles=refSpecFiles, saveFolder=42 , ROI=input_ROI, IS_ROI=input_IS_ROI, sampleColour=sampleColour, ncores=0, saveISPlots=TRUE, verbose=TRUE), error_msg2, fixed=TRUE)
  
  # saveFolder is not of length 1
  error_msg3 <- 'Check input, "saveFolder" must be a path'
  expect_error(peakPantheR_ROIStatistics(referenceSpectraFiles=refSpecFiles, saveFolder=c('onePath','anotherPath','morePath') , ROI=input_ROI, IS_ROI=input_IS_ROI, sampleColour=sampleColour, ncores=0, saveISPlots=TRUE, verbose=TRUE), error_msg3, fixed=TRUE)
})

#' @title Plot variable histogram and density
#'
#' @description Plot the histogram and density of the variable
#'
#' @param var (float) vector of values to plot
#' @param varName (str) Name of the variable to plot
#' @param density (bool) If TRUE plot overlay the density on the variable
#' @param ... Passes arguments to ggplot2::geom_histogram, e.g. \code{bins=20},
#' \code{binwidth=1}
#' 
#' @return Grob (ggplot object)
plotHistogram <- function(var, varName = "Variable", density = TRUE, ...) {
    
    input_var <- var[!is.na(var)]
    
    ## Plot histogram
    p_hist <- ggplot2::ggplot(data.frame(x = input_var), ggplot2::aes(x = x)) +
                ggplot2::xlab(varName) + ggplot2::theme_bw()
    # with density
    if (density) {
        p_hist <- p_hist +
        ggplot2::geom_histogram(ggplot2::aes(y=ggplot2::after_stat(density)),
                                colour = "black", 
                                fill = "white", 
                                ...) +
        ggplot2::geom_density(alpha = 0.1, fill = "blue") +
        ggplot2::ylab("density")
        # without density
    } else {
        p_hist <- p_hist + ggplot2::geom_histogram(colour = "black",
                                                    fill = "white", ...) +
                                                    ggplot2::ylab("count")
    }
    
    return(p_hist)
}

################################################################################
#                                                                              #
#  --- peakPantheR: detect and integrate pre-defined features in MS files ---  #
#                                                                              #
################################################################################


#' @title Search, integrate and report targeted features in a raw spectra
#'
#' @description Report for a raw spectra the TIC, acquisition time, integrated
#' targeted features, fitted curves and datapoints for each region of interest.
#' Optimised to reduce the number of file access. Features not detected can be
#' integrated using fallback integration regions (FIR).
#'
#' @param singleSpectraDataPath (str) path to netCDF or mzML raw data file
#' (centroided, \strong{only with the channel of interest}).
#' @param targetFeatTable a \code{\link{data.frame}} of compounds to target as
#' rows. Columns: \code{cpdID} (str), \code{cpdName} (str), \code{rtMin} (float
#' in seconds), \code{rt} (float in seconds, or \emph{NA}), \code{rtMax} (float
#' in seconds), \code{mzMin} (float), \code{mz} (float or \emph{NA}),
#' \code{mzMax} (float).
#' @param peakStatistic (bool) If TRUE calculates additional peak statistics:
#' 'ppm_error', 'rt_dev_sec', 'tailing factor' and 'asymmetry factor'
#' @param plotEICsPath (str or NA) If not NA, will save a \emph{.png} of all ROI
#' EICs at the path provided (\code{'filepath/filename.png'} expected). If NA no
#' plot saved
#' @param getAcquTime (bool) If TRUE will extract sample acquisition date-time
#' from the mzML metadata (the additional file access will impact run time)
#' @param FIR (data.frame or NULL) If not NULL, integrate Fallback Integration
#' Regions (FIR) when a feature is not found.  Compounds as row are identical to
#' \code{targetFeatTable}, columns are \code{rtMin} (float in seconds),
#' \code{rtMax} (float in seconds), \code{mzMin} (float), \code{mzMax} (float).
#' @param centroided (bool) use TRUE if the data is centroided, used by
#' \code{\link[MSnbase]{readMSData}} when reading the raw data file
#' @param curveModel (str) specify the peak-shape model to fit,
#' by default \code{skewedGaussian}.
#' Accepted values are \code{skewedGaussian} and \code{emgGaussian}
#' @param verbose (bool) If TRUE message calculation progress, time taken and
#' number of features found
#' @param ... Passes arguments to \code{findTargetFeatures} to alter
#' peak-picking parameters (e.g. \code{curveModel}, \code{sampling},
#' \code{params} as a list of parameters for each ROI or 'guess',...)
#'
#' @return a list: \code{list()$TIC} \emph{(int)} TIC value,
#' \code{list()$peakTable} \emph{(data.frame)} targeted features results
#' (see Details), \code{list()$curveFit} \emph{(list)} list of
#' \code{peakPantheR_curveFit} or NA for each ROI, \code{list()$acquTime}
#' \emph{(POSIXct or NA)} date-time of sample acquisition from mzML metadata,
#' \code{list()$ROIsDataPoint} \emph{(list)} a list of \code{data.frame} of raw
#' data points for each ROI (retention time 'rt', mass 'mz' and intensity 'int'
#' (as column) of each raw data points (as row)).
#'
#' \subsection{Details:}{
#' The returned \emph{peakTable} \code{data.frame} is structured as follow:
#' \tabular{ll}{
#' cpdID \tab database compound ID\cr
#' cpdName \tab compound name\cr
#' found \tab was the peak found\cr
#' rt \tab retention time of peak apex (sec)\cr
#' rtMin \tab leading edge of peak retention time (sec) determined at 0.5\% of
#' apex intensity\cr
#' rtMax \tab trailing edge of peak retention time (sec) determined at 0.5\% of
#' apex intensity\cr
#' mz \tab weighted (by intensity) mean of peak m/z across scans\cr
#' mzMin \tab m/z peak minimum (between rtMin, rtMax)\cr
#' mzMax \tab m/z peak maximum (between rtMin, rtMax)\cr
#' peakArea \tab integrated peak area\cr
#' peakAreaRaw \tab integrated peak area from raw data points\cr
#' maxIntMeasured \tab maximum peak intensity in raw data\cr
#' maxIntPredicted \tab maximum peak intensity based on curve fit\cr
#' is_filled \tab Logical indicate if the feature was integrated using FIR
#' (Fallback Integration Region)\cr
#' ppm_error \tab difference in ppm between the expected and measured m/z\cr
#' rt_dev_sec \tab difference in seconds between the expected and measured rt\cr
#' tailingFactor \tab the tailing factor is a measure of peak tailing.It is
#' defined as the distance from the front slope of the peak to the back slope
#' divided by twice the distance from the center line of the peak to the front
#' slope, with all measurements made at 5\% of the maximum peak height. The
#' tailing factor of a peak will typically be similar to the asymmetry factor
#' for the same peak, but the two values cannot be directly converted\cr
#' asymmetryFactor \tab the asymmetry factor is a measure of peak tailing. It is
#' defined as the distance from the center line of the peak to the back slope
#' divided by the distance from the center line of the peak to the front slope,
#' with all measurements made at 10\% of the maximum peak height. The asymmetry
#' factor of a peak will typically be similar to the tailing factor for the same
#' peak, but the two values cannot be directly converted\cr
#' }
#' }
#'
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Load data
#' library(faahKO)
#' netcdfFilePath <- system.file('cdf/KO/ko15.CDF', package = 'faahKO')
#'
#' ## targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' res <- peakPantheR_singleFileSearch(netcdfFilePath,targetFeatTable,
#'                                     peakStatistic=TRUE)
#' # Polarity can not be extracted from netCDF files, please set manually the
#' #    polarity with the 'polarity' method.
#' # Reading data from 2 windows
#' # Data read in: 0.16 secs
#' # Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
#' #   mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and
#' #   ROI$mzMax for ROI #1
#' # Found 2/2 features in 0.05 secs
#' # Peak statistics done in: 0 secs
#' # Feature search done in: 0.75 secs
#' 
#' res
#' # $TIC
#' # [1] 2410533091
#' #
#' # $peakTable
#' #   found    rtMin       rt    rtMax    mzMin    mz    mzMax peakArea
#' # 1  TRUE 3309.759 3346.828 3385.410 522.1948 522.2 522.2052 26133727
#' # 2  TRUE 3345.377 3386.529 3428.279 496.2000 496.2 496.2000 35472141
#' #   peakAreaRaw maxIntMeasured maxIntPredicted cpdID cpdName is_filled
#' # 1    26071378         889280        901015.8  ID-1   Cpd 1     FALSE
#' # 2    36498367        1128960       1113576.7  ID-2   Cpd 2     FALSE
#' #    ppm_error   rt_dev_sec  tailingFactor  asymmetryFactor
#' # 1 0.02337616    1.9397590       1.015357         1.026824
#' # 2 0.02460103    0.9518072       1.005378         1.009318
#' #
#' # $acquTime
#' # [1] NA
#' #
#' #
#' # $curveFit
#' # $curveFit[[1]]
#' # $amplitude
#' # [1] 162404.8
#' # 
#' # $center
#' # [1] 3341.888
#' # 
#' # $sigma
#' # [1] 0.07878613
#' # 
#' # $gamma
#' # [1] 0.00183361
#' # 
#' # $fitStatus
#' # [1] 2
#' # 
#' # $curveModel
#' # [1] 'skewedGaussian'
#' # 
#' # attr(,'class')
#' # [1] 'peakPantheR_curveFit'
#' # 
#' # $curveFit[[2]]
#' # $amplitude
#' # [1] 199249.1
#' # 
#' # $center
#' # [1] 3382.577
#' # 
#' # $sigma
#' # [1] 0.07490442
#' # 
#' # $gamma
#' # [1] 0.00114719
#' # 
#' # $fitStatus
#' # [1] 2
#' # 
#' # $curveModel
#' # [1] 'skewedGaussian'
#' # 
#' # attr(,'class')
#' # [1] 'peakPantheR_curveFit'
#' #
#' #
#' # $ROIsDataPoint
#' # $ROIsDataPoint[[1]]
#' #          rt    mz    int
#' # 1  3315.154 522.2   2187
#' # 2  3316.719 522.2   3534
#' # 3  3318.284 522.2   6338
#' # 4  3319.849 522.2  11718
#' # 5  3321.414 522.2  21744
#' # 6  3322.979 522.2  37872
#' # 7  3324.544 522.2  62424
#' # 8  3326.109 522.2  98408
#' # 9  3327.673 522.2 152896
#' # 10 3329.238 522.2 225984
#' # ...
#' #
#' # $ROIsDataPoint[[2]]
#' #          rt    mz     int
#' # 1  3280.725 496.2    1349
#' # 2  3290.115 496.2    2069
#' # 3  3291.680 496.2    3103
#' # 4  3293.245 496.2    5570
#' # 5  3294.809 496.2   10730
#' # 6  3296.374 496.2   20904
#' # 7  3297.939 496.2   38712
#' # 8  3299.504 496.2   64368
#' # 9  3301.069 496.2   97096
#' # 10 3302.634 496.2  136320
#' # ...
#' }
#'
#' @family peakPantheR
#' @family realTimeAnnotation
#' @family parallelAnnotation
#'
#' @export
peakPantheR_singleFileSearch <- function(singleSpectraDataPath, targetFeatTable,
    peakStatistic = FALSE, plotEICsPath = NA, getAcquTime = FALSE, FIR = NULL,
    centroided = TRUE, curveModel='skewedGaussian', verbose = TRUE, ...) {
    stime <- Sys.time()
    # Check input
    resInp <- singleFileSearch_checkInput(singleSpectraDataPath,targetFeatTable,
                                            plotEICsPath, FIR, curveModel)
    singleSpectraDataPath <- resInp$specPath
    plotEICsPath <- resInp$plotPath
    useFIR <- resInp$useFIR
    
    # Read file
    raw_data <- MSnbase::readMSData(singleSpectraDataPath,
                                    centroided = centroided, mode = "onDisk")
    
    # Get TIC
    TICvalue <- sum(MSnbase::tic(raw_data))
        #, initial=FALSE to calculate from raw and not header
    
    # Get AcquTime
    AcquTime <- NA
    if (getAcquTime) {
        AcquTime <- getAcquisitionDatemzML(mzMLPath = singleSpectraDataPath,
                                            verbose = verbose) }
    
    # Get ROIsDataPoint (return empty list if no windows)
    ROIsDataPoint <- extractSignalRawData(raw_data,
                                    rt = targetFeatTable[, c("rtMin", "rtMax")],
                                    mz = targetFeatTable[, c("mzMin", "mzMax")],
                                    verbose = verbose)

    # Integrate
    resInt <- singleFileSearch_integrate(raw_data, targetFeatTable,
            ROIsDataPoint, peakStatistic, useFIR, FIR, plotEICsPath,
            curveModel, verbose,...)
    finalOutput <- resInt$finalOutput
    curveFit <- resInt$curveFit

    etime <- Sys.time()
    if (verbose) { message("Feature search done in: ",
                            round(as.double(difftime(etime, stime)), 2),
                            " ", units(difftime(etime, stime))) }
    
    # clear variables
    rm(raw_data)
    gc(verbose = FALSE)
    
    return(list(TIC = TICvalue, peakTable = finalOutput, acquTime = AcquTime,
                curveFit = curveFit, ROIsDataPoint = ROIsDataPoint))
}


# -----------------------------------------------------------------------------
# peakPantheR_singleFileSearch helper functions

# Check inputs
singleFileSearch_checkInput <- function(singleSpectraDataPath, targetFeatTable,
                                        plotEICsPath, FIR, curveModel) {
    singleSpectraDataPath <- normalizePath(singleSpectraDataPath,
                                            mustWork = FALSE)
    if (!file.exists(singleSpectraDataPath)) {
    stop("Check input, file \"", singleSpectraDataPath, "\" does not exist") }

    if (dim(targetFeatTable)[1] != 0) {
        # rtMin < rtMax and mzMin < mzMax
        if (!all(targetFeatTable[, "rtMax"] >= targetFeatTable[, "rtMin"])) {
            stop("Check input, \"rtMin\" must be <= to \"rtMax\"") }
        if (!all(targetFeatTable[, "mzMax"] >= targetFeatTable[, "mzMin"])) {
            stop("Check input, \"mzMin\" must be <= to \"mzMax\"") } }

    if (!is.na(plotEICsPath)) {
        plotEICsPath <- normalizePath(plotEICsPath, mustWork = FALSE)
        # folder exist
        if (!file.exists(dirname(plotEICsPath))) {
            stop("Check input, plotEICsPath folder \"", dirname(plotEICsPath),
                "\" does not exist") }
        # png extension
        if (stringr::str_sub(basename(plotEICsPath), start = -4) != ".png") {
            stop("Check input, plotEICsPath file name \"",
                basename(plotEICsPath), "\" lacks a \".png\" extension") } }

    useFIR <- FALSE
    if (!is.null(FIR)) {
        # FIR is Data.frame
        if (!is.data.frame(FIR)) {
            stop("Check input, FIR must be a data.frame not ", class(FIR)) }
        # FIR number of rows
        if (dim(FIR)[1] != dim(targetFeatTable)[1]) {
            stop('Check input, FIR must have the same number of rows as',
                        ' targetFeatTable') }
        # FIR columns
        if (!all(c("mzMin", "mzMax", "rtMin", "rtMax") %in% colnames(FIR))) {
            stop('Check input, FIR must have \"mzMin\", \"mzMax\", ',
                        '\"rtMin\" and \"rtMax\" as columns') }
        useFIR <- TRUE
    }
    # known curveModel
    known_curveModel <- c("skewedGaussian", "emgGaussian")
    if (!(curveModel %in% known_curveModel)) {
        stop(paste0("Err","or: \"curveModel\" must be one of: ",
            paste(known_curveModel, collapse=', '))) }

    return(list(specPath=singleSpectraDataPath, plotPath=plotEICsPath,
                useFIR=useFIR))
}
# Integrate if there is at minimum 1 target feature
singleFileSearch_integrate <- function(raw_data, targetFeatTable, ROIsDataPoint,
                        peakStatistic, useFIR, FIR, plotEICsPath,
                        curveModel, verbose, ...){
    if (dim(targetFeatTable)[1] != 0) { #Only integrate if there is min 1 target
        # Integrate features using ROI
        foundPeaks <- findTargetFeatures(ROIsDataPoint, targetFeatTable,
                                        curveModel=curveModel,
                                        verbose = verbose, ...)
        foundPeakTable <- foundPeaks$peakTable
        curveFit <- foundPeaks$curveFit
        # Add compound information
        finalOutput <- foundPeakTable
        finalOutput$cpdID <- targetFeatTable$cpdID
        finalOutput$cpdName <- targetFeatTable$cpdName
        finalOutput$is_filled <- as.logical(FALSE)
        # Add deviation, Tailing factor, Asymmetry factor
        if (peakStatistic) {
            finalOutput <- getTargetFeatureStatistic(curveFit, targetFeatTable,
                                                finalOutput, verbose = verbose)}
        # Fill features not found based on FIR
        if (useFIR) {
            finalOutput <- integrateFIR(raw_data, FIR, finalOutput,
                                        verbose = verbose) }
        # Save all EICs plot
        if (!is.na(plotEICsPath)) {
            saveSingleFileMultiEIC(ROIsDataPoint, curveFit, finalOutput,
                                    plotEICsPath, width = 15, height = 15,
                                    verbose = verbose) }

    } else { #No targeted features, initialise empty integration res and EICs
        if (verbose) {
            message("- No target features passed in 'targetFeatTable', ",
                            "no integration, only TIC will be reported -") }
        if (peakStatistic) {
            finalOutput <- data.frame(matrix(vector(), 0, 18,
                dimnames = list(c(), c("cpdID", "cpdName", "found", "rt",
                "rtMin", "rtMax", "mz", "mzMin", "mzMax", "peakArea",
                "peakAreaRaw", "maxIntMeasured", "maxIntPredicted",
                "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor",
                "asymmetryFactor"))), stringsAsFactors = FALSE)
        } else {
            finalOutput <- data.frame(matrix(vector(), 0, 14,
                dimnames = list(c(), c("cpdID", "cpdName", "found", "rt",
                "rtMin", "rtMax", "mz",  "mzMin",  "mzMax", "peakArea",
                "peakAreaRaw", "maxIntMeasured",
                "maxIntPredicted", "is_filled"))), stringsAsFactors = FALSE)}
        curveFit <- list()
    }
    return(list(finalOutput=finalOutput, curveFit=curveFit))
}

# 1. Generation of test_fakemzML.mzML
#
# The test_fakemzML.mzML was generated by converting a
# Waters .RAW file from an LC-MS experiment to the .mzML format,
# and then manually removing part of the <scan> tags containing raw data.
# Only the <mzML> and <run> sections, 3 example scans
# and the total ion current readout were kept. In this way the
# correct .mzML file structure is preserved for use in unit testing,
# but the file size is greatly decreased.
# The .RAW file was converted to .mzML using
# the ProteoWizard project msconvert tool.

#' @title Plot peak value and peakwidth by acquisition time or in input order
#'
#' @description For a single ROI, plot the peak value and peakwidth (RT, m/z,
#' ...) of detected peaks across multiple samples, by acquisition time or in
#' input order. If \code{rotateAxis=FALSE} x is run order / plot order, y is the
#' \code{apexValue} / \code{widthMin} / \code{widthMax}, if
#' \code{rotateAxis=TRUE} x is the measurement values and y the run order.
#'
#' @param apexValue (float) vector of apex value
#' @param widthMin (float) vector of detected peak minimum peakwidth value or
#' NULL (if NULL no peakwidth)
#' @param widthMax (float) vector of detected peak maximum peakwidth value or
#' NULL (uf NULL no peakwidth)
#' @param acquTime (POSIXct) vector of sample acquisition time as POSIXct or
#' NULL (if NULL points are plotted in the order values are passed as input with
#' the first on top or left)
#' @param varName (str) Name of the variable to plot
#' @param sampleColour (str) NULL or vector colour for each sample (same length
#' as \code{apexValue}, \code{widthMin}, \code{widthMax}, \code{acquTime})
#' @param rotateAxis (bool) if TRUE x and y axis are reversed
#' @param verbose (bool) if TRUE message when NA scans are removed
#' 
#' @return Grob (ggplot object)
#' 
#' @import scales
#' 
#' @export
#' 
#' @examples
#' ## Input data
#' apexVal <- c(1, 2, 3, 4)
#' minVal  <- c(0, 0, 2, 2)
#' maxVal  <- c(2, 4, 4, 5)
#' acqTime <- as.POSIXct(c('2017-07-13 21:06:14', '2017-07-14 21:06:14', 
#'                         '2017-07-15 21:06:14', '2017-07-16 21:06:14'))
#' 
#' ## Plot 4 sampels with colour
#' peakPantheR_plotPeakwidth(apexValue=apexVal, widthMin=minVal,widthMax=maxVal,
#'                         acquTime=NULL, varName='Test variable 1',
#'                         sampleColour=c('blue','red','green','orange'),
#'                         rotateAxis=FALSE, verbose=FALSE)
#' 
#' ## Plot 4 samples with colour by acquisition time
#' peakPantheR_plotPeakwidth(apexValue=apexVal, widthMin=minVal,widthMax=maxVal,
#'                         acquTime=acqTime, varName='Test variable 2',
#'                         sampleColour=c('blue','red','green','orange'),
#'                         rotateAxis=FALSE, verbose=FALSE)
#' 
#' ## Plot 4 samples with colour, rotate axis
#' peakPantheR_plotPeakwidth(apexValue=apexVal, widthMin=minVal,widthMax=maxVal,
#'                         acquTime=NULL, varName='Test variable 3',
#'                         sampleColour=c('blue','red','green','orange'),
#'                         rotateAxis=TRUE, verbose=FALSE)
#' 
#' ## Plot 4 samples with colour by acquisition time, rotate axis
#' peakPantheR_plotPeakwidth(apexValue=apexVal, widthMin=minVal,widthMax=maxVal,
#'                         acquTime=acqTime, varName='Test variable 4',
#'                         sampleColour=c('blue','red','green','orange'),
#'                         rotateAxis=FALSE, verbose=FALSE)
peakPantheR_plotPeakwidth <- function(apexValue, widthMin=NULL, widthMax = NULL,
    acquTime=NULL, varName="variable", sampleColour = NULL, rotateAxis = FALSE,
    verbose = TRUE) {

    ## Check input
    resInp <- plotPeakwidth_checkInput(apexValue, widthMin, widthMax, acquTime,
                                        sampleColour, verbose)
    nbSpl <- resInp$nbSpl; useRunOrder <- resInp$useRunOrder
    useWidth <- resInp$useWidth; colourSpl <- resInp$colourSpl
    sampleIDColour <- resInp$sampleIDColour

    ## Init plot draw default x/y will fill depending on rotation
    p <- ggplot2::ggplot(NULL, ggplot2::aes(x), environment = environment()) +
        ggplot2::theme_bw()
    # set fill and colour scale, with one color per sample ID
    p <- p + ggplot2::scale_colour_manual(values = colourSpl, guide = "none")
    p <- p + ggplot2::scale_fill_manual(values = colourSpl, guide = "none")

    ## tmp main axis 
    if (useRunOrder) {
        x_axis <- acquTime
    } else {
        x_axis <- seq(0, nbSpl - 1)  #first spectra on left
    }

    ## Plot peakwidth, apex values and apex points (flip x/y if rotateAxis)
    # must rotateAxis inside the function otherwise the checks for NA wouldn't 
    # work with flipped inputs
    p <- plotPeakwidth_plotWidthApex(p, useWidth, rotateAxis, x_axis,
                    widthMin, widthMax, sampleIDColour, apexValue, verbose)

    # some expected messages
    if (verbose) {
        if (useRunOrder) { message("Values plotted by run order")
        } else {           message("Values plotted by input order") }
    }

    ## final ordering and axis labelling
    p <- plotPeakwidth_rotateAxis_setLabels(p, rotateAxis, useRunOrder, 
                                            x_axis, acquTime, varName, verbose)

    return(p)
}


# -----------------------------------------------------------------------------
# peakPantheR_plotPeakwidth helper functions

# Check the input
plotPeakwidth_checkInput <- function(apexValue, widthMin, widthMax, acquTime,
                                    sampleColour, verbose){
    nbSpl <- length(apexValue)
    # check acquTime
    useRunOrder <- FALSE
    if (!is.null(acquTime)) {
        # acquTime is not a POSIXct
        if (!is(acquTime, "POSIXct")) {
            stop("Err","or: \"acquTime\" must be a vector of POSIXct") }
        # NA in acquTime
        if (any(is.na(acquTime))) {
            if (verbose) {
                message('War','ning: \"acquTime\" contains NA, run order ',
                                'will not be plotted') }
            acquTime <- NULL  # helps with unittesting as dates in $plot_env
                                # introduce OS differences
        } else {
            # check acquTime length
            if (nbSpl != length(acquTime)) {
                stop('Err','or: \"apexValue\" and \"acquTime\" must be the',
                            ' same length')
            } else { useRunOrder <- TRUE }
        }
    }
    # check widthMin & widthMax
    useWidth <- FALSE
    if (!is.null(widthMin) & !is.null(widthMax)) {
        # check length
        if ((nbSpl != length(widthMin)) | (nbSpl != length(widthMax))) {
            stop('\"apexValue\", \"widthMin\" and \"widthMax\" must be ',
                        'the same length')
        } else { useWidth <- TRUE }
    }
    # set default colour (add a sample color ID that will be match in the plot)
    colourSpl <- rep("black", nbSpl)
    if (!is.null(sampleColour)) {
        if (nbSpl == length(sampleColour)) {
            colourSpl <- sampleColour
        } else {
            if (verbose) {
                message('War','ning: sampleColour length must match the ',
                                'number of samples; default colour used') }
        }
    }
    sampleIDColour <- paste("spl", seq(1, nbSpl), sep = "")
    names(colourSpl) <- sampleIDColour

    return(list(nbSpl=nbSpl, useRunOrder=useRunOrder, useWidth=useWidth,
                colourSpl=colourSpl, sampleIDColour=sampleIDColour))
}
# Plot peakwidth, apex values and apex points (flip x/y if rotateAxis)
plotPeakwidth_plotWidthApex <- function(p, useWidth, rotateAxis, 
            x_axis, widthMin, widthMax, sampleIDColour, apexValue, verbose) {

    ## Apex value
    # add apex point (add the color ID to each point, and an ID pointing to the 
    # black stroke)
    tmp_pt <- data.frame(x = x_axis, y = apexValue, colr = sampleIDColour,
                        stringsAsFactors = FALSE)
    tmp_pt <- tmp_pt[!is.na(tmp_pt$y), ]

    # with peakwidth and black stroke
    if (useWidth) {
        ## Prepare peakWidth (must go first to be the bottom most layer)
        tmp_pwidth <- data.frame(x = c(x_axis, x_axis),
                                y = c(widthMin, widthMax),
                                colr = c(sampleIDColour, sampleIDColour),
                                stringsAsFactors = FALSE)
        tmp_pwidth <- tmp_pwidth[!is.na(tmp_pwidth$y),]#remove val without pkwdt
        if (rotateAxis) { #flip x and y
            p <- p + ggplot2::geom_line(data=tmp_pwidth,
                ggplot2::aes(x=y, y=x, group=x, colour=colr), linewidth=1)
            p <- p + ggplot2::geom_point(data = tmp_pt, 
                ggplot2::aes(x=y, y=x, fill=colr), 
                colour="black", shape=21, stroke=1, size=2)
        } else {
            p <- p + ggplot2::geom_line(data = tmp_pwidth, 
                ggplot2::aes(x=x, y=y, group=x, colour=colr), linewidth=1)
            p <- p + ggplot2::geom_point(data = tmp_pt, 
                ggplot2::aes(x=x, y=y, fill=colr), 
                colour="black", shape=21, stroke=1, size=2)
        }
        if (verbose) { message("Peakwidth values plotted") }

    # without peakwidth and black stroke
    } else {
        if (rotateAxis) { #flip x and y
            p <- p + ggplot2::geom_point(data=tmp_pt,
                ggplot2::aes(x=y, y=x, colour=colr))
        } else {
            p <- p + ggplot2::geom_point(data = tmp_pt, 
                ggplot2::aes(x=x, y=y, colour=colr))
        }
    }
    return(p)
}
# Set labels and rotate axis if necessary
plotPeakwidth_rotateAxis_setLabels <- function(p, rotateAxis, useRunOrder, 
                                        x_axis, acquTime, varName, verbose) {
    coord_y_datetime <- function(xlim = NULL, ylim = NULL, expand = TRUE) {
        if (!is.null(ylim)) { ylim <- lubridate::as_datetime(ylim) }
        ggplot2::coord_cartesian(xlim = xlim, ylim = ylim, expand = expand)
    }
    ## Ordering of values and axis labelling
    if (useRunOrder) {
        if (rotateAxis) { # date axis rotated (+ flipped)
            p <- p + coord_y_datetime(ylim = c(max(acquTime), min(acquTime))) +
                ggplot2::xlab(varName) + 
                ggplot2::ylab("Acquisition Time") +
                ggplot2::theme(
                        axis.text.y=ggplot2::element_text(angle=45, hjust=1)) +
                ggplot2::scale_y_datetime(expand = c(0.007, 0.007))
            if (verbose) { message("x and y axis rotated") }
        } else {          # date axis not rotated
            p <- p + ggplot2::xlab("Acquisition Time") +
                ggplot2::ylab(varName) + 
                ggplot2::theme(
                        axis.text.x=ggplot2::element_text(angle=45, hjust=1)) +
                ggplot2::scale_x_datetime(expand = c(0.007, 0.007))
        }
    } else {
        if (rotateAxis) { # input order axis rotated (+ flipped)
            suppressMessages(
            p <- p +  #needs to go first
                ggplot2::scale_y_continuous(breaks=NULL, expand=c(0.007,0.007))+
                ggplot2::ylim(max(x_axis), min(x_axis)) +
                ggplot2::xlab(varName) + 
                ggplot2::theme(axis.title.y = ggplot2::element_blank(),
                                axis.text.y = ggplot2::element_blank(),
                                axis.ticks.y = ggplot2::element_blank())
            )
            if (verbose) { message("x and y axis rotated") }
        } else {          # input order axis not rotated
            p <- p + ggplot2::ylab(varName) + 
                ggplot2::theme(axis.title.x = ggplot2::element_blank(),
                                axis.text.x = ggplot2::element_blank(),
                                axis.ticks.x = ggplot2::element_blank()) +
                ggplot2::scale_x_continuous(breaks=NULL, expand=c(0.007,0.007))
        }
    }
    return(p)
}

context('peakPantheR_plotPeakWidth()')

# remove Rplots.pdf created by ggplot2
on.exit( tryCatch({ file.remove('./Rplots.pdf') }, error=function(e){ invisible() }, warning=function(w){ invisible() }) )


## Input data
apexVal <- c(1, 2, 3, 4)
minVal  <- c(0, 0, 2, 2)
maxVal  <- c(2, 4, 4, 5)
acqTime <- as.POSIXct(c("2017-07-13 21:06:14", "2017-07-14 21:06:14", "2017-07-15 21:06:14", "2017-07-16 21:06:14"))



test_that('plot 3 samples, no color', {
	# generate plot
  result_3splNoColNoRot <- peakPantheR_plotPeakwidth(apexValue=apexVal[1:3], widthMin=minVal[1:3], widthMax=maxVal[1:3], 
                                                     acquTime=NULL, varName='Test variable', sampleColour=NULL,
                                                     rotateAxis=FALSE, verbose=FALSE)
  
  # Check plot properties
  expect_true(ggplot2::is_ggplot(result_3splNoColNoRot))
  expect_null(result_3splNoColNoRot$labels$x)
  expect_equal(result_3splNoColNoRot$labels$y, "Test variable")
  expect_equal(length(result_3splNoColNoRot$layers), 2)
})

test_that('plot 3 samples, no color, acquisition time', {
  # generate plot
  result_3splNoColNoRotRunOrder <- peakPantheR_plotPeakwidth(apexValue=apexVal[1:3], widthMin=minVal[1:3], widthMax=maxVal[1:3], 
                                                             acquTime=acqTime[1:3], varName='Test variable 2', sampleColour=NULL,
                                                             rotateAxis=FALSE, verbose=FALSE)
  
  # Check plot properties
  expect_true(ggplot2::is_ggplot(result_3splNoColNoRotRunOrder))
  expect_equal(result_3splNoColNoRotRunOrder$labels$x, "Acquisition Time")
  expect_equal(result_3splNoColNoRotRunOrder$labels$y, "Test variable 2")
  expect_equal(length(result_3splNoColNoRotRunOrder$layers), 2)
})

test_that('plot 3 samples, no color, rotate axis', {
  # generate plot
  result_3splNoColRot <- peakPantheR_plotPeakwidth(apexValue=apexVal[1:3], widthMin=minVal[1:3], widthMax=maxVal[1:3],
                                                   acquTime=NULL, varName='Test variable 3', sampleColour=NULL,
                                                   rotateAxis=TRUE, verbose=FALSE)
  
  # Check plot properties
  expect_true(ggplot2::is_ggplot(result_3splNoColRot))
  expect_equal(result_3splNoColRot$labels$x, "Test variable 3")
  expect_null(result_3splNoColRot$labels$y)
  expect_equal(length(result_3splNoColRot$layers), 2)
})

test_that('plot 3 samples, no color, acquisition time, rotate axis', {
  # generate plot
  result_3splNoColRotRunOrder <- peakPantheR_plotPeakwidth(apexValue=apexVal[1:3], widthMin=minVal[1:3], widthMax=maxVal[1:3], 
                                                           acquTime=acqTime[1:3], varName='Test variable 4', sampleColour=NULL,
                                                           rotateAxis=TRUE, verbose=FALSE)
  
  # Check plot properties
  expect_true(ggplot2::is_ggplot(result_3splNoColRotRunOrder))
  expect_equal(result_3splNoColRotRunOrder$labels$x, "Test variable 4")
  expect_equal(result_3splNoColRotRunOrder$labels$y, "Acquisition Time")
  expect_equal(length(result_3splNoColRotRunOrder$layers), 2)
})

test_that('plot 4 samples with color', {
  # generate plot
  result_4splWithColNoRot <- peakPantheR_plotPeakwidth(apexValue=apexVal, widthMin=minVal, widthMax=maxVal, 
                                                       acquTime=NULL, varName='Test variable 5', sampleColour=c('blue','red','green','orange'),
                                                       rotateAxis=FALSE, verbose=FALSE)
  
  # Check plot properties
  expect_true(ggplot2::is_ggplot(result_4splWithColNoRot))
  expect_null(result_4splWithColNoRot$labels$x)
  expect_equal(result_4splWithColNoRot$labels$y, "Test variable 5")
  expect_equal(length(result_4splWithColNoRot$layers), 2)
})

test_that('plot 4 samples with color, acquisition time', {
  # generate plot
  result_4splWithColNoRotRunOrder <- peakPantheR_plotPeakwidth(apexValue=apexVal, widthMin=minVal, widthMax=maxVal,
                                                               acquTime=acqTime, varName='Test variable 6',
                                                               sampleColour=c('blue','red','green','orange'), rotateAxis=FALSE, verbose=FALSE)
  
  # Check plot properties
  expect_true(ggplot2::is_ggplot(result_4splWithColNoRotRunOrder))
  expect_equal(result_4splWithColNoRotRunOrder$labels$x, "Acquisition Time")
  expect_equal(result_4splWithColNoRotRunOrder$labels$y, "Test variable 6")
  expect_equal(length(result_4splWithColNoRotRunOrder$layers), 2)
})

test_that('plot 4 samples with color, rotate axis', {
  # generate plot
  result_4splWithColRot <- peakPantheR_plotPeakwidth(apexValue=apexVal, widthMin=minVal, widthMax=maxVal, 
                                                     acquTime=NULL, varName='Test variable 7', sampleColour=c('blue','red','green','orange'),
                                                     rotateAxis=TRUE, verbose=FALSE)
  
  # Check plot properties
  expect_true(ggplot2::is_ggplot(result_4splWithColRot))
  expect_equal(result_4splWithColRot$labels$x, "Test variable 7")
  expect_null(result_4splWithColRot$labels$y)
  expect_equal(length(result_4splWithColRot$layers), 2)
})

test_that('plot 4 samples with color, acquisition time, rotate axis', {
  # generate plot
  result_4splWithColRotRunOrder <- peakPantheR_plotPeakwidth(apexValue=apexVal, widthMin=minVal, widthMax=maxVal,
                                                             acquTime=acqTime, varName='Test variable 8',
                                                             sampleColour=c('blue','red','green','orange'), rotateAxis=FALSE, verbose=FALSE)
  
  # Check plot properties
  expect_true(ggplot2::is_ggplot(result_4splWithColRotRunOrder))
  expect_equal(result_4splWithColRotRunOrder$labels$x, "Acquisition Time")
  expect_equal(result_4splWithColRotRunOrder$labels$y, "Test variable 8")
  expect_equal(length(result_4splWithColRotRunOrder$layers), 2)
})

test_that('sampleColour length warning, peakwidth and rotate and input order message', {
  # Expected message
  expected_message <- c("Warning: sampleColour length must match the number of samples; default colour used\n", "Peakwidth values plotted\n", "Values plotted by input order\n", "x and y axis rotated\n")
  
  # generate plot
  result_plotColourWarning <- evaluate_promise(peakPantheR_plotPeakwidth(apexVal, minVal, maxVal, acquTime=NULL,
                                                                         varName='Test variable 9', sampleColour=c('blue','red','green'),
                                                                         rotateAxis=TRUE, verbose=TRUE))
  
  # check messages confirming the replacements
  expect_equal(result_plotColourWarning$messages, expected_message)
  
  # Check plot properties
  expect_true(ggplot2::is_ggplot(result_plotColourWarning$result))
  expect_equal(result_plotColourWarning$result$labels$x, "Test variable 9")
  expect_null(result_plotColourWarning$result$labels$y)
  expect_equal(length(result_plotColourWarning$result$layers), 2)
})

test_that('sampleColour length warning, no peakwidth (missing widthMax) and no rotate and run order message', {
  # Expected message
  expected_message <- c("Warning: sampleColour length must match the number of samples; default colour used\n", "Values plotted by run order\n")
  
  # generate plot
  result_plotColourWarningRunOrder <- evaluate_promise(peakPantheR_plotPeakwidth(apexVal, widthMin=minVal, widthMax=NULL, acquTime=acqTime,
                                                                                 varName='Test variable 10', sampleColour=c('blue','red','green'),
                                                                                 rotateAxis=FALSE, verbose=TRUE))
  
  # check messages confirming the replacements
  expect_equal(result_plotColourWarningRunOrder$messages, expected_message)
  
  # Check plot properties
  expect_true(ggplot2::is_ggplot(result_plotColourWarningRunOrder$result))
  expect_equal(result_plotColourWarningRunOrder$result$labels$x, "Acquisition Time")
  expect_equal(result_plotColourWarningRunOrder$result$labels$y, "Test variable 10")
  expect_equal(length(result_plotColourWarningRunOrder$result$layers), 1) # no geom_line
})

test_that('missing widthMin, no peakwidth plot', {
  # Expected message
  expected_message <- c("Values plotted by input order\n")
  
  # generate plot
  result_plotNoWidthMin <- evaluate_promise(peakPantheR_plotPeakwidth(apexValue=apexVal, widthMin=NULL, widthMax=maxVal, acquTime=NULL,
                                                                      varName='Test variable 11', sampleColour=NULL,
                                                                      rotateAxis=FALSE, verbose=TRUE))
  
  # check messages confirming the replacements
  expect_equal(result_plotNoWidthMin$messages, expected_message)
  
  # Check plot properties
  expect_true(ggplot2::is_ggplot(result_plotNoWidthMin$result))
  expect_null(result_plotNoWidthMin$result$labels$x)
  expect_equal(result_plotNoWidthMin$result$labels$y, "Test variable 11")
  expect_equal(length(result_plotNoWidthMin$result$layers), 1) # no geom_line
})

test_that('missing widthMax, no peakwidth plot', {
  # Expected message
  expected_message <- c("Values plotted by input order\n")
  
  # generate plot
  result_plotNoWidthMax <- evaluate_promise(peakPantheR_plotPeakwidth(apexValue=apexVal, widthMin=NULL, widthMax=maxVal, acquTime=NULL,
                                                                      varName='Test variable 11', sampleColour=NULL,
                                                                      rotateAxis=FALSE, verbose=TRUE))
  
  # check messages confirming the replacements
  expect_equal(result_plotNoWidthMax$messages, expected_message)
  
  # Check plot properties
  expect_true(ggplot2::is_ggplot(result_plotNoWidthMax$result))
  expect_null(result_plotNoWidthMax$result$labels$x)
  expect_equal(result_plotNoWidthMax$result$labels$y, "Test variable 11")
  expect_equal(length(result_plotNoWidthMax$result$layers), 1) # no geom_line
})

test_that('NA in acquisition time, revert to input order plot', {
  # Expected message
  expected_message <- c("Warning: \"acquTime\" contains NA, run order will not be plotted\n", "Peakwidth values plotted\n", "Values plotted by input order\n" )
  
  # generate plot
  result_plotFallBackNAAcqu <- evaluate_promise(peakPantheR_plotPeakwidth(apexValue=apexVal, widthMin=minVal, widthMax=maxVal,
                                                                          acquTime=c(acqTime[1:3],NA), varName='Test variable 12',
                                                                          sampleColour=NULL, rotateAxis=FALSE, verbose=TRUE))
  
  # check messages confirming the replacements
  expect_equal(result_plotFallBackNAAcqu$messages, expected_message)
  
  # Check plot properties
  expect_true(ggplot2::is_ggplot(result_plotFallBackNAAcqu$result))
  expect_null(result_plotFallBackNAAcqu$result$labels$x)
  expect_equal(result_plotFallBackNAAcqu$result$labels$y, "Test variable 12")
  expect_equal(length(result_plotFallBackNAAcqu$result$layers), 2)
})

test_that('raise errors', {
  # acquTime is not POSIXct
  msg1    <- c('Error: \"acquTime\" must be a vector of POSIXct')
  expect_error(peakPantheR_plotPeakwidth(apexVal=apexVal, acquTime='not a POSIXct'), msg1, fixed=TRUE)
  
  # apexValue/acquTime length is wrong
  msg2    <- c('\"apexValue\" and \"acquTime\" must be the same length')
  expect_error(peakPantheR_plotPeakwidth(apexValue=apexVal[1:3], acquTime=acqTime), msg2, fixed=TRUE)
  
  # apexValue length is wrong
  msg3    <- c('"apexValue", "widthMin" and "widthMax" must be the same length')
  expect_error(peakPantheR_plotPeakwidth(apexVal[1:3], widthMin=minVal, widthMax=maxVal), msg3, fixed=TRUE)
  
  # widthMin length is wrong
  msg4    <- c('"apexValue", "widthMin" and "widthMax" must be the same length')
  expect_error(peakPantheR_plotPeakwidth(apexVal, widthMin=minVal[1:3], widthMax=maxVal), msg4, fixed=TRUE)
  
  # widthMax length is wrong
  msg5    <- c('"apexValue", "widthMin" and "widthMax" must be the same length')
  expect_error(peakPantheR_plotPeakwidth(apexVal, widthMin=minVal, widthMax=maxVal[1:3]), msg5, fixed=TRUE)
})

#' @title Extract and plot a EIC from a raw data file
#'
#' @description Simple plot of an Extracted Ion Chromatogram (EIC) from a raw 
#' data file and a provided mz and rt window. Return ggplot plot object.
#' 
#' @param spectraPath (str) Path to the raw data file to read (uses 
#' `MSnbase::readMSData()`)
#' @param rt (numeric(2) or two-column matrix) the lower and upper retention
#' time range from which the data should be extracted. If a matrix is passed,
#' each row corresponds to a different window. If not provided, the full
#' retention time range will be extracted.
#' @param mz (numeric(2) or two-column matrix) the lower and upper mass range
#' from which the data should be extracted. If a matrix is passed, each row
#' corresponds to a different window. If not provided, the full mass range will
#' be extracted.
#' @param valuesOnly (str) If 'Raw' only load the file and return a table of raw
#' extracted values (exported version of `extractSignalRawData()`). If `EIC`
#' return a table of EIC data point. Else return the EIC plot (default).
#' @param centroided (bool) Indicate to `MSnbase::readMSData()` whether the spectra
#' file is centroided or not (default to 'TRUE').
#' @param msLevel (int) The MS level at which the data should be extracted 
#' (default to MS level 1).
#' @param verbose (bool) Output proress information or not.
#' 
#' @return Grob (ggplot object) of the EIC plot, if `valuesOnly='Raw'` returns 
#' a data.frame of raw datapoints with as columns 'rt', 'mz' and 'int'. If 
#' `valuesOnly='EIC'` returns a data.frame of EIC datapoints with as columns 
#' 'rt' and 'int'.
#'
#' @export
#' 
#' @examples
#' ## Use a file form the faahKO package and plot an EIC of interest
#' library(faahKO)
#' spectraPath <- system.file('cdf/KO/ko15.CDF',package='faahKO')
#' peakPantheR_quickEIC(spectraPath,
#'                      rt = c(3290., 3410.),
#'                      mz = c(522.194778, 522.205222))
peakPantheR_quickEIC <- function(spectraPath, rt, mz, valuesOnly='Plot',
                            centroided=TRUE, msLevel=1L, verbose=TRUE) {
    # Initialise file with MSnbase
    rawSpec <- MSnbase::readMSData(spectraPath,
                                    centroided=centroided,
                                    mode='onDisk')
    # Extract signals
    dataPoints  <- data.frame(extractSignalRawData(rawSpec,
                                                    rt = rt, 
                                                    mz = mz, 
                                                    msLevel = msLevel,
                                                    verbose=verbose))
    # Raw valuesOnly
    if (valuesOnly == 'Raw') { return(dataPoints) }

    # Compute EIC
    EIC <- generateIonChromatogram(dataPoints, aggregationFunction = 'sum')

    # EIC valuesOnly
    if (valuesOnly == 'EIC') {return(EIC)}

    # Plot
    p_spec <- ggplot2::ggplot(NULL, ggplot2::aes(x),
                                environment = environment()) +
        ggplot2::theme_bw() + ggplot2::xlab("Retention Time (sec)") +
        ggplot2::ylab("Intensity") +
        ggplot2::scale_y_continuous(expand = c(0.01, 0.01)) +
        ggplot2::geom_line(data = EIC, ggplot2::aes(x = rt, y = int))
    return(p_spec)
}

#' @title Integrate fallback integration regions
#'
#' @description Integrate region defined in FIR if a feature is not found
#'
#' @param rawSpec an \code{\link[MSnbase]{OnDiskMSnExp-class}}
#' @param FIR (data.frame) Fallback Integration Regions (FIR) to integrate when
#' a feature is not found. Compounds as row are identical to the targeted
#' features, columns are \code{rtMin} (float in seconds), \code{rtMax} (float in
#' seconds), \code{mzMin} (float), \code{mzMax} (float)
#' @param foundPeakTable a \code{data.frame} as generated by
#' \link{findTargetFeatures}, with features as rows and peak properties as
#' columns. The following columns are mandatory: \code{found}, \code{is_filled},
#' \code{mz}, \code{mzMin}, \code{mzMax}, \code{rt}, \code{rtMin}, \code{rtMax},
#' \code{peakArea}, \code{peakAreaRaw}, \code{maxIntMeasured},
#' \code{maxIntPredicted}.
#' @param verbose (bool) if TRUE message progress
#'
#' @return an updated foundPeakTable with FIR integration values
integrateFIR <- function(rawSpec, FIR, foundPeakTable, verbose = TRUE) {
    # Check input
    if (dim(FIR)[1] != dim(foundPeakTable)[1]) {
        stop("Check input, FIR must have the same number of rows as ",
            "foundPeakTable") }
    # init
    stime               <- Sys.time()
    needsFilling        <- !(foundPeakTable$found)
    needsFilling_idx    <- seq(dim(foundPeakTable)[1])[needsFilling]
    outTable            <- foundPeakTable

    # only run where replacement is needed
    if (sum(needsFilling) != 0) { if (verbose) {
            message(sum(needsFilling), " features to integrate with FIR") }
        # store results
        tmpResult <- data.frame(matrix(vector(), sum(needsFilling), 10,
                                dimnames = list(c(),
                                                c("mzMin", "mz", "mzMax",
                                                "rtMin", "rt", "rtMax",
                                                "peakArea", "peakAreaRaw",
                                                "maxIntMeasured",
                                                "maxIntPredicted"))))
        # extract data for all fallback windows from raw (list of windows)
        all_peakData <- extractSignalRawData(rawSpec,
            mz = data.frame(mzMin = FIR$mzMin[needsFilling_idx],
                mzMax = FIR$mzMax[needsFilling_idx]),
            rt = data.frame(rtMin = FIR$rtMin[needsFilling_idx],
                rtMax = FIR$rtMax[needsFilling_idx]),
            verbose = verbose)
        # iterate over features to integrate
        tmpResult <- integrateFIR_features(needsFilling_idx, all_peakData, FIR,
                                            tmpResult, verbose)
        # Replace results with FIR integration
        outTable[needsFilling_idx,
            c("mzMin", "mz", "mzMax", "rtMin", "rt", "rtMax", "peakArea",
            "peakAreaRaw", "maxIntMeasured", "maxIntPredicted")] <-
            tmpResult[needsFilling_idx, c("mzMin", "mz", "mzMax", "rtMin", "rt",
            "rtMax", "peakArea", "peakAreaRaw", "maxIntMeasured",
            "maxIntPredicted")]
        outTable$is_filled[needsFilling_idx] <- TRUE
        outTable$found[needsFilling_idx] <- TRUE }

    # Output
    etime <- Sys.time()
    if (verbose) {
        message("FIR integrated in: ",round(as.double(difftime(etime,stime)),2),
                " ", units(difftime(etime, stime)))
    }
    return(outTable)
}

# -----------------------------------------------------------------------------
# integrateFIR helper functions

# iterate over features to integrate
integrateFIR_features <- function(needsFilling_idx, all_peakData, FIR,
                                    tmpResult, verbose){
    for (cnt in seq_len(length(needsFilling_idx))) {
        peakData    <- all_peakData[[cnt]]
        i           <- needsFilling_idx[cnt]

        if (dim(peakData)[1] != 0){ #Only continue if a scan is found in the box
            rtRange <- unique(peakData$rt)
            # mzMin, mzMax, rtMin, rtMax (values taken from input)
            tmpResult[i, c("mzMin", "mzMax", "rtMin", "rtMax")] <- FIR[i,
                c("mzMin", "mzMax", "rtMin", "rtMax")]
            # rt (rt of max intensity)
            tmpResult[i, "rt"] <- peakData$rt[which.max(peakData$i)]
            # mz (weighted average of total intensity across all rt for each mz)
            # total intensity across rt for each mz
            mzRange <- unique(peakData$mz)
            mzTotalIntensity <- vapply(mzRange, function(x) {
            sum(peakData$i[peakData$mz == x])}, FUN.VALUE = numeric(1))
            # mz (is weighted average)
            tmpResult[i, "mz"] <- stats::weighted.mean(mzRange,mzTotalIntensity)
            # maxIntMeasured (max intensity)
            tmpResult[i, "maxIntMeasured"] <- max(peakData$i)
            # maxIntPredicted is NA (we don't have a fit)
            tmpResult[i, "maxIntPredicted"] <- as.numeric(NA)
            # into max intensity across mz for each rt
            rtMaxIntensity <- vapply(rtRange, function(x) {
            max(peakData$i[peakData$rt == x])}, FUN.VALUE = numeric(1))
            # peakArea/peakAreaRaw calculated trapezoid rule
            peakArea <- pracma::trapz(x=rtRange, y=rtMaxIntensity)
            tmpResult[i, "peakArea"] <- peakArea
            tmpResult[i, "peakAreaRaw"] <- peakArea

        } else { # If no scan found in that region, return default values
            if (verbose) { message('No scan present in the FIR # ', i,
                            ': rt and mz are set as the middle of the FIR box;',
                            ' peakArea, maxIntMeasured and maxIntPredicted are',
                            ' set to 0') }
            tmpResult[i, c("mzMin", "mzMax", "rtMin", "rtMax")] <- FIR[i,
                c("mzMin", "mzMax", "rtMin", "rtMax")]
            tmpResult[i, "rt"] <- mean(c(FIR$rtMin[i], FIR$rtMax[i]))
            tmpResult[i, "mz"] <- mean(c(FIR$mzMin[i], FIR$mzMax[i]))
            tmpResult[i, "peakArea"] <- 0
            tmpResult[i, "peakAreaRaw"] <- 0
            tmpResult[i, "maxIntMeasured"] <- 0
            tmpResult[i, "maxIntPredicted"] <- 0  }
    }
    return(tmpResult)
}

context('integrateFIR()')

skip_if_not_installed('faahKO',  minimum_version = '1.18.0')
library(faahKO)


## Input and expected data
# use ko15.CDf file from the pkg faahKO
raw_data <- MSnbase::readMSData( system.file('cdf/KO/ko15.CDF', package = "faahKO"), centroided=TRUE, mode='onDisk')

# full peakTable
full_peakTable     <- data.frame(matrix(vector(), 4, 18, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "peakAreaRaw", "maxIntMeasured", "maxIntPredicted", "cpdID", "cpdName", "is_filled", "ppm_error", "rt_dev_sec", "tailingFactor", "asymmetryFactor"))),stringsAsFactors=FALSE)
full_peakTable[1,] <- c(TRUE, 3309.7589296586070, 3346.8277590361445, 3385.4098874628098, 522.2, 522.2, 522.2, 26133727, 26133727, 889280, 901015.80529226747, 'ID-1', 'testCpd 1', FALSE, 0.023376160866574614, 1.93975903614455092, 1.0153573486330891, 1.0268238825675249)
full_peakTable[2,] <- c(TRUE, 3345.3766648628907, 3386.5288072289159, 3428.2788374983961, 496.2, 496.2, 496.2, 36498367, 36498367, 1128960, 1113576.69008227298, 'ID-2', 'testCpd 2', FALSE, 0.024601030353423384, 0.95180722891564074, 1.0053782620427065, 1.0093180792278085)
full_peakTable[3,] <- c(TRUE, 3451.2075903614455, 3451.5072891566265, 3501.6697504924518, 464.2, 464.2, 464.2, 7744647, 7744647, 380736, 389632.13549519412, 'ID-3', 'testCpd 3', FALSE, 0.026296922148575364, 2.92771084337346110, 207.6939219686769036, 380.5019028782010082)
full_peakTable[4,] <- c(TRUE, 3670.9201232710743, 3704.1427831325304, 3740.0172511251831, 536.2, 536.2, 536.2, 8692184, 8692184, 330176, 326763.87246511364, 'ID-4', 'testCpd 4', FALSE, 0.022765817240815486, 2.44578313253032320, 1.0305289730128095, 1.0536948855480386)
full_peakTable[,c(1,14)]       <- sapply(full_peakTable[,c(1,14)], as.logical)
full_peakTable[,c(2:11,15:18)] <- sapply(full_peakTable[,c(2:11,15:18)], as.numeric)

# FIR (matching found peaks dimension, need widening mz as test data doesn't result in mz width)
input_FIR           <- full_peakTable[,c("mzMin", "mzMax", "rtMin", "rtMax")]
input_FIR$mzMin     <- input_FIR$mzMin - 0.0005
input_FIR$mzMax     <- input_FIR$mzMax + 0.0005


test_that('no modification, no missing, no verbose', {
  # Expected peakTable
  expected_peakTable  <- full_peakTable

  # results (output, warnings and messages)
  result_integrateFIR <- evaluate_promise(integrateFIR(rawSpec=raw_data, foundPeakTable=full_peakTable, FIR=input_FIR, verbose=TRUE))

  # Check results
  expect_equal(result_integrateFIR$result, expected_peakTable)

  # Check messages (timing)
  expect_equal(length(result_integrateFIR$messages), 1)
})

test_that('3 missing with 1 which gives no scan, verbose', {
  # Input FIR with 1 empty box (without the widened mz cpd 3 should not return any matching scan)
  input_FIR_empty                     <- input_FIR
  input_FIR_empty[3,]                 <- full_peakTable[3,c("mzMin", "mzMax", "rtMin", "rtMax")]
  # Input peakTable (with 2, 3, 4 not found)
  notFound_peakTable                  <- full_peakTable
  notFound_peakTable[2,c(1:11,15:18)] <- c(FALSE, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA)
  notFound_peakTable[3,c(1:11,15:18)] <- c(FALSE, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA)
  notFound_peakTable[4,c(1:11,15:18)] <- c(FALSE, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA)
  # Expected peakTable
  expected_peakTable                  <- full_peakTable
  expected_peakTable[2,c(1:11,14:18)] <- c(TRUE, 3345.3766648628898, 3385.577, 3428.2788374984002, 496.1995, 496.20001220703125, 496.2005, 36498367, 36498367, 1128960, as.numeric(NA), TRUE, as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  expected_peakTable[3,c(1:11,14:18)] <- c(TRUE, 3451.20759036145, 3476.4386704269500, 3501.66975049245, 464.2, 464.2, 464.2, 0, 0, 0, 0, TRUE, as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  expected_peakTable[4,c(1:11,14:18)] <- c(TRUE, 3670.9201232710702, 3701.697, 3740.0172511251799, 536.1995, 536.20001220703125, 536.2005, 8692184, 8692184, 330176, as.numeric(NA), TRUE, as.numeric(NA), as.numeric(NA), as.numeric(NA), as.numeric(NA))
  expected_peakTable[,c(1,14)]        <- sapply(expected_peakTable[,c(1,14)], as.logical)
  # Expected message
  expected_messages                   <- c("3 features to integrate with FIR\n", "Reading data from 3 windows\n", "No scan present in the FIR # 3: rt and mz are set as the middle of the FIR box; peakArea, maxIntMeasured and maxIntPredicted are set to 0\n")

  # results (output, warnings and messages)
  result_integrateFIR           <- evaluate_promise(integrateFIR(rawSpec=raw_data, foundPeakTable=notFound_peakTable, FIR=input_FIR_empty, verbose=TRUE))

  # Check results
  expect_equal(result_integrateFIR$result, expected_peakTable)

  # Check messages (cannot check time on message)
  expect_equal(length(result_integrateFIR$messages), 5)
  expect_equal(result_integrateFIR$messages[c(1,2,4)], expected_messages)
})

test_that('raise errors', {
  # paths to trigger errors
  wrongFIRsize    <- input_FIR[1:3,]

  # foundPeakTable and FIR dimension mismatch
  expect_error(integrateFIR(rawSpec=raw_data, foundPeakTable=full_peakTable, FIR=wrongFIRsize, verbose=TRUE), "Check input, FIR must have the same number of rows as foundPeakTable")
})

# Results - Tab panel  ---------------------------------------------------- #

tabPanel("View results",

  # Conditional UI based on import successfully initialised
  uiOutput("notAnnotForResUI"),
  conditionalPanel(
    condition = "output.AnnotationDone=='yes'",
    fluidRow(

  ## Sidebar
      column(width=2,
        # Current annotation status
        uiOutput("showAnnotStatusRes")
      ), # end Sidebar column

  ## Main panel
      column(10,
        # Overall title
        h3('Results', style="color:#3e648d;font-weight:bold"),

        # Tabs
        shiny::tabsetPanel( id="ResultTabs", type="pills",

        # Overall results - TAB
          tabPanel("Overall results",
            uiOutput("peakTableControlUI"),
            uiOutput("overallResultsUI")
          ),    # end tabPanel

        # Per feature - TAB
          tabPanel("Results per targeted feature",
            uiOutput("resPerFeatControlUI"),
            uiOutput("featureResultsUI")
          ),    # end tabPanel

        # Per sample - TAB
          tabPanel("Results per sample",
            uiOutput("resPerSplControlUI"),
            uiOutput("sampleResultsUI")
          )     # end tabPanel
        )       # end tabsetPanels
      )   # end Main panel column
    )     # end fluidRow (sidebar + menu)
  )       # end conditional panel
)
# end RESULTS Tab panel --------------------------------------------------- #

#' @title An S4 class to represent peakPantheR annotation results
#'
#' @description The \code{peakPantheRAnnotation} class is designed to run and
#' store peakPantheR parallel annotation results. Instances of the class are
#' created with the \code{peakPantheRAnnotation} constructor function, which
#' initialises an object of proper dimension with \code{spectraPaths} (set
#' samples to process) and \code{targetFeatTable} (set compounds to target).
#' \code{spectraPaths} is a character vector of spectra file paths.
#' \code{targetFeatTable} is a \code{\link{data.frame}} of compounds to target
#' as rows and parameters as columns: \code{cpdID} (int), \code{cpdName} (str),
#' \code{rtMin} (float in seconds), \code{rt} (float in seconds, or \emph{NA}),
#' \code{rtMax} (float in seconds), \code{mzMin} (float), \code{mz} (float or
#' \emph{NA}), \code{mzMax} (float).
#'
#' @details The \code{validObject} method ensures the conformity of an object to
#' the \code{peakPantheRAnnotation-class}. The number of compounds is based on
#' \code{cpdID()} length, and the number of samples is based on
#' \code{filepath()} length. Slot type is not checked as \code{setClass}
#' enforces it. peakTables and EICs type are checked on the first list element.
#' \code{annotationTable(object, column)} where \emph{column} is a column from
#' \emph{peakTable}, returns a data.frame of values with the samples as rows,
#' ROI as columns.
#'
#' @slot cpdID A character vector of compound IDs, of length number of compounds
#' @slot cpdName A character vector of compound names, of length number of
#' compounds
#' @slot ROI A data.frame of Regions Of Interest (ROI) with compounds as row and
#' ROI parameters as columns: \code{rtMin} (float in seconds), \code{rt} (float
#' in seconds, or \emph{NA}), \code{rtMax} (float in seconds), \code{mzMin}
#' (float), \code{mz} (float or \emph{NA}), \code{mzMax} (float).
#' @slot FIR A data.frame of Fallback Integration Regions (FIR) with compounds
#' as row and FIR parameters as columns: \code{rtMin} (float in seconds),
#' \code{rtMax} (float in seconds), \code{mzMin} (float), \code{mzMax} (float).
#' @slot uROI A data.frame of updated Regions Of Interest (uROI) with compounds
#' as row and uROI parameters as columns: \code{rtMin} (float in seconds),
#' \code{rt} (float in seconds, or \emph{NA}), \code{rtMax} (float in seconds),
#' \code{mzMin} (float), \code{mz} (float or \emph{NA}), \code{mzMax} (float).
#' @slot filepath A character vector of file paths, of length number of spectra
#' files
#' @slot cpdMetadata A data.frame of compound metadata, with compounds as row
#' and metadata as columns
#' @slot spectraMetadata A data.frame of sample metadata, with samples as row
#' and metadata as columns
#' @slot acquisitionTime A character vector of acquisition date-time (converted
#' from POSIXct) or NA
#' @slot uROIExist A logical stating if uROI have been set
#' @slot useUROI A logical stating if uROI are to be used
#' @slot useFIR A logical stating if FIR are to be used
#' @slot TIC A numeric vector of TIC or NA, of length number of spectra files
#' @slot peakTables A list of peakTable data.frame, of length number of spectra
#' files. Each peakTable data.frame has compounds as rows and peak annotation
#' results as columns.
#' @slot dataPoints A list of length number of spectra files. Each list element
#' is a \emph{ROIsDataPoint} list of \code{data.frame} of raw data points for
#' each ROI/uROI (retention time 'rt', mass 'mz' and intensity 'int' (as column)
#' of each raw data points (as row))
#' @slot peakFit A list of length number of spectra files. Each list element is
#' a \emph{curveFit} list of \code{peakPantheR_curveFit} or NA for each ROI
#' @slot isAnnotated A logical stating if the annotation has taken place
#'
#' \subsection{Details:}{
#' The \emph{peakTables} \code{data.frame} are structured as follow:
#' \tabular{ll}{
#' cpdID \tab database compound ID\cr
#' cpdName \tab compound name\cr
#' found \tab was the peak found\cr
#' rt \tab retention time of peak apex (sec)\cr
#' rtMin \tab leading edge of peak retention time (sec) determined at 0.5\% of
#' apex intensity\cr
#' rtMax \tab trailing edge of peak retention time (sec) determined at 0.5\% of
#' apex intensity\cr
#' mz \tab weighted (by intensity) mean of peak m/z across scans\cr
#' mzMin \tab m/z peak minimum (between rtMin, rtMax)\cr
#' mzMax \tab m/z peak maximum (between rtMin, rtMax)\cr
#' peakArea \tab integrated peak area\cr
#' peakAreaRaw \tab integrated peak area from raw data points\cr
#' maxIntMeasured \tab maximum peak intensity in raw data\cr
#' maxIntPredicted \tab maximum peak intensity based on curve fit\cr
#' is_filled \tab Logical indicate if the feature was integrated using FIR
#' (Fallback Integration Region)\cr
#' ppm_error \tab difference in ppm between the expected and measured m/z\cr
#' rt_dev_sec \tab difference in seconds between the expected and measured rt\cr
#' tailingFactor \tab the tailing factor is a measure of peak tailing.It is
#' defined as the distance from the front slope of the peak to the back slope
#' divided by twice the distance from the center line of the peak to the front
#' slope, with all measurements made at 5\% of the maximum peak height. The
#' tailing factor of a peak will typically be similar to the asymmetry factor
#' for the same peak, but the two values cannot be directly converted\cr
#' asymmetryFactor \tab the asymmetry factor is a measure of peak tailing. It is
#' defined as the distance from the center line of the peak to the back slope
#' divided by the distance from the center line of the peak to the front slope,
#' with all measurements made at 10\% of the maximum peak height. The asymmetry
#' factor of a peak will typically be similar to the tailing factor for the same
#' peak, but the two values cannot be directly converted\cr
#' }
#' }
#'
#' @return (peakPantheRAnnotation)
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(),2,8,dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' annotation
#' # An object of class peakPantheRAnnotation
#' #  2 compounds in 3 samples.
#' #   updated ROI do not exist (uROI)
#' #   does not use updated ROI (uROI)
#' #   does not use fallback integration regions (FIR)
#' #   is not annotated
#'
#' slotNames(annotation)
#' # [1] 'cpdID'       'cpdName'      'ROI'             'FIR'       'uROI'
#' # [6] 'filepath'    'cpdMetadata'  'spectraMetadata' 'acquisitionTime'
#' # [10] 'uROIExist'  'useUROI'      'useFIR'          'TIC'       'peakTables'
#' # [15] 'dataPoints' 'peakFit'      'isAnnotated'
#'
#' ## Slots shouldn't be accessed directly, accessors are available:
#' cpdID(annotation)
#' # [1] 'ID-1' 'ID-2'
#' cpdName(annotation)
#' # [1] 'Cpd 1' 'Cpd 2'
#' ROI(annotation)
#' #   rtMin       rt rtMax    mzMin    mz    mzMax cpdID cpdName
#' # 1  3310 3344.888  3390 522.1948 522.2 522.2052  ID-1   Cpd 1
#' # 2  3280 3385.577  3440 496.1950 496.2 496.2050  ID-2   Cpd 2
#' FIR(annotation)
#' #   rtMin rtMax mzMin mzMax cpdID cpdName
#' # 1    NA    NA    NA    NA  ID-1   Cpd 1
#' # 2    NA    NA    NA    NA  ID-2   Cpd 2
#' uROI(annotation)
#' #   rtMin rt rtMax mzMin mz mzMax cpdID cpdName
#' # 1    NA NA    NA    NA NA    NA  ID-1   Cpd 1
#' # 2    NA NA    NA    NA NA    NA  ID-2   Cpd 2
#' filepath(annotation)
#' # [1] 'C:/R/R-3.6.0/library/faahKO/cdf/KO/ko15.CDF'
#' # [2]'C:/R/R-3.6.0/library/faahKO/cdf/KO/ko16.CDF'
#' # [3] 'C:/R/R-3.6.0/library/faahKO/cdf/KO/ko18.CDF'
#' cpdMetadata(annotation)
#' # data frame with 0 columns and 2 rows
#' spectraMetadata(annotation)
#' # data frame with 0 columns and 3 rows
#' acquisitionTime(annotation)
#' # [1] NA NA NA
#' uROIExist(annotation)
#' # [1] FALSE
#' useUROI(annotation)
#' # [1] FALSE
#' useFIR(annotation)
#' # [1] FALSE
#' TIC(annotation)
#' # [1] NA NA NA
#' peakTables(annotation)
#' # [[1]]
#' # NULL
#' # [[2]]
#' # NULL
#' # [[3]]
#' # NULL
#' dataPoints(annotation)
#' # [[1]]
#' # NULL
#' # [[2]]
#' # NULL
#' # [[3]]
#' # NULL
#' peakFit(annotation)
#' # [[1]]
#' # NULL
#' # [[2]]
#' # NULL
#' # [[3]]
#' # NULL
#' isAnnotated(annotation)
#' # [1] FALSE
#' }
#'
#' @family peakPantheR
#' @family parallelAnnotation
#'
#' @aliases peakPantheRAnnotation-class
#'
#' @import methods
#'
#' @export
peakPantheRAnnotation   <- setClass("peakPantheRAnnotation",
                                slot = c(cpdID = "character",
                                        cpdName = "character",
                                        ROI = "data.frame",
                                        FIR = "data.frame",
                                        uROI = "data.frame",
                                        filepath = "character",
                                        cpdMetadata = "data.frame",
                                        spectraMetadata = "data.frame",
                                        acquisitionTime = "character",
                                        uROIExist = "logical",
                                        useUROI = "logical",
                                        useFIR = "logical",
                                        TIC = "numeric",
                                        peakTables = "list",
                                        dataPoints = "list",
                                        peakFit = "list",
                                        isAnnotated = "logical"))

#' @title Search, integrate and report targeted features in a multiple spectra
#'
#' @description Integrate all target features in all files defined in the
#' initialised input object and store results. The use of updated ROI and the
#' integration of FIR are controled by the input object slots \code{useUROI} and
#' \code{useFIR}. Files are processed in parallel using
#' \link{peakPantheR_singleFileSearch}; \code{ncores} controls the number of
#' cores used for parallelisation, with \code{ncores=0} corresponding to serial
#' processing. If the processing of a file fails (file does not exist or error
#' during execution) the sample is removed from the outputed object.
#'
#' @param object (peakPantheRAnnotation) Initialised peakPantheRAnnotation
#' object defining the samples to process and compounds to target. The slots
#' \code{useUROI} and \code{useFIR} controls if uROI must be used and FIR
#' integrated if a feature is not found
#' @param ncores (int) Number of cores to use for parallelisation. Default 0 for
#' no parallelisation.
#' @param getAcquTime (bool) If TRUE will extract sample acquisition date-time
#' from the mzML metadata (the additional file access will impact run time)
#' @param resetWorkers (int) If 0, the parallel cluster is only initiated once.
#' If >0 the cluster will be reset (and the memory of each worker freed) once
#' \code{ncores * resetWorkers} files have been processed. Default value is 1,
#' the cluster is reset once \code{ncores} files have been processed. While
#' potentially impacting performance (need to wait until all \code{ncores *
#' resetWorkers} files are processed before restarting the cluster), shutting
#' down the workers processes regularly will ensure the OS can reallocate memory
#' more efficiently. For values >1, ensure sufficient system memory is available
#' @param centroided (bool) use TRUE if the data is centroided, used by
#' \code{\link[MSnbase]{readMSData}} when reading the raw data files
#' @param curveModel (str) specify the peak-shape model to fit,
#' by default \code{skewedGaussian}.
#' Accepted values are \code{skewedGaussian} and \code{emgGaussian}
#' @param verbose (bool) If TRUE message calculation progress, time taken,
#' number of features found (total and matched to targets) and failures
#' @param ... Passes arguments to \code{findTargetFeatures} to alter
#' peak-picking parameters
#'
#' @return a list: \code{list()$result} \emph{(peakPantheRAnnotation)} fully
#' annotated object, \code{list()$failures} \emph{(list)} list of failed samples
#' and error message
#'
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Load data
#' library(faahKO)
#' 
#' # 3 files
#' input_spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                         system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                         system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#' 
#' # 4 features
#' input_ROI     <- data.frame(matrix(vector(), 4, 8,
#'                     dimnames=list(c(), c('cpdID', 'cpdName', 'rtMin', 'rt',
#'                                         'rtMax', 'mzMin', 'mz', 'mzMax'))),
#'                     stringsAsFactors=FALSE)
#' input_ROI[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                     522.2, 522.205222)
#' input_ROI[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                     496.2, 496.204962)
#' input_ROI[3,] <- c('ID-3', 'Cpd 3', 3420., 3454.435, 3495., 464.195358,
#'                     464.2, 464.204642)
#' input_ROI[4,] <- c('ID-4', 'Cpd 4', 3670., 3701.697, 3745., 536.194638,
#'                     536.2, 536.205362)
#' input_ROI[,c(3:8)] <- vapply(input_ROI[,c(3:8)], as.numeric,
#'                             FUN.VALUE=numeric(4))
#' 
#' # Initialise object
#' initAnnotation <- peakPantheRAnnotation(spectraPaths=input_spectraPaths,
#'                                         targetFeatTable=input_ROI)
#' # to use updated ROI:
#' # uROIExist=TRUE, useUROI=TRUE, uROI=input_uROI
#' # to use FallBack Integration Regions:
#' # useFIR=TRUE, FIR=input_FIR
#' 
#' # Run serially
#' result_parallelAnnotation <- peakPantheR_parallelAnnotation(initAnnotation,
#'                                                         ncores=0,
#'                                                         getAcquTime=FALSE,
#'                                                         verbose=TRUE)
#' # Processing 4 compounds in 3 samples:
#' #  uROI:\tFALSE
#' #  FIR:\tFALSE
#' # ----- ko15 -----
#' # Polarity can not be extracted from netCDF files, please set manually the
#' #  polarity with the 'polarity' method.
#' # Reading data from 4 windows
#' # Data read in: 0.24 secs
#' # Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
#' #  mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and
#' #  ROI$mzMax for ROI #1
#' # Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
#' #  mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and
#' #  ROI$mzMax for ROI #3
#' # Found 4/4 features in 0.06 secs
#' # Peak statistics done in: 0.02 secs
#' # Feature search done in: 0.76 secs
#' # ----- ko16 -----
#' # Polarity can not be extracted from netCDF files, please set manually the
#' #  polarity with the 'polarity' method.
#' # Reading data from 4 windows
#' # Data read in: 0.24 secs
#' # Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
#' #  mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and
#' #  ROI$mzMax for ROI #1
#' # Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
#' #  mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and
#' #  ROI$mzMax for ROI #2
#' # Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
#' #  mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and
#' #  ROI$mzMax for ROI #3
#' # Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
#' #  mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and
#' #  ROI$mzMax for ROI #4
#' # Found 4/4 features in 0.08 secs
#' # Peak statistics done in: 0 secs
#' # Feature search done in: 0.71 secs
#' # ----- ko18 -----
#' # Polarity can not be extracted from netCDF files, please set manually the
#' #  polarity with the 'polarity' method.
#' # Reading data from 4 windows
#' # Data read in: 0.25 secs
#' # Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
#' #  mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and
#' #  ROI$mzMax for ROI #1
#' # Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
#' #  mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and
#' #  ROI$mzMax for ROI #2
#' # Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
#' #  mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and
#' #  ROI$mzMax for ROI #4
#' # Found 4/4 features in 0.06 secs
#' # Peak statistics done in: 0 secs
#' # Feature search done in: 0.71 secs
#' # ----------------
#' # Parallel annotation done in: 2.18 secs
#' 
#' # No failures
#' result_parallelAnnotation$failures
#'
#' result_parallelAnnotation$annotation
#' # An object of class peakPantheRAnnotation
#' #  4 compounds in 3 samples. 
#' #    updated ROI do not exist (uROI)
#' #    does not use updated ROI (uROI)
#' #    does not use fallback integration regions (FIR)
#' #    is annotated
#' }
#'
#' @family peakPantheR
#' @family parallelAnnotation
#'
#' @import foreach
#' @import doParallel
#' @import mzR
#'
#' @export
peakPantheR_parallelAnnotation <- function(object, ncores = 0,
    getAcquTime = TRUE, resetWorkers = 1, centroided = TRUE,
    curveModel='skewedGaussian', verbose=TRUE, ...){

    # Check inputs, Initialise variables and outputs
    initRes <- parallelAnnotation_init(object, resetWorkers, verbose)
    file_paths<-initRes$file_paths; target_peak_table<-initRes$target_peak_table
    input_FIR<-initRes$input_FIR; resetWorkersMulti<-initRes$resetWorkersMulti

    stime <- Sys.time()
    
    # Run singleFileSearch
    # (list, each item is the result of a file, errors are passed into the list)
    allFilesRes <- parallelAnnotation_runSingleFileSearch(object, file_paths,
        target_peak_table, input_FIR, ncores, getAcquTime, resetWorkersMulti,
        centroided, curveModel, verbose,...)

    # Collect, process and reorder results
    res <- parallelAnnotation_process(allFilesRes, object, verbose)
    outObject <- res$outObject; fail_table <- res$fail_table

    ## check validity and exit
    validObject(outObject)
    
    etime <- Sys.time()
    if (verbose) {
        message("----------------")
        message("Parallel annotation done in: ",
            round(as.double(difftime(etime, stime)), 2), " ",
            units(difftime(etime, stime)))
        message("  ", dim(fail_table)[1], " failure(s)")
    }
    
    return(list(annotation = outObject, failures = fail_table))
}


# ------------------------------------------------------------------------------

## Check input file exist, wrap \code{peakPantheR_singleFileSearch} in a
## try cratch, add a failure status
# @param singleSpectraDataPath (str) path to netCDF or mzML raw data
# file (centroided, \strong{only with the channel of interest}).
# @param targetFeatTable a \code{\link{data.frame}} of compounds to
# target as rows. Columns: \code{cpdID} (str), \code{cpdName} (str),
# \code{rtMin} (float in seconds), \code{rt} (float in seconds, or
# \emph{NA}), \code{rtMax} (float in seconds), \code{mzMin} (float),
# \code{mz} (float or \emph{NA}), \code{mzMax} (float).
# @param FIR (data.frame or NULL) If not NULL, integrate Fallback
# Integration Regions (FIR) when a feature is not found. Compounds as
# row are identical to \code{targetFeatTable}, columns are \code{rtMin}
# (float in seconds), \code{rtMax} (float in seconds), \code{mzMin}
# (float), \code{mzMax} (float).
# @param getAcquTime (bool) If TRUE will extract sample acquisition
# date-time from the mzML metadata (the additional file access will
# impact run time) @param verbose (bool) If TRUE message calculation
# progress, time taken and number of features found (total and matched
# to targets)
# @param ... Passes arguments to \code{findTargetFeatures} to alter
# peak-picking parameters @return a list: \code{list()$TIC} \emph{(int)}
# TIC value, \code{list()$peakTable} \emph{(data.frame)} targeted
# features results (see Details), \code{list()$curveFit} \emph{(list)}
# list of \code{peakPantheR_curveFit} or NA for each ROI,
# \code{list()$acquTime} \emph{(POSIXct or NA)} date-time of sample
# acquisition from mzML metadata, \code{list()$ROIsDataPoint}
# \emph{(list)} a list of \code{data.frame} of raw data points for each
# ROI (retention time 'rt', mass 'mz' and intensity 'int' (as column) of
# each raw data points (as row)). \code{list()$failure} \emph{(named str
#  or NULL)} a string detailing the error (named with the
#  singleSpectraDataPath) or NA if the processing is successful.
parallelAnnotation_parallelHelper <- function(singleSpectraDataPath,
targetFeatTable, inFIR=NULL, inGetAcquTime=FALSE,centr=TRUE,
curveModel='skewedGaussian', inVerbose=TRUE,...){
    # Check input path exist or exit with error message
    if (!file.exists(singleSpectraDataPath)) {
        if (inVerbose) { message("Err","or file does not exist: ",
                singleSpectraDataPath) }
        # add error status
        failureMsg <- paste("Err","or file does not exist: ",
            singleSpectraDataPath, sep = "")
        names(failureMsg) <- singleSpectraDataPath
        # return basic values and failure message
        return(list(TIC = as.numeric(NA), peakTable = NULL,
        acquTime = as.character(NA), curveFit = NULL,
        ROIsDataPoint = NULL, failure = failureMsg)) }
    # Run singleFileSearch in try catch
    file_name <- tools::file_path_sans_ext(basename(singleSpectraDataPath))
    # progress
    if (inVerbose) { message("----- ", file_name, " -----") }
    # try catch
    result <- tryCatch({
        # singleFileSearch
        tmpResult <- peakPantheR_singleFileSearch(singleSpectraDataPath,
            targetFeatTable, peakStatistic = TRUE, plotEICsPath = NA,
            getAcquTime = inGetAcquTime, FIR = inFIR, centroided = centr,
            curveModel = curveModel, verbose = inVerbose, ...)
        # add failure status
        failureMsg <- NA
        names(failureMsg) <- singleSpectraDataPath
        tmpResult$failure <- failureMsg
        # last evaluation of Try is returned
        return(tmpResult)
    }, error = function(err) {
        # message error
        if (inVerbose) {
            message("-----")
            message("Err","or processing file: ", file_name)
            message(err$message)
            message("\n-----") }
        # add error status
        failureMsg <- err$message
        names(failureMsg) <- singleSpectraDataPath
        # return basic values and failure message
        return(list(TIC = as.numeric(NA), peakTable = NULL,
        acquTime = as.character(NA), curveFit = NULL,
            ROIsDataPoint = NULL, failure = failureMsg))  })
    gc(verbose = FALSE) # try clearing variables
    return(result)  # return singleFileSearch results with failure status
}


## Check inputs, Initialise variables and outputs
parallelAnnotation_init <- function(object, resetWorkers, verbose) {
    # check validity of object
    validObject(object)
    # check resetWorkers value
    if (!is.numeric(resetWorkers)) {
        stop("Check input, resetWorkers must be an integer")
    }
    resetWorkersMulti <- as.integer(resetWorkers)
    if (resetWorkersMulti < 0) {
        stop("Check input, resetWorkers must be a positive integer")
    }

    # Initialise parameters from object
    use_uROI <- useUROI(object)
    use_FIR <- useFIR(object)
    file_paths <- filepath(object)
    if (use_uROI) {
        target_peak_table <- uROI(object)
    } else {
        target_peak_table <- ROI(object)
    }
    if (use_FIR) {
        input_FIR <- FIR(object)
    } else {
        input_FIR <- NULL
    }

    # Output parameters
    if (verbose & isAnnotated(object)) {
        message("!! Data was already annotated, results will be overwritten !!")
    }
    if (verbose) {
        message("Processing ", nbCompounds(object), " compounds in ",
            nbSamples(object), " samples:")
        message("  uROI:\t", use_uROI)
        message("  FIR:\t", use_FIR)
    }

    return(list(file_paths=file_paths, target_peak_table=target_peak_table,
                input_FIR=input_FIR, resetWorkersMulti=resetWorkersMulti))
}


## Run singleFileSearch
# (list, each item is the result of a file, errors are passed into the list)
parallelAnnotation_runSingleFileSearch <- function(object, file_paths,
target_peak_table, input_FIR, ncores, getAcquTime, resetWorkersMulti,centroided,
curveModel, verbose,...) {
    if (ncores != 0) { # Parallel
        # Reinitialise the cluster after ncores files
        # (reset worker processes, freed memory can be reallocated by the OS)
        if (resetWorkersMulti != 0) {
            # Init
            nFiles <- nbSamples(object)
            allFilesRes <- vector("list", nFiles)
            nFilesPerClust <- (ncores * resetWorkersMulti)
            nClust <- ceiling(nFiles/nFilesPerClust)
            if (verbose) { message("Running ", nClust, " clusters of ",
                    nFilesPerClust, " files over ", ncores, " cores:") }
            # in each round start a new cluster and store results
            for (iClust in seq(0, nClust - 1, 1)) {
                if (verbose) {
                    message("  starting cluster ", iClust + 1, "/", nClust) }
                # init
                idxStart <- 1 + iClust * nFilesPerClust
                idxEnd <- min((nFilesPerClust + iClust * nFilesPerClust),nFiles)
                    # to not overshoot the number of files
                tmp_file_paths <- file_paths[idxStart:idxEnd]
                cl <- parallel::makeCluster(ncores)
                doParallel::registerDoParallel(cl)
                # Run
                allFilesRes[idxStart:idxEnd] <- foreach::foreach(
                x = tmp_file_paths, .packages = c("MSnbase", "mzR"),
                .inorder=TRUE) %dopar% parallelAnnotation_parallelHelper(x,
                target_peak_table,inFIR=input_FIR,inGetAcquTime=getAcquTime,
                centr = centroided, curveModel=curveModel,
                inVerbose = verbose, ...)
                parallel::stopCluster(cl) } # Close
        } else { # Single cluster init (workload can be balanced across workers)
            cl <- parallel::makeCluster(ncores)
            doParallel::registerDoParallel(cl)
            allFilesRes <- foreach::foreach(   # Run
                x = file_paths, .packages = c("MSnbase", "mzR"),
                .inorder = TRUE) %dopar% parallelAnnotation_parallelHelper(x,
                target_peak_table, inFIR=input_FIR, inGetAcquTime=getAcquTime,
                centr=centroided, curveModel=curveModel,
                inVerbose = verbose,...)#.errorhandling='pass'
                # #.export=c('findTargetFeatures', 'getTargetFeatureStatistic')
            parallel::stopCluster(cl) } # Close
    } else { # Serial
        allFilesRes <- lapply(file_paths,
            function(x) parallelAnnotation_parallelHelper(x, target_peak_table,
                inFIR=input_FIR, inGetAcquTime=getAcquTime, centr=centroided,
                curveModel=curveModel, inVerbose=verbose, ...)) }
    return(allFilesRes) }


## Collect, process and reorder results
parallelAnnotation_process <- function(allFilesRes, object, verbose) {
    # identify annotations that failed
    fail_status <- unlist(lapply(allFilesRes,
        function(x) {x$failure}), use.names = TRUE)
    failures <- !is.na(fail_status)
    names(failures) <- NULL
    fail_table <- data.frame(matrix(c(names(fail_status)[failures],
        fail_status[failures]), ncol = 2, byrow = FALSE,
        dimnames = list(c(), c("file", "error"))), stringsAsFactors = FALSE)
    # message failures
    if ((sum(failures) != 0) & verbose) {
        message("----------------")
        message(sum(failures), " file(s) failed to process:\n",
            paste0(capture.output(fail_table), collapse = "\n")) }
    # remove failures
    allFilesRes <- allFilesRes[!failures]
    # reshape the output object to match (remove failed samples)
    outObject <- object[!failures, ]
    # unlist result into final object (if there is a minimum of 1 file left)
    if (sum(!failures) > 0) {
        # acquisitionTime
        outObject@acquisitionTime <- vapply(allFilesRes, function(x) {
            as.character(x$acquTime)}, FUN.VALUE = character(1))
        # TIC
        outObject@TIC <- vapply(allFilesRes, function(x) {
            x$TIC}, FUN.VALUE = numeric(1))
        # peakTables (all columns but cpdID and cpdName)
        outObject@peakTables <- lapply(allFilesRes,
            function(x) {x$peakTable[, !names(x$peakTable) %in%
                c("cpdID", "cpdName")] })
        # dataPoints
        outObject@dataPoints <- lapply(allFilesRes,function(x){x$ROIsDataPoint})
        # peakFit
        outObject@peakFit <- lapply(allFilesRes, function(x) { x$curveFit })
        # isAnnotated
        outObject@isAnnotated <- TRUE
    } else { # All files failed
        if (verbose) { message("No file left in the object!") }
    }
    # reorder results by acquisition date if available
    if (sum(is.na(acquisitionTime(outObject))) == 0) {
        if (verbose) {
            message("Annotation object reordered by sample acquisition date") }
        outObject <- outObject[order(acquisitionTime(outObject)), ]
    } else {
        if (verbose) {
            message('Annotation object cannot be reordered by sample ',
                    'acquisition date') }
    }
    return(list(outObject=outObject, fail_table=fail_table)) }

#####################################################################
## Constructor for \link{peakPantheRAnnotation-class}, see function
# peakPantheRAnnotation() for the initialisation checks
setMethod("initialize", "peakPantheRAnnotation",
    function(.Object, ...) {
    ## load ... arguments
    .Object <- methods::callNextMethod(.Object, ...)
    
    # Check the validity of results
    methods::validObject(.Object)
    
    return(.Object)
})



#####################################################################
## show method for \link{peakPantheRAnnotation-class}
setMethod("show",
    signature = "peakPantheRAnnotation",
    definition = function(object) {
        cat("An object of class ", class(object), "\n", sep = "")
        cat(" ", length(object@cpdName), " compounds in ",
            length(object@filepath), " samples. \n", sep = "")
        if (object@uROIExist) {
            cat("  updated ROI exist (uROI)\n", sep = "")
        } else {
            cat("  updated ROI do not exist (uROI)\n", sep = "")
        }
        if (object@useUROI) {
            cat("  uses updated ROI (uROI)\n", sep = "")
        } else {
            cat("  does not use updated ROI (uROI)\n", sep = "")
        }
        if (object@useFIR) {
            cat("  uses fallback integration regions (FIR)\n", sep = "")
        } else {
            cat("  does not use fallback integration regions (FIR)\n", sep = "")
        }
        if (object@isAnnotated) {
            cat("  is annotated\n", sep = "")
        } else {
            cat("  is not annotated\n", sep = "")
        }
        invisible(NULL)
})



#####################################################################
## validObject method for \link{peakPantheRAnnotation-class}.
# Number of compounds based on @cpdID length, number of samples based on
# @filepath length. Slot type is not checked as \code{setClass} enforces it.
# peakTables, dataPoints and peakFit type are checked on first list element.
setValidity("peakPantheRAnnotation",
            function(object) valid_peakPantheRAnnotation(object))



#####################################################################
## Accessors
#####################################################################

# cpdID
setGeneric("cpdID", function(object, ...) standardGeneric("cpdID"))
#' cpdID accessor
#' @param object peakPantheRAnnotation
#' @return (str) A character vector of compound IDs, of length number of
#' compounds
#' @docType methods
#' @aliases cpdID
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                         c('cpdID','cpdName','rtMin','rt','rtMax','mzMin',
#'                         'mz', 'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' cpdID(annotation)
#' # [1] 'ID-1' 'ID-2'
#' }
setMethod("cpdID", "peakPantheRAnnotation", function(object) {
    object@cpdID
})

# cpdName
setGeneric("cpdName", function(object, ...) standardGeneric("cpdName"))
#' cpdName accessor
#' @param object peakPantheRAnnotation
#' @return (str) A character vector of compound names, of length number of
#' compounds
#' @docType methods
#' @aliases cpdName
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                 c('cpdID', 'cpdName', 'rtMin', 'rt', 'rtMax', 'mzMin', 'mz',
#'                 'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' cpdName(annotation)
#' # [1] 'Cpd 1' 'Cpd 2'
#' }
setMethod("cpdName", "peakPantheRAnnotation", function(object) {
    object@cpdName
})

# ROI targetFeatTable with ROI
setGeneric("ROI", function(object, ...) standardGeneric("ROI"))
#' ROI accessor returns targetFeatTable with cpdID, cpdName added
#' @param object peakPantheRAnnotation
#' @return (data.frame) target feature table with compounds as row and ROI
#' parameters as columns
#' @docType methods
#' @aliases ROI
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                 c('cpdID', 'cpdName', 'rtMin', 'rt', 'rtMax', 'mzMin', 'mz',
#'                 'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' ROI(annotation)
#' #   rtMin       rt rtMax    mzMin    mz    mzMax cpdID cpdName
#' # 1  3310 3344.888  3390 522.1948 522.2 522.2052  ID-1   Cpd 1
#' # 2  3280 3385.577  3440 496.1950 496.2 496.2050  ID-2   Cpd 2
#' }
setMethod("ROI", "peakPantheRAnnotation", function(object) {
    out <- object@ROI
    out$cpdID <- object@cpdID
    out$cpdName <- object@cpdName
    return(out)
})

# uROI targetFeatTable with uROI
setGeneric("uROI", function(object, ...) standardGeneric("uROI"))
#' uROI accessor returns targetFeatTable with cpdID, cpdName added
#' @param object peakPantheRAnnotation
#' @return (data.frame) target feature table with compounds as row and uROI
#' parameters as columns
#' @docType methods
#' @aliases uROI
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable     <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                 c('cpdID', 'cpdName', 'rtMin', 'rt', 'rtMax', 'mzMin', 'mz',
#'                 'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' ## default values without annotation
#' uROI(annotation)
#' #   rtMin rt rtMax mzMin mz mzMax cpdID cpdName
#' # 1    NA NA    NA    NA NA    NA  ID-1   Cpd 1
#' # 2    NA NA    NA    NA NA    NA  ID-2   Cpd 2
#' }
setMethod("uROI", "peakPantheRAnnotation", function(object) {
    out <- object@uROI
    out$cpdID <- object@cpdID
    out$cpdName <- object@cpdName
    return(out)
})

# FIR similar to targetFeatTable with FIR
setGeneric("FIR", function(object, ...) standardGeneric("FIR"))
#' FIR accessor returns targetFeatTable with cpdID, cpdName added
#' @param object peakPantheRAnnotation
#' @return (data.frame) target feature table with compounds as row and FIR
#' parameters as columns
#' @docType methods
#' @aliases FIR
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                 c('cpdID', 'cpdName', 'rtMin', 'rt', 'rtMax', 'mzMin', 'mz',
#'                 'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' ## default values without annotation
#' FIR(annotation)
#' #   rtMin rtMax mzMin mzMax cpdID cpdName
#' # 1    NA    NA    NA    NA  ID-1   Cpd 1
#' # 2    NA    NA    NA    NA  ID-2   Cpd 2
#' }
setMethod("FIR", "peakPantheRAnnotation", function(object) {
    out <- object@FIR
    out$cpdID <- object@cpdID
    out$cpdName <- object@cpdName
    return(out)
})

# filepath
setGeneric("filepath", function(object, ...) standardGeneric("filepath"))
#' filepath accessor
#' @param object peakPantheRAnnotation
#' @return (str) A character vector of file paths, of length number of spectra
#' files
#' @docType methods
#' @aliases filepath
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                 c('cpdID', 'cpdName', 'rtMin', 'rt', 'rtMax', 'mzMin', 'mz',
#'                 'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#' filepath(annotation)
#' # [1] 'C:/R/R-3.6.0/library/faahKO/cdf/KO/ko15.CDF'
#' # [2] 'C:/R/R-3.6.0/library/faahKO/cdf/KO/ko16.CDF'
#' # [3] 'C:/R/R-3.6.0/library/faahKO/cdf/KO/ko18.CDF'
#' }
setMethod("filepath", "peakPantheRAnnotation", function(object) {
    object@filepath
})

# cpdMetadata
setGeneric("cpdMetadata",
            function(object, ...) standardGeneric("cpdMetadata"))
#' cpdMetadata accessor
#' @param object peakPantheRAnnotation
#' @return (data.frame) A data.frame of compound metadata, with compounds as row
#' and metadata as columns
#' @docType methods
#' @aliases cpdMetadata
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' ## default values not initialised
#' cpdMetadata(annotation)
#' # data frame with 0 columns and 2 rows
#' }
setMethod("cpdMetadata", "peakPantheRAnnotation", function(object) {
    object@cpdMetadata
})

# spectraMetadata
setGeneric("spectraMetadata",
            function(object, ...) standardGeneric("spectraMetadata"))
#' spectraMetadata accessor
#' @param object peakPantheRAnnotation
#' @return (data.frame) A data.frame of sample metadata, with samples as row and
#' metadata as columns
#' @docType methods
#' @aliases spectraMetadata
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' ## default values not initialised
#' spectraMetadata(annotation)
#' # data frame with 0 columns and 3 rows
#' }
setMethod("spectraMetadata", "peakPantheRAnnotation",
    function(object) {
    object@spectraMetadata
})

# acquisitionTime return converted to POSIXct
setGeneric("acquisitionTime",
            function(object, ...) standardGeneric("acquisitionTime"))
#' acquisitionTime accessor returns value as.POSIXct
#' @param object peakPantheRAnnotation
#' @return (POSIXct) A character vector of acquisition date-time (converted from
#' POSIXct) or NA
#' @docType methods
#' @aliases acquisitionTime
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                 c('cpdID', 'cpdName', 'rtMin', 'rt', 'rtMax', 'mzMin', 'mz',
#'                 'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' ## acquisitionTime can only be extracted from NetCDF files
#' acquisitionTime(annotation)
#' # [1] NA NA NA
#' }
setMethod("acquisitionTime", "peakPantheRAnnotation",
    function(object) {
    as.POSIXct(object@acquisitionTime)
})

# uROIExist
setGeneric("uROIExist", function(object, ...) standardGeneric("uROIExist"))
#' uROIExist accessor
#' @param object peakPantheRAnnotation
#' @return (bool) flag if uROI have been set
#' @docType methods
#' @aliases uROIExist
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' uROIExist(annotation)
#' # [1] FALSE
#' }
setMethod("uROIExist", "peakPantheRAnnotation", function(object) {
    object@uROIExist
})

# useUROI
setGeneric("useUROI", function(object, ...) standardGeneric("useUROI"))
#' useUROI accessor
#' @param object peakPantheRAnnotation
#' @return (bool) flag if uROI are to be used
#' @docType methods
#' @aliases useUROI
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' useUROI(annotation)
#' # [1] FALSE
#' }
setMethod("useUROI", "peakPantheRAnnotation", function(object) {
    object@useUROI
})

# useFIR
setGeneric("useFIR", function(object, ...) standardGeneric("useFIR"))
#' useFIR accessor
#' @param object peakPantheRAnnotation
#' @return (bool) flag if FIR are to be used
#' @docType methods
#' @aliases useFIR
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' useFIR(annotation)
#' # [1] FALSE
#' }
setMethod("useFIR", "peakPantheRAnnotation", function(object) {
    object@useFIR
})

# TIC
setGeneric("TIC", function(object, ...) standardGeneric("TIC"))
#' TIC accessor
#' @param object peakPantheRAnnotation
#' @return (float) A numeric vector of Total Ion Chromatogram or NA, of length
#' number of spectra files
#' @docType methods
#' @aliases TIC
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' ## default values without annotation
#' TIC(annotation)
#' # [1] NA NA NA
#' }
setMethod("TIC", "peakPantheRAnnotation", function(object) {
    object@TIC
})

# peakTables
setGeneric("peakTables", function(object, ...) standardGeneric("peakTables"))
#' peakTables accessor with cpdID and cpdName added back
#' @param object peakPantheRAnnotation
#' @return (data.frame) A list of peakTable data.frame, of length number of
#' spectra files. Each peakTable data.frame has compounds as rows and peak
#' annotation results as columns, with added compound ID and name.
#' @docType methods
#' @aliases peakTables
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' ## default values without annotation
#' peakTables(annotation)
#' # [[1]]
#' # NULL
#' # [[2]]
#' # NULL
#' # [[3]]
#' # NULL
#' }
setMethod("peakTables", "peakPantheRAnnotation", function(object) {
    tmpPeakTables <- lapply(object@peakTables, function(x) {
        if (!is.null(x)) {
            return(cbind.data.frame(x, cpdID = object@cpdID,
                                    cpdName = object@cpdName,
                                    stringsAsFactors = FALSE))
        } else {
            return(NULL)
        }
    })
    
    return(tmpPeakTables)
})

# dataPoints
setGeneric("dataPoints", function(object, ...) standardGeneric("dataPoints"))
#' dataPoints accessor
#' @param object peakPantheRAnnotation
#' @return A list of length number of spectra files. Each list element is a
#' \emph{ROIsDataPoint} list of \code{data.frame} of raw data points for each
#' ROI/uROI (retention time 'rt', mass 'mz' and intensity 'int' (as column) of
#' each raw data points (as row))
#' @docType methods
#' @aliases dataPoints
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' ## default values without annotation
#' dataPoints(annotation)
#' # [[1]]
#' # NULL
#' # [[2]]
#' # NULL
#' # [[3]]
#' # NULL
#' }
setMethod("dataPoints", "peakPantheRAnnotation", function(object) {
    object@dataPoints
})

# peakFit
setGeneric("peakFit", function(object, ...) standardGeneric("peakFit"))
#' peakFit accessor
#' @param object peakPantheRAnnotation
#' @return A list of length number of spectra files. Each list element is a
#' \emph{curveFit} list of \code{peakPantheR_curveFit} or NA for each ROI
#' @docType methods
#' @aliases peakFit
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' ## default values without annotation
#' peakFit(annotation)
#' # [[1]]
#' # NULL
#' # [[2]]
#' # NULL
#' # [[3]]
#' # NULL
#' }
setMethod("peakFit", "peakPantheRAnnotation", function(object) {
    object@peakFit
})

# isAnnotated
setGeneric("isAnnotated", function(object, ...) standardGeneric("isAnnotated"))
#' isAnnotated accessor
#' @param object peakPantheRAnnotation
#' @return (bool) flag if the annotation has taken place
#' @docType methods
#' @aliases isAnnotated
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' isAnnotated(annotation)
#' # [1] FALSE
#' }
setMethod("isAnnotated", "peakPantheRAnnotation", function(object) {
    object@isAnnotated
})

# nbSamples
setGeneric("nbSamples", function(object, ...) standardGeneric("nbSamples"))
#' nbSamples accessor established on filepath
#' @param object peakPantheRAnnotation
#' @return (int) number of samples
#' @docType methods
#' @aliases nbSamples
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' nbSamples(annotation)
#' # [1] 3
#' }
setMethod("nbSamples", "peakPantheRAnnotation", function(object) {
    return(length(object@filepath))
})

# nbCompounds
setGeneric("nbCompounds", function(object, ...) standardGeneric("nbCompounds"))
#' nbCompounds accessor established on cpdID
#' @param object peakPantheRAnnotation
#' @return (int) number of samples
#' @docType methods
#' @aliases nbCompounds
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ##  compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' nbCompounds(annotation)
#' # [1] 2
#' }
setMethod("nbCompounds", "peakPantheRAnnotation", function(object) {
    return(length(object@cpdID))
})

# annotationTable
setGeneric("annotationTable",
            function(object, column) standardGeneric("annotationTable"))
#' @title annotationTable accessor
#' @description annotationTable returns a dataframe (row samples, col compounds)
#' filled with a specific peakTable column
#' @param object peakPantheRAnnotation
#' @param column a peakTable columns
#' @return (data.frame) (row samples, col compounds) filled with a specific
#' peakTable column
#' @docType methods
#' @aliases annotationTable
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' ## default values without annotation
#' annotationTable(annotation)
#' #                                             ID-1 ID-2
#' # C:/R/R-3.6.0/library/faahKO/cdf/KO/ko15.CDF   NA   NA
#' # C:/R/R-3.6.0/library/faahKO/cdf/KO/ko16.CDF   NA   NA
#' # C:/R/R-3.6.0/library/faahKO/cdf/KO/ko18.CDF   NA   NA
#' }
setMethod("annotationTable", "peakPantheRAnnotation",
    function(object, column) {
    
    ## Expect the object to be valid, therefore peakTables is a list of NULL or
    ## data.frame
    nbCpd <- length(object@cpdID)
    nbSample <- length(object@filepath)
    
    ## Exit with empty data.frame if no samples or only NULL
    if (nbSample < 1) {
        # an empty data.frame
        tmpAnnotation <- data.frame(matrix(vector(), nbSample, nbCpd),
                                    stringsAsFactors = FALSE)
        rownames(tmpAnnotation) <- object@filepath
        colnames(tmpAnnotation) <- object@cpdID
        return(tmpAnnotation)
    }
    if (is.null(object@peakTables[[1]])) {
        # an empty data.frame
        tmpAnnotation <- data.frame(matrix(vector(), nbSample, nbCpd),
                                    stringsAsFactors = FALSE)
        rownames(tmpAnnotation) <- object@filepath
        colnames(tmpAnnotation) <- object@cpdID
        return(tmpAnnotation)
    }
    
    ## Check the column exist
    if (!(column %in% colnames(object@peakTables[[1]]))) {
        stop("input column is not a column of peakTables")
    }
    
    ## Concatenate all the results in a single data.frame
    tmpAnnotation <- vector("list", nbSample)
    for (fl in seq_len(nbSample)) {
        tmpAnnotation[fl] <- object@peakTables[[fl]][column]
    }
    tmpAnnotation <- data.frame(t(do.call("cbind", tmpAnnotation)))
    rownames(tmpAnnotation) <- object@filepath
    colnames(tmpAnnotation) <- object@cpdID
    return(tmpAnnotation)
})

# EICs
setGeneric("EICs",
            function(object, aggregationFunction = "sum", ...)
            standardGeneric("EICs"))
#' EICs accessor
#' @param object peakPantheRAnnotation
#' @param aggregationFunction (str) Function to use in order to aggregate
#' intensities across mz in each scan. One of \code{sum}, \code{max},
#' \code{min}, \code{mean}
#' @return (float) Extracted Ion Chromatogram aggregated across mz in each scan
#' @docType methods
#' @aliases EICs
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' ## default values without annotation
#' EICs(annotation)
#' # [[1]]
#' # list()
#' # [[2]]
#' # list()
#' # [[3]]
#' # list()
#' }
setMethod("EICs", "peakPantheRAnnotation",
            function(object, aggregationFunction) {
    tmpEICs <- lapply(object@dataPoints, function(ROIsDataPoint) {
        lapply(ROIsDataPoint, function(x) {
            generateIonChromatogram(x,aggregationFunction = aggregationFunction)
        })
    })
    return(tmpEICs)
})

# filename
setGeneric("filename", function(object, ...) standardGeneric("filename"))
#' filename accessor by spliting filepath
#' @param object peakPantheRAnnotation
#' @return (str) filename
#' @docType methods
#' @aliases filename
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                     targetFeatTable=targetFeatTable)
#'
#' filename(annotation)
#' # [1] 'ko15' 'ko16' 'ko18'
#' }
setMethod("filename", "peakPantheRAnnotation", function(object) {
    return(tools::file_path_sans_ext(basename(object@filepath)))
})




#####################################################################
# Sub-setting object
#####################################################################
#' extract parts of peakPantheRAnnotation class
#' @param x object from which to extract element(s) or in which to replace
#' element(s).
#' @param i (sample) indices specifying elements to extract or replace
#' @param j (compound) indices specifying elements to extract or replace
#' @param drop not applicable
#' @return (peakPantheRAnnotation) object subset
#'
#' @aliases [,peakPantheRAnnotation-method
#' @docType methods
#'
setMethod("[", "peakPantheRAnnotation",
    function(x, i, j, drop = "missing") {
    ## i is row, samples j is col, compounds
    
    # check inputs and fallback
    if (missing(i) & missing(j)) { return(x) }
    if (missing(i)) { i <- seq_len(nbSamples(x)) }
    if (missing(j)) { j <- seq_len(nbCompounds(x)) }
    # check dim size
    if (max(i) > nbSamples(x)) {
        stop("i index out of bound: maximum ", nbSamples(x)) }
    if (max(j) > nbCompounds(x)) {
        stop("j index out of bound: maximum ", nbCompounds(x)) }
    
    # sub-setting
    .cpdID <- x@cpdID[j]
    .cpdName <- x@cpdName[j]
    .ROI <- x@ROI[j, , drop = FALSE]
    .FIR <- x@FIR[j, , drop = FALSE]
    .uROI <- x@uROI[j, , drop = FALSE]
    .filepath <- x@filepath[i]
    .cpdMetadata <- x@cpdMetadata[j, , drop = FALSE]
    .spectraMetadata <- x@spectraMetadata[i, , drop = FALSE]
    .acquisitionTime <- x@acquisitionTime[i]
    .uROIExist <- x@uROIExist
    .useUROI <- x@useUROI
    .useFIR <- x@useFIR
    .TIC <- x@TIC[i]
    .isAnnotated <- x@isAnnotated
    
    # peakTables, dataPoints & peakFit
    subRes <- subsetting_peakTables_dataPoints_peakFit(x, i, j)
    .peakTables <- subRes$tmp_peakTables
    .dataPoints <- subRes$tmp_dataPoints
    .peakFit <- subRes$tmp_peakFit

    # load value in new object that will need to pass validObject()
    peakPantheRAnnotation(cpdID = .cpdID, cpdName = .cpdName, ROI = .ROI,
        FIR = .FIR, uROI = .uROI, filepath = .filepath,
        cpdMetadata = .cpdMetadata, spectraMetadata = .spectraMetadata,
        acquisitionTime = .acquisitionTime, uROIExist = .uROIExist,
        useUROI = .useUROI, useFIR = .useFIR, TIC = .TIC,
        peakTables = .peakTables, dataPoints = .dataPoints, peakFit = .peakFit,
        isAnnotated = .isAnnotated)
})
# sub-setting peakTables, dataPoints and peakFit
subsetting_peakTables_dataPoints_peakFit <- function(x, i, j) {

    # peakTables, filter samples first, then compounds in each table
    tmp_peakTables <- x@peakTables[i]
    if (all(vapply(tmp_peakTables, is.null, FUN.VALUE = logical(1)))) {
        # no cpd filter if all NULL
        .peakTables <- tmp_peakTables
    } else {
        # cpd filter in each table
        .peakTables <- lapply(tmp_peakTables, function(x, y) {
            x[y, ]
        }, y = j)
    }

    # dataPoints, filter samples first, then compounds in each ROIsDataPoint
    tmp_dataPoints <- x@dataPoints[i]
    if (all(vapply(tmp_dataPoints, is.null, FUN.VALUE = logical(1)))) {
        # no cpd filter if all NULL
        .dataPoints <- tmp_dataPoints
    } else {
        # cpd filter in each ROIsDataPoint
        .dataPoints <- lapply(tmp_dataPoints, function(x, y) {
            x[y]
        }, y = j)
    }

    # peakFit, filter samples first, then compounds in each curveFit list
    tmp_peakFit <- x@peakFit[i]
    if (all(vapply(tmp_peakFit, is.null, FUN.VALUE = logical(1)))) {
        # no cpd filter if all NULL
        .peakFit <- tmp_peakFit
    } else {
        # cpd filter in each curveFit list
        .peakFit <- lapply(tmp_peakFit, function(x, y) {
            x[y]
        }, y = j)
    }

    return(list(tmp_peakTables = .peakTables, tmp_dataPoints = .dataPoints,
                tmp_peakFit = .peakFit))
}



#####################################################################
## Fit diagnosis
#####################################################################

## Set uROI and FIR based on annotation results
setGeneric("annotationParamsDiagnostic",
            function(object, verbose = TRUE, ...)
            standardGeneric("annotationParamsDiagnostic"))
#' @title Set uROI and FIR based on annotation results
#' @description Set updated ROI (uROI) and Fallback Integration Regions (FIR)
#' based on the annotation results. If the object is not annotated, it is
#' returned untouched. ROI is not modified. If uROI exist they are left
#' untouched, otherwise they are set as the minimum and maximum found peaks
#' limits (+/-5\% of ROI in retention time). If FIR are used they are left
#' untouched, otherwise they are set as the median of the found limits (rtMin,
#' rtMax, mzMin, mzMax).
#' @param object (peakPantheRAnnotation) Annotated peakPantheRAnnotation object
#' @param verbose (bool) If TRUE message progress of uROI and FIR calculation
#' @return (peakPantheRAnnotation) object with updated ROI and FIR set from
#' annotation results
#' @docType methods
#' @aliases annotationParamsDiagnostic
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' emptyAnnotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                         targetFeatTable=targetFeatTable)
#'
#' annotationParamsDiagnostic(emptyAnnotation, verbose=TRUE)
#' # Warning: the object has not been annotated, return the object untouched
#' # An object of class peakPantheRAnnotation
#' #  2 compounds in 3 samples.
#' #   updated ROI do not exist (uROI)
#' #   does not use updated ROI (uROI)
#' #   does not use fallback integration regions (FIR)
#' #   is not annotated
#' }
setMethod("annotationParamsDiagnostic", "peakPantheRAnnotation",
    function(object, verbose) {
    # init
    outAnnotation <- object

    # not annotated, pass
    if (!outAnnotation@isAnnotated) {
        if (verbose) {
            message('War','ning: the object has not been annotated, return',
                            ' the object untouched') }
        return(outAnnotation) }
    
    # uROI uROI doesn't exist, set uROI with min/max of found peaks (if NA, use
    # ROI value)
    if (!outAnnotation@uROIExist) {
        outAnnotation <- annotationParamsDiagnostic_uROI(outAnnotation, verbose)
    } else { # uROI exist (even not used), no replacement
        if (verbose) { message("uROI already exist, will not be changed") }
    }
    
    # FIR FIR not used, recalculate (if NA, use uROI value that was set
    # previously)
    if (!outAnnotation@useFIR) {
        outAnnotation <- annotationParamsDiagnostic_FIR(outAnnotation, verbose)
    } else { # FIR used, do not recalculate
        if (verbose) { message("FIR in use, will not be changed") }
    }

    return(outAnnotation)
})
# annotationParamsDiagnostic for uROI
annotationParamsDiagnostic_uROI <- function(outAnnotation, verbose) {
    if (verbose) { message('uROI will be set as mimimum/maximum of',
                        ' found peaks (+/-5% of ROI in retention time)') }
    # rt ROI 5% (no NA in ROI rtMin/rtMax)
    rtMargin <- (ROI(outAnnotation)$rtMax - ROI(outAnnotation)$rtMin) * 0.05
    # rtMin (min found peak -5% of ROI)
    rtMinUROI <- unname(vapply(annotationTable(outAnnotation,"rtMin"), min,
        na.rm = TRUE, FUN.VALUE = numeric(1)))
    rtMinUROI <- rtMinUROI - rtMargin
    if (sum(is.infinite(rtMinUROI)) != 0) {
        if (verbose) { message('uROI min rtMin which are NA are',
                            ' replaced with ROI rtMin') }
        rtMinUROI[is.infinite(rtMinUROI)] <-
            outAnnotation@ROI[is.infinite(rtMinUROI), "rtMin"] }
    # rtMax (max found peak +5% of ROI)
    rtMaxUROI <- unname(vapply(annotationTable(outAnnotation,"rtMax"), max,
        na.rm = TRUE, FUN.VALUE = numeric(1)))
    rtMaxUROI <- rtMaxUROI + rtMargin
    if (sum(is.infinite(rtMaxUROI)) != 0) {
        if (verbose) { message('uROI max rtMax which are NA are',
                            ' replaced with ROI rtMax') }
        rtMaxUROI[is.infinite(rtMaxUROI)] <-
            outAnnotation@ROI[is.infinite(rtMaxUROI), "rtMax"] }
    # mzMin
    mzMinUROI <- unname(vapply(annotationTable(outAnnotation,"mzMin"), min,
        na.rm = TRUE, FUN.VALUE = numeric(1)))
    if (sum(is.infinite(mzMinUROI)) != 0) {
        if (verbose) {
            message("uROI min mzMin which are NA are replaced ROI mzMin") }
        mzMinUROI[is.infinite(mzMinUROI)] <-
            outAnnotation@ROI[is.infinite(mzMinUROI), "mzMin"] }
    # mzMax
    mzMaxUROI <- unname(vapply(annotationTable(outAnnotation,"mzMax"), max,
        na.rm = TRUE, FUN.VALUE = numeric(1)))
    if (sum(is.infinite(mzMaxUROI)) != 0) {
        if (verbose) {
            message("uROI max mzMax which are NA are replaced ROI mzMax") }
        mzMaxUROI[is.infinite(mzMaxUROI)] <-
            outAnnotation@ROI[is.infinite(mzMaxUROI), "mzMax"] }
    # store new uROI values
    outAnnotation@uROI[, "rtMin"] <- rtMinUROI
    outAnnotation@uROI[, "rtMax"] <- rtMaxUROI
    outAnnotation@uROI[, "mzMin"] <- mzMinUROI
    outAnnotation@uROI[, "mzMax"] <- mzMaxUROI
    outAnnotation@uROI[, c("rt", "mz")] <- outAnnotation@ROI[,c("rt", "mz")]
    # set uROIExist
    outAnnotation@uROIExist <- TRUE

    return(outAnnotation)
}
# annotationParamsDiagnostic for FIR
annotationParamsDiagnostic_FIR <- function(outAnnotation, verbose) {
    if (verbose) { message('FIR will be calculated as the median of',
                        ' found \"rtMin\",\"rtMax\",\"mzMin\",\"mzMax\"') }

    # rtMin
    rtMinFIR <- unname(vapply(annotationTable(outAnnotation, "rtMin"),
        stats::median, na.rm = TRUE, FUN.VALUE = numeric(1)))
    if (sum(is.na(rtMinFIR)) != 0) {
        if (verbose) {
            message('FIR median rtMin which are NA are replaced ',
                            'with uROI rtMin') }
        rtMinFIR[is.na(rtMinFIR)] <-
            outAnnotation@uROI[is.na(rtMinFIR), "rtMin"] }

    # rtMax
    rtMaxFIR <- unname(vapply(annotationTable(outAnnotation, "rtMax"),
        stats::median, na.rm = TRUE, FUN.VALUE = numeric(1)))
    if (sum(is.na(rtMaxFIR)) != 0) {
        if (verbose) {
            message('FIR median rtMax which are NA are replaced ',
                            'with uROI rtMax') }
        rtMaxFIR[is.na(rtMaxFIR)] <-
            outAnnotation@uROI[is.na(rtMaxFIR), "rtMax"] }

    # mzMin
    mzMinFIR <- unname(vapply(annotationTable(outAnnotation, "mzMin"),
        stats::median, na.rm = TRUE, FUN.VALUE = numeric(1)))
    if (sum(is.na(mzMinFIR)) != 0) {
        if (verbose) {
            message("FIR median mzMin which are NA are replaced uROI mzMin") }
        mzMinFIR[is.na(mzMinFIR)] <-
            outAnnotation@uROI[is.na(mzMinFIR), "mzMin"] }

    # mzMax
    mzMaxFIR <- unname(vapply(annotationTable(outAnnotation, "mzMax"),
        stats::median, na.rm = TRUE, FUN.VALUE = numeric(1)))
    if (sum(is.na(mzMaxFIR)) != 0) {
        if (verbose) {
            message("FIR median mzMax which are NA are replaced uROI mzMax") }
        mzMaxFIR[is.na(mzMaxFIR)] <-
            outAnnotation@uROI[is.na(mzMaxFIR), "mzMax"] }

    # store new FIR values
    outAnnotation@FIR[, "rtMin"] <- rtMinFIR
    outAnnotation@FIR[, "rtMax"] <- rtMaxFIR
    outAnnotation@FIR[, "mzMin"] <- mzMinFIR
    outAnnotation@FIR[, "mzMax"] <- mzMaxFIR

    return(outAnnotation)
}


## Save annotation parameters as CSV
setGeneric("outputAnnotationParamsCSV",
            function(object, saveFolder, verbose = TRUE, noSave=FALSE,...)
            standardGeneric("outputAnnotationParamsCSV"))
#' @title Save annotation parameters as CSV
#' @description Save annotation parameters (ROI, uROI and FIR) to disk as a CSV
#' file for editing
#' @param object (peakPantheRAnnotation) Annotated peakPantheRAnnotation object
#' @param verbose (bool) If TRUE message progress
#' @param saveFolder (str) Path of folder where annotationParameters_summary.csv
#' will be saved
#' @param noSave (bool) If TRUE the resulting table will be returned without
#' saving to disk
#' @return None
#' @docType methods
#' @aliases outputAnnotationParamsCSV
#' @export
#' @examples
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' spectraPaths <- c('./path/file1', './path/file2', './path/file3')
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' emptyAnnotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                         targetFeatTable=targetFeatTable)
#'
#' # temporary file
#' savePath        <- tempdir()
#'
#' # statistics of an empty annotation
#' outputAnnotationParamsCSV(emptyAnnotation, saveFolder=savePath, verbose=TRUE)
setMethod("outputAnnotationParamsCSV", "peakPantheRAnnotation",
            function(object, saveFolder, verbose, noSave) {
    # create table
    outTable <- data.frame(matrix(, nrow = nbCompounds(object), ncol = 0))
    outTable <- cbind(outTable, cpdID = cpdID(object), cpdName=cpdName(object))
    # ROI
    tmp_ROI <- ROI(object)[, c("rt", "mz", "rtMin", "rtMax", "mzMin", "mzMax")]
    colnames(tmp_ROI) <- c("ROI_rt", "ROI_mz", "ROI_rtMin", "ROI_rtMax",
                            "ROI_mzMin", "ROI_mzMax")
    outTable <- cbind(outTable, X = rep("|", nbCompounds(object)), tmp_ROI)
    # uROI
    tmp_uROI <- uROI(object)[,c("rtMin", "rtMax", "mzMin", "mzMax", "rt", "mz")]
    colnames(tmp_uROI) <- c("uROI_rtMin", "uROI_rtMax", "uROI_mzMin",
        "uROI_mzMax", "uROI_rt", "uROI_mz")
    outTable <- cbind(outTable, X = rep("|", nbCompounds(object)), tmp_uROI)
    # FIR
    tmp_FIR <- FIR(object)[, c("rtMin", "rtMax", "mzMin", "mzMax")]
    colnames(tmp_FIR) <- c("FIR_rtMin", "FIR_rtMax", "FIR_mzMin", "FIR_mzMax")
    outTable <- cbind(outTable, X = rep("|", nbCompounds(object)), tmp_FIR)
    
    # save table
    if (!noSave){
    dir.create(saveFolder, recursive = TRUE, showWarnings = FALSE)
    targetFile <- paste(saveFolder, "/annotationParameters_summary.csv",sep="")
    if (verbose) {
        message("Annotation parameters saved at ", targetFile)
    }
    utils::write.csv(outTable, file = targetFile, row.names = FALSE,
                    fileEncoding = "UTF-8")
    } else {
        return(outTable)
    }
})


## Generate fit diagnostic plots
setGeneric("annotationDiagnosticPlots",
            function(object, sampleColour = NULL, sampling = 250,
                    verbose = TRUE, ...)
            standardGeneric("annotationDiagnosticPlots"))
#' @title Generate fit diagnostic plots
#' @description Generate fit diagnostic plots for each ROI: \code{EICFit} the
#' raw data and detected feature fit, \code{rtPeakwidthVert} detected peaks
#' retention time apex and peakwidth (vertical and no run order),
#' \code{rtPeakwidthHorzRunOrder} detected peaks retention time apex and
#' peakwidth by run order, \code{mzPeakwidthHorzRunOrder} detected peaks m/z
#' apex and peakwidth by run order, \code{areaRunOrder} detected peaks area by
#' run order, \code{rtHistogram} histogram of detected peaks retention time,
#' \code{mzHistogram} histogram of detected peaks m/z, \code{areaHistogram}
#' histogram of detected peaks area.
#' @param object (peakPantheRAnnotation) Annotated peakPantheRAnnotation object
#' @param sampleColour (str) NULL or vector colour for each sample
#' @param sampling (int) Number of points to employ when plotting fittedCurve
#' @param verbose (bool) if TRUE message the plot generation progress
#' @return A list (one list per compound) of diagnostic plots:
#' \code{result[[i]]$EICFit}, \code{result[[i]]$rtPeakwidthVert},
#' \code{result[[i]]$rtPeakwidthHorzRunOrder},
#' \code{result[[i]]$mzPeakwidthHorzRunOrder}, \code{result[[i]]$areaRunOrder},
#' \code{result[[i]]$rtHistogram}, \code{result[[i]]$mzHistogram},
#' \code{result[[i]]$areaHistogram}, \code{result[[i]]$title}
#' @docType methods
#' @aliases annotationDiagnosticPlots
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' emptyAnnotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                         targetFeatTable=targetFeatTable)
#'
#' annotationDiagnosticPlots(emptyAnnotation)
#' # Warning: the object has not been annotated, return an empty diagnostic plot
#' # list
#' # [[1]]
#' # NULL
#' # [[2]]
#' # NULL
#' }
setMethod("annotationDiagnosticPlots", "peakPantheRAnnotation",
    function(object, sampleColour, sampling, verbose) {

    # Init
    nbCpd <- nbCompounds(object)
    outList <- vector("list", nbCpd)

    # Check object was annotated
    if (!object@isAnnotated) {
        message('War','ning: the object has not been annotated, return an',
                        ' empty diagnostic plot list')
        return(outList)
    }
    
    # Iterate over compounds
    for (cpd in seq_len(nbCpd)) {
        tmp_annotation <- object[, cpd]
        tmp_plotList <- singleDiagnosticPlots(tmp_annotation, sampleColour,
                                                sampling, verbose)
        # store results
        outList[[cpd]] <- tmp_plotList

        if (verbose) {
            message("Compound ", cpd, "/", nbCpd, " done")
        }
    }
    return(outList)
})
# diagnostic plots for 1 cpd
singleDiagnosticPlots <- function(tmp_annotation,sampleColour,sampling,verbose){
    tmp_plotList <- vector("list", 9)
    names(tmp_plotList) <- c("EICFit", "rtPeakwidthVert",
        "rtPeakwidthHorzRunOrder", "mzPeakwidthHorzRunOrder","areaRunOrder",
        "rtHistogram", "mzHistogram", "areaHistogram", "title")
    tmp_plotList$title <- paste(cpdID(tmp_annotation), "-",              # title
                                cpdName(tmp_annotation))
    tmp_plotList$EICFit <- peakPantheR_plotEICFit(                  # plotEICFit
        ROIDataPointSampleList = unlist(dataPoints(tmp_annotation),
                                        recursive = FALSE),
        curveFitSampleList = unlist(peakFit(tmp_annotation),
                                        recursive = FALSE),
        rtMin = annotationTable(tmp_annotation, "rtMin")[, 1],
        rtMax = annotationTable(tmp_annotation, "rtMax")[, 1],
        sampling = sampling, sampleColour = sampleColour, verbose = verbose)
    tmp_plotList$rtPeakwidthVert <- peakPantheR_plotPeakwidth( # RT Pkwidth vert
        apexValue = annotationTable(tmp_annotation, "rt")[, 1],
        widthMin = annotationTable(tmp_annotation, "rtMin")[, 1],
        widthMax = annotationTable(tmp_annotation, "rtMax")[, 1],
        acquTime = NULL, sampleColour = sampleColour,
        varName = "Retention Time (sec)", rotateAxis=TRUE, verbose=verbose)
    tmp_plotList$rtPeakwidthHorzRunOrder <- peakPantheR_plotPeakwidth(#RTPkw hor
        apexValue = annotationTable(tmp_annotation, "rt")[, 1],
        widthMin = annotationTable(tmp_annotation, "rtMin")[, 1],
        widthMax = annotationTable(tmp_annotation, "rtMax")[, 1],
        acquTime = acquisitionTime(tmp_annotation),
        sampleColour = sampleColour, varName = "Retention Time (sec)",
        rotateAxis = FALSE, verbose = verbose)
    tmp_plotList$mzPeakwidthHorzRunOrder <- peakPantheR_plotPeakwidth(#MZpkw hor
        apexValue = annotationTable(tmp_annotation, "mz")[, 1],
        widthMin = annotationTable(tmp_annotation, "mzMin")[, 1],
        widthMax = annotationTable(tmp_annotation, "mzMax")[, 1],
        acquTime = acquisitionTime(tmp_annotation), sampleColour = sampleColour,
        varName = "m/z", rotateAxis = FALSE, verbose = verbose)
    tmp_plotList$areaRunOrder <- peakPantheR_plotPeakwidth(# peakarea hor runord
        apexValue = annotationTable(tmp_annotation, "peakArea")[, 1],
        widthMin = NULL, widthMax = NULL,
        acquTime = acquisitionTime(tmp_annotation),
        sampleColour = sampleColour, varName = "Peak Area",
        rotateAxis = FALSE, verbose = verbose)
    tmp_plotList$rtHistogram <- plotHistogram(                # RT plotHistogram
        var = annotationTable(tmp_annotation, "rt")[, 1],
        varName = "Retention Time (sec)", density = TRUE)
    tmp_plotList$mzHistogram <- plotHistogram(                # mz plotHistogram
        var = annotationTable(tmp_annotation, "mz")[, 1],
        varName = "m/z", density = TRUE)
    tmp_plotList$areaHistogram <- plotHistogram(       # peak area plotHistogram
        var = annotationTable(tmp_annotation, "peakArea")[, 1],
        varName = "Peak Area", density = TRUE)
    return(tmp_plotList) }


## Save to disk the annotation parameters as CSV and a diagnostic plot per
## fitted compound
setGeneric("outputAnnotationDiagnostic",
    function(object, saveFolder, savePlots = TRUE, sampleColour = NULL,
            verbose = TRUE, ncores = 0, svgPlot = FALSE, ...)
    standardGeneric("outputAnnotationDiagnostic"))
#' @title Save to disk the annotation parameters as CSV and a diagnostic plot
#' per fitted compound
#' @description Save to disk the annotation parameters as CSV (as generated by
#' \code{outputAnnotationParamsCSV()}) and a diagnostic plot per fitted compound
#' (as generated by \code{annotationDiagnosticMultiplot()}) if \code{savePlots}
#' is TRUE
#' @param object (peakPantheRAnnotation) Annotated peakPantheRAnnotation object
#' @param saveFolder (str) Path of folder where annotationParameters_summary.csv
#' and plots will be saved
#' @param savePlots (bool) If TRUE save a diagnostic plot for each compound
#' @param sampleColour (str) NULL or vector colour for each sample
#' @param verbose (bool) If TRUE message progress
#' @param ncores (int) Number of cores to use to save plots in parallel
#' @param svgPlot (bool) If TRUE save plots as 'svg', otherwise as 'png'
#' @param ... Additional parameters for plotting i.e. \code{sampling} for the
#' number of points to employ when plotting fittedCurve
#' @return None
#' @docType methods
#' @aliases outputAnnotationDiagnostic
#' @export
#' @import svglite
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' emptyAnnotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                         targetFeatTable=targetFeatTable)
#'
#' # Calculate annotation
#' annotation <- peakPantheR_parallelAnnotation(emptyAnnotation, ncores=0,
#'                                 getAcquTime=FALSE, verbose=FALSE)$annotation
#'
#' # temporary location
#' savePath1       <- tempdir()
#' outputAnnotationDiagnostic(annotation, saveFolder=savePath1, savePlots=FALSE,
#'                             verbose=TRUE)
#' }
setMethod("outputAnnotationDiagnostic", "peakPantheRAnnotation",
    function(object, saveFolder, savePlots, sampleColour, verbose, ncores,
            svgPlot,...) {
    # Save standardised csv
    outputAnnotationParamsCSV(object, saveFolder = saveFolder,verbose = verbose)
    
    # Save all fit diagnostic
    if (savePlots) {
        # iterate over compound (more progressive plot generation and save than
        # generating all plots at once)
        nbCpd <- nbCompounds(object)
        
        # run in parallel
        if (ncores > 0) {
            if (verbose) {
                message("Saving ", nbCpd, " diagnostic plots in ", saveFolder) }
            
            # Open parallel interface
            cl <- parallel::makeCluster(ncores)
            doParallel::registerDoParallel(cl)
            # Run
            savedPlots <- foreach::foreach(x = seq_len(nbCpd),
                                            .inorder = TRUE) %dopar%
                outputAnnotationDiagnostic_saveSingleMultiPlot(cpdNb = x,
                    annotation = object, saveFolder = saveFolder,
                    sampleColour = sampleColour, nbCpd = nbCpd,
                    verbose = verbose, svgPlot = svgPlot, ...)
            # Close
            parallel::stopCluster(cl)
            if (verbose) { message("All plots saved") }

        # run serial
        } else {
            if (verbose) { message("Saving diagnostic plots:") }
            for (cpd in seq_len(nbCpd)) {
                outputAnnotationDiagnostic_saveSingleMultiPlot(cpdNb = cpd,
                    annotation = object, saveFolder = saveFolder,
                    sampleColour = sampleColour, nbCpd = nbCpd,
                    verbose = verbose, svgPlot = svgPlot, ...)
            }
        }
    }
})
# outputAnnotationDiagnostic
outputAnnotationDiagnostic_saveSingleMultiPlot <- function(cpdNb, annotation,
                    saveFolder, sampleColour, nbCpd, verbose, svgPlot, ...) {
    # @param cpdNb (int) cpd number betweem 1 and nbCompounds()
    # @param annotation (peakPantheRAnnotation) Annotation object
    # @param saveFolder (str) Path where plots will be saved
    # @param sampleColour (str) NULL or vector colour for each sample
    # @param verbose (bool) message progress
    # @param svgPlot (bool) If TRUE save plots as 'svg', otherwise as 'png'
    # @param ... Additional parameters for plotting

    # subset annotation to only 1 cpd
    tmp_annotation <- annotation[, cpdNb]

    # diagnostic plots
    tmp_diagPlotList <- annotationDiagnosticPlots(tmp_annotation,
        sampleColour = sampleColour, verbose = FALSE, ...)

    # multiplot
    suppressMessages(suppressWarnings(
        tmp_multiPlot <- annotationDiagnosticMultiplot(tmp_diagPlotList)))

    # save
    if (svgPlot) { ext_format <- "svg" } else { ext_format <- "png" }
    if (length(tmp_multiPlot) != 0) {
        # A4 page size
        tmp_targetFile <- paste("cpd_", cpdNb, ".", ext_format, sep = "")
        ggplot2::ggsave(file = tmp_targetFile, plot = tmp_multiPlot[[1]],
            device = ext_format, path = saveFolder, dpi = 100, width = 21,
            height = 29.7, units = "cm", limitsize = FALSE)
        grDevices::dev.off()
        # output path
        if (verbose) {
            message("  Compound ", cpdNb, "/", nbCpd,
                    " diagnostic plot saved at ",
                    saveFolder, "/", tmp_targetFile) }

    # no plot to save
    } else {
        if (verbose) {
            message("  No plot to save for compound ", cpdNb, "/", nbCpd) }
    }
    return(NA)
}



## Save to disk all annotation results
setGeneric("outputAnnotationResult",
    function(object, saveFolder, annotationName = "annotationResult",
            verbose = TRUE)
    standardGeneric("outputAnnotationResult"))
#' @title Save to disk all annotation results as csv files
#' @description Save to disk all annotation results as
#' \code{annotationName_ ... .csv} files: compound metadata (\code{cpdMetadata},
#' \code{cpdID}, \code{cpdName}) and spectra metadata (\code{spectraMetadata},
#' \code{acquisitionTime}, \code{TIC}), summary of fit (ratio of peaks found:
#' \code{ratio_peaks_found}, ratio of peaks filled: \code{ratio_peaks_filled},
#' mean ppm_error: \code{ppm_error}, mean rt_dev_sec: \code{rt_dev_sec}), and a
#' file for each column of \code{peakTables} (with samples as rows and compounds
#' as columns)
#' @param object (peakPantheRAnnotation) Annotated peakPantheRAnnotation object
#' @param saveFolder (str) Path of folder where the annotation result csv will
#' be saved
#' @param annotationName (str) name of annotation to use in the saved csv
#' @param verbose (bool) If TRUE message progress
#' @return None
#' @docType methods
#' @aliases outputAnnotationResult
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' emptyAnnotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                         targetFeatTable=targetFeatTable)
#'
#' # Calculate annotation
#' annotation      <- peakPantheR_parallelAnnotation(emptyAnnotation, ncores=0,
#'                                 getAcquTime=FALSE, verbose=FALSE)$annotation
#'
#' # temporary location
#' savePath1       <- tempdir()
#' outputAnnotationResult(annotation, saveFolder=savePath1,
#'                         annotationName='testProject', verbose=TRUE)
#' }
setMethod("outputAnnotationResult", "peakPantheRAnnotation",
    function(object, saveFolder, annotationName, verbose) {
    
    # Check object was annotated
    if (!object@isAnnotated) {
        stop("Object has not been annotated, no annotation results to save")
    }
    
    # Init folder
    dir.create(saveFolder, recursive = TRUE, showWarnings = FALSE)
    
    # Save compound metadata
    outputAnnotationResult_saveCpdMetadata(object, saveFolder, annotationName,
                                            verbose)
    
    # Save spectra metadata
    outputAnnotationResult_saveSpectraMetadata(object, saveFolder,
                                                annotationName, verbose)

    # Save peakTables columns
    outputAnnotationResult_savePeaktable(object, saveFolder, annotationName,
                                            verbose)
    
    # Save summary table
    outputAnnotationResult_saveSummary(object, saveFolder, annotationName,
                                        verbose)
})
# save compound metadata
outputAnnotationResult_saveCpdMetadata <- function(object, saveFolder,
                                                    annotationName, verbose) {
    tmp_cpdID <- cpdID(object)
    tmp_cpdName <- cpdName(object)
    tmp_cpdMetadata <- cpdMetadata(object)

    # tmp_rt, tmp_mz
    nbCpd <- nbCompounds(object)
    tmp_rt <- rep(as.numeric(NA), nbCpd)
    tmp_mz <- rep(as.numeric(NA), nbCpd)
    all_rt <- annotationTable(object, column = "rt")
    all_mz <- annotationTable(object, column = "mz")
    to_keep <- annotationTable(object, column = "found") &
        !annotationTable(object, column = "is_filled")

    for (i in seq_len(nbCpd)) {
        tmp_rt[i] <- mean(all_rt[to_keep[, i], i])
        tmp_mz[i] <- mean(all_mz[to_keep[, i], i])
    }

    tmp_outCpdMeta <- data.frame(cpdID = tmp_cpdID, cpdName = tmp_cpdName,
        rt = tmp_rt, mz = tmp_mz)
    colnames(tmp_outCpdMeta) <- c("cpdID", "cpdName", "Retention Time", "m/z")
    tmp_outCpdMeta <- cbind(tmp_outCpdMeta, tmp_cpdMetadata)

    path_cpdMeta <- paste(saveFolder, "/", annotationName,
                        "_cpdMetadata.csv", sep = "")
    utils::write.csv(tmp_outCpdMeta, file = path_cpdMeta, row.names = FALSE,
                    fileEncoding = "UTF-8")

    if (verbose) { message("Compound metadata saved at ", path_cpdMeta) }
}
# save spectra Metadata
outputAnnotationResult_saveSpectraMetadata <- function(object, saveFolder,
                                                    annotationName, verbose) {
    tmp_filepath <- filepath(object)
    tmp_acqTime <- acquisitionTime(object)
    tmp_TIC <- TIC(object)
    tmp_spectraMetadata <- spectraMetadata(object)
    tmp_outSpecMeta <- data.frame(filepath = tmp_filepath,
                            acquisitionTime = tmp_acqTime, TIC = tmp_TIC)
    tmp_outSpecMeta <- cbind(tmp_outSpecMeta, tmp_spectraMetadata)

    path_specMeta <- paste(saveFolder, "/", annotationName,
        "_spectraMetadata.csv", sep = "")
    utils::write.csv(tmp_outSpecMeta, file = path_specMeta, row.names = FALSE,
                    fileEncoding = "UTF-8")

    if (verbose) { message("Spectra metadata saved at ", path_specMeta) }
}
# save peakTables
outputAnnotationResult_savePeaktable <- function(object, saveFolder,
                                                    annotationName, verbose) {
    for (i in colnames(object@peakTables[[1]])) {
        tmp_var <- annotationTable(object = object, column = i)
        path_var <- paste(saveFolder, "/", annotationName, "_", i, ".csv",
                        sep = "")
        utils::write.csv(tmp_var, file = path_var, row.names = TRUE,
                        fileEncoding = "UTF-8")

        if (verbose) {
            message("Peak measurement \"", i, "\" saved at ", path_var)
        }
    }
}
# save summary table
outputAnnotationResult_saveSummary <- function(object, saveFolder,
                                                annotationName, verbose) {
    tmp_summary <- data.frame(matrix(ncol = 0, nrow = nbCompounds(object)),
                            stringsAsFactors = FALSE)
    rownames(tmp_summary) <- paste(cpdID(object), "-", cpdName(object))

    # used FIR (found = not filled, filled from is_filled)
    if (object@useFIR) {
        tmp_summary$ratio_peaks_found <- 1 - (colSums(
            annotationTable(object, column = "is_filled"))/nbSamples(object))
        tmp_summary$ratio_peaks_filled <- (colSums(
            annotationTable(object, column = "is_filled"))/nbSamples(object))

    # didn't use FIR (found from found, None are filled)
    } else {
        tmp_summary$ratio_peaks_found <- colSums(
            annotationTable(object, column = "found"))/nbSamples(object)
        tmp_summary$ratio_peaks_filled <- 0
    }

    tmp_summary$ppm_error <-
        colMeans(annotationTable(object, column = "ppm_error"), na.rm = TRUE)
    tmp_summary$rt_dev_sec <-
        colMeans(annotationTable(object, column = "rt_dev_sec"), na.rm = TRUE)
    path_summary <- paste(saveFolder, "/", annotationName, "_summary.csv",
                            sep = "")
    utils::write.csv(tmp_summary, file = path_summary, row.names = TRUE,
                    fileEncoding = "UTF-8")

    if (verbose) { message("Summary saved at ", path_summary) }
}


#####################################################################
# Reset a peakPantheRAnnotation and alter samples or compounds information

setGeneric("resetAnnotation",
    function(previousAnnotation, spectraPaths = NULL, targetFeatTable = NULL,
        uROI = NULL, FIR = NULL, cpdMetadata = NULL, spectraMetadata = NULL,
        uROIExist = NULL, useUROI = NULL, useFIR = NULL, verbose = TRUE, ...)
    standardGeneric("resetAnnotation"))
#' @title Reset a peakPantheRAnnotation and alter samples and compounds
#' information
#' @description Reset a peakPantheRAnnotation (remove results and set
#' \code{isAnnotated=FALSE}). If a different number of samples (
#' \code{spectraPaths}) or compounds (\code{targetFeatTable}) are passed, the
#' object will be initialised to the new size. For input values left as NULL,
#' the slots (\code{filepath} (from \code{spectraPaths}), \code{ROI},
#' \code{cpdID}, \code{cpdName} (from \code{targetFeatTable}), \code{uROI},
#' \code{FIR}, \code{cpdMetadata}, \code{spectraMetadata}, \code{uROIExist},
#' \code{useUROI} and \code{useFIR}) will be filled with values from
#' \code{previousAnnotation}.
#' @param previousAnnotation (peakPantheRAnnotation) object to reset
#' @param spectraPaths NULL or a character vector of spectra file paths, to set
#' samples to process
#' @param targetFeatTable NULL or a \code{\link{data.frame}} of compounds to
#' target as rows and parameters as columns: \code{cpdID} (str), \code{cpdName}
#' (str), \code{rtMin} (float in seconds), \code{rt} (float in seconds, or
#' \emph{NA}), \code{rtMax} (float in seconds), \code{mzMin} (float), \code{mz}
#' (float or \emph{NA}), \code{mzMax} (float). Set compounds to target.
#' @param uROI NULL or a data.frame of updated Regions Of Interest (uROI) with
#' compounds as row and uROI parameters as columns: \code{rtMin} (float in
#' seconds), \code{rt} (float in seconds, or \emph{NA}), \code{rtMax} (float in
#' seconds), \code{mzMin} (float), \code{mz} (float or \emph{NA}), \code{mzMax}
#' (float).
#' @param FIR NULL or a data.frame of Fallback Integration Regions (FIR) with
#' compounds as row and FIR parameters as columns: \code{rtMin} (float in
#' seconds), \code{rtMax} (float in seconds), \code{mzMin} (float), \code{mzMax}
#' (float).
#' @param cpdMetadata NULL or a data.frame of compound metadata, with compounds
#' as row and metadata as columns
#' @param spectraMetadata NULL or a data.frame of sample metadata, with samples
#' as row and metadata as columns
#' @param uROIExist NULL or a logical stating if uROI have been set
#' @param useUROI NULL or a logical stating if uROI are to be used
#' @param useFIR NULL or a logical stating if FIR are to be used
#' @param verbose (bool) If TRUE message progress
#' @param ... Additional slots and values to set when resetting the object
#' (\code{cpdID}, \code{cpdName}, \code{ROI}, \code{filepath}, \code{TIC},
#' \code{acquisitionTime}, \code{peakTables}, \code{dataPoints}, \code{peakFit})
#' @return (peakPantheRAnnotation) object reset with previous results removed
#' and slots updated
#' @docType methods
#' @aliases resetAnnotation
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' smallAnnotation  <- peakPantheRAnnotation(spectraPaths=spectraPaths, 
#'                                         targetFeatTable=targetFeatTable)
#' smallAnnotation
#' # An object of class peakPantheRAnnotation
#' #  2 compounds in 2 samples.
#' #  updated ROI do not exist (uROI)
#' #  does not use updated ROI (uROI)
#' #  does not use fallback integration regions (FIR)
#' #  is not annotated
#'
#' # Reset and change number of spectra
#' newSpectraPaths  <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
#' largerAnnotation <- resetAnnotation(smallAnnotation,
#'                                     spectraPaths=newSpectraPaths,
#'                                     verbose=TRUE)
#' largerAnnotation
#' # An object of class peakPantheRAnnotation
#' #  2 compounds in 3 samples.
#' #  updated ROI do not exist (uROI)
#' #  does not use updated ROI (uROI)
#' #  does not use fallback integration regions (FIR)
#' #  is not annotated
#' }
setMethod("resetAnnotation", "peakPantheRAnnotation",
    function(previousAnnotation, spectraPaths, targetFeatTable, uROI, FIR,
        cpdMetadata, spectraMetadata, uROIExist, useUROI, useFIR, verbose, ...){
    
    # If input is NULL, use previousAnnotation value, else use the passed value
    # If number of compounds or spectra is changed, the previous values
    # (metadata, uROI, FIR) cannot be reused (risk a mismatch of the metadata)
    if (verbose) { message("peakPantheRAnnotation object being reset:") }
    
    # targetFeatTable (cpdID, cpdName, ROI), uROI, FIR, cpdMetadata
    resetTarg <- resetAnnot_targetFeatTable_uROI_FIR_cpdMeta(previousAnnotation,
                            targetFeatTable, uROI, FIR, cpdMetadata, verbose)
    .targetFeatTable <- resetTarg$target
    .uROI <- resetTarg$uROI
    .FIR <- resetTarg$FIR
    .cpdMetadata <- resetTarg$cpd

    # spectraPaths, spectraMetadata
    resSpectra <- resetAnnot_spectraPathMetadata(previousAnnotation,
                                        spectraPaths, spectraMetadata, verbose)
    .spectraPaths <- resSpectra$spectraPaths
    .spectraMetadata <- resSpectra$spectraMetadata

    # uROIExist
    .uROIExist <- resetAnnot_uROIExist(uROIExist, targetFeatTable,
                                        previousAnnotation, verbose)

    # useUROI, useFIR
    resUSE <- resetAnnot_useUROIuseFIR(useUROI, useFIR, targetFeatTable,
                                        previousAnnotation, verbose)
    .useUROI <- resUSE$useUROI
    .useFIR <- resUSE$useFIR

    # is annotated
    .isAnnotated <- FALSE

    # Create new object In all case (old or new value) spectraPaths and
    # targetFeatTable will trigger the resetting of all results
    peakPantheRAnnotation(spectraPaths = .spectraPaths,
        targetFeatTable = .targetFeatTable, uROI = .uROI, FIR = .FIR,
        cpdMetadata = .cpdMetadata, spectraMetadata = .spectraMetadata,
        uROIExist = .uROIExist, useUROI = .useUROI, useFIR = .useFIR,
        isAnnotated = .isAnnotated, ...)
})
# resetAnnotation targetFeatTable (cpdID, cpdName, ROI), uROI, FIR, cpdMetadata
resetAnnot_targetFeatTable_uROI_FIR_cpdMeta <- function(previousAnnotation,
                            targetFeatTable, uROI, FIR, cpdMetadata, verbose) {
    # reuse previous values
    if (all(is.null(targetFeatTable))) {
        reset <- resetAnnot_prevTargetFeatTable(previousAnnotation, uROI, FIR,
                                                cpdMetadata, verbose)
    } else {
        # reset to default
        reset <- resetAnnot_newTargetFeatTable(targetFeatTable, uROI, FIR,
                                                cpdMetadata, verbose)
    }
    return(reset)
}
# resetAnnotation reuse previous targetFeatTable
resetAnnot_prevTargetFeatTable <- function(previousAnnotation, uROI, FIR,
                                            cpdMetadata, verbose) {
    # previous values
    .targetFeatTable <- ROI(previousAnnotation)
    if (verbose) {
        message("  Previous \"ROI\", \"cpdID\" and \"cpdName\" value kept")
    }
    # uROI
    if (all(is.null(uROI))) { # previous values
        .uROI <- uROI(previousAnnotation)[, c("rtMin", "rt", "rtMax",
                                                "mzMin", "mz", "mzMax")]
        if (verbose) {
            message("  Previous \"uROI\" value kept")
        }
    } else { # new value
        .uROI <- uROI
        if (verbose) {
            message("  New \"uROI\" value set")
        }
    }
    # FIR
    if (all(is.null(FIR))) { # previous values
        .FIR <- FIR(previousAnnotation)[, c("rtMin", "rtMax", "mzMin",
                                            "mzMax")]
        if (verbose) {
            message("  Previous \"FIR\" value kept")
        }
    } else { # new value
        .FIR <- FIR
        if (verbose) {
            message("  New \"FIR\" value set")
        }
    }
    # cpdMetadata
    if (all(is.null(cpdMetadata))) { # previous values
        .cpdMetadata <- cpdMetadata(previousAnnotation)
        if (verbose) {
            message("  Previous \"cpdMetadata\" value kept")
        }
    } else { # new value
        .cpdMetadata <- cpdMetadata
        if (verbose) {
            message("  New \"cpdMetadata\" value set")
        }
    }
    return(list(target=.targetFeatTable, uROI=.uROI, FIR=.FIR,
                cpd=.cpdMetadata))
}
# resetAnnotation new targetFeatTable
resetAnnot_newTargetFeatTable <- function(targetFeatTable, uROI, FIR,
                                            cpdMetadata, verbose) {
    # reset to default
    .targetFeatTable <- targetFeatTable
    if (verbose) {
        message('  New \"targetFeatTable\" value set (\"ROI\", ',
                        '\"cpdID\", \"cpdName\"') }
    # uROI
    if (all(is.null(uROI))) {
        # Do not reuse old values if we change the compounds targeted
        .uROI <- data.frame(rtMin = numeric(), rt = numeric(),
            rtMax = numeric(), mzMin = numeric(), mz = numeric(),
            mzMax = numeric(), stringsAsFactors = FALSE)
        if (verbose) {
            message('  Targeted compounds changed, previous ',
                            '\"uROI\" cannot be kept and set to default') }
    } else { # new value
        .uROI <- uROI
        if (verbose) { message("  New \"uROI\" value set") }
    }
    # FIR
    if (all(is.null(FIR))) {
        # Do not reuse old values if we change the compounds targeted
        .FIR <- data.frame(rtMin = numeric(), rtMax = numeric(),
            mzMin = numeric(), mzMax = numeric(), stringsAsFactors = FALSE)
        if (verbose) {
            message('  Targeted compounds changed, previous \"FIR\"',
                            ' cannot be kept and set to default')
        }
    } else { # new value
        .FIR <- FIR
        if (verbose) { message("  New \"FIR\" value set") }
    }
    # cpdMetadata
    if (all(is.null(cpdMetadata))) {
        # Do not reuse old values if we change the compounds targeted
        .cpdMetadata <- data.frame()
        if (verbose) {
            message('  Targeted compounds changed, previous ',
                    '\"cpdMetadata\" cannot be kept and set to default')
        }
    } else { # new value
        .cpdMetadata <- cpdMetadata
        if (verbose) { message("  New \"cpdMetadata\" value set") }
    }
    return(list(target=.targetFeatTable, uROI=.uROI, FIR=.FIR,
                cpd=.cpdMetadata))
}
# resetAnnotation spectraPaths, spectraMetadata
resetAnnot_spectraPathMetadata <- function(previousAnnotation, spectraPaths,
                                            spectraMetadata, verbose) {
    # reuse previous values
    if (all(is.null(spectraPaths))) {
        .spectraPaths <- filepath(previousAnnotation)
        if (verbose) {
            message("  Previous \"filepath\" value kept")
        }

        # spectraMetadata
        if (all(is.null(spectraMetadata))) {
            # previous values
            .spectraMetadata <- spectraMetadata(previousAnnotation)
            if (verbose) {
                message("  Previous \"spectraMetadata\" value kept")
            }
        } else {
            # new value
            .spectraMetadata <- spectraMetadata
            if (verbose) {
                message("  New \"spectraMetadata\" value set")
            }
        }

        # new values
    } else {
        .spectraPaths <- spectraPaths
        if (verbose) {
            message("  New \"spectraPaths\" value set")
        }

        # spectraMetadata
        if (all(is.null(spectraMetadata))) {
            # Do not reuse old values if we change the spectra
            .spectraMetadata <- data.frame()
            if (verbose) {
                message('  Targeted spectra changed, previous ',
                    '\"spectraMetadata\" cannot be kept and set to default')
            }
        } else {
            # new value
            .spectraMetadata <- spectraMetadata
            if (verbose) {
                message("  New \"spectraMetadata\" value set")
            }
        }
    }
    return(list(spectraPaths = .spectraPaths,
                spectraMetadata = .spectraMetadata))
}
# resetAnnotation uROIExist
resetAnnot_uROIExist <- function(uROIExist, targetFeatTable, previousAnnotation,
                                verbose) {
    if (all(is.null(uROIExist))) {
        # previous values cannot be used if targetFeatTable is changed
        if (all(is.null(targetFeatTable))) {
            # previous values
            .uROIExist <- uROIExist(previousAnnotation)
            if (verbose) {
                message("  Previous \"uROIExist\" value kept")
            }
        } else {
            # Do not reuse old values if we change the compounds
            .uROIExist <- FALSE
            if (verbose) {
                message('  Targeted compounds changed, previous ',
                            '\"uROIExist\" cannot be kept and set to default')
            }
        }
    } else {
        # new value
        .uROIExist <- uROIExist
        if (verbose) {
            message("  New \"uROIExist\" value set")
        }
    }
    return(.uROIExist)
}
# resetAnnotation useUROI, useFIR
resetAnnot_useUROIuseFIR <- function(useUROI, useFIR, targetFeatTable,
                                    previousAnnotation, verbose){
    # useUROI
    if (all(is.null(useUROI))) {
        # previous values cannot be used if targetFeatTable is changed
        if (all(is.null(targetFeatTable))) {
            # previous values
            .useUROI <- useUROI(previousAnnotation)
            if (verbose) {
                message("  Previous \"useUROI\" value kept")
            }
        } else {
            # Do not reuse old values if we change the compound
            .useUROI <- FALSE
            if (verbose) {
                message('  Targeted compounds changed, previous ',
                            '\"useUROI\" cannot be kept and set to default')
            }
        }
    } else { # new value
        .useUROI <- useUROI
        if (verbose) { message("  New \"useUROI\" value set") }
    }

    # useFIR
    if (all(is.null(useFIR))) {
        # previous values cannot be used if targetFeatTable is changed
        if (all(is.null(targetFeatTable))) {
            # previous values
            .useFIR <- useFIR(previousAnnotation)
            if (verbose) {
                message("  Previous \"useFIR\" value kept")
            }
        } else { # Do not reuse old values if we change the compound
            .useFIR <- FALSE
            if (verbose) {
                message('  Targeted compounds changed, previous ',
                                '\"useFIR\" cannot be kept and set to default')
            }
        }
    } else { # new value
        .useFIR <- useFIR
        if (verbose) { message("  New \"useFIR\" value set") }
    }
    return(list(useUROI = .useUROI, useFIR = .useFIR))
}


## Reset FIR windows to uROI or ROI values
setGeneric("resetFIR",
    function(object, verbose = TRUE, ...)
    standardGeneric("resetFIR"))
#' Reset FIR windows to uROI or ROI values
#' Reset FIR windows to uROI (or ROI if \code{uROIExist=FALSE})
#' @param object (peakPantheRAnnotation) object for which FIR are to be reset
#' @param verbose (bool) If TRUE message progress
#' @return (peakPantheRAnnotation) object with FIR values reset
#' @docType methods
#' @aliases resetFIR
#' @export
#' @examples
#' ## Initialise a peakPantheRAnnotation object with 2 targeted compounds
#'
#' ## targetFeatTable
#' input_targetFeatTable <- data.frame(matrix(vector(), 2, 8,
#'                             dimnames=list(c(), c('cpdID', 'cpdName', 'rtMin',
#'                             'rt', 'rtMax', 'mzMin', 'mz', 'mzMax'))),
#'                             stringsAsFactors=FALSE)
#' input_targetFeatTable[1,] <- c('ID-1', 'Cpd 1',  3.,  1.,  4.,  5.,  2.,  6.)
#' input_targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 19., 17., 20., 21., 18., 22.)
#' input_targetFeatTable[,c(3:8)] <- sapply(input_targetFeatTable[,c(3:8)],
#'                                         as.numeric)
#' ## FIR
#' input_FIR       <- data.frame(matrix(vector(), 2, 4, dimnames=list(c(), 
#'                                 c('rtMin', 'rtMax', 'mzMin', 'mzMax'))),
#'                                 stringsAsFactors=FALSE)
#' input_FIR[1,]   <- c(13., 14., 15., 16.)
#' input_FIR[2,]   <- c(29., 30., 31., 32.)
#'
#' annotation <- peakPantheRAnnotation(targetFeatTable = input_targetFeatTable,
#'                                     FIR = input_FIR, uROIExist = FALSE)
#'
#' ## Reset FIR with ROI values as uROI are not set
#' updatedAnnotation <- resetFIR(annotation, verbose=TRUE)
#' # FIR will be reset with ROI values as uROI values are not set
setMethod("resetFIR", "peakPantheRAnnotation",
    function(object, verbose) {
    
    # uROI exist
    if (uROIExist(object)) {
        if (verbose) {
            message("FIR will be reset with uROI values")
        }
        newFIR <- uROI(object)[, c("rtMin", "rtMax", "mzMin", "mzMax")]
        
        # uROI not defined
    } else {
        if (verbose) {
            message('FIR will be reset with ROI values as uROI values ',
                            'are not set')
        }
        newFIR <- ROI(object)[, c("rtMin", "rtMax", "mzMin", "mzMax")]
    }
    
    object@FIR[, c("rtMin", "rtMax", "mzMin", "mzMax")] <-
        newFIR[, c("rtMin", "rtMax", "mzMin", "mzMax")]
    return(object)
})

#####################################################################
# Retention time correction: adjust uROI rt based on previously integrated
# signal
setGeneric("retentionTimeCorrection",
    function(annotationObject, rtCorrectionReferences=NULL,
    method='polynomial', params=list(polynomialOrder=2),
    robust=FALSE, rtWindowWidth=15,  diagnostic=TRUE)
    standardGeneric("retentionTimeCorrection"))
#' @title Apply retention time correction methods to adjust the retention time
#' information in the uROI of peakPantheRAnnotation object
#' @description Performs retention time correction to re-adjust the expected
#' retention time position of compounds.
#' Requires an annotated peakPantheRAnnotation object (\code{isAnnotated=TRUE}).
#' The original \code{rt} value is used as expected and the observed deviation
#' measured in the \code{rt_dev_sec} field is taken as the deviation to be
#' corrected.
#' @param annotationObject (peakPantheRAnnotation) object with previous fit
#' results to adjust retention time values in uROI and FIR
#' annotationObject, rtCorrectionReferences=NULL,
#' @param rtCorrectionReferences (list) of compounds IDs (\code{cpdID})
#' to be used as retention time references.
#' All \code{cpdID} entries must be present in the object and previously
#' annotated. If NULL, use all compounds.
#' @param method (str) name of RT correction method to use (currently
#' \code{polynomial} or \code{constant}
#' @param params (list) list of parameters to pass to each correction method.
#' Currently allowed inputs are \code{polynomialOrder}
#' for \code{method='polynomial'}
#' @param robust (bool) whether to use the RANSAC algorithm to flag and ignore
#' outliers during retention time correction
#' @param rtWindowWidth (numeric) full width in seconds
#' of the retention time window defined around the corrected retention time
#' value for each compound
#' @param diagnostic (bool) If TRUE returns diagnostic plots (specific to each
#' correction method)
#' @return (list) containing entries `annotation`, with the new and retention
#' time corrected peakPantheRAnnotation, and `plot` (if \code{diagnostic=TRUE}).
#' @docType methods
#' @aliases retentionTimeCorrection
#' @export
#' @examples
#' if(requireNamespace('faahKO')){
#' ## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
#' ## compounds
#'
#' # Paths to spectra files
#' library(faahKO)
#' spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
#'                     system.file('cdf/KO/ko16.CDF', package = 'faahKO'))
#'
#' # targetFeatTable
#' targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#'                     c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
#'                     'mzMax'))), stringsAsFactors=FALSE)
#' targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
#'                         522.2, 522.205222)
#' targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
#'                         496.2, 496.204962)
#' targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
#'                                     FUN.VALUE=numeric(2))
#'
#' smallAnnotation  <- peakPantheRAnnotation(spectraPaths=spectraPaths,
#'                                         targetFeatTable=targetFeatTable)
#' # annotate files serially
#' annotation_result <- peakPantheR_parallelAnnotation(smallAnnotation,
#'                                          ncores=0, verbose=TRUE)
#' data_annotation   <- annotation_result$annotation
#'
#' # Example with constant correction
#' rtCorrectionOutput <- retentionTimeCorrection(
#'                            annotationObject = data_annotation,
#'                            rtCorrectionReferences=c('ID-1'),
#'                            method='constant', params=list(),
#'                            robust=FALSE,
#'                            rtWindowWidth=15,
#'                            diagnostic=TRUE)
#'
#' rtCorrectedAnnotation <- rtCorrectionOutput$annotation
#'
#' # rtCorrectedAnnotation
#' # An object of class peakPantheRAnnotation
#' #  2 compounds in 2 samples.
#' #  updated ROI exists, with a modified rt (uROI)
#' #  uses updated ROI (uROI)
#' #  uses fallback integration regions (FIR)
#' #  is annotated
#'
#' rtCorrectionPlot <- rtCorrectionOutput$plot
#' # rtCorrectedPlot
#' # A ggplot2 object
#' #  Scatterplot where x=`r` in the and y=`rt_dev_sec` from data_annotation
#' #  Points colored depending on whether the reference was used to fit
#' # the correction model
#' }
setMethod("retentionTimeCorrection", "peakPantheRAnnotation",
    function(annotationObject, rtCorrectionReferences, method, params, robust,
    rtWindowWidth, diagnostic) {

    newAnnotation <- annotationObject
    rtCorrectionReferences <- rtCorrection_checkInput(newAnnotation,
    rtCorrectionReferences,rtWindowWidth)
    featureInformation <-
        rtCorrection_prepareTargetFeatTable(newAnnotation,
        rtCorrectionReferences)

    referenceTable <- featureInformation$referenceTable
    targetFeatTable <- featureInformation$targetFeatTable
    newAnnotation <- rtCorrection_prepareAnnotation(newAnnotation,
        targetFeatTable, rtWindowWidth)
    referenceTable <-  referenceTable[!is.na(referenceTable$rt_dev_sec), ]

    correctedRtTable  <-
        peakPantheR_applyRTCorrection(
            targetFeatTable[!is.na(targetFeatTable$rt_dev_sec), ],
            referenceTable, method,
            params, robust)

    # Only compounds with measured rt_dev_sec can be adjusted
    newAnnotation@uROI[!is.na(targetFeatTable$rt_dev_sec), "rtMin"] <-
        correctedRtTable$correctedRt - (rtWindowWidth/2)
    newAnnotation@uROI[!is.na(targetFeatTable$rt_dev_sec), "rtMax"] <-
        correctedRtTable$correctedRt + (rtWindowWidth/2)
    newAnnotation@uROI[!is.na(targetFeatTable$rt_dev_sec), "rt"] <-
        correctedRtTable$correctedRt
    newAnnotation@FIR[!is.na(targetFeatTable$rt_dev_sec), "rtMin"] <-
        correctedRtTable$correctedRt - (rtWindowWidth/2)
    newAnnotation@FIR[!is.na(targetFeatTable$rt_dev_sec), "rtMax"] <-
        correctedRtTable$correctedRt + (rtWindowWidth/2)

    if (isTRUE(diagnostic)) {
        rtAdjustmentPlot <-
            rtCorrection_diagnosticPlot(correctedRtTable)
        return(list(annotation=newAnnotation, plot=rtAdjustmentPlot))
    } else {return(list(annotation=newAnnotation))}
})

# Helper functions for the retentionTimeCorrection method
rtCorrection_checkInput <- function(annotation, rtCorrectionReferences,
                                    rtWindowWidth) {
    if (isFALSE(annotation@isAnnotated)) {
        stop('The retention time correction functionality requires an ',
        'annotated peakPantheRAnnotation object ',
        '(annotationObject@isAnnotated = TRUE)')
    }
    if (is.null(rtCorrectionReferences)) {
        rtCorrectionReferences <- annotation@cpdID
    } else {
        if (!all(rtCorrectionReferences %in% annotation@cpdID)) {
            stop('All compound IDs in rtCorrectionReferences must be ',
                        'present on the annotationObject')
        }
    }
    if (isFALSE(is.numeric(rtWindowWidth)) | rtWindowWidth <= 0) {
        stop('rtWindowWidth must be a positive number')
    }

    return(rtCorrectionReferences)
}

# Helper functions for the retentionTimeCorrection method
rtCorrection_prepareTargetFeatTable <- function(newAnnotation,
                                            rtCorrectionReferences) {

    # Obtain the mean retention time deviation per second
    rt_dev_secMatrix <- annotationTable(newAnnotation,"rt_dev_sec")
    rt_dev_sec_mean <- unname(vapply(rt_dev_secMatrix, mean, na.rm=TRUE,
    FUN.VALUE = numeric(1)))
    # If uROI exists and useROI was set to true, use that value as original RT,
    # otherwise the "theoretical rt" is taken from ROI
    if (newAnnotation@uROIExist & newAnnotation@useUROI) {
        rt_expected <- newAnnotation@uROI[, "rt"]
    } else { rt_expected <- newAnnotation@ROI[, "rt"] }

    targetFeatTable <- data.frame(cpdID=newAnnotation@cpdID,
            cpdName=newAnnotation@cpdName, rt=rt_expected,
            rt_dev_sec=rt_dev_sec_mean)
    # subset the references from the table of all compounds to correct
    referenceTable <- targetFeatTable[targetFeatTable$cpdID
                                %in% rtCorrectionReferences, ]
    # Exclude features with mean rt_dev_sec = NA from the correction
    # function fitting
    if (any(is.na(referenceTable$rt_dev_sec))) {
        warning("The following references could not be integrated ",
            "previously and will be excluded: ",
            referenceTable[is.na(referenceTable$rt_dev_sec), "cpdID"])
    }
    if (any(is.na(targetFeatTable$rt_dev_sec))) {
        warning("The following compounds could not be ",
            "integrated previously and will be not be corrected: ",
            targetFeatTable[is.na(targetFeatTable$rt_dev_sec), "cpdID"])
    }

    return(list(targetFeatTable=targetFeatTable,
                referenceTable=referenceTable))
}

# Helper functions for the retentionTimeCorrection method
rtCorrection_prepareAnnotation <-function(newAnnotation, targetFeatTable,
                                    rtWindowWidth) {

    if  (isFALSE(newAnnotation@useUROI)) {
        newAnnotation@uROI[is.na(targetFeatTable$rt_dev_sec), "rt"] <-
        newAnnotation@ROI[is.na(targetFeatTable$rt_dev_sec), "rt"]
        newAnnotation@uROI[is.na(targetFeatTable$rt_dev_sec), "rtMin"] <-
        newAnnotation@uROI[is.na(targetFeatTable$rt_dev_sec), "rt"] -
        (rtWindowWidth/2)
        newAnnotation@uROI[is.na(targetFeatTable$rt_dev_sec), "rtMax"] <-
        newAnnotation@uROI[is.na(targetFeatTable$rt_dev_sec), "rt"] +
        (rtWindowWidth/2)
        newAnnotation@FIR[is.na(targetFeatTable$rt_dev_sec), "rtMin"] <-
        newAnnotation@uROI[is.na(targetFeatTable$rt_dev_sec), "rt"] -
        (rtWindowWidth/2)
        newAnnotation@FIR[is.na(targetFeatTable$rt_dev_sec), "rtMax"] <-
        newAnnotation@uROI[is.na(targetFeatTable$rt_dev_sec), "rt"] +
        (rtWindowWidth/2)
    }
        # set uROIExist to TRUE
    # after corrrection useUROI is TRUE as corrected info is updated there
    newAnnotation@uROIExist <- TRUE
    newAnnotation@useUROI <- TRUE
    return(newAnnotation)
}

rtCorrection_diagnosticPlot <- function(correctedRtTable) {
    rtAdjustmentPlot <- ggplot2::ggplot(data=correctedRtTable) +
                        ggplot2::geom_point(ggplot2::aes(x=rt,
                        y=rt_dev_sec, col=isReference)) +
                        ggplot2::geom_line(ggplot2::aes(x=rt,
                        y=predictedRtDrift))

        rtAdjustmentPlot <- rtAdjustmentPlot +
        ggplot2::xlab('Retention time') +
        ggplot2::ylab('Retention time deviation')
    return(rtAdjustmentPlot)
}

## Init a new peakPantheRAnnotation object after loading ROI, uROI and FIR
## parameters from CSV
#' @title Load fit parameters from CSV
#' 
#' @description Initialise a new \code{peakPantheRAnnotation} object after
#' loading ROI, uROI and FIR parameters from CSV. \code{spectraPaths},
#' \code{spectraMetadata} or \code{cpdMetadata} are not initialised and will
#' need to be filled before annotation. \code{useUROI} and \code{useFIR} are set
#' to \code{FALSE} and will need to be set accordingly. \code{uROIExist} is
#' established depending on the uROI columns present in the CSV and will be set
#' to \code{TRUE} only if no \code{NA} are present
#' 
#' @param CSVParamPath (str) Path to a CSV file of fit parameters (e.g. as
#' saved by \code{outputAnnotationDiagnostic})
#' @param verbose (bool) If TRUE message progress
#' 
#' @return (peakPantheRAnnotation) Object initialised with ROI, uROI and FIR
#' read from the CSV file
#' 
#' @export
#'
#' @examples
#' ## Input data
#' input_CSV <- data.frame(matrix(nrow=2,ncol=21,dimnames=list(c(),
#'     c('cpdID', 'cpdName',
#'     'X','ROI_rt', 'ROI_mz', 'ROI_rtMin', 'ROI_rtMax','ROI_mzMin','ROI_mzMax',
#'     'X','uROI_rtMin', 'uROI_rtMax', 'uROI_mzMin', 'uROI_mzMax', 'uROI_rt',
#'     'uROI_mz', 'X', 'FIR_rtMin', 'FIR_rtMax', 'FIR_mzMin', 'FIR_mzMax'))))
#' input_CSV[1,]  <- c('ID-1', 'Cpd 1', '|', 1.,  2.,  3.,  4.,  5.,  6.,  '|',
#'                     7.,  8.,  9.,  10., 11., 12., '|', 13., 14., 15., 16.)
#' input_CSV[2,]  <- c('ID-2', 'Cpd 2', '|', 17., 18., 19., 20., 21., 22., '|',
#'                     23., 24., 25., 26., 27., 28., '|', 29., 30., 31., 32.)
#' input_CSV[,-c(1,2,3,10,17)]  <- vapply(input_CSV[,-c(1,2,3,10,17)],
#'                                             as.numeric, FUN.VALUE=numeric(2))
#'
#' # temporary file location
#' savePath1      <- tempfile(pattern='file', tmpdir=tempdir(), fileext='.csv')
#' # save csv
#' utils::write.csv(input_CSV, file=savePath1, row.names=FALSE)
#'
#' # Load parameters from CSV
#' loadedAnnotation <- peakPantheR_loadAnnotationParamsCSV(savePath1,
#'                                                         verbose=TRUE)
#' # uROIExist set to TRUE
#' # New peakPantheRAnnotation object initialised for 2 compounds
#' # An object of class peakPantheRAnnotation
#' #  2 compounds in 0 samples.
#' #   updated ROI exist (uROI)
#' #   does not use updated ROI (uROI)
#' #   does not use fallback integration regions (FIR)
#' #   is not annotated
peakPantheR_loadAnnotationParamsCSV <- function(CSVParamPath, verbose = TRUE) {
    # Check file exist
    if (!file.exists(CSVParamPath)) {
        stop('specified "CSVParamPath" does not exist')
    }

    # Read file
    tmp_csv   <- read.csv(CSVParamPath, header=TRUE, sep=",", quote="\"",
                            stringsAsFactors=FALSE)

    # Process input (either simple or advanced description)
    if ("rtMin" %in% colnames(tmp_csv)) {
        params  <- prepare_basic_target_parameters(tmp_csv)
    } else {
        params  <- prepare_advanced_target_parameters(tmp_csv, verbose=verbose)
    }

    # Initialise new object
    tmp_annotation <- peakPantheRAnnotation(
                                        targetFeatTable=params$targetFeatTable,
                                        uROI=params$uROI,
                                        FIR=params$FIR,
                                        uROIExist=params$uROIExist)
    if (verbose) { message('New peakPantheRAnnotation object initialised for ',
                            nbCompounds(tmp_annotation), ' compounds')
    }
    return(tmp_annotation)
}

#' Process target region parameters (simple format) for object initialisation
#'
#' Process the simple target region parameters (\code{cpdID, cpdName, mzMin,
#' mzMax, mz, rtMin, rtMax, rt}) and return input variables for
#' \code{peakPantheRAnnotation()}
#'
#' @param paramTable (data.frame) Target region parameters
#'
#' @return (list) List of \code{targetFeatTable, uROI, FIR, uROIExist}
prepare_basic_target_parameters <- function(paramTable){

    # Check columns
    expected_col <- c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin",
                        "mz", "mzMax")
    if (!(all(expected_col %in% colnames(paramTable)))) {
        stop('Columns in "CSVParamPath" must be: "cpdID", "cpdName", "rtMin", ',
            '"rt", "rtMax", "mzMin", "mz", "mzMax"')
    }

    # Prepare ROI, default uROI and FIR
    # ensure order
    tmp_targetFeatTable <- paramTable[, c("cpdID", "cpdName", "rtMin", "rt",
                                            "rtMax", "mzMin", "mz", "mzMax")]
    tmp_targetFeatTable[,c(3:8)] <- vapply(tmp_targetFeatTable[,c(3:8)],
                    as.numeric, FUN.VALUE=numeric(dim(tmp_targetFeatTable)[1]))
    # default values (empty)
    tmp_uROI <- data.frame(rtMin=numeric(), rt=numeric(), rtMax=numeric(),
                            mzMin=numeric(), mz=numeric(), mzMax=numeric(),
                            stringsAsFactors=FALSE)
    tmp_FIR <- data.frame(rtMin=numeric(), rtMax=numeric(), mzMin=numeric(),
                            mzMax=numeric(), stringsAsFactors=FALSE)
    tmp_uROIExist <- FALSE

    # check loaded data
    # ROI check no NA (stop otherwise crash at next step)
    check_NA_in_target_parameters(tmp_targetFeatTable)
    # ROI is in order
    if ( !(all(tmp_targetFeatTable$rtMin <= tmp_targetFeatTable$rtMax) &
            all(tmp_targetFeatTable$mzMin <= tmp_targetFeatTable$mzMax)) ) {
        stop('Check ROI values: "rtMin" < "rtMax" and "mzMin" < "mzMax"')
    }

    return(list(targetFeatTable=tmp_targetFeatTable, uROI=tmp_uROI, FIR=tmp_FIR,
                uROIExist=tmp_uROIExist))
}
# Check if NA in ROI, split function too long
check_NA_in_target_parameters <- function(targetFeatTable) {
    if (any(is.na(targetFeatTable[,c("rtMin","rtMax","mzMin","mzMax")]))) {
        stop(paste0('Check ROI values: "ROI$rtMin", "ROI$rtMax", ',
                    '"ROI$mzMin" and "ROI$mzMax" cannot be NA'))
    }
}

#' Process target region parameters (with uROI, FIR) for object initialisation
#'
#' Process target region parameters with uROI and FIR (\code{cpdID, cpdName,
#' ROI_rt, ROI_mz, ROI_rtMin, ROI_rtMax, ROI_mzMin, ROI_mzMax, uROI_rtMin,
#' uROI_rtMax, uROI_mzMin, uROI_mzMax, uROI_rt, uROI_mz, FIR_rtMin, FIR_rtMax,
#' FIR_mzMin, FIR_mzMax}) and return input variables for
#' \code{peakPantheRAnnotation()}
#'
#' @param paramTable (data.frame) Target region parameters
#' @param verbose (bool) If TRUE message progress
#'
#' @return (list) List of \code{targetFeatTable, uROI, FIR, uROIExist}
prepare_advanced_target_parameters <- function(paramTable, verbose) {
    # Check columns
    expected_col <- c("cpdID", "cpdName", "ROI_rt", "ROI_mz", "ROI_rtMin",
                        "ROI_rtMax", "ROI_mzMin", "ROI_mzMax", "uROI_rtMin",
                        "uROI_rtMax", "uROI_mzMin", "uROI_mzMax", "uROI_rt",
                        "uROI_mz", "FIR_rtMin", "FIR_rtMax", "FIR_mzMin",
                        "FIR_mzMax")
    if (!(all(expected_col %in% colnames(paramTable)))) {
        stop('Columns in "CSVParamPath" must be: "cpdID", "cpdName", "ROI_rt",',
            ' "ROI_mz", "ROI_rtMin", "ROI_rtMax", "ROI_mzMin", "ROI_mzMax", ',
            '"uROI_rtMin", "uROI_rtMax", "uROI_mzMin", "uROI_mzMax", "uROI_rt"',
            ', "uROI_mz", "FIR_rtMin", "FIR_rtMax", "FIR_mzMin", "FIR_mzMax"') }
    # Prepare ROI, uROI and FIR
    tmp_targetFeatTable <- paramTable[,c("cpdID", "cpdName", "ROI_rtMin",
                                        "ROI_rt", "ROI_rtMax", "ROI_mzMin",
                                        "ROI_mz", "ROI_mzMax")]
    colnames(tmp_targetFeatTable) <- c("cpdID", "cpdName", "rtMin", "rt",
                                        "rtMax", "mzMin", "mz", "mzMax")
    tmp_targetFeatTable[,c(3:8)] <- vapply(tmp_targetFeatTable[,c(3:8)],
                    as.numeric, FUN.VALUE=numeric(dim(tmp_targetFeatTable)[1]))
    tmp_uROI <- paramTable[,c("uROI_rtMin", "uROI_rt", "uROI_rtMax",
                                "uROI_mzMin", "uROI_mz", "uROI_mzMax")]
    colnames(tmp_uROI) <- c("rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax")
    tmp_uROI[,c(1,2:6)] <- vapply(tmp_uROI[,c(1,2:6)],
                    as.numeric, FUN.VALUE=numeric(dim(tmp_targetFeatTable)[1]))
    tmp_FIR <- paramTable[,c("FIR_rtMin", "FIR_rtMax", "FIR_mzMin",
                            "FIR_mzMax")]
    colnames(tmp_FIR) <- c("rtMin", "rtMax", "mzMin", "mzMax")
    tmp_FIR[,c(1,2:4)] <- vapply(tmp_FIR[,c(1,2:4)],
                    as.numeric, FUN.VALUE=numeric(dim(tmp_targetFeatTable)[1]))
    # establish if uROIExist
    tmp_uROIExist <- !any(is.na(tmp_uROI[,c("rtMin", "rtMax", "mzMin",
                                            "mzMax")]))
    if (tmp_uROIExist) { if (verbose) { message('uROIExist set to TRUE') }
    } else { if (verbose) { message('NA in uROI, uROIExist is set to FALSE') }}
    # Check ROI (no NA, is in order)
    check_NA_in_target_parameters(tmp_targetFeatTable)
    if ( !(all(tmp_targetFeatTable$rtMin <= tmp_targetFeatTable$rtMax) &
            all(tmp_targetFeatTable$mzMin <= tmp_targetFeatTable$mzMax)) ) {
        stop('Check ROI values: "rtMin" < "rtMax" and "mzMin" < "mzMax"') }
    # Check uROI is in order (cannot have NA and uROIExist)
    if (tmp_uROIExist) {
        if ( !(all(tmp_uROI$rtMin <= tmp_uROI$rtMax) &
                all(tmp_uROI$mzMin <= tmp_uROI$mzMax)) ) {
            stop('Check uROI values: "rtMin" < "rtMax" and "mzMin" < "mzMax"')}}
    return(list(targetFeatTable=tmp_targetFeatTable, uROI=tmp_uROI, FIR=tmp_FIR,
                uROIExist=tmp_uROIExist))
}

context('peakPantheR_loadAnnotationParamsCSV()')

## Test the generation of a new object by reading parameters from CSV (simple: Spl, advanced: Adv)
# Also test prepare_basic_target_parameters() and prepare_advanced_target_parameters()

## Input data
input_CSV_adv      <- data.frame(matrix(nrow=2,ncol=21,dimnames=list(c(), c('cpdID', 'cpdName', 'X', 'ROI_rt', 'ROI_mz','ROI_rtMin', 'ROI_rtMax', 'ROI_mzMin', 'ROI_mzMax', 'X', 'uROI_rtMin', 'uROI_rtMax', 'uROI_mzMin', 'uROI_mzMax', 'uROI_rt', 'uROI_mz', 'X', 'FIR_rtMin', 'FIR_rtMax', 'FIR_mzMin', 'FIR_mzMax'))))
input_CSV_adv[1,]  <- c('ID-1', 'Cpd 1', '|', 1.,  2.,  3.,  4.,  5.,  6.,  '|', 7.,  8.,  9.,  10., 11., 12., '|', 13., 14., 15., 16.)
input_CSV_adv[2,]  <- c('ID-2', 'Cpd 2', '|', 17., 18., 19., 20., 21., 22., '|', 23., 24., 25., 26., 27., 28., '|', 29., 30., 31., 32.)
input_CSV_adv[,-c(1,2,3,10,17)]  <- sapply(input_CSV_adv[,-c(1,2,3,10,17)], as.numeric)

input_CSV_spl           <- input_CSV_adv[, c('cpdID', 'cpdName', 'ROI_rtMin', 'ROI_rt', 'ROI_rtMax', 'ROI_mzMin', 'ROI_mz', 'ROI_mzMax')]
colnames(input_CSV_spl) <- c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax")

## Expected result
# targetFeatTable
input_targetFeatTable_adv     <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable_adv[1,] <- c("ID-1", "Cpd 1",  3.,  1.,  4.,  5.,  2.,  6.)
input_targetFeatTable_adv[2,] <- c("ID-2", "Cpd 2", 19., 17., 20., 21., 18., 22.)
input_targetFeatTable_adv[,c(3:8)] <- sapply(input_targetFeatTable_adv[,c(3:8)], as.numeric)

input_targetFeatTable_spl     <- input_targetFeatTable_adv

# uROI
input_uROI_adv      <- data.frame(matrix(vector(), 2, 6, dimnames=list(c(), c("rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))), stringsAsFactors=FALSE)
input_uROI_adv[1,]  <- c( 7., 11. , 8.,  9., 12., 10.)
input_uROI_adv[2,]  <- c(23., 27., 24., 25., 28., 26.)

# FIR
input_FIR_adv       <- data.frame(matrix(vector(), 2, 4, dimnames=list(c(), c("rtMin", "rtMax", "mzMin", "mzMax"))), stringsAsFactors=FALSE)
input_FIR_adv[1,]   <- c(13., 14., 15., 16.)
input_FIR_adv[2,]   <- c(29., 30., 31., 32.)

# object
input_annotation_adv  <- peakPantheRAnnotation(targetFeatTable = input_targetFeatTable_adv,
                                               uROI = input_uROI_adv, 
                                               FIR = input_FIR_adv,
                                               uROIExist = TRUE)


input_annotation_spl  <- peakPantheRAnnotation(targetFeatTable = input_targetFeatTable_spl)


test_that('Spl - default with ROI, verbose, no verbose', {
  # temporary file
  savePath1           <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.csv')
  
  # save csv
  utils::write.csv(input_CSV_spl, file=savePath1, row.names=FALSE)
  
  # expected
  expected_annotation <- input_annotation_spl
  expected_message    <- c("New peakPantheRAnnotation object initialised for 2 compounds\n")
  
  # results (output, warnings and messages)
  result_load1        <- evaluate_promise(peakPantheR_loadAnnotationParamsCSV(savePath1, verbose=TRUE))
  
  # Check result
  expect_equal(result_load1$result, expected_annotation)
  
  # Check result messages
  expect_equal(length(result_load1$messages), 1)
  expect_equal(result_load1$messages, expected_message)
  
  ## no verbose
  result_load2        <- evaluate_promise(peakPantheR_loadAnnotationParamsCSV(savePath1, verbose=FALSE))
  expect_equal(length(result_load2$messages), 0)
})

test_that('Adv - default with ROI, uROI, FIR, uROIExist=TRUE, verbose, no verbose', {
  # temporary file
  savePath2           <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.csv')
  
  # save csv
  utils::write.csv(input_CSV_adv, file=savePath2, row.names=FALSE)
  
  # expected
  expected_annotation <- input_annotation_adv
  expected_message    <- c("uROIExist set to TRUE\n", "New peakPantheRAnnotation object initialised for 2 compounds\n")
  
  # results (output, warnings and messages)
  result_load1        <- evaluate_promise(peakPantheR_loadAnnotationParamsCSV(savePath2, verbose=TRUE))
  
  # Check result
  expect_equal(result_load1$result, expected_annotation)
  
  # Check result messages
  expect_equal(length(result_load1$messages), 2)
  expect_equal(result_load1$messages, expected_message)
  
  ## no verbose
  result_load2        <- evaluate_promise(peakPantheR_loadAnnotationParamsCSV(savePath2, verbose=FALSE))
  expect_equal(length(result_load2$messages), 0)
})

test_that('Adv - ROI with NA, uROI with NA, FIR with NA, uROIExist=FALSE, verbose, no verbose', {
  # NA in ROI_rt mz are allowed, NA in uROI set uROIExist=FALSE, NA in FIR is allowed
  
  # temporary file
  savePath3           <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.csv')
  
  # save csv
  tmp_CSV             <- input_CSV_adv
  tmp_CSV$ROI_rt      <- c(as.numeric(NA), 17.)
  tmp_CSV$ROI_mz      <- c(2, as.numeric(NA))
  tmp_CSV$uROI_rtMin  <- c(7., as.numeric(NA))
  tmp_CSV$FIR_mzMax   <- c(as.numeric(NA), as.numeric(NA))
  utils::write.csv(tmp_CSV, file=savePath3, row.names=FALSE)
  
  # expected
  tmp_targetFeatTable     <- input_targetFeatTable_adv
  tmp_targetFeatTable$rt  <- c(as.numeric(NA), 17.)
  tmp_targetFeatTable$mz  <- c(2, as.numeric(NA))
  tmp_uROI                <- input_uROI_adv
  tmp_uROI$rtMin          <- c(7., as.numeric(NA))
  tmp_FIR                 <- input_FIR_adv
  tmp_FIR$mzMax           <- c(as.numeric(NA), as.numeric(NA))
  expected_annotation     <- peakPantheRAnnotation(targetFeatTable=tmp_targetFeatTable, uROI=tmp_uROI, FIR=tmp_FIR, uROIExist=FALSE)
  expected_message        <- c("NA in uROI, uROIExist is set to FALSE\n", "New peakPantheRAnnotation object initialised for 2 compounds\n")
  
  # results (output, warnings and messages)
  result_load1        <- evaluate_promise(peakPantheR_loadAnnotationParamsCSV(savePath3, verbose=TRUE))
  
  # Check result
  expect_equal(result_load1$result, expected_annotation)
  
  # Check result messages
  expect_equal(length(result_load1$messages), 2)
  expect_equal(result_load1$messages, expected_message)
  
  ## no verbose
  result_load2        <- evaluate_promise(peakPantheR_loadAnnotationParamsCSV(savePath3, verbose=FALSE))
  expect_equal(length(result_load2$messages), 0)
})

test_that('raise errors - peakPantheR_loadAnnotationParamsCSV', {
  # file doesn't exist
  noFile  <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.csv')
  msg1    <- paste('specified "CSVParamPath" does not exist', sep='')
  expect_error(peakPantheR_loadAnnotationParamsCSV(noFile, verbose=TRUE), msg1, fixed=TRUE)
})

test_that('prepare_basic_target_parameters', {
  # expected
  expected_targetFeatTable  <- input_targetFeatTable_spl
  expected_uROI             <- data.frame(rtMin=numeric(), rt=numeric(), rtMax=numeric(), mzMin=numeric(), mz=numeric(), mzMax=numeric(), stringsAsFactors=FALSE)
  expected_FIR              <- data.frame(rtMin=numeric(), rtMax=numeric(), mzMin=numeric(), mzMax=numeric(), stringsAsFactors=FALSE)
  expected_uROIExist        <- FALSE

  # results (output, warnings and messages)
  result_prepare1     <- evaluate_promise(prepare_basic_target_parameters(input_CSV_spl))
  
  # Check result
  expect_equal(result_prepare1$result$targetFeatTable, expected_targetFeatTable)
  expect_equal(result_prepare1$result$uROI, expected_uROI)
  expect_equal(result_prepare1$result$FIR, expected_FIR)
  expect_equal(result_prepare1$result$uROIExist, expected_uROIExist)
  
  # Check result messages
  expect_equal(length(result_prepare1$messages), 0)
})

test_that('raise errors - prepare_basic_target_parameters', {
  # expected columns are missing
  wrongCol_spl  <- data.frame(matrix(nrow=2,ncol=2,dimnames=list(c(), c('wrongCol','notAColumn'))))
  msg2          <- paste('Columns in "CSVParamPath" must be: "cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"')
  expect_error(prepare_basic_target_parameters(wrongCol_spl), msg2, fixed=TRUE)
  
  # ROI rtMin > rtMax
  wrongROIrt_spl        <- input_CSV_spl
  wrongROIrt_spl$rtMin  <- c(5, 19)
  msg3                  <- paste('Check ROI values: "rtMin" < "rtMax" and "mzMin" < "mzMax"')
  expect_error(prepare_basic_target_parameters(wrongROIrt_spl), msg3, fixed=TRUE)
  
  # ROI mzMin > mzMax
  wrongROImz_spl        <- input_CSV_spl
  wrongROImz_spl$mzMin  <- c(5, 23)
  msg4                  <- paste('Check ROI values: "rtMin" < "rtMax" and "mzMin" < "mzMax"')
  expect_error(prepare_basic_target_parameters(wrongROImz_spl), msg4, fixed=TRUE)
  
  # ROI rtMin is NA
  wrongROIrtMin        <- input_CSV_spl
  wrongROIrtMin$rtMin  <- c(NA, 19)
  msg4                  <- paste('Check ROI values: "ROI$rtMin", "ROI$rtMax", "ROI$mzMin" and "ROI$mzMax" cannot be NA')
  expect_error(prepare_basic_target_parameters(wrongROIrtMin), msg4, fixed=TRUE)
  # ROI rtMax is NA
  wrongROIrtMax        <- input_CSV_spl
  wrongROIrtMax$rtMax  <- c(4, NA)
  msg5                  <- paste('Check ROI values: "ROI$rtMin", "ROI$rtMax", "ROI$mzMin" and "ROI$mzMax" cannot be NA')
  expect_error(prepare_basic_target_parameters(wrongROIrtMax), msg5, fixed=TRUE)
  # ROI mzMin is NA
  wrongROImzMin        <- input_CSV_spl
  wrongROImzMin$mzMin  <- c(NA, 21)
  msg6                  <- paste('Check ROI values: "ROI$rtMin", "ROI$rtMax", "ROI$mzMin" and "ROI$mzMax" cannot be NA')
  expect_error(prepare_basic_target_parameters(wrongROImzMin), msg6, fixed=TRUE)
  # ROI mzMax is NA
  wrongROImzMax        <- input_CSV_spl
  wrongROImzMax$mzMax  <- c(6, NA)
  msg7                  <- paste('Check ROI values: "ROI$rtMin", "ROI$rtMax", "ROI$mzMin" and "ROI$mzMax" cannot be NA')
  expect_error(prepare_basic_target_parameters(wrongROImzMax), msg7, fixed=TRUE)
})

test_that('prepare_advanced_target_parameters', {
  # expected
  expected_targetFeatTable  <- input_targetFeatTable_adv
  expected_uROI             <- input_uROI_adv
  expected_FIR              <- input_FIR_adv
  expected_uROIExist        <- TRUE
  expected_message          <- c("uROIExist set to TRUE\n")
  
  
  # results (output, warnings and messages)
  result_prepare1     <- evaluate_promise(prepare_advanced_target_parameters(input_CSV_adv, verbose=TRUE))
  
  # Check result
  expect_equal(result_prepare1$result$targetFeatTable, expected_targetFeatTable)
  expect_equal(result_prepare1$result$uROI, expected_uROI)
  expect_equal(result_prepare1$result$FIR, expected_FIR)
  expect_equal(result_prepare1$result$uROIExist, expected_uROIExist)
  
  # Check result messages
  expect_equal(length(result_prepare1$messages), 1)
  expect_equal(result_prepare1$messages, expected_message)
})

test_that('raise errors - prepare_advanced_target_parameters', {

  # expected columns are missing
  wrongCol_adv  <- data.frame(matrix(nrow=2,ncol=2,dimnames=list(c(), c('wrongCol','notAColumn'))))
  msg5          <- paste('Columns in "CSVParamPath" must be: "cpdID", "cpdName", "ROI_rt", "ROI_mz", "ROI_rtMin", "ROI_rtMax", "ROI_mzMin", "ROI_mzMax", "uROI_rtMin", "uROI_rtMax", "uROI_mzMin", "uROI_mzMax", "uROI_rt", "uROI_mz", "FIR_rtMin", "FIR_rtMax", "FIR_mzMin", "FIR_mzMax"')
  expect_error(prepare_advanced_target_parameters(wrongCol_adv, verbose=TRUE), msg5, fixed=TRUE)
  
  # ROI rtMin > rtMax
  wrongROIrt_adv            <- input_CSV_adv
  wrongROIrt_adv$ROI_rtMin  <- c(5, 19)
  msg6                      <- paste('Check ROI values: "rtMin" < "rtMax" and "mzMin" < "mzMax"')
  expect_error(prepare_advanced_target_parameters(wrongROIrt_adv, verbose=FALSE), msg6, fixed=TRUE)
  
  # ROI mzMin > mzMax
  wrongROImz_adv            <- input_CSV_adv
  wrongROImz_adv$ROI_mzMin  <- c(5, 23)
  msg7                      <- paste('Check ROI values: "rtMin" < "rtMax" and "mzMin" < "mzMax"')
  expect_error(prepare_advanced_target_parameters(wrongROImz_adv, verbose=FALSE), msg7, fixed=TRUE)
  
  # ROI rtMin is NA
  wrongROIrtMin             <- input_CSV_adv
  wrongROIrtMin$ROI_rtMin   <- c(NA, 19)
  msg10                     <- paste('Check ROI values: "ROI$rtMin", "ROI$rtMax", "ROI$mzMin" and "ROI$mzMax" cannot be NA')
  expect_error(prepare_advanced_target_parameters(wrongROIrtMin, verbose=FALSE), msg10, fixed=TRUE)
  # ROI rtMax is NA
  wrongROIrtMax             <- input_CSV_adv
  wrongROIrtMax$ROI_rtMax   <- c(4, NA)
  msg11                     <- paste('Check ROI values: "ROI$rtMin", "ROI$rtMax", "ROI$mzMin" and "ROI$mzMax" cannot be NA')
  expect_error(prepare_advanced_target_parameters(wrongROIrtMax, verbose=FALSE), msg11, fixed=TRUE)
  # ROI mzMin is NA
  wrongROImzMin             <- input_CSV_adv
  wrongROImzMin$ROI_mzMin   <- c(NA, 21)
  msg12                     <- paste('Check ROI values: "ROI$rtMin", "ROI$rtMax", "ROI$mzMin" and "ROI$mzMax" cannot be NA')
  expect_error(prepare_advanced_target_parameters(wrongROImzMin, verbose=FALSE), msg12, fixed=TRUE)
  # ROI mzMax is NA
  wrongROImzMax             <- input_CSV_adv
  wrongROImzMax$ROI_mzMax   <- c(6, NA)
  msg13                     <- paste('Check ROI values: "ROI$rtMin", "ROI$rtMax", "ROI$mzMin" and "ROI$mzMax" cannot be NA')
  expect_error(prepare_advanced_target_parameters(wrongROImzMax, verbose=FALSE), msg13, fixed=TRUE)

  # uROI rtMin > rtMax
  wrongUROIrt               <- input_CSV_adv
  wrongUROIrt$uROI_rtMin    <- c(7, 25)
  msg8                      <- paste('Check uROI values: "rtMin" < "rtMax" and "mzMin" < "mzMax"')
  expect_error(prepare_advanced_target_parameters(wrongUROIrt, verbose=FALSE), msg8, fixed=TRUE)
  
  # uROI mzMin < mzMax
  wrongUROImz               <- input_CSV_adv
  wrongUROImz$uROI_mzMin    <- c(12, 25)
  msg9                      <- paste('Check uROI values: "rtMin" < "rtMax" and "mzMin" < "mzMax"')
  expect_error(prepare_advanced_target_parameters(wrongUROImz, verbose=FALSE), msg9, fixed=TRUE)
})

## ----biocstyle, echo = FALSE, results = "asis"--------------------------------
BiocStyle::markdown()

## ----echo = FALSE-------------------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)

## ----init, message = FALSE, echo = FALSE, results = "hide"--------------------
## Silently loading all packages
library(BiocStyle)
library(peakPantheR)
library(faahKO)
library(pander)

## ----eval = FALSE-------------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("peakPantheR")

## ----eval = FALSE-------------------------------------------------------------
# # Install devtools
# if(!require("devtools")) install.packages("devtools")
# devtools::install_github("phenomecentre/peakPantheR")

## ----eval = FALSE-------------------------------------------------------------
# library(peakPantheR)
# 
# peakPantheR_start_GUI(browser = TRUE)
# #  To exit press ESC in the command line

## ----fig.align="center", out.width = "700px", echo = FALSE--------------------
knitr::include_graphics("figures/example-UI.png")

## ----eval = FALSE-------------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("faahKO")

## -----------------------------------------------------------------------------
library(faahKO)
## file paths
input_spectraPaths  <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                        system.file('cdf/KO/ko16.CDF', package = "faahKO"),
                        system.file('cdf/KO/ko18.CDF', package = "faahKO"))
input_spectraPaths

## ----eval=FALSE---------------------------------------------------------------
# # targetFeatTable
# input_targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#                         c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin",
#                         "mz", "mzMax"))), stringsAsFactors=FALSE)
# input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390., 522.194778,
#                                 522.2, 522.205222)
# input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440., 496.195038,
#                                 496.2, 496.204962)
# input_targetFeatTable[,c(1,3:8)] <- sapply(input_targetFeatTable[,c(1,3:8)],
#                                             as.numeric)

## ----results = "asis", echo = FALSE-------------------------------------------
# use pandoc for improved readability
input_targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), 
                        c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", 
                        "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390., 522.194778, 
                                522.2, 522.205222)
input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440., 496.195038,
                                496.2, 496.204962)
input_targetFeatTable[,c(1,3:8)] <- sapply(input_targetFeatTable[,c(1,3:8)], 
                                            as.numeric)
rownames(input_targetFeatTable) <- NULL
pander::pandoc.table(input_targetFeatTable, digits = 9)

## ----eval=FALSE---------------------------------------------------------------
# library(faahKO)
# 
# # Define the file paths (3 samples)
# input_spectraPaths  <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
#                         system.file('cdf/KO/ko16.CDF', package = "faahKO"),
#                         system.file('cdf/KO/ko18.CDF', package = "faahKO"))
# 
# # Define the targeted features (2 features)
# input_targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#                         c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin",
#                         "mz", "mzMax"))), stringsAsFactors=FALSE)
# input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390.,
#                                 522.194778, 522.2, 522.205222)
# input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440.,
#                                 496.195038, 496.2, 496.204962)
# input_targetFeatTable[,3:8] <- sapply(input_targetFeatTable[,3:8], as.numeric)
# 
# # Define some random compound and spectra metadata
# # cpdMetadata
# input_cpdMetadata     <- data.frame(matrix(data=c('a','b',1,2), nrow=2, ncol=2,
#                             dimnames=list(c(), c('testcol1','testcol2')),
#                             byrow=FALSE), stringsAsFactors=FALSE)
# # spectraMetadata
# input_spectraMetadata <- data.frame(matrix(data=c('c','d','e',3,4,5), nrow=3,
#                             ncol=2,
#                             dimnames=list(c(),c('testcol1','testcol2')),
#                             byrow=FALSE), stringsAsFactors=FALSE)
# 
# # Initialise a simple peakPantheRAnnotation object
# # [3 files, 2 features, no uROI, no FIR]
# initAnnotation <- peakPantheRAnnotation(spectraPaths=input_spectraPaths,
#                                         targetFeatTable=input_targetFeatTable,
#                                         cpdMetadata=input_cpdMetadata,
#                                         spectraMetadata=input_spectraMetadata)
# 
# # Rename and save the annotation to disk
# annotationObject <- initAnnotation
# save(annotationObject,
#         file = './example_annotation_ppR_UI.RData',
#         compress=TRUE)
# 

## ----eval=FALSE---------------------------------------------------------------
# # Define targeted features without uROI and FIR (2 features)
# input_targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
#                         c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin",
#                         "mz", "mzMax"))), stringsAsFactors=FALSE)
# input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390.,
#                                 522.194778, 522.2, 522.205222)
# input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440.,
#                                 496.195038, 496.2, 496.204962)
# input_targetFeatTable[,3:8] <- sapply(input_targetFeatTable[,3:8], as.numeric)
# 
# # save to disk
# write.csv(input_targetFeatTable,
#             file = './1-fitParams_example_UI.csv',
#             row.names = FALSE)

## ----results = "asis", echo = FALSE-------------------------------------------
# use pandoc for improved readability
input_targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
                        c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin",
                        "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390.,
                                522.194778, 522.2, 522.205222)
input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440.,
                                496.195038, 496.2, 496.204962)
input_targetFeatTable[,3:8] <- sapply(input_targetFeatTable[,3:8], as.numeric)
rownames(input_targetFeatTable) <- NULL
pander::pandoc.table(input_targetFeatTable, digits = 9)

## ----eval=FALSE---------------------------------------------------------------
# # Define the spectra paths and metada
# input_spectraMeta <- data.frame(matrix(vector(), 3, 3,
#                         dimnames=list(c(),c("filepath","testcol1","testcol2"))),
#                         stringsAsFactors=FALSE)
# input_spectraMeta[1,] <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
#                             "c", 3)
# input_spectraMeta[2,] <- c(system.file('cdf/KO/ko16.CDF', package = "faahKO"),
#                             "d", 4)
# input_spectraMeta[3,] <- c(system.file('cdf/KO/ko18.CDF', package = "faahKO"),
#                             "e", 5)
# 
# # save to disk
# write.csv(input_spectraMeta,
#             file = './2-spectraMetaWPath_example_UI.csv',
#             row.names = FALSE)

## ----results = "asis", echo = FALSE-------------------------------------------
# use pandoc for improved readability
input_spectraMeta <- data.frame(matrix(vector(), 3, 3,
                        dimnames=list(c(),c("filepath","testcol1","testcol2"))),
                        stringsAsFactors=FALSE)
input_spectraMeta[1,] <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                            "c", 3)
input_spectraMeta[2,] <- c(system.file('cdf/KO/ko16.CDF', package = "faahKO"),
                            "d", 4)
input_spectraMeta[3,] <- c(system.file('cdf/KO/ko18.CDF', package = "faahKO"),
                            "e", 5)
rownames(input_spectraMeta) <- NULL
pander::pandoc.table(input_spectraMeta, digits = 0)

## ----eval=FALSE---------------------------------------------------------------
# # Define the feature metada
# input_featMeta <- data.frame(matrix(vector(), 2, 2,
#                     dimnames=list(c(),c("testcol1","testcol2"))),
#                     stringsAsFactors=FALSE)
# input_featMeta[1,] <- c("a", 1)
# input_featMeta[2,] <- c("b", 2)
# 
# # save to disk
# write.csv(input_featMeta,
#             file = './3-featMeta_example_UI.csv',
#             row.names = FALSE)

## ----results = "asis", echo = FALSE-------------------------------------------
# use pandoc for improved readability
input_featMeta <- data.frame(matrix(vector(), 2, 2,
                    dimnames=list(c(),c("testcol1","testcol2"))),
                    stringsAsFactors=FALSE)
input_featMeta[1,] <- c("a", 1)
input_featMeta[2,] <- c("b", 2)
rownames(input_featMeta) <- NULL
pander::pandoc.table(input_featMeta, digits = 0)

## ----echo = FALSE-------------------------------------------------------------
devtools::session_info()

#' @title Parse acquisition date from a mzML file
#'
#' @description Extract acquisition date (`'startTimeStamp'``) from a mzML file.
#' In case of failure (or the file is not a \code{mzML}) returns NULL
#'
#' @param mzMLPath (str) path to mzML raw data file
#' @param verbose (bool) if TRUE message progress
#'
#' @return POSIXct or NA
#' 
#' @import XML
getAcquisitionDatemzML <- function(mzMLPath, verbose = TRUE) {
    ## Check input
    mzMLPath <- normalizePath(mzMLPath, mustWork = FALSE)
    # not a mzML extension
    if (tolower(stringr::str_sub(basename(mzMLPath), start = -5)) != ".mzml") {
        if (verbose) { message("Check input, mzMLPath must be a .mzML") }
        return(NA)
    }
    stime <- Sys.time()
    
    ## Parse XML
    acqTime <- tryCatch({
        ## try
        nLinesRead <- 1
        mzMLFile <- file(mzMLPath, "r")
        while (TRUE) {
            currentLine <- readLines(mzMLFile, n = 1)
            if (length(currentLine) == 0) {
                if (verbose) {
                    message("startTimeStamp tag not found, mzMLPath is not a",
                            " valid mzML file") }
                return(NA)
            }
            if (nLinesRead == 2) {
                if (isFALSE(grepl("<indexedmzML", currentLine, fixed=TRUE))) {
                    if (verbose) {
                    message("Check input, mzMLPath is not a valid mzML file")
                    }
                    return(NA)
                }
            }
            regex1 <-  stringr::str_extract(string = currentLine,
                    pattern = "(?<=startTimeStamp=\")(.*?)(?=\")")
            if (isFALSE(is.na(regex1))) {
                acqTime <- strptime(regex1, format = "%Y-%m-%dT%H:%M:%S")
                ## Output
                etime <- Sys.time()
                if (verbose) {
                    message("Acquisition date parsed in: ",
                            round(as.double(difftime(etime, stime)), 2),
                    " ", units(difftime(etime, stime)))}
                return(acqTime) }
            nLinesRead <- nLinesRead + 1 }
    }, error = function(cond) {
        ## catch
        if (verbose) {
            message("Check input, failure while parsing mzMLPath")}
        return(NA)
    })
}

context('getAcquisitionDatemzML()')

# test we get Acquisition date from mzML


test_that('get Acquisition date from real mzML, verbose', {
  realmzML      <- system.file("extdata/test_fakemzML.mzML", package = "peakPantheR")
  expected_date <- as.POSIXct("2017-07-13 21:06:14")

  # results (output, warnings and messages)
  result  <- evaluate_promise(getAcquisitionDatemzML(realmzML, verbose=TRUE))
  
  # Check result
  expect_equal(result$result, expected_date)

  # Check message (cannot match execution time)
  expect_equal(length(result$messages), 1)
})

test_that('get Acquisition date from real mzML, no verbose', {
  realmzML      <- system.file("extdata/test_fakemzML.mzML", package = "peakPantheR")
  expected_date <- as.POSIXct("2017-07-13 21:06:14")

  # results (output, warnings and messages)
  result  <- evaluate_promise(getAcquisitionDatemzML(realmzML, verbose=FALSE))

  # Check result
  expect_equal(result$result, expected_date)

  # Check no message
  expect_equal(length(result$messages), 0)
})

test_that('detect not an mzML, verbose', {
  wrongmzML     <- './path/notAmzML.png'
  expected_date <- NA
  msg           <- "Check input, mzMLPath must be a .mzML\n"

  # results (output, warnings and messages)
  result  <- evaluate_promise(getAcquisitionDatemzML(wrongmzML, verbose=TRUE))

  # Check result
  expect_equal(result$result, expected_date)

  # Check message (cannot match execution time)
  expect_equal(result$messages, msg)
})

test_that('detect not an mzML, no verbose', {
  wrongmzML     <- './path/notAmzML.png'
  expected_date <- NA

  # results (output, warnings and messages)
  result  <- evaluate_promise(getAcquisitionDatemzML(wrongmzML, verbose=FALSE))

  # Check result
  expect_equal(result$result, expected_date)

  # Check message (cannot match execution time)
  expect_equal(length(result$messages), 0)
})

test_that('detect wrong XML, verbose', {
  # create wrong XML file
  xmlfile               <- XML::xmlParse(system.file("extdata/test_fakemzML.mzML", package = "peakPantheR"))
  xmltop                <- XML::xmlRoot(xmlfile)
  XML::xmlName(xmltop)  <- "not_indexedmzML"
  wrongmzML             <- file.path(tempdir(),'notValidXML.mzML')
  XML::saveXML(xmlfile, file=wrongmzML)
  # expected
  expected_date <- NA
  msg           <- "Check input, mzMLPath is not a valid mzML file\n"

  # results (output, warnings and messages)
  result  <- evaluate_promise(getAcquisitionDatemzML(wrongmzML, verbose=TRUE))

  # Check result
  expect_equal(result$result, expected_date)

  # Check message (cannot match execution time)
  expect_equal(result$messages[1], msg)
})

test_that('detect wrong XML, no verbose', {
  # create wrong XML file
  xmlfile               <- XML::xmlParse(system.file("extdata/test_fakemzML.mzML", package = "peakPantheR"))
  xmltop                <- XML::xmlRoot(xmlfile)
  XML::xmlName(xmltop)  <- "not_indexedmzML"
  wrongmzML             <- file.path(tempdir(),'notValidXML.mzML')
  XML::saveXML(xmlfile, file=wrongmzML)
  # expected
  expected_date <- NA

  # results (output, warnings and messages)
  result  <- evaluate_promise(getAcquisitionDatemzML(wrongmzML, verbose=FALSE))

  # Check result
  expect_equal(result$result, expected_date)

  # Check message (cannot match execution time)
  expect_equal(length(result$messages), 0)
})

test_that('detect mzML without startTimeStamp, verbose', {
  # create wrong XML file
  xmlfile               <- XML::xmlParse(system.file("extdata/test_fakemzML.mzML", package = "peakPantheR"))
  xmltop                <- XML::xmlRoot(xmlfile)
  XML::removeAttributes(xmltop[[1]][[6]], .attrs=c("startTimeStamp"))
  wrongmzML             <- file.path(tempdir(),'notValidXML.mzML')
  XML::saveXML(xmlfile, file=wrongmzML)
  # expected
  expected_date <- NA
  msg           <- "startTimeStamp tag not found, mzMLPath is not a valid mzML file\n"

  # results (output, warnings and messages)
  result  <- evaluate_promise(getAcquisitionDatemzML(wrongmzML, verbose=FALSE))

  # Check result
  expect_equal(result$result, expected_date)

  # Check message (cannot match execution time)
  expect_equal(length(result$messages), 0)
})

test_that('detect mzML without startTimeStamp, no verbose', {
  # create wrong XML file
  xmlfile               <- XML::xmlParse(system.file("extdata/test_fakemzML.mzML", package = "peakPantheR"))
  xmltop                <- XML::xmlRoot(xmlfile)
  XML::removeAttributes(xmltop[[1]][[6]], .attrs=c("startTimeStamp"))
  wrongmzML             <- file.path(tempdir(),'notValidXML.mzML')
  XML::saveXML(xmlfile, file=wrongmzML)
  # expected
  expected_date <- NA
  msg           <- "startTimeStamp tag not found, mzMLPath is not a valid mzML file\n"

  # results (output, warnings and messages)
  result  <- evaluate_promise(getAcquisitionDatemzML(wrongmzML, verbose=TRUE))

  # Check result
  expect_equal(result$result, expected_date)

  # Check message (cannot match execution time)
  expect_equal(result$messages[1], msg)
})

test_that('try catch failure mzML, verbose', {
  wrongmzML     <- './path/notAmzML.mzML'
  expected_date <- NA
  msg           <- "Check input, failure while parsing mzMLPath\n"

  # results (output, warnings and messages)
  result  <- evaluate_promise(getAcquisitionDatemzML(wrongmzML, verbose=TRUE))

  # Check result
  expect_equal(result$result, expected_date)

  # Check message (cannot match execution time)
  expect_equal(length(result$messages), 1)
  expect_equal(result$messages[1], msg)
})

test_that('try catch failure mzML, no verbose', {
  wrongmzML     <- './path/notAmzML.mzML'
  expected_date <- NA

  # results (output, warnings and messages)
  result  <- evaluate_promise(getAcquisitionDatemzML(wrongmzML, verbose=FALSE))

  # Check result
  expect_equal(result$result, expected_date)

  # Check message (cannot match execution time)
  expect_equal(length(result$messages), 0)
})

#' Correct targeted retention time based on reference compounds
#'
#' Correct targeted features retention time using the RT and RT deviation of
#' previously fitted compounds. The `method` and `params` are used to select and
#' parametrise the retention time correction method employed. When `robust`
#' is set to TRUE, the RANSAC algorithm is used
#' to automatically flag outliers and robustify the correction function fitting.
#'
#' @param targetFeatTable a \code{\link{data.frame}} of compounds to target as
#' rows and parameters as columns: \code{cpdID} (str), \code{cpdName} (str),
#' \code{rtMin} (float in seconds), \code{rt} (float in seconds, or \emph{NA}),
#' \code{rtMax} (float in seconds), \code{mzMin} (float), \code{mz} (float or
#' \emph{NA}), \code{mzMax} (float).
#' @param referenceTable a \code{\link{data.frame}} of reference compound
#' information as rows and properties as columns: \code{cpdID} (str),
#' \code{cpdName} (str), \code{rt} (float), \code{rt_dev_sec} (float)
#' @param method (str) name of RT correction method to use (currently
#' \code{polynomial})
#' @param params (list) list of parameters to pass to
#' the each correction method.
#' Currently allowed inputs are \code{polynomialOrder} for
#' \code{method='polynomial'}
#' @param robust (bool) whether to use the RANSAC algorithm to flag and
#' ignore outliers during retention time correction
#' 
#' @return a targetFeatTable with corrected RT
peakPantheR_applyRTCorrection <- function(targetFeatTable, referenceTable,
    method='polynomial', params=list(polynomialOrder=3), robust=TRUE) {
    # Check inputs
    applyRTCorrection_checkInput(targetFeatTable, referenceTable,
                                method, robust)
    params <- applyRTCorrection_checkInputParams(params, method, referenceTable)

    if (dim(referenceTable)[1] == 1) {
        method <- 'constant'
    }
    ## Run correction
    correctedFeatTable <- applyRTCorrection_correctFeatTable(
    targetFeatTable, referenceTable, method, params, robust)
    return(correctedFeatTable)
}

# Run correction
applyRTCorrection_correctFeatTable <- function(targetFeatTable, referenceTable,
    method='polynomial', params=list(polynomialOrder=3), robust=TRUE) {

    corrected_targetFeatTable <- targetFeatTable
    corrected_targetFeatTable$isReference <- 'External set'
    if (method == 'polynomial') {
        if (isTRUE(robust)) {
            rtCorrectionOutput <- fit_RANSAC(referenceTable$rt,
        referenceTable$rt_dev_sec, polynomialOrder=params[['polynomialOrder']])

        which_outlier <-
            corrected_targetFeatTable$cpdID %in% referenceTable$cpdID[
            !rtCorrectionOutput$inlier]
        which_reference <-
        corrected_targetFeatTable$cpdID %in% referenceTable$cpdID[
            rtCorrectionOutput$inlier]
        corrected_targetFeatTable$isReference[which_outlier] <-
            'Reference set outlier'
        corrected_targetFeatTable$isReference[which_reference]<-'Reference set'
        }  else {
            rtCorrectionOutput <-
                fit_polynomial(referenceTable$rt, referenceTable$rt_dev_sec,
                    polynomialOrder=params[['polynomialOrder']])
            which_reference <-
                corrected_targetFeatTable$cpdID %in% referenceTable$cpdID
            corrected_targetFeatTable$isReference[which_reference] <-
                'Reference set'}
        # Get the correction outputs
        correctionFunction <- rtCorrectionOutput$model
        correctedRtDrift <- stats::predict(correctionFunction,
                            newdata=data.frame(x=targetFeatTable$rt))
        corrected_targetFeatTable$correctedRt <-
            corrected_targetFeatTable$rt - correctedRtDrift
        corrected_targetFeatTable$predictedRtDrift <- correctedRtDrift }
    else if (method == 'constant') {
        # {redicted drift = constant observed drift in the reference
        corrected_targetFeatTable$correctedRt <-
            corrected_targetFeatTable$rt - rep(referenceTable$rt_dev_sec,
                dim(corrected_targetFeatTable)[1])
        corrected_targetFeatTable$predictedRtDrift <-
            rep(referenceTable$rt_dev_sec, dim(corrected_targetFeatTable)[1])
        which_reference <-
            corrected_targetFeatTable$cpdID %in% referenceTable$cpdID
        corrected_targetFeatTable$isReference[which_reference]<- 'Reference set'
    }  # for future Rt correction algorithms, just extend the else if()
    return(corrected_targetFeatTable)
}

# applyRTCorrection check input
applyRTCorrection_checkInput <- function(targetFeatTable, referenceTable,
                                            method, robust) {
    ## Check targetFeatTable and reference Table
    applyRTCorrection_checkTargetFeatTable(targetFeatTable)
    applyRTCorrection_checkInput_checkReferenceTable(referenceTable)
    ## Check method
    KNOWN_CORRECTIONMETHODs <- c("polynomial", "constant")
    if (!(method %in% KNOWN_CORRECTIONMETHODs)) {
        stop('Err', 'or: \"method\" must be one of: ',
            '\"polynomial\", \"constant\"')}
    if ((dim(referenceTable)[1] == 1) & (method != 'constant')) {
        stop("No function can be fitted with a single reference. ",
        "Use method=\`constant\` instead.") }

    if (method == 'constant') {
        if (dim(referenceTable)[1] > 1) {
            stop("`constant` Rt correction can ",
            "only use a single reference") } }

    ## Check robust argument
    if (!is.logical(robust)) { stop("robust must be either TRUE or FALSE")}
}

# check input targetFeatTable
applyRTCorrection_checkTargetFeatTable <- function(targetFeatTable) {
    # is data.frame
    if (!is(targetFeatTable, "data.frame")){
        stop("specified targetFeatTable is not a data.frame")
    }

    # required columns are present
    if (!all(c("cpdID", "cpdName", "rt",
            "rt_dev_sec") %in% colnames(targetFeatTable))){
        stop("expected columns in targetFeatTable ",
        "are \"cpdID\", \"cpdName\", ",
        "\"rt\", \"rt_dev_sec\"")
    }

    # column type
    if (dim(targetFeatTable)[1] != 0){
        if (!is.character(targetFeatTable$cpdID[1])){
            stop("targetFeatTable$cpdID must be character")
        }
        if (!is.character(targetFeatTable$cpdName[1])){
            stop("targetFeatTable$cpdName must be character")
        }
        if (!(is.numeric(targetFeatTable$rt[1]) |
                is.na(targetFeatTable$rt[1]))){
            stop("targetFeatTable$rt must be numeric or NA")
        }
        if (!(is.numeric(targetFeatTable$rt_dev_sec[1]) |
            is.na(targetFeatTable$rt_dev_sec[1]))) {
            stop("targetFeatTable$rt_dev_sec must be numeric or NA") }

    }
}

# check input referenceTable
applyRTCorrection_checkInput_checkReferenceTable <- function(referenceTable) {
    ## Check referenceTable
    # is data.frame
    if (!is(referenceTable, "data.frame")){
        stop("specified referenceTable is not a data.frame")
    }
    # required columns are present
    if (!all(c("cpdID", "cpdName", "rt", "rt_dev_sec") %in%
    colnames(referenceTable))){
        stop("expected columns in referenceTable are \"cpdID\", \"cpdName\", ",
        "\"rt\" and \"rt_dev_sec\"")
    }
    # column type
    if (dim(referenceTable)[1] != 0){
        if (!is.character(referenceTable$cpdID[1])){
            stop("referenceTable$cpdID must be character")
        }
        if (!is.character(referenceTable$cpdName[1])){
            stop("referenceTable$cpdName must be character")
        }
        if (!(is.numeric(referenceTable$rt[1]) | is.na(referenceTable$rt[1]))){
            stop("referenceTable$rt must be numeric or NA")
        }
        if (!(is.numeric(referenceTable$rt_dev_sec[1]) |
        is.na(referenceTable$rt_dev_sec[1]))){
            stop("referenceTable$rt_dev_sec must be numeric or NA") }
    }
}

# check input parameters
applyRTCorrection_checkInputParams <- function(params, method, referenceTable) {
    ## Check params input
    if (!is.list(params)) { stop('Check input, "params" must be list') }
    # Verify if parameters passed on params are adequate for chosen method
    if (is.list(params)) {
        if (method == 'polynomial') {
            if (!any(names(params) == 'polynomialOrder')) {
                stop("polynomialOrder must be provided in params") }
            else {
                if (!is(params[['polynomialOrder']], 'numeric')) {
                    stop("polynomialOrder must be an integer and equal ",
                    "or greater than 1") }
                else if (!isTRUE(all.equal(params[['polynomialOrder']],
                    as.integer(params[['polynomialOrder']]))) |
                    (params[['polynomialOrder']] < 1)) {
                    stop("polynomialOrder must be an ",
                    "integer and equal or greater than 1") }
                else if (params[['polynomialOrder']] >=
                dim(referenceTable)[1]) {
                    warning("`polynomialOrder` is larger than the number of ",
                    "references passed. `polynomialOrder` will be ",
                    "set equal to number of reference compounds - 1")
                    params[['polynomialOrder']] <- dim(referenceTable)[1] - 1}}
        }
    }
    return(params)
}

## ----------------------------------------------------------------------------
##        RANSAC
## ----------------------------------------------------------------------------
# New retention time calibration functions can be added below
# All fitting functions should have the following argument:
# of the kind x = theoretical rt, y= Deviation (Rt_{obs} - Rt{exp}}

# General purpose polynomial function - can be used for linear fits
fit_polynomial <- function(x, y, polynomialOrder, returnFitted=FALSE) {

    polyModel <- stats::lm(y ~ poly(x, degree=polynomialOrder))

    if (isTRUE(returnFitted)) {
        modelFitted <- stats::fitted(polyModel)
        modelResiduals <- stats::residuals(polyModel)
        return(list(model=polyModel, fitted=modelFitted,
                    residuals=modelResiduals))
    }
    else {return(list(model=polyModel))}
}

## Port of the RANSAC regressor from scikit-learn
fit_RANSAC <- function(x, y, polynomialOrder=3,
                        residual_threshold=NULL,
                        loss='absolute_loss',
                        min_samples=NULL,
                        max_trials = 1000, max_skips=Inf, stop_n_inliers=Inf,
                        stop_score=Inf, stop_probability=0.99) {

    loss_function <- fit_RANSAC_checkInput(x, y, polynomialOrder, loss)
    # If residual_threshold is NULL, use the MAD (median absolute deviation)
    # of the y variable as cutoff
    if (is.null(residual_threshold)) {
        residual_threshold <- stats::median(abs(y - stats::median(y)))
    }
    # Minimum number of samples - by default the max between
    # polynomial function degree + 1 and half of the dataset size
    if (is.null(min_samples)) {
        min_samples <- max(polynomialOrder + 1 , ceiling(0.5*length(x)))
    }

    ransacFit <- fit_RANSAC_coreOptimization(x, y,
    max_trials=max_trials, min_samples, polynomialOrder,
    loss_function,residual_threshold=residual_threshold,
    stop_n_inliers, stop_score, stop_probability)

    fit_RANSAC_checkFinalConvergence(ransacFit$inlier_mask_best,
                                    max_skips, ransacFit$n_skips_no_inliers)
    # estimate final model using all inliers
    final_fitted_regressor <- fit_polynomial(ransacFit$X_inlier_best,
                            ransacFit$y_inlier_best, polynomialOrder)
    inlier_mask <- ransacFit$inlier_mask_best
    return(list(model=final_fitted_regressor$model, inlier=inlier_mask))
}

# Absolute Loss function - part of RANSAC implementation
loss_absolute <- function(y_true, y_pred) {
    loss_value <- abs(y_true - y_pred)
    return(loss_value)
}

# Square loss function - part of RANSAC implementation
loss_squared <- function(y_true, y_pred) {
    loss_value <- (y_true - y_pred) ^ 2
    return(loss_value)
}

# dynamic max trials - part of the RANSAC implementation
dynamic_max_trials <- function(n_inliers, n_samples, min_samples, probability) {
    epsilon <- .Machine$double.eps
    inlier_ratio <- n_inliers / n_samples

    nom <- max(epsilon, 1 - probability)
    denom <- max(epsilon, 1 - inlier_ratio ^ min_samples)

    if (nom == 1) {return(0)}
    if (denom == 1) {return(Inf)}

    return(abs(ceiling(log(nom) / log(denom))))
}

# Core RANSAC algorithm
fit_RANSAC_coreOptimization <- function(x, y, max_trials,
    min_samples, polynomialOrder, loss_function, residual_threshold,
    stop_n_inliers=Inf, stop_score=Inf, stop_probability=0.99) {
    x_data <- data.frame(x, columns=c('x')) # data frame to call "predict.lm"
    n_trials<- 0
    n_skips_no_inliers <- 0
    n_inliers_best <- 0
    score_best <- 0
    while (n_trials < max_trials) {
        n_trials <- n_trials + 1
        data_subset_idx <- sample(seq(1, length(x)), min_samples, replace=FALSE)
        current_x <- x[data_subset_idx]
        current_y <- y[data_subset_idx]
        fitted_model_current_iteration <-
            tryCatch(fit_polynomial(current_x, current_y,
            polynomialOrder=polynomialOrder), error=function(e) e)
        if(inherits(fitted_model_current_iteration, "error")){
            n_skips_no_inliers <- n_skips_no_inliers + 1
            next }
        y_pred <- stats::predict(fitted_model_current_iteration$model,
                    newdata=x_data)
        residuals_subset <- loss_function(y, y_pred)  # residual calculation
        inlier_mask_subset <- residuals_subset < residual_threshold
        n_inliers_subset <- sum(inlier_mask_subset)
        # less inliers -> skip current random sample
        if (n_inliers_subset < n_inliers_best) {
            n_skips_no_inliers <- n_skips_no_inliers + 1
            next }
        # extract inlier data set
        inlier_idxs_subset <- seq(1, length(x))[inlier_mask_subset]
        X_inlier_subset <- x[inlier_idxs_subset]
        y_inlier_subset <- y[inlier_idxs_subset]
        score_subset <- summary(fit_polynomial(X_inlier_subset,
                        y_inlier_subset, polynomialOrder =
                        polynomialOrder)$model)$r.squared
        if ((n_inliers_subset == n_inliers_best) & (score_subset < score_best))
            {  next  } # same n_inliers but worse score = skip current random
        n_inliers_best <- n_inliers_subset
        score_best <- score_subset
        inlier_mask_best <- inlier_mask_subset
        X_inlier_best <- X_inlier_subset
        y_inlier_best <- y_inlier_subset
        max_trials <- min(max_trials, dynamic_max_trials(n_inliers_best,
        length(x), min_samples, stop_probability))
    if ((n_inliers_best >= stop_n_inliers) | (score_best >= stop_score))
    { break } } # break if sufficient number of inliers or score is reached
    return(list(X_inlier_best=X_inlier_best, y_inlier_best=y_inlier_best,
                n_skips_no_inliers=n_skips_no_inliers,
                inlier_mask_best=inlier_mask_best))
}

# check convergence - part of RANSAC implementation
fit_RANSAC_checkFinalConvergence <- function(inlier_mask_best, max_skips,
                                                n_skips_no_inliers) {
    # if none of the iterations met the required criteria
    if (is.null(inlier_mask_best)) {
        if ((n_skips_no_inliers) > max_skips) {
            warning("Number of skipped iterations larger than `max_skips`")
        } else {
            warning("RANSAC could not find a valid consensus set")
        }
    } else {
        if ((n_skips_no_inliers) > max_skips) {
            warning("RANSAC found a valid consensus set but exited
                    early due to skipping more iterations than
                    `max_skips`") }
    }
}

# check input - part of RANSAC implementation
fit_RANSAC_checkInput <- function(x,y, polynomialOrder, loss) {
    if (length(x) != length(y)) {
        stop('x and y must have the same length')
    }

    if (isFALSE(all.equal(polynomialOrder,
        as.integer(polynomialOrder)) | (polynomialOrder >= 1))) {
        stop("`polynomialOrder` must be an integer and equal or greater than 1")
    }
    # Select the loss function
    if (loss == "absolute_loss") {
        loss_function <- loss_absolute
    } else if (loss == "squared_loss") {
        loss_function <- loss_squared
    } else {
        stop("Allowed `loss` functions are `loss_squared` and `loss_absolute`")
    }
    return(loss_function)
    }

context('peakPantheR_applyRTCorrection()')


## Input and expected data
test_that('raises error()', {
  # Input
  tmp_targetFeatTable           <- data.frame(matrix(vector(), 2, 4, dimnames=list(c(), c("cpdID", "cpdName", "rt", "rt_dev_sec"))), stringsAsFactors=FALSE)
  tmp_targetFeatTable[1,]       <- c("ID-1", "Cpd 1", 11, 1.1)
  tmp_targetFeatTable[2,]       <- c("ID-2", "Cpd 2", 25, 3.2)
  tmp_targetFeatTable[, c(3:4)]  <- sapply(tmp_targetFeatTable[, c(3:4)], as.numeric)
  
  tmp_referenceTable            <- data.frame(matrix(vector(), 2, 4, dimnames=list(c(), c("cpdID", "cpdName", "rt", "rt_dev_sec"))), stringsAsFactors=FALSE)
  tmp_referenceTable[1,]        <- c("Ref-1", "Ref Cpd 1", 10, 1)
  tmp_referenceTable[2,]        <- c("Ref-2", "Ref Cpd 2", 20, 1.5)
  tmp_referenceTable[, c(3:4)]   <- sapply(tmp_referenceTable[, c(3:4)], as.numeric)

  # Trigger checks on targetFeatTable input
  # targetFeatTable is not a data.frame
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable='notADataFrame', referenceTable=tmp_referenceTable), 'specified targetFeatTable is not a data.frame')
  # targetFeatTable has wrong columns
  wrongDF1           <- tmp_targetFeatTable
  colnames(wrongDF1) <- c("not cpdID", "cpdName", "rt", "rt_dev_sec")
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=wrongDF1, referenceTable=tmp_referenceTable), "expected columns in targetFeatTable are \"cpdID\", \"cpdName\", \"rt\", \"rt_dev_sec\"")
  # targetFeatTable$cpdID not character
  wrongDF2        <- tmp_targetFeatTable
  wrongDF2$cpdID  <- c(5, 5)
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=wrongDF2, referenceTable=tmp_referenceTable), 'cpdID must be character')
  # targetFeatTable$cpdName not character
  wrongDF3          <- tmp_targetFeatTable
  wrongDF3$cpdName  <- c(5, 5)
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=wrongDF3, referenceTable=tmp_referenceTable), 'cpdName must be character')
  # targetFeatTable$rt not numeric
  wrongDF4        <- tmp_targetFeatTable
  wrongDF4$rt  <- c('notNumeric', 'notNumeric')
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=wrongDF4, referenceTable=tmp_referenceTable), 'rt must be numeric')
  # targetFeatTable$rt_dev_sec not numeric or NA
  wrongDF5        <- tmp_targetFeatTable
  wrongDF5$rt_dev_sec     <- c('notNumeric', 'notNumeric')
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=wrongDF5, referenceTable=tmp_referenceTable), 'rt_dev_sec must be numeric or NA')

  # wrong referenceTable
  # referenceTable is not a data.frame
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable='notADataFrame'), 'specified referenceTable is not a data.frame')
  # referenceTable has wrong columns
  wrongDF6           <- tmp_referenceTable
  colnames(wrongDF6) <- c("not cpdID", "cpdName", "rt", "rt_dev_sec")
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=wrongDF6), 'expected columns in referenceTable are "cpdID", "cpdName", "rt" and "rt_dev_sec"')
  # referenceTable$cpdID not character
  wrongDF11        <- tmp_referenceTable
  wrongDF11$cpdID  <- c(5, 5)
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=wrongDF11), 'cpdID must be character')
  # referenceTable$cpdName not character
  wrongDF12          <- tmp_referenceTable
  wrongDF12$cpdName  <- c(5, 5)
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=wrongDF12), 'cpdName must be character')
  # referenceTable$rt not numeric or NA
  wrongDF13        <- tmp_referenceTable
  wrongDF13$rt     <- c('notNumeric', 'notNumeric')
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=wrongDF13), 'rt must be numeric or NA')
  # referenceTable$rt_dev_sec not numeric or NA
  wrongDF14             <- tmp_referenceTable
  wrongDF14$rt_dev_sec  <- c('notNumeric', 'notNumeric')
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=wrongDF14), 'rt_dev_sec must be numeric or NA')
  
  # unknown correction method
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable, method='notKnown'), 'Error: \"method\" must be one of: \"polynomial\", \"constant\"', fixed=TRUE)

  # params is not a list or character
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable, params=5), 'Check input, "params" must be list', fixed=TRUE)
  # Robust is not logical
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable, robust=3, params=list(polynomialOrder=1)), 'robust must be either TRUE or FALSE', fixed=TRUE)

  # Method <-> params SPECIFIC CHECKS
  # method = polynomial but no polynomial order
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable, method='polynomial', params=list(a=5), robust=TRUE), 'polynomialOrder must be provided in params', fixed=TRUE)
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable, method='polynomial', params=list(a=5), robust=FALSE), 'polynomialOrder must be provided in params', fixed=TRUE)
  # method = polynomial but polynomial order is not valid
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable, method='polynomial', params=list(polynomialOrder='NA'), robust=FALSE), 'polynomialOrder must be an integer and equal or greater than 1', fixed=TRUE)
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable, method='polynomial', params=list(polynomialOrder=1.3), robust=FALSE), 'polynomialOrder must be an integer and equal or greater than 1', fixed=TRUE)
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable, method='polynomial', params=list(polynomialOrder=1.3), robust=TRUE), 'polynomialOrder must be an integer and equal or greater than 1', fixed=TRUE)

  # method = polynomial, valid polynomial order but exceeds number of references
  expect_warning(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable, method='polynomial', params=list(polynomialOrder=4), robust=FALSE), '`polynomialOrder` is larger than the number of references passed. `polynomialOrder` will be set equal to number of reference compounds - 1', fixed=TRUE)
  expect_warning(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable, method='polynomial', params=list(polynomialOrder=4), robust=TRUE), '`polynomialOrder` is larger than the number of references passed. `polynomialOrder` will be set equal to number of reference compounds - 1', fixed=TRUE)
  # test sugestion of method=constant if ref = 1
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable[1, ], method='polynomial', params=list(polynomialOrder=1), robust=FALSE), 'No function can be fitted with a single reference. Use method=\`constant\` instead.', fixed=TRUE)
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable[1, ], method='polynomial', params=list(polynomialOrder=1), robust=TRUE), 'No function can be fitted with a single reference. Use method=\`constant\` instead.', fixed=TRUE)
  # test constant method throws error with multiple references
  expect_error(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable, method='constant', params=list(polynomialOrder=1), robust=TRUE), "`constant` Rt correction can only use a single reference", fixed=TRUE)

})

test_that('polynomial method rt correction', {
  # Input
  tmp_targetFeatTable           <- data.frame(matrix(vector(), 5, 4, dimnames=list(c(), c("cpdID", "cpdName", "rt", "rt_dev_sec"))), stringsAsFactors=FALSE)
  tmp_targetFeatTable[1,]       <- c("ID-1", "Cpd 1", 11, 1.1)
  tmp_targetFeatTable[2,]       <- c("ID-2", "Cpd 2", 25, 1.5)
  tmp_targetFeatTable[3,]       <- c("ID-3", "Cpd 3", 35, 3.5)
  tmp_targetFeatTable[4,]       <- c("ID-4", "Cpd 4", 45, 5.5)
  tmp_targetFeatTable[5,]       <- c("ID-5", "Cpd 5", 55, 7)
  tmp_targetFeatTable[, c(3:4)]  <- sapply(tmp_targetFeatTable[, c(3:4)], as.numeric)

  tmp_referenceTable            <- data.frame(matrix(vector(), 5, 4, dimnames=list(c(), c("cpdID", "cpdName", "rt", "rt_dev_sec"))), stringsAsFactors=FALSE)
  tmp_referenceTable[1,]        <- c("Ref-1", "Ref Cpd 1", 11, 1.1)
  tmp_referenceTable[2,]        <- c("Ref-2", "Ref Cpd 2", 25, 1.5)
  tmp_referenceTable[3,]        <- c("Ref-3", "Ref Cpd 3", 35, 3.5)
  tmp_referenceTable[4,]        <- c("Ref-4", "Ref Cpd 4", 45, 5.5)
  tmp_referenceTable[5,]        <- c("Ref-5", "Ref Cpd 5", 55, 7)

  tmp_referenceTable[, c(3:4)]   <- sapply(tmp_referenceTable[, c(3:4)], as.numeric)
  # Expected model and results

  expected_corrected  <- data.frame(cpdID=character(), cpdName=character(), rt=double(), rt_dev_sec=double(), isReference=character(),
                                    correctedRt=double(), predictedRtDrift=double(), stringsAsFactors=FALSE)

  expected_corrected[1,]       <- list(cpdID="ID-1", cpdName="Cpd 1",  rt=11, rt_dev_sec=1.1, isReference="External set",   correctedRt=10.61282,  predictedRtDrift=0.387176)
  expected_corrected[2,]       <- list(cpdID="ID-2", cpdName="Cpd 2",  rt=25, rt_dev_sec=1.5, isReference="External set",   correctedRt=22.60164,  predictedRtDrift=2.398363)
  expected_corrected[3,]       <- list(cpdID="ID-3", cpdName="Cpd 3",  rt=35, rt_dev_sec=3.5, isReference="External set",  correctedRt= 31.16508,  predictedRtDrift=3.834925)
  expected_corrected[4,]       <- list(cpdID="ID-4", cpdName="Cpd 4",  rt=45, rt_dev_sec=5.5, isReference="External set",   correctedRt=39.72851,  predictedRtDrift=5.271487)
  expected_corrected[5,]       <- list(cpdID="ID-5", cpdName="Cpd 5",  rt=55, rt_dev_sec=7, isReference="External set",   correctedRt=48.29195,  predictedRtDrift=6.708049)

  expected_corrected_dg3                 <-  data.frame(cpdID=character(), cpdName=character(), rt=double(), rt_dev_sec=double(), isReference=character(),
                                    correctedRt=double(), predictedRtDrift=double(), stringsAsFactors=FALSE)
  expected_corrected_dg3[1,]       <- list(cpdID="ID-1", cpdName="Cpd 1",  rt=11, rt_dev_sec=1.1, isReference="External set",   correctedRt=9.907072,  predictedRtDrift=1.092928)
  expected_corrected_dg3[2,]       <- list(cpdID="ID-2", cpdName="Cpd 2",  rt=25, rt_dev_sec=1.5, isReference="External set",   correctedRt=23.457678,  predictedRtDrift=1.542322)
  expected_corrected_dg3[3,]       <- list(cpdID="ID-3", cpdName="Cpd 3",  rt=35, rt_dev_sec=3.5, isReference="External set",  correctedRt= 31.574063,  predictedRtDrift=3.425937)
  expected_corrected_dg3[4,]       <- list(cpdID="ID-4", cpdName="Cpd 4",  rt=45, rt_dev_sec=5.5, isReference="External set",   correctedRt=39.447720,  predictedRtDrift=5.552280)
  expected_corrected_dg3[5,]       <- list(cpdID="ID-5", cpdName="Cpd 5",  rt=55, rt_dev_sec=7, isReference="External set",   correctedRt=48.013466,  predictedRtDrift=6.986534)

   # results (output, warnings and messages)
  polynomial_regression_dg1 <- evaluate_promise(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable, method='polynomial', params=list(polynomialOrder=1), robust=FALSE))
  polynomial_regression_dg3 <- evaluate_promise(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable,
                                                                          method='polynomial', params=list(polynomialOrder=3), robust=FALSE))
  # Check results
  expect_equal(polynomial_regression_dg1$result, expected_corrected, tolerance=1e-5)
  expect_equal(polynomial_regression_dg3$result, expected_corrected_dg3, tolerance=1e-5)

  # test isReference is assigned correctely
  tmp_referenceTable            <- data.frame(matrix(vector(), 5, 4, dimnames=list(c(), c("cpdID", "cpdName", "rt", "rt_dev_sec"))), stringsAsFactors=FALSE)
  tmp_referenceTable[1,]        <- c("Ref-1", "Ref Cpd 1", 11, 1.1)
  tmp_referenceTable[2,]        <- c("ID-2", "Cpd 2", 25, 1.5)
  tmp_referenceTable[3,]        <- c("Ref-3", "Ref Cpd 3", 35, 3.5)
  tmp_referenceTable[4,]        <- c("Ref-4", "Ref Cpd 4", 45, 5.5)
  tmp_referenceTable[5,]        <- c("Ref-5", "Ref Cpd 5", 55, 7)

  tmp_referenceTable[, c(3:4)]   <- sapply(tmp_referenceTable[, c(3:4)], as.numeric)

  expected_corrected_isRef                 <-  data.frame(cpdID=character(), cpdName=character(), rt=double(), rt_dev_sec=double(), isReference=character(),
                                    correctedRt=double(), predictedRtDrift=double(), stringsAsFactors=FALSE)
  expected_corrected_isRef[1,]       <- list(cpdID="ID-1", cpdName="Cpd 1",  rt=11, rt_dev_sec=1.1, isReference="External set",   correctedRt=10.61282,  predictedRtDrift=0.387176)
  expected_corrected_isRef[2,]       <- list(cpdID="ID-2", cpdName="Cpd 2",  rt=25, rt_dev_sec=1.5, isReference="Reference set",   correctedRt=22.60164,  predictedRtDrift=2.398363)
  expected_corrected_isRef[3,]       <- list(cpdID="ID-3", cpdName="Cpd 3",  rt=35, rt_dev_sec=3.5, isReference="External set",  correctedRt= 31.16508,  predictedRtDrift=3.834925)
  expected_corrected_isRef[4,]       <- list(cpdID="ID-4", cpdName="Cpd 4",  rt=45, rt_dev_sec=5.5, isReference="External set",   correctedRt=39.72851,  predictedRtDrift=5.271487)
  expected_corrected_isRef[5,]       <- list(cpdID="ID-5", cpdName="Cpd 5",  rt=55, rt_dev_sec=7, isReference="External set",   correctedRt=48.29195,  predictedRtDrift=6.708049)

  polynomial_regression_ref <- evaluate_promise(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable, method='polynomial', params=list(polynomialOrder=1), robust=FALSE))

  expect_equal(polynomial_regression_ref$result, expected_corrected_isRef, tolerance=1e-5)
  # Check messages (cannot check time on message)
  #expect_equal(length(result_integrateFIR$messages), 5)
  #expect_equal(result_integrateFIR$messages[c(1,2,4)], expected_messages)
})

test_that('RANSAC rt correction function', {
  # Input
  tmp_targetFeatTable           <- data.frame(matrix(vector(), 5, 4, dimnames=list(c(), c("cpdID", "cpdName", "rt", "rt_dev_sec"))), stringsAsFactors=FALSE)
  tmp_targetFeatTable[1,]       <- c("ID-1", "Cpd 1", 11, 1.1)
  tmp_targetFeatTable[2,]       <- c("ID-2", "Cpd 2", 25, 1.5)
  tmp_targetFeatTable[3,]       <- c("ID-3", "Cpd 3", 35, 3.5)
  tmp_targetFeatTable[4,]       <- c("ID-4", "Cpd 4", 45, 5.5)
  tmp_targetFeatTable[5,]       <- c("ID-5", "Cpd 5", 55, 7)
  tmp_targetFeatTable[, c(3:4)]  <- sapply(tmp_targetFeatTable[, c(3:4)], as.numeric)

  tmp_referenceTable            <- data.frame(matrix(vector(), 5, 4, dimnames=list(c(), c("cpdID", "cpdName", "rt", "rt_dev_sec"))), stringsAsFactors=FALSE)
  tmp_referenceTable[1,]        <- c("Ref-1", "Ref Cpd 1", 11, 1.1)
  tmp_referenceTable[2,]        <- c("Ref-2", "Ref Cpd 2", 25, 1.5)
  tmp_referenceTable[3,]        <- c("Ref-3", "Ref Cpd 3", 35, 3.5)
  tmp_referenceTable[4,]        <- c("Ref-4", "Ref Cpd 4", 45, 5.5)
  tmp_referenceTable[5,]        <- c("Ref-5", "Ref Cpd 5", 55, 7)

  tmp_referenceTable[, c(3:4)]   <- sapply(tmp_referenceTable[, c(3:4)], as.numeric)
  # Expected model and results

  expected_corrected  <- data.frame(cpdID=character(), cpdName=character(), rt=double(), rt_dev_sec=double(), isReference=character(),
                                    correctedRt=double(), predictedRtDrift=double(), stringsAsFactors=FALSE)

  expected_corrected[1,]       <- list(cpdID="ID-1", cpdName="Cpd 1",  rt=11, rt_dev_sec=1.1, isReference="External set",   correctedRt=10.61282,  predictedRtDrift=0.387176)
  expected_corrected[2,]       <- list(cpdID="ID-2", cpdName="Cpd 2",  rt=25, rt_dev_sec=1.5, isReference="External set",   correctedRt=22.60164,  predictedRtDrift=2.398363)
  expected_corrected[3,]       <- list(cpdID="ID-3", cpdName="Cpd 3",  rt=35, rt_dev_sec=3.5, isReference="External set",  correctedRt= 31.16508,  predictedRtDrift=3.834925)
  expected_corrected[4,]       <- list(cpdID="ID-4", cpdName="Cpd 4",  rt=45, rt_dev_sec=5.5, isReference="External set",   correctedRt=39.72851,  predictedRtDrift=5.271487)
  expected_corrected[5,]       <- list(cpdID="ID-5", cpdName="Cpd 5",  rt=55, rt_dev_sec=7, isReference="External set",   correctedRt=48.29195,  predictedRtDrift=6.708049)

  expected_corrected_dg3                 <-  data.frame(cpdID=character(), cpdName=character(), rt=double(), rt_dev_sec=double(), isReference=character(),
                                    correctedRt=double(), predictedRtDrift=double(), stringsAsFactors=FALSE)
  expected_corrected_dg3[1,]       <- list(cpdID="ID-1", cpdName="Cpd 1",  rt=11, rt_dev_sec=1.1, isReference="External set",   correctedRt=9.907072,  predictedRtDrift=1.092928)
  expected_corrected_dg3[2,]       <- list(cpdID="ID-2", cpdName="Cpd 2",  rt=25, rt_dev_sec=1.5, isReference="External set",   correctedRt=23.457678,  predictedRtDrift=1.542322)
  expected_corrected_dg3[3,]       <- list(cpdID="ID-3", cpdName="Cpd 3",  rt=35, rt_dev_sec=3.5, isReference="External set",  correctedRt= 31.574063,  predictedRtDrift=3.425937)
  expected_corrected_dg3[4,]       <- list(cpdID="ID-4", cpdName="Cpd 4",  rt=45, rt_dev_sec=5.5, isReference="External set",   correctedRt=39.447720,  predictedRtDrift=5.552280)
  expected_corrected_dg3[5,]       <- list(cpdID="ID-5", cpdName="Cpd 5",  rt=55, rt_dev_sec=7, isReference="External set",   correctedRt=48.013466,  predictedRtDrift=6.986534)

   # results (output, warnings and messages)
  set.seed(19472)
  polynomial_ransac_dg1 <- evaluate_promise(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable, method='polynomial', params=list(polynomialOrder=1), robust=TRUE))
  set.seed(390753)
  polynomial_ransac_dg3 <- evaluate_promise(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable, method='polynomial', params=list(polynomialOrder=3), robust=TRUE))
  # Check results
  expect_equal(polynomial_ransac_dg1$result, expected_corrected, tolerance=1e-5)
  expect_equal(polynomial_ransac_dg3$result, expected_corrected_dg3, tolerance=1e-5)

  # Add outlier
  tmp_referenceTable_outlier            <- data.frame(matrix(vector(), 5, 4, dimnames=list(c(), c("cpdID", "cpdName", "rt", "rt_dev_sec"))), stringsAsFactors=FALSE)
  tmp_referenceTable_outlier[1,]        <- c("Ref-1", "Ref Cpd 1", 11, 1.1)
  tmp_referenceTable_outlier[2,]        <- c("ID-2", "Cpd 2", 25, 15)
  tmp_referenceTable_outlier[3,]        <- c("Ref-3", "Ref Cpd 3", 35, 3.5)
  tmp_referenceTable_outlier[4,]        <- c("Ref-4", "Ref Cpd 4", 45, 5.5)
  tmp_referenceTable_outlier[5,]        <- c("Ref-5", "Ref Cpd 5", 55, 7)

  tmp_referenceTable_outlier[, c(3:4)]   <- sapply(tmp_referenceTable_outlier[, c(3:4)], as.numeric)
  # Modify outlier in targetFeatTable
  tmp_targetFeatTable[2,]       <- c("ID-2", "Cpd 2", 25, 15)
  tmp_targetFeatTable[, c(3:4)]   <- sapply(tmp_targetFeatTable[, c(3:4)], as.numeric)


  expected_corrected_outlier                 <-  data.frame(cpdID=character(), cpdName=character(), rt=double(), rt_dev_sec=double(), isReference=character(),
                                    correctedRt=double(), predictedRtDrift=double(), stringsAsFactors=FALSE)
  expected_corrected_outlier[1,]       <- list(cpdID="ID-1", cpdName="Cpd 1",  rt=11, rt_dev_sec=1.1, isReference="External set",   correctedRt=10.14133,  predictedRtDrift=0.8586692)
  expected_corrected_outlier[2,]       <- list(cpdID="ID-2", cpdName="Cpd 2",  rt=25, rt_dev_sec=15.0, isReference="Reference set outlier", correctedRt=22.26570,  predictedRtDrift=2.7343018)
  expected_corrected_outlier[3,]       <- list(cpdID="ID-3", cpdName="Cpd 3",  rt=35, rt_dev_sec=3.5, isReference="External set",  correctedRt=30.92596,  predictedRtDrift=4.0740394)
  expected_corrected_outlier[4,]       <- list(cpdID="ID-4", cpdName="Cpd 4",  rt=45, rt_dev_sec=5.5, isReference="External set",   correctedRt=39.58622,  predictedRtDrift=5.4137769)
  expected_corrected_outlier[5,]       <- list(cpdID="ID-5", cpdName="Cpd 5",  rt=55, rt_dev_sec=7, isReference="External set",   correctedRt=48.24649,  predictedRtDrift=6.7535145)

  expected_corrected_outlier_dg3                 <-  data.frame(cpdID=character(), cpdName=character(), rt=double(), rt_dev_sec=double(), isReference=character(),
                                    correctedRt=double(), predictedRtDrift=double(), stringsAsFactors=FALSE)
  expected_corrected_outlier_dg3[1,]       <- list(cpdID="ID-1", cpdName="Cpd 1",  rt=11, rt_dev_sec=1.1, isReference="External set",   correctedRt= 9.90000,  predictedRtDrift=1.100000)
  expected_corrected_outlier_dg3[2,]       <- list(cpdID="ID-2", cpdName="Cpd 2",  rt=25, rt_dev_sec=15.0, isReference="Reference set outlier",   correctedRt=23.25802,  predictedRtDrift=1.741979)
  expected_corrected_outlier_dg3[3,]       <- list(cpdID="ID-3", cpdName="Cpd 3",  rt=35, rt_dev_sec=3.5, isReference="External set",  correctedRt=31.50000,  predictedRtDrift=3.500000)
  expected_corrected_outlier_dg3[4,]       <- list(cpdID="ID-4", cpdName="Cpd 4",  rt=45, rt_dev_sec=5.5, isReference="External set",   correctedRt=39.50000,  predictedRtDrift=5.500000)
  expected_corrected_outlier_dg3[5,]       <- list(cpdID="ID-5", cpdName="Cpd 5",  rt=55, rt_dev_sec=7, isReference="External set",   correctedRt=48.00000,  predictedRtDrift=7.000000)

  # Case where outlier is not detected
  expected_corrected_outlier_nd                 <-  data.frame(cpdID=character(), cpdName=character(), rt=double(), rt_dev_sec=double(), isReference=character(),
                                    correctedRt=double(), predictedRtDrift=double(), stringsAsFactors=FALSE)
  expected_corrected_outlier_nd[1,]       <- list(cpdID="ID-1", cpdName="Cpd 1",  rt=11, rt_dev_sec=1.1, isReference="External set",   correctedRt= 9.90000,  predictedRtDrift=1.100000)
  expected_corrected_outlier_nd[2,]       <- list(cpdID="ID-2", cpdName="Cpd 2",  rt=25, rt_dev_sec=15.0, isReference="Reference set",   correctedRt=10.00000,  predictedRtDrift=15.000000)
  expected_corrected_outlier_nd[3,]       <- list(cpdID="ID-3", cpdName="Cpd 3",  rt=35, rt_dev_sec=3.5, isReference="External set",  correctedRt=31.50000 ,  predictedRtDrift=3.500000)
  expected_corrected_outlier_nd[4,]       <- list(cpdID="ID-4", cpdName="Cpd 4",  rt=45, rt_dev_sec=5.5, isReference="External set",   correctedRt=50.23268,  predictedRtDrift=-5.232684)
  expected_corrected_outlier_nd[5,]       <- list(cpdID="ID-5", cpdName="Cpd 5",  rt=55, rt_dev_sec=7, isReference="External set",   correctedRt=48.00000,  predictedRtDrift=7.000000)

  set.seed(22142)
  polynomial_ransac_outlier <- evaluate_promise(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable_outlier, method='polynomial', params=list(polynomialOrder=1), robust=TRUE))
  set.seed(12149)
  polynomial_ransac_outlier_dg3 <- evaluate_promise(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable_outlier,
                                                                          method='polynomial', params=list(polynomialOrder=3), robust=TRUE))
  set.seed(4871414)
  polynomial_ransac_outlier_nd <- evaluate_promise(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable_outlier,
                                                                          method='polynomial', params=list(polynomialOrder=3), robust=TRUE))

  expect_equal(polynomial_ransac_outlier$result, expected_corrected_outlier, tolerance=1e-5)
  expect_equal(polynomial_ransac_outlier_dg3$result, expected_corrected_outlier_dg3, tolerance=1e-5)
  expect_equal(polynomial_ransac_outlier_nd$result, expected_corrected_outlier_nd, tolerance=1e-5)

})

test_that('constant rt correction', {
  # Input
  tmp_targetFeatTable           <- data.frame(matrix(vector(), 5, 4, dimnames=list(c(), c("cpdID", "cpdName", "rt", "rt_dev_sec"))), stringsAsFactors=FALSE)
  tmp_targetFeatTable[1,]       <- c("ID-1", "Cpd 1", 11, 1.1)
  tmp_targetFeatTable[2,]       <- c("ID-2", "Cpd 2", 25, 1.5)
  tmp_targetFeatTable[3,]       <- c("ID-3", "Cpd 3", 35, 3.5)
  tmp_targetFeatTable[4,]       <- c("ID-4", "Cpd 4", 45, 5.5)
  tmp_targetFeatTable[5,]       <- c("ID-5", "Cpd 5", 55, 7)
  tmp_targetFeatTable[, c(3:4)]  <- sapply(tmp_targetFeatTable[, c(3:4)], as.numeric)

  tmp_referenceTable            <- data.frame(matrix(vector(), 1, 4, dimnames=list(c(), c("cpdID", "cpdName", "rt", "rt_dev_sec"))), stringsAsFactors=FALSE)
  tmp_referenceTable[1,]        <- c("Ref-1", "Ref Cpd 1", 11, 1.1)

  tmp_referenceTable[, c(3:4)]   <- sapply(tmp_referenceTable[, c(3:4)], as.numeric)
  # Expected model and results

  expected_corrected  <- data.frame(cpdID=character(), cpdName=character(), rt=double(), rt_dev_sec=double(), isReference=character(),
                                    correctedRt=double(), predictedRtDrift=double(), stringsAsFactors=FALSE)

  expected_corrected[1,]       <- list(cpdID="ID-1", cpdName="Cpd 1",  rt=11, rt_dev_sec=1.1, isReference="External set",   correctedRt=9.9,  predictedRtDrift=1.1)
  expected_corrected[2,]       <- list(cpdID="ID-2", cpdName="Cpd 2",  rt=25, rt_dev_sec=1.5, isReference="External set",   correctedRt=23.9,  predictedRtDrift=1.1)
  expected_corrected[3,]       <- list(cpdID="ID-3", cpdName="Cpd 3",  rt=35, rt_dev_sec=3.5, isReference="External set",  correctedRt=33.9,  predictedRtDrift=1.1)
  expected_corrected[4,]       <- list(cpdID="ID-4", cpdName="Cpd 4",  rt=45, rt_dev_sec=5.5, isReference="External set",   correctedRt=43.9,  predictedRtDrift=1.1)
  expected_corrected[5,]       <- list(cpdID="ID-5", cpdName="Cpd 5",  rt=55, rt_dev_sec=7, isReference="External set",   correctedRt=53.9,  predictedRtDrift=1.1)

  constant_correction <- evaluate_promise(peakPantheR_applyRTCorrection(targetFeatTable=tmp_targetFeatTable, referenceTable=tmp_referenceTable, method='constant', params=list(polynomialOrder=1), robust=FALSE))

  expect_equal(constant_correction$result, expected_corrected, tolerance=1e-5)
})

## Results Tab -----------------------------------------------------------------

# - message if no import (conditional panel)
# - status of the annotation (and failures) on the sidebar

# - [TAB] show overall results (peakTable, focus on one property at the time)
# - [TAB] show the values per compound
# - [TAB] show the values per sample

## not annotated
output$notAnnotForResUI <- renderUI ({
  if(runSuccess()=='yes') return()
  tagList(
    HTML("<div class=\"alert alert-dismissible alert-danger\"> <button type=\"button\" class=\"close\" data-dismiss=\"alert\">×</button> <h4 style=\"font-weight:bold\">No annotation computed</h4>Annotate a <i>peakPantheRAnnotation</i></div>")#,
    #includeHTML("data/aboutResults.html")
  )
})


## show the status of the peakPantheRAnnotation (need to be separated from the previous panel so they can be hidden)
output$showAnnotStatusRes <- renderUI({
  # Capture the annotation shown and split by line into a list
  tmp_text <- annotation_showText_UI_helper(annotation_showMethod_UI_helper(values$annotation))
  fail_text <- paste(dim(values$failures)[1], 'annotation failure(s)')
  # render the panel
  wellPanel(
    h4('Status:', style="color:#3e648d;font-weight:bold"),
    helpText(tmp_text[[1]],style="color:black"),
    helpText(tmp_text[[2]],style="color:black"),
    helpText(tmp_text[[3]],style="color:black"),
    helpText(tmp_text[[4]],style="color:black"),
    helpText(tmp_text[[5]],style="color:black"),
    helpText(tmp_text[[6]],style="color:black"),
    tags$hr(),
    if (!is.null(values$failures)) {
      helpText(fail_text,style="color:black")
    }
  )
})


## Overall results -------------------------------------------------------------
# UI to control the peakTable
peakTableCol <- c('peakArea','found','rtMin','rt','rtMax','mzMin','mz','mzMax','maxIntMeasured','maxIntPredicted','is_filled','ppm_error','rt_dev_sec','tailingFactor','asymmetryFactor')
output$peakTableControlUI <- renderUI({
  wellPanel(
    fluidRow(
      column(12, offset=1,
        h4('Key fitting properties across all targeted features and samples:', style="color:#3e648d;font-weight:bold"),
      ) # end column
    ),  # end fluidRow
    fluidRow(
      column(6, offset=1,
        selectInput("peakTableColRes", label="Property to show", choices=peakTableCol)
      ) # end column
    )   # end fluidRow
  )     # end wellPanel
})

# render the peakTable
output$table_overall <- DT::renderDT ({
  DT::datatable(data = peakPantheR::annotationTable(values$annotation, column=input$peakTableColRes),
                options  = list(orderClasses = TRUE),
                rownames = TRUE)
})

# UI block peakTable
output$overallResultsUI <- renderUI ({
  fluidRow(
    column(width = 12, offset = 0,
      DT::DTOutput("table_overall")
    )
  )
})


## Results by Features ---------------------------------------------------------
# UI to control the results per sample
output$resPerFeatControlUI <- renderUI({
  wellPanel(
    fluidRow(
      column(12, offset=1,
        h4('Fitting results per feature:', style="color:#3e648d;font-weight:bold"),
      ) # end column
    ),  # end fluidRow
    fluidRow(
      column(6, offset=1,
        selectInput("featResToShow", label="Feature", choices=unname(values$featNmeList))
      ) # end column
    )   # end fluidRow
  )     # end wellPanel
})

# render the results per sample (filter based on filename)
output$feat_result_table <- DT::renderDT ({
  tmp_annotation         <- values$annotation[, values$featNmeList == input$featResToShow]
  tmp_feat_res           <- do.call(rbind, peakTables(tmp_annotation))
  rownames(tmp_feat_res) <- values$filename
  DT::datatable(data = tmp_feat_res,
                options  = list(orderClasses = TRUE),
                rownames = TRUE)
})

# UI block sample results
output$featureResultsUI <- renderUI ({
  fluidRow(
    column(width = 12, offset = 0,
      DT::DTOutput("feat_result_table")
    )
  )
})


## Results by Samples ----------------------------------------------------------
# UI to control the results per sample
output$resPerSplControlUI <- renderUI({
  wellPanel(
    fluidRow(
      column(12, offset=1,
        h4('Fitting results per sample:', style="color:#3e648d;font-weight:bold"),
      ) # end column
    ),  # end fluidRow
    fluidRow(
      column(6, offset=1,
        selectInput("splResToShow", label="Sample", choices=values$filename)
      ) # end column
    )   # end fluidRow
  )     # end wellPanel
})

# render the results per sample (filter based on filename)
output$spl_result_table <- DT::renderDT ({
  tmp_annotation        <- values$annotation[values$filename == input$splResToShow, ]
  tmp_spl_res           <- peakTables(tmp_annotation)[[1]]
  rownames(tmp_spl_res) <- values$featNmeList
  DT::datatable(data = tmp_spl_res,
                options  = list(orderClasses = TRUE),
                rownames = TRUE)
})

# UI block sample results
output$sampleResultsUI <- renderUI ({
  fluidRow(
    column(width = 12, offset = 0,
      DT::DTOutput("spl_result_table")
    )
  )
})

## Export Tab ------------------------------------------------------------------

# - export files to recreate input
#   - RData
#   - the different csv (with metadata)
# - export results (tables to disk)
# - export plot to disk


## not annotated
output$noImportForExportUI <- renderUI ({
  if(importSuccess()=='yes') return()
  tagList(
    HTML("<div class=\"alert alert-dismissible alert-danger\"> <button type=\"button\" class=\"close\" data-dismiss=\"alert\">×</button> <h4 style=\"font-weight:bold\">No annotation imported</h4>Create or load a <i>peakPantheRAnnotation</i></div>")#,
    #includeHTML("data/aboutExport.html")
  )
})


## show the status of the peakPantheRAnnotation (need to be separated from the previous panel so they can be hidden)
output$showAnnotStatusExp <- renderUI({
  # Capture the annotation shown and split by line into a list
  tmp_text  <- annotation_showText_UI_helper(annotation_showMethod_UI_helper(values$annotation))
  fail_text <- paste(dim(values$failures)[1], 'annotation failure(s)')
  # render the panel
  wellPanel(
    h4('Status:', style="color:#3e648d;font-weight:bold"),
    helpText(tmp_text[[1]],style="color:black"),
    helpText(tmp_text[[2]],style="color:black"),
    helpText(tmp_text[[3]],style="color:black"),
    helpText(tmp_text[[4]],style="color:black"),
    helpText(tmp_text[[5]],style="color:black"),
    helpText(tmp_text[[6]],style="color:black"),
    tags$hr(),
    if (!is.null(values$failures)) {
      helpText(fail_text,style="color:black")
    }
  )
})


# -- Export input files and annotation
# export peakPantheRAnnotation as .rData
output$downloadRData <- downloadHandler(
    # filename
    filename = function() { paste0(input$saveNameRData, '.rdata') },
    # data to save
    content = function(file) {
        annotationObject <- values$annotation
        save( annotationObject, file=file, compress=TRUE)
    }
)
output$dlRData <- renderUI({
  tagList(
      textInput("saveNameRData", label = ".rData name", value="annotation_rData"),
      div(
        downloadButton('downloadRData',"Download", class="btn btn-primary btn-lg"),
        align="center"
      )
  )
})

# export inputs as CSVs
output$downloadCSVFitParam <- downloadHandler(
    # filename
    filename = function() { paste0(input$saveNameCSVFitParam, '.csv') },
    # data to save
    content = function(file) {
        write.csv(outputAnnotationParamsCSV(values$annotation, saveFolder=NULL, verbose=FALSE, noSave=TRUE), file, row.names=FALSE)
    }
)
output$downloadCSVSpecMeta <- downloadHandler(
    # filename
    filename = function() { paste0(input$saveNameCSVSpecMeta, '.csv') },
    # data to save
    content = function(file) {
        write.csv(outputAnnotationSpectraMetadata_UI_helper(values$annotation), file, row.names=FALSE)
    }
)
output$downloadCSVFeatMeta <- downloadHandler(
    # filename
    filename = function() { paste0(input$saveNameCSVFeatMeta, '.csv') },
    # data to save
    content = function(file) {
        write.csv(outputAnnotationFeatureMetadata_UI_helper(values$annotation), file, row.names=FALSE)
    }
)
output$dlCSVs <- renderUI({
  tagList(
    fluidRow(
    column(6,textInput("saveNameCSVFitParam", label = "Fit parameters", value="fit_parameters")),
    column(6,HTML("<br/>"),downloadButton('downloadCSVFitParam',label=HTML("Download<br/>Fit parameters"), class="btn btn-primary btn-sm"))
    ),
    fluidRow(
    column(6,textInput("saveNameCSVSpecMeta", label = "Files to process", value="files_to_process")),
    column(6,HTML("<br/>"),downloadButton('downloadCSVSpecMeta',label=HTML("Download<br/>Files to process"), class="btn btn-primary btn-sm"))
    ),
    fluidRow(
    column(6,textInput("saveNameCSVFeatMeta", label = "Features metadata", value="features_metadata")),
    column(6,HTML("<br/>"),downloadButton('downloadCSVFeatMeta',label=HTML("Download<br/>Features metadata"), class="btn btn-primary btn-sm"))
    )
  )
})




# -- Export results and diagnostic plots

# UI block for diagnostic and result export only exist if isAnnotated()
output$exportDiagResUI <- renderUI({
  ## not annotated
  if(runSuccess()!='yes') {
    return(
      tagList(
        HTML("<div class=\"alert alert-dismissible alert-danger\"> <button type=\"button\" class=\"close\" data-dismiss=\"alert\">×</button> <h4 style=\"font-weight:bold\">No annotation computed</h4>Annotate a <i>peakPantheRAnnotation</i></div>")#,
      )
    )
  }

  ## is annotated only
  tagList(
    fluidRow(

    # save diagnostic and result control UI
      column(6,
        uiOutput("controlExportPlotUI")
      ),
      column(6,
        uiOutput("controlExportCSVUI")
      ),

    # successful Save UI
      column(6,
        uiOutput("successSavePlot")
      ),
      column(6,
        uiOutput("successSaveCSV")
      ),

    # save targetFolder UI
      column(12,
        div(
          textInput("targetFolderSave", label="Target Folder", value="C:/result_ppR_annotation"),
          helpText("Select a target folder with write permission", style="color:#666666;font-style:italic"),
          align="center"
        )
      ), # end column targetFolder UI

    # button save Diagnostic plots
      column(6,
        div(
          actionButton("save_plot",
            label="Save diagnostic plots",
            class = "btn btn-primary btn-lg"
          ),
          align="center"
        ) # end div
      ),  # end column button save diagnostic plots

    # button save results csv
      column(6,
        div(
          actionButton("save_resultCSV",
            label="Save results",
            class = "btn btn-primary btn-lg"
          ),
          align="center"
        ) # end div
      ) # end column button save results csv
    ) # end fluidRow
  ) # end tagList
})


# UI for diagnostic plots
output$controlExportPlotUI <- renderUI ({
  tagList(
    wellPanel(
      fluidRow(
        column(12,
          h5(HTML(paste0("Number of feature diagnostics to plot: ", nbCompounds(values$annotation))), style="color:#3e648d;font-weight:bold"),
          tags$hr()
        ),
        column(12,
          selectInput("plotMetaSplColrDiag", label="Colour by Sample Metadata", choices=values$spectraMetadataCol)
        ),
        column(12,
          checkboxInput("parallelisationDiag",
            label = p("Parallelisation", style="color:#3e648d;font-weight:bold"),
            value = FALSE
          ),
          uiOutput("cpuSliderDiag")
        )
      ) # end fluidRow
    )  # end wellPanel real Fig ui
  ) # end tagList
})
# Cpu slider appears if parallelisation is selected
output$cpuSliderDiag <- renderUI({
  if(input$parallelisationDiag == 0) return(NULL)
  tagList(
    sliderInput("ncoresDiag", label = paste("Available cores: ",maxCores,sep=""), min = 0, max = maxCores, value = maxCores, step=1)
  )
})
# correction when slider doesn't appear
ncoresInputPlot <- reactive ({
  if( input$parallelisationDiag != 0 ) { input$ncoresDiag }
  else { return(0) }
})

# UI for results CSV
output$controlExportCSVUI <- renderUI ({
  tagList(
    wellPanel(
      fluidRow(
        column(6,
            textInput("projectName", label="Project name", value="ppR-project")
        ), 	# end column
        column(6,
          p("_summary.csv"),
          p("_cpdMetadata.csv"),
          p("_spectraMetadata.csv"),
          p("_found.csv"),
          p("_peakArea.csv"),
          p("_ppm_error.csv"),
          p("...")
        )
      ) # end fluidRow
    )  # end wellPanel
  ) # end tagList
})


## Save diagnostic plots
savePlotOnTrigger <- eventReactive( input$save_plot, {
  outputAnnotationDiagnostic(values$annotation,
                             saveFolder = input$targetFolderSave,
                             savePlots = TRUE,
                             sampleColour = spectra_metadata_colourScheme_UI_helper(values$annotation, input$plotMetaSplColrDiag),
                             ncores = ncoresInputPlot(),
                             verbose = TRUE)
  return(list('done'))
})
## Save result csv
saveResOnTrigger <- eventReactive( input$save_resultCSV, {
  outputAnnotationResult(values$annotation,
                         saveFolder = input$targetFolderSave,
                         annotationName = input$projectName,
                         verbose = TRUE)
  return(list('done'))
})

# If save Plot is successful
output$successSavePlot <- renderUI({
  if( !is.null(savePlotOnTrigger()) ){
    if( is.list(savePlotOnTrigger()) ){
      return(
        HTML("<div class=\"alert alert-dismissible alert-success\"> <button type=\"button\" class=\"close\" data-dismiss=\"alert\">×</button><h4>Success</h4><p>Plots saved to disk.</p></div>")
      )
    }
  }
  return(NULL)
})
# If save Plot is successful
output$successSaveCSV <- renderUI({
  if( !is.null(saveResOnTrigger()) ){
    if( is.list(saveResOnTrigger()) ){
      return(
        HTML("<div class=\"alert alert-dismissible alert-success\"> <button type=\"button\" class=\"close\" data-dismiss=\"alert\">×</button><h4>Success</h4><p>Results saved to disk.</p></div>")
      )
    }
  }
  return(NULL)
})

#' @title Calculate chromatographic peak properties
#'
#' @description Calculate the ppm error, retention time deviation, tailing
#' factor and asymmetry factor for each measured feature.
#'
#' @param fittedCurves (list) A list (one entry per ROI window) of
#' \code{peakPantheR_curveFit} or NA
#' @param targetFeatTable a \code{\link{data.frame}} of compounds to target as
#' rows. Columns: \code{cpdID} (str), \code{cpdName} (str), \code{rtMin} (float
#' in seconds), \code{rt} (float in seconds, or \emph{NA}), \code{rtMax} (float
#' in seconds), \code{mzMin} (float), \code{mz} (float or \emph{NA}),
#' \code{mzMax} (float).
#' @param foundPeakTable a \code{data.frame} as generated by
#' \code{\link{findTargetFeatures}}, with features as rows and peak properties
#' as columns. The following columns are mandatory: \code{mzMin}, \code{mz},
#' \code{mzMax}, \code{rtMin}, \code{rt}, \code{rtMax}
#' @param verbose (bool) if TRUE message when NA scans are removed
#'
#' @return A \code{data.frame} with measured compounds as rows and measurements
#' and properties as columns (see Details).
#'
#' \subsection{Details:}{
#' The returned \code{data.frame} is structured as follow:
#' \tabular{ll}{
#' found \tab was the peak found\cr
#' rt \tab retention time of peak apex (sec)\cr
#' rtMin \tab leading edge of peak retention time (sec) determined at 0.5\% of
#' apex intensity\cr
#' rtMax \tab trailing edge of peak retention time (sec) determined at 0.5\% of
#' apex intensity\cr
#' mz \tab weighted (by intensity) mean of peak m/z across scans\cr
#' mzMin \tab m/z peak minimum (between rtMin, rtMax)\cr
#' mzMax \tab m/z peak maximum (between rtMin, rtMax)\cr
#' peakArea \tab integrated peak area\cr
#' peakAreaRaw \tab integrated peak area from raw data points\cr
#' maxIntMeasured \tab maximum peak intensity in raw data\cr
#' maxIntPredicted \tab maximum peak intensity based on curve fit\cr
#' ppm_error \tab difference in ppm between the expected and measured m/z\cr
#' rt_dev_sec \tab difference in seconds between the expected and measured rt\cr
#' tailingFactor \tab the tailing factor is a measure of peak tailing. It is
#' defined as the distance from the front slope of the peak to the back slope
#' divided by twice the distance from the center line of the peak to the front
#' slope, with all measurements made at 5\% of the maximum peak height. The
#' tailing factor of a peak will typically be similar to the asymmetry factor
#' for the same peak, but the two values cannot be directly converted\cr
#' asymmetryFactor \tab the asymmetry factor is a measure of peak tailing. It is
#' defined as the distance from the center line of the peak to the back slope
#' divided by the distance from the center line of the peak to the front slope,
#' with all measurements made at 10\% of the maximum peak height. The asymmetry
#' factor of a peak will typically be similar to the tailing factor for the same
#' peak, but the two values cannot be directly converted\cr
#' }
#' }
#'
#' @details
#' ## Examples cannot be computed as the function is not exported:
#' # fittedCurve
#' cFit1 <- list(amplitude=162404.8057918259, center=3341.888,
#'                 sigma=0.078786133031045896, gamma=0.0018336101984172684,
#'                 fitStatus=2, curveModel='skewedGaussian')
#' class(cFit1) <- 'peakPantheR_curveFit'
#' cFit2        <- list(amplitude=199249.10572753669, center=3382.577,
#'                     sigma=0.074904415304607966, gamma=0.0011471899372353885,
#'                     fitStatus=2, curveModel='skewedGaussian')
#' class(cFit2)    <- 'peakPantheR_curveFit'
#' input_fitCurves <- list(cFit1, cFit2)
#' 
#' # ROI
#' input_ROI <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), c('cpdID',
#'                 'cpdName', 'rtMin', 'rt', 'rtMax', 'mzMin', 'mz', 'mzMax'))),
#'                 stringsAsFactors=FALSE)
#' input_ROI[1,] <- c('ID-1', 'testCpd 1', 3310., 3344.88, 3390., 522.19, 522.2,
#'                     522.21)
#' input_ROI[2,] <- c('ID-2', 'testCpd 2', 3280., 3385.58, 3440., 496.19, 496.2,
#'                     496.21)
#' input_ROI[,3:8] <- vapply(input_ROI[,3:8], as.numeric, FUN.VALUE=numeric(2))
#'
#' # foundPeakTable
#' input_foundPeakTable <- data.frame(matrix(vector(), 2, 10, dimnames=list(c(),
#'                     c('found', 'rtMin', 'rt', 'rtMax', 'mzMin', 'mz',
#'                     'mzMax','peakArea','maxIntMeasured','maxIntPredicted'))),
#'                     stringsAsFactors=FALSE)
#' input_foundPeakTable[1,]  <- c(TRUE, 3309.758, 3346.827, 3385.410, 522.19,
#'                                 522.2, 522.21, 26133726, 889280, 901015)
#' input_foundPeakTable[2,]  <- c(TRUE, 3345.376, 3386.529, 3428.279, 496.19,
#'                                 496.2, 496.21, 35472141, 1128960, 1113576)
#' input_foundPeakTable[,1]  <- vapply(input_foundPeakTable[,c(1)], as.logical,
#'                                 FUN.VALUE=logical(1))
#'
#' # Run peak statistics
#' peakStatistics <- getTargetFeatureStatistic(input_fitCurves, input_ROI,
#'                                             input_foundPeakTable)
#' peakStatistics   
#' #   found    rtMin       rt    rtMax  mzMin    mz  mzMax peakArea
#' # 1  TRUE 3309.758 3346.827 3385.410 522.19 522.2 522.21 26133726
#' # 2  TRUE 3345.376 3386.529 3428.279 496.19 496.2 496.21 35472141
#' #    maxIntMeasured maxIntPredicted ppm_error rt_dev_sec tailingFactor
#' # 1          889280          901015         0      1.947      1.015385
#' # 2         1128960         1113576         0      0.949      1.005372
#' #    asymmetryFactor
#' # 1         1.026886
#' # 2         1.009304
getTargetFeatureStatistic <- function(fittedCurves, targetFeatTable,
                                        foundPeakTable, verbose = FALSE) {
    stime <- Sys.time()
    # Check input
    if (dim(targetFeatTable)[1] != dim(foundPeakTable)[1]) {
        stop("Number of features in \"targetFeatTable\" (",
            dim(targetFeatTable)[1], ") and \"foundPeakTable\" (",
            dim(foundPeakTable)[1], ") do not match!") }
    if (length(fittedCurves) != dim(foundPeakTable)[1]) {
        stop("Number of fitted curves in \"fittedCurves\" (",
            length(fittedCurves),
            ") and number of features in \"foundPeakTable\" (",
            dim(foundPeakTable)[1], ") do not match!") }
    # Calculate the statistics
    peakStat <- data.frame(matrix(vector(), dim(targetFeatTable)[1], 4,
                            dimnames = list(c(), c("ppm_error", "rt_dev_sec",
                                                    "tailingFactor",
                                                    "asymmetryFactor"))),
                            stringsAsFactors = FALSE)
    # only work with found features
    for (i in seq_len(dim(targetFeatTable)[1])) {
        if (foundPeakTable$found[i]) {
            # ppm_error
            if (!is.na(targetFeatTable$mz[i])) {
                peakStat$ppm_error[i] <- (abs(foundPeakTable$mz[i] -
                        targetFeatTable$mz[i])/targetFeatTable$mz[i]) * 1e+06 }
            # rt_dev_sec
            if (!is.na(targetFeatTable$rt[i])) {
                peakStat$rt_dev_sec[i] <- foundPeakTable$rt[i] -
                        targetFeatTable$rt[i] }
            # Tailing Factor
            peakStat$tailingFactor[i] <- peak_shape_stat(fittedCurves[[i]],
                                                    foundPeakTable$rt[i],
                                                    foundPeakTable$rtMin[i],
                                                    foundPeakTable$rtMax[i],
                                                    statistic = "tailingFactor")
            # Asymmetry Factor
            peakStat$asymmetryFactor[i] <- peak_shape_stat(fittedCurves[[i]],
                                                foundPeakTable$rt[i],
                                                foundPeakTable$rtMin[i],
                                                foundPeakTable$rtMax[i],
                                                statistic = "asymmetryFactor")}}
    # group the results and return
    finalTable <- cbind.data.frame(foundPeakTable, peakStat)
    etime <- Sys.time()
    if (verbose) { message("Peak statistics done in: ",
                            round(as.double(difftime(etime, stime)), 2),
                            " ", units(difftime(etime, stime))) }
    return(finalTable)
}


# -----------------------------------------------------------------------------
# getTargetFeatureStatistic helper functions

## Define the peak_shape_stat()
#
# Calculate tailing factor and asymmetry factor
#
# Tailing Factor and Asymmetry Factor following the equations from
# http://www.chromforum.org/viewtopic.php?t=20079
#
# @param fittedCurve A \code{peakPantheR_curveFit} for the feature of
#   interest or NA
# @param apexRT (float) retention time in seconds of the measured peak apex
# @param rtMin (float) leading edge retention time in seconds (at 0.5%)
# @param rtMax (float) tailing edge retention time in seconds (at 0.5%)
# @param statistic (str) either \code{tailingFactor} or
#   \code{asymmetryFactor}
# @param sampling (int) Number of points to employ when subsampling the
#   fittedCurve between rtMin and rtMax @return Tailing factor or Asymmetry
#   factor value
peak_shape_stat <- function(fittedCurve, apexRT, rtMin, rtMax,
                            statistic = "tailingFactor", sampling = 250) {
    # Check input
    if (is.na(apexRT)) { return(as.numeric(NA)) }
    if (is.na(rtMin)) { return(as.numeric(NA)) }
    if (is.na(rtMax)) { return(as.numeric(NA)) }
    if (all(is.na(fittedCurve))) { return(as.numeric(NA)) }

    # A B and C are retention time in seconds. C is RT of the apex, A is
    # left side at x%, B is right side at x%.
    # x% (cutoff) depends of the statistic employed.
    ## Find position of A and B
    cutoff <- switch(statistic, tailingFactor = 0.05, asymmetryFactor = 0.1)
    cutoff_int <- cutoff * predictCurve(fittedCurve, x = apexRT)
        # intensity at cutoff pt of interest
    ## A side
    # reverse order for up slope
    A_grid_rt   <- seq(from = apexRT, to = rtMin,
                        by = ((rtMin - apexRT)/(sampling - 1)))
    A_slope_int <- predictCurve(fittedCurve, x = A_grid_rt)
    # pos of 1st point past cutoff
    A_cutoff_pt <- match(-1, sign(A_slope_int - cutoff_int))
    # should not be NA as rtMin Max is at 0.5% while cutoff is 5 or 10%
    if (is.na(A_cutoff_pt)) { return(NA) }
    # points left and right from pt of interest
    A_key_pt    <- c(A_cutoff_pt - 1, A_cutoff_pt)
    # linear interpolation of exact rt
    A           <- stats::approx(x = A_slope_int[A_key_pt],
                                y = A_grid_rt[A_key_pt],
                                xout = cutoff_int)$y
    ## B side
    B_grid_rt   <- seq(from = apexRT, to = rtMax,
                        by = ((rtMax - apexRT)/(sampling - 1)))
    B_slope_int <- predictCurve(fittedCurve, x = B_grid_rt)
    # pos of 1st point past cutoff
    B_cutoff_pt <- match(-1, sign(B_slope_int - cutoff_int))
    # should not be NA as rtMin Max is at 0.5% while cutoff is 5 or 10%
    if (is.na(B_cutoff_pt)) { return(NA) }
    # points left and right from pt of interest
    B_key_pt    <- c(B_cutoff_pt - 1, B_cutoff_pt)
    # linear interpolation of exact rt
    B           <- stats::approx(x = B_slope_int[B_key_pt],
                                y = B_grid_rt[B_key_pt],
                                xout = cutoff_int)$y
    ## C is apexRT
    C <- apexRT
    ## Peak statistic
    if (statistic == "tailingFactor") { return((B - A)/(2 * (C - A))) }
    if (statistic == "asymmetryFactor") { return((B - C)/(C - A)) }
}

context('getTargetFeatureStatistic()')

skip_if_not_installed('faahKO',  minimum_version = '1.18.0')
library(faahKO)


## Input and expected data
# use ko15.CDf file from the pkg faahKO

# fitted curves
cFit1           <- list(amplitude=162404.8057918259, center=3341.888, sigma=0.078786133031045896, gamma=0.0018336101984172684, fitStatus=2, curveModel="skewedGaussian")
class(cFit1)    <- 'peakPantheR_curveFit'
cFit2           <- list(amplitude=199249.10572753669, center=3382.577, sigma=0.074904415304607966, gamma=0.0011471899372353885, fitStatus=2, curveModel="skewedGaussian")
class(cFit2)    <- 'peakPantheR_curveFit'
cFit3           <- list(amplitude=31645.961277502651, center=3451.435, sigma=0.064803553287811053, gamma=2.8557893789555022, fitStatus=2, curveModel="skewedGaussian")
class(cFit3)    <- 'peakPantheR_curveFit'
cFit4           <- list(amplitude=59193.591103772116, center=3698.697, sigma=0.082789238806238355, gamma=0.0026044299691057823, fitStatus=2, curveModel="skewedGaussian")
class(cFit4)    <- 'peakPantheR_curveFit'
input_fitCurves <- list(cFit1, cFit2, cFit3, cFit4)
 
# ROI
input_ROI       <- data.frame(matrix(vector(), 4, 8, dimnames=list(c(), c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax"))),stringsAsFactors=FALSE)
input_ROI[1,] 	<- c("ID-1", "testCpd 1", 3310., 3344.888, 3390., 522.194778, 522.2, 522.205222)
input_ROI[2,] 	<- c("ID-2", "testCpd 2", 3280., 3385.577, 3440., 496.195038, 496.2, 496.204962)
input_ROI[3,] 	<- c("ID-3", "testCpd 3", 3420., 3454.435, 3495., 464.195358, 464.2, 464.204642)
input_ROI[4,] 	<- c("ID-4", "testCpd 4", 3670., 3701.697, 3745., 536.194638, 536.2, 536.205362)
input_ROI[,3:8] <- sapply(input_ROI[,3:8], as.numeric)

# foundPeakTable
input_foundPeakTable      <- data.frame(matrix(vector(), 4, 10, dimnames=list(c(), c("found", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax", "peakArea", "maxIntMeasured", "maxIntPredicted"))),stringsAsFactors=FALSE)
input_foundPeakTable[1,]  <- c(TRUE, 3309.7589296586070, 3346.8277590361445, 3385.4098874628098, 522.194778, 522.20001220703125, 522.205222, 26133726.6811244078, 889280, 901015.80529226747)
input_foundPeakTable[2,]  <- c(TRUE, 3345.3766648628907, 3386.5288072289159, 3428.2788374983961, 496.20001220703125, 496.20001220703125, 496.20001220703125, 35472141.3330242932, 1128960, 1113576.69008227298)
input_foundPeakTable[3,]  <- c(TRUE, 3451.2075903614455, 3451.5072891566265, 3501.6697504924518, 464.195358, 464.20001220703125, 464.204642, 7498427.1583901159, 380736, 389632.13549519412)
input_foundPeakTable[4,]  <- c(TRUE, 3670.9201232710743, 3704.1427831325304, 3740.0172511251831, 536.20001220703125, 536.20001220703125, 536.20001220703125, 8626279.9788195733, 330176, 326763.87246511364)
input_foundPeakTable[,1]  <- vapply(input_foundPeakTable[,c(1)], as.logical, FUN.VALUE=logical(1))

# peakStatistics
peakStatistics                  <- input_foundPeakTable
peakStatistics$ppm_error        <- c(0.023376160866574614, 0.024601030353423384, 0.026296922148575364, 0.022765817240815486)
peakStatistics$rt_dev_sec       <- c(1.93975903614455092, 0.95180722891564074, -2.92771084337346110, 2.44578313253032320)
peakStatistics$tailingFactor    <- c(1.0153573486330891, 1.0053782620427065, 207.6939219686769036, 1.0305289730128095)
peakStatistics$asymmetryFactor  <- c(1.0268238825675249, 1.0093180792278085, 380.5019028782010082, 1.0536948855480386)


test_that('no verbose', {
  # expected foundPeaks
  expected_featureStatistic <- peakStatistics

	# results (output, warnings and messages)
  result_featureStatistic   <- evaluate_promise(getTargetFeatureStatistic(input_fitCurves, input_ROI, input_foundPeakTable, verbose=FALSE))

  # Check result table
  expect_equal(result_featureStatistic$result, expected_featureStatistic)

  # Check no result messages
  expect_equal(length(result_featureStatistic$messages), 0)
})

test_that('verbose', {
  # expected foundPeaks
  expected_featureStatistic <- peakStatistics
  
  # results (output, warnings and messages)
  result_featureStatistic   <- evaluate_promise(getTargetFeatureStatistic(input_fitCurves, input_ROI, input_foundPeakTable, verbose=TRUE))
  
  # Check result table
  expect_equal(result_featureStatistic$result, expected_featureStatistic)
  
  # Check messages (cannot match timing)
  expect_equal(length(result_featureStatistic$messages), 1)
})

test_that('no rtMin (#1), no rtMax (#2), no fittedCurve (#3), no apexRT (#4)', {
  # foundPeakTable
  tmp_foundPeakTable              <- input_foundPeakTable
  tmp_foundPeakTable[1, 'rtMin']  <- NA
  tmp_foundPeakTable[2, 'rtMax']  <- NA
  tmp_foundPeakTable[4, 'rt']     <- NA
  
  # curveFit
  tmp_fitCurves      <- input_fitCurves
  tmp_fitCurves[[3]] <- NA
  
  # expected foundPeaks
  expected_featureStatistic                 <- peakStatistics
  expected_featureStatistic$rtMin[1]        <- as.numeric(NA)
  expected_featureStatistic$rtMax[2]        <- as.numeric(NA)
  expected_featureStatistic$rt[4]           <- as.numeric(NA)
  expected_featureStatistic$tailingFactor   <- as.numeric(NA)
  expected_featureStatistic$asymmetryFactor <- as.numeric(NA)
  expected_featureStatistic$rt_dev_sec[4]   <- as.numeric(NA)
  
  # results (output, warnings and messages)
  result_featureStatistic   <- evaluate_promise(getTargetFeatureStatistic(tmp_fitCurves, input_ROI, tmp_foundPeakTable, verbose=TRUE))
  
  # Check result table
  expect_equal(result_featureStatistic$result, expected_featureStatistic)
  
  # Check messages (cannot match timing)
  expect_equal(length(result_featureStatistic$messages), 1)
})

test_that('no rt (#2), no mz (#3)', {
  # foundPeakTable
  tmp_ROI           <- input_ROI
  tmp_ROI[2, 'rt']  <- NA
  tmp_ROI[3, 'mz']  <- NA

  # expected foundPeaks
  expected_featureStatistic               <- peakStatistics
  expected_featureStatistic$rt_dev_sec[2] <- as.numeric(NA)
  expected_featureStatistic$ppm_error[3]  <- as.numeric(NA)
  
  # results (output, warnings and messages)
  result_featureStatistic   <- evaluate_promise(getTargetFeatureStatistic(input_fitCurves, tmp_ROI, input_foundPeakTable, verbose=TRUE))
  
  # Check result table
  expect_equal(result_featureStatistic$result, expected_featureStatistic)
  
  # Check messages (cannot match timing)
  expect_equal(length(result_featureStatistic$messages), 1)
})

test_that('wrong rtMin (#2), wrong rtMax (#3)', {
  # if rtMin > 5/10% and rtMax > 5/10% maxInt, cannot calculate statistic. (Shouldn't happen as rtMin rtMax are at 0.5% maxInt)
  # foundPeakTable
  tmp_foundPeakTable             <- input_foundPeakTable
  tmp_foundPeakTable[2, 'rtMin'] <- 3380.
  tmp_foundPeakTable[3, 'rtMax'] <- 3460.
  
  # expected foundPeaks
  expected_featureStatistic                     <- peakStatistics
  expected_featureStatistic$rtMin[2]            <- 3380.
  expected_featureStatistic$rtMax[3]            <- 3460.
  expected_featureStatistic$tailingFactor[2]    <- as.numeric(NA)
  expected_featureStatistic$asymmetryFactor[2]  <- as.numeric(NA)
  expected_featureStatistic$tailingFactor[3]    <- as.numeric(NA)
  expected_featureStatistic$asymmetryFactor[3]  <- as.numeric(NA)
  
  # results (output, warnings and messages)
  result_featureStatistic   <- evaluate_promise(getTargetFeatureStatistic(input_fitCurves, input_ROI, tmp_foundPeakTable, verbose=TRUE))
  
  # Check result table
  expect_equal(result_featureStatistic$result, expected_featureStatistic)
  
  # Check messages (cannot match timing)
  expect_equal(length(result_featureStatistic$messages), 1)
})

test_that('raise errors', {
  # targetFeatTable and foundPeakTable dimension mismatch
  expect_error(getTargetFeatureStatistic(input_fitCurves, input_ROI[1:3,], input_foundPeakTable, verbose=TRUE), 'Number of features in "targetFeatTable" (3) and "foundPeakTable" (4) do not match!', fixed=TRUE)

  # Number of fittedCurve and foundPeakTable mismatch
  expect_error(getTargetFeatureStatistic(input_fitCurves[1:3], input_ROI, input_foundPeakTable, verbose=TRUE), 'Number of fitted curves in "fittedCurves" (3) and number of features in "foundPeakTable" (4) do not match!', fixed=TRUE)
})

#' @title Generate a multiplot of all diagnostic plots
#'
#' @description Generate a multiplot of all diagnostic plots (as generated by
#' \code{annotationDiagnosticPlots()}) for each compound
#' 
#' @param annotationDiagnosticPlotList (list) List of (one per compound) of list
#' of diagnostic plots as generated by \code{annotationDiagnosticPlots()}
#' 
#' @return A list of multiplots (one per compound)
annotationDiagnosticMultiplot <- function(annotationDiagnosticPlotList) {
    # Init
    nbCpd <- length(annotationDiagnosticPlotList)
    outList <- vector("list", nbCpd)
    
    # Iterate over compounds
    for (cpd in seq_len(nbCpd)) {

        res <- annotDiagMultiplot_singleCpd(annotationDiagnosticPlotList, cpd)

        if (is.null(res)) { # pass to next compound in case of failure
            outList[[cpd]] <- NULL
            next
        } else {
            outList[[cpd]] <- res
        }
    }

    return(outList)
}


# -----------------------------------------------------------------------------
# annotationDiagnosticMultiplot helper functions

# prepare data for a single compound
annotDiagMultiplot_singleCpd <- function(annotationDiagnosticPlotList, cpd) {
    # Check input
    if (!all(c("EICFit", "rtPeakwidthVert", "rtPeakwidthHorzRunOrder",
                "mzPeakwidthHorzRunOrder", "areaRunOrder", "rtHistogram",
                "mzHistogram", "areaHistogram", "title") %in%
        names(annotationDiagnosticPlotList[[cpd]]))) {
        # cannot generate multiplot, pass to next compound
        message("Required plots missing for compound #", cpd)
        return(NULL) } # pass to next compound

    # Init plot
    resInit <- annotDiagMultiplot_initPlot(annotationDiagnosticPlotList, cpd)
    tmp_EIC <- resInit$EIC
    tmp_rtPeakwidthVert <- resInit$rtPeakwidthVert
    tmp_rtPeakwidthHorz <- resInit$rtPeakwidthHorz
    tmp_mzPeakwidthHorz <- resInit$mzPeakwidthHorz
    tmp_peakAreaHorz <- resInit$peakAreaHorz
    tmp_rtHisto <- resInit$rtHisto
    tmp_mzHisto <- resInit$mzHisto
    tmp_areaHisto <- resInit$areaHisto

    # link axis (same lim)
    resLink <- annotDiagMultiplot_linkAxes(tmp_EIC, tmp_rtPeakwidthVert,
        tmp_rtPeakwidthHorz, tmp_rtHisto, tmp_mzPeakwidthHorz, tmp_mzHisto,
        tmp_peakAreaHorz, tmp_areaHisto)
    tmp_EIC <- resLink$EIC
    tmp_rtPeakwidthVert <- resLink$rtPeakwidthVert
    tmp_rtPeakwidthHorz <- resLink$rtPeakwidthHorz
    tmp_mzPeakwidthHorz <- resLink$mzPeakwidthHorz
    tmp_peakAreaHorz <- resLink$peakAreaHorz
    tmp_rtHisto <- resLink$rtHisto
    tmp_mzHisto <- resLink$mzHisto
    tmp_areaHisto <- resLink$areaHisto

    # generate multiplot
    result <- annotDiagMultiplot_generateMulti(annotationDiagnosticPlotList,cpd,
        tmp_EIC, tmp_rtPeakwidthVert, tmp_rtPeakwidthHorz, tmp_rtHisto,
        tmp_mzPeakwidthHorz, tmp_mzHisto, tmp_peakAreaHorz, tmp_areaHisto)

    return(result)
}
# Initialise the subplots
annotDiagMultiplot_initPlot <- function(annotationDiagnosticPlotList, cpd) {

    tmp_EIC <- annotationDiagnosticPlotList[[cpd]]$EICFit +
                ggplot2::theme(axis.title.x = ggplot2::element_blank(),
                                axis.text.x = ggplot2::element_blank())
    tmp_rtPeakwidthVert <- annotationDiagnosticPlotList[[cpd]]$rtPeakwidthVert
    tmp_rtPeakwidthHorz <-
        annotationDiagnosticPlotList[[cpd]]$rtPeakwidthHorzRunOrder +
        ggplot2::theme(axis.title.x = ggplot2::element_blank(),
                        axis.text.x = ggplot2::element_blank())
    tmp_mzPeakwidthHorz <-
        annotationDiagnosticPlotList[[cpd]]$mzPeakwidthHorzRunOrder +
        ggplot2::theme(axis.title.x = ggplot2::element_blank(),
                        axis.text.x = ggplot2::element_blank())
    tmp_peakAreaHorz <- annotationDiagnosticPlotList[[cpd]]$areaRunOrder
    tmp_rtHisto <- annotationDiagnosticPlotList[[cpd]]$rtHistogram +
        ggplot2::coord_flip() +
        ggplot2::theme(axis.title.x = ggplot2::element_blank(),
                        axis.text.x = ggplot2::element_blank(),
                        axis.title.y = ggplot2::element_blank(),
                        axis.text.y = ggplot2::element_blank())
    tmp_mzHisto <- annotationDiagnosticPlotList[[cpd]]$mzHistogram +
        ggplot2::coord_flip() +
        ggplot2::theme(axis.title.x = ggplot2::element_blank(),
                        axis.text.x = ggplot2::element_blank(),
                        axis.title.y = ggplot2::element_blank(),
                        axis.text.y = ggplot2::element_blank())
    tmp_areaHisto <- annotationDiagnosticPlotList[[cpd]]$areaHistogram +
        ggplot2::coord_flip() +
        ggplot2::theme(axis.title.x = ggplot2::element_blank(),
                        axis.text.x = ggplot2::element_blank(),
                        axis.title.y = ggplot2::element_blank(),
                        axis.text.y = ggplot2::element_blank())

    return(list(EIC=tmp_EIC, rtPeakwidthVert=tmp_rtPeakwidthVert,
                rtPeakwidthHorz=tmp_rtPeakwidthHorz,
                mzPeakwidthHorz=tmp_mzPeakwidthHorz,
                peakAreaHorz=tmp_peakAreaHorz,
                rtHisto=tmp_rtHisto, mzHisto=tmp_mzHisto,
                areaHisto=tmp_areaHisto))
}
# Link subplots axes
annotDiagMultiplot_linkAxes <- function(tmp_EIC, tmp_rtPeakwidthVert,
        tmp_rtPeakwidthHorz, tmp_rtHisto, tmp_mzPeakwidthHorz, tmp_mzHisto,
        tmp_peakAreaHorz, tmp_areaHisto) {

    # link axis (same lim)
    # EIC + rt peakwidth (is rotated, but ggplot fixed it no more use y pkwidth)
    minXEIC <- min(ggplot2::layer_scales(tmp_EIC)$x$range$range[1],
                ggplot2::layer_scales(tmp_rtPeakwidthVert)$x$range$range[1])
    maxXEIC <- max(ggplot2::layer_scales(tmp_EIC)$x$range$range[2],
                ggplot2::layer_scales(tmp_rtPeakwidthVert)$x$range$range[2])
    tmp_EIC <- tmp_EIC + ggplot2::xlim(minXEIC, maxXEIC)
    tmp_rtPeakwidthVert <- tmp_rtPeakwidthVert + ggplot2::xlim(minXEIC, maxXEIC)

    # rt peakwidth runOrder + histo (is rotated, use x in histo)
    minYRT <- min(ggplot2::layer_scales(tmp_rtPeakwidthHorz)$y$range$range[1],
                    ggplot2::layer_scales(tmp_rtHisto)$x$range$range[1])
    maxYRT <- max(ggplot2::layer_scales(tmp_rtPeakwidthHorz)$y$range$range[2],
                    ggplot2::layer_scales(tmp_rtHisto)$x$range$range[2])
    tmp_rtPeakwidthHorz <- tmp_rtPeakwidthHorz + ggplot2::ylim(minYRT, maxYRT)
    tmp_rtHisto <- tmp_rtHisto + ggplot2::xlim(minYRT, maxYRT)

    # mz peakwidth runOrder + histo (is rotated, use x in histo)
    minYMZ <- min(ggplot2::layer_scales(tmp_mzPeakwidthHorz)$y$range$range[1],
                    ggplot2::layer_scales(tmp_mzHisto)$x$range$range[1])
    maxYMZ <- max(ggplot2::layer_scales(tmp_mzPeakwidthHorz)$y$range$range[2],
                    ggplot2::layer_scales(tmp_mzHisto)$x$range$range[2])
    tmp_mzPeakwidthHorz <- tmp_mzPeakwidthHorz + ggplot2::ylim(minYMZ, maxYMZ)
    tmp_mzHisto <- tmp_mzHisto + ggplot2::xlim(minYMZ, maxYMZ)

    # Area runOrder + histo (is rotated, use x in histo)
    minYArea <- min(ggplot2::layer_scales(tmp_peakAreaHorz)$y$range$range[1],
                    ggplot2::layer_scales(tmp_areaHisto)$x$range$range[1])
    maxYArea <- max(ggplot2::layer_scales(tmp_peakAreaHorz)$y$range$range[2],
                    ggplot2::layer_scales(tmp_areaHisto)$x$range$range[2])
    tmp_peakAreaHorz <- tmp_peakAreaHorz + ggplot2::ylim(minYArea, maxYArea)
    tmp_areaHisto <- tmp_areaHisto + ggplot2::xlim(minYArea, maxYArea)

    return(list(EIC=tmp_EIC, rtPeakwidthVert=tmp_rtPeakwidthVert,
                rtPeakwidthHorz=tmp_rtPeakwidthHorz,
                mzPeakwidthHorz=tmp_mzPeakwidthHorz,
                peakAreaHorz=tmp_peakAreaHorz,
                rtHisto=tmp_rtHisto, mzHisto=tmp_mzHisto,
                areaHisto=tmp_areaHisto))
}
# Generate multiplot
annotDiagMultiplot_generateMulti <- function(annotationDiagnosticPlotList, cpd,
    tmp_EIC, tmp_rtPeakwidthVert, tmp_rtPeakwidthHorz, tmp_rtHisto,
    tmp_mzPeakwidthHorz, tmp_mzHisto, tmp_peakAreaHorz, tmp_areaHisto) {
    ## Convert to gtables + match plotted width
    p_EIC <- ggplot2::ggplot_gtable(ggplot2::ggplot_build(tmp_EIC))
    p_rtPeakwidthVert <- ggplot2::ggplot_gtable(
                            ggplot2::ggplot_build(tmp_rtPeakwidthVert))
    p_rtPeakwidthHorz <- ggplot2::ggplot_gtable(
                            ggplot2::ggplot_build(tmp_rtPeakwidthHorz))
    p_mzPeakwidthHorz <- ggplot2::ggplot_gtable(
                            ggplot2::ggplot_build(tmp_mzPeakwidthHorz))
    p_peakAreaHorz <- ggplot2::ggplot_gtable(
                            ggplot2::ggplot_build(tmp_peakAreaHorz))
    p_rtHisto <- ggplot2::ggplot_gtable(ggplot2::ggplot_build(tmp_rtHisto))
    p_mzHisto <- ggplot2::ggplot_gtable(ggplot2::ggplot_build(tmp_mzHisto))
    p_areaHisto <- ggplot2::ggplot_gtable(ggplot2::ggplot_build(tmp_areaHisto))
    # find the widths of each of the plots, calculate the maximum and then apply
    # it to each of them individually. This effectively applies a uniform layout
    # to each of the plots ## EIC + rt peakwidth
    maxWidthEIC <- grid::unit.pmax(p_EIC$widths[2:5],
                                    p_rtPeakwidthVert$widths[2:5])
    p_EIC$widths[2:5] <- maxWidthEIC
    p_rtPeakwidthVert$widths[2:5] <- maxWidthEIC
    # rt peakwidth + mz peakwidth + peak area (x axis)
    maxWidthRtMzArea <- grid::unit.pmax(p_rtPeakwidthHorz$widths[2:5],
                                        p_mzPeakwidthHorz$widths[2:5],
                                        p_peakAreaHorz$widths[2:5])
    p_rtPeakwidthHorz$widths[2:5] <- maxWidthRtMzArea
    p_mzPeakwidthHorz$widths[2:5] <- maxWidthRtMzArea
    p_peakAreaHorz$widths[2:5] <- maxWidthRtMzArea
    # rt + mz + area peakwidth and histo (y axis)
    maxHeight <- grid::unit.pmin(p_rtPeakwidthHorz$heights[2:3],
                                p_mzPeakwidthHorz$heights[2:3],
                                p_peakAreaHorz$heights[2:3],
                                p_rtHisto$widths[2:5], p_mzHisto$widths[2:3],
                                p_areaHisto$widths[2:3])
    p_rtPeakwidthHorz$heights[2:3] <- maxHeight
    p_mzPeakwidthHorz$heights[2:3] <- maxHeight
    p_peakAreaHorz$heights[2:3] <- maxHeight
    p_rtHisto$widths[2:3] <- maxHeight
    p_mzHisto$widths[2:3] <- maxHeight
    p_areaHisto$widths[2:3] <- maxHeight
    ## Generate
    return(gridExtra::grid.arrange(p_EIC, p_rtPeakwidthVert,
        p_rtPeakwidthHorz, p_rtHisto, p_mzPeakwidthHorz, p_mzHisto,
        p_peakAreaHorz, p_areaHisto, widths = c(6, 1),
        heights = c(100, 45, 30, 30, 30, 14),
        layout_matrix = rbind(c(1, 1),c(2, 2),c(3, 4),c(5, 6),c(7, 8),c(7, 9)),
        top = annotationDiagnosticPlotList[[cpd]]$title) )
}

## Import Tab ------------------------------------------------------------------

# - init from files or load an annotation (init implicitely checks validity, load doesn't)
#   in case of error in the creation/loading function, the app is stopped with an error message
# - does a valid check to return a success bar (or error bar)
# - show created/loaded annotation on screen


## Generate/Load data on trigger (store 'annotation' in a reactiveValue, depending on input button pressed)
# check input but doesn't validate inputs

# Default annotation as reactiveValue
values <- reactiveValues(annotation = NULL, failures = NULL, featNmeList = NULL, spectraMetadataCol = NULL, filename = NULL)

# New annotation
observeEvent(input$triggerImportNewAnnotation, {
  # only if the right button is clicked
  if(input$triggerImportNewAnnotation != 0) {
    # catch the errors in the import functions and kill the app
    res_data <- tryCatch({
      # deal with spectraPaths and spectraMetadata
      spectraInfo <- spectraPaths_and_metadata_UI_helper(spectraPaths = input$spectraPaths$datapath,
                                                         spectraMetadataPath = input$spectraPathsWithMetadata$datapath)
      # create annotation
      initialise_annotation_from_files_UI_helper(CSVParamPath = input$CSVParamPath$datapath,
                                                 spectraPaths = spectraInfo$spectra,
                                                 cpdMetadataPath = input$cpdMetadataPath$datapath,
                                                 spectraMetadata = spectraInfo$meta,
                                                 verbose = FALSE)
    },
    error = function(e) {
      stopApp(e[[1]])
    })

    # valid_peakPantheRAnnotation() is part of annotation creation
    values$annotation <- res_data

    ## Need to add metadata needed later
    # Set a list of feature name for later use
    values$featNmeList        <- paste(peakPantheR::cpdID(values$annotation), peakPantheR::cpdName(values$annotation), sep=' - ')
    names(values$featNmeList) <- seq_len(length(values$featNmeList))
    # Set a list of spectraMetadata columns (+ None)
    tmp_splCol                <- c(list('None'), colnames(peakPantheR::spectraMetadata(values$annotation)))
    values$spectraMetadataCol <- tmp_splCol
    # Set a list of filename (sample list)
    values$filename           <- peakPantheR::filename(values$annotation)

  } # end if
})  # end New Annotation

# Load annotation
observeEvent(input$triggerLoadPreviousAnnotation, {
  # only if the right button is clicked
  if(input$triggerLoadPreviousAnnotation!=0) {
    # catch the errors in the import functions and kill the app
    res_data <- tryCatch({
      load_annotation_from_file_UI_helper(annotationPath = input$pathAnnotation$datapath)
    },
    error = function(e) {
      stopApp(e[[1]])
    })

    # no validation on load (cannot guarantee annotation is valid)
    values$annotation <- res_data

    ## Need to add metadata needed later
    # Set a list of feature name for later use
    values$featNmeList        <- paste(peakPantheR::cpdID(values$annotation), peakPantheR::cpdName(values$annotation), sep=' - ')
    names(values$featNmeList) <- seq_len(length(values$featNmeList))
    # Set a list of spectraMetadata columns (+ None)
    tmp_splCol                <- c(list('None'), colnames(peakPantheR::spectraMetadata(values$annotation)))
    values$spectraMetadataCol <- tmp_splCol
    # Set a list of filename (sample list)
    values$filename           <- peakPantheR::filename(values$annotation)

  } # end if
})  # end Load Annotation


## Check Import/Load are a success
# value for success
importSuccess <- reactive({
  # return 'no' until a trigger button is clicked
  if(input$triggerImportNewAnnotation == 0 & input$triggerLoadPreviousAnnotation == 0) { return('no')} # import not clicked

  isolate({
    # use a validObject as check to stop having a 'yes' in case of problems
    # if error raised during import, could have a values$annotation that exist (NULL) and validObject(NULL) is TRUE... so check it's a peakPantheRAnnotation
    if( isTRUE(validObject(values$annotation, test=TRUE)) & is(values$annotation, 'peakPantheRAnnotation') ) {
      return('yes')

    # not valid, return 'no'.
    # If it isn't a peakPantheRAnnotation, the app is killed with the error raised in the loading part (ordering is out of our control)
    # If it is an annotation, it is killed here
    } else {
      if(is(values$annotation, 'peakPantheRAnnotation')) {
        stopApp( paste('Error:', validObject(values$annotation, test=TRUE)))
      }
      return('no')
    }
  })
})


# success/failure UI
output$resultImportCheck <- renderUI({
  if(importSuccess()=='no') {
    # not imported yet
    if(input$triggerImportNewAnnotation == 0 & input$triggerLoadPreviousAnnotation == 0) {
      return()
    # import control failed
    } else {
      return(
        tagList(
          HTML("<div class=\"alert alert-dismissible alert-danger\"><button type=\"button\" class=\"close\" data-dismiss=\"alert\">×</button><h4 style=\"font-weight:bold\">Error</h4>Check input data</div>")
        )
      )
    }
  # import is successful
  } else if(importSuccess()=='yes') {
    tagList(
      HTML("<div class=\"alert alert-dismissible alert-success\"><button type=\"button\" class=\"close\" data-dismiss=\"alert\">×</button><h4 style=\"font-weight:bold\">Success</h4>Input data generated</div>")
    )
  }
})


## Show imported annotation
output$showImportResult <- renderUI({
  # not imported yet or failed
  if(importSuccess()=='no') {
      if(input$triggerImportNewAnnotation == 0 & input$triggerLoadPreviousAnnotation == 0) { return() }

  # import is successful
  } else if(importSuccess()=='yes') {
    # Capture the annotation shown and split by line into a list
    tmp_text <- utils::capture.output(show(values$annotation))
    tmp_text <- strsplit(tmp_text, '\n')
    # render the panel
    wellPanel(
      h4(HTML("Created/Imported <em>peakPantheRAnnotation</em>:"), style="color:#3e648d;font-weight:bold"),
      # preformatted text, pass the arguments as a list (shiny doesn't accept '\n' in character for print)
      do.call(tags$pre, tmp_text )
    )
  }
})


# Conditional panels control (next tab)
output$importDone <- renderText({
  if( importSuccess()=='yes' ) { 'yes' } else { 'no' }
})

# IMPORT - Tab panel  -------------------------------------------------------- #

tabPanel("Import Data",
  fluidRow( # top block containing both imports

    ## Init from files
    # get param csv path
    # get a list of file paths
    # cpd/spl metadata (no checks, just length, allowed not to be here)
    # returns init object!
    column(7,
      h4(HTML("Create a new <em>peakPantheRAnnotation</em>"), style="color:#3e648d;font-weight:bold"),
      wellPanel(
        fluidRow(
          column(12,
            h5(HTML("Targeted features"), style="color:#3e648d;font-weight:bold")),
          column(12,
            span("Select a .CSV file containing the ", shiny::span(strong("fit parameters")), shiny::span(em('with targeted features as rows and target windows as columns (at minimum: "cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", "mz", "mzMax")')),style="color:black")),
          column(12,
            fileInput('CSVParamPath', 'Choose a .CSV File',
                      accept=c('text/csv','text/comma-separated-values,text/plain','.csv','.tsv'))),
          tags$hr()
        ), # end fluidRow Fit params

        fluidRow(
          column(12,
            h5(HTML("Files to process (choose one of two)"), style="color:#3e648d;font-weight:bold")
          ), #end title column
          column(6,
            h6(HTML("File path only"), style="color:#3e648d;font-weight:bold"),
            span("Select the ", shiny::span(strong("files to process")),style="color:black"),
            fileInput('spectraPaths', 'Choose files to process',
                      multiple=TRUE)
          ), # end left files selector column (only files)
          column(6,
            h6(HTML("File paths and metadata"), style="color:#3e648d;font-weight:bold"),
            span(" Select a .CSV file containing the ", shiny::span(strong("file paths "), em("(column `filepath`)"), strong(" and metadata")), style="color:black"),
            fileInput('spectraPathsWithMetadata', 'Choose a .CSV File',
                      accept=c('text/csv','text/comma-separated-values,text/plain','.csv','.tsv'))
          ), # end right file selector column (files + metadata)
          tags$hr()
        ), # end fluidRow File paths (either without or with metadata)

        fluidRow(
          column(12,
            h5(HTML("Targeted features metadata (Optional)"), style="color:#3e648d;font-weight:bold")),
          column(12,
            span("Select a .CSV file containing the ", shiny::span(strong("targeted features metadata")), style="color:black")),
          column(12,
            fileInput('cpdMetadataPath', 'Choose a .CSV File',
                      accept=c('text/csv','text/comma-separated-values,text/plain','.csv','.tsv')))
        ), # end FluidRow feature metadata

        fluidRow(
          div(actionButton("triggerImportNewAnnotation", label="Import",
                           class="btn btn-primary btn-lg"), align="center")
        ) # end import button row
      )# end left panel
    ),	# end left column (New annotation)


    ## Init from previous object
    # load
    column(5,
      h4(HTML("Load a <em>peakPantheRAnnotation</em>"), style="color:#3e648d;font-weight:bold"),
      wellPanel(
        h5(HTML("Annotation"), style="color:#3e648d;font-weight:bold"),
        span("Select a .RData file containing a ", shiny::span(strong("peakPantheRAnnotation")), " named", shiny::span(em("annotationObject")),style="color:black"),
        fluidRow(
          fileInput('pathAnnotation', 'Choose a .RData File',
                    accept=c('application/octet-stream','.RData','.rdata'))
        ),

        fluidRow(
          div(actionButton("triggerLoadPreviousAnnotation", label="Import",
                           class="btn btn-primary btn-lg"), align="center")
        ) # end import button row
      )# end right panel
    )   # end right column (Load annotation)
  ),     # end fluidRow (top block containing both imports)


  ## Result of both imports
  fluidRow(
    # Row success / fail
    column(width=12,
      # success band 12 col wide (green) / FAIL (red)
      uiOutput("resultImportCheck")  # error/success message
    ), # end column

    # Row show annotation
    column(width=12,
      # box with the Show(init) results
      uiOutput("showImportResult")  # show() object properties
    ) # end column (row) of shown results
  )   # end fluidRow (bottom block containing both imports result)
)     # end tabPanel
# end IMPORT Tab panel ------------------------------------------------------- #

## valid method for \link{peakPantheRAnnotation-class}
## Number of compounds based on @cpdID length, number of samples based on
## @filepath length.
## Slot type is not checked as \code{setClass} enforces it.
## peakTables, peakFit and dataPoints type are checked on first list element.
valid_peakPantheRAnnotation <- function(object) {
    # init
    msg <- NULL
    valid <- TRUE
    nbCpd <- length(object@cpdID)
    nbSample <- length(object@filepath)

    # number of cpdName
    vcheck <- valid_ppR_cpdName(object, valid, msg, nbCpd)
    valid <- vcheck$valid; msg <- vcheck$msg
    # ROI nb rows, nb columns, column names, column types
    vcheck <- valid_ppR_ROI(object, valid, msg, nbCpd)
    valid <- vcheck$valid; msg <- vcheck$msg
    # FIR nb rows, nb columns, column names, column types
    vcheck <- valid_ppR_FIR(object, valid, msg, nbCpd)
    valid <- vcheck$valid; msg <- vcheck$msg
    # uROI nb rows, nb columns, column names, column types
    vcheck <- valid_ppR_uROI(object, valid, msg, nbCpd)
    valid <- vcheck$valid; msg <- vcheck$msg
    # number of compounds (rows) in cpdMetadata
    vcheck <- valid_ppR_cpdMetadata(object, valid, msg, nbCpd)
    valid <- vcheck$valid; msg <- vcheck$msg
    # number of spectra (rows) in spectraMetadata
    vcheck <- valid_ppR_spectraMetadata(object, valid, msg, nbSample)
    valid <- vcheck$valid; msg <- vcheck$msg
    # number of acquisitionTime
    vcheck <- valid_ppR_acquisitionTime(object, valid, msg, nbSample)
    valid <- vcheck$valid; msg <- vcheck$msg
    # cannot useUROI if uROIExist=FALSE
    vcheck <- valid_ppR_useUROIuROIExist(object, valid, msg)
    valid <- vcheck$valid; msg <- vcheck$msg
    # number of TIC
    vcheck <- valid_ppR_TIC(object, valid, msg, nbSample)
    valid <- vcheck$valid; msg <- vcheck$msg
    # peakTables (nb, type, nb of rows, nb columns, column names)
    vcheck <- valid_ppR_peakTables(object, valid, msg, nbCpd, nbSample)
    valid <- vcheck$valid; msg <- vcheck$msg
    # dataPoints (nb, type, nb of rows, nb columns, column names)
    vcheck <- valid_ppR_dataPoints(object, valid, msg, nbCpd, nbSample)
    valid <- vcheck$valid; msg <- vcheck$msg
    # peakFit (nb, type, nb of rows, nb columns, column names)
    vcheck <- valid_ppR_peakFit(object, valid, msg, nbCpd, nbSample)
    valid <- vcheck$valid; msg <- vcheck$msg

    # output
    if (valid) { return(TRUE) } else { return(msg) }
}


# -----------------------------------------------------------------------------
# valid_peakPantheRAnnotation helper functions

## number of cpdName
valid_ppR_cpdName <- function(object, valid, msg, nbCpd) {
    if (length(object@cpdName) != nbCpd) {
        valid <- FALSE
        msg <- c(msg, paste0("cpdName has ", length(object@cpdName),
            " elements (compound). Should be ",  nbCpd))
    }
    return(list(valid=valid, msg=msg))
}

## ROI (nb rows, nb columns, column names, column types)
valid_ppR_ROI <- function(object, valid, msg, nbCpd) {
    # ROI number of rows
    if (dim(object@ROI)[1] != nbCpd) {
        valid <- FALSE
        msg <- c(msg, paste0("ROI has ", dim(object@ROI)[1],
            " rows (compound). Should be ", nbCpd)) }
    # ROI number of columns
    if (dim(object@ROI)[2] != 6) {
        valid <- FALSE
        msg <- c(msg, paste0("ROI has ", dim(object@ROI)[2], " columns. Should",
        " be 6 (\"rtMin\", \"rt\", \"rtMax\", \"mzMin\", \"mz\", \"mzMax\")"))
    } else {
        # ROI column names
        if (!all(colnames(object@ROI) %in% c("rtMin", "rt", "rtMax", "mzMin",
            "mz", "mzMax"))) {
            valid <- FALSE
            msg <- c(msg, paste0('ROI columns should be \"rtMin\", \"rt\", ',
                '\"rtMax\", \"mzMin\", \"mz\", \"mzMax\", not ',
                paste(colnames(object@ROI), collapse = " ")))
        } else {
            vcheck <- valid_ppR_ROI_content(object, valid, msg, nbCpd)
            valid <- vcheck$valid; msg <- vcheck$msg
        }
    }
    return(list(valid=valid, msg=msg))
}
# check NA and column types, split due to function length
valid_ppR_ROI_content <- function(object, valid, msg, nbCpd){
    # Missing rtMin, rtMax, mzMin or mzMax
    if (any(is.na(object@ROI[,c("rtMin","rtMax","mzMin","mzMax")]))) {
        valid <- FALSE
        msg <- c(msg, paste0("ROI$rtMin, ROI$rtMax, ROI$mzMin and ",
                            "ROI$mzMax cannot be NA")) }
    # ROI column type
    if (nbCpd >= 1) {
        if (!is.numeric(object@ROI$rtMin[1])) {
            valid <- FALSE
            msg <- c(msg, paste0("ROI$rtMin should be numeric, not ",
                                typeof(object@ROI$rtMin[1]))) }
        if (!is.numeric(object@ROI$rt[1])) {
            valid <- FALSE
            msg <- c(msg, paste0("ROI$rt should be numeric, not ",
                                typeof(object@ROI$rt[1]))) }
        if (!is.numeric(object@ROI$rtMax[1])) {
            valid <- FALSE
            msg <- c(msg, paste0("ROI$rtMax should be numeric, not ",
                                typeof(object@ROI$rtMax[1]))) }
        if (!is.numeric(object@ROI$mzMin[1])) {
            valid <- FALSE
            msg <- c(msg, paste0("ROI$mzMin should be numeric, not ",
                                typeof(object@ROI$mzMin[1]))) }
        if (!is.numeric(object@ROI$mz[1])) {
            valid <- FALSE
            msg <- c(msg, paste0("ROI$mz should be numeric, not ",
                                typeof(object@ROI$mz[1]))) }
        if (!is.numeric(object@ROI$mzMax[1])) {
            valid <- FALSE
            msg <- c(msg, paste0("ROI$mzMax should be numeric, not ",
                                typeof(object@ROI$mzMax[1]))) }}
    return(list(valid=valid, msg=msg))
}

## FIR (nb rows, nb columns, column names, column types)
valid_ppR_FIR <- function(object, valid, msg, nbCpd) {
    # FIR number of rows
    if (dim(object@FIR)[1] != nbCpd) {
        valid <- FALSE
        msg <- c(msg, paste0("FIR has ", dim(object@FIR)[1],
                            " rows (compound). Should be ", nbCpd)) }
    # FIR number of columns
    if (dim(object@FIR)[2] != 4) {
        valid <- FALSE
        msg <- c(msg, paste0("FIR has ", dim(object@FIR)[2],
        " columns. Should be 4 (\"rtMin\", \"rtMax\", \"mzMin\", \"mzMax\")"))
    } else {
        # FIR column names
        if (!all(colnames(object@FIR) %in%
            c("rtMin", "rtMax", "mzMin", "mzMax"))) {
            valid <- FALSE
            msg <- c(msg, paste0('FIR columns should be \"rtMin\", \"rtMax\", ',
                                '\"mzMin\", \"mzMax\", not ',
                                paste(colnames(object@FIR), collapse = " ")))
        } else {
            # Missing rtMin, rtMax, mzMin or mzMax
            if (object@useFIR) {
                # FIR is set to NA when not in use
                if (any(is.na(object@FIR$rtMin))|any(is.na(object@FIR$rtMax))|
                    any(is.na(object@FIR$mzMin))|any(is.na(object@FIR$mzMax))) {
                    valid <- FALSE
                    msg <- c(msg, paste0("FIR$rtMin, FIR$rtMax, FIR$mzMin and ",
                                        "FIR$mzMax cannot be NA")) }
            }
            # FIR column type
            if (nbCpd >= 1) {
                if (!is.numeric(object@FIR$rtMin[1])) {
                    valid <- FALSE
                    msg <- c(msg, paste0("FIR$rtMin should be numeric, not ",
                                        typeof(object@FIR$rtMin[1]))) }
                if (!is.numeric(object@FIR$rtMax[1])) {
                    valid <- FALSE
                    msg <- c(msg, paste0("FIR$rtMax should be numeric, not ",
                                        typeof(object@FIR$rtMax[1]))) }
                if (!is.numeric(object@FIR$mzMin[1])) {
                    valid <- FALSE
                    msg <- c(msg, paste0("FIR$mzMin should be numeric, not ",
                                        typeof(object@FIR$mzMin[1]))) }
                if (!is.numeric(object@FIR$mzMax[1])) {
                    valid <- FALSE
                    msg <- c(msg, paste0("FIR$mzMax should be numeric, not ",
                                        typeof(object@FIR$mzMax[1]))) }}}}
    return(list(valid=valid, msg=msg))
}

## uROI (nb rows, nb columns, column names, column types)
valid_ppR_uROI <- function(object, valid, msg, nbCpd) {
    # uROI number of rows
    if (dim(object@uROI)[1] != nbCpd) {
        valid <- FALSE
        msg <- c(msg, paste0("uROI has ", dim(object@uROI)[1],
                            " rows (compound). Should be ", nbCpd)) }
    # uROI number of columns
    if (dim(object@uROI)[2] != 6) {
        valid <- FALSE
        msg <- c(msg, paste0("uROI has ",dim(object@uROI)[2],' columns. Should',
        ' be 6 (\"rtMin\", \"rt\", \"rtMax\", \"mzMin\", \"mz\", \"mzMax\")'))
    } else {
        # uROI column names
        if (!all(colnames(object@uROI) %in% c("rtMin", "rt", "rtMax", "mzMin",
                                                "mz", "mzMax"))) {
            valid <- FALSE
            msg <- c(msg, paste0('uROI columns should be \"rtMin\", \"rt\", ',
                            '\"rtMax\", \"mzMin\", \"mz\", \"mzMax\", not ',
                            paste(colnames(object@uROI), collapse = " ")))
        } else {
            vcheck <- valid_ppR_uROI_content(object, valid, msg, nbCpd)
            valid <- vcheck$valid; msg <- vcheck$msg
        }
    }
    return(list(valid=valid, msg=msg))
}
# check NA and column types, split due to function length
valid_ppR_uROI_content <- function(object, valid, msg, nbCpd){
    # Missing rtMin, rtMax, mzMin or mzMax
    if (object@uROIExist) {
        # only check uROI if declared as existing
        if (any(is.na(object@uROI[,c("rtMin","rtMax","mzMin","mzMax")]))) {
            valid <- FALSE
            msg <- c(msg, paste0("uROI$rtMin, uROI$rtMax, uROI$mzMin and ",
                                "uROI$mzMax cannot be NA")) }
    }
    # uROI column type
    if (nbCpd >= 1) {
        if (!is.numeric(object@uROI$rtMin[1])) {
            valid <- FALSE
            msg <- c(msg, paste0("uROI$rtMin should be numeric, not ",
                                typeof(object@uROI$rtMin[1]))) }
        if (!is.numeric(object@uROI$rt[1])) {
            valid <- FALSE
            msg <- c(msg, paste0("uROI$rt should be numeric, not ",
                                typeof(object@uROI$rt[1]))) }
        if (!is.numeric(object@uROI$rtMax[1])) {
            valid <- FALSE
            msg <- c(msg, paste0("uROI$rtMax should be numeric, not ",
                                typeof(object@uROI$rtMax[1]))) }
        if (!is.numeric(object@uROI$mzMin[1])) {
            valid <- FALSE
            msg <- c(msg, paste0("uROI$mzMin should be numeric, not ",
                                typeof(object@uROI$mzMin[1]))) }
        if (!is.numeric(object@uROI$mz[1])) {
            valid <- FALSE
            msg <- c(msg, paste0("uROI$mz should be numeric, not ",
                                typeof(object@uROI$mz[1]))) }
        if (!is.numeric(object@uROI$mzMax[1])) {
            valid <- FALSE
            msg <- c(msg, paste0("uROI$mzMax should be numeric, not ",
                                typeof(object@uROI$mzMax[1]))) }}
    return(list(valid=valid, msg=msg))
}

## cpdMetadata
valid_ppR_cpdMetadata <- function(object, valid, msg, nbCpd) {
    # number of compounds (rows) in cpdMetadata
    if (dim(object@cpdMetadata)[1] != nbCpd) {
        valid <- FALSE
        msg <- c(msg, paste0("cpdMetadata has ", dim(object@cpdMetadata)[1],
                            " rows (compounds). Should be ", nbCpd)) }
    return(list(valid=valid, msg=msg))
}

## spectraMetadata
valid_ppR_spectraMetadata <- function(object, valid, msg, nbSample) {
    # number of spectra (rows) in spectraMetadata
    if (dim(object@spectraMetadata)[1] != nbSample) {
        valid <- FALSE
        msg <- c(msg, paste0("spectraMetadata has ",
                            dim(object@spectraMetadata)[1],
                            " rows (spectra). Should be ", nbSample)) }
    return(list(valid=valid, msg=msg))
}

## acquisitionTime
valid_ppR_acquisitionTime <- function(object, valid, msg, nbSample) {
    # number of acquisitionTime
    if (length(object@acquisitionTime) != nbSample) {
        valid <- FALSE
        msg <- c(msg, paste0("acquisitionTime has ",
            length(object@acquisitionTime), " elements (samples). Should be ",
            nbSample)) }
    return(list(valid=valid, msg=msg))
}

## useUROI & uROIExist
valid_ppR_useUROIuROIExist <- function(object, valid, msg) {
    # cannot useUROI if uROIExist=FALSE
    if (object@useUROI & !(object@uROIExist)) {
        valid <- FALSE
        msg <- c(msg, paste0("useUROI cannot be TRUE while uROIExist is FALSE"))
    }
    return(list(valid=valid, msg=msg))
}

## TIC
valid_ppR_TIC <- function(object, valid, msg, nbSample) {
    # number of TIC
    if (length(object@TIC) != nbSample) {
        valid <- FALSE
        msg <- c(msg, paste0("TIC has ", length(object@TIC),
                            " elements (samples). Should be ", nbSample)) }
    return(list(valid=valid, msg=msg))
}

## peakTables (nb, type, nb of rows, nb columns, column names)
valid_ppR_peakTables <- function(object, valid, msg, nbCpd, nbSample) {
    # number of peakTables
    if (length(object@peakTables) != nbSample) { valid <- FALSE
        msg <- c(msg, paste0("peakTables has ", length(object@peakTables),
                            " elements (samples). Should be ", nbSample))
    } else { # only check peakTables if min 1 sample and not NULL
        if (nbSample >= 1) { # if ALL peakTables are not NULL
            peakTables_isNULL <- vapply(object@peakTables, is.null,
                                        FUN.VALUE = logical(1))
            if (!all(peakTables_isNULL)) {
                # if one peakTable is NULL but not all, raise an error
                if (any(peakTables_isNULL)) { valid <- FALSE
                    msg <- c(msg, paste0('peakTables must all either be ',
                                        'data.frame or NULL'))
                } else { # individual peakTable is data.frame
                    if (!is.data.frame(object@peakTables[[1]])) { valid <- FALSE
                        msg <- c(msg, paste0('peakTables must be data.frame or',
                                ' NULL not ', typeof(object@peakTables[[1]])))
                    } else { # individual peakTable data.frame number of rows
                        if (dim(object@peakTables[[1]])[1] != nbCpd) {
                            valid <- FALSE
                            msg <- c(msg, paste0("peakTables[[1]] has ",
                                            dim(object@peakTables[[1]])[1],
                                    " rows (compounds). Should be ", nbCpd))}
                        # individual peakTable data.frame number of columns
                        if (dim(object@peakTables[[1]])[2] != 16) {
                            valid <- FALSE
                            msg <- c(msg, paste0("peakTables[[1]] has ",
                                                dim(object@peakTables[[1]])[2],
                                                " columns. Should be 16"))
                        } else { # individual peakTable data.frame column names
                            if (!all(colnames(object@peakTables[[1]]) %in%
                            c("found", "rt", "rtMin", "rtMax", "mz", "mzMin",
                            "mzMax", "peakArea","peakAreaRaw", "maxIntMeasured",
                            "maxIntPredicted", "is_filled", "ppm_error",
                            "rt_dev_sec", "tailingFactor", "asymmetryFactor"))){
                                valid <- FALSE
                                msg <- c(msg, paste0("peakTables[[1]] columns ",
                                    "should be 'found', 'rt', 'rtMin', ",
                                    "'rtMax', 'mz', 'mzMin', 'mzMax', ",
                                    "'peakArea', 'peakAreaRaw', ",
                                    "'maxIntMeasured', ",
                                    "'maxIntPredicted', 'is_filled', ",
                                    "'ppm_error', 'rt_dev_sec', ",
                                    "'tailingFactor', 'asymmetryFactor', not ",
                                    paste(colnames(object@peakTables[[1]]),
                                    collapse = " "))) }}}}}}}
    return(list(valid=valid, msg=msg)) }

## dataPoints (nb, type, nb of rows, nb columns, column names)
valid_ppR_dataPoints <- function(object, valid, msg, nbCpd, nbSample) {
    # number of dataPoints
    if (length(object@dataPoints) != nbSample) {
        valid <- FALSE
        msg <- c(msg, paste0("dataPoints has ", length(object@dataPoints),
                            " elements (samples). Should be ", nbSample))
    } else { # only check dataPoints if min 1 sample and not NULL
        if (nbSample >= 1) { # if ALL dataPoints are not NULL
            dataPoints_isNULL <- vapply(object@dataPoints, is.null,
                                        FUN.VALUE = logical(1))
            if (!all(dataPoints_isNULL)) {
                # if one dataPoints is NULL but not all, raise an error
                if (any(dataPoints_isNULL)) {
                    valid <- FALSE
                    msg <- c(msg, paste0("dataPoints must all either be list ",
                                        "of ROI data points or NULL"))
                } else { # individual dataPoints is list
                    if (!(is.list(object@dataPoints[[1]]))) {
                        valid <- FALSE
                        msg <- c(msg, paste0("dataPoints[[1]] must be a list ",
                            "of ROI data points, not ",paste(class(
                            object@dataPoints[[1]]), collapse = " ")))
                    } else {
                        # check content of dataPoints list
                        ccheck <- valid_ppR_dataPoints_content(object, valid,
                                                                msg, nbCpd)
                        valid <- ccheck$valid; msg <- ccheck$msg
                    }
                }
            }
        }
    }
    return(list(valid=valid, msg=msg))
}
## dataPoints exist, check content
valid_ppR_dataPoints_content <- function(object, valid, msg, nbCpd) {
    # individual dataPoints has entry for each compound(ROI)
    if (length(object@dataPoints[[1]]) != nbCpd) {
        valid <- FALSE
        msg <- c(msg, paste0("dataPoints[[1]] contains, ",
                length(object@dataPoints[[1]]),
                " dataPoints (compound). Should be ",nbCpd))
    } else {
        if (nbCpd >= 1) { # ind dtPts compound entry is df
            if (!is.data.frame(object@dataPoints[[1]][[1]]))
            {
                valid <- FALSE
                msg <- c(msg, paste0("dataPoints[[1]][[1]]",
                            " must be a data.frame, not ",
                        class(object@dataPoints[[1]][[1]])))
            } else {# ind peakTable df nb of columns
                if (dim(object@dataPoints[[1]][[1]])[2]!=3){
                    valid <- FALSE
                    msg <- c(msg,
                        paste0("dataPoints[[1]][[1]] has ",
                        dim(object@dataPoints[[1]][[1]])[2],
                        " columns. Should be 3"))
                } else {
                # individual peakTable dt.frame column names
                    if (!all(colnames(
                        object@dataPoints[[1]][[1]]) %in%
                        c("rt", "mz", "int"))) {
                        valid <- FALSE
                        msg <- c(msg,
                            paste0("dataPoints[[1]][[1]] ",
                            "columns should be 'rt', 'mz',",
                            " 'int', not ", paste(colnames(
                            object@dataPoints[[1]][[1]]),
                            collapse = " ")))
                    }
                }
            }
        }
    }
    return(list(valid=valid, msg=msg))
}

## peakFit (nb, type, nb of rows, nb columns, column names)
valid_ppR_peakFit <- function(object, valid, msg, nbCpd, nbSample) {
    # number of peakFit
    if (length(object@peakFit) != nbSample) {
        valid <- FALSE
        msg <- c(msg, paste0("peakFit has ", length(object@peakFit),
                " elements (samples). Should be ", nbSample))
    } else { # only check peakFit if min 1 sample and not NULL
        if (nbSample >= 1) { # if ALL peakFit are not NULL
            peakFit_isNULL <- vapply(object@peakFit, is.null,
                                    FUN.VALUE = logical(1))
            if (!all(peakFit_isNULL)) {
                # if one peakFit is NULL but not all, raise an error
                if (any(peakFit_isNULL)) {
                    valid <- FALSE
                    msg <- c(msg, paste0("peakFit must all either be list of ",
                                        "ROI curveFit or NULL"))
                } else { # individual peakFit is list
                    if (!(is.list(object@peakFit[[1]]))) {
                        valid <- FALSE
                        msg <- c(msg, paste0("peakFit[[1]] must be a list of ",
                                            "ROI curveFit or NA, not ",
                                            paste(class(object@peakFit[[1]]),
                                                collapse = " ")))
                    } else {
                        # individual peakFit has entry for each compound (ROI)
                        if (length(object@peakFit[[1]]) != nbCpd) {
                            valid <- FALSE
                            msg <- c(msg, paste0("peakFit[[1]] contains, ",
                                        length(object@peakFit[[1]]),
                                        " peakPantheR_curveFit or",
                                        " NA (compound). Should be ", nbCpd))
                        } else { # only check peakFit if min 1 compound
                            if (nbCpd >= 1) {
                                # individual peakFit compound entry is
                                # peakPantheR_curveFit or NA
                                if (!all(is.na(object@peakFit[[1]][[1]])) &
                                    !is.peakPantheR_curveFit(
                                        object@peakFit[[1]][[1]])){
                                    valid <- FALSE
                                    msg <- c(msg, paste0("peakFit[[1]][[1]] ",
                                        "must be NA or a peakPantheR_curveFit,",
                                        " not ",
                                        class(object@peakFit[[1]][[1]])))
                                }}}}}}}}
    return(list(valid=valid, msg=msg))
}

# RUN - Tab panel  ----------------------------------------------------------- #

tabPanel("Run annotation",
  # Conditional UI based on import successfully initialised
  uiOutput("noImportForFitUI"),
  conditionalPanel(
    condition = "output.importDone=='yes'",
    fluidRow(

  ## Sidebar
      column(width=2,
        # Current annotation status
        uiOutput("showAnnotStatus")
      ), # end Sidebar column

  ## Main panel
      column(10,
        # Overall title
        h3('Run calculation', style="color:#3e648d;font-weight:bold"),

        # Message already annotated
        uiOutput("alreadyAnnotatedUI"),

        # Run options and trigger
        wellPanel(
          fluidRow(
            # useUROI, useFIR & curveModel
            column(5, offset=1,
              uiOutput("useUROICheckbox"),       # use uROI
              uiOutput("useFIRCheckbox"),        # use FIR
              uiOutput("curveModelSelectInput"), # curveModel
            ), # end column (useUROI, useFIR)

            # ncores cpuslider
            column(4, offset=1,
			  checkboxInput("parallelisation",
                label = span("Parallelisation", style="color:#3e648d;font-weight:bold"),
				value = FALSE
              ),
              uiOutput("cpuSlider")
            )  # end column (cpu slider)
          ),   # end fluidRow run parameters
          tags$hr(),

          fluidRow(
            div(actionButton("goAnnotation", label="Annotate", class="btn btn-primary btn-lg"), align="center")
          )  # end fluidRow run button
        ),   # end wellPanel (run options + trigger)

        # Progress bar
        fluidRow(
          uiOutput("progressBarUI")
        ),

        # Success (green) / fail (red) row
        fluidRow(
          column(12, 
            uiOutput("successAnnotationUI") # error/success message
          )
        ) # end fluiRow (success panel)
      )   # end Main panel column
    )     # end fluidRow (sidebar + menu)
  )       # end conditional panel
)
# end RUN Tab panel ---------------------------------------------------------- #

---
title: "Real Time Annotation"
date: "2020-10-11"
package: peakPantheR
output:
    BiocStyle::html_document:
        toc_float: true
vignette: >
    %\VignetteIndexEntry{Real Time Annotation}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
    %\VignetteDepends{peakPantheR,faahKO,pander,BiocStyle}
    %\VignettePackage{peakPantheR}
    %\VignetteKeywords{mass spectrometry, metabolomics}
---

```{r biocstyle, echo = FALSE, results = "asis" }
BiocStyle::markdown()
```
```{r, echo = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
```

**Package**: `r Biocpkg("peakPantheR")`<br />
**Authors**: Arnaud Wolfer<br />

```{r init, message = FALSE, echo = FALSE, results = "hide" }
## Silently loading all packages
library(BiocStyle)
library(peakPantheR)
library(faahKO)
library(pander)
```

# Introduction

The `peakPantheR` package is designed for the detection, integration and 
reporting of pre-defined features in MS files (_e.g. compounds, fragments, 
adducts, ..._).

The **Real Time Annotation** is set to detect and integrate **multiple** 
compounds in **one** file at a time. 
It therefore can be deployed on a LC-MS instrument to integrate a set of 
pre-defined features (_e.g. spiked standards_) as soon as the acquisition of a 
sample is completed.

Using the `r Biocpkg("faahKO")` raw MS dataset as an example, this vignette 
will:

* Detail the **Real Time Annotation** concept
* Apply the **Real Time Annotation** to a subset of pre-defined features in the 
`r Biocpkg("faahKO")` dataset

## Abbreviations
- **ROI**: _Regions Of Interest_ 
    * reference _RT_ / _m/z_ windows in which to search for a feature
- **uROI**: _updated Regions Of Interest_
    * modifed ROI adapted to the current dataset which override the reference 
    ROI
- **FIR**: _Fallback Integration Regions_ 
    * _RT_ / _m/z_ window to integrate if no peak is found
- **TIC**: _Total Ion Chromatogram_ 
    * the intensities summed across all masses for each scan
- **EIC**: _Extracted Ion Chromatogram_ 
    * the intensities summed over a mass range, for each scan


# Real Time Annotation Concept

Real time compound integration is set to process **multiple** compounds in 
**one** file at a time.

To achieve this, `peakPantheR` will:

* load a list of expected _RT_ / _m/z_ regions of interest (**ROI**)
* detect features in each ROI and keep the highest intensity one
* determine peak statistics for each feature
* return:
    + TIC
    + a table with all detected compounds for that file (_row: compound, col: 
    statistic_)
    + EIC for each ROI
    + sample acquisition date-time from the mzML metadata (_if available_)
    + save EIC plots to disk


# Real Time Annotation Example

In the following example we will target two pre-defined features in a single raw
MS spectra file from the `r Biocpkg("faahKO")` package. For more details on the 
installation and input data employed, please consult the 
[Getting Started with peakPantheR](getting-started.html) vignette.


## Input Data

The path to a MS file from the `r Biocpkg("faahKO")` is located and used as 
input spectra:
```{r}
library(faahKO)
## file paths
input_spectraPath  <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"))
input_spectraPath
```


Two targeted features (_e.g. compounds, fragments, adducts, ..._) are defined 
and stored in a table with as columns:

* `cpdID` (character)
* `cpdName` (character)
* `rtMin` (sec)
* `rtMax` (sec)
* `rt` (sec, optional / `NA`)
* `mzMin` (m/z)
* `mzMax` (m/z)
* `mz` (m/z, optional / `NA`)

```{r, eval = FALSE}
# targetFeatTable
input_targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), 
                        c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", 
                        "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390., 
                                522.194778, 522.2, 522.205222)
input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440., 
                                496.195038, 496.2, 496.204962)
input_targetFeatTable[,c(3:8)] <- sapply(input_targetFeatTable[,c(3:8)], 
                                            as.numeric)
```

```{r, results = "asis", echo = FALSE}
# use pandoc for improved readability
input_targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), 
                        c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", 
                        "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390., 
                                522.194778, 522.2, 522.205222)
input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440., 
                                496.195038, 496.2, 496.204962)
input_targetFeatTable[,c(3:8)] <- sapply(input_targetFeatTable[,c(3:8)], 
                                            as.numeric)
rownames(input_targetFeatTable) <- NULL
pander::pandoc.table(input_targetFeatTable, digits = 9)
```


## Run Single File Annotation

`peakPantheR_singleFileSearch()` takes as input a `singleSpectraDataPath` 
pointing to the file to process and `targetFeatTable` defining the features to 
integrate. The resulting annotation contains all the fitting and integration 
properties:
```{r}
library(peakPantheR)
annotation <- peakPantheR_singleFileSearch(
                                    singleSpectraDataPath = input_spectraPath,
                                    targetFeatTable = input_targetFeatTable,
                                    peakStatistic = TRUE,
                                    curveModel = 'skewedGaussian',
                                    verbose = TRUE)
```
```{r}
annotation$TIC
```
```{r}
## acquisition time cannot be extracted from NetCDF files
annotation$acquTime
```
```{r, eval = FALSE}
annotation$peakTable
```
```{r, results = "asis", echo = FALSE}
# use pandoc for improved readability
pander::pandoc.table(annotation$peakTable, digits = 7)
```
```{r}
annotation$curveFit
```
```{r}
annotation$ROIsDataPoint
```


`peakPantheR_singleFileSearch()` takes multiple parameters that can alter the 
file annotation:

* `peakStatistic` if `TRUE` calculates additional peak statistics: 
_'ppm_error'_, _'rt_dev_sec'_, _'tailing factor'_ and _'asymmetry factor'_
* `plotEICsPath` if not `NA` will save a `.png` of all ROI EICs at the path 
provided (expects `'filepath/filename.png'` for example). If `NA` no plot is 
saved
* `getAcquTime` if `TRUE` the sample acquisition date-time is extracted from the
`mzML` metadata. Acquisition time cannot be extracted from other file formats. 
The additional file access will impact run time
* `FIR` if not `NULL`, defines the Fallback Integration Regions (**FIR**) to 
integrate when a feature is not found.
* `curveModel`, defines the peak-shape model to fit to each EIC. By default,
a _'skewedGaussian'_ model is used. The other alternative is the exponentially
modified gaussian _'emgGaussian'_ model.
* `verbose` if `TRUE` messages calculation progress, time taken and number of 
features found (_total and matched to targets_)
* `...` passes arguments to `findTargetFeatures` to alter peak-picking 
parameters (e.g. the curveModel, the sampling or fitting parameters)



The summary plot generated by `plotEICsPath`, corresponding to the EICs of each 
integrated regions of interest is as follow:
```{r, out.width = "700px", echo = FALSE}
knitr::include_graphics("figures/singleFileSearch_EICsPlot.png")
```

> EICs plot: Each panel correspond to a targeted feature, with the EIC extracted
on the `mzMin`, `mzMax` range found. The red dot marks the RT peak apex, and the
red line highlights the RT peakwidth range found (`rtMin`, `rtMax`)



# See Also

* [Getting Started with peakPantheR](getting-started.html)
* [Parallel Annotation](parallel-annotation.html)
* [Graphical user interface use](peakPantheR-GUI.html)


# Session Information
```{r, echo = FALSE}
devtools::session_info()
```

---
title: "peakPantheR Graphical User Interface"
author: "Arnaud Wolfer"
date: "2020-10-11"
output: rmarkdown::html_vignette
vignette: >
    %\VignetteIndexEntry{peakPantheR Graphical User Interface}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
    %\VignetteDepends{peakPantheR,faahKO,pander,BiocStyle,foreach,doParallel}
    %\VignettePackage{peakPantheR}
    %\VignetteKeywords{mass spectrometry, metabolomics}
---

```{r biocstyle, echo = FALSE, results = "asis" }
BiocStyle::markdown()
```
```{r, echo = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
```

**Package**: `r Biocpkg("peakPantheR")`<br />
**Authors**: Arnaud Wolfer, Goncalo Correia<br />

```{r init, message = FALSE, echo = FALSE, results = "hide" }
## Silently loading all packages
library(BiocStyle)
library(peakPantheR)
library(faahKO)
library(pander)
```

# Introduction

The `peakPantheR` package is designed for the detection, integration and
reporting of pre-defined features in MS files (_e.g. compounds, fragments,
adducts, ..._).

The graphical user interface implements all of `peakPantheR`'s functionalities 
and can be preferred to understand the methodology, select the best parameters 
on a subset of the samples before running the command line, or to visually 
explore results.

Using the `r Biocpkg("faahKO")` raw MS dataset as an example, this vignette
will:

* Detail the step-by-step use of the **graphical user interface**
* Apply the **GUI** to a subset of pre-defined features in the
`r Biocpkg("faahKO")` dataset

## Abbreviations
- **ROI**: _Regions Of Interest_
    * reference _RT_ / _m/z_ windows in which to search for a feature
- **uROI**: _updated Regions Of Interest_
    * modifed ROI adapted to the current dataset which override the reference
    ROI
- **FIR**: _Fallback Integration Regions_
    * _RT_ / _m/z_ window to integrate if no peak is found


## Example Data

This vignette employ the `.csv` or `.RData` files generated from 
`r Biocpkg("faahKO")` in the vignette 
[Getting Started with peakPantheR](getting-started.html).


## Getting Started

The graphical user interface is started as follow:
```{r, eval = FALSE}
library(peakPantheR)

peakPantheR_start_GUI(browser = TRUE)
#  To exit press ESC in the command line
```

> The graphical interface is divided in 5 main tabs, **Import Data**, 
**Run annotation**, **Diagnostic: plot & update**, **View results** and 
**Export results**.

$~$

# Graphical User Interface
## Import

The first input format is using a `.RData` file containing a 
_peakPantheRAnnotation_ named `annotationObject`. This object can be annotated 
or not, for example loading a previously run annotation (_see the **Export** 
section for more details_).

```{r, out.width = "700px", echo = FALSE}
knitr::include_graphics("figures/01-import_RData.png")
```

$~$

The second input format consists of multiple `.csv` files describing the 
targeted features, spectra to process and corresponding metadata (optional). 
Spectra can also be directly selected on disk.

```{r, out.width = "700px", echo = FALSE}
knitr::include_graphics("figures/02-import_CSV.png")
```

$~$

## Run Annotation

With the targeted features and spectra defined, **Run annotation** handles the 
fitting parameter selection as well as downstream computation.
First the use of updated regions of interest (`uROI`) and fallback integration 
regions (`FIR`) can be selected if available. If `uROI` haven't been previously 
defined, the option will be crossed out.
Secondly the curve fitting model to use can be selected from the interface.
Finally `Parallelisation` enables the selection of the number of CPU cores to 
employ for parallel file access and processing.

```{r, out.width = "700px", echo = FALSE}
knitr::include_graphics("figures/03-Run_annotation.png")
```

$~$

## Diagnostic: plot & update

> **Note:**
>
> The targeted regions of interest (`ROI`) should represent a good starting 
point for feature integration, however it might be necessary to refine these 
boundary box to the specific analytical run considered. This ensures a 
successful integration over all the spectra irrespective of potential 
chromatographic equilibration differences or retention time drift.
>
> Updated regions of interest (`uROI`) can be defined and will supplant `ROI`. 
`uROI` can for example be manually defined to "tighten" or correct the `ROI` 
and avoid erroneous integration. Another use of `uROI` is to encompass the 
integration region in each sample throughout the run without targeting any 
excess spectral region that would interfere with the correct analysis.
>
> Fallback integration regions (`FIR`) are defined as spectral regions that will
be integrated (_i.e. integrating the baseline signal_) when no successful 
chromatographic peak could be detected in a sample. `FIR` shouldn't reasonably
stretch further than the minimum and maximum bound (_RT_ / _m/z_) of all found 
peaks across all samples for a given feature: this way no excess signal will be 
considered.

$~$

With all features integrated in all samples, **Diagnostic** provide tools to 
assess the quality of the peak integration and refine integration boundaries by 
setting `uROI` and `FIR` adapted to the specific chromatographic run being 
processed.

$~$

`Annotation statistics` summarises the success in integrating each targeted 
feature. The `ratio of peaks found (%)`, `ratio of peaks filled (%)` and the 
average `ppm error` and `RT deviation (s)` will highlight a feature that wasn't 
reliably integrated over a large number of samples. Visual evaluation 
(_see below_) and the adjustment of `uROI` or `FIR` might assist in tuning the 
integration of said feature.

```{r, out.width = "700px", echo = FALSE}
knitr::include_graphics("figures/04-Diagnostic_statistics.png")
```

$~$

`Update uROI/FIR` automatically sets `uROI` and `FIR` for each feature based on
the _RT_ / _m/z_ boundaries of the peaks successfully integrated.

```{r, out.width = "700px", echo = FALSE}
knitr::include_graphics("figures/05-Diagnostic_updateUROIFIR.png")
```

$~$

`Diagnostic plot` offer a visualisation of a selected feature across all samples
in order of analysis. This visualisation highlights the fitting of the feature 
in each sample, as well as the change in _RT_ / _m/z_ (of the peak apex) and 
peak area through time. Samples can be automatically coloured based on a 
_sample metadata_ column.

```{r, out.width = "700px", echo = FALSE}
knitr::include_graphics("figures/06-Diagnostic_plot.png")
```

$~$

Once `uROI` and `FIR` are successfully set, it is possible to go back to the 
**Run annotation** tab and refit all features in all samples (_Note: this will 
overwrite the current results_).

$~$

## View results

If the features integration are satisfactory, **View results** regroups all 
the integration results

$~$

`Overall results` displays a fitting property for all targeted features (_as 
columns_) and all spectra (_as rows_).

```{r, out.width = "700px", echo = FALSE}
knitr::include_graphics("figures/07-Results_Overall.png")
```

$~$

`Results per targeted feature` displays all fitting properties (_as columns_) 
for all samples (_as rows_) for a selected targeted feature.

```{r, out.width = "700px", echo = FALSE}
knitr::include_graphics("figures/08-Results_byFeature.png")
```

$~$

`Results per sample` displays all fitting properties (_as columns_) for all 
targeted features (_as rows_) for a selected sample.

```{r, out.width = "700px", echo = FALSE}
knitr::include_graphics("figures/09-Results_bySample.png")
```

$~$

## Export

The **Export** tab manages the saving of input parameters, annotation results 
and automated reporting.

The `peakPantheRAnnotation` in it's current state can be saved as a `.RData` 
file which can subsequently be reloaded. The `.csv` files defining the current 
analysis can also be exported to reproduce the current processing.

All diagnostic plots from the `Diagnostic` tab can be automatically saved to 
disk for rapid evaluation. This can be executed in parallel if a large number 
of plots have to be generated.

Finally each fitting property can be saved as a `.csv` file with all samples as 
_rows_ and all targeted features as _columns_. Additionally a _summary_ table 
will present the integration success rate for each targeted feature.

```{r, out.width = "700px", echo = FALSE}
knitr::include_graphics("figures/10-Export.png")
```

$~$

# Final Note

If a very high number of targeted features and samples are to be processed, 
`peakPantheR`'s command line functions are more efficient, as they can be 
integrated in scripts and the reporting automated.

$~$

# See Also

* [Getting Started with peakPantheR](getting-started.html)
* [Real Time Annotation](real-time-annotation.html)
* [Parallel Annotation](parallel-annotation.html)

$~$

# Session Information
```{r, echo = FALSE}
devtools::session_info()
```

---
title: "Parallel Annotation"
date: "2020-10-11"
package: peakPantheR
output:
    BiocStyle::html_document:
        toc_float: true
vignette: >
    %\VignetteIndexEntry{Parallel Annotation}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
    %\VignetteDepends{peakPantheR,faahKO,pander,BiocStyle,foreach,doParallel}
    %\VignettePackage{peakPantheR}
    %\VignetteKeywords{mass spectrometry, metabolomics}
---

```{r biocstyle, echo = FALSE, results = "asis" }
BiocStyle::markdown()
```
```{r, echo = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
```

**Package**: `r Biocpkg("peakPantheR")`<br />
**Authors**: Arnaud Wolfer, Goncalo Correia<br />

```{r init, message = FALSE, echo = FALSE, results = "hide" }
## Silently loading all packages
library(BiocStyle)
library(peakPantheR)
library(faahKO)
library(pander)
library(doParallel)
library(foreach)
```

# Introduction

The `peakPantheR` package is designed for the detection, integration and 
reporting of pre-defined features in MS files (_e.g. compounds, fragments, 
adducts, ..._).

The **Parallel Annotation** is set to detect and integrate **multiple** 
compounds in **multiple** files in **parallel** and store results in a 
**single** object. It can be employed to integrate a large number of expected 
features across a dataset.

Using the `r Biocpkg("faahKO")` raw MS dataset as an example, this vignette 
will:

* Detail the **Parallel Annotation** concept
* Apply the **Parallel Annotation** to a subset of pre-defined features in the 
`r Biocpkg("faahKO")` dataset

## Abbreviations
- **ROI**: _Regions Of Interest_ 
    * reference _RT_ / _m/z_ windows in which to search for a feature
- **uROI**: _updated Regions Of Interest_
    * modifed ROI adapted to the current dataset which override the reference 
    ROI
- **FIR**: _Fallback Integration Regions_ 
    * _RT_ / _m/z_ window to integrate if no peak is found
- **TIC**: _Total Ion Chromatogram_ 
    * the intensities summed across all masses for each scan
- **EIC**: _Extracted Ion Chromatogram_ 
    * the intensities summed over a mass range, for each scan


# Parallel Annotation Concept

Parallel compound integration is set to process **multiple** compounds in 
**multiple** files in **parallel**, and store results in a **single** object.


```{r, out.width = "700px", echo = FALSE}
knitr::include_graphics("figures/parallelAnnotation.png")
```

To achieve this, `peakPantheR` will:

1. load a list of expected _RT_ / _m/z_ ROI and a list of files to process
2. initialise an output object with expected ROI and file paths
3. first pass (_without peak filling_) on a subset of representative samples 
(e.g QC samples):
    + for each file, detect features in each ROI and keep highest intensity
    + determine peak statistics for each feature
    + store results + EIC for each ROI
4. visual inspection of first pass results, update ROI:
    + diagnostic plots: all EICs, peak apex _RT_ / _m/z_ & peak width evolution
    + correct ROI (remove interfering feature, correct _RT_ shift)
    + define fallback integration regions (FIR) if no feature is detected 
    (median _RT_ / _m/z_ start and end of found features)
5. initialise a new output object, with updated regions of interest (uROI) and 
fallback integration regions (FIR), with all samples
6. second pass (_with peak filling_) on all samples:
    + for each file, detect features in each uROI and keep highest intensity
    + determine peak statistics for each feature
    + integrate FIR when no peaks are found
    + store results + EIC for each uROI
7. summary statistics:
    + plot EICs, apex and peakwidth evolution
    + compare first and second pass
8. return the resulting object and/or table (_row: file, col: compound_)


```{r, out.width = "700px", echo = FALSE}
knitr::include_graphics("figures/parallelAnnotation_procedure.png")
```

> Diagram of the workflow and functions used for parallel annotation.


# Parallel Annotation Example

We can target 2 pre-defined features in 6 raw MS spectra file from the 
`r Biocpkg("faahKO")` package using `peakPantheR_parallelAnnotation()`. For more
details on the installation and input data employed, please consult the 
[Getting Started with peakPantheR](getting-started.html) vignette.


## Input Data

First the paths to 3 MS file from the `r Biocpkg("faahKO")` are located and used
as input spectras. In this example these 3 samples are considered as 
representative of the whole run (e.g. Quality Control samples):
```{r}
library(faahKO)
## file paths
input_spectraPaths  <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                        system.file('cdf/KO/ko16.CDF', package = "faahKO"),
                        system.file('cdf/KO/ko18.CDF', package = "faahKO"))
input_spectraPaths
```

Two targeted features (_e.g. compounds, fragments, adducts, ..._) are defined 
and stored in a table with as columns:

* `cpdID` (character)
* `cpdName` (character)
* `rtMin` (sec)
* `rtMax` (sec)
* `rt` (sec, optional / `NA`)
* `mzMin` (m/z)
* `mzMax` (m/z)
* `mz` (m/z, optional / `NA`)

```{r, eval = FALSE}
# targetFeatTable
input_targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), 
                        c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", 
                            "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390., 
                                522.194778, 522.2, 522.205222)
input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440., 
                                496.195038, 496.2, 496.204962)
input_targetFeatTable[,c(3:8)] <- sapply(input_targetFeatTable[,c(3:8)], 
                                        as.numeric)
```
```{r, results = "asis", echo = FALSE}
# use pandoc for improved readability
input_targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), 
                        c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", 
                        "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390., 
                                522.194778, 522.2, 522.205222)
input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440., 
                                496.195038, 496.2, 496.204962)
input_targetFeatTable[,c(3:8)] <- sapply(input_targetFeatTable[,c(3:8)], 
                                        as.numeric)
rownames(input_targetFeatTable) <- NULL
pander::pandoc.table(input_targetFeatTable, digits = 9)
```


Additional compound and spectra metadata can be provided but isn't employed 
during the fitting procedure:

```{r, eval=FALSE}
# spectra Metadata
input_spectraMetadata  <- data.frame(matrix(c("sample type 1", "sample type 2", 
                            "sample type 1"), 3, 1, 
                            dimnames=list(c(),c("sampleType"))),
                            stringsAsFactors=FALSE)
```
```{r, results = "asis", echo = FALSE}
# use pandoc for improved readability
input_spectraMetadata  <- data.frame(matrix(c("sample type 1", "sample type 2", 
                                            "sample type 1"), 3, 1, 
                                            dimnames=list(c(),c("sampleType"))),
                                    stringsAsFactors=FALSE)
pander::pandoc.table(input_spectraMetadata)
```


## Initialise and Run Parallel Annotation

A `peakPantheRAnnotation` object is first initialised with the path to the files
to process (`spectraPaths`), features to integrate (`targetFeatTable`) and 
additional information and parameters such as `spectraMetadata`, `uROI`, `FIR` 
and if they should be used (`useUROI=TRUE`, `useFIR=TRUE`):
```{r}
library(peakPantheR)
init_annotation <- peakPantheRAnnotation(spectraPaths = input_spectraPaths,
                        targetFeatTable = input_targetFeatTable,
                        spectraMetadata = input_spectraMetadata)
```

The resulting `peakPantheRAnnotation` object is not annotated, does not contain 
and use `uROI` and `FIR`
```{r}
init_annotation
```

`peakPantheR_parallelAnnotation()` will run the annotation across files in 
parallel (if `ncores` >0) and return the successful annotations 
(`result$annotation`) and failures (`result$failures`):
```{r}
# annotate files serially
annotation_result <- peakPantheR_parallelAnnotation(init_annotation, ncores=0,
                                                    curveModel='skewedGaussian',
                                                    verbose=TRUE)

# successful fit
nbSamples(annotation_result$annotation)
data_annotation   <- annotation_result$annotation
data_annotation

# list failed fit
annotation_result$failures
```


## Process Parallel Annotation Results

Based on the fit results, updated ROI (`uROI`) and fallback integration region 
(`FIR`) can be automatically determined using `annotationParamsDiagnostic()`:

* `uROI` are established as the min/max (`rt` and `m/z`) of the found peaks 
(+/- 5% in RT)
* `FIR` are established as the median of found `rtMin`, `rtMax`, `mzMin`,
`mzMax`

```{r}
updated_annotation  <- annotationParamsDiagnostic(data_annotation, verbose=TRUE)

# uROI now exist
updated_annotation
```

`outputAnnotationDiagnostic()` will save to disk 
`annotationParameters_summary.csv` containing the original `ROI` and newly 
determined `uROI` and `FIR` for manual validation. Additionnaly a diagnostic 
plot for each compound is saved for reference and can be generated in parallel 
with the argument `ncores`:
```{r, eval=FALSE}
# create a colourScale based on the sampleType
uniq_sType <- sort(unique(spectraMetadata(updated_annotation)$sampleType),
                    na.last=TRUE)
col_sType  <- unname( setNames(c('blue', 'red'),
                c(uniq_sType))[spectraMetadata(updated_annotation)$sampleType] )

# create a temporary location to save the diagnotic (otherwise provide the path
# to the selected location)
output_folder <- tempdir()

# output fit diagnostic to disk
outputAnnotationDiagnostic(updated_annotation, saveFolder=output_folder, 
                            savePlots=TRUE, sampleColour=col_sType, 
                            verbose=TRUE, ncores=2)
```

The data saved in `annotationParameters_summary.csv` is as follow:
```{r, results = "asis", echo = FALSE}
# use pandoc for improved readability, display the diagnostic results
tmp_csv <- data.frame(matrix(nrow=2,ncol=21,dimnames=list(c(), c('cpdID', 
        'cpdName', 'X', 'ROI_rt', 'ROI_mz','ROI_rtMin', 'ROI_rtMax', 
        'ROI_mzMin', 'ROI_mzMax', 'X', 'uROI_rtMin', 'uROI_rtMax', 'uROI_mzMin',
        'uROI_mzMax', 'uROI_rt', 'uROI_mz', 'X', 'FIR_rtMin', 'FIR_rtMax', 
        'FIR_mzMin', 'FIR_mzMax'))), stringsAsFactors=FALSE)
tmp_csv[1,] <- c('ID-1','Cpd 1', '|', 3344.888, 522.2, 3310., 3390., 522.194778,
                522.205222,'|', 3305.75893, 3411.436284, 522.194778, 522.205222,
                3344.888, 522.2, '|', 3326.10635, 3407.272648, 522.194778, 
                522.205222)
tmp_csv[2,] <- c('ID-2','Cpd 2', '|', 3385.577, 496.2, 3280., 3440., 496.195038,
                496.204962,'|',3337.376665, 3462.449033, 496.195038, 496.204962,
                3385.577, 496.2, '|', 3365.023857, 3453.404957, 496.195038, 
                496.204962)
tmp_csv[,-c(1,2,3,10,17)]  <- sapply(tmp_csv[,-c(1,2,3,10,17)], as.numeric)
colnames(tmp_csv) <- c('cpdID', 'cpdName', 'X', 'ROI_rt', 'ROI_mz','ROI_rtMin', 
                    'ROI_rtMax', 'ROI_mzMin', 'ROI_mzMax', 'X', 'uROI_rtMin', 
                    'uROI_rtMax', 'uROI_mzMin', 'uROI_mzMax', 'uROI_rt', 
                    'uROI_mz', 'X', 'FIR_rtMin', 'FIR_rtMax', 'FIR_mzMin', 
                    'FIR_mzMax')
pander::pandoc.table(tmp_csv, digits=9)
```

```{r, out.width = "700px", echo = FALSE}
knitr::include_graphics(
    "figures/parallel_annotation_diagnostic_cpd1.png")
```

> Diagnostic plot for compound 1: The top panel is an overlay of the extracted 
EIC across all samples with the fitted curve as dotted line. The panel under the
EIC represent each found peak RT peakwidth (`rtMin`, `rtMax` and apex marked as 
dot), ordered with the first sample at the top. The bottom 3 panels represent 
found `RT` (peakwidth), `m/z` (peakwidth) and `peak area` by run order, with the
corresponding histograms to the right


`ROI` exported to `.csv` can be updated based on the diagnostic plots; `uROI` 
(updated ROI potentially used for all samples) and `FIR` (fallback integration 
regions for when no peak is found) can also be tweaked to better fit the peaks.


### Retention time correction
The optional `retentionTimeCorrection()` method provides an interface to adjust
the expected ROI rt values and account for chromatographic batch effects. By
comparing expected and found rt values for a set of reference compounds, a model
of the chromatographic shift for the present batch can be established. This 
model can be in turned used to correct the expected retention time of all
targeted compounds.
In order to apply this method, the `peakPantheRAnnotation` must be previously
annotated (`isAnnotated=TRUE`). 
The retention time correction algorithm to use can be selected using the 
`method` argument (currently `polynomial` and
`constant` methods are available).
`retentionTimeCorrection()` fits a correction function to model the dependency
of the mean `rt_dev_sec` per reference feature with the expected databased 
retention time. 
If `useUROI=TRUE`, the expected retention time value is taken from the `UROI_rt`
field, otherwise `ROI_rt` is used. 
If `robust=TRUE`, the RANSAC algorithm is used to automatically detect outliers
and exclude them from the fit (this should only be used with a large number of 
reference features).
`retentionTimeCorrection()` returns a list with 2 elements:
* a modified `peakPantheRAnnotation` object
* a `ggplot2` diagnostic plot (optional, depending on whether `TRUE` or `FALSE`
is passed to the `diagnostic` argument).
The returned `peakPantheRAnnotation` object contains the same `uROI` and `FIR` 
`mz` values as the original annotation, but the retention time related 
parameters (`rt`, `rtMin` and `rtMax`) are replaced by the adjusted values. 
The `rtMax` and `rtMin` are set as the corrected `rt` value plus or minus half 
the value passed to the `rtWindowWidth` argument, respectively. 
`useUROI` is also set to TRUE.
To continue with the workflow, simply set a new annotation object with the 
fit parameters established by `retentionTimeCorrection()` and call 
`peakPantheR_parallelAnnotation()` for the final annotation.

```{r, results="asis", echo=FALSE}
# Example with constant correction.
rtCorrectionOutput <- retentionTimeCorrection(updated_annotation,
                            rtCorrectionReferences=c('ID-1'),
                            method='constant',
                            robust=FALSE,
                            rtWindowWidth=15,
                            diagnostic=TRUE)
updated_annotation <- rtCorrectionOutput$annotation
# The ggplot2 plot object
rtCorrectionOutput$plot

# Example with second degree polynomial, without using RANSAC
# # to obtain a robust fit
rtCorrectionOutput <- retentionTimeCorrection(updated_annotation,
                            rtCorrectionReferences=NULL,
                            method='polynomial',
                            params=list(polynomialOrder=2),
                            robust=FALSE, rtWindowWidth=15,
                            diagnostic=TRUE)
```


## New Initialisation with Updated Parameters to be Applied to All Study Samples

Following this manual validation of the fit on reference samples, the modified 
parameters in the `.csv` file can be reloaded and applied to all study samples.


### Load new fit parameters

`peakPantheR_loadAnnotationParamsCSV()` will load the new `.csv` parameters (as 
generated by `outputAnnotationDiagnostic()`) and initialise a 
`peakPantheRAnnotation` object without `spectraPaths`, `spectraMetadata` or 
`cpdMetadata` which will need to be added before annotation. `useUROI` and 
`useFIR` are set to `FALSE` by default and will need to be modified according to
the analysis to run. `uROIExist` is established depending on the `.csv` uROI 
column, and will only be set to TRUE if no `NA` are present. It is possible to 
reset the `FIR` values with the `uROI` windows using `resetFIR()`. 

```{r, eval=FALSE}
update_csv_path <- '/path_to_new_csv/'

# load csv
new_annotation <- peakPantheR_loadAnnotationParamsCSV(update_csv_path)
#> uROIExist set to TRUE
#> New peakPantheRAnnotation object initialised for 2 compounds

new_annotation
#> An object of class peakPantheRAnnotation
#>  2 compounds in 0 samples. 
#>   updated ROI exist (uROI)
#>   does not use updated ROI (uROI)
#>   does not use fallback integration regions (FIR)
#>   is not annotated

new_annotation <- resetFIR(new_annotation)
#> FIR will be reset with uROI values
```

### Add new samples to process

Now that the fit parameters were set on 3 representative samples (e.g. QC), the 
same processing can be applied to all study samples. `resetAnnotation()` will 
reinitialise all the results and modify the samples or compounds targeted as 
required:

```{r}
## new files
new_spectraPaths   <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                        system.file('cdf/WT/wt15.CDF', package = "faahKO"),
                        system.file('cdf/KO/ko16.CDF', package = "faahKO"),
                        system.file('cdf/WT/wt16.CDF', package = "faahKO"),
                        system.file('cdf/KO/ko18.CDF', package = "faahKO"),
                        system.file('cdf/WT/wt18.CDF', package = "faahKO"))

new_spectraPaths
```

Below we define the metadata of these new samples:
```{r}
## new spectra metadata
new_spectraMetadata  <- data.frame(matrix(c("KO", "WT", "KO", "WT", "KO", "WT"),
                                        6, 1, dimnames=list(c(), c("Group"))), 
                                    stringsAsFactors=FALSE)
```
```{r, results = "asis", echo = FALSE}
# use pandoc for improved readability
new_spectraMetadata  <- data.frame(matrix(c("KO", "WT", "KO", "WT", "KO", "WT"),
                                        6, 1, dimnames=list(c(), c("Group"))), 
                                    stringsAsFactors=FALSE)
pander::pandoc.table(new_spectraMetadata)
```

```{r, echo=FALSE}
new_annotation <- resetAnnotation(updated_annotation, 
                                spectraPaths=new_spectraPaths, 
                                spectraMetadata=new_spectraMetadata, 
                                useUROI=TRUE, useFIR=TRUE, verbose=FALSE) 
```

```{r, eval=FALSE}
## add new samples to the annotation loaded from csv, useUROI, useFIR

new_annotation <- resetAnnotation(new_annotation, spectraPaths=new_spectraPaths,
                                spectraMetadata=new_spectraMetadata, 
                                useUROI=TRUE, useFIR=TRUE)
#> peakPantheRAnnotation object being reset:
#>   Previous "ROI", "cpdID" and "cpdName" value kept
#>   Previous "uROI" value kept
#>   Previous "FIR" value kept
#>   Previous "cpdMetadata" value kept
#>   New "spectraPaths" value set
#>   New "spectraMetadata" value set
#>   Previous "uROIExist" value kept
#>   New "useUROI" value set
#>   New "useFIR" value set
```
```{r}
new_annotation
```


## Run Final Parallel Annotation

We can now run the final annotation on all samples with the optimised targeted 
features:
```{r}
# annotate files serially
new_annotation_result <- peakPantheR_parallelAnnotation(new_annotation, 
                                                        ncores=0, verbose=FALSE)

# successful fit
nbSamples(new_annotation_result$annotation)

final_annotation      <- new_annotation_result$annotation
final_annotation

# list failed fit
new_annotation_result$failures
```


### Output final results

The final fits can be saved to disk with `outputAnnotationDiagnostic()`:
```{r, eval=FALSE}
# create a colourScale based on the sampleType
uniq_group <- sort(unique(spectraMetadata(final_annotation)$Group),na.last=TRUE)
col_group  <- unname( setNames(c('blue', 'red'),
                    c(uniq_sType))[spectraMetadata(final_annotation)$Group] )

# create a temporary location to save the diagnotic (otherwise provide the path
# to the selected location)
final_output_folder <- tempdir()

# output fit diagnostic to disk
outputAnnotationDiagnostic(final_annotation, saveFolder=final_output_folder,
                        savePlots=TRUE, sampleColour=col_group, verbose=TRUE)
```

For each processed sample, a `peakTables` contains all the fit information for 
all compounds targeted. `annotationTable( , column)` will group the values 
across all samples and compounds for any `peakTables` column:
```{r, eval=FALSE}
# peakTables for the first sample
peakTables(final_annotation)[[1]]
```
```{r, results = "asis", echo = FALSE}
# use pandoc for improved readability
pander::pandoc.table(peakTables(final_annotation)[[1]])
```
```{r, eval=FALSE}
# Extract the found peak area for all compounds and all samples
annotationTable(final_annotation, column='peakArea')
```
```{r, results = "asis", echo = FALSE}
# use pandoc for improved readability
pander::pandoc.table(annotationTable(final_annotation, column='peakArea'))
```

Finally all annotation results can be saved to disk as `.csv` with 
`outputAnnotationResult()`. These `.csv` will contain the compound metadata, 
spectra metadata and a file for each column of peakTables (with samples as rows 
and compounds as columns):

```{r, eval=FALSE}
# create a temporary location to save the diagnotic (otherwise provide the path
# to the selected location)
final_output_folder <- tempdir()

# save
outputAnnotationResult(final_annotation, saveFolder=final_output_folder, 
                        annotationName='ProjectName', verbose=TRUE)
#> Compound metadata saved at /final_output_folder/ProjectName_cpdMetadata.csv
#> Spectra metadata saved at 
#>     /final_output_folder/ProjectName_spectraMetadata.csv
#> Peak measurement "found" saved at /final_output_folder/ProjectName_found.csv
#> Peak measurement "rtMin" saved at /final_output_folder/ProjectName_rtMin.csv
#> Peak measurement "rt" saved at /final_output_folder/ProjectName_rt.csv
#> Peak measurement "rtMax" saved at /final_output_folder/ProjectName_rtMax.csv
#> Peak measurement "mzMin" saved at /final_output_folder/ProjectName_mzMin.csv
#> Peak measurement "mz" saved at /final_output_folder/ProjectName_mz.csv
#> Peak measurement "mzMax" saved at /final_output_folder/ProjectName_mzMax.csv
#> Peak measurement "peakArea" saved at 
#>     /final_output_folder/ProjectName_peakArea.csv
#> Peak measurement "maxIntMeasured" saved at 
#>     /final_output_folder/ProjectName_maxIntMeasured.csv
#> Peak measurement "maxIntPredicted" saved at 
#>     /final_output_folder/ProjectName_maxIntPredicted.csv
#> Peak measurement "is_filled" saved at 
#>     /final_output_folder/ProjectName_is_filled.csv
#> Peak measurement "ppm_error" saved at 
#>     /final_output_folder/ProjectName_ppm_error.csv
#> Peak measurement "rt_dev_sec" saved at 
#>     /final_output_folder/ProjectName_rt_dev_sec.csv
#> Peak measurement "tailingFactor" saved at 
#>     /final_output_folder/ProjectName_tailingFactor.csv
#> Peak measurement "asymmetryFactor" saved at 
#>     /final_output_folder/ProjectName_asymmetryFactor.csv
#> Summary saved at /final_output_folder/ProjectName_summary.csv
```

# See Also

* [Getting Started with peakPantheR](getting-started.html)
* [Real Time Annotation](real-time-annotation.html)
* [Graphical user interface use](peakPantheR-GUI.html)


# Session Information
```{r, echo = FALSE}
devtools::session_info()
```

---
title: "Getting Started with the peakPantheR package"
date: "2020-10-11"
package: peakPantheR
output:
    BiocStyle::html_document:
        toc_float: true
bibliography: references.bib
vignette: >
    %\VignetteIndexEntry{Getting Started with the peakPantheR package}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
    %\VignetteDepends{peakPantheR,faahKO,pander,BiocStyle}
    %\VignettePackage{peakPantheR}
    %\VignetteKeywords{mass spectrometry, metabolomics}
---

```{r biocstyle, echo = FALSE, results = "asis" }
BiocStyle::markdown()
```
```{r, echo = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
```

**Package**: `r Biocpkg("peakPantheR")`<br />
**Authors**: Arnaud Wolfer<br />

```{r init, message = FALSE, echo = FALSE, results = "hide" }
## Silently loading all packages
library(BiocStyle)
library(peakPantheR)
library(faahKO)
library(pander)
```


Package for _Peak Picking and ANnoTation of High resolution Experiments in R_,
implemented in `R` and `Shiny`


# Overview

`peakPantheR` implements functions to detect, integrate and report pre-defined
features in MS files (_e.g. compounds, fragments, adducts, ..._).

It is designed for:

* **Real time** feature detection and integration (see
[Real Time Annotation](real-time-annotation.html))
    + process `multiple` compounds in `one` file at a time
* **Post-acquisition** feature detection, integration and reporting (see 
[Parallel Annotation](parallel-annotation.html))
    + process `multiple` compounds in `multiple` files in `parallel`, store 
    results in a `single` object

`peakPantheR` can process LC/MS data files in _NetCDF_, _mzML_/_mzXML_ and 
_mzData_ format as data import is achieved using Bioconductor's 
`r Biocpkg("mzR")` package. 

# Installation

To install `peakPantheR` from Bioconductor:
```{r, eval = FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("peakPantheR")
```

Install the development version of `peakPantheR` directly from GitHub with:
```{r, eval = FALSE}
# Install devtools
if(!require("devtools")) install.packages("devtools")
devtools::install_github("phenomecentre/peakPantheR")
```


## Getting Started

To get started `peakPantheR`'s [graphical user interface](peakPantheR-GUI.html)
implements all the functions to detect and integrate **multiple** compounds in
**multiple** files in **parallel** and store results in a **single** object. It
can be employed to integrate a large number of expected features across a
dataset:
```{r, eval = FALSE}
library(peakPantheR)

peakPantheR_start_GUI(browser = TRUE)
#  To exit press ESC in the command line
```
```{r, fig.align="center", out.width = "700px", echo = FALSE}
knitr::include_graphics("figures/example-UI.png")
```

The GUI is to be preferred to understand the methodology, select the best
parameters on a subset of the samples before running the command line, or to
visually explore results.

If a very high number of samples and features is to be processed,
`peakpantheR`'s command line functions are more efficient, as they can
be integrated in scripts and the reporting automated.


# Input Data

Both real time and parallel compound integration require a common set of 
information:

* Path(s) to `netCDF` / `mzML` MS file(s)
* An expected region of interest (`RT` / `m/z` window) for each compound.


## MS files

For demonstration purpose we can annotate a set a set of raw MS spectra (in 
_NetCDF_ format) provided by the `r Biocpkg("faahKO")` package. Briefly, this 
subset of the data from [@Saghatelian04] invesigate the metabolic consequences 
of knocking out the fatty acid amide hydrolase (FAAH) gene in mice. The dataset 
consists of samples from the spinal cords of 6 knock-out and 6 wild-type mice. 
Each file contains data in centroid mode acquired in positive ion mode form 
200-600 m/z and 2500-4500 seconds.

Below we install the `r Biocpkg("faahKO")` package and locate raw CDF files of 
interest:
```{r, eval = FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("faahKO")
```
```{r}
library(faahKO)
## file paths
input_spectraPaths  <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                        system.file('cdf/KO/ko16.CDF', package = "faahKO"),
                        system.file('cdf/KO/ko18.CDF', package = "faahKO"))
input_spectraPaths
```

## Expected regions of interest

Expected regions of interest (targeted features) are specified using the 
following information:

* `cpdID` (character)
* `cpdName` (character)
* `rtMin` (sec)
* `rtMax` (sec)
* `rt` (sec, optional / `NA`)
* `mzMin` (m/z)
* `mzMax` (m/z)
* `mz` (m/z, optional / `NA`)

Below we define 2 features of interest that are present in the 
`r Biocpkg("faahKO")` dataset and can be employed in subsequent vignettes:
```{r, eval=FALSE}
# targetFeatTable
input_targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), 
                        c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", 
                        "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390., 522.194778, 
                                522.2, 522.205222)
input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440., 496.195038,
                                496.2, 496.204962)
input_targetFeatTable[,c(1,3:8)] <- sapply(input_targetFeatTable[,c(1,3:8)], 
                                            as.numeric)
```
```{r, results = "asis", echo = FALSE}
# use pandoc for improved readability
input_targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), 
                        c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin", 
                        "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390., 522.194778, 
                                522.2, 522.205222)
input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440., 496.195038,
                                496.2, 496.204962)
input_targetFeatTable[,c(1,3:8)] <- sapply(input_targetFeatTable[,c(1,3:8)], 
                                            as.numeric)
rownames(input_targetFeatTable) <- NULL
pander::pandoc.table(input_targetFeatTable, digits = 9)
```

# Preparing input for the graphical user interface

While the command line functions accept Data.Frame and vectors as input, the
graphical user interface (GUI) will read the same information from a set of
`.csv` files, or an already set-up `peakPantheRAnnotation` object in `.RData`
format.

We can now generate GUI input files for the `r Biocpkg("faahKO")` example
dataset presented previously:

## _peakPantheRAnnotation_ .RData

A `peakPantheRAnnotation` (previously annotated or not) can be passed as input
in a `.RData` file. The `peakPantheRAnnotation` object must be named
_annotationObject_:

```{r, eval=FALSE}
library(faahKO)

# Define the file paths (3 samples)
input_spectraPaths  <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                        system.file('cdf/KO/ko16.CDF', package = "faahKO"),
                        system.file('cdf/KO/ko18.CDF', package = "faahKO"))

# Define the targeted features (2 features)
input_targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
                        c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin",
                        "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390.,
                                522.194778, 522.2, 522.205222)
input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440.,
                                496.195038, 496.2, 496.204962)
input_targetFeatTable[,3:8] <- sapply(input_targetFeatTable[,3:8], as.numeric)

# Define some random compound and spectra metadata
# cpdMetadata
input_cpdMetadata     <- data.frame(matrix(data=c('a','b',1,2), nrow=2, ncol=2,
                            dimnames=list(c(), c('testcol1','testcol2')),
                            byrow=FALSE), stringsAsFactors=FALSE)
# spectraMetadata
input_spectraMetadata <- data.frame(matrix(data=c('c','d','e',3,4,5), nrow=3,
                            ncol=2,
                            dimnames=list(c(),c('testcol1','testcol2')),
                            byrow=FALSE), stringsAsFactors=FALSE)

# Initialise a simple peakPantheRAnnotation object
# [3 files, 2 features, no uROI, no FIR]
initAnnotation <- peakPantheRAnnotation(spectraPaths=input_spectraPaths,
                                        targetFeatTable=input_targetFeatTable,
                                        cpdMetadata=input_cpdMetadata,
                                        spectraMetadata=input_spectraMetadata)

# Rename and save the annotation to disk
annotationObject <- initAnnotation
save(annotationObject,
        file = './example_annotation_ppR_UI.RData',
        compress=TRUE)

```

## CSV file input

Another input option for the GUI input consists of a set of `.csv` files.

### Targeted features
Targeted features are defined in a `.csv` with as rows each feature to target
(the first row must be the column name), and as columns the fit parameters to
use. At minimum the following parameters must be defined:

`cpdID`, `cpdName`, `rtMin`, `rt`, `rtMax`, `mzMin`, `mz`, `mzMax`

If `uROI` and `FIR` are to be set, the following columns must be provided:

`cpdID`, `cpdName`, `ROI_rt`, `ROI_mz`, `ROI_rtMin`, `ROI_rtMax`, `ROI_mzMin`,
`ROI_mzMax`, `uROI_rtMin`, `uROI_rtMax`, `uROI_mzMin`, `uROI_mzMax`, `uROI_rt`,
`uROI_mz`, `FIR_rtMin`, `FIR_rtMax`, `FIR_mzMin`, `FIR_mzMax`

```{r, eval=FALSE}
# Define targeted features without uROI and FIR (2 features)
input_targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
                        c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin",
                        "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390.,
                                522.194778, 522.2, 522.205222)
input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440.,
                                496.195038, 496.2, 496.204962)
input_targetFeatTable[,3:8] <- sapply(input_targetFeatTable[,3:8], as.numeric)

# save to disk
write.csv(input_targetFeatTable,
            file = './1-fitParams_example_UI.csv',
            row.names = FALSE)
```
```{r, results = "asis", echo = FALSE}
# use pandoc for improved readability
input_targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
                        c("cpdID", "cpdName", "rtMin", "rt", "rtMax", "mzMin",
                        "mz", "mzMax"))), stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c("ID-1", "Cpd 1", 3310., 3344.888, 3390.,
                                522.194778, 522.2, 522.205222)
input_targetFeatTable[2,] <- c("ID-2", "Cpd 2", 3280., 3385.577, 3440.,
                                496.195038, 496.2, 496.204962)
input_targetFeatTable[,3:8] <- sapply(input_targetFeatTable[,3:8], as.numeric)
rownames(input_targetFeatTable) <- NULL
pander::pandoc.table(input_targetFeatTable, digits = 9)
```

### Files to process and spectra metadata (optional)
It is possible to select the files on disk directly through the GUI, or to
select a `.csv` file containing each file path as well as spectra metadata.
Each row correspond to a different spectra (the first row must define the
column names) while columns correspond to the path on disk and metadata. At
minimum a column `filepath` must be present, with subsequent columns defining
metadata properties.

```{r, eval=FALSE}
# Define the spectra paths and metada
input_spectraMeta <- data.frame(matrix(vector(), 3, 3,
                        dimnames=list(c(),c("filepath","testcol1","testcol2"))),
                        stringsAsFactors=FALSE)
input_spectraMeta[1,] <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                            "c", 3)
input_spectraMeta[2,] <- c(system.file('cdf/KO/ko16.CDF', package = "faahKO"),
                            "d", 4)
input_spectraMeta[3,] <- c(system.file('cdf/KO/ko18.CDF', package = "faahKO"),
                            "e", 5)

# save to disk
write.csv(input_spectraMeta,
            file = './2-spectraMetaWPath_example_UI.csv',
            row.names = FALSE)
```
```{r, results = "asis", echo = FALSE}
# use pandoc for improved readability
input_spectraMeta <- data.frame(matrix(vector(), 3, 3,
                        dimnames=list(c(),c("filepath","testcol1","testcol2"))),
                        stringsAsFactors=FALSE)
input_spectraMeta[1,] <- c(system.file('cdf/KO/ko15.CDF', package = "faahKO"),
                            "c", 3)
input_spectraMeta[2,] <- c(system.file('cdf/KO/ko16.CDF', package = "faahKO"),
                            "d", 4)
input_spectraMeta[3,] <- c(system.file('cdf/KO/ko18.CDF', package = "faahKO"),
                            "e", 5)
rownames(input_spectraMeta) <- NULL
pander::pandoc.table(input_spectraMeta, digits = 0)
```

### Feature meatadata (optional)
It is possible to define some feature metadata, with targeted features as
rows (same order as the fitting parameters, first row defining the column
names), and as columns the metadata.

```{r, eval=FALSE}
# Define the feature metada
input_featMeta <- data.frame(matrix(vector(), 2, 2,
                    dimnames=list(c(),c("testcol1","testcol2"))),
                    stringsAsFactors=FALSE)
input_featMeta[1,] <- c("a", 1)
input_featMeta[2,] <- c("b", 2)

# save to disk
write.csv(input_featMeta,
            file = './3-featMeta_example_UI.csv',
            row.names = FALSE)
```
```{r, results = "asis", echo = FALSE}
# use pandoc for improved readability
input_featMeta <- data.frame(matrix(vector(), 2, 2,
                    dimnames=list(c(),c("testcol1","testcol2"))),
                    stringsAsFactors=FALSE)
input_featMeta[1,] <- c("a", 1)
input_featMeta[2,] <- c("b", 2)
rownames(input_featMeta) <- NULL
pander::pandoc.table(input_featMeta, digits = 0)
```

# See Also

* [Real Time Annotation](real-time-annotation.html)
* [Parallel Annotation](parallel-annotation.html)
* [Graphical user interface use](peakPantheR-GUI.html)

# Session Information

```{r, echo = FALSE}
devtools::session_info()
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GUI_utils.R
\name{outputAnnotationFeatureMetadata_UI_helper}
\alias{outputAnnotationFeatureMetadata_UI_helper}
\title{UI export helper - feature metadata}
\usage{
outputAnnotationFeatureMetadata_UI_helper(annot)
}
\arguments{
\item{annot}{(peakPantheRAnnotation) Annotation object}
}
\value{
(data.frame) Features metadata
}
\description{
Return a table with features as rows and all feature metadata as columns
}
\examples{
## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
## compounds

# Paths to spectra files
spectraPaths <- c('./path/file1', './path/file2', './path/file3')

# targetFeatTable
targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
                    c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
                    'mzMax'))), stringsAsFactors=FALSE)
targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
                        522.2, 522.205222)
targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
                        496.2, 496.204962)
targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
                                    FUN.VALUE=numeric(2))

emptyAnnotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
                                        targetFeatTable=targetFeatTable)

# featureMetadata of an empty annotation
outputAnnotationFeatureMetadata_UI_helper(emptyAnnotation)
# data frame with 0 columns and 2 rows
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_peakPantheRAnnotation.R
\docType{methods}
\name{resetAnnotation,peakPantheRAnnotation-method}
\alias{resetAnnotation,peakPantheRAnnotation-method}
\alias{resetAnnotation}
\title{Reset a peakPantheRAnnotation and alter samples and compounds
information}
\usage{
\S4method{resetAnnotation}{peakPantheRAnnotation}(
    previousAnnotation,
    spectraPaths = NULL,
    targetFeatTable = NULL,
    uROI = NULL,
    FIR = NULL,
    cpdMetadata = NULL,
    spectraMetadata = NULL,
    uROIExist = NULL,
    useUROI = NULL,
    useFIR = NULL,
    verbose = TRUE,
    ...
)
}
\arguments{
\item{previousAnnotation}{(peakPantheRAnnotation) object to reset}

\item{spectraPaths}{NULL or a character vector of spectra file paths, to set
samples to process}

\item{targetFeatTable}{NULL or a \code{\link{data.frame}} of compounds to
target as rows and parameters as columns: \code{cpdID} (str), \code{cpdName}
(str), \code{rtMin} (float in seconds), \code{rt} (float in seconds, or
\emph{NA}), \code{rtMax} (float in seconds), \code{mzMin} (float), \code{mz}
(float or \emph{NA}), \code{mzMax} (float). Set compounds to target.}

\item{uROI}{NULL or a data.frame of updated Regions Of Interest (uROI) with
compounds as row and uROI parameters as columns: \code{rtMin} (float in
seconds), \code{rt} (float in seconds, or \emph{NA}), \code{rtMax} (float in
seconds), \code{mzMin} (float), \code{mz} (float or \emph{NA}), \code{mzMax}
(float).}

\item{FIR}{NULL or a data.frame of Fallback Integration Regions (FIR) with
compounds as row and FIR parameters as columns: \code{rtMin} (float in
seconds), \code{rtMax} (float in seconds), \code{mzMin} (float), \code{mzMax}
(float).}

\item{cpdMetadata}{NULL or a data.frame of compound metadata, with compounds
as row and metadata as columns}

\item{spectraMetadata}{NULL or a data.frame of sample metadata, with samples
as row and metadata as columns}

\item{uROIExist}{NULL or a logical stating if uROI have been set}

\item{useUROI}{NULL or a logical stating if uROI are to be used}

\item{useFIR}{NULL or a logical stating if FIR are to be used}

\item{verbose}{(bool) If TRUE message progress}

\item{...}{Additional slots and values to set when resetting the object
(\code{cpdID}, \code{cpdName}, \code{ROI}, \code{filepath}, \code{TIC},
\code{acquisitionTime}, \code{peakTables}, \code{dataPoints}, \code{peakFit})}
}
\value{
(peakPantheRAnnotation) object reset with previous results removed
and slots updated
}
\description{
Reset a peakPantheRAnnotation (remove results and set
\code{isAnnotated=FALSE}). If a different number of samples (
\code{spectraPaths}) or compounds (\code{targetFeatTable}) are passed, the
object will be initialised to the new size. For input values left as NULL,
the slots (\code{filepath} (from \code{spectraPaths}), \code{ROI},
\code{cpdID}, \code{cpdName} (from \code{targetFeatTable}), \code{uROI},
\code{FIR}, \code{cpdMetadata}, \code{spectraMetadata}, \code{uROIExist},
\code{useUROI} and \code{useFIR}) will be filled with values from
\code{previousAnnotation}.
}
\examples{
if(requireNamespace('faahKO')){
## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
## compounds

# Paths to spectra files
library(faahKO)
spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
                    system.file('cdf/KO/ko16.CDF', package = 'faahKO'))

# targetFeatTable
targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
                    c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
                    'mzMax'))), stringsAsFactors=FALSE)
targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
                        522.2, 522.205222)
targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
                        496.2, 496.204962)
targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
                                    FUN.VALUE=numeric(2))

smallAnnotation  <- peakPantheRAnnotation(spectraPaths=spectraPaths, 
                                        targetFeatTable=targetFeatTable)
smallAnnotation
# An object of class peakPantheRAnnotation
#  2 compounds in 2 samples.
#  updated ROI do not exist (uROI)
#  does not use updated ROI (uROI)
#  does not use fallback integration regions (FIR)
#  is not annotated

# Reset and change number of spectra
newSpectraPaths  <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
                    system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
                    system.file('cdf/KO/ko18.CDF', package = 'faahKO'))
largerAnnotation <- resetAnnotation(smallAnnotation,
                                    spectraPaths=newSpectraPaths,
                                    verbose=TRUE)
largerAnnotation
# An object of class peakPantheRAnnotation
#  2 compounds in 3 samples.
#  updated ROI do not exist (uROI)
#  does not use updated ROI (uROI)
#  does not use fallback integration regions (FIR)
#  is not annotated
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotHistogram.R
\name{plotHistogram}
\alias{plotHistogram}
\title{Plot variable histogram and density}
\usage{
plotHistogram(var, varName = "Variable", density = TRUE, ...)
}
\arguments{
\item{var}{(float) vector of values to plot}

\item{varName}{(str) Name of the variable to plot}

\item{density}{(bool) If TRUE plot overlay the density on the variable}

\item{...}{Passes arguments to ggplot2::geom_histogram, e.g. \code{bins=20},
\code{binwidth=1}}
}
\value{
Grob (ggplot object)
}
\description{
Plot the histogram and density of the variable
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generateIonChromatogram.R
\name{generateIonChromatogram}
\alias{generateIonChromatogram}
\title{Generate ion chromatogram from raw data points}
\usage{
generateIonChromatogram(ROIDataPoint, aggregationFunction = "sum")
}
\arguments{
\item{ROIDataPoint}{(data.frame) retention time 'rt', mass 'mz' and intensity
'int' (as column) of each raw data points (as row) to use for the ion
chromatogram}

\item{aggregationFunction}{(str) Function to use in order to aggregate
intensities across mz in each scan. One of \code{sum}, \code{max},
\code{min}, \code{mean}}
}
\value{
A data.frame of retention time 'rt' and aggregated intensities 'int'
}
\description{
On the input raw data, aggregate intensites across the mz range
at each retention time to generate an ion chromatogram: \code{sum} for
EIC/TIC, \code{max}, \code{min} or \code{mean}. The number of data points
returned correspond to the number of unique scans/retention time measurements
in the input data
}
\details{
## Examples cannot be computed as the function is not exported:
## Input data points
in_rt <- c(3362.102, 3362.102, 3363.667, 3363.667, 3365.232, 3365.232,
            3366.797, 3366.797, 3368.362, 3368.362)
in_mz <- c(496.2, 497.2, 496.2, 497.2, 496.2, 497.2, 496.2, 497.2, 496.2,
            497.2)
in_int <- c(39616, 11432, 63344, 18224, 107352, 30936, 182144, 51776, 295232,
            81216)
input_ROIDataPoints <- data.frame(rt=in_rt, mz=in_mz, int=in_int)

## Aggregate mz to generate EIC
EIC <- generateIonChromatogram(input_ROIDataPoints,aggregationFunction='sum')
EIC
#         rt    int
# 1 3362.102  51048
# 2 3363.667  81568
# 3 3365.232 138288
# 4 3366.797 233920
# 5 3368.362 376448
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_peakPantheRAnnotation.R
\docType{methods}
\name{[,peakPantheRAnnotation,ANY,ANY,ANY-method}
\alias{[,peakPantheRAnnotation,ANY,ANY,ANY-method}
\alias{[,peakPantheRAnnotation-method}
\title{extract parts of peakPantheRAnnotation class}
\usage{
\S4method{[}{peakPantheRAnnotation,ANY,ANY,ANY}(x, i, j, drop = "missing")
}
\arguments{
\item{x}{object from which to extract element(s) or in which to replace
element(s).}

\item{i}{(sample) indices specifying elements to extract or replace}

\item{j}{(compound) indices specifying elements to extract or replace}

\item{drop}{not applicable}
}
\value{
(peakPantheRAnnotation) object subset
}
\description{
extract parts of peakPantheRAnnotation class
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integrateFIR.R
\name{integrateFIR}
\alias{integrateFIR}
\title{Integrate fallback integration regions}
\usage{
integrateFIR(rawSpec, FIR, foundPeakTable, verbose = TRUE)
}
\arguments{
\item{rawSpec}{an \code{\link[MSnbase]{OnDiskMSnExp-class}}}

\item{FIR}{(data.frame) Fallback Integration Regions (FIR) to integrate when
a feature is not found. Compounds as row are identical to the targeted
features, columns are \code{rtMin} (float in seconds), \code{rtMax} (float in
seconds), \code{mzMin} (float), \code{mzMax} (float)}

\item{foundPeakTable}{a \code{data.frame} as generated by
\link{findTargetFeatures}, with features as rows and peak properties as
columns. The following columns are mandatory: \code{found}, \code{is_filled},
\code{mz}, \code{mzMin}, \code{mzMax}, \code{rt}, \code{rtMin}, \code{rtMax},
\code{peakArea}, \code{peakAreaRaw}, \code{maxIntMeasured},
\code{maxIntPredicted}.}

\item{verbose}{(bool) if TRUE message progress}
}
\value{
an updated foundPeakTable with FIR integration values
}
\description{
Integrate region defined in FIR if a feature is not found
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GUI_utils.R
\name{initialise_annotation_from_files_UI_helper}
\alias{initialise_annotation_from_files_UI_helper}
\title{UI data import helper - initialise new annotation from files}
\usage{
initialise_annotation_from_files_UI_helper(
    CSVParamPath,
    spectraPaths,
    cpdMetadataPath = NULL,
    spectraMetadata = NULL,
    verbose = TRUE
)
}
\arguments{
\item{CSVParamPath}{(str) Path to a CSV file of fit parameters}

\item{spectraPaths}{(str) character vector of spectra file paths, to set
samples to process}

\item{cpdMetadataPath}{NULL or path to a csv of compound metadata, with
compounds as row and metadata as columns}

\item{spectraMetadata}{NULL or DataFrame of sample metadata, with
samples as row and metadata as columns}

\item{verbose}{(bool) If TRUE message progress}
}
\value{
(peakPantheRAnnotation) Object initialised with ROI, uROI and FIR
read from the CSV file
}
\description{
Fully initialise a \code{peakPantheRAnnotation} using the target files path,
CSV parameter path and metadata.
}
\examples{
## Input data
input_CSV <- data.frame(matrix(nrow=2,ncol=21,dimnames=list(c(),
    c('cpdID', 'cpdName',
    'X','ROI_rt', 'ROI_mz', 'ROI_rtMin', 'ROI_rtMax','ROI_mzMin','ROI_mzMax',
    'X','uROI_rtMin', 'uROI_rtMax', 'uROI_mzMin', 'uROI_mzMax', 'uROI_rt',
    'uROI_mz', 'X', 'FIR_rtMin', 'FIR_rtMax', 'FIR_mzMin', 'FIR_mzMax'))))
input_CSV[1,]  <- c('ID-1', 'Cpd 1', '|', 1.,  2.,  3.,  4.,  5.,  6.,  '|',
                    7.,  8.,  9.,  10., 11., 12., '|', 13., 14., 15., 16.)
input_CSV[2,]  <- c('ID-2', 'Cpd 2', '|', 17., 18., 19., 20., 21., 22., '|',
                    23., 24., 25., 26., 27., 28., '|', 29., 30., 31., 32.)
input_CSV[,-c(1,2,3,10,17)]  <- vapply(input_CSV[,-c(1,2,3,10,17)],
                                            as.numeric, FUN.VALUE=numeric(2))

input_spectraPaths    <- c('./path/file1', './path/file2', './path/file3')

# temporary file location
savePath1      <- tempfile(pattern='file', tmpdir=tempdir(), fileext='.csv')
# save csv
utils::write.csv(input_CSV, file=savePath1, row.names=FALSE)

# Load parameters from CSV
loadedAnnotation <- initialise_annotation_from_files_UI_helper(savePath1,
                                                        input_spectraPaths,
                                                        verbose=TRUE)
# An object of class peakPantheRAnnotation
#   2 compounds in 3 samples.
#   updated ROI exist (uROI)
#   does not use updated ROI (uROI)
#   does not use fallback integration regions (FIR)
#   is not annotated
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GUI_utils.R
\name{annotation_diagnostic_multiplot_UI_helper}
\alias{annotation_diagnostic_multiplot_UI_helper}
\title{UI diagnostic plot helper - single feature multiplot}
\usage{
annotation_diagnostic_multiplot_UI_helper(
    cpdNb,
    annotation,
    splNum = NULL,
    splColrColumn = NULL,
    ...
)
}
\arguments{
\item{cpdNb}{(int) position of the feature to extract (1 to nbCpd)}

\item{annotation}{(peakPantheRAnnotation) Annotation object}

\item{splNum}{(int) NULL or number of spectra to plot, chosen randomly
from all spectra. If NULL or equal to the total number of spectra, plot all
spectra}

\item{splColrColumn}{(str) NULL, None or a spectraMetadata column for
colouring each sample}

\item{...}{Additional parameters for plotting}
}
\value{
(ggplotObject) Diagnostic multiplot for a feature
}
\description{
Return a ggplot object of a feature diagnostic multiplot
}
\examples{
## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
## compounds

# Paths to spectra files
spectraPaths <- c('./path/file1', './path/file2', './path/file3')

# targetFeatTable
targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
                    c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
                    'mzMax'))), stringsAsFactors=FALSE)
targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
                        522.2, 522.205222)
targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
                        496.2, 496.204962)
targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
                                    FUN.VALUE=numeric(2))

emptyAnnotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
                                        targetFeatTable=targetFeatTable)

# Plot of an empty annotation
annotation_diagnostic_multiplot_UI_helper(cpdNb = 2,
                                            annotation = emptyAnnotation,
                                            splNum = NULL,
                                            splColrColumn = NULL)
# Warning: the object has not been annotated, return an empty diagnostic
# plot list
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GUI_utils.R
\name{spectraPaths_and_metadata_UI_helper}
\alias{spectraPaths_and_metadata_UI_helper}
\title{UI data import helper - prepare file paths and metadata}
\usage{
spectraPaths_and_metadata_UI_helper(
    spectraPaths = NULL,
    spectraMetadataPath = NULL
)
}
\arguments{
\item{spectraPaths}{NULL or character vector of spectra file paths, to set
samples to process}

\item{spectraMetadataPath}{NULL or path to a csv of spectra metadata, with
spectra as row and metadata as columns. (spectraPaths in column `filepath`)}
}
\value{
spectraPaths (str) and spectraMetadata (DataFrame or NULL) read from
the CSV file
}
\description{
Return spectraPaths and spectraMetadata from a .csv file (if available).
If reading from the spectraMetadata file, the spectraPaths are taken from
the `filepath` column
}
\examples{
## Input data
# spectraPath
input_spectraPaths    <- c('./path/file1', './path/file2', './path/file3')

# spectraMetadata
input_spectraMetadata <- data.frame(matrix(data=c(input_spectraPaths,
                            c('a','b','c')), nrow=3, ncol=2,
                            dimnames=list(c(),c('filepath', 'testcol')),
                            byrow=FALSE), stringsAsFactors=FALSE)

# temporary file location
spectraMetaPath <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.csv')
# save csv
utils::write.csv(input_spectraMetadata,
                    file=spectraMetaPath,
                    row.names=FALSE)

# load data from CSV
spectraPaths_and_metadata_UI_helper(spectraPaths = NULL,
                                    spectraMetadataPath = spectraMetaPath)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getTargetFeatureStatistic.R
\name{getTargetFeatureStatistic}
\alias{getTargetFeatureStatistic}
\title{Calculate chromatographic peak properties}
\usage{
getTargetFeatureStatistic(
    fittedCurves,
    targetFeatTable,
    foundPeakTable,
    verbose = FALSE
)
}
\arguments{
\item{fittedCurves}{(list) A list (one entry per ROI window) of
\code{peakPantheR_curveFit} or NA}

\item{targetFeatTable}{a \code{\link{data.frame}} of compounds to target as
rows. Columns: \code{cpdID} (str), \code{cpdName} (str), \code{rtMin} (float
in seconds), \code{rt} (float in seconds, or \emph{NA}), \code{rtMax} (float
in seconds), \code{mzMin} (float), \code{mz} (float or \emph{NA}),
\code{mzMax} (float).}

\item{foundPeakTable}{a \code{data.frame} as generated by
\code{\link{findTargetFeatures}}, with features as rows and peak properties
as columns. The following columns are mandatory: \code{mzMin}, \code{mz},
\code{mzMax}, \code{rtMin}, \code{rt}, \code{rtMax}}

\item{verbose}{(bool) if TRUE message when NA scans are removed}
}
\value{
A \code{data.frame} with measured compounds as rows and measurements
and properties as columns (see Details).

\subsection{Details:}{
The returned \code{data.frame} is structured as follow:
\tabular{ll}{
found \tab was the peak found\cr
rt \tab retention time of peak apex (sec)\cr
rtMin \tab leading edge of peak retention time (sec) determined at 0.5\% of
apex intensity\cr
rtMax \tab trailing edge of peak retention time (sec) determined at 0.5\% of
apex intensity\cr
mz \tab weighted (by intensity) mean of peak m/z across scans\cr
mzMin \tab m/z peak minimum (between rtMin, rtMax)\cr
mzMax \tab m/z peak maximum (between rtMin, rtMax)\cr
peakArea \tab integrated peak area\cr
peakAreaRaw \tab integrated peak area from raw data points\cr
maxIntMeasured \tab maximum peak intensity in raw data\cr
maxIntPredicted \tab maximum peak intensity based on curve fit\cr
ppm_error \tab difference in ppm between the expected and measured m/z\cr
rt_dev_sec \tab difference in seconds between the expected and measured rt\cr
tailingFactor \tab the tailing factor is a measure of peak tailing. It is
defined as the distance from the front slope of the peak to the back slope
divided by twice the distance from the center line of the peak to the front
slope, with all measurements made at 5\% of the maximum peak height. The
tailing factor of a peak will typically be similar to the asymmetry factor
for the same peak, but the two values cannot be directly converted\cr
asymmetryFactor \tab the asymmetry factor is a measure of peak tailing. It is
defined as the distance from the center line of the peak to the back slope
divided by the distance from the center line of the peak to the front slope,
with all measurements made at 10\% of the maximum peak height. The asymmetry
factor of a peak will typically be similar to the tailing factor for the same
peak, but the two values cannot be directly converted\cr
}
}
}
\description{
Calculate the ppm error, retention time deviation, tailing
factor and asymmetry factor for each measured feature.
}
\details{
## Examples cannot be computed as the function is not exported:
# fittedCurve
cFit1 <- list(amplitude=162404.8057918259, center=3341.888,
                sigma=0.078786133031045896, gamma=0.0018336101984172684,
                fitStatus=2, curveModel='skewedGaussian')
class(cFit1) <- 'peakPantheR_curveFit'
cFit2        <- list(amplitude=199249.10572753669, center=3382.577,
                    sigma=0.074904415304607966, gamma=0.0011471899372353885,
                    fitStatus=2, curveModel='skewedGaussian')
class(cFit2)    <- 'peakPantheR_curveFit'
input_fitCurves <- list(cFit1, cFit2)

# ROI
input_ROI <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(), c('cpdID',
                'cpdName', 'rtMin', 'rt', 'rtMax', 'mzMin', 'mz', 'mzMax'))),
                stringsAsFactors=FALSE)
input_ROI[1,] <- c('ID-1', 'testCpd 1', 3310., 3344.88, 3390., 522.19, 522.2,
                    522.21)
input_ROI[2,] <- c('ID-2', 'testCpd 2', 3280., 3385.58, 3440., 496.19, 496.2,
                    496.21)
input_ROI[,3:8] <- vapply(input_ROI[,3:8], as.numeric, FUN.VALUE=numeric(2))

# foundPeakTable
input_foundPeakTable <- data.frame(matrix(vector(), 2, 10, dimnames=list(c(),
                    c('found', 'rtMin', 'rt', 'rtMax', 'mzMin', 'mz',
                    'mzMax','peakArea','maxIntMeasured','maxIntPredicted'))),
                    stringsAsFactors=FALSE)
input_foundPeakTable[1,]  <- c(TRUE, 3309.758, 3346.827, 3385.410, 522.19,
                                522.2, 522.21, 26133726, 889280, 901015)
input_foundPeakTable[2,]  <- c(TRUE, 3345.376, 3386.529, 3428.279, 496.19,
                                496.2, 496.21, 35472141, 1128960, 1113576)
input_foundPeakTable[,1]  <- vapply(input_foundPeakTable[,c(1)], as.logical,
                                FUN.VALUE=logical(1))

# Run peak statistics
peakStatistics <- getTargetFeatureStatistic(input_fitCurves, input_ROI,
                                            input_foundPeakTable)
peakStatistics   
#   found    rtMin       rt    rtMax  mzMin    mz  mzMax peakArea
# 1  TRUE 3309.758 3346.827 3385.410 522.19 522.2 522.21 26133726
# 2  TRUE 3345.376 3386.529 3428.279 496.19 496.2 496.21 35472141
#    maxIntMeasured maxIntPredicted ppm_error rt_dev_sec tailingFactor
# 1          889280          901015         0      1.947      1.015385
# 2         1128960         1113576         0      0.949      1.005372
#    asymmetryFactor
# 1         1.026886
# 2         1.009304
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_peakPantheRAnnotation.R
\docType{methods}
\name{retentionTimeCorrection,peakPantheRAnnotation-method}
\alias{retentionTimeCorrection,peakPantheRAnnotation-method}
\alias{retentionTimeCorrection}
\title{Apply retention time correction methods to adjust the retention time
information in the uROI of peakPantheRAnnotation object}
\usage{
\S4method{retentionTimeCorrection}{peakPantheRAnnotation}(
    annotationObject,
    rtCorrectionReferences = NULL,
    method = "polynomial",
    params = list(polynomialOrder = 2),
    robust = FALSE,
    rtWindowWidth = 15,
    diagnostic = TRUE
)
}
\arguments{
\item{annotationObject}{(peakPantheRAnnotation) object with previous fit
results to adjust retention time values in uROI and FIR
annotationObject, rtCorrectionReferences=NULL,}

\item{rtCorrectionReferences}{(list) of compounds IDs (\code{cpdID})
to be used as retention time references.
All \code{cpdID} entries must be present in the object and previously
annotated. If NULL, use all compounds.}

\item{method}{(str) name of RT correction method to use (currently
\code{polynomial} or \code{constant}}

\item{params}{(list) list of parameters to pass to each correction method.
Currently allowed inputs are \code{polynomialOrder}
for \code{method='polynomial'}}

\item{robust}{(bool) whether to use the RANSAC algorithm to flag and ignore
outliers during retention time correction}

\item{rtWindowWidth}{(numeric) full width in seconds
of the retention time window defined around the corrected retention time
value for each compound}

\item{diagnostic}{(bool) If TRUE returns diagnostic plots (specific to each
correction method)}
}
\value{
(list) containing entries `annotation`, with the new and retention
time corrected peakPantheRAnnotation, and `plot` (if \code{diagnostic=TRUE}).
}
\description{
Performs retention time correction to re-adjust the expected
retention time position of compounds.
Requires an annotated peakPantheRAnnotation object (\code{isAnnotated=TRUE}).
The original \code{rt} value is used as expected and the observed deviation
measured in the \code{rt_dev_sec} field is taken as the deviation to be
corrected.
}
\examples{
if(requireNamespace('faahKO')){
## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
## compounds

# Paths to spectra files
library(faahKO)
spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
                    system.file('cdf/KO/ko16.CDF', package = 'faahKO'))

# targetFeatTable
targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
                    c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
                    'mzMax'))), stringsAsFactors=FALSE)
targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
                        522.2, 522.205222)
targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
                        496.2, 496.204962)
targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
                                    FUN.VALUE=numeric(2))

smallAnnotation  <- peakPantheRAnnotation(spectraPaths=spectraPaths,
                                        targetFeatTable=targetFeatTable)
# annotate files serially
annotation_result <- peakPantheR_parallelAnnotation(smallAnnotation,
                                            ncores=0, verbose=TRUE)
data_annotation   <- annotation_result$annotation

# Example with constant correction
rtCorrectionOutput <- retentionTimeCorrection(
                        annotationObject = data_annotation,
                        rtCorrectionReferences=c('ID-1'),
                        method='constant', params=list(),
                        robust=FALSE,
                        rtWindowWidth=15,
                        diagnostic=TRUE)

rtCorrectedAnnotation <- rtCorrectionOutput$annotation

# rtCorrectedAnnotation
# An object of class peakPantheRAnnotation
#  2 compounds in 2 samples.
#  updated ROI exists, with a modified rt (uROI)
#  uses updated ROI (uROI)
#  uses fallback integration regions (FIR)
#  is annotated

rtCorrectionPlot <- rtCorrectionOutput$plot
# rtCorrectedPlot
# A ggplot2 object
#  Scatterplot where x=`r` in the and y=`rt_dev_sec` from data_annotation
#  Points colored depending on whether the reference was used to fit
# the correction model
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotAnnotationDiagnosticMultiplot.R
\name{annotationDiagnosticMultiplot}
\alias{annotationDiagnosticMultiplot}
\title{Generate a multiplot of all diagnostic plots}
\usage{
annotationDiagnosticMultiplot(annotationDiagnosticPlotList)
}
\arguments{
\item{annotationDiagnosticPlotList}{(list) List of (one per compound) of list
of diagnostic plots as generated by \code{annotationDiagnosticPlots()}}
}
\value{
A list of multiplots (one per compound)
}
\description{
Generate a multiplot of all diagnostic plots (as generated by
\code{annotationDiagnosticPlots()}) for each compound
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/peakPantheR_ROIStatistics.R
\name{peakPantheR_ROIStatistics}
\alias{peakPantheR_ROIStatistics}
\title{Save to disk each ROI EIC and mean IS RT}
\usage{
peakPantheR_ROIStatistics(
    referenceSpectraFiles,
    saveFolder,
    ROI = NULL,
    IS_ROI = NULL,
    sampleColour = NULL,
    ncores = 0,
    saveISPlots = TRUE,
    verbose = TRUE
)
}
\arguments{
\item{referenceSpectraFiles}{(str) A character vector of paths to the
reference spectra files}

\item{saveFolder}{(str) Path to the folder where EICs and IS mean RT
(\code{IS_mean_RT.csv}) will be saved}

\item{ROI}{(data.frame) NULL or a data.frame of Regions Of Interest (ROI)
with compounds as row and ROI parameters as columns: \code{rtMin} (float in
seconds), \code{rt} (float in seconds, or \emph{NA}), \code{rtMax} (float in
seconds), \code{mzMin} (float), \code{mz} (float or \emph{NA}), \code{mzMax}
(float) (if NULL, ROI EICs are not saved)}

\item{IS_ROI}{(data.frame) NULL or a data.frame of IS ROI with IS as row and
ROI parameters as columns: \code{rtMin} (float in seconds), \code{rt} (float
in seconds, or \emph{NA}), \code{rtMax} (float in seconds), \code{mzMin}
(float), \code{mz} (float or \emph{NA}), \code{mzMax} (float)  (if NULL IS
mean RT is not calculated and saved in \code{IS_mean_RT.csv})}

\item{sampleColour}{(str) NULL or vector colour for each sample}

\item{ncores}{(int) Number of cores to use to integrate IS in parallel}

\item{saveISPlots}{(bool) If TRUE save a diagnostic plot for each IS to
\code{saveFolder/IS_search} compound}

\item{verbose}{(bool) If TRUE message progress}
}
\value{
None
}
\description{
Using reference samples (\code{referenceSpectraFiles}), save
(to \code{saveFolder}) each ROI EIC (\code{ROI}) and reports the mean apex RT
for all IS (\code{IS_ROI}) across samples
}
\examples{
if(requireNamespace('faahKO')){
## Initialise a peakPantheRAnnotation object with 2 samples and 1 targeted
## compound

# Paths to spectra files
library(faahKO)
spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
                    system.file('cdf/KO/ko16.CDF', package = 'faahKO'))

# targetFeatTable
targetFeatTable <- data.frame(matrix(vector(), 1, 8, dimnames=list(c(),
                    c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
                    'mzMax'))), stringsAsFactors=FALSE)
targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
                        522.2, 522.205222)
targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
                                    FUN.VALUE=numeric(1))

# input
refSpecFiles  <- spectraPaths
input_ROI     <- targetFeatTable
input_IS_ROI  <- targetFeatTable
sampleColour  <- c('blue', 'red')

# temporary saveFolder
saveFolder1   <- tempdir()

# Calculate ROI statiscs
peakPantheR_ROIStatistics(refSpecFiles, saveFolder1, ROI=input_ROI,
                            IS_ROI=input_IS_ROI, sampleColour=sampleColour,
                            ncores=0, saveISPlots=TRUE, verbose=TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_peakPantheRAnnotation.R
\docType{methods}
\name{useUROI,peakPantheRAnnotation-method}
\alias{useUROI,peakPantheRAnnotation-method}
\alias{useUROI}
\title{useUROI accessor}
\usage{
\S4method{useUROI}{peakPantheRAnnotation}(object)
}
\arguments{
\item{object}{peakPantheRAnnotation}
}
\value{
(bool) flag if uROI are to be used
}
\description{
useUROI accessor
}
\examples{
if(requireNamespace('faahKO')){
## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
## compounds

# Paths to spectra files
library(faahKO)
spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
                    system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
                    system.file('cdf/KO/ko18.CDF', package = 'faahKO'))

# targetFeatTable
targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
                    c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
                    'mzMax'))), stringsAsFactors=FALSE)
targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
                        522.2, 522.205222)
targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
                        496.2, 496.204962)
targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
                                    FUN.VALUE=numeric(2))

annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
                                    targetFeatTable=targetFeatTable)

useUROI(annotation)
# [1] FALSE
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_peakPantheRAnnotation.R,
%   R/initialize_peakPantheRAnnotation.R
\name{peakPantheRAnnotation}
\alias{peakPantheRAnnotation}
\alias{peakPantheRAnnotation-class}
\title{An S4 class to represent peakPantheR annotation results}
\usage{
peakPantheRAnnotation(spectraPaths = NULL, targetFeatTable = NULL,
    cpdID = character(), cpdName = character(), ROI = data.frame(rtMin
    = numeric(), rt = numeric(), rtMax = numeric(), mzMin = numeric(), mz =
    numeric(), mzMax = numeric(), stringsAsFactors = FALSE),
    FIR = data.frame(rtMin = numeric(), rtMax = numeric(), mzMin =
    numeric(), mzMax = numeric(), stringsAsFactors = FALSE),
    uROI = data.frame(rtMin = numeric(), rt = numeric(), rtMax = numeric(),
    mzMin = numeric(), mz = numeric(), mzMax = numeric(), stringsAsFactors =
    FALSE), filepath = character(), cpdMetadata = data.frame(),
    spectraMetadata = data.frame(), acquisitionTime = character(),
    uROIExist = FALSE, useUROI = FALSE, useFIR = FALSE,
    TIC = numeric(), peakTables = list(), dataPoints = list(),
    peakFit = list(), isAnnotated = FALSE)

peakPantheRAnnotation(spectraPaths = NULL, targetFeatTable = NULL,
    cpdID = character(), cpdName = character(), ROI = data.frame(rtMin
    = numeric(), rt = numeric(), rtMax = numeric(), mzMin = numeric(), mz =
    numeric(), mzMax = numeric(), stringsAsFactors = FALSE),
    FIR = data.frame(rtMin = numeric(), rtMax = numeric(), mzMin =
    numeric(), mzMax = numeric(), stringsAsFactors = FALSE),
    uROI = data.frame(rtMin = numeric(), rt = numeric(), rtMax = numeric(),
    mzMin = numeric(), mz = numeric(), mzMax = numeric(), stringsAsFactors =
    FALSE), filepath = character(), cpdMetadata = data.frame(),
    spectraMetadata = data.frame(), acquisitionTime = character(),
    uROIExist = FALSE, useUROI = FALSE, useFIR = FALSE,
    TIC = numeric(), peakTables = list(), dataPoints = list(),
    peakFit = list(), isAnnotated = FALSE)
}
\arguments{
\item{spectraPaths}{NULL or a character vector of spectra file paths, to set
samples to process}

\item{targetFeatTable}{NULL or a \code{\link{data.frame}} of compounds to
target as rows and parameters as columns: \code{cpdID} (str), \code{cpdName}
(str), \code{rtMin} (float in seconds), \code{rt} (float in seconds, or
\emph{NA}), \code{rtMax} (float in seconds), \code{mzMin} (float), \code{mz}
(float or \emph{NA}), \code{mzMax} (float). Set compounds to target.}

\item{cpdID}{A character vector of compound IDs, of length number of
compounds}

\item{cpdName}{A character vector of compound names, of length number of
compounds}

\item{ROI}{A data.frame of Regions Of Interest (ROI) with compounds as
row and ROI parameters as columns: \code{rtMin} (float in seconds), \code{rt}
(float in seconds, or \emph{NA}), \code{rtMax} (float in seconds),
\code{mzMin} (float), \code{mz} (float or \emph{NA}), \code{mzMax} (float).}

\item{FIR}{A data.frame of Fallback Integration Regions (FIR) with
compounds as row and FIR parameters as columns: \code{rtMin} (float in
seconds),
\code{rtMax} (float in seconds), \code{mzMin} (float), \code{mzMax} (float).}

\item{uROI}{A data.frame of updated Regions Of Interest (uROI) with
compounds as row and uROI parameters as columns: \code{rtMin} (float in
seconds),
\code{rt} (float in seconds, or \emph{NA}), \code{rtMax} (float in seconds),
\code{mzMin} (float), \code{mz} (float or \emph{NA}), \code{mzMax} (float).}

\item{filepath}{A character vector of file paths, of length number of spectra
files}

\item{cpdMetadata}{A data.frame of compound metadata, with compounds as row
and metadata as columns}

\item{spectraMetadata}{A data.frame of sample metadata, with samples as row
and metadata as columns}

\item{acquisitionTime}{A character vector of acquisition date-time (converted
from POSIXct) or NA}

\item{uROIExist}{A logical stating if uROI have been set}

\item{useUROI}{A logical stating if uROI are to be used}

\item{useFIR}{A logical stating if FIR are to be used}

\item{TIC}{A numeric vector of TIC or NA, of length number of spectra files}

\item{peakTables}{A list of peakTable data.frame, of length number of spectra
files. Each peakTable data.frame has compounds as rows and peak annotation
results as columns.}

\item{dataPoints}{A list of length number of spectra files. Each list element
is a \emph{ROIsDataPoint} list of \code{data.frame} of raw data points for
each ROI/uROI (retention time 'rt', mass 'mz' and intensity 'int' (as column)
of each raw data points (as row))}

\item{peakFit}{A list of length number of spectra files. Each list element is
a \emph{curveFit} list of \code{peakPantheR_curveFit} or NA for each ROI}

\item{isAnnotated}{A logical stating in the annotation took place}
}
\value{
(peakPantheRAnnotation)
}
\description{
The \code{peakPantheRAnnotation} class is designed to run and
store peakPantheR parallel annotation results. Instances of the class are
created with the \code{peakPantheRAnnotation} constructor function, which
initialises an object of proper dimension with \code{spectraPaths} (set
samples to process) and \code{targetFeatTable} (set compounds to target).
\code{spectraPaths} is a character vector of spectra file paths.
\code{targetFeatTable} is a \code{\link{data.frame}} of compounds to target
as rows and parameters as columns: \code{cpdID} (int), \code{cpdName} (str),
\code{rtMin} (float in seconds), \code{rt} (float in seconds, or \emph{NA}),
\code{rtMax} (float in seconds), \code{mzMin} (float), \code{mz} (float or
\emph{NA}), \code{mzMax} (float).

\code{peakPantheRAnnotation()}: create an instance of the
\code{peakPantherAnnotation} class.
}
\details{
The \code{validObject} method ensures the conformity of an object to
the \code{peakPantheRAnnotation-class}. The number of compounds is based on
\code{cpdID()} length, and the number of samples is based on
\code{filepath()} length. Slot type is not checked as \code{setClass}
enforces it. peakTables and EICs type are checked on the first list element.
\code{annotationTable(object, column)} where \emph{column} is a column from
\emph{peakTable}, returns a data.frame of values with the samples as rows,
ROI as columns.
}
\section{Slots}{

\describe{
\item{\code{cpdID}}{A character vector of compound IDs, of length
number of compounds}

\item{\code{cpdName}}{A character vector of compound names, of length number of
compounds}

\item{\code{ROI}}{A data.frame of Regions Of Interest (ROI) with
compounds as row and
ROI parameters as columns: \code{rtMin} (float in seconds), \code{rt} (float
in seconds, or \emph{NA}), \code{rtMax} (float in seconds), \code{mzMin}
(float), \code{mz} (float or \emph{NA}), \code{mzMax} (float).}

\item{\code{FIR}}{A data.frame of Fallback Integration Regions (FIR)
with compounds
as row and FIR parameters as columns: \code{rtMin} (float in seconds),
\code{rtMax} (float in seconds), \code{mzMin} (float), \code{mzMax} (float).}

\item{\code{uROI}}{A data.frame of updated Regions Of Interest
(uROI) with compounds
as row and uROI parameters as columns: \code{rtMin} (float in seconds),
\code{rt} (float in seconds, or \emph{NA}), \code{rtMax} (float in seconds),
\code{mzMin} (float), \code{mz} (float or \emph{NA}), \code{mzMax} (float).}

\item{\code{filepath}}{A character vector of file paths, of
length number of spectra
files}

\item{\code{cpdMetadata}}{A data.frame of compound metadata,
with compounds as row and metadata as columns}

\item{\code{spectraMetadata}}{A data.frame of sample metadata,
with samples as row and metadata as columns}

\item{\code{acquisitionTime}}{A character
vector of acquisition date-time (converted from POSIXct) or NA}

\item{\code{uROIExist}}{A logical stating if uROI have been set}

\item{\code{useUROI}}{A logical stating if uROI are to be used}

\item{\code{useFIR}}{A logical stating if FIR are to be used}

\item{\code{TIC}}{A numeric vector of TIC or NA,
of length number of spectra files}

\item{\code{peakTables}}{A list of peakTable data.frame, of
length number of spectra files. Each peakTable data.frame
has compounds as rows and peak annotation results as columns.}

\item{\code{dataPoints}}{A list of length number of spectra files.
Each list element is a \emph{ROIsDataPoint} list of \code{data.frame}
of raw data points for each ROI/uROI (retention time 'rt',
mass 'mz' and intensity 'int' (as column) of each raw data points (as row))}

\item{\code{peakFit}}{A list of length number of spectra files.
Each list element is a \emph{curveFit} list of
\code{peakPantheR_curveFit} or NA for each ROI}

\item{\code{isAnnotated}}{A logical stating if the annotation has taken place

\subsection{Details:}{
The \emph{peakTables} \code{data.frame} are structured as follow:
\tabular{ll}{
cpdID \tab database compound ID\cr
cpdName \tab compound name\cr
found \tab was the peak found\cr
rt \tab retention time of peak apex (sec)\cr
rtMin \tab leading edge of peak retention time (sec) determined at 0.5\% of
apex intensity\cr
rtMax \tab trailing edge of peak retention time (sec) determined at 0.5\% of
apex intensity\cr
mz \tab weighted (by intensity) mean of peak m/z across scans\cr
mzMin \tab m/z peak minimum (between rtMin, rtMax)\cr
mzMax \tab m/z peak maximum (between rtMin, rtMax)\cr
peakArea \tab integrated peak area\cr
peakAreaRaw \tab integrated peak area from raw data points\cr
maxIntMeasured \tab maximum peak intensity in raw data\cr
maxIntPredicted \tab maximum peak intensity based on curve fit\cr
is_filled \tab Logical indicate if the feature was integrated using FIR
(Fallback Integration Region)\cr
ppm_error \tab difference in ppm between the expected and measured m/z\cr
rt_dev_sec \tab difference in seconds between the expected and measured rt\cr
tailingFactor \tab the tailing factor is a measure of peak tailing.It is
defined as the distance from the front slope of the peak to the back slope
divided by twice the distance from the center line of the peak to the front
slope, with all measurements made at 5\% of the maximum peak height. The
tailing factor of a peak will typically be similar to the asymmetry factor
for the same peak, but the two values cannot be directly converted\cr
asymmetryFactor \tab the asymmetry factor is a measure of peak tailing. It is
defined as the distance from the center line of the peak to the back slope
divided by the distance from the center line of the peak to the front slope,
with all measurements made at 10\% of the maximum peak height. The asymmetry
factor of a peak will typically be similar to the tailing factor for the same
peak, but the two values cannot be directly converted\cr
}
}}
}}

\examples{
if(requireNamespace('faahKO')){
## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
## compounds

# Paths to spectra files
library(faahKO)
spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
                    system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
                    system.file('cdf/KO/ko18.CDF', package = 'faahKO'))

# targetFeatTable
targetFeatTable <- data.frame(matrix(vector(),2,8,dimnames=list(c(),
                    c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
                    'mzMax'))), stringsAsFactors=FALSE)
targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
                        522.2, 522.205222)
targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
                        496.2, 496.204962)
targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
                                    FUN.VALUE=numeric(2))

annotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
                                    targetFeatTable=targetFeatTable)

annotation
# An object of class peakPantheRAnnotation
#  2 compounds in 3 samples.
#   updated ROI do not exist (uROI)
#   does not use updated ROI (uROI)
#   does not use fallback integration regions (FIR)
#   is not annotated

slotNames(annotation)
# [1] 'cpdID'       'cpdName'      'ROI'             'FIR'       'uROI'
# [6] 'filepath'    'cpdMetadata'  'spectraMetadata' 'acquisitionTime'
# [10] 'uROIExist'  'useUROI'      'useFIR'          'TIC'       'peakTables'
# [15] 'dataPoints' 'peakFit'      'isAnnotated'

## Slots shouldn't be accessed directly, accessors are available:
cpdID(annotation)
# [1] 'ID-1' 'ID-2'
cpdName(annotation)
# [1] 'Cpd 1' 'Cpd 2'
ROI(annotation)
#   rtMin       rt rtMax    mzMin    mz    mzMax cpdID cpdName
# 1  3310 3344.888  3390 522.1948 522.2 522.2052  ID-1   Cpd 1
# 2  3280 3385.577  3440 496.1950 496.2 496.2050  ID-2   Cpd 2
FIR(annotation)
#   rtMin rtMax mzMin mzMax cpdID cpdName
# 1    NA    NA    NA    NA  ID-1   Cpd 1
# 2    NA    NA    NA    NA  ID-2   Cpd 2
uROI(annotation)
#   rtMin rt rtMax mzMin mz mzMax cpdID cpdName
# 1    NA NA    NA    NA NA    NA  ID-1   Cpd 1
# 2    NA NA    NA    NA NA    NA  ID-2   Cpd 2
filepath(annotation)
# [1] 'C:/R/R-3.6.0/library/faahKO/cdf/KO/ko15.CDF'
# [2]'C:/R/R-3.6.0/library/faahKO/cdf/KO/ko16.CDF'
# [3] 'C:/R/R-3.6.0/library/faahKO/cdf/KO/ko18.CDF'
cpdMetadata(annotation)
# data frame with 0 columns and 2 rows
spectraMetadata(annotation)
# data frame with 0 columns and 3 rows
acquisitionTime(annotation)
# [1] NA NA NA
uROIExist(annotation)
# [1] FALSE
useUROI(annotation)
# [1] FALSE
useFIR(annotation)
# [1] FALSE
TIC(annotation)
# [1] NA NA NA
peakTables(annotation)
# [[1]]
# NULL
# [[2]]
# NULL
# [[3]]
# NULL
dataPoints(annotation)
# [[1]]
# NULL
# [[2]]
# NULL
# [[3]]
# NULL
peakFit(annotation)
# [[1]]
# NULL
# [[2]]
# NULL
# [[3]]
# NULL
isAnnotated(annotation)
# [1] FALSE
}

}
\seealso{
Other peakPantheR: 
\code{\link{peakPantheR_parallelAnnotation}()},
\code{\link{peakPantheR_singleFileSearch}()}

Other parallelAnnotation: 
\code{\link{peakPantheR_parallelAnnotation}()},
\code{\link{peakPantheR_singleFileSearch}()}
}
\concept{parallelAnnotation}
\concept{peakPantheR}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/curveFit_utils.R
\name{skewedGaussian_minpack.lm}
\alias{skewedGaussian_minpack.lm}
\title{Implementation of the Skewed Gaussian peak shape for use with
minpack.lm}
\usage{
skewedGaussian_minpack.lm(params, xx)
}
\arguments{
\item{params}{(list) skewed gaussian parameters (\code{params$gamma},
\code{params$center}, \code{params$sigma}, \code{params$amplitude})}

\item{xx}{(numeric) values at which to evalute the skewed gaussian}
}
\value{
value of the skewed gaussian evaluated at xx
}
\description{
Implementation of the Skewed Gaussian peak shape for use with
minpack.lm
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/findTargetFeatures.R
\name{findTargetFeatures}
\alias{findTargetFeatures}
\title{Find and integrate target features in each ROI}
\usage{
findTargetFeatures(
    ROIsDataPoints,
    ROI,
    curveModel = "skewedGaussian",
    params = "guess",
    sampling = 250,
    maxApexResidualRatio = 0.2,
    verbose = FALSE,
    ...
)
}
\arguments{
\item{ROIsDataPoints}{(list) A list (one entry per ROI window) of data.frame
with signal as row and retention time ('rt'), mass ('mz') and intensity
('int) as columns. Must match each row of ROI.}

\item{ROI}{(data.frame) A data.frame of compounds to target as rows. Columns:
\code{rtMin} (float in seconds), \code{rtMax} (float in seconds),
\code{mzMin} (float), \code{mzMax} (float)}

\item{curveModel}{(str) Name of the curve model to fit (currently
\code{skewedGaussian} and \code{emgGaussian})}

\item{params}{(list or str) Either 'guess' for automated parametrisation or
list (one per ROI windows) of 'guess' or list of curve fit parameters}

\item{sampling}{(int) Number of points to employ when subsampling the
fittedCurve (rt, rtMin, rtMax, integral calculation)}

\item{maxApexResidualRatio}{(float) Ratio of maximum allowed fit residual at
the peak apex, compared to the fit max intensity. (e.g. 0.2 for a maximum
residual of 20\% of apex intensity)}

\item{verbose}{(bool) If TRUE message the time taken and number of features
found}

\item{...}{Passes arguments to \code{fitCurve} to alter peak fitting
(\code{params})}
}
\value{
A list: \code{list()$peakTable} (\emph{data.frame}) with targeted
features as rows and peak measures as columns (see Details),
\code{list()$curveFit} (\emph{list}) a list of \code{peakPantheR_curveFit} or
NA for each ROI.

\subsection{Details:}{
The returned \code{data.frame} is structured as follow:
\tabular{ll}{
found \tab was the peak found\cr
rt \tab retention time of peak apex (sec)\cr
rtMin \tab leading edge of peak retention time (sec) determined at 0.5\% of
apex intensity\cr
rtMax \tab trailing edge of peak retention time (sec) determined at 0.5\% of
apex intensity\cr
mz \tab weighted (by intensity) mean of peak m/z across scans\cr
mzMin \tab m/z peak minimum (between rtMin, rtMax)\cr
mzMax \tab m/z peak maximum (between rtMin, rtMax)\cr
peakArea \tab integrated peak area\cr
peakAreaRaw \tab integrated peak area from raw data points\cr
maxIntMeasured \tab maximum peak intensity in raw data\cr
maxIntPredicted \tab maximum peak intensity based on curve fit (at apex)\cr
}
}
}
\description{
For each ROI, fit a curve and integrate the largest feature in
the box. Each entry in \code{ROIsDataPoints} must match the corresponding row
in \code{ROI}. The curve shape to employ for fitting can be changed with
\code{curveModel} while fitting parameters can be changed with \code{params}
(list with one param per ROI window). \code{rtMin} and \code{rtMax} are
established at 0.5% of apex intensity using a moving window from the apex
outward (the window is the ROI width); if after 8 iterations \code{rtMin} or
\code{rtMax} is not found, NA is returned and the peak fit rejected.
\code{peakArea} is calculated from \code{rtMin} to \code{rtMax}.
\code{peakAreaRaw} is calculated from \code{rtMin} to \code{rtMax} but
using the raw data points instead of the modelled line-shape.
\code{mz} is the weighted (by intensity)
average mz of datapoints falling into the \code{rtMin} to \code{rtMax}
range, \code{mzMin} and \code{mzMax} are the
minimum and maxmimum mass in these range. If \code{rtMin} or \code{rtMax}
falls outside of ROI (extracted scans), \code{mzMin} or \code{mzMax} are
returned as the input ROI limits and \code{mz} is an approximation on the
datapoints available (if no scan of the ROI fall between rtMin/rtMax, mz
would be NA, the peak is rejected). If any of the two following ratio are
superior to \code{maxApexResidualRatio}, the fit is rejected: 1) ratio of fit
residuals at the apex (predicted apex fit intensity vs measured apex
intensity: fit overshoots the apex), 2) ratio of predicted apex fit intensity
vs maximum measured peak intensity (fit misses the real apex in the peak).
}
\details{
## Examples cannot be computed as the function is not exported:
## Load data
library(faahKO)
library(MSnbase)
netcdfFilePath <- system.file('cdf/KO/ko15.CDF', package = 'faahKO')
raw_data <- MSnbase::readMSData(netcdfFilePath,centroided=TRUE,mode='onDisk')

## targetFeatTable
targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
                        c('cpdID','cpdName','rtMin','rt','rtMax','mzMin',
                        'mz','mzMax'))), stringsAsFactors=FALSE)
targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
                        522.2, 522.205222)
targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
                        496.2, 496.204962)
targetFeatTable[,3:8] <- vapply(targetFeatTable[,3:8], as.numeric,
                        FUN.VALUE=numeric(2))

ROIsPt <- extractSignalRawData(raw_data,
                                rt=targetFeatTable[,c('rtMin','rtMax')],
                                mz=targetFeatTable[,c('mzMin','mzMax')],
                                verbose=TRUE)
# Reading data from 2 windows

foundPeaks <- findTargetFeatures(ROIsPt, targetFeatTable, verbose=TRUE)
# Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
# mzMin/mzMax calculation,
# approximate mz and returning ROI$mzMin and ROI$mzMax for ROI #1
# Found 2/2 features in 0.07 secs

foundPeaks
# $peakTable
#   found    rtMin       rt    rtMax    mzMin    mz    mzMax peakArea
# 1  TRUE 3309.759 3346.828 3385.410 522.1948 522.2 522.2052 26133727
# 2  TRUE 3345.377 3386.529 3428.279 496.2000 496.2 496.2000 35472141
#   peakAreaRaw maxIntMeasured maxIntPredicted
# 1    26071378         889280        901015.8
# 2    36498367        1128960       1113576.7
#
# $curveFit
# $curveFit[[1]]
# $amplitude
# [1] 162404.8
# 
# $center
# [1] 3341.888
# 
# $sigma
# [1] 0.07878613
# 
# $gamma
# [1] 0.00183361
# 
# $fitStatus
# [1] 2
# 
# $curveModel
# [1] 'skewedGaussian'
# 
# attr(,'class')
# [1] 'peakPantheR_curveFit'
# 
# $curveFit[[2]]
# $amplitude
# [1] 199249.1
# 
# $center
# [1] 3382.577
# 
# $sigma
# [1] 0.07490442
# 
# $gamma
# [1] 0.00114719
# 
# $fitStatus
# [1] 2
# 
# $curveModel
# [1] 'skewedGaussian'
# 
# attr(,'class')
# [1] 'peakPantheR_curveFit'
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_peakPantheRAnnotation.R
\docType{methods}
\name{outputAnnotationDiagnostic,peakPantheRAnnotation-method}
\alias{outputAnnotationDiagnostic,peakPantheRAnnotation-method}
\alias{outputAnnotationDiagnostic}
\title{Save to disk the annotation parameters as CSV and a diagnostic plot
per fitted compound}
\usage{
\S4method{outputAnnotationDiagnostic}{peakPantheRAnnotation}(
    object,
    saveFolder,
    savePlots = TRUE,
    sampleColour = NULL,
    verbose = TRUE,
    ncores = 0,
    svgPlot = FALSE,
    ...
)
}
\arguments{
\item{object}{(peakPantheRAnnotation) Annotated peakPantheRAnnotation object}

\item{saveFolder}{(str) Path of folder where annotationParameters_summary.csv
and plots will be saved}

\item{savePlots}{(bool) If TRUE save a diagnostic plot for each compound}

\item{sampleColour}{(str) NULL or vector colour for each sample}

\item{verbose}{(bool) If TRUE message progress}

\item{ncores}{(int) Number of cores to use to save plots in parallel}

\item{svgPlot}{(bool) If TRUE save plots as 'svg', otherwise as 'png'}

\item{...}{Additional parameters for plotting i.e. \code{sampling} for the
number of points to employ when plotting fittedCurve}
}
\value{
None
}
\description{
Save to disk the annotation parameters as CSV (as generated by
\code{outputAnnotationParamsCSV()}) and a diagnostic plot per fitted compound
(as generated by \code{annotationDiagnosticMultiplot()}) if \code{savePlots}
is TRUE
}
\examples{
if(requireNamespace('faahKO')){
## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
## compounds

# Paths to spectra files
library(faahKO)
spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
                    system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
                    system.file('cdf/KO/ko18.CDF', package = 'faahKO'))

# targetFeatTable
targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
                    c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
                    'mzMax'))), stringsAsFactors=FALSE)
targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
                        522.2, 522.205222)
targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
                        496.2, 496.204962)
targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
                                    FUN.VALUE=numeric(2))

emptyAnnotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
                                        targetFeatTable=targetFeatTable)

# Calculate annotation
annotation <- peakPantheR_parallelAnnotation(emptyAnnotation, ncores=0,
                                getAcquTime=FALSE, verbose=FALSE)$annotation

# temporary location
savePath1       <- tempdir()
outputAnnotationDiagnostic(annotation, saveFolder=savePath1, savePlots=FALSE,
                            verbose=TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getAcquisitionDatemzML.R
\name{getAcquisitionDatemzML}
\alias{getAcquisitionDatemzML}
\title{Parse acquisition date from a mzML file}
\usage{
getAcquisitionDatemzML(mzMLPath, verbose = TRUE)
}
\arguments{
\item{mzMLPath}{(str) path to mzML raw data file}

\item{verbose}{(bool) if TRUE message progress}
}
\value{
POSIXct or NA
}
\description{
Extract acquisition date (`'startTimeStamp'``) from a mzML file.
In case of failure (or the file is not a \code{mzML}) returns NULL
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotEICDetectedPeakwidth.R
\name{plotEICDetectedPeakwidth}
\alias{plotEICDetectedPeakwidth}
\title{Plot samples raw data and detected feature for a single ROI}
\usage{
plotEICDetectedPeakwidth(
    ROIDataPointSampleList,
    cpdID,
    cpdName,
    rt,
    rtMin,
    rtMax,
    mzMin,
    mzMax,
    ratio = 0.85,
    sampling = 250,
    curveFitSampleList = NULL,
    sampleColour = NULL,
    verbose = TRUE
)
}
\arguments{
\item{ROIDataPointSampleList}{(list) list of \code{data.frame} of raw data
points for each sample (retention time 'rt', mass 'mz' and intensity 'int'
(as column) of each raw data points (as row)).}

\item{cpdID}{(str) Compound ID}

\item{cpdName}{(str) Compound Name}

\item{rt}{(float) vector of detected peak apex retention time (in sec)}

\item{rtMin}{(float) vector of detected peak minimum retention time (in sec)}

\item{rtMax}{(float) vector ofdetected peak maximum retention time (in sec)}

\item{mzMin}{(float) ROI minimum m/z (matching EIC)}

\item{mzMax}{(float) ROI maximum m/z (matching EIC)}

\item{ratio}{(float) value between 0 and 1 defining the vertical percentage
taken by the EICs subplot}

\item{sampling}{(int) Number of points to employ when plotting fittedCurve}

\item{curveFitSampleList}{(list) NULL or a list of
\code{peakPantheR_curveFit} (or NA) for each sample}

\item{sampleColour}{(str) NULL or vector colour for each sample (same length
as \code{EICs}, \code{rt}, \code{rtMin}, \code{rtMax})}

\item{verbose}{(bool) if TRUE message when NA scans are removed}
}
\value{
Grob (ggplot object)
}
\description{
Plot a ROI across multiple samples (x axis is RT, y axis is
intensity) with the matching detected peak rt and peakwidth under it. If
curveFit is provided, the fitted curve for each compound is added. RT and
peakwidth are plotted in the order spectra are passed, with the first spectra
on top.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GUI_utils.R
\name{load_annotation_from_file_UI_helper}
\alias{load_annotation_from_file_UI_helper}
\title{UI data import helper - check loaded annotation}
\usage{
load_annotation_from_file_UI_helper(annotationPath)
}
\arguments{
\item{annotationPath}{(str) Path to a RData file containing a
peakPantheRAnnotation names `annotationObject`}
}
\value{
(peakPantheRAnnotation) Object loaded from file
}
\description{
Load a .RData file (check it exists) and that a peakPantheRAnnotation named
"annotationObject" is present. Returns the annotation if everything is valid
}
\examples{
## Initialise a peakPantheRAnnotation object with 3 samples and 2 compounds

## Inputs
# spectraPaths
spectraPaths <- c('./path/file1', './path/file2', './path/file3')

# targetFeatTable
targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
                    c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
                    'mzMax'))), stringsAsFactors=FALSE)
targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
                        522.2, 522.205222)
targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
                        496.2, 496.204962)
targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
                                    FUN.VALUE=numeric(2))

annotationObject <- peakPantheRAnnotation(spectraPaths=spectraPaths,
                                        targetFeatTable=targetFeatTable)

# save annotation to disk
annotPath <- tempfile(pattern="file", tmpdir=tempdir(), fileext='.RData')
save(annotationObject, file=annotPath, compress=TRUE)

# Load annotation
load_annotation_from_file_UI_helper(annotationPath = annotPath)
# An object of class peakPantheRAnnotation
#   2 compounds in 3 samples.
#   updated ROI do not exist (uROI)
#   does not use updated ROI (uROI)
#   does not use fallback integration regions (FIR)
#   is not annotated
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rtCorrection_utils.R
\name{peakPantheR_applyRTCorrection}
\alias{peakPantheR_applyRTCorrection}
\title{Correct targeted retention time based on reference compounds}
\usage{
peakPantheR_applyRTCorrection(
    targetFeatTable,
    referenceTable,
    method = "polynomial",
    params = list(polynomialOrder = 3),
    robust = TRUE
)
}
\arguments{
\item{targetFeatTable}{a \code{\link{data.frame}} of compounds to target as
rows and parameters as columns: \code{cpdID} (str), \code{cpdName} (str),
\code{rtMin} (float in seconds), \code{rt} (float in seconds, or \emph{NA}),
\code{rtMax} (float in seconds), \code{mzMin} (float), \code{mz} (float or
\emph{NA}), \code{mzMax} (float).}

\item{referenceTable}{a \code{\link{data.frame}} of reference compound
information as rows and properties as columns: \code{cpdID} (str),
\code{cpdName} (str), \code{rt} (float), \code{rt_dev_sec} (float)}

\item{method}{(str) name of RT correction method to use (currently
\code{polynomial})}

\item{params}{(list) list of parameters to pass to
the each correction method.
Currently allowed inputs are \code{polynomialOrder} for
\code{method='polynomial'}}

\item{robust}{(bool) whether to use the RANSAC algorithm to flag and
ignore outliers during retention time correction}
}
\value{
a targetFeatTable with corrected RT
}
\description{
Correct targeted features retention time using the RT and RT deviation of
previously fitted compounds. The `method` and `params` are used to select and
parametrise the retention time correction method employed. When `robust`
is set to TRUE, the RANSAC algorithm is used
to automatically flag outliers and robustify the correction function fitting.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/curveFit_utils.R
\name{skewedGaussian_guess}
\alias{skewedGaussian_guess}
\title{Guess function for initial skewed gaussian parameters and bounds}
\usage{
skewedGaussian_guess(x, y)
}
\arguments{
\item{x}{(numeric) x values (e.g. retention time)}

\item{y}{(numeric) y observed values (e.g. spectra intensity)}
}
\value{
A list of guessed starting parameters \code{list()$init_params},
lower \code{list()$lower_bounds} and upper bounds \code{list()$upper_bounds}
(\code{$gamma}, \code{$center}, \code{$sigma}, \code{$amplitude})
}
\description{
Guess function for initial skewed gaussian parameters and
bounds, at the moment only checks the x position
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/peakPantheR_plotEICFit.R
\name{peakPantheR_plotEICFit}
\alias{peakPantheR_plotEICFit}
\title{Plot samples raw data and detected feature for a single ROI}
\usage{
peakPantheR_plotEICFit(
    ROIDataPointSampleList,
    curveFitSampleList = NULL,
    rtMin = NULL,
    rtMax = NULL,
    sampling = 250,
    sampleColour = NULL,
    verbose = TRUE
)
}
\arguments{
\item{ROIDataPointSampleList}{(list) list of \code{data.frame} of raw data
points for each sample (retention time 'rt', mass 'mz' and intensity 'int'
(as column) of each raw data points (as row)).}

\item{curveFitSampleList}{(list) NULL or a list of
\code{peakPantheR_curveFit} (or NA) for each sample}

\item{rtMin}{(float) NULL or vector of detected peak minimum retention time
(in sec)}

\item{rtMax}{(float) NULL or vector of detected peak maximum retention time
(in sec)}

\item{sampling}{(int) Number of points to employ when plotting fittedCurve}

\item{sampleColour}{(str) NULL or vector colour for each sample (same length
as \code{ROIDataPointSampleList}, \code{rtMin}, \code{rtMax})}

\item{verbose}{(bool) if TRUE message when NA scans are removed}
}
\value{
Grob (ggplot object)
}
\description{
plot a ROI across multiple samples (x axis is RT, y axis is
intensity). If curveFit is provided, the fitted curve for each sample is
added.
}
\examples{
## Input data
# fake sample 1
# ROI data points
rt1             <- seq(990, 1010, by=20/250)
mz1             <- rep(522., length(rt1))
int1            <- (dnorm(rt1, mean=1000, sd=1.5) * 100) + 1
tmp_DataPoints1 <- data.frame(rt=rt1, mz=mz1, int=int1)
# fittedCurve
fit1        <- list(amplitude=37.068916502809756, center=999.3734222573454,
                    sigma=0.58493182568124724, gamma=0.090582029276037035,
                    fitStatus=2, curveModel='skewedGaussian')
class(fit1)     <- 'peakPantheR_curveFit'

# fake sample 2
# ROI data points
rt2             <- seq(990, 1010, by=20/250)
mz2             <- rep(522., length(rt2))
int2            <- (dnorm(rt2, mean=1002, sd=1.5) * 100) + 1
tmp_DataPoints2 <- data.frame(rt=rt2, mz=mz2, int=int2)
# fittedCurve
fit2        <- list(amplitude=37.073067416755556, center=1001.3736564832565,
                    sigma=0.58496485738212201, gamma=0.090553713725151905,
                    fitStatus=2, curveModel='skewedGaussian')
class(fit2)     <- 'peakPantheR_curveFit'

## Plot features in 1 sample without colours
peakPantheR_plotEICFit(ROIDataPointSampleList=list(tmp_DataPoints1),
                        curveFitSampleList=list(fit1),
                        rtMin=995., rtMax=1005.,
                        sampling=250, sampleColour=NULL, verbose=FALSE)

## Plot features in 2 samples with colours
peakPantheR_plotEICFit(
                ROIDataPointSampleList=list(tmp_DataPoints1,tmp_DataPoints2),
                curveFitSampleList=list(fit1, fit2),
                rtMin=c(995., 997.), rtMax=c(1005.,1007.),
                sampling=250, sampleColour=c('blue', 'red'), verbose=FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/peakPantheR_loadAnnotationParamsCSV.R
\name{prepare_basic_target_parameters}
\alias{prepare_basic_target_parameters}
\title{Process target region parameters (simple format) for object
initialisation}
\usage{
prepare_basic_target_parameters(paramTable)
}
\arguments{
\item{paramTable}{(data.frame) Target region parameters}
}
\value{
(list) List of \code{targetFeatTable, uROI, FIR, uROIExist}
}
\description{
Process the simple target region parameters (\code{cpdID, cpdName, mzMin,
mzMax, mz, rtMin, rtMax, rt}) and return input variables for
\code{peakPantheRAnnotation()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/peakPantheR_plotPeakwidth.R
\name{peakPantheR_plotPeakwidth}
\alias{peakPantheR_plotPeakwidth}
\title{Plot peak value and peakwidth by acquisition time or in input order}
\usage{
peakPantheR_plotPeakwidth(
    apexValue,
    widthMin = NULL,
    widthMax = NULL,
    acquTime = NULL,
    varName = "variable",
    sampleColour = NULL,
    rotateAxis = FALSE,
    verbose = TRUE
)
}
\arguments{
\item{apexValue}{(float) vector of apex value}

\item{widthMin}{(float) vector of detected peak minimum peakwidth value or
NULL (if NULL no peakwidth)}

\item{widthMax}{(float) vector of detected peak maximum peakwidth value or
NULL (uf NULL no peakwidth)}

\item{acquTime}{(POSIXct) vector of sample acquisition time as POSIXct or
NULL (if NULL points are plotted in the order values are passed as input with
the first on top or left)}

\item{varName}{(str) Name of the variable to plot}

\item{sampleColour}{(str) NULL or vector colour for each sample (same length
as \code{apexValue}, \code{widthMin}, \code{widthMax}, \code{acquTime})}

\item{rotateAxis}{(bool) if TRUE x and y axis are reversed}

\item{verbose}{(bool) if TRUE message when NA scans are removed}
}
\value{
Grob (ggplot object)
}
\description{
For a single ROI, plot the peak value and peakwidth (RT, m/z,
...) of detected peaks across multiple samples, by acquisition time or in
input order. If \code{rotateAxis=FALSE} x is run order / plot order, y is the
\code{apexValue} / \code{widthMin} / \code{widthMax}, if
\code{rotateAxis=TRUE} x is the measurement values and y the run order.
}
\examples{
## Input data
apexVal <- c(1, 2, 3, 4)
minVal  <- c(0, 0, 2, 2)
maxVal  <- c(2, 4, 4, 5)
acqTime <- as.POSIXct(c('2017-07-13 21:06:14', '2017-07-14 21:06:14', 
                        '2017-07-15 21:06:14', '2017-07-16 21:06:14'))

## Plot 4 sampels with colour
peakPantheR_plotPeakwidth(apexValue=apexVal, widthMin=minVal,widthMax=maxVal,
                        acquTime=NULL, varName='Test variable 1',
                        sampleColour=c('blue','red','green','orange'),
                        rotateAxis=FALSE, verbose=FALSE)

## Plot 4 samples with colour by acquisition time
peakPantheR_plotPeakwidth(apexValue=apexVal, widthMin=minVal,widthMax=maxVal,
                        acquTime=acqTime, varName='Test variable 2',
                        sampleColour=c('blue','red','green','orange'),
                        rotateAxis=FALSE, verbose=FALSE)

## Plot 4 samples with colour, rotate axis
peakPantheR_plotPeakwidth(apexValue=apexVal, widthMin=minVal,widthMax=maxVal,
                        acquTime=NULL, varName='Test variable 3',
                        sampleColour=c('blue','red','green','orange'),
                        rotateAxis=TRUE, verbose=FALSE)

## Plot 4 samples with colour by acquisition time, rotate axis
peakPantheR_plotPeakwidth(apexValue=apexVal, widthMin=minVal,widthMax=maxVal,
                        acquTime=acqTime, varName='Test variable 4',
                        sampleColour=c('blue','red','green','orange'),
                        rotateAxis=FALSE, verbose=FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/peakPantheR_quickEIC.R
\name{peakPantheR_quickEIC}
\alias{peakPantheR_quickEIC}
\title{Extract and plot a EIC from a raw data file}
\usage{
peakPantheR_quickEIC(
    spectraPath,
    rt,
    mz,
    valuesOnly = "Plot",
    centroided = TRUE,
    msLevel = 1L,
    verbose = TRUE
)
}
\arguments{
\item{spectraPath}{(str) Path to the raw data file to read (uses 
`MSnbase::readMSData()`)}

\item{rt}{(numeric(2) or two-column matrix) the lower and upper retention
time range from which the data should be extracted. If a matrix is passed,
each row corresponds to a different window. If not provided, the full
retention time range will be extracted.}

\item{mz}{(numeric(2) or two-column matrix) the lower and upper mass range
from which the data should be extracted. If a matrix is passed, each row
corresponds to a different window. If not provided, the full mass range will
be extracted.}

\item{valuesOnly}{(str) If 'Raw' only load the file and return a table of raw
extracted values (exported version of `extractSignalRawData()`). If `EIC`
return a table of EIC data point. Else return the EIC plot (default).}

\item{centroided}{(bool) Indicate to `MSnbase::readMSData()` whether the spectra
file is centroided or not (default to 'TRUE').}

\item{msLevel}{(int) The MS level at which the data should be extracted 
(default to MS level 1).}

\item{verbose}{(bool) Output proress information or not.}
}
\value{
Grob (ggplot object) of the EIC plot, if `valuesOnly='Raw'` returns 
a data.frame of raw datapoints with as columns 'rt', 'mz' and 'int'. If 
`valuesOnly='EIC'` returns a data.frame of EIC datapoints with as columns 
'rt' and 'int'.
}
\description{
Simple plot of an Extracted Ion Chromatogram (EIC) from a raw 
data file and a provided mz and rt window. Return ggplot plot object.
}
\examples{
## Use a file form the faahKO package and plot an EIC of interest
library(faahKO)
spectraPath <- system.file('cdf/KO/ko15.CDF',package='faahKO')
peakPantheR_quickEIC(spectraPath,
                        rt = c(3290., 3410.),
                        mz = c(522.194778, 522.205222))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/curveFit_utils.R
\name{emgGaussian_minpack.lm_objectiveFun}
\alias{emgGaussian_minpack.lm_objectiveFun}
\title{Exponentially Modified Gaussian minpack.lm objective function}
\usage{
emgGaussian_minpack.lm_objectiveFun(params, observed, xx)
}
\arguments{
\item{params}{(list) exponentially modified gaussian parameters
(\code{params$gamma}, \code{params$center}, \code{params$sigma},
\code{params$amplitude})}

\item{observed}{(numeric) observed y value at xx}

\item{xx}{(numeric) value at which to evalute the exponentially
modified gaussian}
}
\value{
difference between observed and expected exponentially modified
gaussian value evaluated at xx
}
\description{
Exponentially Modified Gaussian (EMG) minpack.lm objective
function, calculates residuals using the EMG Peak Shape
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GUI_utils.R
\name{annotation_showText_UI_helper}
\alias{annotation_showText_UI_helper}
\title{UI show annotation helper - UI sidebar string}
\usage{
annotation_showText_UI_helper(annotProp)
}
\arguments{
\item{annotProp}{(list) Named list of annotation properties as
created by \code{annotation_showMethod_UI_helper()}}
}
\value{
(str) Textual description of the annotation to show on UI
}
\description{
Return a text description of an annotation for UI presentation
}
\examples{
# Input
properties_default <- list(nbCompounds = 0,
                           nbSamples = 0,
                           uROIExist = FALSE,
                           useUROI = FALSE,
                           useFIR = FALSE,
                           isAnnotated = FALSE)

# Generate description
annotation_showText_UI_helper(properties_default)
# [[1]]
# [1] "Not annotated"
#
# [[2]]
# [1] "0 compounds"
#
# [[3]]
# [1] "0 samples"
#
# [[4]]
# [1] "updated ROI do not exist (uROI)"
#
# [[5]]
# [1] "does not use updated ROI (uROI)"
#
# [[6]]
# [1] "does not use fallback integration regions (FIR)"
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_peakPantheRAnnotation.R
\docType{methods}
\name{annotationDiagnosticPlots,peakPantheRAnnotation-method}
\alias{annotationDiagnosticPlots,peakPantheRAnnotation-method}
\alias{annotationDiagnosticPlots}
\title{Generate fit diagnostic plots}
\usage{
\S4method{annotationDiagnosticPlots}{peakPantheRAnnotation}(
    object,
    sampleColour,
    sampling,
    verbose
)
}
\arguments{
\item{object}{(peakPantheRAnnotation) Annotated peakPantheRAnnotation object}

\item{sampleColour}{(str) NULL or vector colour for each sample}

\item{sampling}{(int) Number of points to employ when plotting fittedCurve}

\item{verbose}{(bool) if TRUE message the plot generation progress}
}
\value{
A list (one list per compound) of diagnostic plots:
\code{result[[i]]$EICFit}, \code{result[[i]]$rtPeakwidthVert},
\code{result[[i]]$rtPeakwidthHorzRunOrder},
\code{result[[i]]$mzPeakwidthHorzRunOrder}, \code{result[[i]]$areaRunOrder},
\code{result[[i]]$rtHistogram}, \code{result[[i]]$mzHistogram},
\code{result[[i]]$areaHistogram}, \code{result[[i]]$title}
}
\description{
Generate fit diagnostic plots for each ROI: \code{EICFit} the
raw data and detected feature fit, \code{rtPeakwidthVert} detected peaks
retention time apex and peakwidth (vertical and no run order),
\code{rtPeakwidthHorzRunOrder} detected peaks retention time apex and
peakwidth by run order, \code{mzPeakwidthHorzRunOrder} detected peaks m/z
apex and peakwidth by run order, \code{areaRunOrder} detected peaks area by
run order, \code{rtHistogram} histogram of detected peaks retention time,
\code{mzHistogram} histogram of detected peaks m/z, \code{areaHistogram}
histogram of detected peaks area.
}
\examples{
if(requireNamespace('faahKO')){
## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
## compounds

# Paths to spectra files
library(faahKO)
spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
                    system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
                    system.file('cdf/KO/ko18.CDF', package = 'faahKO'))

# targetFeatTable
targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
                    c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
                    'mzMax'))), stringsAsFactors=FALSE)
targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
                        522.2, 522.205222)
targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
                        496.2, 496.204962)
targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
                                    FUN.VALUE=numeric(2))

emptyAnnotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
                                        targetFeatTable=targetFeatTable)

annotationDiagnosticPlots(emptyAnnotation)
# Warning: the object has not been annotated, return an empty diagnostic plot
# list
# [[1]]
# NULL
# [[2]]
# NULL
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/peakPantheR_loadAnnotationParamsCSV.R
\name{peakPantheR_loadAnnotationParamsCSV}
\alias{peakPantheR_loadAnnotationParamsCSV}
\title{Load fit parameters from CSV}
\usage{
peakPantheR_loadAnnotationParamsCSV(CSVParamPath, verbose = TRUE)
}
\arguments{
\item{CSVParamPath}{(str) Path to a CSV file of fit parameters (e.g. as
saved by \code{outputAnnotationDiagnostic})}

\item{verbose}{(bool) If TRUE message progress}
}
\value{
(peakPantheRAnnotation) Object initialised with ROI, uROI and FIR
read from the CSV file
}
\description{
Initialise a new \code{peakPantheRAnnotation} object after
loading ROI, uROI and FIR parameters from CSV. \code{spectraPaths},
\code{spectraMetadata} or \code{cpdMetadata} are not initialised and will
need to be filled before annotation. \code{useUROI} and \code{useFIR} are set
to \code{FALSE} and will need to be set accordingly. \code{uROIExist} is
established depending on the uROI columns present in the CSV and will be set
to \code{TRUE} only if no \code{NA} are present
}
\examples{
## Input data
input_CSV <- data.frame(matrix(nrow=2,ncol=21,dimnames=list(c(),
    c('cpdID', 'cpdName',
    'X','ROI_rt', 'ROI_mz', 'ROI_rtMin', 'ROI_rtMax','ROI_mzMin','ROI_mzMax',
    'X','uROI_rtMin', 'uROI_rtMax', 'uROI_mzMin', 'uROI_mzMax', 'uROI_rt',
    'uROI_mz', 'X', 'FIR_rtMin', 'FIR_rtMax', 'FIR_mzMin', 'FIR_mzMax'))))
input_CSV[1,]  <- c('ID-1', 'Cpd 1', '|', 1.,  2.,  3.,  4.,  5.,  6.,  '|',
                    7.,  8.,  9.,  10., 11., 12., '|', 13., 14., 15., 16.)
input_CSV[2,]  <- c('ID-2', 'Cpd 2', '|', 17., 18., 19., 20., 21., 22., '|',
                    23., 24., 25., 26., 27., 28., '|', 29., 30., 31., 32.)
input_CSV[,-c(1,2,3,10,17)]  <- vapply(input_CSV[,-c(1,2,3,10,17)],
                                            as.numeric, FUN.VALUE=numeric(2))

# temporary file location
savePath1      <- tempfile(pattern='file', tmpdir=tempdir(), fileext='.csv')
# save csv
utils::write.csv(input_CSV, file=savePath1, row.names=FALSE)

# Load parameters from CSV
loadedAnnotation <- peakPantheR_loadAnnotationParamsCSV(savePath1,
                                                        verbose=TRUE)
# uROIExist set to TRUE
# New peakPantheRAnnotation object initialised for 2 compounds
# An object of class peakPantheRAnnotation
#  2 compounds in 0 samples.
#   updated ROI exist (uROI)
#   does not use updated ROI (uROI)
#   does not use fallback integration regions (FIR)
#   is not annotated
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/saveSingleFileMultiEIC.R
\name{saveSingleFileMultiEIC}
\alias{saveSingleFileMultiEIC}
\title{Save to disk a plot of all ROI EIC and detected feature range}
\usage{
saveSingleFileMultiEIC(
    ROIsDataPoint,
    curveFit,
    foundPeakTable,
    savePath,
    width = 15,
    height = 15,
    verbose = TRUE
)
}
\arguments{
\item{ROIsDataPoint}{(list) a list of \code{data.frame} of raw data points
for each ROI (retention time 'rt', mass 'mz' and intensity 'int' (as column)
of each raw data points (as row)).}

\item{curveFit}{(list) a list of \code{peakPantheR_curveFit} or NA for each
ROI}

\item{foundPeakTable}{(data.frame) \code{data.frame} as generated by
\link{findTargetFeatures}, with features as rows and peak properties as
columns. The following columns are mandatory: \code{cpdID}, \code{cpdName},
\code{rt}, \code{rtmin}, \code{rtmax}, \code{mzmin}, \code{mzmax}.}

\item{savePath}{(str) Full path to save a \emph{.png} of all ROI EICs,
expect \code{'filepath/filename.png'}.}

\item{width}{(float) Width in cm for a single ROI plot (if more than one plot
in total, 2 columns will be used). dpi set to a 100.}

\item{height}{(float) height in a cm for a single ROI plot. dpi set to 100}

\item{verbose}{(bool) if TRUE message progress}
}
\value{
None
}
\description{
Plot and save a \code{.png} of all ROI (x is RT, y is
intensity), with the matching detected peak rt and peakwidth under it.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/peakPantheR_start_GUI.R
\name{peakPantheR_start_GUI}
\alias{peakPantheR_start_GUI}
\title{peakPantheR Graphical User Interface}
\usage{
peakPantheR_start_GUI(browser = TRUE)
}
\arguments{
\item{browser}{If TRUE open the graphical user interface in a web browser
instead of a R window. Default is TRUE}
}
\value{
None, start GUI. To exit press \code{ESC} in the command line.
}
\description{
peakPantheR Graphical User Interface (GUI) implements all the functions for
the parallel detection, integration and reporting of pre-defined features in
multiple mass spectrometry data files. To exit press \code{ESC} in the
command line.
}
\examples{
print("Start graphical interface, press 'ESC' in the command line to stop")
# peakPantheR_start_GUI()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/curveFit_utils.R
\name{emgGaussian_minpack.lm}
\alias{emgGaussian_minpack.lm}
\title{Implementation of the Exponentially Modified Gaussian (EMG) peak
shape for use with minpack.lm}
\usage{
emgGaussian_minpack.lm(params, xx)
}
\arguments{
\item{params}{(list) exponentiall modified gaussian parameters
(\code{params$gamma}, \code{params$center}, \code{params$sigma},
\code{params$amplitude})}

\item{xx}{(numeric) values at which to evalute the exponentially
modified gaussian}
}
\value{
value of the exponentially modified gaussian evaluated at xx
}
\description{
Implementation of the  Exponentially Modified Gaussian
(EMG) peak shape for use with minpack.lm
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_peakPantheRAnnotation.R
\docType{methods}
\name{annotationParamsDiagnostic,peakPantheRAnnotation-method}
\alias{annotationParamsDiagnostic,peakPantheRAnnotation-method}
\alias{annotationParamsDiagnostic}
\title{Set uROI and FIR based on annotation results}
\usage{
\S4method{annotationParamsDiagnostic}{peakPantheRAnnotation}(
    object,
    verbose
)
}
\arguments{
\item{object}{(peakPantheRAnnotation) Annotated peakPantheRAnnotation object}

\item{verbose}{(bool) If TRUE message progress of uROI and FIR calculation}
}
\value{
(peakPantheRAnnotation) object with updated ROI and FIR set from
annotation results
}
\description{
Set updated ROI (uROI) and Fallback Integration Regions (FIR)
based on the annotation results. If the object is not annotated, it is
returned untouched. ROI is not modified. If uROI exist they are left
untouched, otherwise they are set as the minimum and maximum found peaks
limits (+/-5\% of ROI in retention time). If FIR are used they are left
untouched, otherwise they are set as the median of the found limits (rtMin,
rtMax, mzMin, mzMax).
}
\examples{
if(requireNamespace('faahKO')){
## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
## compounds

# Paths to spectra files
library(faahKO)
spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
                    system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
                    system.file('cdf/KO/ko18.CDF', package = 'faahKO'))

# targetFeatTable
targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
                    c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
                    'mzMax'))), stringsAsFactors=FALSE)
targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
                        522.2, 522.205222)
targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
                        496.2, 496.204962)
targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
                                    FUN.VALUE=numeric(2))

emptyAnnotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
                                        targetFeatTable=targetFeatTable)

annotationParamsDiagnostic(emptyAnnotation, verbose=TRUE)
# Warning: the object has not been annotated, return the object untouched
# An object of class peakPantheRAnnotation
#  2 compounds in 3 samples.
#   updated ROI do not exist (uROI)
#   does not use updated ROI (uROI)
#   does not use fallback integration regions (FIR)
#   is not annotated
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/peakPantheR_parallelAnnotation.R
\name{peakPantheR_parallelAnnotation}
\alias{peakPantheR_parallelAnnotation}
\title{Search, integrate and report targeted features in a multiple spectra}
\usage{
peakPantheR_parallelAnnotation(
    object,
    ncores = 0,
    getAcquTime = TRUE,
    resetWorkers = 1,
    centroided = TRUE,
    curveModel = "skewedGaussian",
    verbose = TRUE,
    ...
)
}
\arguments{
\item{object}{(peakPantheRAnnotation) Initialised peakPantheRAnnotation
object defining the samples to process and compounds to target. The slots
\code{useUROI} and \code{useFIR} controls if uROI must be used and FIR
integrated if a feature is not found}

\item{ncores}{(int) Number of cores to use for parallelisation. Default 0 for
no parallelisation.}

\item{getAcquTime}{(bool) If TRUE will extract sample acquisition date-time
from the mzML metadata (the additional file access will impact run time)}

\item{resetWorkers}{(int) If 0, the parallel cluster is only initiated once.
If >0 the cluster will be reset (and the memory of each worker freed) once
\code{ncores * resetWorkers} files have been processed. Default value is 1,
the cluster is reset once \code{ncores} files have been processed. While
potentially impacting performance (need to wait until all \code{ncores *
resetWorkers} files are processed before restarting the cluster), shutting
down the workers processes regularly will ensure the OS can reallocate memory
more efficiently. For values >1, ensure sufficient system memory is available}

\item{centroided}{(bool) use TRUE if the data is centroided, used by
\code{\link[MSnbase]{readMSData}} when reading the raw data files}

\item{curveModel}{(str) specify the peak-shape model to fit,
by default \code{skewedGaussian}.
Accepted values are \code{skewedGaussian} and \code{emgGaussian}}

\item{verbose}{(bool) If TRUE message calculation progress, time taken,
number of features found (total and matched to targets) and failures}

\item{...}{Passes arguments to \code{findTargetFeatures} to alter
peak-picking parameters}
}
\value{
a list: \code{list()$result} \emph{(peakPantheRAnnotation)} fully
annotated object, \code{list()$failures} \emph{(list)} list of failed samples
and error message
}
\description{
Integrate all target features in all files defined in the
initialised input object and store results. The use of updated ROI and the
integration of FIR are controled by the input object slots \code{useUROI} and
\code{useFIR}. Files are processed in parallel using
\link{peakPantheR_singleFileSearch}; \code{ncores} controls the number of
cores used for parallelisation, with \code{ncores=0} corresponding to serial
processing. If the processing of a file fails (file does not exist or error
during execution) the sample is removed from the outputed object.
}
\examples{
if(requireNamespace('faahKO')){
## Load data
library(faahKO)

# 3 files
input_spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
                        system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
                        system.file('cdf/KO/ko18.CDF', package = 'faahKO'))

# 4 features
input_ROI     <- data.frame(matrix(vector(), 4, 8,
                    dimnames=list(c(), c('cpdID', 'cpdName', 'rtMin', 'rt',
                                        'rtMax', 'mzMin', 'mz', 'mzMax'))),
                    stringsAsFactors=FALSE)
input_ROI[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
                    522.2, 522.205222)
input_ROI[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
                    496.2, 496.204962)
input_ROI[3,] <- c('ID-3', 'Cpd 3', 3420., 3454.435, 3495., 464.195358,
                    464.2, 464.204642)
input_ROI[4,] <- c('ID-4', 'Cpd 4', 3670., 3701.697, 3745., 536.194638,
                    536.2, 536.205362)
input_ROI[,c(3:8)] <- vapply(input_ROI[,c(3:8)], as.numeric,
                            FUN.VALUE=numeric(4))

# Initialise object
initAnnotation <- peakPantheRAnnotation(spectraPaths=input_spectraPaths,
                                        targetFeatTable=input_ROI)
# to use updated ROI:
# uROIExist=TRUE, useUROI=TRUE, uROI=input_uROI
# to use FallBack Integration Regions:
# useFIR=TRUE, FIR=input_FIR

# Run serially
result_parallelAnnotation <- peakPantheR_parallelAnnotation(initAnnotation,
                                                        ncores=0,
                                                        getAcquTime=FALSE,
                                                        verbose=TRUE)
# Processing 4 compounds in 3 samples:
#  uROI:\tFALSE
#  FIR:\tFALSE
# ----- ko15 -----
# Polarity can not be extracted from netCDF files, please set manually the
#  polarity with the 'polarity' method.
# Reading data from 4 windows
# Data read in: 0.24 secs
# Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
#  mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and
#  ROI$mzMax for ROI #1
# Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
#  mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and
#  ROI$mzMax for ROI #3
# Found 4/4 features in 0.06 secs
# Peak statistics done in: 0.02 secs
# Feature search done in: 0.76 secs
# ----- ko16 -----
# Polarity can not be extracted from netCDF files, please set manually the
#  polarity with the 'polarity' method.
# Reading data from 4 windows
# Data read in: 0.24 secs
# Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
#  mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and
#  ROI$mzMax for ROI #1
# Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
#  mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and
#  ROI$mzMax for ROI #2
# Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
#  mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and
#  ROI$mzMax for ROI #3
# Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
#  mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and
#  ROI$mzMax for ROI #4
# Found 4/4 features in 0.08 secs
# Peak statistics done in: 0 secs
# Feature search done in: 0.71 secs
# ----- ko18 -----
# Polarity can not be extracted from netCDF files, please set manually the
#  polarity with the 'polarity' method.
# Reading data from 4 windows
# Data read in: 0.25 secs
# Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
#  mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and
#  ROI$mzMax for ROI #1
# Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
#  mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and
#  ROI$mzMax for ROI #2
# Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
#  mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and
#  ROI$mzMax for ROI #4
# Found 4/4 features in 0.06 secs
# Peak statistics done in: 0 secs
# Feature search done in: 0.71 secs
# ----------------
# Parallel annotation done in: 2.18 secs

# No failures
result_parallelAnnotation$failures

result_parallelAnnotation$annotation
# An object of class peakPantheRAnnotation
#  4 compounds in 3 samples. 
#    updated ROI do not exist (uROI)
#    does not use updated ROI (uROI)
#    does not use fallback integration regions (FIR)
#    is annotated
}

}
\seealso{
Other peakPantheR: 
\code{\link{peakPantheRAnnotation}},
\code{\link{peakPantheR_singleFileSearch}()}

Other parallelAnnotation: 
\code{\link{peakPantheRAnnotation}},
\code{\link{peakPantheR_singleFileSearch}()}
}
\concept{parallelAnnotation}
\concept{peakPantheR}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_peakPantheRAnnotation.R
\docType{methods}
\name{resetFIR,peakPantheRAnnotation-method}
\alias{resetFIR,peakPantheRAnnotation-method}
\alias{resetFIR}
\title{Reset FIR windows to uROI or ROI values
Reset FIR windows to uROI (or ROI if \code{uROIExist=FALSE})}
\usage{
\S4method{resetFIR}{peakPantheRAnnotation}(object, verbose)
}
\arguments{
\item{object}{(peakPantheRAnnotation) object for which FIR are to be reset}

\item{verbose}{(bool) If TRUE message progress}
}
\value{
(peakPantheRAnnotation) object with FIR values reset
}
\description{
Reset FIR windows to uROI or ROI values
Reset FIR windows to uROI (or ROI if \code{uROIExist=FALSE})
}
\examples{
## Initialise a peakPantheRAnnotation object with 2 targeted compounds

## targetFeatTable
input_targetFeatTable <- data.frame(matrix(vector(), 2, 8,
                            dimnames=list(c(), c('cpdID', 'cpdName', 'rtMin',
                            'rt', 'rtMax', 'mzMin', 'mz', 'mzMax'))),
                            stringsAsFactors=FALSE)
input_targetFeatTable[1,] <- c('ID-1', 'Cpd 1',  3.,  1.,  4.,  5.,  2.,  6.)
input_targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 19., 17., 20., 21., 18., 22.)
input_targetFeatTable[,c(3:8)] <- sapply(input_targetFeatTable[,c(3:8)],
                                        as.numeric)
## FIR
input_FIR       <- data.frame(matrix(vector(), 2, 4, dimnames=list(c(), 
                                c('rtMin', 'rtMax', 'mzMin', 'mzMax'))),
                                stringsAsFactors=FALSE)
input_FIR[1,]   <- c(13., 14., 15., 16.)
input_FIR[2,]   <- c(29., 30., 31., 32.)

annotation <- peakPantheRAnnotation(targetFeatTable = input_targetFeatTable,
                                    FIR = input_FIR, uROIExist = FALSE)

## Reset FIR with ROI values as uROI are not set
updatedAnnotation <- resetFIR(annotation, verbose=TRUE)
# FIR will be reset with ROI values as uROI values are not set
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/curveFit_utils.R
\name{gaussian_erf}
\alias{gaussian_erf}
\title{Gaussian Error function}
\usage{
gaussian_erf(x)
}
\arguments{
\item{x}{(numeric) value at which to evaluate the gaussian error function}
}
\value{
Value of the gaussian error function evaluated at x
}
\description{
Implementation of the gaussian error function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/curveFit_utils.R
\name{predictCurve}
\alias{predictCurve}
\title{Predict curve values}
\usage{
predictCurve(fittedCurve, x)
}
\arguments{
\item{fittedCurve}{(peakPantheR_curveFit) A 'peakPantheR_curveFit': a list of
curve fitting parameters, curve shape model \code{curveModel} and nls.lm fit
status \code{fitStatus}.}

\item{x}{(numeric) values at which to evaluate the fitted curve}
}
\value{
fitted curve values at x
}
\description{
Evaluate fitted curve values at \code{x} data points
}
\details{
## Examples cannot be computed as the function is not exported:
## Input a fitted curve
fittedCurve <- list(amplitude=275371.1, center=3382.577, sigma=0.07904697,
                    gamma=0.001147647, fitStatus=2,
                    curveModel='skewedGaussian')
class(fittedCurve)  <- 'peakPantheR_curveFit'
input_x <- c(3290, 3300, 3310, 3320, 3330, 3340, 3350, 3360, 3370, 3380,
            3390, 3400, 3410)

## Predict y at each input_x
pred_y  <- predictCurve(fittedCurve, input_x)
pred_y
# [1] 2.347729e-08 1.282668e-05 3.475590e-03 4.676579e-01 3.129420e+01
# [6] 1.043341e+03 1.736915e+04 1.447754e+05 6.061808e+05 1.280037e+06
# [11] 1.369651e+06 7.467333e+05 2.087477e+05
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/curveFit_utils.R
\name{is.peakPantheR_curveFit}
\alias{is.peakPantheR_curveFit}
\title{Check if object is of class peakPantheR_curveFit}
\usage{
is.peakPantheR_curveFit(x)
}
\arguments{
\item{x}{object to test}
}
\value{
(bool) TRUE or FALSE
}
\description{
Check if object is of class peakPantheR_curveFit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/peakPantheR.R
\docType{package}
\name{peakPantheR}
\alias{peakPantheR}
\alias{peakPantheR-package}
\title{peakPantheR: A package for Peak Picking and ANnoTation of High resolution
Experiments}
\description{
\pkg{peakPantheR} detects, integrates and reports pre-defined features in
mass spectrometry data files. It enables the real time annotation of multiple
compounds in a single file, or the parallel annotation of multiple compounds
in multiple files.
}
\details{
The main functions of \pkg{peakPantheR} are
\code{\link{peakPantheR_singleFileSearch}} for realtime annotation, and
\code{\link{peakPantheR_parallelAnnotation}} for parallel annotation. The
\code{peakPantheRAnnotation} object stores parallel annotation results, while
reporting functions help assess the quality of annotation and update
fitting parameters. Refer to the vignettes for graphical user interface and
command line tutorials.
}
\seealso{
Useful links:
\itemize{
    \item \url{https://github.com/phenomecentre/peakPantheR}
    \item Report bugs at
    \url{https://github.com/phenomecentre/peakPantheR/issues/new}
}

}
\author{
\strong{Maintainer}: Arnaud Wolfer \email{adwolfer@gmail.com}
    (\href{https://orcid.org/0000-0001-5856-3218}{ORCID})

Authors:
\itemize{
    \item Goncalo Correia \email{gscorreia89@gmail.com}
        (\href{https://orcid.org/0000-0001-8271-9294}{ORCID})
}

Other contributors:
\itemize{
    \item Jake Pearce \email{jake.pearce@gmail.com} [contributor]
    \item Caroline Sands \email{caz119@hotmail.co.uk} [contributor]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/curveFit_utils.R
\name{skewedGaussian_minpack.lm_objectiveFun}
\alias{skewedGaussian_minpack.lm_objectiveFun}
\title{Skewed Gaussian minpack.lm objective function}
\usage{
skewedGaussian_minpack.lm_objectiveFun(params, observed, xx)
}
\arguments{
\item{params}{(list) skewed gaussian parameters (\code{params$gamma},
\code{params$center}, \code{params$sigma}, \code{params$amplitude})}

\item{observed}{(numeric) observed y value at xx}

\item{xx}{(numeric) value at which to evalute the skewed gaussian}
}
\value{
difference between observed and expected skewed gaussian value
evaluated at xx
}
\description{
Skewed Gaussian minpack.lm objective function, calculates
residuals using the skewed gaussian Peak Shape
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/peakPantheR_loadAnnotationParamsCSV.R
\name{prepare_advanced_target_parameters}
\alias{prepare_advanced_target_parameters}
\title{Process target region parameters (with uROI, FIR) for object
initialisation}
\usage{
prepare_advanced_target_parameters(paramTable, verbose)
}
\arguments{
\item{paramTable}{(data.frame) Target region parameters}

\item{verbose}{(bool) If TRUE message progress}
}
\value{
(list) List of \code{targetFeatTable, uROI, FIR, uROIExist}
}
\description{
Process target region parameters with uROI and FIR (\code{cpdID, cpdName,
ROI_rt, ROI_mz, ROI_rtMin, ROI_rtMax, ROI_mzMin, ROI_mzMax, uROI_rtMin,
uROI_rtMax, uROI_mzMin, uROI_mzMax, uROI_rt, uROI_mz, FIR_rtMin, FIR_rtMax,
FIR_mzMin, FIR_mzMax}) and return input variables for
\code{peakPantheRAnnotation()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_peakPantheRAnnotation.R
\docType{methods}
\name{outputAnnotationParamsCSV,peakPantheRAnnotation-method}
\alias{outputAnnotationParamsCSV,peakPantheRAnnotation-method}
\alias{outputAnnotationParamsCSV}
\title{Save annotation parameters as CSV}
\usage{
\S4method{outputAnnotationParamsCSV}{peakPantheRAnnotation}(
    object,
    saveFolder,
    verbose,
    noSave
)
}
\arguments{
\item{object}{(peakPantheRAnnotation) Annotated peakPantheRAnnotation object}

\item{saveFolder}{(str) Path of folder where annotationParameters_summary.csv
will be saved}

\item{verbose}{(bool) If TRUE message progress}

\item{noSave}{(bool) If TRUE the resulting table will be returned without
saving to disk}
}
\value{
None
}
\description{
Save annotation parameters (ROI, uROI and FIR) to disk as a CSV
file for editing
}
\examples{
## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
## compounds

# Paths to spectra files
spectraPaths <- c('./path/file1', './path/file2', './path/file3')

# targetFeatTable
targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
                    c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
                    'mzMax'))), stringsAsFactors=FALSE)
targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
                        522.2, 522.205222)
targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
                        496.2, 496.204962)
targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
                                    FUN.VALUE=numeric(2))

emptyAnnotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
                                        targetFeatTable=targetFeatTable)

# temporary file
savePath        <- tempdir()

# statistics of an empty annotation
outputAnnotationParamsCSV(emptyAnnotation, saveFolder=savePath, verbose=TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/curveFit_utils.R
\name{fitCurve}
\alias{fitCurve}
\title{Curve fitting using minpack.lm}
\usage{
fitCurve(x, y, curveModel = "skewedGaussian", params = "guess")
}
\arguments{
\item{x}{(numeric) x values (e.g. retention time)}

\item{y}{(numeric) y observed values (e.g. spectra intensity)}

\item{curveModel}{(str) name of the curve model to fit (currently
\code{skewedGaussian} and \code{emgGaussian})}

\item{params}{(list or str) either 'guess' for automated parametrisation or
list of initial parameters (\code{$init_params}), lower parameter bounds
(\code{$lower_bounds}) and upper parameter bounds (\code{$upper_bounds})}
}
\value{
A 'peakPantheR_curveFit': a list of fitted curve parameters,
\code{fitStatus} from \code{nls.lm$info} and curve shape name
\code{curveModel}. \code{fitStatus=0} unsuccessful completion: improper input
parameters, \code{fitStatus=1} successful completion: first convergence test
is successful, \code{fitStatus=2} successful completion: second convergence
test is successful, \code{fitStatus=3} successful completion: both
convergence test are successful, \code{fitStatus=4} questionable completion:
third convergence test is successful but should be carefully examined
(maximizers and saddle points might satisfy), \code{fitStatus=5} unsuccessful
completion: excessive number of function evaluations/iterations
}
\description{
Fit different curve models using \code{minpack}. Fitting
parameters can be passed or guessed.
}
\details{
## Examples cannot be computed as the function is not exported:
## x is retention time, y corresponding intensity
input_x  <- c(3362.102, 3363.667, 3365.232, 3366.797, 3368.362, 3369.927,
            3371.492, 3373.057, 3374.622, 3376.187, 3377.752, 3379.317,
            3380.882, 3382.447, 3384.012, 3385.577, 3387.142, 3388.707,
            3390.272, 3391.837, 3393.402, 3394.966, 3396.531, 3398.096,
            3399.661, 3401.226, 3402.791, 3404.356, 3405.921, 3407.486,
            3409.051)
input_y  <- c(51048, 81568, 138288, 233920, 376448, 557288, 753216, 938048,
            1091840, 1196992, 1261056, 1308992, 1362752, 1406592, 1431360,
            1432896, 1407808, 1345344, 1268480, 1198592, 1126848, 1036544,
            937600, 849792, 771456, 692416, 614528, 546088, 492752,
            446464, 400632)

## Fit
fitted_curve <- fitCurve(input_x, input_y, curveModel='skewedGaussian',
                        params='guess')

## Returns the optimal fitting parameters
fitted_curve
#
# $amplitude
# [1] 275371.1
# 
# $center
# [1] 3382.577
# 
# $sigma
# [1] 0.07904697
# 
# $gamma
# [1] 0.001147647
# 
# $fitStatus
# [1] 2
# 
# $curveModel
# [1] 'skewedGaussian'
#
# attr(,'class')
# [1] 'peakPantheR_curveFit'
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GUI_utils.R
\name{annotation_showMethod_UI_helper}
\alias{annotation_showMethod_UI_helper}
\title{UI show annotation helper - list of properties}
\usage{
annotation_showMethod_UI_helper(annotation)
}
\arguments{
\item{annotation}{(peakPantherAnnotation) Object to describe}
}
\value{
(list) Named list of annotation properties
}
\description{
show method specific to the UI, that returns each field in a named list to
ease display
}
\examples{
# Initialise an empty annotation, no uROI, no use of FIR
annotInit     <- peakPantheRAnnotation()

# return properties
annotation_showMethod_UI_helper(annotInit)
# $nbCompounds
# [1] 0
#
# #$nbSamples
# [1] 0
#
# #$uROIExist
# [1] FALSE
#
# $useUROI
# [1] FALSE
#
# $useFIR
# [1] FALSE
#
# $isAnnotated
# [1] FALSE
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/peakPantheR_singleFileSearch.R
\name{peakPantheR_singleFileSearch}
\alias{peakPantheR_singleFileSearch}
\title{Search, integrate and report targeted features in a raw spectra}
\usage{
peakPantheR_singleFileSearch(
    singleSpectraDataPath,
    targetFeatTable,
    peakStatistic = FALSE,
    plotEICsPath = NA,
    getAcquTime = FALSE,
    FIR = NULL,
    centroided = TRUE,
    curveModel = "skewedGaussian",
    verbose = TRUE,
    ...
)
}
\arguments{
\item{singleSpectraDataPath}{(str) path to netCDF or mzML raw data file
(centroided, \strong{only with the channel of interest}).}

\item{targetFeatTable}{a \code{\link{data.frame}} of compounds to target as
rows. Columns: \code{cpdID} (str), \code{cpdName} (str), \code{rtMin} (float
in seconds), \code{rt} (float in seconds, or \emph{NA}), \code{rtMax} (float
in seconds), \code{mzMin} (float), \code{mz} (float or \emph{NA}),
\code{mzMax} (float).}

\item{peakStatistic}{(bool) If TRUE calculates additional peak statistics:
'ppm_error', 'rt_dev_sec', 'tailing factor' and 'asymmetry factor'}

\item{plotEICsPath}{(str or NA) If not NA, will save a \emph{.png} of all ROI
EICs at the path provided (\code{'filepath/filename.png'} expected). If NA no
plot saved}

\item{getAcquTime}{(bool) If TRUE will extract sample acquisition date-time
from the mzML metadata (the additional file access will impact run time)}

\item{FIR}{(data.frame or NULL) If not NULL, integrate Fallback Integration
Regions (FIR) when a feature is not found.  Compounds as row are identical to
\code{targetFeatTable}, columns are \code{rtMin} (float in seconds),
\code{rtMax} (float in seconds), \code{mzMin} (float), \code{mzMax} (float).}

\item{centroided}{(bool) use TRUE if the data is centroided, used by
\code{\link[MSnbase]{readMSData}} when reading the raw data file}

\item{curveModel}{(str) specify the peak-shape model to fit,
by default \code{skewedGaussian}.
Accepted values are \code{skewedGaussian} and \code{emgGaussian}}

\item{verbose}{(bool) If TRUE message calculation progress, time taken and
number of features found}

\item{...}{Passes arguments to \code{findTargetFeatures} to alter
peak-picking parameters (e.g. \code{curveModel}, \code{sampling},
\code{params} as a list of parameters for each ROI or 'guess',...)}
}
\value{
a list: \code{list()$TIC} \emph{(int)} TIC value,
\code{list()$peakTable} \emph{(data.frame)} targeted features results
(see Details), \code{list()$curveFit} \emph{(list)} list of
\code{peakPantheR_curveFit} or NA for each ROI, \code{list()$acquTime}
\emph{(POSIXct or NA)} date-time of sample acquisition from mzML metadata,
\code{list()$ROIsDataPoint} \emph{(list)} a list of \code{data.frame} of raw
data points for each ROI (retention time 'rt', mass 'mz' and intensity 'int'
(as column) of each raw data points (as row)).

\subsection{Details:}{
The returned \emph{peakTable} \code{data.frame} is structured as follow:
\tabular{ll}{
cpdID \tab database compound ID\cr
cpdName \tab compound name\cr
found \tab was the peak found\cr
rt \tab retention time of peak apex (sec)\cr
rtMin \tab leading edge of peak retention time (sec) determined at 0.5\% of
apex intensity\cr
rtMax \tab trailing edge of peak retention time (sec) determined at 0.5\% of
apex intensity\cr
mz \tab weighted (by intensity) mean of peak m/z across scans\cr
mzMin \tab m/z peak minimum (between rtMin, rtMax)\cr
mzMax \tab m/z peak maximum (between rtMin, rtMax)\cr
peakArea \tab integrated peak area\cr
peakAreaRaw \tab integrated peak area from raw data points\cr
maxIntMeasured \tab maximum peak intensity in raw data\cr
maxIntPredicted \tab maximum peak intensity based on curve fit\cr
is_filled \tab Logical indicate if the feature was integrated using FIR
(Fallback Integration Region)\cr
ppm_error \tab difference in ppm between the expected and measured m/z\cr
rt_dev_sec \tab difference in seconds between the expected and measured rt\cr
tailingFactor \tab the tailing factor is a measure of peak tailing.It is
defined as the distance from the front slope of the peak to the back slope
divided by twice the distance from the center line of the peak to the front
slope, with all measurements made at 5\% of the maximum peak height. The
tailing factor of a peak will typically be similar to the asymmetry factor
for the same peak, but the two values cannot be directly converted\cr
asymmetryFactor \tab the asymmetry factor is a measure of peak tailing. It is
defined as the distance from the center line of the peak to the back slope
divided by the distance from the center line of the peak to the front slope,
with all measurements made at 10\% of the maximum peak height. The asymmetry
factor of a peak will typically be similar to the tailing factor for the same
peak, but the two values cannot be directly converted\cr
}
}
}
\description{
Report for a raw spectra the TIC, acquisition time, integrated
targeted features, fitted curves and datapoints for each region of interest.
Optimised to reduce the number of file access. Features not detected can be
integrated using fallback integration regions (FIR).
}
\examples{
if(requireNamespace('faahKO')){
## Load data
library(faahKO)
netcdfFilePath <- system.file('cdf/KO/ko15.CDF', package = 'faahKO')

## targetFeatTable
targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
                    c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
                    'mzMax'))), stringsAsFactors=FALSE)
targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
                        522.2, 522.205222)
targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
                        496.2, 496.204962)
targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
                                    FUN.VALUE=numeric(2))

res <- peakPantheR_singleFileSearch(netcdfFilePath,targetFeatTable,
                                    peakStatistic=TRUE)
# Polarity can not be extracted from netCDF files, please set manually the
#    polarity with the 'polarity' method.
# Reading data from 2 windows
# Data read in: 0.16 secs
# Warning: rtMin/rtMax outside of ROI; datapoints cannot be used for
#   mzMin/mzMax calculation, approximate mz and returning ROI$mzMin and
#   ROI$mzMax for ROI #1
# Found 2/2 features in 0.05 secs
# Peak statistics done in: 0 secs
# Feature search done in: 0.75 secs

res
# $TIC
# [1] 2410533091
#
# $peakTable
#   found    rtMin       rt    rtMax    mzMin    mz    mzMax peakArea
# 1  TRUE 3309.759 3346.828 3385.410 522.1948 522.2 522.2052 26133727
# 2  TRUE 3345.377 3386.529 3428.279 496.2000 496.2 496.2000 35472141
#   peakAreaRaw maxIntMeasured maxIntPredicted cpdID cpdName is_filled
# 1    26071378         889280        901015.8  ID-1   Cpd 1     FALSE
# 2    36498367        1128960       1113576.7  ID-2   Cpd 2     FALSE
#    ppm_error   rt_dev_sec  tailingFactor  asymmetryFactor
# 1 0.02337616    1.9397590       1.015357         1.026824
# 2 0.02460103    0.9518072       1.005378         1.009318
#
# $acquTime
# [1] NA
#
#
# $curveFit
# $curveFit[[1]]
# $amplitude
# [1] 162404.8
# 
# $center
# [1] 3341.888
# 
# $sigma
# [1] 0.07878613
# 
# $gamma
# [1] 0.00183361
# 
# $fitStatus
# [1] 2
# 
# $curveModel
# [1] 'skewedGaussian'
# 
# attr(,'class')
# [1] 'peakPantheR_curveFit'
# 
# $curveFit[[2]]
# $amplitude
# [1] 199249.1
# 
# $center
# [1] 3382.577
# 
# $sigma
# [1] 0.07490442
# 
# $gamma
# [1] 0.00114719
# 
# $fitStatus
# [1] 2
# 
# $curveModel
# [1] 'skewedGaussian'
# 
# attr(,'class')
# [1] 'peakPantheR_curveFit'
#
#
# $ROIsDataPoint
# $ROIsDataPoint[[1]]
#          rt    mz    int
# 1  3315.154 522.2   2187
# 2  3316.719 522.2   3534
# 3  3318.284 522.2   6338
# 4  3319.849 522.2  11718
# 5  3321.414 522.2  21744
# 6  3322.979 522.2  37872
# 7  3324.544 522.2  62424
# 8  3326.109 522.2  98408
# 9  3327.673 522.2 152896
# 10 3329.238 522.2 225984
# ...
#
# $ROIsDataPoint[[2]]
#          rt    mz     int
# 1  3280.725 496.2    1349
# 2  3290.115 496.2    2069
# 3  3291.680 496.2    3103
# 4  3293.245 496.2    5570
# 5  3294.809 496.2   10730
# 6  3296.374 496.2   20904
# 7  3297.939 496.2   38712
# 8  3299.504 496.2   64368
# 9  3301.069 496.2   97096
# 10 3302.634 496.2  136320
# ...
}

}
\seealso{
Other peakPantheR: 
\code{\link{peakPantheRAnnotation}},
\code{\link{peakPantheR_parallelAnnotation}()}

Other parallelAnnotation: 
\code{\link{peakPantheRAnnotation}},
\code{\link{peakPantheR_parallelAnnotation}()}
}
\concept{parallelAnnotation}
\concept{peakPantheR}
\concept{realTimeAnnotation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_peakPantheRAnnotation.R
\docType{methods}
\name{outputAnnotationResult,peakPantheRAnnotation-method}
\alias{outputAnnotationResult,peakPantheRAnnotation-method}
\alias{outputAnnotationResult}
\title{Save to disk all annotation results as csv files}
\usage{
\S4method{outputAnnotationResult}{peakPantheRAnnotation}(
    object,
    saveFolder,
    annotationName = "annotationResult",
    verbose = TRUE
)
}
\arguments{
\item{object}{(peakPantheRAnnotation) Annotated peakPantheRAnnotation object}

\item{saveFolder}{(str) Path of folder where the annotation result csv will
be saved}

\item{annotationName}{(str) name of annotation to use in the saved csv}

\item{verbose}{(bool) If TRUE message progress}
}
\value{
None
}
\description{
Save to disk all annotation results as
\code{annotationName_ ... .csv} files: compound metadata (\code{cpdMetadata},
\code{cpdID}, \code{cpdName}) and spectra metadata (\code{spectraMetadata},
\code{acquisitionTime}, \code{TIC}), summary of fit (ratio of peaks found:
\code{ratio_peaks_found}, ratio of peaks filled: \code{ratio_peaks_filled},
mean ppm_error: \code{ppm_error}, mean rt_dev_sec: \code{rt_dev_sec}), and a
file for each column of \code{peakTables} (with samples as rows and compounds
as columns)
}
\examples{
if(requireNamespace('faahKO')){
## Initialise a peakPantheRAnnotation object with 3 samples and 2 targeted
## compounds

# Paths to spectra files
library(faahKO)
spectraPaths <- c(system.file('cdf/KO/ko15.CDF', package = 'faahKO'),
                    system.file('cdf/KO/ko16.CDF', package = 'faahKO'),
                    system.file('cdf/KO/ko18.CDF', package = 'faahKO'))

# targetFeatTable
targetFeatTable <- data.frame(matrix(vector(), 2, 8, dimnames=list(c(),
                    c('cpdID','cpdName','rtMin','rt','rtMax','mzMin','mz',
                    'mzMax'))), stringsAsFactors=FALSE)
targetFeatTable[1,] <- c('ID-1', 'Cpd 1', 3310., 3344.888, 3390., 522.194778,
                        522.2, 522.205222)
targetFeatTable[2,] <- c('ID-2', 'Cpd 2', 3280., 3385.577, 3440., 496.195038,
                        496.2, 496.204962)
targetFeatTable[,c(3:8)] <- vapply(targetFeatTable[,c(3:8)], as.numeric,
                                    FUN.VALUE=numeric(2))

emptyAnnotation <- peakPantheRAnnotation(spectraPaths=spectraPaths,
                                        targetFeatTable=targetFeatTable)

# Calculate annotation
annotation      <- peakPantheR_parallelAnnotation(emptyAnnotation, ncores=0,
                                getAcquTime=FALSE, verbose=FALSE)$annotation

# temporary location
savePath1       <- tempdir()
outputAnnotationResult(annotation, saveFolder=savePath1,
                        annotationName='testProject', verbose=TRUE)
}
}
